Package: rockchalk
Type: Package
Title: Regression Estimation and Presentation
Version: 1.8.164
Date: 2026-04-15
Authors@R: c(person("Paul E.","Johnson",email="pauljohn32@freefaculty.org",role = c("aut", "cre")),
    person("Gabor", "Grothendieck",email="ggrothendieck@gmail.com", role="ctb"),
    person("Dimitri", "Papadopoulos OrfanosGabor", email="dimitri.papadopoulos@cea.fr", role="ctb"))
Maintainer: Paul E. Johnson <pauljohn32@freefaculty.org>
Description: A collection of functions for interpretation and presentation
    of regression analysis.  These functions are used
    to produce the statistics lectures in
    <https://pj.freefaculty.org/guides/>. Includes regression
    diagnostics, regression tables, and plots of interactions and
    "moderator" variables. The emphasis is on "mean-centered" and
    "residual-centered" predictors. The vignette 'rockchalk' offers a
    fairly comprehensive overview.  The vignette 'Rstyle' has advice
    about coding in R.  The package title 'rockchalk' refers to our
    school motto, 'Rock Chalk Jayhawk, Go K.U.'.
License: GPL (>= 3.0)
LazyLoad: yes
Depends: R (>= 2.10)
Imports: grDevices, methods, lme4, carData, MASS, kutils, reformulas
        (>= 0.4.3)
Suggests: tables, Hmisc, car, mvtnorm, scatterplot3d, HH
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-15 15:36:21 UTC; pauljohn
Author: Paul E. Johnson [aut, cre],
  Gabor Grothendieck [ctb],
  Dimitri Papadopoulos OrfanosGabor [ctb]
Repository: CRAN
Date/Publication: 2026-04-16 07:10:08 UTC
Built: R 4.6.0; ; 2026-04-26 06:11:33 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(getFocal,character)
S3method(getFocal,default)
S3method(getFocal,factor)
S3method(meanCenter,default)
S3method(model.data,default)
S3method(newdata,default)
S3method(pctable,character)
S3method(pctable,default)
S3method(pctable,formula)
S3method(plot,testSlopes)
S3method(plotPlane,default)
S3method(plotSlopes,lm)
S3method(predict,mcreg)
S3method(predict,rcreg)
S3method(print,mcreg)
S3method(print,pctable)
S3method(print,stdreg)
S3method(print,summarize)
S3method(print,summary.mcreg)
S3method(print,summary.pctable)
S3method(print,summary.stdreg)
S3method(residualCenter,default)
S3method(standardize,lm)
S3method(summary,mcreg)
S3method(summary,pctable)
S3method(summary,stdreg)
export(addLines)
export(centerNumerics)
export(centralValues)
export(combineLevels)
export(cutByQuantile)
export(cutBySD)
export(cutByTable)
export(cutFancy)
export(descriptiveTable)
export(dir.create.unique)
export(drawnorm)
export(focalVals)
export(formatSummarizedFactors)
export(formatSummarizedNumerics)
export(genCorrelatedData)
export(genCorrelatedData2)
export(genCorrelatedData3)
export(genX)
export(getDeltaRsquare)
export(getFocal)
export(getPartialCor)
export(gmc)
export(kurtosis)
export(lazyCor)
export(lazyCov)
export(lmAuxiliary)
export(magRange)
export(makeSymmetric)
export(mcDiagnose)
export(mcGraph1)
export(mcGraph2)
export(mcGraph3)
export(meanCenter)
export(model.data)
export(mvrnorm)
export(newdata)
export(outreg)
export(outreg2HTML)
export(padW0)
export(pctable)
export(perspEmpty)
export(plotCurves)
export(plotPlane)
export(plotSeq)
export(plotSlopes)
export(predictOMatic)
export(rbindFill)
export(removeNULL)
export(residualCenter)
export(skewness)
export(standardize)
export(summarize)
export(summarizeFactors)
export(summarizeNumerics)
export(testSlopes)
export(vech2Corr)
export(vech2mat)
export(waldt)
import(MASS)
import(carData)
import(grDevices)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.default)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(kutils,mgsub)
importFrom(lme4,VarCorr)
importFrom(methods,as)
importFrom(methods,formalArgs)
importFrom(reformulas,formatVC)
importFrom(stats,AIC)
importFrom(stats,addmargins)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,df.residual)
importFrom(stats,dnorm)
importFrom(stats,drop1)
importFrom(stats,embed)
importFrom(stats,family)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,napredict)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,predict.lm)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,resid)
importFrom(stats,rlogis)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(utils,browseURL)
importFrom(utils,methods)
importFrom(utils,modifyList)

### Filename: Normal2_2008.R
### Paul Johnson  March 28, 2008

### This code should be available somewhere in
### http://pj.freefaculty.org/R.  If it is not email me
### <pauljohn@ku.edu>

sampleSize <- 500

mymean <- 0

mystd <- 1.5

myx <- seq( mymean - 3*mystd, mymean + 3*mystd, length.out = sampleSize)

## Creates an empty container to hold samples:
mySamples <- list()

## for loop creates 100 samples and stores them
for (i in 1:100) {
  mySamples[[i]] <- rnorm(sampleSize,  mean = mymean, sd = mystd)
}


par(mfcol = c(5,2))

for (i in 1:10) {
  hist(mySamples[[i]],main = paste("Sample", i) )
}


### force ranges to match
for (i in 1:10) {
  mytitle <- paste("Sample", i, "mean", round(mean(mySamples[[i]]),2),"std.dev.", round(sd(mySamples[[i]]),2) )
  myhist <- hist(mySamples[[i]],main= mytitle, freq = FALSE, xlim = c(-6,6), xlab = "Normally Distributed Variable" )
}





par(mfcol = c(1,1))



### sapply applies the mean function to each sample. The "s" in
### sapply stands for "simplify".
myMeans <- sapply(mySamples, mean)

hist(myMeans,freq = FALSE, main = paste("Sampling Distribution of ",100," Samples"))

mtext(text = paste("True Mean = ", mymean,  "True Standard Deviation = ", mystd) , side = 3)

### Note the range of the means:
summary(myMeans)

### Compare against the range in one sample, say the 5th
summary(mySamples[[5]])

text( -0.2, 2.5, paste("Mean of Means =", round(mean(myMeans),3),"\n Std. of Means ", round(sd(myMeans),3)))


par(mfcol = c(2,1))

 myhist <- hist(mySamples[[1]],main= mytitle, freq = FALSE, xlim = c(-6,6), xlab = "Normally Distributed Variable" )


hist(myMeans,freq = FALSE, main = paste("Sampling Distribution of ",100," Samples"), xlim = c(-6,6))


### Another way. write a function to create the data and draw it.
### Use lapply to call it over and over




sampleSize <- 500

par(mfrow = c(5,2))


createDist <-function(i){
  z <- rnorm(sampleSize,  mean = mymean, sd = mystd)
  title <- paste("Run", i, "Normal mean=",mymean," std.dev=",mystd)
  hist(z,breaks = 20,main = title)
}

createDist(1)
createDist(2)
createDist(3)
createDist(4)
createDist(5)
createDist(6)
createDist(7)
createDist(8)
createDist(9)
createDist(10)


lapply(1:10,createDist);

par(mfrow = c(1,1))









### Another way to show a sampling distribution. This uses R's built in "Replicate"


sampleSize <- 10000

repls <- 500

mytitle <- paste("Replications=",repls,"SampleSize=",sampleSize,"Mean=",mymean,"Std.Dev=",mystd)

means <- replicate(repls,mean(rnorm(sampleSize,mean = mymean,sd = mystd)))

myhist <- hist(means, main = mytitle, breaks = 15)

# Display some results in text inside the histo

# I know there should be a better way, but don't know it

# Note that in text, \n has a "next line" effect

mylabel <- paste("E(x)=",mymean,"\nobserved

mean=",round(mean(means),2),"\nobserved std.dev \nof means=",round(sd(means),2))

# This puts text into the histogram, guessing

# the positions from the 6'th bin's position

text(myhist$breaks[6],.8*max(myhist$counts),mylabel ,pos = 2)







sampleSize <- 10000

repls <- 500

mytitle <- paste("uniforms")

means <- replicate(repls,mean(runif(sampleSize,min = 0,max = 222)))

par(mfcol = c(5,2))

for (i in 1:10)
hist(runif(sampleSize, min = 0, max = 222))

x11()

myhist <- hist(means, main = mytitle, breaks = 15)

# Display some results in text inside the histo

# I know there should be a better way, but don't know it

# Note that in text, \n has a "next line" effect

mylabel <- paste("E(x)=",mymean,"\nobserved

mean=",round(mean(means),2),"\nobserved std.dev \nof means=",round(sd(means),2))

# This puts text into the histogram, guessing

# the positions from the 6'th bin's position

text(myhist$breaks[6],.8*max(myhist$counts),mylabel ,pos = 2n)

## Title: plot-barplot-1
## Author: Paul Johnson <pauljohn at ku.edu>
## Date posted: 2013-02-05
## Description. Basic exploration of barplot

## One big hassle is that R barplot() assumes we
## provide the table of bar heights.  We get one
## "set" of bars for each row in the table.

## Start easy:

## Suppose we know how high we want the bars.
##

myBars <- c(0.2, 0.4, 0.5)
barplot(height = myBars)

## fiddle the width of the bars
barplot(height = myBars, width = c(0.2, 1, 0.3))

## Sometimes we have 2 rows of data to plot. That
## is a matrix

myBars <- matrix(c(0.2, 0.4, 0.5, 0.33, 0.7, 0.1), nrow = 2, byrow = TRUE)

barplot(height = myBars)
## I rather have side-by-side plot
barplot(height = myBars, beside = TRUE)


## We will play games with barplot, but first
## "HOW DO WE GET THE NUMBERS WE WANT TO PLOT?"

## Well, it depends on what you are trying to show.

## Answer: I usually use a 2 step process.
## Step 1. table() gets the counts
## Step 2. prop.table() converts that to proportions.

## Let's make up some categorical variables for testing

## Here is 100 scores for x1
set.seed(234234)
x1 <- sample(c("Red","Black","Blue"), size = 100, replace = TRUE)
x2 <- sample(c("male","female"), size = 100, replace = TRUE)

x1 <- as.factor(x1)
x2 <- as.factor(x2)

plot(x2, x1)

## I want a side-by-side barplot instead.

## So we have to manufacture the data ourselves
table(x2, x1)

t1 <- table(x2, x1)
t1.prop <- prop.table(t1, margin = 2)

## See: column proportions.
t1.prop

barplot(height = t1.prop, beside = TRUE)

## Sometimes you want the average value of a variable
## to serve as the height of the bars.  R leaves us (too) many
## different ways to get that. I like "aggregate".

## Lets make up a numeric variable

x3 <- rnorm(100, m = 40, s = 20)

## We need the result to be a table, with one column for male, one
## for female, and rows for Red Black Blue

## My first instinct was to calculate like this
aggregate(x3, by = list(x1,x2), mean)

## But the data is not formatted in a way that easily
## goes into a table.

## The gdata package has a nice function that does it
## called aggregate.table, but if you run it like so:
## library(gdata)
## aggregate.table(x3, by1 = x1, by2 = x2, mean)
##
## it says the function is deprecated, and instead it
## suggests:

tapply(X = x3, INDEX = list(x1, x2), FUN = mean)

## That looks ok

myBars <- tapply(X = x3, INDEX = list(x1, x2), FUN = mean)
barplot(height = myBars)

barplot(height = myBars, beside = TRUE)

## Lets fiddle with the bar labels
##
barplot(height = myBars, beside = TRUE, names.arg = colnames(myBars))
##
barplot(height = myBars, beside = TRUE, names.arg = c(row.names(myBars),row.names(myBars)))


## I'd like to get both labels.
## Unfortunately, if we want to add labels for the groups.

## But we don't know where to place the labels. For example
mtext(colnames(myBars), side = 1, line = 2, at = c(1,2))
## This is no good as well.
mtext(colnames(myBars), side = 1, line = 2, at = c(4,8))

## So, we have to ask the barplot for its coordinate system,
## like this:

bp1 <- barplot(height = myBars, beside = TRUE, names.arg = c(row.names(myBars),row.names(myBars)))

## Inspect bp1, it is just positions on the horizontal scale

bp1

## The positions of the bars are there, but you have to look at the layout
## for a while to make sense of it.

## The numbers indicate that the "center" for each group of bars is found at
## positions 2.5 and 6.5.

mtext(colnames(myBars), side = 1, line = 3, at = c(2.5, 6.5))

## The end
##
##



##
##
## But wait, there's more.  Apparently I wrote the exact same
## example before. And forgot. How silly.

## Anyway, in case more examples help.

## Paul Johnson
## barplot data input. What a Hassle.
## 2011-06-22

## Here's a case where it is best to understand what R wants,
## before trying to work your example.  Or end goal is to
## make a grouped barplot.

## Start easy, give barplot one column


x <- c(.14, .23, .66)
barplot(x)


## table or aggregate can produce same kind of thing
## Get some small integers in a data frame
rawdata <- rpois(200, lambda = 2)

x <- table(rawdata)

## Convert to proportions

x <- x / sum(x)

barplot(x)


## Now work on richer information

## Suppose the input is a matrix with 2 columns

x <- matrix( c(.14, .23, .66, .44, .53, .55), ncol = 2)

## look at x

x

barplot(x)

## I hate stacked charts

## I think this would be called a grouped bar plot.
barplot(x, beside = TRUE)

## That has no names because my input table had no names.

## Think of the columns as sex
colnames(x) <- c("Male","Female")
x

## Row represents cities
rownames(x) <- c("NY","LA","SF")
x


barplot(x, beside = TRUE)

## How to decorate that?
## Name individual bars? OK:
barplot(x, beside = TRUE, names.arg = c("A","B","C","D","E","F"))

## Instead, lets go for two-layered output.
## Grab the output from barplot in order
## to see where bars are positioned.
bp1 <- barplot(x, beside = TRUE)

mtext(text = c("first","second","third"), side = 1, line = 0, at= bp1[,1])

mtext(text = c("fourth","fifth","sixth"), side = 1, line = 0, at = bp1[ ,2])

## Instead, lets write vertically inside the bars!
## Let's write at one-half of the column's height (that's why
## I have 0.5*x in the text commands

bp1 <- barplot(x, beside = TRUE)
text( bp1[ ,1], 0.5*x[ ,1], c("first","second","third"))

## srt will rotate text strings by degree
bp1 <- barplot(x, beside = TRUE)
text( bp1[ ,1], 0.5*x[ ,1], c("first","second","third"), srt = 66)


bp1 <- barplot(x, beside = TRUE)
text( bp1[ ,1], 0.5*x[ ,1], c("first","second","third"), srt = 90)
text( bp1[ ,2], 0.5*x[ ,2], c("first","second","third"), srt = 90)


### Note problem: Fill colors in legend not correct
bp1 <- barplot(x, beside = TRUE)
legend("topleft", legend = c("first","second","third"), fill = c(1,2,3))

### Need to figure out what colors barplot uses
### I believe it is drawing colors from the function "gray.colors"
gray.colors(3)

bp1 <- barplot(x, beside = TRUE)
legend("topleft", legend = c("first","second","third"), fill= gray.colors(3))



### So, what do you get out of this?

### barplot wants you to give it a matrix, one column per group of bars.

### So if your data is like this


### data
### id  sex region iq age
### 01  M    W     122 12
### 02  F    W     111 08
### 03  M    E     89  07
### 04  F    S     144 19
### 05  F    N     123 44

### You want a barplot that shows this the
## mean "iq" subdivided by sex, then region

##          | |               |
##        | | |               |
##      | | | |               | | | |
##      E W N S               E W N S

##         Male                 Female

## So we need a matrix with 2 columns, Male and Female,
## Rows for regions and cells are means.

## First, manufacture the data

id <- 1:1000
sex <- sample(x= c("M","F"), size = 1000, replace = T)
region <- sample(x= c("E","W","N","S"), size = 1000, replace = T)
iq <- rnorm(1000, m = 100, sd = 15)
age <- rpois(1000, lambda = 20)
dat <- data.frame(id, sex, region, iq, age)

## Use R's "aggregate" go produce that
aggdat <- aggregate(dat$iq, by = list(sex = sex,region = region), FUN= mean)
aggdat
colnames(aggdat)[3] = "meaniq"

## aggdat is in the "long" format, but we need the "wide" format
x <- unstack(aggdat, meaniq ~ sex )
x <- as.matrix(x)
barplot(x, beside = TRUE)


## And I think that's all I need to show

set.seed(123)
x1 <- rnorm(100)
x2 <- rnorm(100)
x3 <- rnorm(100)
x4 <- rnorm(100)
y <- rnorm(100)
dat <- data.frame(y, x1,x2,x3,x4)
rm(x1,x2,x3,x4,y)
m1 <- lm(y~ x1*x2 + x4, data = dat)
 
m1RC <- residualCenter(m1)

m1RCs <- summary(m1RC)
## The stage 1 centering regressions can be viewed as well
## lapply(m1RC$rcRegressions, summary)

## Verify residualCenter() output against the manual calculation
dat$x1rcx2 <- as.numeric(resid(lm(I(x1*x2) ~ x1 + x2, data = dat)))
m1m <- lm(y ~ x1 + x2 + x4 + x1rcx2, data=dat)
summary(m1m)
cbind("residualCenter" = coef(m1RC), "manual" = coef(m1m))


m2 <- lm(y~ x1*x2*x3 + x4, data=dat)
m2RC <- residualCenter(m2)
m2RCs <- summary(m2RC)

## Verify that result manually
dat$x2rcx3 <- as.numeric(resid(lm(I(x2*x3) ~ x2 + x3, data = dat)))
dat$x1rcx3 <- as.numeric(resid(lm(I(x1*x3) ~ x1 + x3, data = dat)))
dat$x1rcx2rcx3 <- as.numeric( resid(lm(I(x1*x2*x3) ~ x1 + x2 + x3 + x1rcx2 +
                                       x1rcx3 + x2rcx3 , data=dat)))
(m2m <- lm(y ~ x1 + x2 + x3+ x4 + x1rcx2 + x1rcx3 + x2rcx3 + x1rcx2rcx3,
           data = dat))

cbind("residualCenter" = coef(m2RC), "manual" = coef(m2m))


### As good as pequod's lmres
### not run because pequod generates R warnings
###
### if (require(pequod)){
###  pequodm1 <- lmres(y ~ x1*x2*x3 + x4, data=dat) 
###  pequodm1s <- summary(pequodm1)
###  coef(pequodm1s)
### }

### Works with any number of interactions. See:

m3 <- lm(y~ x1*x2*x3*x4, data=dat)
m3RC <- residualCenter(m3)
summary(m3RC)
##'
## Verify that one manually (Gosh, this is horrible to write out)
dat$x1rcx4 <- as.numeric(resid(lm(I(x1*x4) ~ x1 + x4, data=dat)))
dat$x2rcx4 <- as.numeric(resid(lm(I(x2*x4) ~ x2 + x4, data=dat)))
dat$x3rcx4 <- as.numeric(resid(lm(I(x3*x4) ~ x3 + x4, data=dat)))
dat$x1rcx2rcx4 <- as.numeric(resid(lm(I(x1*x2*x4) ~ x1 + x2 + x4 +
                                      x1rcx2 + x1rcx4 + x2rcx4, data=dat)))
dat$x1rcx3rcx4 <- as.numeric(resid(lm(I(x1*x3*x4) ~ x1 + x3 + x4 +
                                      x1rcx3 + x1rcx4 + x3rcx4, data=dat)))
dat$x2rcx3rcx4 <- as.numeric(resid(lm(I(x2*x3*x4) ~ x2 + x3 + x4 +
                                      x2rcx3 + x2rcx4 + x3rcx4, data=dat)))
dat$x1rcx2rcx3rcx4 <-
    as.numeric(resid(lm(I(x1*x2*x3*x4) ~ x1 + x2 + x3 + x4 +
                        x1rcx2 + x1rcx3 + x2rcx3 + x1rcx4  + x2rcx4 +
                        x3rcx4  + x1rcx2rcx3 + x1rcx2rcx4 + x1rcx3rcx4 +
                        x2rcx3rcx4, data=dat)))
(m3m <- lm(y ~ x1 + x2 + x3 + x4 + x1rcx2 + x1rcx3 + x2rcx3 + x1rcx4 +
           x2rcx4 + x3rcx4 + x1rcx2rcx3 + x1rcx2rcx4 + x1rcx3rcx4 +
           x2rcx3rcx4 + x1rcx2rcx3rcx4, data=dat))

cbind("residualCenter"=coef(m3RC), "manual"=coef(m3m))

### If you want to fit a sequence of models, as in pequod, can do.

tm <-terms(m2)
tmvec <- attr(terms(m2), "term.labels")
f1 <- tmvec[grep(":", tmvec, invert = TRUE)]
f2 <- tmvec[grep(":.*:", tmvec, invert = TRUE)]
f3 <- tmvec[grep(":.*:.*:", tmvec, invert = TRUE)]

## > f1
## [1] "x1" "x2" "x3" "x4"
## > f2
## [1] "x1"    "x2"    "x3"    "x4"    "x1:x2" "x1:x3" "x2:x3"
## > f3
## [1] "x1"       "x2"       "x3"       "x4"       "x1:x2"    "x1:x3"    "x2:x3"   
## [8] "x1:x2:x3"

f1 <- lm(as.formula(paste("y","~", paste(f1, collapse=" + "))), data=dat)
f1RC <- residualCenter(f1)
summary(f1RC)

f2 <- lm(as.formula(paste("y","~", paste(f2, collapse=" + "))), data=dat)
f2RC <- residualCenter(f2)
summary(f2RC)

f3 <- lm(as.formula(paste("y","~", paste(f3, collapse=" + "))), data=dat)
f3RC <- residualCenter(f3)
summary(f3RC)

library(rockchalk)


set.seed(23452345)
N <- 100
x1 <- gl(12, 2, labels = LETTERS[1:12])
x2 <- gl(8, 3, labels = LETTERS[12:24])
x1 <- sample(x = x1, size=N, replace = TRUE)
x2 <- sample(x = x2, size=N, replace = TRUE)
z1 <- rnorm(N)
a1 <- rnorm(N, mean = 1.2, sd = 11.7)
a2 <- rpois(N, lambda = 10 + abs(a1))
a3 <- rgamma(N, 0.5, 4)
b1 <- rnorm(N, mean = 211.3, sd = 0.4)
dat <- data.frame(z1, a1, x2, a2, x1, a3, b1)
summary(dat)

summarize(dat)

summarize(dat, digits = 4)

summarize(dat, stats = c("min", "max", "mean", "sd"),
          probs = c(0.25, 0.75))

summarize(dat, probs = c(0, 0.20, 0.80),
          stats = c("nobs", "mean", "med", "entropy"))

summarize(dat, probs = c(0, 0.20, 0.50),
          stats = c("nobs", "nmiss", "mean", "entropy"), maxLevels=10)

dat.sum <- summarize(dat, probs = c(0, 0.20, 0.50),
                     stats = c("nobs", "nmiss", "mean", "entropy"), maxLevels=10)
dat.sum
## Inspect unformatted structure of objects within return
dat.sum[["numerics"]]
dat.sum[["factors"]]

## Only quantile values, no summary stats for numeric variables
## Discrete variables get entropy
summarize(dat, 
          probs = c(0, 0.25, 0.50, 0.75, 1.0),
          stats = "entropy", digits = 2)

## Quantiles and the mean for numeric variables.
## No diversity stats for discrete variables (entropy omitted)
summarize(dat, 
          probs = c(0, 0.25, 0.50, 0.75, 1.0),
          stats = "mean")

summarize(dat, 
          probs = NULL,
          stats = "mean")

## Note: output is not beautified by a print method
dat.sn <- summarizeNumerics(dat)
dat.sn
formatSummarizedNumerics(dat.sn)
formatSummarizedNumerics(dat.sn, digits = 5)

dat.summ <- summarize(dat)


dat.sf <- summarizeFactors(dat, maxLevels = 20)
dat.sf
formatSummarizedFactors(dat.sf)

## See actual values of factor summaries, without
## beautified printing
summarizeFactors(dat, maxLevels = 5)
formatSummarizedFactors(summarizeFactors(dat, maxLevels = 5))

summarize(dat, alphaSort = TRUE) 

summarize(dat, digits = 6, alphaSort = FALSE)


summarize(dat, maxLevels = 2)

datsumm <- summarize(dat, stats = c("mean", "sd", "var", "entropy", "nobs"))

## Unbeautified numeric data frame, variables on the rows
datsumm[["numerics"]]
## Beautified versions 1. shows saved version:
attr(datsumm, "numeric.formatted")
## 2. Run formatSummarizedNumerics to re-specify digits:
formatSummarizedNumerics(datsumm[["numerics"]], digits = 10)

datsumm[["factors"]]
formatSummarizedFactors(datsumm[["factors"]])
formatSummarizedFactors(datsumm[["factors"]], digits = 6, maxLevels = 10)

## Filename: plot-Normal-1.R
## Paul Johnson March 31, 2008

## This code
## should be available somewhere in
## http://pj.freefaculty.org/R/WorkingExamples.  If it is not email
## me <pauljohn@ku.edu>



mymean <- 0

mystd <- 1.5

myx <- seq( mymean - 3*mystd,  mymean+ 3*mystd, length.out = 500)


myDensity <- dnorm(myx,mean = mymean,sd = mystd)

plot(myx, myDensity, type = "n", xlab = "X", ylab = "Probability Density ")

lines(myx,myDensity,lty = 4, col = 4) ### change line type & color if you want


#maybe broaden out x

myx <- seq( mymean - 6*mystd,  mymean+ 6*mystd, length.out = 500)

myDensity <- dnorm(myx,mean = mymean,sd = mystd)
plot(myx, myDensity, type = "n", xlab = "X", ylab = "Probability Density ")

lines(myx,myDensity,lty = 4, col = 4) ### change line type & color if you want



par(mfcol = c(2,1))


plot(myx,  myDensity, type = "n",  xlab = "X", ylab = "Probability Density Function")
lines(myx, myDensity )

myCumulProb <- pnorm(myx, mean = mymean, sd = mystd)
plot(myx, myCumulProb, type = "n", xlab = "X", ylab = "Cumulative Distribution Function")
lines(myx,myCumulProb,lty = 4, col = 4) ### change line type & color if you want


par(mfcol = c(1,1))




# What does one random sample from this distribution look like?

mySample <- rnorm(50, mean = mymean, sd = mystd)
hist(mySample, freq = FALSE, xlab = "x", main = "Histogram and Density of one Sample")
hist(mySample, freq = FALSE, xlab = "x", main = "Histogram and Density of one Sample", breaks = 20)

lines(density(mySample))



# Compare against true probabilities
## 4 lines simply re-do previous 4, in case you closed the graph already.
mySample <- rnorm(50, mean = mymean, sd = mystd)
hist(mySample, freq = FALSE, xlab = "x", main = "Histogram and Density of one Sample", breaks = 20)
lines(density(mySample))

xrange <- seq(min(mySample), max(mySample), by = 0.1)
trueProbs <- dnorm(xrange,mean = mymean,sd = mystd)

lines( xrange, trueProbs, lty = 6, col = 7)




hist (mySample, freq = FALSE, xlab = "X", main= paste("Normal Sample (50 Observations)\n Mean=", mymean, "Std.Dev=", mystd))



t1 <-  bquote( mu== .(mymean))

t2 <-  bquote( sigma== .(mystd))


hist (mySample, freq = FALSE, xlab = "X", main = t1 )

## Can't figure how to combine t1 and t2 into plot title

## Can drop sigma in as margin text, though, on side 3.
mtext(t2, 3)

library(rockchalk)
dat <- genCorrelatedData(1000, stde=5)

m1 <- lm(y ~ x1 * x2, data=dat)

m1mc <- meanCenter(m1)
summary(m1mc)

m1rc <- residualCenter(m1)
summary(m1rc)


newdf <- apply(dat, 2, summary)
newdf <- as.data.frame(newdf)

predict(m1rc, newdata=newdf)

library(rockchalk)
N <- 100
dat <- genCorrelatedData(N = N, means = c(100, 200), sds = c(20, 30),
                         rho = 0.4, stde = 10)
dat$x3 <- rnorm(100, m = 40, s = 4)

m1 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m1)
mcDiagnose(m1)

m1c <- meanCenter(m1)
summary(m1c)
mcDiagnose(m1c)

m2 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m2)
mcDiagnose(m2)

m2c <- meanCenter(m2, standardize = TRUE)
summary(m2c)
mcDiagnose(m2c)

m2c2 <- meanCenter(m2, centerOnlyInteractors = FALSE)
summary(m2c2)

m2c3 <- meanCenter(m2, centerOnlyInteractors = FALSE, centerDV = TRUE)
summary(m2c3)

dat <- genCorrelatedData(N = N, means = c(100, 200), sds = c(20, 30),
                         rho = 0.4, stde = 10)
dat$x3 <- rnorm(100, m = 40, s = 4)
dat$x3 <- gl(4, 25, labels = c("none", "some", "much", "total"))

m3 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m3)
## visualize, for fun
plotPlane(m3, "x1", "x2")

m3c1 <- meanCenter(m3)
summary(m3c1)

## Not exactly the same as a "standardized" regression because the
## interactive variables are centered in the model frame,
## and the term "x1:x2" is never centered again.
m3c2 <- meanCenter(m3, centerDV = TRUE,
                   centerOnlyInteractors = FALSE, standardize = TRUE)
summary(m3c2)

m3st <- standardize(m3)
summary(m3st)

## Make a bigger dataset to see effects better
N <- 500
dat <- genCorrelatedData(N = N, means = c(200,200), sds = c(60,30),
                         rho = 0.2, stde = 10)
dat$x3 <- rnorm(100, m = 40, s = 4)
dat$x3 <- gl(4, 25, labels = c("none", "some", "much", "total"))
dat$y2 <- with(dat,
               0.4 - 0.15 * x1 + 0.04 * x1^2 -
               drop(contrasts(dat$x3)[dat$x3, ] %*% c(-1.9, 0, 5.1))  +
               1000* rnorm(nrow(dat)))
dat$y2 <- drop(dat$y2)

m4literal <- lm(y2 ~ x1 + I(x1*x1) + x2 + x3, data = dat)
summary(m4literal)
plotCurves(m4literal, plotx="x1")
## Superficially, there is multicollinearity (omit the intercept)
cor(model.matrix(m4literal)[ -1 , -1 ])

m4literalmc <- meanCenter(m4literal, terms = "x1")
summary(m4literalmc)

m4literalmcs <- meanCenter(m4literal, terms = "x1", standardize = TRUE)
summary(m4literalmcs)

m4 <- lm(y2 ~ poly(x1, 2, raw = TRUE) + x2 + x3, data = dat)
summary(m4)
plotCurves(m4, plotx="x1")

m4mc1 <- meanCenter(m4, terms = "x1")
summary(m4mc1)

m4mc2 <- meanCenter(m4, terms = "x1", standardize = TRUE)
summary(m4mc2)

m4mc3 <- meanCenter(m4, terms = "x1", centerDV = TRUE, standardize = TRUE)
summary(m4mc3)

library(rockchalk)
library(carData)
m1 <- lm(statusquo ~ income * age + education + sex + age, data = Chile)
m1ps <- plotSlopes(m1, modx = "income", plotx = "age")
m1psts <- testSlopes(m1ps)
plot(m1psts)


dat2 <- genCorrelatedData(N = 400, rho = .1, means = c(50, -20),
                          stde = 300, beta = c(2, 0, 0.1, -0.4))
m2 <- lm(y ~ x1*x2, data = dat2)
m2ps <- plotSlopes(m2, plotx = "x1", modx = "x2")
m2psts <- testSlopes(m2ps)
plot(m2psts)
m2ps <- plotSlopes(m2, plotx = "x1", modx = "x2", modxVals = "std.dev", n = 5)
m2psts <- testSlopes(m2ps)
plot(m2psts)

## Try again with longer variable names

colnames(dat2) <- c("oxygen","hydrogen","species")
m2a <- lm(species ~ oxygen*hydrogen, data = dat2)
m2aps1 <- plotSlopes(m2a, plotx = "oxygen", modx = "hydrogen")
m2aps1ts <- testSlopes(m2aps1)
plot(m2aps1ts)
m2aps2 <- plotSlopes(m2a, plotx = "oxygen", modx = "hydrogen",
                     modxVals = "std.dev", n = 5)
m2bps2ts <- testSlopes(m2aps2)
plot(m2bps2ts)



dat3 <- genCorrelatedData(N = 400, rho = .1, stde = 300,
                          beta = c(2, 0, 0.3, 0.15),
                          means = c(50,0), sds = c(10, 40))
m3 <- lm(y ~ x1*x2, data = dat3)
m3ps <- plotSlopes(m3, plotx = "x1", modx = "x2")
m3sts <- testSlopes(m3ps)
plot(testSlopes(m3ps))
plot(testSlopes(m3ps), shade = FALSE)

## Finally, if model has no relevant interactions, testSlopes does nothing.
m9 <- lm(statusquo ~ age + income * education + sex + age, data = Chile)
m9ps <- plotSlopes(m9, modx = "education", plotx = "age", plotPoints = FALSE)
m9psts <- testSlopes(m9ps)

cat("\n\n Basics: R math, workspace, indexes, class, seq, rep, identical, all.equal \n\n")

1 + 4
exp(7)
sqrt(81)
log(4)
4^2

x <- c(1,2,3,4,5,6)
x

ls()
objects() #same

x2 <- seq(1, 6, by = 1)
x2
x3 <- seq(1, 6)
x3
x4 <- 1:6
x4
identical(x, x2, x3, x4)

## caution: why identical if class not same?
class(x)
class(x2)
class(x3)
class(x4)

xint <- as.integer(x)
class(xint)

x2 <- seq(1L, 6L, by = 1L)
class(x2)

x2 <- seq.int(1,6)
class(x2)


ls()
rm(x, x2, x3, x4)
ls()

x <- seq(0.5, 4.1, by = 0.3)
x

x^2
exp(x)
sqrt(x)
log(x)
1/x


x[4]
x[4:6]
x[c(1,4,5)]
x[ x > 4 ]
x[ -2 ]
x[ -c(2,3) ]


attributes(x)
is.null(x)
is.vector(x)
is.numeric(x)
is.integer(x)
is.character(x)
is.data.frame(x)
is.logical(x)
is.matrix(x)
is.list(x)

rm(x)

y <- c(80, 90, 100, 110, 120, 130)
x <- seq_along(y)
x
x + y
x - y
y * x
x * y
y^x
3*x + 4*y

w <- 2*x + sqrt(y)
w
mxyw <- cbind(x,y,w)
class(mxyw)
mxyw[ , 2:3]
mxyw[ , "w"]
mxyw[ , c("y", "w")]
mxyw[ 2:3, "w"]


z <- c(TRUE, FALSE, TRUE, TRUE, TRUE, FALSE)
is.logical(z)
which(z == TRUE)
which(z)
which(z != TRUE)
which(!z)

print("Exclamation mark (!) means NOT")
cat("Exclamation mark (!) means NOT \n")


zTrueIndex <- which(z == TRUE)
x[zTrueIndex]
y[zTrueIndex]
x[zTrueIndex] + y[zTrueIndex]

xy <- data.frame(x, y, w, z)
xy
colnames(xy)
dim(xy)
fix(xy) #Hit "Quit"

xy[1, ]
xy[ ,2]
xy[1:3, ]
xy[c(1,4,6), ]
xy[ , -3]
xy[-c(1,2), ]
xy[ , "w"]
xy[ , c("x","y")]
xy[ , c("z")]
xy[["z"]]
xy[xy$z, ]
xy[!xy$z, ]


newx1 <- xy[ , "x"]
newx1
is.data.frame(newx1)
class(newx1)
is.vector(newx1)

newx2 <- xy[ , "x", drop=F] # drop: R magic/curse?
newx2
is.data.frame(newx2)
class(newx2)
is.vector(newx2)


all.equal(newx1, newx2)
all.equal(newx1, as.vector(newx2))
identical(newx1, as.vector(newx2))



subset(xy, subset = x < 3)
xy[ x<3, ]
xy[ x<3 & z == TRUE, ]
xy[ x<3 & z, ]
xy[ x<3 & !z, ]





x1 <- c(1,1,1,1,1,1,1,1,1,1)
x2 <- rep(1, times = 10)
x2
x2 <- rep(1, 10)
x1 == x2
identical(x1, x2)



rep(c(1, 5, 8), length.out = 9)
rep(c(1, 5, 8), each = 3)
rep(c(1, 5, 8), each = 3, length.out = 9)
rep(c(1, 5, 8), each = 3, length.out = 10)
rep(c(1, 5, 8), times = 3)
rep(c(1, 5, 8), each = 3, times = 3)





rm(x)
x <- vector(12, mode="integer")
x
x[4] <- 13L
class(x)
is.integer(x)
x[5] <- -5L
is.integer(x)
x[9] <- -11
is.integer(x) # type promotion b/c -11 not integer
class(x)
x <- as.integer(x)
is.integer(x)
class(x)
rm(x)

x <- vector(12, mode="integer")
x
x[4] <- 13
class(x)
is.integer(x)
x[5] <- -5L
is.integer(x)
x[9] <- -11.1
is.integer(x)
x <- as.integer(x) #rounds
is.integer(x)
class(x)


xs1 <- seq(from = 5, to = 20, by=1)
xs2 <- seq(from = 5, to = 20, length.out=16)
xs1 == xs2
identical(xs1, xs2)

xs3 <- 5:20
xs1 == xs3
identical(xs1, xs3) ##hmmm. huh?
is.vector(xs3)
is.vector(xs1)
length(xs1)
length(xs3)
all.equal(xs1, xs3) #ok!

which( xs1 != xs3 )
which( xs1 == xs3 )

for( i in seq_along(xs3)){
  print(xs1[i]-xs3[i])
}
## hmmm. again
is.integer(xs1)
is.numeric(xs1)
is.numeric(xs3)
is.integer(xs3) ## aha!
class(xs1)
class(xs2)
class(xs3)

identical(xs3, as.integer(xs1)) ##puzzle solved

### Filename: Normal1_2009_plotmathExample.R
### Paul Johnson June 3, 2009
### This code should be available somewhere in http://pj.freefaculty.org/R.  If it is not
### email me <pauljohn@ku.edu>


###Set mu and sigma at your pleasure:
mu <- 10.03
sigma <- 12.5786786

myx <- seq( mu - 3.5*sigma,  mu+ 3.5*sigma, length.out = 500)

myDensity <- dnorm(myx, mean = mu, sd = sigma)


# It is challenging to combine plotmath with values of mu and sigma in one expression.
# Either bquote or substitute can be used.  First use bquote:

myTitle1 <- bquote (paste("x ~ Normal(", mu== .(round(mu, 2)), ',', sigma== .(round(sigma, 2)),")") )

### Using substitute:
### myTitle1 <-  substitute( "x ~ Normal" ~~ group( "(", list(mu==mu1, sigma^2==sigma2)#, ")") ,  list(mu1=round(mu, 2), sigma2=round(sigma^2, 2)))

### Or combine the two:
### t1 <- substitute ( mu == a ,  list (a = mu))
### t2 <- substitute ( sigma == a, list(a = round(sigma, 2)))
### myTitle1 <- bquote(paste("x ~ Normal(", .(t1), ",", .(t2),")" ) )


## To save the plot in a file, change FALSE to TRUE
saveplot <- FALSE

if (saveplot == TRUE){
  pdf(file = "Normal-2009.pdf", height = 6.5, width = 6.5, onefile = F, paper = "special", pointsize = 10)

}else {
  dev.new(height = 6.5, width = 6.5, pointsize = 9)
}
### xpd needed to allow writing outside strict box of graph
par(xpd = TRUE, ps = 10)

plot(myx, myDensity, type = "l", xlab = "x", ylab = "Probability Density ", main = myTitle1, axes = FALSE)
axis(2, pos= mu - 3.6*sigma)
axis(1, pos = 0)
lines(c(myx[1], myx[length(myx)]), c(0, 0)) ### closes off axes

# bquote creates an expression that text plotters can use
t1 <-  bquote( mu== .(mu))

# Find a value of myx that is "very close to" mu
centerX <- max(which (myx <= mu))
# plot light vertical line under peak of density
lines( c(mu, mu), c(0, myDensity[centerX]), lty= 14, lwd = .2)

# label the mean in the bottom margin
mtext(bquote( mu == .(mu)), 1, at = mu, line = -1)

### find position 20% "up" vertically, to use for arrow coordinate
ss = 0.2 * max(myDensity)
# Insert interval to represent width of one sigma
arrows( x0 = mu, y0= ss, x1 = mu+sigma, y1 = ss, code = 3, angle = 90, length = 0.1)

### Write the value of sigma above that interval
t2 <-  bquote( sigma== .(round(sigma, 2)))
text( mu+0.5*sigma, 1.15*ss, t2)

### Create a formula for the Normal
normalFormula <- expression (f(x) == frac (1, sigma* sqrt(2*pi)) * e^{~~ - ~~ frac(1,2)~~ bgroup("(", frac(x-mu,sigma),")")^2})
# Draw the Normal formula
text ( mu + 0.5*sigma, max(myDensity)- 0.10 * max(myDensity),  normalFormula, pos = 4)

### Theory says we should have 2.5% of the area to the left of: -1.96 * sigma.
### Find the X coordinate of that "critical value"
criticalValue <- mu -1.96 * sigma
### Then grab all myx values that are "to the left" of that critical value.
specialX <-  myx[myx <= criticalValue]

### mark the critical value in the graph
text ( criticalValue, 0 , label= paste(round(criticalValue, 2)), pos = 1)
### Take sequence parallel to values of myx inside critical region
specialY <- myDensity[myx < criticalValue]
#  Polygon makes a nice shaded illustration
polygon(c(specialX[1], specialX, specialX[length(specialX )]), c(0, specialY, 0), density = c(-110), col = "lightgray" )

shadedArea <- round(pnorm(mu - 1.96 * sigma, mean = mu, sd = sigma), 4)


### I want to insert annotation about area on left side.

al1 <- bquote(Prob(x <= .(round(criticalValue, 2))))
al2 <- bquote(phantom(0) == F( .(round(criticalValue, 2)) ))
al3 <- bquote(phantom(0) == .(round(shadedArea, 3)))

### Hard to position this text "just right"
### Have tried many ideas, this may be least bad.
### Get center position in shaded area
medX <- median(specialX)
### density at that center point:
denAtMedX <- myDensity[indexMed <- max(which(specialX < medX))]

text(medX, denAtMedX+0.0055, labels = al1)
text(medX, denAtMedX+0.004, labels = al2)
text(medX, denAtMedX+0.0025, labels = al3)

### point from text toward shaded area
arrows( x0 = medX, y0 = myDensity[indexMed]+0.002 , x1= mu-2.5 *sigma, y1= 0.40*myDensity[length(specialX)] ,   length = 0.1)



ss <- 0.1 * max(myDensity)
### Mark interval from mu to mu-1.96*sigma
arrows( x0 = mu, y0= ss, x1 = mu-1.96*sigma, y1 = ss, code = 3, angle = 90, length = 0.1)
### Put text above interval
text( mu - 2.0*sigma, 1.15*ss, bquote(paste(.(round(criticalValue, 2)),phantom(1) = =mu-1.96 * sigma,sep = "")), pos = 4 )




criticalValue <- mu +1.96 * sigma
### Then grab all myx values that are "to the left" of that critical value.
specialX <-  myx[myx >= criticalValue]

### mark the critical value in the graph
text ( criticalValue, 0 , label= paste(round(criticalValue, 2)), pos = 1)
### Take sequence parallel to values of myx inside critical region
specialY <- myDensity[myx > criticalValue]
#  Polygon makes a nice shaded illustration
polygon(c(specialX[1], specialX, specialX[length(specialX )]), c(0, specialY, 0), density = c(-110), col = "lightgray" )

shadedArea <- round(pnorm(mu + 1.96 * sigma, mean = mu, sd = sigma, lower.tail = F), 4)


### Insert simpler comment on right side.

al2 <- bquote(phantom(0) == 1 - F( .(round(criticalValue, 2)) ))
al3 <- bquote(phantom(0) == .(round(shadedArea, 3)))

### Hard to position this text "just right"
### Have tried many ideas, this may be least bad.
### Get center position in shaded area
medX <- median(specialX)
### density at that center point:
denAtMedX <- myDensity[indexMed <- max(which(specialX < medX))]


text(medX, denAtMedX+0.004, labels = al2)
text(medX, denAtMedX+0.0025, labels = al3)

### point from text toward shaded area
arrows( x0 = medX, y0 = myDensity[indexMed]+0.002 , x1= mu+2.5 *sigma, y1= 0.40*myDensity[length(specialX)] ,   length = 0.1)



ss <- 0.05 * max(myDensity)
### Mark interval from mu to mu+1.96*sigma
arrows( x0 = mu, y0= ss, x1 = mu+1.96*sigma, y1 = ss, code = 3, angle = 90, length = 0.1)
### Put text above interval
text( mu + 1.96*sigma, 1.15*ss, bquote(paste(.(round(criticalValue, 2)), phantom(1) = =mu+1.96 * sigma, sep = "")), pos = 2 )


if (saveplot == TRUE)  dev.off()

library(rockchalk)
N <- 100
dat <- genCorrelatedData(N = N, means = c(100,200), sds = c(20,30), rho = 0.4, stde = 10)
dat$x3 <- rnorm(100, m = 40, s = 4)

m1 <- lm(y ~ x1 + x2 + x3, data = dat)
summary(m1)

m1s <- standardize(m1)
summary(m1s)



m2 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m2)

m2s <- standardize(m2)
summary(m2s)

m2c <- meanCenter(m2)
summary(m2c)

library(rockchalk)






## first, check if model.data works when there is no data argument
## This used to fail, now OK

x1 <- rnorm(100, m = 100, s = 10)
x2 <- rnorm(100, m = 50, s =20)
y <- rnorm(100, m = 40, s = 3)

m0 <- lm(y ~ log(10+x1) + x2)
m0.data <- model.data(m0)
head(m0.data)


m1 <- lm(log(43 + y) ~ log(10+x1) + x2)
m1.data <- model.data(m1)
head(m1.data)

d <- 3

m2 <- lm(log(d + y) ~ log(10+x1) + x2)
m2.data <- model.data(m2)
head(m2.data)

m3 <- lm(log(y + d) ~ log(10+x1) + x2)
m3.data <- model.data(m3)
head(m3.data)



## check numeric and categorical predictors

x1 <- rpois(100, l=6)
x2 <- rnorm(100, m=50, s=10)
x3 <- rnorm(100)
xcat1 <- gl(2,50, labels=c("M","F"))
xcat2 <- cut(rnorm(100), breaks=c(-Inf, 0, 0.4, 0.9, 1, Inf),
             labels=c("R", "M", "D", "P", "G"))
dat <- data.frame(x1, x2, x3, xcat1, xcat2)
rm(x1, x2, x3, xcat1, xcat2)
dat$xcat1n <- with(dat, contrasts(xcat1)[xcat1, ,drop=FALSE])
dat$xcat2n <- with(dat, contrasts(xcat2)[xcat2, ])

STDE <- 20
dat$y <- with(dat,
              0.03 + 0.8*x1 + 0.1*x2 + 0.7*x3 +
              xcat1n %*% c(2) + xcat2n %*% c(0.1,-2,0.3, 0.1) +
              STDE*rnorm(100))



m1 <- lm(y ~ poly(x1, 2), data=dat)
m1.data <- model.data(m1)
head(m1.data)
attr(m1.data, "varNamesRHS")

## Check to make sure d is not mistaken for a data column
d <- 2
m2 <- lm(y ~ poly(x1, d), data=dat)
m2.data <- model.data(m2)
head(m2.data)
attr(m2.data, "varNamesRHS")


## Check to see how the 10 in log is handled
m3 <- lm(y ~ log(10 + x1) + poly(x1, d) + sin(x2), data=dat)
m3.data <- model.data(m3)
head(m3.data)
attr(m3.data, "varNamesRHS")


m4 <- lm(log(50+y) ~ log(d+10+x1) + poly(x1, 2), data=dat)
m4.data <- model.data(m4)
head(m4.data)
attr(m4.data, "varNamesRHS")


m5 <- lm(y ~ x1*x1, data=dat)
m5.data <- model.data(m5)
head(m5.data)
attr(m5.data, "varNamesRHS")


m6 <- lm(y ~ x1 + I(x1^2), data=dat)
m6.data <- model.data(m6)
head(m6.data)
attr(m6.data, "varNamesRHS")


## Put in some missings.
## poly doesn't work if there are missings, but
## can test with log
dat$x1[sample(100, 5)] <- NA
dat$y[sample(100, 5)] <- NA
dat$x2[sample(100, 5)] <- NA
dat$x3[sample(100,10)] <- NA

m1 <- lm(y ~ log(10 + x1), data=dat)
m1.data <- model.data(m1)
head(m1.data)
summarize(m1.data)
attr(m1.data, "varNamesRHS")



m2 <- lm(y ~ log(x1 + 10), data=dat)
m2.data <- model.data(m2)
head(m2.data)
summarize(m1.data)
attr(m1.data, "varNamesRHS")

d <- 2
m3 <- lm(log(50+y) ~ log(d+10+x1) + x2 + sin(x3), data=dat)
m3.data <- model.data(m3)
head(m3.data)
summarize(m3.data)
attr(m3.data, "varNamesRHS")


m4 <- lm(y ~ I(x1) + I(x1^2) + log(x2), data=dat)
m4.data <- model.data(m4)
summarize(m4.data)
attr(m4.data, "varNamesRHS")


m5 <- lm(y ~ x1 + I(x1^2) + cos(x2), data=dat)
m5.data <- model.data(m5)
head(m5.data)
summarize(m5.data)
attr(m5.data, "varNamesRHS")



## Now try with some variables in the dataframe, some not

x10 <- rnorm(100)
x11 <- rnorm(100)



m6 <- lm(y ~ x1 + I(x1^2) + cos(x2) + log(10 + x10) + sin(x11) +
         x10*x11, data = dat)
m6.data <- model.data(m6)
head(m6.data)
dim(m6.data)
summarize(m5.data)
attr(m6.data, "varNamesRHS")

## Paul Johnson
## pauljohn@ku.edu 2012-03-09
##
## This is an R program that uses some functions in the newly revised
## rockchalk package to demonstrate my point that "centered variables"
## don't really make a difference in regressions with interaction.
##
## Centering does not help with multicollinearity, but I mean to say
## more than that. It does not help with regression interpretation,
## if one correctly understands what the parameter estimates and the
## predicted values mean in a regression with interaction.

## Here the idea is the following. The centered b's and se's "seem" different,
## but they are actually calculated on the EXACT SAME fitted plane. The
## difference is that the notcentered model has the y axis positioned at
## x1=0,x2=0, while in the centered model it is instead at
## x1=meanx1, x2=meanx2. The predicted values for any x1, x2 combination
## are EXACTLY the same with either model, as are the estimates of
## uncertainty (in the form of confidence intervals or standard errors).

## Thus it should be possible to take the estimates from the
## notcentered regression and calculate the slopes at x1=meanx1,
## x2=meanx2, and re-produce them.  AND I CAN!! This demonstrates
## that claim at the end.


library(rockchalk)

set.seed(222233)
dat3 <- genCorrelatedData3(y ~ 0.1 + 0.2 * x1 + 0.3 * x2 - 0.2 * x1:x2, N = 100000, rho = .31,
                            stde = 80, means = c(x1 = 50, x2 = 50), sds = c(10, 10))

## We now explore a regression model like this:

## y = b0 + b1 * x1 + b2 *x2 + b3 *x1*x2 + e

## The mean-centered model replaces x1 and x2 by the mean-centered
## versions  of those variables, (x1 - mean(x1)) and (x2 - mean(x2)).

## The usual problem that causes researchers to turn to "mean centering"
## is the following.  The linear model seems "good", but the interactive
## model seems "bad" without centering.  Here's an example.

## First, fit the model without the interaction term.
## y = b0 + b1 * x1 + b2 *x2 + e

m1 <- lm(y ~ x1 + x2, data = dat3)
summary(m1)
dev.new()
plotPlane(m1, plotx1="x1", plotx2="x2")


## Coefficients:
##             Estimate Std. Error t value Pr(>|t|)
## (Intercept) 424.1485    45.8004   9.261 5.29e-15 ***
## x1           -8.9042     0.8499 -10.477  < 2e-16 ***
## x2           -9.2701     0.8039 -11.531  < 2e-16 ***
## ---
## Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1

## Residual standard error: 77.77 on 97 degrees of freedom
## Multiple R-squared: 0.8011,	Adjusted R-squared: 0.797
## F-statistic: 195.4 on 2 and 97 DF,  p-value: < 2.2e-16
##
## Yeah, its a "good model". Everything is "significant".
##
##
## Now the problem.
## Add an interaction. Watch, it ruins everything!
##

m2 <- lm(y ~ x1 * x2, data = dat3)
summary(m2)
dev.new()
plotPlane(m2, plotx1 = "x1", plotx2 = "x2", theta = -10)


## Coefficients:
##              Estimate Std. Error t value Pr(>|t|)
## (Intercept) -74.45719  179.94096  -0.414  0.67995
## x1            1.57257    3.75575   0.419  0.67636
## x2            0.64618    3.55471   0.182  0.85614
## x1:x2        -0.20526    0.07181  -2.859  0.00522 **
## ---
## Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1

## Residual standard error: 75.05 on 96 degrees of freedom
## Multiple R-squared: 0.8167,	Adjusted R-squared: 0.811
## F-statistic: 142.6 on 3 and 96 DF,  p-value: < 2.2e-16


## Booh, the model's "no good". x1 and x2 "don't matter"
## any more.

## We'd better mean-center x1 and x2.

m2mc <- meanCenter(m2)
summary(m2mc)

## You can verify that manually, if you don't trust my meanCenter
## function. Results are the same, of course. First, mean center the
## variables with R's scale function:

dat3$x1c <- as.numeric(scale(dat3$x1, center = TRUE, scale = FALSE))
dat3$x2c <-  as.numeric(scale(dat3$x2, center = TRUE, scale = FALSE))
## The as.numeric is required because scale returns a matrix
## with one column, not a vector. How annoying!

m2manualmc <- lm(y ~ x1c*x2c, dat3)
summary(m2manualmc)
m2mcmanualpred <- fitted(m2manualmc)

## Confirmed that matches the output from meanCenter.  I suppose
## you are sorry you did not trust me now.


## Coefficients:
##               Estimate Std. Error t value Pr(>|t|)
## (Intercept) -456.69849    8.01647 -56.970  < 2e-16 ***
## x1            -8.38383    0.84005  -9.980  < 2e-16 ***
## x2            -9.47969    0.77920 -12.166  < 2e-16 ***
## x1:x2         -0.20526    0.07181  -2.859  0.00522 **
## ---
## Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1

## Residual standard error: 75.05 on 96 degrees of freedom
## Multiple R-squared: 0.8167,	Adjusted R-squared: 0.811
## F-statistic: 142.6 on 3 and 96 DF,  p-value: < 2.2e-16


## Yeah! Mean centering saved the day! Woo Hoo! All your
## t values are big and your p values are small.

## Unfortunately, the difference in the output is just a mirage. The
## two fits are describing slope estimates at a DIFFERENT POINT on the
## same curving plane.

## To see that, note that the fitted models (centered or not centered) offer
## EXACTLY THE SAME predicted values! I don't mean "similar"
## I mean exactly the same. Look like this:

par(mfcol = c(1,2))

plotPlane(m2, "x1", "x2", plotPoints = FALSE, theta = -25,
          main = "Not Mean Centered", ticktype = "detailed")
plotPlane(m2mc, "x1", "x2", plotPoints = FALSE, theta = -25,
          main = "Mean Centered", ticktype = "detailed")

par(mfcol = c(1,1))

## Maybe you are not into 3d illustrations.  Too bad. I can show the
## same in two dimensions.  Let's create a scatterplot displaying the
## predicted values from the ordinary and the mean-centered models

plot(fitted(m2), fitted(m2mc), xlab = "predicted from uncentered x",
     ylab = "predicted from centered x",
     main = "(Not)Centered Predictions Identical")

##
##
## So, how can it be the two models are EXACTLY the same and yet
## the mean centered one "looks better"?
##
## Here's the simple answer. This is a nonlinear model, the slope
## changes from point to point.  If we pick a point where it is
## steep and make a "snapshot" of the slope, we conclude "the
## slope is huge!"  If we pick a point where the slope is small
## we conclude "there's no effect of x".  Its really just that
## simple. Mean centering amounts to deciding where to measure
## the slope, and sometimes we are lucky enough to accidentally
## pick a steep spot by mean centering.
##
## Now the proponents of the mean-centering approach respond to
## me as follows. "You ignore the fact that the standard errors
## are smaller in the mean centered model. So there is a difference."

## In response, I say this. Just as the slope varies from point to
## point, the standard error also changes from point to point.  If you
## choose a point in the data, say where x1 = 48 and x2 = 47.2, and then
## find the equivalent mean-centered point, which is x1c = -1.24 and
## x2c = -1.305, you will find the slopes and the standard errors are
## exactly the same.  The slope at x1,x2 in the not centered model is
## exactly the same as the slope in the centered model at x1c,x2c.
## More importantly, from either model, we can calculate the same
## values of not only the slope, but also the standard error, for any
## particular point in the data.

## What is the easiest way to see that the slopes and standard errors
## are the same?  It seems obvious to me that, since the predicted
## value plane is identical in the two models, then the slopes have
## to be the same as well. But perhaps that relys on intuition that
## is unique to me.

## Take the the noncentered model and calculate the slope and standard
## error at the mean of x1,x2.  The slope at that point in the x1
## dimension, in terms of the noncentered fit, is

## b1 + b3*x2
## Which we estimate from the noncentered model as:
coef(m2)["x1"] + coef(m2)["x1:x2"] * mean(dat3$x2, na.rm = TRUE)
##       x1
## -8.383827
##
## And the slope in the x2 dimension is
## b2 + b3*x1

coef(m2)["x2"] + coef(m2)["x1:x2"] * mean(dat3$x1, na.rm = TRUE)
##      x2
## -9.479689
##
## Please note, those estimates of the slopes EXACTLY match the
## coefficient estimates reported by the centered model. That is to
## say, if you restrict your attention to a particular value of
## (x1,x2), the centered and noncentered models produce EXATCLY the same
## slope estimates.

## And the standard errors are the same as well.
## Reproduce the standard errors in centered model from noncentered model

V <- vcov(m2)

sqrt(V["x1","x1"] + mean(dat3$x2)^2 * V["x1:x2","x1:x2"] +
     2 * mean(dat3$x2) * V["x1","x1:x2"])

## [1] 0.8400474

## That's the SAME number reported in the Std.Err. column for the centered
## model.

sqrt(V["x2","x2"] + mean(dat3$x1)^2 * V["x1:x2","x1:x2"] +
     2 * mean(dat3$x1) * V["x2","x1:x2"])

##[1]  0.7791997
##
## Bingo, Fits exactly. The estimates of the centered model are reproduced
## exactly from the notcentered model once the correct co-ordinate
## translation is put in place. The EXACT same t ratios, etc.

## Centering has NO EFFECT whatsoever on multicollinearity. It does
## not affect predicted values, it does not affect our confidence
## in estimates of slopes or predicted values.

## In short, if you understand what a multiple regression with
## interaction "really does," it is impossible to reach any conclusion
## except the following: mean centering makes no difference at all in
## the estimation or interpretation of regression models with
## interaction effects.

## Mean centering only aids the interpretation if one is too lazy to
## understand the curvature of the estimated plane and work with the
## predicted values that are relevant to a problem.  It is not more
## easier to interpret the EXACT SAME NUMBER from one model or the
## other.




m2rc <- residualCenter(m2)
summary(m2rc)

op <- par(no.readonly = TRUE)
par(mar = c(2,2,2,1), mfcol = c(2,2))

plotPlane(m1, "x1", "x2", plotPoints = TRUE, theta = -25,
          main = "No Interaction", ticktype = "detailed")

plotPlane(m2, "x1", "x2", plotPoints = TRUE, theta = -25,
          main = "Not Centered", ticktype = "detailed")

plotPlane(m2rc, "x1", "x2", plotPoints = TRUE, theta = -25,
          main = "Residual Centered", ticktype = "detailed")

plotPlane(m2mc, "x1", "x2", plotPoints = TRUE, theta = -25,
          main = "Mean Centered", ticktype = "detailed")

par(mfcol = c(1,1))


## I had trouble believing those plots. Is it possible that the
## predicted values from the residual centered regression are exactly the
## same as the predictions from the orinary non-centered m2 as well as
## the mean centered m2mc?  That seemed crazy, so I decided to
## double-check by manually calculating a residual-centered regression
## and extracting predicted values.

rcreg <-  lm ( I(x1*x2)  ~ x1 + x2, data = dat3)
rcx1x2 <- resid(rcreg)
m2manualrc <- lm(y ~ x1 + x2 + rcx1x2, data = dat3)
predm2manualrc <- predict(m2manualrc, newdata = dat3)



m2 <- lm( y ~ x1 * x2, data = dat3)
m2mc <- meanCenter(m2)
m2rc <- residualCenter(m2)

m2pred <- predict(m2, newdata = dat3)
dat3mc <- dat3
dat3mc$x1 <- dat3mc$x1c
dat3mc$x2 <- dat3mc$x2c
m2mcpred <- predict(m2mc, newdata = dat3mc)
m2rcpred <- predict(m2rc, newdata = dat3)

dat4 <- data.frame("m2pred"= m2pred, "m2mcpred" = m2mcpred,
                   "m2mcmaunal"= m2mcmanualpred, "m2rcpred" = m2rcpred,
                   "m2rcpred2" = predm2manualrc)

head(dat4)

cor(dat4)

## Well, the predicted values are the same.
## and the coefficients on the interaction terms are all the same.
## We already saw that the ordinary  interaction and the
## mean-centered regressions are identical.  Is it possible
## the residual centered version is just another identical
## model viewed from yet another point in the x1,x2 plane?
## Notice:
## m2
## lm(formula = y ~ x1 * x2, data = dat3)

## Coefficients:
## (Intercept)           x1           x2        x1:x2
##    -74.4572       1.5726       0.6462      -0.2053


## From rockchalk:::residualCenter(m2):
## m2rc
## lm(formula = y ~ x1 + x2 + x1.X.x2, data = dat)

## Coefficients:
## (Intercept)           x1           x2      x1.X.x2
##    424.1485      -8.9042      -9.2701      -0.2053



## From rockchalk:::meanCenter(m2)
## > m2mc
## The centered variables are:
## [1] "x1" "x2"
## The call that requested centering was:
## meanCenter.default(model = m2)

## Call:
## lm(formula = y ~ x1 * x2, data = stddat)

## Coefficients:
## (Intercept)           x1           x2        x1:x2
##   -456.6985      -8.3838      -9.4797      -0.2053



## If mean-centering is identical to residual centering,
## there must be some matrix algebra to prove it.

## b-hat = (X'X)-1 X'y

##          X = |1 x1 x2 x1*x2|

## mean centered:
##          X = |1  x1-m(x1) x2-m(x2)  (x1-m(x1))(x2-m(x2))|


## let pred(x1*x2) be the predicted value from the regression
## (x1*x2) = b0 + b1*x1 + b2*x2

## Then the design matrix for
## residual centered:
##          X = |1  x1 x2  (x1*x2)-pred(x1*x2) |

## I think if I were good at matrix algebra with partitioned
## matrices, I could demonstrate why the estimated coefficients
## are actually equivalent.

## stackListItems.R
## Paul Johnson <pauljohn at ku.edu>
## 2010-09-07

## I asked about this in r-help last week and promised
## a summary of answers.

## We face this problem all the time. A procedure
## generates a list of data frames. How to stack them
## together?

## The short answer is that the plyr package's rbind.fill
## method is probably the fastest method that is not
## prone to trouble and does not require much user caution.

## result <- rbind.fill(mylist)

## A slower alternative that also works is

## result <- do.call("rbind", mylist)

## That is always available in R and it works well enough, even
## though it is not quite as fast. Both of these are much faster than
## a loop that repeatedly applies "rbind".

## Truly blazing speed can be found if we convert this into
## matrices, but that is not possible if the list actually
## contains data frames.



## Here is a test case

df1 <- data.frame(x = rnorm(100), y = rnorm(100))
df2 <- data.frame(x = rnorm(100), y = rnorm(100))
df3 <- data.frame(x = rnorm(100), y = rnorm(100))
df4 <- data.frame(x = rnorm(100), y = rnorm(100))

mylist <-  list(df1, df2, df3, df4)

## Here's the way we have done it. We understand this,
## we believe the result, it is easy to remember. It is
## also horribly slow for a long list.

resultDF <- mylist[[1]]
for (i in 2:4) resultDF <- rbind(resultDF, mylist[[i]])


## It works better to just call rbind once, as in:

resultDF2 <- rbind( mylist[[1]], mylist[[2]], mylist[[3]], mylist[[4]])


## That is faster because it calls rbind only once.

## But who wants to do all of that typing? How tiresome.
## Thanks to Erik Iverson in r-help, I understand that

resultDF3 <- do.call("rbind", mylist)

## is doing the EXACT same thing.
## Erik explained that "do.call( "rbind", mylist)"
## is *constructing* a function call from the list of arguments.
## It is shorthand for "rbind(mylist[[1]], mylist[[2]], mylist[[3]])"
## assuming mylist has 3 elements.

## Check the result:
all.equal( resultDF2, resultDF3)

## You often see people claim it is fast to allocate all
## of the required space in one shot and then fill it in.
## I got this algorithm from code in the
## "complete" function in the "mice" package.
## It allocates a big matrix of 0's and
## then it places the individual data frames into that matrix.

m <- 4
nr <- nrow(df1)
nc <- ncol(df1)
resultDF4 <- as.data.frame(matrix(0, nrow = nr*m, ncol = nc))
for (j in  1:m) resultDF4[(((j-1)*nr) + 1):(j*nr), ] <- mylist[[j]]

## This is a bit error prone for my taste. If the data frames have
## different numbers of rows, some major code surgery will be needed.

##
## Dennis Murphy pointed out the plyr package, by Hadley Wickham.
## Dennis said " ldply() in the plyr package. The following is the same
## idea as do.call(rbind, l), only faster."

library("plyr")
resultDF5  <- ldply(mylist, rbind)
all.equal(resultDF, resultDF5)



## Plyr author Hadley Wickham followed up with "I think all you want here is rbind.fill:"

resultDF6 <- rbind.fill(mylist)
all.equal(resultDF, resultDF6)


## Gabor Grothendieck noted that if the elements in mylist were matrices, this would all work faster.

mylist2 <- lapply(mylist, as.matrix)

matrixDoCall <-  do.call("rbind", mylist2)

all.equal(as.data.frame(matrixDoCall), resultDF)


## Gabor also showed a better way than 'system.time' to find out how
## long this takes on average using the rbenchmark package. Awesome!

#> library(rbenchmark)
#> benchmark(
#+ df = do.call("rbind", mylist),
#+ mat = do.call("rbind", L),
#+ order = "relative", replications = 250
#+ )



## To see the potentially HUGE impact of these changes, we need to
## make a bigger test case. I just used system.time to evaluate, but
## if this involved a close call, I'd use rbenchmark.

phony <- function(i){
  data.frame(w = rnorm(1000), x = rnorm(1000), y = rnorm(1000), z = rnorm(1000))
}
mylist <- lapply(1:1000, phony)




### First, try my usual way
resultDF <- mylist[[1]]
system.time(
for (i in 2:1000) resultDF <- rbind(resultDF, mylist[[i]])
           )
## wow, that's slow:
## user  system elapsed
## 168.040   4.770 173.028


### Now do.call method:
system.time( resultDF3 <- do.call("rbind", mylist) )
all.equal(resultDF, resultDF3)

## Faster! Takes one-twelfth as long
##   user  system elapsed
##  14.64    0.85   15.49


### Third, my adaptation of the complete function in the mice
### package:
m <- length(mylist)
nr <- nrow(mylist[[1]])
nc <- ncol(mylist[[1]])

system.time(
   resultDF4 <- as.data.frame(matrix(0, nrow = nr*m, ncol = nc))
 )

colnames(resultDF4) <- colnames(mylist[[1]])
system.time(
   for (j in  1:m) resultDF4[(((j-1)*nr) + 1):(j*nr), ] <- mylist[[j]]
)

all.equal(resultDF, resultDF4)
##Disappointingly slow on the big case:
#   user  system elapsed
# 80.400   3.970  84.573


### That took much longer than I expected, Gabor's
### hint about the difference between matrix and data.frame
### turns out to be important. Do it again, but don't
### make the intermediate storage thing a data.frame:
mylist2 <- lapply(mylist, as.matrix)

m <- length(mylist2)
nr <- nrow(mylist2[[1]])
nc <- ncol(mylist2[[1]])

system.time(
   resultDF4B <- matrix(0, nrow = nr*m, ncol = nc)
 )

colnames(resultDF4B) <- colnames(mylist[[1]])
system.time(
   for (j in  1:m) resultDF4B[(((j-1)*nr) + 1):(j*nr), ] <- mylist2[[j]]
)

### That's FAST!
###    user  system elapsed
###   0.07    0.00    0.07

all.equal(resultDF, as.data.frame(resultDF4B))



### Now the two moethods from plyr.


system.time( resultDF5  <- ldply(mylist, rbind))

## Just about as fast, much less error prone
##  user  system elapsed
##  1.290   0.000   1.306

all.equal(resultDF, resultDF5)


system.time(resultDF6 <- rbind.fill(mylist))
##   user  system elapsed
##  0.450   0.000   0.459

all.equal(resultDF, resultDF6)



## Gabor was right. If we have matrices, do.call is
## just about as good as anything.

system.time(matrixDoCall <-  do.call("rbind", mylist2) )
##   user  system elapsed
##  0.030   0.000   0.032


all.equal(as.data.frame(matrixDoCall), resultDF)

## Paul Johnson Sept. 11, 2005  POLS 707
## Plot Some functions


x <- seq(from = 0, to = 50, length.out = 200)

y <- 3 + 4 * x

plot (x, y, main = "Linear Equation", type = "l")



y <- 3 + 4*x - 0.09 * x * x

plot (x, y, main = "Quadratic Equation", type = "l")




y <- 10 - 5 * (1/x)

plot (x,  y, main = "Reciprocal with negative coefficient", type = "l")

#Recall (1/x) = x ^(-1)


y <- 10 + 5 * (1/x)

plot (x,  y, main = "Reciprocal with positive coefficient", type = "l")



y <- exp (x)

plot (x, y, main = "Exponential of x" , type = "l")


y <- exp (-x)

plot (x, y, main = "Exponential of -x", type = "l")

# Note exp(-x) is same as 1 / exp(x)

y <- 1 / exp (x)

plot (x, y, main = "Reciprocal of Exponential of x", type = "l")






y <- exp (-(x-24)^2 )

plot (x, y, main = "Exponential of x^2", type = "l")



y <- exp ( -  (1/100) *  (x-24)^2 )
plot (x, y, main = "Exponential of x^2", type = "l")




# If y has to be constrained (say, between 0 and 100 for percents)
# there are many possibilities

z <- -10 + .4*x
y <- 100 * exp(z)/ (1 + exp(z))

# same as y <- 100 / (1 + exp(-z))
plot (x,  y, main = "S-shaped curve from the logistic", type = "l")


#Look in R's stats package for functions that start with SS.
# The SSlogis function has this formula
# y = Asym/(1+exp((xmid-input)/scale))
# you should see my example creates similar

##library(rockchalk)


set.seed(12345)

dfadd <- genCorrelatedData2(100, means = c(0,0,0,0), sds = 1, rho = 0,
       beta = c(0.03, 0.01, 0.1, 0.4, -0.1), stde = 2)

dfadd$xcat1 <- gl(2,50, labels=c("M","F"))

dfadd$xcat2 <- cut(rnorm(100), breaks=c(-Inf, 0, 0.4, 0.9, 1, Inf),
                   labels=c("R", "M", "D", "P", "G"))

dfadd$y2 <- 0.03 + 0.1*dfadd$x1 + 0.1*dfadd$x2 +
    0.25*dfadd$x1*dfadd$x2 + 0.4*dfadd$x3 - 0.1*dfadd$x4 +
    0.2 * as.numeric(dfadd$xcat1) +
    contrasts(dfadd$xcat2)[as.numeric(dfadd$xcat2), ] %*% c(-0.1, 0.1, 0.2, 0) +
    1 * rnorm(100)

summarize(dfadd)

## linear ordinary regression
m1 <- lm(y ~ x1 + x2 + x3 + x4, data = dfadd)
summary(m1)

mcDiagnose(m1)

## These will be parallel lines

plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = "std.dev.",
           n = 5, main = "A plotSlopes result with \"std.dev.\" values of modx")


m1ps <- plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = c(-2,0,2))

m1pp <- plotPlane(m1, plotx1 = "x1", plotx2 = "x2",
                  ticktype = "detailed", npp = 10)

addLines(from = m1ps, to = m1pp, lty = 1, lwd = 2)

m1pp <- plotPlane(m1, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed",
                  npp = 10)
addLines(from = m1ps, to = m1pp, lty = 2, lwd = 5, col = "green")

## Other approach would wrap same into the linesFrom argument in plotPlane

plotPlane(m1, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed",
          npp = 10, linesFrom = m1ps)

## Need to think more on whether dotted lines from ps object should
## be converted to solid lines in plotPlane.

library(rockchalk)
N <- 100
dat <- genCorrelatedData3(y~ 0 + 0.2*x1 + 0.2*x2, N=N, means=c(100,200),
                          sds=c(20,30), rho=0.4, stde=10)
dat$x3 <- rnorm(100, m=40, s=4)
m1 <- lm(y ~ x1 + x2 + x3, data=dat)
summary(m1)
m1d <- mcDiagnose(m1)

m2 <- lm(y ~ x1 * x2 + x3, data=dat)
summary(m2)
m2d <- mcDiagnose(m2)



m3 <- lm(y ~ log(10+x1) + x3 + poly(x2,2), data=dat)
summary(m3)
m3d <- mcDiagnose(m3)

N <- 100
x1 <- 50 + rnorm(N)
x2 <- log(rgamma(N, 2,1))
x3 <- rpois(N, lambda=17)
z1 <- gl(5, N/5)
dummies <- contrasts(z1)[ as.numeric(z1), ]
dimnames(dummies) <- NULL ## Avoids row name conflict in data.frame below
y3 <- x1  -.5 * x2 + 0.1 * x2^2 + dummies %*% c(0.1,-0.1,-0.2,0.2)+ 5 * rnorm(N)
dat <- data.frame(x1=x1, x2=x2, x3=x3,  z1=z1, y3 = y3)

m3 <- lm(y3 ~ x1 + poly(x2,2)  + log(x1) + z1, dat)
summary(m3)

mcDiagnose(m3)

library(rockchalk)


set.seed(12345)
x1 <- rnorm(100)
x2 <- rnorm(100)
x3 <- rnorm(100)
x4 <- rnorm(100)
y <- rnorm(100)
y2 <- 0.03 + 0.1*x1 + 0.1*x2 + 0.25*x1*x2 + 0.4*x3 -0.1*x4 + 1*rnorm(100)
dat <- data.frame(x1,x2,x3,x4,y, y2)
rm(x1, x2, x3, x4, y, y2)

## linear ordinary regression
m1 <- lm(y ~ x1 + x2 +x3 + x4, data = dat)

plotPlane(m1, plotx1 = "x3", plotx2 = "x4")

plotPlane(m1, plotx1 = "x3", plotx2 = "x4", drawArrows = TRUE)

plotPlane(m1, plotx1 = "x1", plotx2 = "x4", drawArrows = TRUE)


plotPlane(m1, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE, npp = 10)
plotPlane(m1, plotx1 = "x3", plotx2 = "x2", drawArrows = TRUE, npp = 40)

plotPlane(m1, plotx1 = "x3", plotx2 = "x2", drawArrows = FALSE,
          npp = 5, ticktype = "detailed")


## regression with interaction
m2 <- lm(y ~ x1  * x2 +x3 + x4, data = dat)

plotPlane(m2, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE)


plotPlane(m2, plotx1 = "x1", plotx2 = "x4", drawArrows = TRUE)
plotPlane(m2, plotx1 = "x1", plotx2 = "x3", drawArrows = TRUE)

plotPlane(m2, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE,
          phi = 10, theta = 30)



## regression with quadratic;
## Required some fancy footwork in plotPlane, so be happy
dat$y3 <- 0 + 1 * dat$x1 + 2 * dat$x1^2 + 1 * dat$x2 +
    0.4*dat$x3 + 8 * rnorm(100)
m3 <- lm(y3 ~ poly(x1,2) + x2 +x3 + x4, data = dat)
summary(m3)

plotPlane(m3, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE,
          x1lab = "my great predictor", x2lab = "a so-so predictor",
          ylab = "Most awesomest DV ever")

plotPlane(m3, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE,
          x1lab = "my great predictor", x2lab = "a so-so predictor",
          ylab = "Most awesomest DV ever", phi = -20)

plotPlane(m3, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE,
          phi = 10, theta = 30)

plotPlane(m3, plotx1 = "x1", plotx2 = "x4", drawArrows = TRUE,
          ticktype = "detailed")
plotPlane(m3, plotx1 = "x1", plotx2 = "x3", drawArrows = TRUE)

plotPlane(m3, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE,
          phi = 10, theta = 30)

m4 <- lm(y ~ sin(x1) + x2*x3 +x3 + x4, data = dat)
summary(m4)


plotPlane(m4, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE)
plotPlane(m4, plotx1 = "x1", plotx2 = "x3", drawArrows = TRUE)



eta3 <- 1.1 + .9*dat$x1 - .6*dat$x2 + .5*dat$x3
dat$y4 <- rbinom(100, size = 1, prob = exp( eta3)/(1+exp(eta3)))
gm1 <- glm(y4 ~ x1 + x2 + x3, data = dat, family = binomial(logit))
summary(gm1)
plotPlane(gm1, plotx1 = "x1", plotx2 = "x2")
plotPlane(gm1, plotx1 = "x1", plotx2 = "x2", phi = -10)

plotPlane(gm1, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed")
plotPlane(gm1, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed",
          npp = 30, theta = 30)
plotPlane(gm1, plotx1 = "x1", plotx2 = "x3", ticktype = "detailed",
          npp = 70, theta = 60)

plotPlane(gm1, plotx1 = "x1", plotx2 = "x2", ticktype = c("detailed"),
          npp = 50, theta = 40)

dat$x2 <- 5 * dat$x2
dat$x4 <- 10 * dat$x4
eta4 <- 0.1 + .15*dat$x1 - 0.1*dat$x2 + .25*dat$x3 + 0.1*dat$x4
dat$y4 <- rbinom(100, size = 1, prob = exp( eta4)/(1+exp(eta4)))
gm2 <- glm(y4 ~ x1 + x2 + x3 + x4, data = dat, family = binomial(logit))
summary(gm2)
plotPlane(gm2, plotx1 = "x1", plotx2 = "x2")
plotPlane(gm2, plotx1 = "x2", plotx2 = "x1")
plotPlane(gm2, plotx1 = "x1", plotx2 = "x2", phi = -10)
plotPlane(gm2, plotx1 = "x1", plotx2 = "x2", phi = 5, theta = 70, npp = 40)

plotPlane(gm2, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed")
plotPlane(gm2, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed",
          npp = 30, theta = -30)
plotPlane(gm2, plotx1 = "x1", plotx2 = "x3", ticktype = "detailed",
          npp = 70, theta = 60)

plotPlane(gm2, plotx1 = "x4", plotx2 = "x3", ticktype = "detailed",
          npp = 50, theta = 10)

plotPlane(gm2, plotx1 = "x1", plotx2 = "x2", ticktype = c("detailed"))

## factors
set.seed(123123)
x <- c("A", "B", "C", "A", "A", "B", "B", "B", "C", "C")
y <- rnorm(10)

xf1 <- factor(x)

attributes(xf1)
class(xf1)
levels(xf1)
summary(xf1)

table(xf1)

mxf1 <- lm(y ~ xf1)
summary(mxf1)

contrasts(xf1)
contrasts(xf1)[xf1, ]

mxf1.2 <- lm(y ~ -1 + xf1)
summary(mxf1.2)
anova(mxf1, mxf1.2, test = "F")  ##digital noise
zapsmall(anova(mxf1, mxf1.2, test = "F"))


xf2 <- factor(x, levels = c("C", "B", "A"))
levels(xf2)

table(xf2, xf1)

mxf2 <- lm(y ~ xf2)
summary(mxf2)

anova(mxf2, mxf1, test = "F")
zapsmall(anova(mxf2, mxf1, test = "F"))


xf3 <- factor(x, levels = c("B", "A", "C"), labels = c("New Long Name For Level B",
    "Name for Level A", "Original C is Called This Now"))

table(xf3, xf1)

levels(xf3)[3] <- "C level was too long"
table(xf3, xf1)

levels(xf3) <- c("Really B", "Actually A", "Vitamin C")
table(xf3, xf1)


xf4 <- relevel(xf3, ref="Vitamin C")
table(xf4, xf1)

levels(xf3)

xf5 <- relevel(xf3, ref=levels(xf3)[3])
levels(xf5)[1] <- "C is first again"
table(xf5, xf1)

rm(x, xf1, xf2, xf3, mxf1, mxf1.2, mxf2, y)
ls()
rm(list=ls())
ls()

x <- c(44, 33, 16, 11, 21, 16, 44, 24, 16, 15)

xf1 <- factor(x)
levels(xf1)
table(xf1, x)

xf1n <- as.numeric(xf1)
xf1n
table(xf1n)
summary(xf1n)
table(xf1n, x)

?factor
xRecovered <- as.numeric(levels(xf1))[xf1]
xRecovered


identical(x, xRecovered)
table(xRecovered, xf1n)
plot(x, xRecovered)

rm(x, xf1, xf1n, xRecovered)



y <- rnorm(100)

xf1 <- gl(4, 25)
levels(xf1)
table(xf1)  #boring


xf1 <- gl(4, 25, labels = c("whatever", "whichever", "whenever", "whoever"))
attributes(xf1)
levels(xf1)
table(xf1)


xf2 <- sample(xf1, 100, replace = TRUE)
t1 <- table(xf2)
barplot(t1)


t1b <- prop.table(t1)
barplot(t1b, ylab = "You (not me) asked for proportions")

contrasts(xf2)
designMatrix <- contrasts(xf2)[as.numeric(xf2), ]
designMatrix2 <- contrasts(xf2)[xf2, ]
identical(designMatrix, designMatrix2)
designMatrix

y1 <- designMatrix %*% c(0.5, -0.9, 1.3) + rnorm(100)

m1 <- lm(y1 ~ xf2)
summary(m1)
termplot(m1)
termplot(m1, se = TRUE, partial = TRUE)

xf2.2 <- relevel(xf2, ref=levels(xf2)[3])
m2.2 <- lm(y1 ~ xf2.2)
summary(m2.2)
termplot(m2.2, se = TRUE, partial = TRUE)



y2 <- designMatrix %*% c(0.2, 0.6, 0.9) + 3 * rnorm(100)

m2 <- lm(y2 ~ xf2)
summary(m2)
termplot(m2)
termplot(m2, se = TRUE, partial = TRUE)



xf2o <- as.ordered(xf2)
m1o <- lm(y1 ~ xf2o)
summary(m1o)
m2o <- lm(y2 ~ xf2o)
summary(m2o)


justCurious <- lm(y1 ~ -1 + xf2o)
summary(justCurious)


op <- par(no.readonly = TRUE)
par(mfcol = c(2, 1))
termplot(m1, partial = TRUE, se = TRUE, ylab = "Fit with xf2")
termplot(m1o, partial = TRUE, se = TRUE, ylab = "Fit with xf2o")
par(op)


plot(fitted(m1), fitted(m1o), xlab = "Predicted by model fitted with xf2",
    ylab = "Predicted by model fitted with as.ordered(xf2)", main = "Predictions Identical. Models Same?")


xf3 <- relevel(xf2, "whoever")
m3 <- lm(y1 ~ xf3)
termplot(m3, partial = TRUE, se = TRUE, ylab = "Fit with xf3")
par(mfcol = c(2, 1))
termplot(m1, partial = TRUE, se = TRUE, ylab = "Fit with xf2")
termplot(m3, partial = TRUE, se = TRUE, ylab = "Fit with xf3")
par(op)

cat("\n\n Basics: vector, matrix, attribute, attr, dim, structure \n\n, t, %*%, outer, crossprod")


x <- c(13, 11, 18, 2, 134, 154, 2, 3, 4, 12)
is.vector(x)
is.matrix(x)
attributes(x)
?attributes

matrix(x, nrow=2)
?matrix

X1m1 <- matrix(x, nrow=2)
class(X1m1)
X1m2 <- matrix(x, ncol=5)
X1m1 == X1m2

identical(X1m1, X1m2)
?identical

attributes(X1m1)
attributes(X1m2)

X1m3 <- x
attr(X1m3, "dim") <- c(2,5)
?attr
X1m3
is.vector(X1m3)
is.matrix(X1m3)
is.data.frame(X1m3)
attributes(X1m3)
identical(X1m1, X1m3)


X1m4 <- x
dim(X1m4) <- c(2,5)
?dim
X1m4
attributes(X1m4)

identical(X1m1, X1m4)


X1m5 <- structure(x, dim=c(2,5))
?structure
X1m5

identical(X1m1, X1m5)


X1m6 <- rbind(x[c(1,3,5,7,9)], x[c(2,4,6,8,10)])
class(x[1:5])
class(X1m6)
identical(X1m1, X1m6)

X1m7 <- cbind(x[1:2], x[3:4], x[5:6], x[7:8],x[9:10])
identical(X1m1, X1m7)


x1m7 <- X1m7
identical(x, x1m7)
dim(x1m7) <- c(1,10)
x1m7
identical(x, x1m7) #humphf!

x1m7new1 <- drop(x1m7)
?drop
x1m7new1
identical(x, x1m7new1) #whew


x1m7new2 <-  as.vector(x1m7)
identical(x, x1m7new2)



?t
tX1m1 <- t(X1m1)
tX1m1

X2m1 <- matrix(x, nrow=5, byrow=TRUE)
X2m1
dim(X2m1)
identical(X2m1, t(X1m1))



X2m2 <- X1m1
X2m2
dim(X2m2)
dim(X2m2) <- c(5,2)
dim(X2m2)
X2m2
identical(X1m1, X2m2)
X2m1 == X2m2 #ack!



y <- c(12, 12, 41, 22)
x %*% y #heck

y <- rep(c(1,2,3,4,5), 2)
x %*% y
t(x) %*% y
y %*% x


outer(x,y)
outer(x,y, FUN="*")
outer(x,y, FUN="+")
outer(x,y, FUN="/")
outer(x,y, FUN=function(a, b) {b - 2*a})


X3 <- matrix(rnorm(10), nrow=2)

crossprod(X1m1, X3)
t(X1m1) %*% X3
?crossprod



## Puzzle

x <- 1:6

X <- data.frame(x1 = x, x2 = x, x3 = x)
Y <- data.frame(z1 = rnorm(6), z2 = rnorm(6), x3 = rnorm(6))

Z <- cbind(X, Y)
colnames(Z)

Z2 <- data.frame(X,Y)
colnames(Z2)

## Manufacture some predictors
set.seed(12345)

dat <- genCorrelatedData2 (N = 100, means = rep(0,4), sds = 1, rho = 0.2,
                           beta = c(0.3, 0.5, -0.45, 0.5, -0.1, 0, 0.6),
                           stde = 2)

dat$xcat1 <- gl(2, 50, labels = c("M", "F"))
dat$xcat2 <- cut(rnorm(100), breaks = c(-Inf, 0, 0.4, 0.9, 1, Inf),
                 labels = c("R", "M", "D", "P", "G"))
## incorporate effect of categorical predictors
dat$y <- dat$y + 1.9 * dat$x1 * contrasts(dat$xcat1)[dat$xcat1] +
           contrasts(dat$xcat2)[dat$xcat2 , ] %*% c(0.1, -0.16, 0, 0.2)

m1 <- lm(y ~ x1 * x2 + x3 + x4 + xcat1* xcat2, data = dat)
summary(m1)

## New in rockchalk 1.7.x. No modx required:
plotSlopes(m1, plotx = "x1")
## Confidence interval, anybody?
plotSlopes(m1, plotx = "x1", interval = "conf")

## Prediction interval.
plotSlopes(m1, plotx = "x1", interval = "pred")

plotSlopes(m1, plotx = "x1", modx = "xcat2", modxVals = c("R", "M"))

plotSlopes(m1, plotx = "x1", modx = "xcat2",  interval = "pred")

plotSlopes(m1, plotx = "xcat1", modx = "xcat2",  interval = "conf", space = c(0,1))

plotSlopes(m1, plotx = "xcat1", modx = "xcat2", 
           modxVals = c("Print R" = "R" , "Show M" = "M"), gridArgs = "none")

## Now experiment with a moderator variable
## let default quantile algorithm do its job
plotSlopes(m1, plotx = "xcat2", interval = "none")
plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "none")
plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence",
           legendArgs = list(title = "xcat2"), ylim = c(-3, 3), lwd = 0.4)
plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence",
           legendArgs = list(title = "xcat2"), ylim = c(-3, 3), lwd = 0.4, width = 0.25)
m1.ps <- plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "prediction")
m1.ps <- plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "prediction", space=c(0,2))
plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "prediction", gridArgs = "none")

plotSlopes(m1, plotx = "xcat2", modx = "xcat1", interval = "confidence", ylim = c(-3, 3))
plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence", 
           col = c("black", "blue", "green", "red", "orange"), lty = c(1, 4, 6, 3))

plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence", 
           col = gray.colors(4, end = 0.5), lty = c(1, 4, 6, 3), legendArgs = list(horiz=TRUE))


plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence", 
           col = c("pink", "orange"))

plotSlopes(m1, plotx = "xcat1", interval = "confidence", 
           col = c("black", "blue", "green", "red", "orange"))

plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence", 
           col = c("black", "blue", "green", "red", "orange"),
           gridlwd = 0.2)

## previous uses default equivalent to
## plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = "quantile")
## Want more focal values?
plotSlopes(m1, plotx = "x1", modx = "x2", n = 5)
## Pick focal values yourself?
plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = c(-2, 0, 0.5))
## Alternative algorithm?
plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = "std.dev.",
           main = "Uses \"std.dev.\" Divider for the Moderator",
           xlab = "My Predictor", ylab = "Write Anything You Want for ylab")

## Will catch output object from this one
m1ps <- plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = "std.dev.", n = 5,
                 main = "Setting n = 5 Selects More Focal Values for Plotting")

m1ts <- testSlopes(m1ps)

plot(m1ts)

### Examples with categorical Moderator variable

m3 <- lm (y ~ x1 + xcat1, data = dat)
summary(m3)
plotSlopes(m3, modx = "xcat1", plotx = "x1")
plotSlopes(m3, modx = "xcat1", plotx = "x1", interval = "predict")
plotSlopes(m3, modx = "x1", plotx = "xcat1", interval = "confidence",
           legendArgs = list(x = "bottomright", title = ""))

m4 <- lm (y ~ x1 * xcat1, data = dat)
summary(m4)
plotSlopes(m4, modx = "xcat1", plotx = "x1")
plotSlopes(m4, modx = "xcat1", plotx = "x1", interval = "conf")


m5 <- lm (y ~ x1 + x2 + x1 * xcat2, data = dat)
summary(m5)
plotSlopes(m5, modx = "xcat2", plotx = "x1")
m5ps <- plotSlopes(m5, modx = "xcat2", plotx = "x1", interval = "conf")

testSlopes(m5ps)



## Now examples with real data. How about Chilean voters?
library(carData)
m6 <- lm(statusquo ~ income * sex, data = Chile)
summary(m6)
plotSlopes(m6, modx = "sex", plotx = "income")
m6ps <- plotSlopes(m6, modx = "sex", plotx = "income", col = c("orange", "blue"))

testSlopes(m6ps)

m7 <- lm(statusquo ~ region * income, data= Chile)
summary(m7)
plotSlopes(m7, plotx = "income", modx = "region")

plotSlopes(m7, plotx = "income", modx = "region", plotPoints = FALSE)
plotSlopes(m7, plotx = "income", modx = "region", plotPoints = FALSE,
           interval = "conf")
plotSlopes(m7, plotx = "income", modx = "region", modxVals = c("SA","S", "C"),
           plotPoints = FALSE, interval = "conf")
## Same, choosing 3 most frequent values
plotSlopes(m7, plotx = "income", modx = "region", n = 3, plotPoints = FALSE,
           interval = "conf")


m8 <- lm(statusquo ~ region * income + sex + age, data= Chile)
summary(m8)
plotSlopes(m8, modx = "region", plotx = "income")


m9 <- lm(statusquo ~ income * age + education + sex + age, data = Chile)
summary(m9)
plotSlopes(m9, modx = "income", plotx = "age")

m9ps <- plotSlopes(m9, modx = "income", plotx = "age")
m9psts <- testSlopes(m9ps)
plot(m9psts) ## only works if moderator is numeric

## Demonstrate re-labeling
plotSlopes(m9, modx = "income", plotx = "age", n = 5,
           modxVals = c("Very poor" = 7500,  "Rich" = 125000), 
           main = "Chile Data", legendArgs = list(title = "Designated Incomes"))

plotSlopes(m9, modx = "income", plotx = "age", n = 5, modxVals = c("table"), 
           main = "Moderator: mean plus/minus 2 SD")

## Convert education to numeric, for fun
Chile$educationn <- as.numeric(Chile$education)
m10 <- lm(statusquo ~ income * educationn + sex + age, data = Chile)
summary(m10)
plotSlopes(m10, plotx = "educationn",  modx = "income")

## Now, the occupational prestige data. Please note careful attention
## to consistency of colors selected
data(Prestige)
m11 <- lm(prestige ~ education * type, data = Prestige)

plotSlopes(m11, plotx = "education", modx = "type", interval = "conf")
dev.new()
plotSlopes(m11, plotx = "education", modx = "type",
           modxVals = c("prof"), interval = "conf")
dev.new()
plotSlopes(m11, plotx = "education", modx = "type",
           modxVals = c("bc"), interval = "conf")
dev.new()
plotSlopes(m11, plotx = "education", modx = "type",
           modxVals = c("bc", "wc"), interval = "conf")

library(rockchalk)

## Replicate some R classics.  The budworm.lg data from predict.glm
## will work properly after re-formatting the information as a data.frame:

## example from Venables and Ripley (2002, pp. 190-2.)
df <- data.frame(ldose = rep(0:5, 2),
                 sex = factor(rep(c("M", "F"), c(6, 6))),
                 SF.numdead = c(1, 4, 9, 13, 18, 20, 0, 2, 6, 10, 12, 16))
df$SF.numalive = 20 - df$SF.numdead

budworm.lg <- glm(cbind(SF.numdead, SF.numalive) ~ sex*ldose, data = df,
                  family = binomial)


plotCurves(budworm.lg, plotx = "ldose", modx = "sex", interval = "confidence",
           ylim = c(0, 1))

## See infert
model2 <- glm(case ~ age + parity + education + spontaneous + induced,
              data = infert, family = binomial())

## Curvature so slight we can barely see it
model2pc1 <- plotCurves(model2, plotx = "age", modx = "education",
                        interval = "confidence", ylim = c(0, 1))


model2pc2 <- plotCurves(model2, plotx = "age", modx = "education",
                        modxVals = levels(infert$education)[1],
                        interval = "confidence", ylim = c(0, 1))


model2pc2 <- plotCurves(model2, plotx = "age", modx = "education",
                        modxVals = levels(infert$education)[c(2,3)],
                        interval = "confidence", ylim = c(0, 1))

model2pc2 <- plotCurves(model2, plotx = "age", modx = "education",
                        modxVals = levels(infert$education)[c(2,3)],
                        ylim = c(0, 1), type = "response")





## Manufacture some data
set.seed(12345)
N <- 500
dat <- genCorrelatedData2(N = 500, means = c(5, 0, 0, 0), sds = rep(1, 4),
                          rho = 0.2, beta = rep(1, 5), stde = 20)

dat$xcat1 <- gl(2, N/2, labels = c("Monster", "Human"))
dat$xcat2 <- cut(rnorm(N), breaks = c(-Inf, 0, 0.4, 0.9, 1, Inf),
             labels = c("R", "M", "D", "P", "G"))

###The design matrix for categorical variables, xcat numeric
dat$xcat1n <- with(dat, contrasts(xcat1)[xcat1, ])
dat$xcat2n <- with(dat, contrasts(xcat2)[xcat2, ])


stde <- 2
dat$y <- with(dat, 0.03 + 11.5 * log(x1) * contrasts(dat$xcat1)[dat$xcat1] +
              0.1 * x2 + 0.04 * x2^2 + stde*rnorm(N))

stde <- 1
dat$y2 <- with(dat, 0.03 + 0.1 * x1 + 0.1 * x2 + 0.25 * x1 * x2 + 0.4 * x3 -
               0.1 * x4 + stde * rnorm(N))
stde <- 8
dat$y3 <- with(dat, 3 + 0.5 * x1 + 1.2 * (as.numeric(xcat1)-1) +
-0.8 * (as.numeric(xcat1)-1) * x1 +  stde * rnorm(N))

stde <- 8
dat$y4 <- with(dat, 3 + 0.5 * x1 +
               contrasts(dat$xcat2)[dat$xcat2, ] %*% c(0.1, -0.2, 0.3, 0.05)  +
               stde * rnorm(N))


## Curvature with interaction
m1 <- lm(y ~ log(x1)*xcat1 + x2 + I(x2^2), data=dat)
summary(m1)

## First, with no moderator
plotCurves(m1, plotx = "x1")

plotCurves(m1, plotx = "x1", modx = "xcat1")

## ## Verify that plot by comparing against a manually contructed alternative
## par(mfrow=c(1,2))
## plotCurves(m1, plotx = "x1", modx = "xcat1")
## newdat <- with(dat, expand.grid(x1 = plotSeq(x1, 30), xcat1 = levels(xcat1)))
## newdat$x2 <-  with(dat, mean(x2, na.rm = TRUE))
## newdat$m1p <- predict(m1, newdata = newdat)
## plot( y ~ x1, data = dat, type = "n", ylim = magRange(dat$y, c(1, 1.2)))
## points( y ~ x1, data = dat, col = dat$xcat1, cex = 0.4, lwd = 0.5)
## by(newdat, newdat$xcat1, function(dd) {lines(dd$x1, dd$m1p)})
## legend("topleft", legend=levels(dat$xcat1), col = as.numeric(dat$xcat1), lty = 1)
## par(mfrow = c(1,1))
## ##Close enough!


plotCurves(m1, plotx = "x2", modx = "x1")

plotCurves(m1, plotx = "x2", modx = "xcat1")

plotCurves(m1, plotx = "x2", modx = "xcat1", interval = "conf")


m2 <- lm(y ~ log(x1)*xcat1 + xcat1*(x2 + I(x2^2)), data = dat)
summary(m2)
plotCurves(m2, plotx = "x2", modx = "xcat1")

plotCurves(m2, plotx  ="x2", modx = "x1")


m3a <- lm(y ~ poly(x2, 2) + xcat1, data = dat)

plotCurves(m3a, plotx = "x2")
plotCurves(m3a, plotx = "x2", modx = "xcat1")
#OK

m4 <- lm(log(y+10) ~ poly(x2, 2)*xcat1 + x1, data = dat)
summary(m4)
plotCurves(m4, plotx = "x2")

plotCurves(m4, plotx  ="x2", modx = "xcat1")

plotCurves(m4, plotx = "x2", modx = "x1")

plotCurves(m4, plotx = "x2", modx = "xcat1")

plotCurves(m4, plotx = "x2", modx = "xcat1", modxVals = c("Monster"))


##ordinary interaction
m5 <- lm(y2 ~ x1*x2 + x3 +x4, data = dat)
summary(m5)
plotCurves(m5, plotx = "x1", modx = "x2")
plotCurves(m5, plotx = "x1", modx = "x2", modxVals = c( -2, -1, 0, 1, 2))
plotCurves(m5, plotx = "x1", modx = "x2", modxVals = c(-2))
plotCurves(m5, plotx = "x1", modx = "x2", modxVals = "std.dev.")
plotCurves(m5, plotx = "x1", modx = "x2", modxVals = "quantile")
plotCurves(m5, plotx = "x3", modx = "x2")


if(require(carData)){
    mc1 <- lm(statusquo ~ income * sex, data = Chile)
    summary(mc1)
    plotCurves(mc1, plotx = "income")
    plotCurves(mc1, modx = "sex", plotx = "income")
    plotCurves(mc1, modx = "sex", plotx = "income", modxVals = "M")
    
    mc2 <- lm(statusquo ~ region * income, data =  Chile)
    summary(mc2)
    plotCurves(mc2, modx = "region", plotx = "income")
    plotCurves(mc2, modx = "region", plotx = "income",
               modxVals = levels(Chile$region)[c(1,4)])
    plotCurves(mc2, modx = "region", plotx = "income", modxVals = c("S","M","SA"))
    plotCurves(mc2, modx = "region", plotx = "income", modxVals = c("S","M","SA"),
               interval = "conf")
    
    plotCurves(mc2, modx = "region", plotx = "income", plotPoints = FALSE)
 
    mc3 <- lm(statusquo ~ region * income + sex + age, data =  Chile)
    summary(mc3)
    plotCurves(mc3, modx = "region", plotx = "income")
 
    mc4 <- lm(statusquo ~ income * (age + I(age^2)) + education + sex + age, data = Chile)
    summary(mc4)
    plotCurves(mc4, plotx = "age")
    plotCurves(mc4, plotx = "age", interval = "conf")
     
    plotCurves(mc4, plotx = "age", modx = "income")
    plotCurves(mc4, plotx = "age", modx = "income", plotPoints = FALSE)
    
    plotCurves(mc4,  plotx = "income", modx = "age")
    plotCurves(mc4,  plotx = "income", modx = "age", n = 8)
    
    plotCurves(mc4,  plotx = "income", modx = "age", modxVals = "std.dev.")
    plotCurves(mc4, modx = "income", plotx = "age", plotPoints = FALSE)
}

## Title: plot-barplot-1
## Author: Paul Johnson <pauljohn at ku.edu>
## Date posted: 2011
## Description. barplot label customization

### Paul Johnson
### Twisting the margins of a barplot

### I never thought too much about customizing barplots, but
### now I have learned some tricks to share.  Step through these
### examples to see the possibilities.

set.seed(424242)

x <- rpois(10, lambda = 10)

mynames <- c(rep("Really Long Name", 9), "Really Long Name Long Long")


#RSiteSearch("barplot names margins")


barplot(x)


### Note long names off edge:
barplot(x, names = mynames, las = 2)

### Abbreviate offers one solution
barplot(x, names = abbreviate(mynames), las = 2)

### Other option is to reset margins
###default mar is c(5.1, 4.1, 4.1, 2.1)

### Lets make the bottom margin larger
par(mar = c(15, 4.1, 4.1, 2.1))
barplot(x, names = mynames, las = 2)



### Now the bottom is finished. But I'd like a legend.

legend("topleft", legend = c("A really long label", "B", "Cee What I can do?", "D"), col = 1:2)

### My bar hits the legend. How to fix?


### It is not sufficient to simply "move the legend" up
### because then it runs off the edge of the graph
barplot(x, names = mynames, las = 2)
legend(2, 22, legend = c("A really long label", "B", "Cee What I can do?", "D"), col = 1:2)

### By itself, changing the top margin does not help.
### It is also vital to set the xpd parameter to T so that
### R will draw outside the main plot region
par(mar = c(10, 4.1, 8.1, 2.1))
par(xpd = T)
barplot(x, names = mynames, las = 2)
legend(2, 22, legend = c("A really long label", "B", "Cee What I can do?", "D"), col = 1:2)


### Now lets gain some control on the colors and bars
### The default colors are so dark.  You can't write on top
### of them.  You can grab shades of gray like "gray30" or such.

### I'll just specify 4 colors I know will work. I prefer
### the light gray colors because they can be written over.
mycols <- c("lightgray", "gray70", "gray80", "gray90", "gray75")

barplot(x, names = mynames, las = 2, col = mycols)

legend(2, 20, legend = c("A", "B", "C", "D"), fill = mycols)

### Still, I don't like the solid shades so much.
### fill up the bars with angled lines.
### density determines number of lines inside the bar.

myden <- c(60, 20, 30, 40, 25)
### angles determines the direction of lines inside bars
myangles <- c(20, -30, 60, -40, 10)

barplot(x, names = mynames, las = 2, col = mycols, density = myden, angle = c(20, 60, -20))

legend(1, 20, legend = c("A", "B", "C", "D"), density = myden, angle = myangles)

### for my coupe de grace, lets do some writing in the bars.

### Recall from Verzani you can get the x coordinates from the barplot
barPositions <- barplot(x, names = mynames, las = 2, col = mycols, density = myden, angle = c(20, 60, -20))

barPositions

### The text option srt = 90 turns the text sideways

### I'm just guessing that bars 1 and 8 should be labeled
### at coordinates 5 and 5

text (barPositions[1], 5, "my first bar is great", srt = 90)

text (barPositions[8], 5, "but 8 is greater", srt = 90)

### Recently I had the problem of drawing a "clustered" bar chart.
### Lets suppose our x variable really represents responses from
### 2 groups of respondents, Men and Women.

## Create the matrix
xmatr <- matrix(x, nrow = 5)

### Use beside = T to cause barplot to treat each column
### of values as a group
barplot(xmatr, beside = TRUE, names = c("Men", "Women"))

valueNames <- c("Very Strong", "Somewhat Strong", "Not Strong", "Somewhat Weak", "Very Weak")

### Use mtext to write in the margin so that labels on plot are nice.

par(mar = c(10, 4.1, 5.1, 2.1))

bp <- barplot(xmatr, beside = TRUE, names = NULL)

### bp contains the positions of the bars.
mtext(valueNames, side = 1, las = 3, at = bp)

mtext(c("Men", "Women"), side = 1, at = c(bp[3], bp[8]), line = 8, cex = 2)

### Adding numbers to the bars may clarify
text ( bp, as.vector(xmatr), labels = as.vector(xmatr), pos = 1)

### However, the default color is too dark.

### Retrieve the first 5 items from the default grey color scale

gc <- grey.colors(5)

### Replace the first one with a lighter gray

gc[1] <- "gray80"

bp <- barplot(xmatr, beside = TRUE, names = NULL, col = gc)

### bp contains the positions of the bars.
mtext(valueNames, side = 1, las = 3, at = bp)

mtext(c("Men", "Women"), side = 1, at = c(bp[3], bp[8]), line = 8, cex = 2)

### Adding numbers to the bars may clarify
text ( bp, as.vector(xmatr), labels = as.vector(xmatr), pos = 1)

### That's OK for me.  I wonder if I might not just write inside the
### bars. Hmmm.


bp <- barplot(xmatr, beside = TRUE, names = c("Men", "Women"), col = gc)

text(bp, 0.5*as.vector(xmatr), valueNames, srt = 90)

### Hm. That's OK, but maybe I'd pref uniform vertical
### placement of text.

bp <- barplot(xmatr, beside = TRUE, names = c("Men", "Women"), col = gc)

text(bp, 2, valueNames, srt = 90, pos = 4)

### Hell, now the text is aligned vertically, but not centered
### in the bar. I can't figure out all of the details concerning
###

### I'm not able to say for sure what the best fix might be.
### I can either manipulate the x placement like so

bp <- barplot(xmatr, beside = TRUE, names = c("Men", "Women"), col = gc)
text(bp-0.25, 2, valueNames, srt = 90, pos = 4)

### Or use the offset option in the text command.
### I do not understand why this particular value works.

bp <- barplot(xmatr, beside = TRUE, names = c("Men", "Women"), col = gc)
text(bp, 2, valueNames, srt = 90, pos = 4, offset = -0.15)


### Don't forget: When save this into a file, the graph
### may be re-sized and text might "overflow" the boxes.
### That's especially likely if you have a large graph
### on the screen and then try to save the file with
### dev.copy(postscript...)
### That will resize some things, but not all.

### Here is the way to protect yourself.  Resize your "screen" device
### so that it is the same size--in inches--as the output file.

### On linux, I run

x11( width = 6, height = 6 )

### On windows, I think it is windows( width = 6, height = 6 )

### After you do that, run the par command again. The par command
### has to be executed again each time a device is created. That applies to
### the most recently created device.

p <- barplot(xmatr, beside = TRUE, names = c("Men", "Women"), col = gc)
text(bp-0.25, 2, valueNames, srt = 90, pos = 4)

### If that looks OK, then do this


postscript(file = "mybar-1.eps", height = 6, width = 6, onefile = F, horizontal = F, paper = "special", family = "Times")
p <- barplot(xmatr, beside = TRUE, names = c("Men", "Women"), col = gc)
text(bp-0.25, 2, valueNames, srt = 90, pos = 4)
dev.off()


### note that the par settings have shifted back to the defaults.
### Each device is separate.
### As a result, the barplot we created before with large margins would
### make a horrible plot if you forgot to insert the par() commands here:


postscript(file = "mybar-2.eps", height = 6, width = 6, onefile = F, horizontal = F, paper = "special", family = "Times")
par(mar = c(10, 4.1, 5.1, 2.1))


bp <- barplot(xmatr, beside = TRUE, names = NULL, col = gc)

### bp contains the positions of the bars.
mtext(valueNames, side = 1, las = 3, at = bp)

mtext(c("Men", "Women"), side = 1, at = c(bp[3], bp[8]), line = 8, cex = 2)

### Adding numbers to the bars may clarify
text ( bp, as.vector(xmatr), labels = as.vector(xmatr), pos = 1)
dev.off()


### Finally, suppose you make a mistake of specifying your
### output device size too small. The graph I made before
### with the text in the bars looked good on the screen,
### but it looks bad in a smaller output device. The text
### does not match the bars in this example.

postscript(file = "mybar-4.eps", height = 4, width = 4, onefile = F, horizontal = F, paper = "special", family = "Times")
bp <- barplot(xmatr, beside = TRUE, names = c("Men", "Women"), col = gc)
text(bp-0.25, 2, valueNames, srt = 90, pos = 4)
dev.off()

library(rockchalk)

## Replicate some R classics.  The budworm.lg data from predict.glm
## will work properly after re-formatting the information as a data.frame:

## example from Venables and Ripley (2002, pp. 190-2.)
df <- data.frame(ldose = rep(0:5, 2),
                 sex = factor(rep(c("M", "F"), c(6, 6))),
                 SF.numdead = c(1, 4, 9, 13, 18, 20, 0, 2, 6, 10, 12, 16))
df$SF.numalive = 20 - df$SF.numdead

budworm.lg <- glm(cbind(SF.numdead, SF.numalive) ~ sex*ldose,
                  data = df,  family = binomial)

predictOMatic(budworm.lg)

predictOMatic(budworm.lg, n = 7)

predictOMatic(budworm.lg, predVals = c("ldose"), n = 7)

predictOMatic(budworm.lg, predVals = c(ldose = "std.dev.", sex = "table"))



## Now make up a data frame with several numeric and categorical predictors.

set.seed(12345)
N <- 100
x1 <- rpois(N, l = 6)
x2 <- rnorm(N, m = 50, s = 10)
x3 <- rnorm(N)
xcat1 <- gl(2,50, labels = c("M","F"))
xcat2 <- cut(rnorm(N), breaks = c(-Inf, 0, 0.4, 0.9, 1, Inf),
             labels = c("R", "M", "D", "P", "G"))
dat <- data.frame(x1, x2, x3, xcat1, xcat2)
rm(x1, x2, x3, xcat1, xcat2)
dat$xcat1n <- with(dat, contrasts(xcat1)[xcat1, , drop = FALSE])
dat$xcat2n <- with(dat, contrasts(xcat2)[xcat2, ])
STDE <- 15
dat$y <- with(dat,
              0.03 + 0.8*x1 + 0.1*x2 + 0.7*x3 + xcat1n %*% c(2) +
              xcat2n %*% c(0.1,-2,0.3, 0.1) + STDE*rnorm(N))
## Impose some random missings
dat$x1[sample(N, 5)] <- NA
dat$x2[sample(N, 5)] <- NA
dat$x3[sample(N, 5)] <- NA
dat$xcat2[sample(N, 5)] <- NA
dat$xcat1[sample(N, 5)] <- NA
dat$y[sample(N, 5)] <- NA
summarize(dat)


m0 <- lm(y ~ x1 + x2 + xcat1, data = dat)
summary(m0)
## The model.data() function in rockchalk creates as near as possible
## the input data frame.
m0.data <- model.data(m0)
summarize(m0.data)

## no predVals: analyzes each variable separately
(m0.p1 <- predictOMatic(m0))

## requests confidence intervals from the predict function
(m0.p2 <- predictOMatic(m0, interval = "confidence"))

## predVals as vector of variable names: gives "mix and match" predictions
(m0.p3 <- predictOMatic(m0, predVals = c("x1", "x2")))

## predVals as vector of variable names: gives "mix and match" predictions
(m0.p3s <- predictOMatic(m0, predVals = c("x1", "x2"), divider = "std.dev."))

## "seq" is an evenly spaced sequence across the predictor.
(m0.p3q <- predictOMatic(m0, predVals = c("x1", "x2"), divider = "seq"))

(m0.p3i <- predictOMatic(m0, predVals = c("x1", "x2"),
                         interval = "confidence", n = 3))

(m0.p3p <- predictOMatic(m0, predVals = c("x1", "x2"), divider = pretty))

## predVals as vector with named divider algorithms.
(m0.p3 <- predictOMatic(m0, predVals = c(x1 = "seq", x2 = "quantile")))
## predVals as named vector of divider algorithms

## same idea, decided to double-check
(m0.p3 <- predictOMatic(m0, predVals = c(x1 = "quantile", x2 = "std.dev.")))
getFocal(m0.data$x2, xvals =  "std.dev.", n = 5)


## Change from quantile to standard deviation divider
(m0.p5 <- predictOMatic(m0, divider = "std.dev.", n = 5))

## Still can specify particular values if desired
(m0.p6 <- predictOMatic(m0, predVals = list("x1" = c(6,7),
                            "xcat1" = levels(m0.data$xcat1))))

(m0.p7 <- predictOMatic(m0, predVals = c(x1 = "quantile", x2 = "std.dev.")))
getFocal(m0.data$x2, xvals =  "std.dev.", n = 5)

(m0.p8 <- predictOMatic(m0, predVals = list( x1 = quantile(m0.data$x1,
                        na.rm = TRUE, probs = c(0, 0.1, 0.5, 0.8,
                        1.0)), xcat1 = levels(m0.data$xcat1))))

(m0.p9 <- predictOMatic(m0, predVals = list(x1 = "seq", "xcat1" =
                                levels(m0.data$xcat1)), n = 8) )


(m0.p10 <- predictOMatic(m0, predVals = list(x1 = "quantile",
                                 "xcat1" = levels(m0.data$xcat1)), n =  5) )


(m0.p11 <- predictOMatic(m0, predVals = c(x1 = "std.dev."), n = 10))

## Previous same as

(m0.p11 <- predictOMatic(m0, predVals = c(x1 = "default"), divider =
                         "std.dev.", n = 10))

## Previous also same as

(m0.p11 <- predictOMatic(m0, predVals = c("x1"), divider = "std.dev.", n = 10))


(m0.p11 <- predictOMatic(m0,  predVals = list(x1 = c(0, 5, 8), x2 = "default"),
                         divider = "seq"))



m1 <- lm(y ~ log(10+x1) + sin(x2) + x3, data = dat)
m1.data <- model.data(m1)
summarize(m1.data)


(newdata(m1))
(newdata(m1, predVals = list(x1 = c(6, 8, 10))))
(newdata(m1, predVals = list(x1 = c(6, 8, 10), x3 = c(-1,0,1))))
(newdata(m1, predVals = list(x1 = c(6, 8, 10),
                 x2 = quantile(m1.data$x2, na.rm = TRUE), x3 = c(-1,0,1))))

(m1.p1 <- predictOMatic(m1, divider = "std.dev", n = 5))
(m1.p2 <- predictOMatic(m1, divider = "quantile", n = 5))

(m1.p3 <- predictOMatic(m1, predVals = list(x1 = c(6, 8, 10),
                            x2 = median(m1.data$x2, na.rm = TRUE))))

(m1.p4 <- predictOMatic(m1, predVals = list(x1 = c(6, 8, 10),
                                x2 = quantile(m1.data$x2, na.rm = TRUE))))

(m1.p5 <- predictOMatic(m1))
(m1.p6 <- predictOMatic(m1, divider = "std.dev."))
(m1.p7 <- predictOMatic(m1, divider = "std.dev.", n = 3))
(m1.p8 <- predictOMatic(m1, divider = "std.dev.", interval = "confidence"))


m2 <- lm(y ~ x1 + x2 + x3 + xcat1 + xcat2, data = dat)
##  has only columns and rows used in model fit
m2.data <- model.data(m2)
summarize(m2.data)

## Check all the margins
(predictOMatic(m2, interval = "conf"))

## Lets construct predictions the "old fashioned way" for comparison

m2.new1 <- newdata(m2, predVals = list(xcat1 = levels(m2.data$xcat1),
                           xcat2 = levels(m2.data$xcat2)), n = 5)
predict(m2, newdata = m2.new1)


(m2.p1 <- predictOMatic(m2,
                        predVals = list(xcat1 = levels(m2.data$xcat1),
                            xcat2 = levels(m2.data$xcat2)),
                        xcat2 = c("M","D")))
## See? same!

## Pick some particular values for focus
m2.new2 <- newdata(m2, predVals = list(x1 = c(1,2,3), xcat2 = c("M","D")))
## Ask for predictions
predict(m2, newdata = m2.new2)


## Compare: predictOMatic generates a newdata frame and predictions in one step

(m2.p2 <- predictOMatic(m2, predVals = list(x1 = c(1,2,3),
                                xcat2 = c("M","D"))))

(m2.p3 <- predictOMatic(m2, predVals = list(x2 = c(0.25, 1.0),
                                xcat2 = c("M","D"))))

(m2.p4 <- predictOMatic(m2, predVals = list(x2 = plotSeq(m2.data$x2, 10),
                                xcat2 = c("M","D"))))

(m2.p5 <- predictOMatic(m2, predVals = list(x2 = c(0.25, 1.0),
                                xcat2 = c("M","D")), interval = "conf"))

(m2.p6 <- predictOMatic(m2, predVals = list(x2 = c(49, 51),
                                xcat2 = levels(m2.data$xcat2),
                                x1 = plotSeq(dat$x1))))

plot(y ~ x1, data = m2.data)
by(m2.p6, list(m2.p6$xcat2), function(x) {
    lines(x$x1, x$fit, col = x$xcat2, lty = as.numeric(x$xcat2))
})

m2.newdata <- newdata(m2, predVals = list(x2 = c(48, 50, 52),
                              xcat2 = c("M","D")))
predict(m2, newdata = m2.newdata)

(m2.p7 <- predictOMatic(m2, predVals = list(x2 = c(48, 50, 52),
                                xcat2 = c("M","D"))))

(m2.p8 <- predictOMatic(m2,
             predVals = list(x2 = range(m2.data$x2, na.rm = TRUE),
             xcat2 = c("M","D"))))

(m2.p9 <- predictOMatic(m2, predVals = list(x2 = plotSeq(m2.data$x2),
             x1 = quantile(m2.data$x1, pr =c(0.33, 0.66), na.rm = TRUE),
             xcat2 = c("M","D"))))
plot(y ~ x2 , data = m2.data)

by(m2.p9, list(m2.p9$x1, m2.p9$xcat2), function(x) {lines(x$x2, x$fit)})



(predictOMatic(m2, predVals = list(x2 = c(50, 60), xcat2 = c("M","D")),
               interval = "conf"))

## create a dichotomous dependent variable
y2 <- ifelse(rnorm(N) > 0.3, 1, 0)
dat <- cbind(dat, y2)

m3 <- glm(y2 ~ x1 + x2 + x3 + xcat1, data = dat, family = binomial(logit))
summary(m3)
m3.data <- model.data(m3)
summarize(m3.data)

(m3.p1 <- predictOMatic(m3, divider = "std.dev."))

(m3.p2 <- predictOMatic(m3, predVals = list(x2 = c(40, 50, 60),
                             xcat1 = c("M","F")),
                        divider = "std.dev.", interval = "conf"))

## Want a full accounting for each value of x2?
(m3.p3 <- predictOMatic(m3,
                predVals = list(x2 = unique(m3.data$x2),
                    xcat1 = c("M","F")), interval = "conf"))


## Would like to write a more beautiful print method
## for output object, but don't want to obscure structure from user.
## for (i in names(m3.p1)){
##     dns <- cbind(m3.p1[[i]][i], m3.p1[[i]]$fit)
##     colnames(dns) <- c(i, "predicted")
##     print(dns)
## }

##' rockchalk: regression functions
##'
##'
##' Includes an ever-growing collection of functions that assist in
##' the presentation of regression models.  The initial function was
##' \code{\link{outreg}}, which produces LaTeX tables that summarize
##' one or many fitted regression models.  It also offers plotting
##' conveniences like \code{\link{plotPlane}} and
##' \code{\link{plotSlopes}}, which illustrate some of the variables
##' from a fitted regression model. For a detailed check on
##' multicollinearity, see \code{\link{mcDiagnose}}.  The user should
##' be aware of this fact: Not all of these functions lead to models
##' or types of analysis that we endorese.  Rather, they all lead to
##' analysis that is endorsed by some scholars, and we feel it is
##' important to facilitate the comparison of competing methods.  For
##' example, the function \code{\link{standardize}} will calculate
##' standardized regression coefficients for all predictors in a
##' regression model's design matrix in order to replicate results
##' from other statistical frameworks, no matter how unwise the use of
##' such coefficients might be. The function \code{\link{meanCenter}}
##' will allow the user to more selectively choose variables for
##' centering (and possibly standardization) before they are entered
##' into the design matrix.  Because of the importance of interaction
##' variables in regression analysis, the \code{\link{residualCenter}}
##' and \code{\link{meanCenter}} functions are offered.  While mean
##' centering does not actually help with multicollinearity of
##' interactive terms, many scholars have argued that it does.  The
##' meanCenter function can be compared with the "residual centering"
##' of interaction terms.
##'
##' @name rockchalk-package
##' @aliases rockchalk-package rockchalk
##' @docType package
##' @author Paul E. Johnson \email{pauljohn@@ku.edu}
##'
##' @references http://pj.freefaculty.org/R
##' @keywords regression hplot
NULL






##' Religious beliefs and crime rates
##'
##' The data national-level summary indicators of public opinion about
##' the existence of heaven and hell as well as the national rate of
##' violent crime.
##' @name religioncrime
##' @docType data
##' @usage data(religioncrime)
##' @author Paul E. Johnson \email{pauljohn@@ku.edu} and Anonymous
##' @format data.frame: 51 obs. of 3 variables
##' @keywords datasets
##' @source  Anonymous researcher who claims the data is real.
##' @examples
##' require(rockchalk)
##' data(religioncrime)
##' mod1 <- lm(crime ~ heaven, data=religioncrime)
##' mod2 <- lm(crime ~ hell, data=religioncrime)
##' mod3 <- lm(crime ~ heaven + hell, data=religioncrime)
##' with(religioncrime,
##' mcGraph1(heaven, hell, crime)
##' )
##' with(religioncrime,
##' mcGraph2(heaven, hell, crime)
##' )
##' mod1 <- with(religioncrime,
##' mcGraph3(heaven, hell, crime)
##' )
##' summary(mod1[[1]])
##' ##TODO: Draw more with perspective matrix mod1[[2]]
NULL



##' Cheating and Looting in Japanese Electoral Politics
##'
##' Extracted from the "cheating-replication.dta" data file
##' with permission by the authors, Benjamin Nyblade and Steven
##' Reed. The Stata data file provided by the authors included
##' many constructed variables that have been omitted.  Within
##' R, these can be easily re-contructed by users.
##'
##' Special thanks to NyBlade and Reed for permission to repackage
##' this data. Also special thanks to them for creating an especially
##' transparent variable naming scheme.
##'
##' The data set includes many columns for variables that can easily
##' be re-constructed from the columns that are provided here.  While
##' Stata users might need to manually create 'dummy variables' and
##' interactions, R users generally do not do that manually.
##'
##' These variables from the original data set were omitted:
##'
##' Dummy variables for the year variable: c("yrd1", "yrd2", ...,
##' "yrd17", "yrd18")
##'
##' Dummy variables for the ku variable:
##' c("ku1", "ku2", ..., "ku141", "ku142")
##'
##' Constructed product variables: c("actualratiosq", "viabsq",
##' "viab_candcamp_divm", "viab_candothercamp_divm",
##' "viabsq_candcamp_divm", "viabsq_candothercamp_divm",
##' "absviab_candcamp", "absviab_candothercamp",
##' "absviab_candcamp_divm", "absviab_candothercamp_divm",
##' "viabsq_candcamp", "viabsq_candothercamp", "viab_candcamp",
##' "viab_candothercamp", "candothercamp_divm", "candcamp_divm",
##' "candcampminusm", "candothercampminusm", "predratiosq", "absviab")
##'
##' Mean centered variables: constr2 <- c("viab_candcampminusm",
##' "viab_candothercampminusm", "viabsq_candothercampminusm",
##' "viabsq_candcampminusm")
##'
##' In the end, we are left with these variables:
##'
##' [1] "ku"
##' [2] "prefecture"
##' [3] "dist"
##' [4] "year"
##' [5] "yr"
##' [6] "cdnr"
##' [7] "jiban"
##' [8] "cheating"
##' [9] "looting"
##' [10] "actualratio"
##' [11] "viab"
##' [12] "inc"
##' [13] "cons"
##' [14] "ur"
##' [15] "newcand"
##' [16] "jwins"
##' [17] "cons_cwins"
##' [18] "oth_cwins"
##' [19] "camp"
##' [20] "fleader"
##' [21] "incablast"
##' [22] "predratio"
##' [23] "m"
##' [24] "candcamp"
##' [25] "candothercamp"
##' [26] "kunocheat"
##' [27] "kunoloot"
##'
##' @name cheating
##' @docType data
##' @usage data(cheating)
##' @author Paul E. Johnson \email{pauljohn@@ku.edu}, on behalf of Benjamin Nyblade and Steven Reed
##' @format data.frame: 16623 obs. on 27 variables
##' @keywords datasets
##' @source \url{http://faculty.arts.ubc.ca/bnyblade/publications.html}.
##' @references
##' Benjamin Nyblade and Steven Reed, "Who Cheats? Who
##' Loots? Political Competition and Corruption in Japan, 1947-1993."
##' American Journal of Political Science 52(4): 926-41. October 2008.
##' @examples
##' require(rockchalk)
##' data(cheating)
##'
##' table1model2 <- glm(cheating ~ viab + I(viab^2) + inc + cons + ur
##' + newcand + jwins + cons_cwins + oth_cwins, family = binomial(link
##' = "logit"), data = cheating)
##'
##' predictOMatic(table1model2)
##'
##' predictOMatic(table1model2, interval = "confidence")
##'
##' ## The publication used "rare events logistic", which I'm not bothering
##' ## with here because I don't want to invoke additional imported packages.
##' ## But the ordinary logit results are proof of concept.
NULL

##' Hypothesis tests for Simple Slopes Objects
##'
##' Conducts t-test of the hypothesis that the "simple slope" line for
##' one predictor is statistically significantly different from zero
##' for each value of a moderator variable. The user must first run
##' \code{plotSlopes()}, and then give the output object to
##' \code{plotSlopes()}. A plot method has been implemented for
##' testSlopes objects. It will create an interesting display, but
##' only when the moderator is a numeric variable.
##'
##' This function scans the input object to detect the focal values of
##' the moderator variable (the variable declared as \code{modx} in
##' \code{plotSlopes}). Consider a regression with interactions
##'
##' y <- b0 + b1*x1 + b2*x2 + b3*(x1*x2) + b4*x3 + ... + error
##'
##' If \code{plotSlopes} has been run with the argument plotx="x1" and
##' the argument modx="x2", then there will be several plotted lines,
##' one for each of the chosen values of x2.  The slope of each of
##' these lines depends on x1's effect, b1, as well as the interactive
##' part, b3*x2.
##'
##' This function performs a test of the null hypothesis of the slope
##' of each fitted line in a \code{plotSlopes} object is statistically
##' significant from zero. A simple t-test for each line is offered.
##' No correction for the conduct of multiple hypothesis tests (no
##' Bonferroni correction).
##'
##' When \code{modx} is a numeric variable, it is possible to conduct
##' further analysis. We ask "for which values of \code{modx} would
##' the effect of \code{plotx} be statistically significant?"  This is
##' called a Johnson-Neyman (Johnson-Neyman, 1936) approach in
##' Preacher, Curran, and Bauer (2006). The interval is calculated
##' here.  A plot method is provided to illustrate the result.
##'
##' @param object Output from the plotSlopes function
##' @return A list including 1) the hypothesis test table, 2) a copy of
##' the plotSlopes object, and, for numeric
##' modx variables, 3) the Johnson-Neyman (J-N) interval boundaries.
##' @export
##' @import carData
##' @seealso plotSlopes
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @references
##' Preacher, Kristopher J, Curran, Patrick J.,and Bauer, Daniel J. (2006).
##' Computational Tools for Probing Interactions in Multiple Linear
##' Regression, Multilevel Modeling, and Latent Curve Analysis.
##' Journal of Educational and Behavioral Statistics. 31,4, 437-448.
##'
##' Johnson, P.O. and Neyman, J. (1936). "Tests of certain linear
##' hypotheses and their applications to some educational problems.
##' Statistical Research Memoirs, 1, 57-93.
##' @example inst/examples/testSlopes-ex.R
testSlopes <-
    function(object)
{
    if (!inherits(object, "plotSlopes"))
        stop("use only with \"plotSlopes\" objects")
    model <-  eval(parse(text=object$call$model))
    plotx <- object$call$plotx
    modx <- object$call$modx

    ivs <- attr(terms(model), "term.labels")
    relevantInteractions <- c(paste(plotx, ":", modx, sep = ""),
                              paste(modx, ":", plotx, sep = ""))

    interactionsIn <- relevantInteractions[which(relevantInteractions %in% ivs)]

    if (!any(relevantInteractions %in% ivs)) {
        cat("There were no interactions in the plotSlopes object, so testSlopes can't offer any advice.\n")
        return(NULL)
    }

    ## Whew. We did not return, so let's get down to business.
    modxVar <- object$newdata[ , modx]
    modxVals <- object$modxVals
    bs <- coef(model)
    V <- vcov(model)

    ## If modx is a factor, only need to test a particular set of lines.
    if (is.factor(modxVar)) {
        mcoef <- coef(model)
        modxContrastNames <- c(grep(plotx,  grep(modx, names(mcoef), value = TRUE),  value=TRUE))
        slope <- mcoef[modxContrastNames] + coef(model)[plotx]

        ## 2013-02-19 Sunthud Pornprasertmanit spots bug here:
        ## Problem: diag doesn't work when argument is a single real number.
        ## Fix by inserting drop=FALSE (wrote a blog post about the "drop gotcha"
        seslope <- sqrt(as.vector(V[plotx, plotx, drop = FALSE]) +  diag(V[modxContrastNames, modxContrastNames, drop = FALSE]) + 2* V[modxContrastNames, plotx, drop = FALSE])

        modxContrastNames <- c(plotx, modxContrastNames)
        slope <- c(mcoef[plotx], slope)
        seslope <- c(sqrt(V[plotx, plotx]) , seslope)
        tbsimple <- slope/seslope

        testSlopes <- data.frame(modx = modxContrastNames, b = slope,
                                 se = seslope, t = tbsimple, p = 2 *
                                 pt(abs(tbsimple), df =
                                 model$df.residual, lower.tail =
                                 FALSE))

        row.names(testSlopes) <-  levels(model.frame(model)[ , modx])

        colnames(testSlopes) <- c(deparse(modx), "slope",
                                  "Std. Error", "t value", "Pr(>|t|)")

        cat(paste("These are the straight-line \"simple slopes\" of the variable", plotx, " \n for the selected moderator values. \n"))
        print(testSlopes)
        res <- list("hypotests" = testSlopes, pso = object)
        class(res) <- "testSlopes"
        return(invisible(res))
    }

    ## Aha! We did not return. So this input is numeric. Continue.

    bmodx <- NULL
    bplotx <- bs[plotx]

    bmodx <- bs[interactionsIn]
    bsimple <- bplotx + bmodx * modxVals
    covbsimple <- cbind(1, modxVals^2, 2 * modxVals) %*%
        c(V[plotx, plotx], V[names(bmodx), names(bmodx)], V[plotx, names(bmodx)])
    tbsimple <- bsimple/sqrt(covbsimple)

    testSlopes <- data.frame( modx = modxVals, b = bsimple,
                             se = sqrt(covbsimple), t = tbsimple,
                             p = 2 * pt(abs(tbsimple),
                             df = model$df.residual, lower.tail = FALSE))

    colnames(testSlopes) <- c(deparse(modx), "slope", "Std. Error", "t value", "Pr(>|t|)")

    ## Just for numeric variables, the J-N calculation
    roots <- NULL

    mm <- model.matrix(model)
    b2 <- bmodx <- NULL
    b1 <- bplotx <- bs[plotx]

    b2 <- bs[interactionsIn]

    if(is.null(b2)) stop("b2 is null, there's no interation! Logic error")

    Tcrit <- qt(0.975, model$df)

    ## Quadratic formula. Solve
    ## Tcrit < T = (b1 + b2*modx)/s.e.(b1 + b2*modx)
    ## Tcrit < T = (b1 + b2*modx)/sqrt(Var(b1) + modx^2*Var(b2) + 2 Var[b1,b2])

    jn <- list()
    jn$a <- b2^2 - (Tcrit^2) * V[interactionsIn, interactionsIn]
    jn$b <- 2*b1*b2 - (Tcrit^2) * 2 * V[plotx, interactionsIn]
    jn$c <- b1^2 - (Tcrit^2) * V[plotx, plotx]
    jn$inroot <- (jn$b^2) - 4 * jn$a * jn$c

    ##complex root check, if yes, exit with message.
    if (jn$inroot <= 0) {
        print(paste("There are no real roots to the quadratic equation that represents regions of statistical significance."))
            if (jn$a > 0) {
                print("That means the slope (b1 + b2modx)plotx is statistically significant for all values of modx")
            } else {
                print("In this case, that means the  slope (b1 + b2modx)plotx is never statistically significant")
            }
        res <- list("hypotests" = testSlopes, "jn" = jn, pso = object)
        class(res) <- "testSlopes"
        return(invisible(res))
    }


    ## else (!inroot <= 0)
    ## Whew. Roots not complex, otherwise would have returned
    jn$roots <- c( (-jn$b - sqrt(jn$inroot))/(2*jn$a),
                  (-jn$b + sqrt(jn$inroot))/(2*jn$a) )
    jn$roots <- sort(jn$roots)
    names(jn$roots) <- c("lo","hi")
    if (jn$a > 0) {
        cat(paste("Values of", modx, "OUTSIDE this interval:\n"))
        print(jn$roots)
        cat(paste("cause the slope of (b1 + b2*", modx,")", plotx, " to be statistically significant\n", sep = ""))
    } else {
        cat(paste("Values of", modx, "INSIDE this interval:\n"))
        print(jn$roots)
        cat(paste("cause the slope of (b1 + b2*", modx,")", plotx, " to be statistically significant\n", sep = ""))
    }


    ## print(paste("b1 = b2x at", -b1/b2))
    ## print(paste("quadratic minimum/maximum point at", -jn$b/(2*jn$a)))
    ## if(jn$a > 0) print("that is a minimum") else print("that is a maximum")
    res <- list("hypotests" = testSlopes, "jn" = jn, pso = object)
    class(res) <- "testSlopes"
    return(invisible(res))
}

NULL

##' Plot testSlopes objects
##'
##' plot.testSlopes is a method for the
##' generic function plot. It has been revised so that it creates a plot
##' illustrating the marginal effect, using the
##' Johnson-Neyman interval calculations to highlight the
##' "statistically significantly different from zero" slopes.
##' @return \code{NULL}
##' @author <pauljohn@@ku.edu>
##' @method plot testSlopes
##' @export
##' @param x   A testSlopes object.
##' @param ... Additional arguments that are ignored currently.
##' @param shade Optional. Create colored polygon for significant regions.
##' @param col Optional. Color of the shaded area. Default transparent pink.
plot.testSlopes <-
    function(x, ..., shade = TRUE, col = rgb(1, 0, 0, 0.10))
{
    ## Following should be unnecessary.
    if (!inherits(x, "testSlopes"))
         stop("use only with \"testSlopes\" objects")
    tso <- x
    model <-  eval(parse(text = tso$pso$call$model))
    modx <- tso$pso$call$modx
    modxVar <- model$model[, modx]
    modxRange <- magRange(modxVar, 1.1)

    if (x$jn$inroot <= 0) {
        print("There are no real roots. There is nothing worth plotting")
        return(NULL)
    }

    if ((x$jn$roots["hi"] < modxRange[1]) || (x$jn$roots["lo"] > modxRange[2])) {
            ## both roots outside observed x
    }

    ## tedious exercise to figure out which type of interval of significant values we have
    if (x$jn$roots["lo"] > modxRange[1] && x$jn$roots["hi"] < modxRange[2]){
        ##2 roots inside
        idxStart <- c(modxRange[1], x$jn$roots)
        idxEnd <- c(x$jn$roots, modxRange[2])
        intervals <- if(x$jn$a > 0) c(1,3) else c(2)
        markAt <- x$jn$roots
    } else if (modxRange[1] < x$jn$roots["lo"]){
        ## hi root inside
        idxStart <- c(modxRange[1], x$jn$roots["lo"])
        idxEnd <- c(x$jn$roots["lo"], modxRange[2])
        intervals <- if(x$jn$a > 0) c(1) else c(2)
        markAt <- x$jn$roots["lo"]
    } else if (x$jn$roots["hi"] < modxRange[2]){
        ## lo root inside
        idxStart <- c(modxRange[1], x$jn$roots["hi"])
        idxEnd <- c(x$jn$roots["hi"], modxRange[2])
        intervals <- if(x$jn$a > 0) c(2) else c(1)
        markAt <- x$jn$roots["hi"]
    } else {
        stop("Your observed moderator data does not overlap with
              the region on which the slope would be significant.
              So we are stopping now.")
    }


    if (!is.numeric(modxVar)){
        print("Sorry, but I can't see how it makes sense to use a non-numeric moderator in these plots")
        warning("testSlopes: non-numeric moderator")
        return(NULL)
    }

    plotx <- tso$pso$call$plotx

    modxSeq <- plotSeq(modxRange, length.out=100)

    depVarVals <- model.response(model.frame(model))
    modyRange <- magRange(depVarVals, 1.1)
    Tcrit <- qt(0.975, model$df)

    ivs <- attr(terms(model), "term.labels")
    relevantInteractions <- c(paste(plotx, ":", modx, sep = ""),
                              paste(modx, ":", plotx, sep = ""))
    interactionsIn <- relevantInteractions[which(relevantInteractions %in% ivs)]
    if (length(interactionsIn) > 1) stop("sorry, I haven't figured out what do to do if interactionsIn > 1")

    bs <- coef(model)
    V <- vcov(model)

    bsslope <-  bs[plotx] + bs[interactionsIn]*modxSeq
    bsse <- sqrt(V[plotx,plotx] + modxSeq^2 * V[interactionsIn,interactionsIn] + 2 * modxSeq * V[plotx, interactionsIn])

    ## MM marginal matrix, similar to return of the predict() function
    ## This makes reasonable looking confidence intervals
    MM <- cbind(fit = bsslope,
                lwr = bsslope - Tcrit * bsse,
                upr = bsslope + Tcrit * bsse,
                modxSeq = modxSeq, se = bsse,
                p = 2*pt(abs(bsslope/bsse),
                lower.tail = FALSE, df = model$df))


    ylab <- substitute("Marginal Effect of" ~~ AA: ~~ "(" * hat(b)[AA] + hat(b)[BB:AA]*BB[i] *")", list(AA=plotx, BB=modx))
    ylim <- magRange(range(c(MM[ , "lwr"],MM[, "upr"])), c(1.15, 1))

    op1 <- par(no.readonly = TRUE)
    par(mar = par("mar") + c(0,0.2,0,0))

    plot(fit ~ modxSeq, data = MM, type="l", xlab = paste("The Moderator: ", modx), ylim = ylim, ylab = ylab)
    par <- op1

    abline(h = 0, col = gray(.80))
    lines(upr ~ modxSeq, data = MM, lty = 2, col = gray(.50))
    lines(lwr ~ modxSeq, data = MM, lty = 2, col = gray(.50))


    for (i in intervals){
        modxSeq <- seq(idxStart[i], idxEnd[i],
                   length.out = as.integer(40*(idxEnd[i] - idxStart[i])/diff(modxRange)))
        bsslope <-  bs[plotx] + bs[interactionsIn]*modxSeq
        bsse <- sqrt(V[plotx,plotx] + modxSeq^2 * V[interactionsIn,interactionsIn] + 2 * modxSeq * V[plotx, interactionsIn])

        ## MM marginal matrix, similar to return of the predict() function
        ## This makes reasonable looking confidence intervals
        MMsm <- cbind(fit = bsslope,
                       lwr = bsslope - Tcrit * bsse,
                       upr = bsslope + Tcrit * bsse,
                       modxSeq = modxSeq, se = bsse,
                       p = 2*pt(abs(bsslope/bsse),
                       lower.tail = FALSE, df = model$df))

        arrows(x0 = MMsm[ ,"modxSeq"], y0 = MMsm[ ,"lwr"],
               x1 = MMsm[ ,"modxSeq"], y1 = MMsm[ ,"upr"],
               angle = 90, length = 0.05, code = 3, col = gray(.70))

        if (shade == TRUE){
            polygon(x = c(MMsm[ ,"modxSeq"], rev(MMsm[ ,"modxSeq"])),
                    y = c(MMsm[ ,"upr"], rev(MMsm[ , "lwr"]) ),
                    col = col, border = gray(.80))
        }
    }

    ## draw mtext markers for internal roots
    for (i in markAt){
        lines(x = rep(i, 2),
              y = c(0, ylim[1]), lty=4, col = gray(.70))
        mtext(text = round(i, 2),
              at = i,  side = 1, line = 2, col = rgb(1, 0, 0, 0.70))
    }


    legend("topleft", legend = c("Marginal Effect", "95% Conf. Int."),
           lty = c(1, 2), col = c(1, gray(.50)), bg = "white")

    if (shade == TRUE){
        legend("bottomright", title = "Shaded Region: Null Hypothesis",
               legend = substitute(b[AA] + b[BB:AA]*BB[i] == 0 ~~ "rejected", list(AA=plotx, BB=modx)),
               fill = c(rgb(1,0,0, 0.10)), bg = "white")
    }






    if (0){
        ## Now draw the quadratic equation, to show the solution of slope/se - Tcrit = 0

        ##print("A plot of the quadratic equation should appear now, just for fun")
        if (tso$jn$a > 0) {
            xps <- plotSeq(magRange(tso$jn$roots, 1.3), length.out=100)
            ## cat(paste("Values of modx OUTSIDE this interval:\n"))
        } else {
            xps <- plotSeq(magRange(modxVar, 1.5), length.out=100)
        }

        y <- tso$jn$a * xps^2 + tso$jn$b*xps + tso$jn$c
        plot(xps, y, type="l", xlab=paste("The Moderator:", modx), ylab="slope/se - Tcrit")
        if( !is.null(tso$jn$roots) ){
            if(tso$jn$a < 0 ){
                arrows( tso$jn$roots[1], 0, tso$jn$roots[2], 0, col = "red", angle = 90, lwd = 3, code = 3, length = 0.1)
                text( mean(range(xps)), range(y)[1], pos = 3, label = expression(paste((b[plotx] + b[modx:plotx]*modx)*plotx, " is significant in the red zone")))
            } else {
                arrows(min(xps), 0, tso$jn$roots[1], 0, col = "red", angle = 90, lwd = 3, code = 2, length = 0.1)
                arrows(tso$jn$roots[2], 0, max(xps), 0, col = "red", angle = 90, lwd = 3, code = 1, length = 0.1)
                text( mean(range(xps)), range(y)[2], pos = 1,
                     label = expression(paste((b[plotx] + b[modx:plotx]*modx)*plotx,
                         " is significant in the red zone")))
            }
        }
        abline(h = 0, col = "gray80")
    }

    invisible(MM)
}

##' Extracts numeric variables and presents an summary in
##' a workable format.
##'
##' Finds the numeric variables, and ignores the others. (See
##' \code{summarizeFactors} for a function that handles non-numeric
##' variables). It will provide quantiles (specified \code{probs} as
##' well as other summary statistics, specified \code{stats}.  Results
##' are returned in a data frame. The main benefits from this compared
##' to R's default summary are 1) more summary information is returned
##' for each variable (dispersion), 2) the results are returned in a
##' form that is easy to use in further analysis, 3) the variables in
##' the output may be alphabetized.
##' 
##' @param dat a data frame or a matrix
##' @param alphaSort If TRUE, the columns are re-organized in
##'     alphabetical order. If FALSE, they are presented in the
##'     original order.
##' @param probs Controls calculation of quantiles (see the R
##'     \code{quantile} function's \code{probs} argument). If FALSE or
##'     NULL, no quantile estimates are provided. Default is
##'     \code{c("min" = 0, "med" = 0.5, "max" = 1.0)}, which will
##'     appear in output as \code{c("min", "med", "max")}. Other
##'     values between 0 and 1 are allowed. For example, \code{c(0.3, 0.7)}
##'     will appear in output as \code{pctile_30\%} and
##'     \code{pctile_70\%}.
##' @param stats A vector including any of these: \code{c("min",
##'     "med", "max", "mean", "sd", "var", "skewness", "kurtosis",
##'     "nobs", "nmiss")}. Default includes all except \code{var}.
##'     "nobs" means number of observations with non-missing, finite
##'     scores (not NA, NaN, -Inf, or Inf). "nmiss" is the number of
##'     cases with values of NA. If FALSE or NULL, provide none of
##'     these.
##' @param na.rm default TRUE. Should missing data be removed to
##'     calculate summaries?
##' @param unbiased If TRUE (default), skewness and kurtosis are
##'     calculated with biased corrected (N-1) divisor in the standard
##'     deviation.
##' @param digits Number of digits reported after decimal
##'     point. Default is 2
##' @export
##' @return a data.frame with one column per summary element (rows are
##'     the variables).
##' @seealso summarize and summarizeFactors
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##'
summarizeNumerics <-
    function(dat, alphaSort = FALSE, probs = c(0, 0.5, 1.0),
             stats = c("mean", "sd", "skewness",
                      "kurtosis", "nobs", "nmiss"), na.rm = TRUE,
             unbiased = TRUE, digits = 2)
{
    ##:ess-bp-start::browser@nil:##
if (isTRUE(stats)) stats <- c("mean", "sd", "skewness",
                                     "kurtosis", "nobs", "nmiss")
    if (is.atomic(dat)) {
        datname <- deparse(substitute(dat))
        dat <- data.frame(dat)
        if (NCOL(dat) == 1) {
            colnames(dat) <- datname
        } else {
            colnames(dat) <- paste0(datname, "_",  seq(1, NCOL(dat)))
        }
    } else if (!is.data.frame(dat)) dat <- as.data.frame(dat)

    var2 <- function(x, na.rm, unbiased) {
        if (unbiased) {
            var(x, na.rm = na.rm)
        } else {
            mean((x - mean(x, na.rm = na.rm))^2)
        }
    }

    sd2 <- function(x, na.rm, unbiased) {
        if (unbiased) {
            sd(x, na.rm = na.rm)
        } else {
            sqrt(mean((x - mean(x, na.rm = na.rm))^2))
        }
    }
       
    nums <- sapply(dat, is.numeric)
    if (sum(nums) == 0) return(NULL)
    datn <- dat[, nums, drop = FALSE]
    if (alphaSort)
        datn <- datn[, sort(colnames(datn)), drop = FALSE]

    qtiles <- NULL
    sumdat <- NULL

    if (!missing(probs) && !is.null(probs) && is.logical(probs) &&
        length(probs) == 1 && !is.na(probs) && probs){
        prob.std <- c("min" = 0.0, "med" = 0.5, "max" = 1.0)
        if(isTRUE(probs)){
            probs <- prob.std
        }
        ## if stats includes "min" "med" or "max", adjust probs
        prob.toadd <- names(prob.std)[names(prob.std) %in% stats]
        probs <- sort(unique(c(probs, prob.std[prob.toadd])))
    }

    if (is.numeric(probs)){
        qtiles <- t(apply(datn, 2, stats::quantile,
                          probs = probs,
                          na.rm = na.rm))
        ## TODO: use regex to match leading digit
        colnames(qtiles) <- paste0("pctile_", colnames(qtiles))
        old <- c("pctile_0%", "pctile_50%", "pctile_100%")
        new <- c("min", "med", "max")
        for(i in 1:3){
            colnames(qtiles) <- gsub(old[i], new[i], colnames(qtiles))
        }        
    }
    if (any(c("mean", "sd", "var", "skewness", "kurtosis", "nobs", "nmiss") %in% stats)) {
        res <- list()
        if("mean" %in% stats)                    
            res[["mean"]] <- apply(datn, 2, mean, na.rm = na.rm)
        if("sd" %in% stats)
            res[["sd"]] <- apply(datn, 2, sd2, na.rm = na.rm,
                                 unbiased = unbiased)
        if("var" %in% stats)
            res[["var"]] <- apply(datn, 2, var2, na.rm = na.rm,
                                  unbiased = unbiased)
        if("skewness" %in% stats)
            res[["skewness"]] <- apply(datn, 2, skewness, na.rm = na.rm,
                                            unbiased = unbiased)
        if("kurtosis" %in% stats)
            res[["kurtosis"]] <- apply(datn, 2, kurtosis, na.rm = na.rm,
                                       unbiased = unbiased)

        if("nobs" %in% stats)
            res[["nobs"]] <- apply(datn, 2, function(x) sum(is.finite(x)))
        if("nmiss" %in% stats)
            res[["nmissing"]] <- apply(datn, 2,
                                       function(x) sum(is.na(x)))
        sumdat <- as.data.frame(res)
    }
    if (!is.null(qtiles) && !is.null(sumdat)){
        reslt <- merge(qtiles, sumdat, by = "row.names", sort = FALSE)
        rownames(reslt) <- reslt$Row.names
        reslt[["Row.names"]] <- NULL
    } else if (is.null(qtiles)){
        reslt <- sumdat
    } else if (is.null(sumdat)){
        reslt <- qtiles
    }
   
    attr(reslt, "stats") <- stats
    attr(reslt, "digits") <- digits
    attr(reslt, "probs") <- probs
    reslt
}
NULL

##' Calculate excess kurtosis
##'
##' Kurtosis is a summary of a distribution's shape, using the Normal
##' distribution as a comparison. A distribution with high kurtosis is
##' said to be leptokurtic. It has wider, "fatter" tails and a
##' "sharper", more "peaked" center than a Normal distribution. In a
##' standard Normal distribution, the kurtosis is 3.  The term
##' "excess kurtosis" refers to the difference \eqn{kurtosis - 3}.
##' Many researchers use the term kurtosis to refer to
##' "excess kurtosis" and this function follows suit.  The user may
##' set excess = FALSE, in which case the uncentered kurtosis is
##' returned.
##'
##' If kurtosis is smaller than 3 (or excess kurtosis is negative),
##' the tails are "thinner" than the normal distribution (there is
##' lower chance of extreme deviations around the mean). If kurtosis
##' is greater than 3 (excess kurtosis positive), then the tails are
##' fatter (observations can be spread more widely than in the Normal
##' distribution).
##' 
##' The kurtosis may be calculated with the small-sample
##' bias-corrected estimate of the variance. Set unbiased = FALSE if
##' this is not desired.  It appears somewhat controversial whether
##' this is necessary. According to the
##' US NIST,
##' \url{http://www.itl.nist.gov/div898/handbook/eda/section3/eda35b.htm},
##' kurtosis is defined as
##'                   
##' \deqn{kurtosis =  ( mean((x - mean(x))^4) )/ var(x)^2}
##'             
##' where \eqn{var(x)} is calculated with the denominator \eqn{N},
##' rather than \eqn{N-1}.
##'
##' A distribution is said to be leptokurtic if it is tightly bunched
##' in the center (spiked) and there are long tails. The long tails
##' reflect the probability of extreme values.
##' @param x A numeric variable (vector)
##' @param na.rm default TRUE. If na.rm = FALSE and there are missing
##'     values, the mean and variance are undefined and this function
##'     returns NA.
##' @param excess default TRUE. If true, function returns excess
##'     kurtosis (kurtosis -3). If false, the return is simply
##'     kurtosis as defined above.
##' @param unbiased default TRUE. Should the denominator of the
##'     variance estimate be divided by N-1, rather than N?
##' @export
##' @return A scalar value or NA
##' @author Paul Johnson <pauljohn@@ku.edu>
kurtosis <- function(x, na.rm = TRUE, excess = TRUE, unbiased = TRUE){
    if (!isTRUE(na.rm) & sum(is.na(x) > 0)) return(NA)
    x <- x[!is.na(x)]
    xm <- mean(x)
    xd <- x - xm
    var <- mean(xd^2)
   
    if (unbiased){
        kur <- mean(xd^4)/(var(x)^2)
    } else {
         kur <- mean(xd^4)/var^2
    }
    if (isTRUE(excess)) kur <- kur - 3
    kur
}


##' Calculate skewness
##'
##' Skewness is a summary of the symmetry of a distribution's
##' probability density function. In a Normal distribution, the
##' skewness is 0, indicating symmetry about the expected value.
##'
##' If na.rm = FALSE and there are missing values, the mean and
##' variance are undefined and this function returns NA.
##'
##' The skewness may be calculated with the small-sample bias-corrected
##' estimate of the standard deviation.  It appears somewhat controversial
##' whether this is necessary, hence the argument unbiased is provided.
##' Set unbiased = FALSE if it is desired to have the one recommended
##' by NIST, for example. According to the US NIST,
##' \url{http://www.itl.nist.gov/div898/handbook/eda/section3/eda35b.htm},
##' skewness is defined as the mean of cubed deviations divided by the
##' cube of the standard deviation.
##'       
##'                mean((x - mean(x))^3)
##' skewness =    ___________________
##'                  sd(x)^3
##'
##' where sd(x) is calculated with the denominator N, rather than
##' N-1. This is the Fisher-Pearson coefficient of skewness, they claim.
##' The unbiased variant uses the standard deviation divisor (N-1) to
##' bias-correct the standard deviation. 
##' 
##' @param x A numeric variable (vector)
##' @param na.rm default TRUE. Should missing data be removed?
##' @param unbiased default TRUE. Should the denominator of the
##'     variance estimate be divided by N-1?
##' @export
##' @return A scalar value or NA
##' @author Paul Johnson <pauljohn@@ku.edu>
skewness <- function(x, na.rm = TRUE, unbiased = TRUE){
    if (!isTRUE(na.rm) & sum(is.na(x) > 0)) return(NA)
    x <- x[!is.na(x)]
    xm <- mean(x)
    if (unbiased){
        skew <-  mean((x - xm)^3)/var(x)^(3/2)
    } else {
        skew <- mean((x - xm)^3)/(mean((x - xm)^2))^(3/2)
    }
    skew
}



##' Extracts non-numeric variables, calculates summary information,
##' including entropy as a diversity indicator.
##'
##' This function finds the non- numeric variables and ignores the
##' others. (See \code{summarizeNumerics} for a function that
##' handles numeric variables.)  It then treats all non-numeric
##' variables as if they were factors, and summarizes each. The main
##' benefits from this compared to R's default summary are 1) more
##' summary information is returned for each variable (entropy
##' estimates ofdispersion), 2) the columns in the output are
##' alphabetized. To prevent alphabetization, use alphaSort = FALSE.
##'
##' Entropy is one possible measure of diversity. If all outcomes are
##' equally likely, the entropy is maximized, while if all outcomes
##' fall into one possible category, entropy is at its lowest
##' values. The lowest possible value for entropy is 0, while the
##' maximum value is dependent on the number of categories. Entropy is
##' also called Shannon's information index in some fields of study
##' (Balch, 2000 ; Shannon, 1949 ).
##'
##' Concerning the use of entropy as a diversity index, the user might
##' consult Balch(). For each possible outcome category, let p
##' represent the observed proportion of cases. The diversity
##' contribution of each category is -p * log2(p). Note that if p is
##' either 0 or 1, the diversity contribution is 0.  The sum of those
##' diversity contributions across possible outcomes is the entropy
##' estimate. The entropy value is a lower bound of 0, but there is no
##' upper bound that is independent of the number of possible
##' categories. If m is the number of categories, the maximum possible
##' value of entropy is -log2(1/m).
##'
##' Because the maximum value of entropy depends on the number of
##' possible categories, some scholars wish to re-scale so as to bring
##' the values into a common numeric scale. The normed entropy is
##' calculated as the observed entropy divided by the maximum possible
##' entropy.  Normed entropy takes on values between 0 and 1, so in a
##' sense, its values are more easily comparable. However, the
##' comparison is something of an illusion, since variables with the
##' same number of categories will always be comparable by their
##' entropy, whether it is normed or not.
##'
##' Warning: Variables of class POSIXt will be ignored. This will be
##' fixed in the future. The function works perfectly well with
##' numeric, factor, or character variables.  Other more elaborate
##' structures are likely to be trouble.
##' @param dat A data frame
##' @param maxLevels The maximum number of levels that will be
##'     reported.
##' @param alphaSort If TRUE (default), the columns are re-organized
##'     in alphabetical order. If FALSE, they are presented in the
##'     original order.
##' @param stats Default is \code{c("nobs", "nmiss", "entropy",
##'     "normedEntropy")}.
##' @param digits Default 2.
##' @export
##' @return A list of factor summaries
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @seealso \code{\link{summarizeNumerics}}
##'
##' @references
##'
##' Balch, T. (2000). Hierarchic Social Entropy: An Information
##' Theoretic Measure of Robot Group Diversity. Auton. Robots, 8(3),
##' 209-238.
##'
##' Shannon, Claude. E. (1949). The Mathematical Theory of
##' Communication. Urbana: University of Illinois Press.
##' @examples
##' set.seed(21234)
##' x <- runif(1000)
##' xn <- ifelse(x < 0.2, 0, ifelse(x < 0.6, 1, 2))
##' xf <- factor(xn, levels=c(0,1,2), labels("A","B","C"))
##' dat <- data.frame(xf, xn, x)
##' summarizeFactors(dat)
##' ##see help for summarize for more examples
summarizeFactors <-
    function (dat = NULL, maxLevels = 5, alphaSort = TRUE,
              stats = c("entropy", "normedEntropy", "nobs", "nmiss"),
              digits = 2)
{
    if (is.atomic(dat)){
        datname <- deparse(substitute(dat))
        dat <- data.frame(dat)
        if (NCOL(dat) == 1) {
            colnames(dat) <- datname
        } else {
            colnames(dat) <- paste0(datname, "_",  seq(1, NCOL(dat)))
        }
    } else if (!is.data.frame(dat)) dat <- as.data.frame(dat)
    factors <- sapply(dat, function(x) {!is.numeric(x) & !inherits(x, "POSIXt")})
    if (sum(factors) == 0) return(NULL)
    datf <- dat[, factors, drop = FALSE]
    if (alphaSort)
        datf <- datf[, sort(colnames(datf)), drop = FALSE]
    z <- lapply(datf, summary.factor)
    attr(z, "class") <- c("summarizedFactors")
    attr(z, "maxLevels") <- maxLevels
    attr(z, "stats") <- stats
    attr(z, "digits") <- digits
    z
}
NULL
    


##' Prints out the contents of an object created by summarizeFactors
##' in the style of base::summary
##'
##' An object with class "summarizedFactors" is the input. Such an
##' object should be created with the function
##' rockchalk::summarizeFactors. Each element in that list is then
##' organized for printing in a tabular summary.  This should look
##' almost like R's own summary function, except for the additional
##' information that these factor summaries include.
##'
##' @export
##' @param x A summarizedFactors object produced by summarizeFactors
##' @param ... optional arguments. Only value currently used is digits,
##'   which defaults to 2.
##' @return A table of formatted output
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @seealso \code{\link{summarize}}, \code{\link{summarizeFactors}}, \code{\link{formatSummarizedNumerics}}
##' @examples
##' dat <- data.frame(xcat1 = gl(10, 3), xcat2 = gl(5, 6))
##' summarizeFactors(dat, maxLevels = 8)
##' formatSummarizedFactors(summarizeFactors(dat))
formatSummarizedFactors <- function(x, ...){
    ncw <- function(x) {
        z <- nchar(x, type = "w")
        if (any(na <- is.na(z))) {
            z[na] <- nchar(encodeString(z[na]), "b")
        }
        z
    }
    dots <- list(...)
    if (!is.null(dots$digits)) {
        digits = dots$digits
        dots$digits <- NULL
    } else if (!is.null(attr(x, "digits"))){
        digits <- attr(x, "digits")
    } else {
        digits <- 2
    }
    
    if (!is.null(dots$maxLevels)) {
        maxLevels = dots$maxLevels
        dots$maxLevels <- NULL
    } else if (!is.null(attr(x, "maxLevels"))){
        maxLevels <- attr(x, "maxLevels")
    } else {
        maxLevels <- 5
    }

    
    if (!is.null(dots$stats)) {
        stats <- dots$stats
        dots$stats <- NULL
    } else if (!is.null(attr(x, "stats"))){
        stats <- attr(x, "stats")
    } else {
        stats <- c("entropy", "normedEntropy", "nobs", "nmiss")
    }
    ## Omit stats requests that do not apply to factors
    stats.for.factors <- c("entropy", "normedEntropy", "nobs", "nmiss")
    stats <- stats[stats %in% stats.for.factors]
    
    nv <- length(x)
    ## cycle through variables and reduce them to maxLevels if needed
    ## prune out unwanted elements in stats as well
    for (i in 1L:nv){
        tt <- x[[i]][["table"]]
        if (length(tt) > maxLevels) {
            ## Only sort.list if to many levels
            o <- sort.list(tt, decreasing = TRUE)
            toExclude <- maxLevels:length(tt)
            tt <- c(tt[o[-toExclude]], `(All Others)` = sum(tt[o[toExclude]]))
            x[[i]][["table"]] <- tt
        }
        if(length(stats) == 0) {
            x[[i]][["stats"]] <- NULL
        } else {
            ## keep desired stats
            stats.est <- x[[i]][["stats"]]
            if (!is.null(stats.est)){
                stats.est <- stats.est[names(stats.est) %in% stats]
                x[[i]][["stats"]] <- stats.est
            }
        }
    }
    
    nm <- names(x)
    lw <- numeric(nv)
    ## nr maximum number of rows needed for any table
    nr <- max(unlist(lapply(x, function(y) NROW(y[["table"]]))))
    nsumlines <- max(unlist(lapply(x, function(y){
        if(is.null(y[["stats"]])) 0 else length(y[["stats"]])})))
    ##nr <- nr + nsumlines
    reslt <- list()
    for (i in 1L:nv) {
        tt <- x[[i]][["table"]]
        stats.est <- x[[i]][["stats"]]
        lbs1 <- format(names(tt))
        vals <- as.integer(tt)
        ttnew <- paste(lbs1, ": ", vals, sep = "")
        length(ttnew) <- nr
        if(is.null(stats.est)){
            lw[i] <- ncw(lbs1[1L])
            reslt[[i]] <- ttnew
            next()
        } else {
            wide1 <- ncw(lbs1[1L])
            lbs2 <- format(names(stats.est))
            vals <- format(stats.est, digits = digits)
            stats.est.new <- paste(lbs2, ": ", vals, sep = "")
            ttnew2 <- c(ttnew, stats.est.new)
            wide2 <- ncw(lbs2[1L])
            reslt[[i]] <- ttnew2
            lw[i] <- max(wide1, wide2) + 5
        }
    }
    reslt <- unlist(reslt, use.names = TRUE)
    dim(reslt) <- c(nr + nsumlines, nv)
    if (any(is.na(lw)))
        warning("probably wrong encoding in names(.) of column ",
                paste(which(is.na(lw)), collapse = ", "))
    blanks <- paste(character(max(lw, na.rm = TRUE) + 2L), collapse = " ")
    pad <- floor(lw - ncw(nm)/2)
    nm <- paste(substring(blanks, 1, pad), nm, sep = "")
    dimnames(reslt) <- list(rep.int("", nr + nsumlines), nm)
    attr(reslt, "class") <- c("table")
    reslt
}
NULL


##' Sorts numeric from discrete variables and returns separate
##' summaries for those types of variables.
##'
##' The work is done by the functions \code{summarizeNumerics} and
##' \code{summarizeFactors}.  Please see the help pages for those
##' functions for complete details.
##'
##' The major purpose here is to generate summary data structure
##' that is more useful in subsequent data analysis.  The numeric
##' portion of the summaries are a data frame that can be used in
##' plots or other diagnostics.
##' 
##' The term "factors" was used, but "discrete variables" would have
##' been more accurate.  The factor summaries will collect all
##' logical, factor, ordered, and character variables.
##'
##' Other variable types, such as Dates, will be ignored, with a
##' warning.
##' 
##' @param dat A data frame
##' @param alphaSort If TRUE, the columns are re-organized in
##'     alphabetical order. If FALSE, they are presented in the
##'     original order.
##' @param stats A vector of desired summary statistics. Set
##'     \code{stats = NULL} to omit all stat summaries. Legal elements
##'     are \code{c("min", "med", "max", "mean", "sd", "var",
##'     "skewness", "kurtosis", "entropy", "normedEntropy", "nobs",
##'     "nmiss")}. The statistics \code{c("entropy", "normedEntropy")}
##'     are available only for factor variables, while mean, variance,
##'     and so forth will be calculated only for numeric variables.
##'     \code{"nobs"} is the number of observations with non-missing,
##'     finite scores (not NA, NaN, -Inf, or Inf). \code{"nmiss"} is
##'     the number of cases with values of NA. The default setting for
##'     \code{probs} will cause \code{c("min", "med", "max")} to be
##'     included, they need not be requested explicitly. To disable
##'     them, revise \code{probs}.
##' @param probs For numeric variables, is used with the
##'     \code{quantile} function. The default is
##'     \code{probs = c(0, .50, 1.0)}, which are labeled in output as
##'     \code{c("min", "med", and "max")}. Set \code{probs = NULL}
##'     to prevent these in the output.
##' @param digits Decimal values to display, defaults as 2.
##' @param ... Optional arguments that are passed to
##'     \code{summarizeNumerics} and \code{summarizeFactors}. For
##'     numeric variables, one can specify \code{na.rm} and
##'     \code{unbiased}.  For discrete variables, the key argument is
##'     \code{maxLevels}, which determines the number of levels that
##'     will be reported in tables for discrete variables.
##' @return Return is a list with two objects 1) output from
##'     summarizeNumerics: a data frame with variable names on rows
##'     and summary stats on columns, 2) output from summarizeFactors:
##'     a list with summary information about each discrete
##'     variable. The display on-screen is governed by a method
##'     \code{print.summarize}.
##' @export
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @example inst/examples/summarize-ex.R
summarize <-
    function(dat, alphaSort = FALSE,
             stats = c("mean", "sd", "skewness", "kurtosis", "entropy",
                       "normedEntropy", "nobs", "nmiss"), probs = c(0, 0.50, 1.0),
             digits = 3, ...)
{
    dots <- list(...)
    if (is.atomic(dat)){
	   datname <- deparse(substitute(dat))
	   dat <- data.frame(dat)
           if (NCOL(dat) == 1) {
               colnames(dat) <- datname
           } else {
               colnames(dat) <- paste0(datname, "_",  seq(1, NCOL(dat)))
           }
    } else if (!is.data.frame(dat)) dat <- as.data.frame(dat)

    dotnames <- names(dots)
    ## next should give "alphaSort" "stats" "na.rm" "unbiased"
    nnames <- names(formals(summarizeNumerics))[-1L]
    ## args for numeric vars: names that need keeping if in dots
    ## initial argument list
    numargs <- list(dat = quote(dat), alphaSort = alphaSort, probs = probs,
                     stats = stats, digits = digits)
    argList <- modifyList(numargs, dots[names(dots) %in% nnames])
    datn <- do.call("summarizeNumerics", as.list(argList))

    fnames <- names(formals(summarizeFactors))[-1L]
    
    fargs <- list(dat = quote(dat), alphaSort = alphaSort, 
                     stats = stats, digits = digits)
    ## only possible addition is maxLevvels:
    fnames <- "maxLevels"
    argList <- modifyList(fargs, dots[names(dots) %in% fnames])
    datf <- do.call("summarizeFactors", as.list(argList))
    
    value <- list(numerics = datn, factors = datf)
    attr(value, "class") <- c("summarize", class(value))
    if (!is.null(datn)) {
        attr(value, "numeric.formatted") <- formatSummarizedNumerics(datn, digits = digits)
    }
    if (!is.null(datf)) {
        attr(value, "factor.formatted") <- formatSummarizedFactors(datf, digits = digits)
    }
    value
}
NULL


##' print method for output from summarize
##'
##' Be aware that the unrounded numeric matrix
##' is available as an attribute of the returned object.
##' This method displays a rounded, character-formatted
##' display of the numeric varibles.
##' @param x Object produced by summarize
##' @param digits Decimal values to display, defaults as 2.
##' @param ... optional arguments for print function.
##' @return x, unchanged
##' Prints objects created by summarize
##' @method print summarize
##' @export
print.summarize <- function(x, digits, ...){
    digits <- if(!missing(digits) && !is.null(digits)) digits else NULL
    if(!is.null(x$numerics)){
        cat("Numeric variables\n")
        num.frmt <- formatSummarizedNumerics(x$numerics, digits = digits, ...)
        print(num.frmt, quote = FALSE, print.gap = 3)
    } else {
        num.frmt <- NULL
    }
    if(!is.null(x$factors)){
        cat("\nNonnumeric variables\n")
        fac.frmt <- formatSummarizedFactors(x$factors, digits = digits, ...)
        print(fac.frmt)
    } else {
        fac.frmt <- NULL
    }
    invisible(list(numerics = num.frmt, factors = fac.frmt))
}
    
##' Reformat numeric summarize output as one column per variable,
##' similar to R summary
##'
##' The summarizeNumeric function returns a data frame with the
##' variable names on the rows and summary statistics (mean, median,
##' std. deviation) in the columns.This transposes and abbreviates
##' the information to look more like R summary.
##' 
##' @param x numeric summaries from summarize function
##' @param ... Other arguments, such as digits
##' @return An R \code{table} object
##' @author Paul Johnson
##' @export
##' @examples
##' set.seed(21234)
##' X <- matrix(rnorm(10000), ncol = 10, dimnames = list(NULL, paste0("xvar", 1:10)))
##' Xsum <- summarize(X)
##' Xsum$numerics
##' formatSummarizedNumerics(Xsum$numerics)
##' formatSummarizedNumerics(Xsum$numerics, digits = 5)
##' Xsum.fmt <- formatSummarizedNumerics(Xsum$numerics)
##' str(Xsum.fmt)
formatSummarizedNumerics <- function(x, ...){
    dots <- list(...)
    
    if (!is.null(dots$digits)) {
        digits = dots$digits
        dots$digits <- NULL
    } else if (!is.null(attr(x, "digits"))){
        digits <- attr(x, "digits")
    } else {
        digits <- 2
    }
        
    datnfmt <- x
    for (i in colnames(datnfmt)) datnfmt[ , i] <- round(datnfmt[ , i], digits)
    datnfmt <- format(t(datnfmt), trim = FALSE, scientific = 10)
    datnfmt <- gsub("(\\.0+)$", " ", datnfmt)
    class(datnfmt) <- c("table")
    lcolnames <-  nchar(dimnames(datnfmt)[[2]])
    colmax <- apply(datnfmt, 2, function(i) max(nchar(i)))
    padcount <- ifelse(colmax > lcolnames + 1, round(0.5*(colmax - lcolnames)), 0)
    pads <- vapply(padcount, function(i) {
            paste0(rep(" ", max(0, i, na.rm = TRUE)), collapse = "")
    }, character(1))
    dimnames(datnfmt)[[2]] <- paste0(pads, dimnames(datnfmt)[[2]])
    datnfmt
}

    
##' Tabulates observed values and calculates entropy
##'
##' This adapts code from R base summary.factor. It adds
##' the calculation of entropy as a measure of diversity.
##'
##' @param y a factor (non-numeric variable)
##' @return A list, including the summary table and vector of summary
##'     stats if requested.
##' @author Paul E. Johnson <pauljohn@@ku.edu>
summary.factor <-
    function(y)
{
    ## 5 nested functions to be used later
    divr <- function(p = 0) {
        ifelse(p > 0 & p < 1, -p * log2(p), 0)
    }
    entropy <- function(p) {
        sum(divr(p))
    }
    maximumEntropy <- function(N) -log2(1/N)
    normedEntropy <- function(x) {
		xent <- entropy(x)
	    if(xent == 0) return(0)
		xent/maximumEntropy(length(x))
	}
    nmiss <- sum(is.na(y), na.rm = TRUE)
    y <- factor(y)
    ll <- levels(y)
    tbl <- table(y)
    tt <- c(tbl)
    names(tt) <- dimnames(tbl)[[1L]]
    props <- prop.table(tbl)
    ## Calculate all stats, even if don't ask for them
    stat.est <- c(nobs = sum(tbl),
                  nmiss = nmiss,
                  entropy = entropy(props),
                  normedEntropy = normedEntropy(props))
    ##tt <- list(table = tt, stats = stat.est[names(stat.est) %in% stats])
    tt <- list(table = tt, stats = stat.est)
    tt
}
NULL

##' Central Tendency estimates for variables
##'
##' This is needed for the creation of summaries and predicted values
##' of regression models. It takes a data frame and returns a new data
##' frame with one row in which the mean or mode of the columns is
##' reported.
##'
##' @param x a data frame
##' @return a data frame with the same variables and one row, the summary indicators.
##' @author Paul E. Johnson \email{pauljohn@@ku.edu}
##' @export
##' @examples
##' myDat <- data.frame(x=rnorm(100), y=rpois(100,l=4), z = cut(rnorm(100), c(-10,-1,0,10)))
##' centralValues(myDat)

centralValues <- function(x){
    if( !is.data.frame(x)) stop("represent: x must be a data frame!")
    nc <- NCOL(x)
    nams <- colnames(x)
    represents <- x[1, , drop = FALSE] ## row 1, so we inherit the df's properties, labels
    for (i in 1: nc) {
        xvar <- x[ , i]
        if (is.numeric(xvar)){
            represents[1, i] <- mean(xvar, na.rm = TRUE)
        } else if (is.factor(xvar)) {
            xvartable <- table(xvar)
            xvarmode <- names(xvartable)[which.max(xvartable)]
            represents[1, i] <- xvarmode          
        } else {
            xvar <- factor(xvar)
            xvartable <- table(xvar)
            xvarmode <- names(xvartable)[which.max(xvartable)]
            represents[1, i] <- xvarmode      
        }
    }
    as.data.frame(represents)
    rownames(represents) <- NULL
    represents
}

##' Create correlation matrices.
##'
##' Use can supply either a single value (the common correlation among
##' all variables), a column of the lower triangular values for a
##' correlation matrix, or a candidate matrix. The function will check
##' X and do the right thing. If X is a matrix, check that it
##' is a valid correlation matrix. If its a single value, use that
##' to fill up a matrix. If itis a vector, try to use it as a vech
##' to fill the lower triangle..
##'
##' @param X Required. May be one value, a vech, or a matrix
##' @param d Optional. The number of rows in the correlation matrix to
##' be created. lazyCor will deduce the desired size from X if
##' possible. If X is a single value, d is a required argument.
##' @return A correlation matrix.
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' lazyCor(0.5, 8)
##' lazyCor(c(0.1, 0.2, 0.3))
##' lazyCor(c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6))
lazyCor <-
    function(X, d)
{
    if (is.matrix(X)){
        stopifnot (isSymmetric(X))
        if (!dim(X)[1] == d) stop("lazyCor: the dimension of the matrix supplied is inconsistent with the dimension argument d")
    } else if (length(X) == 1) {
        if ( X < -1 | X > 1 ) stop(paste("The value of a correlation should be in [-1,1]"))
        X <- matrix(X, nrow = d, ncol = d)
        diag(X) <- 1.0
    } else if (is.vector(X)){
        X <- vech2Corr(X)
    } else {
        stop(paste("lazyCor cannot understand the value supplied for argument", deparse(substitute(X)),".\n That should be either a", d, " x ", d, "symmetric matrix, \n or a vech of the strictly lower triangular part of a matrix, or \n one single value, which we will use to fill up a matrix."))
    }
    if (!checkPosDef(X)) stop("'correlation matrix' is not positive definite")
    X
}
NULL

##' Estimate standardized regression coefficients for all variables
##'
##' This is brain-dead standardization of all variables in the design
##' matrix.  It mimics the silly output of SPSS, which standardizes
##' all regressors, even if they represent categorical variables.
##'
##' @param model a fitted lm object
##' @return an lm fitted with the standardized variables
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @seealso \code{\link[rockchalk]{meanCenter}} which will center or
##' re-scale only numberic variables
standardize <-
    function(model)
{
    UseMethod("standardize")
}

##' @return a standardized regression object
##' @rdname standardize
##' @export
##' @method standardize lm
##' @example inst/examples/standardize-ex.R
standardize.lm <-
    function(model)
{
    formulaReplace <- function(fmla, xname, newname){
        do.call("substitute", list(fmla, setNames(list(as.name(newname)), xname)))
    }

    mt <- terms(model)
    mdata <- model.frame(model)
    ys  <- drop(scale(mdata[, 1]))
    ##dm = design matrix, columns of predictors as numerically coded
    dm <- model.matrix(model)[ , -1, drop=FALSE] #no intercept
    dmnames <- paste(colnames(dm),"s", sep = "")
    dmnamesticked <- paste("`",dmnames,"`", sep = "")
    dmnamesticked <- gsub("``","`", dmnamesticked)
    dvname <- paste(colnames(mdata)[1],"s", sep = "")
    dvnameticked <-  paste("`", dvname,"`", sep = "")
    dvnameticked <- gsub("``","`", dvnameticked)
    std <- function(x) if(is.numeric(x)) scale(x) else x
    stddat <- apply(dm, 2, std)  ##standardize numeric vars
    colnames(stddat) <- paste(colnames(stddat), "s", sep = "")
    stddat <- cbind(ys, stddat)
    stddat <- as.data.frame(stddat)
    colnames(stddat) <- c(dvname, dmnames)
    colnames(stddat) <- gsub("`","", colnames(stddat))

    mc <- model$call
    mc$data <- quote(stddat)
    fmla <- paste(dvnameticked, " ~ ", "-1 + ", paste(dmnamesticked, collapse= " + "))
    mc$formula <- formula(fmla)
    res <- eval(mc)
    class(res) <- c("stdreg", class(model))
    res
}


##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @method summary stdreg
##' @export
summary.stdreg <-
    function(object, ...)
{
    dm <- model.matrix(object)
    dm <- dm[ , which(attr(dm, "assign") != 0)] #remove intercept, if any
    dm <- cbind( model.frame(object)[ , deparse(terms(object)[[2]])], dm)
    colnames(dm)[1] <- deparse(terms(object)[[2]])
    dmmeans <- apply(dm, 2, mean)
    dmstds <- apply(dm, 2, sd)
    summstat <- zapsmall(data.frame("mean" = dmmeans, "std.dev." = dmstds))
    summ <- NextMethod(generic = "summary", object = object, ...)
    summ$summstat <- summstat
    class(summ) <- paste("summary.", class(object), sep = "")
    summ
}
NULL


##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @method print stdreg
##' @export 
print.stdreg <- function(x, ...){
    cat("The standardized variables are suffixed with the letter \"s\" \n")
    NextMethod(generic = "print", object = x, ...)
}
NULL

##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @method print summary.stdreg
##' @export 
print.summary.stdreg <-
    function (x, ...)
{
    cat("All variables in the model matrix and the dependent variable
were centered. The centered variables have the letter \"s\" appended to their
non-centered counterparts, even constructed
variables like `x1:x2` and poly(x1,2). We agree, that's probably
ill-advised, but you asked for it by running standardize().\n
The rockchalk function meanCenter is a smarter option, probably. \n
The summary statistics of the variables in the design matrix. \n")
    print(x$summstat)
    NextMethod()
}
NULL





##' meanCenter selectively centers or standarizes variables in a regression model.
##'
##' Works with "lm" class objects, objects estimated by \code{glm()}. This
##' centers some or all of the the predictors and then re-fits the
##' original model with the new variables. This is a convenience to
##' researchers who are often urged to center their predictors.  This
##' is sometimes suggested as a way to ameliorate multi-collinearity
##' in models that include interaction terms (Aiken and West, 1991;
##' Cohen, et al 2002). Mean-centering may enhance interpretation of
##' the regression intercept, but it actually does not help with
##' multicollinearity.  (Echambadi and Hess, 2007). This function
##' facilitates comparison of mean-centered models with others by
##' calculating centered variables.  The defaults will cause a
##' regression's numeric interactive variables to be mean
##' centered. Variations on the arguments are discussed in details.
##'
##' Suppose the user's formula that fits the original model is
##' \code{m1 <- lm(y ~ x1*x2 + x3 + x4, data = dat)}. The fitted model
##' will include estimates for predictors \code{x1}, \code{x2},
##' \code{x1:x2}, \code{x3} and \code{x4}. By default,
##' \code{meanCenter(m1)} scans the output to see if there are
##' interaction terms of the form \code{x1:x2}. If so, then x1 and x2
##' are replaced by centered versions (m1-mean(m1)) and
##' (m2-mean(m2)). The model is re-estimated with those new variables.
##' model (the main effect and the interaction). The resulting thing
##' is "just another regression model", which can be analyzed or
##' plotted like any R regression object.
##'
##' The user can claim control over which variables are centered in
##' several ways. Most directly, by specifying a vector of variable
##' names, the user can claim direct control. For example, the
##' argument \code{terms=c("x1","x2","x3")} would cause 3 predictors
##' to be centered. If one wants all predictors to be centered, the
##' argument \code{centerOnlyInteractors} should be set to
##' FALSE. Please note, this WILL NOT center factor variables. But it
##' will find all numeric predictors and center them.
##'
##' The dependent variable will not be centered, unless the user
##' explicitly requests it by setting centerDV = TRUE.
##'
##' As an additional convenience to the user, the argument
##' \code{standardize = TRUE} can be used.  This will divide each
##' centered variable by its observed standard deviation. For people
##' who like standardized regression, I suggest this is a better
##' approach than the \code{standardize} function (which is brain-dead
##' in the style of SPSS). meanCenter with \code{standardize = TRUE}
##' will only try to standardize the numeric predictors.
##'
##' To be completely clear, I believe mean-centering is not helpful
##' with the multicollinearity problem. It doesn't help, it doesn't
##' hurt.  Only a misunderstanding leads its proponents to claim
##' otherwise. This is emphasized in the vignette "rockchalk" that is
##' distributed with this package.
##'
##' @title meanCenter
##' @param model a fitted regression model (presumably from lm)
##' @param centerOnlyInteractors Default TRUE. If FALSE, all numeric
##'     predictors in the regression data frame are centered before
##'     the regression is conducted.
##' @param centerDV Default FALSE. Should the dependent variable be
##'     centered? Do not set this option to TRUE unless the dependent
##'     variable is a numeric variable. Otherwise, it is an error.
##' @param standardize Default FALSE. Instead of simply mean-centering
##'     the variables, should they also be "standardized" by first
##'     mean-centering and then dividing by the estimated standard
##'     deviation.
##' @param terms Optional. A vector of variable names to be
##'     centered. Supplying this argument will stop meanCenter from
##'     searching for interaction terms that might need to be
##'     centered.
##' @export meanCenter
##' @rdname meanCenter
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @seealso
##'     \code{\link[rockchalk]{standardize}}
##'     \code{\link[rockchalk]{residualCenter}}
##' @references Aiken, L. S. and West, S.G. (1991). Multiple
##'     Regression: Testing and Interpreting Interactions. Newbury
##'     Park, Calif: Sage Publications.
##'
##' Cohen, J., Cohen, P., West, S. G., and Aiken, L. S. (2002). Applied
##' Multiple Regression/Correlation Analysis for the Behavioral Sciences
##' (Third.). Routledge Academic.
##'
##' Echambadi, R., and Hess, J. D. (2007). Mean-Centering Does Not Alleviate
##' Collinearity Problems in Moderated Multiple Regression Models.
##' Marketing Science, 26(3), 438-445.
##' @example inst/examples/meanCenter-ex.R
meanCenter <-
    function(model, centerOnlyInteractors = TRUE, centerDV = FALSE,
             standardize=FALSE, terms = NULL)
{
    UseMethod("meanCenter")
}

##' @return A regression model of the same type as the input model,
##' with attributes representing the names of the centered variables.
##' @rdname meanCenter
##' @export
##' @method meanCenter default
##' @export 
meanCenter.default <-
    function(model, centerOnlyInteractors = TRUE, centerDV = FALSE,
             standardize = FALSE, terms = NULL)
{
    std <- function(x) {
        if(!is.numeric(x)) stop("can't center a factor variable. No Can Do!")
        xmean <- mean(x, na.rm = TRUE)
        if (standardize) {
            xsd <- sd(x, na.rm = TRUE)
        } else {
            xsd <- 1
        }
        x <- (x-xmean)/xsd
        list(x = x, xmean = xmean, xsd = xsd)
    }

    ## rdf <- get_all_vars(formula(model), model$model) #raw data frame
    rdf <- model.data(model)
    t <- terms(model)
    ## TODO 20140417: look at using na.action attribute of model to be more delicate here.
    tl <- attr(t, "term.labels")
    tmdc <- attr(t, "dataClasses") ##term model data classes

    isNumeric <- names(tmdc)[ which(tmdc %in% c("numeric"))]
    isFac <-  names(tmdc)[ which(tmdc %in% c("factor"))]

    if (centerDV & tmdc[1] != "numeric")
        stop("Sorry, the DV is not a numeric column, it does not make sense to center it.")

    ##Build "nc", a vector of variable names that "need centering"
    ##
    if (!centerDV) {
        if (!is.null(terms)){
            nc <- as.vector(terms)
            nc <- unique(nc)
        } else if (centerOnlyInteractors == FALSE){
            nc <- isNumeric[-1] #-1 excludes response
            nc <- unique(nc)
        } else {
            interactTerms <- tl[grep(":", tl)]
            nc <- unique(unlist(strsplit( interactTerms, ":")))
            nc <-  nc[which(nc %in% isNumeric)]
        }
    } else {
        if (!is.null(terms)){
            nc <- as.vector(terms)
            nc <- c(names(tmdc)[1] , nc)
        } else if (centerOnlyInteractors == FALSE){
            nc <- isNumeric
        } else {
            interactTerms <- tl[grep(":", tl)]
            nc <- unique(unlist(strsplit( interactTerms, ":")))
            nc <- nc[which(nc %in% isNumeric)]
            nc <- c(names(tmdc)[1] , nc)
        }
    }

    mc <- model$call
    ## run same model call, replacing non centered data with centered data.
    ##
    stddat <- rdf
    centeredVars <- matrix(NA, nrow=2, ncol=length(nc))
    colnames(centeredVars) <- nc
    rownames(centeredVars) <- c("mean","scale")

    formulaReplace <- function(fmla, xname, newname){
        do.call("substitute", list(fmla, setNames(list(as.name(newname)), xname)))
    }

    newFmla <- mc$formula
    for (i in seq_along(nc)){
        icenter <- std(stddat[, nc[i]])
        centeredVars[1, nc[i]] <- icenter$xmean
        centeredVars[2, nc[i]] <- icenter$xsd
        newname <- paste(as.character(nc[i]), "c", sep = "")
        if (isTRUE(standardize)) newname <- paste(newname, "s", sep = "")
        stddat[ ,newname] <- icenter$x
        newFmla <- formulaReplace(newFmla,  as.character(nc[i]), newname)
        nc[i] <- newname
    }
    colnames(centeredVars) <- nc
    mc$formula <- newFmla
    mc$data <- quote(stddat)
    res <- eval(mc)
    class(res) <- c("mcreg", class(model))
    attr(res, "centeredVars") <- centeredVars
    attr(res, "centerCall") <-  match.call()
    res
}

##' @author <pauljohn@@ku.edu>
##' @export 
##' @method summary mcreg
summary.mcreg <-
    function(object, ...)
{
    centeredVars <- attr(object, "centeredVars")
    dm <- model.matrix(object)
    dm <- dm[ , which(attr(dm, "assign") != 0), drop=FALSE] #remove intercept, if any
    dm <- cbind( model.frame(object)[ , deparse(terms(object)[[2]])], dm)
    colnames(dm)[1] <- deparse(terms(object)[[2]])
    dmmeans <- apply(dm, 2, mean)
    dmstds <- apply(dm, 2, sd)
    summstat <- zapsmall(data.frame("mean" = dmmeans, "std.dev." = dmstds))
    summ <- NextMethod(generic = "summary", object = object, ...)
    summ$summstat <- summstat
    summ$centeredVars <- centeredVars
    class(summ) <- paste("summary.", class(object), sep="")
    summ$mc <- attr(object, "centerCall")
    summ
}
NULL

##' @author <pauljohn@@ku.edu>
##' @method print mcreg
##' @export 
print.mcreg <- function(x, ...){
    centeredVars <- attr(x, "centeredVars")
    cat("The centered variables are: \n")
    print(centeredVars)
    mc <- attr(x, "centerCall")
    cat("The call that requested centering was: \n")
    print(mc)
    NextMethod(generic = "print", object = x, ...)
}
NULL


##' @author <pauljohn@@ku.edu>
##' @method print summary.mcreg
##' @export 
print.summary.mcreg <-
    function (x, ...)
{
    cat("These variables were mean-centered before any transformations were made on the design matrix.\n")
    print(colnames(x$centeredVars))
    cat("The centers and scale factors were \n")
    print(x$centeredVars)

    cat("The summary statistics of the variables in the design matrix (after centering). \n")
    print(x$summstat)
    cat("\nThe following results were produced from: \n")
    print(x$mc)
    ##NextMethod(generic = "print", x = x, ...)
    NextMethod()
}
NULL


##' @author <pauljohn@@ku.edu>
##' @method predict mcreg
##' @export
predict.mcreg <-
    function (object, ...)
{
    originalCall <- object$call
    dots <- list(...)
    newdata <- NULL
    if (! is.null(dots[["newdata"]])) {
        newdata <- dots[["newdata"]]
        dots[["newdata"]] <- NULL
    }
    centeredVars <- attr(object, "centeredVars")
    nc <- colnames(centeredVars) #need centering
    dvname <- parse(text = formula(originalCall)[[2]])
    nc <- setdiff(nc, dvname) #remove dv name if present
    if (is.null(newdata)) {
        newdata <- model.frame(object)##should be centered already
        tmeans <- sapply(newdata[ , nc, drop = FALSE], mean, na.rm = TRUE)
        if (! isTRUE(all.equal(abs(tmeans), rep(0, length(nc)), check.attributes = FALSE)))
            stop(paste("In predict.mcreg, the fitted regression claims to have centered variables,",
                       paste(nc, collapse=" "), "but not all of those centered values have",
                       "observed means very close to 0. Something's wrong"))
    }
    NextMethod(object, newdata = newdata, dots)
}
NULL


##' Find numeric columns, center them, re-name them, and join them with the original data.
##'
##' The meanCentered regression function requires centered-inputs when
##' calculations are predicted. For comparison with ordinary
##' regression, it is convenient to have both centered and the
##' original data side-by-side.  This function handles that.  If the
##' input data has columns c("x1","x2","x3"), then the centered result
##' will have columns c("x1","x2","x3","x1c","x2c","x3c"), where "c"
##' indicates "mean-centered". If standardize=TRUE, then the result
##' will have columns c("x1","x2","x3","x1cs","x2cs","x3cs"), where "cs"
##' indicate "centered and scaled".
##' @param data Required. data frame or matrix.
##' @param center Optional. If nc is NOT supplied, then all numeric columns
##' in data will be centered (possiblly scaled).  Can be specified in 2 formats. 1) Vector of column names that are to be centered, 2) Vector named elements giving values of means to be used in centering.  Values must be named, as in c("x1" = 17, "x2" = 44).
##' (possibly scaled).
##' @param standardize  Default FALSE. If TRUE, the variables are
##' first mean-centered, and then divided by their standard deviations
##' (scaled). User can supply a named vector of scale values by which
##' to divide each variable (otherwise sd is used). Vector must have same
##' names and length as center argument. Variables can be entered in any order (will be resorted inside function).
##' @return A data frame with 1) All original columns 2) additional
##' columns with centered/scaled data, variables renamed "c" or "cs"
##' to indicate the data is centered or centered and
##' scaled. Attributes "centers" and "scales" are created for "record
##' keeping" on centering and scaling values.
##' @author <pauljohn@@ku.edu>
##' @export
##' @examples
##' set.seed(12345)
##' dat <- data.frame(x1=rnorm(100, m = 50), x2 = rnorm(100, m = 50),
##'     x3 = rnorm(100, m = 50), y = rnorm(100),
##'     x4 = gl(2, 50, labels = c("Male","Female")))
##' datc1 <- centerNumerics(dat)
##' head(datc1)
##' summarize(datc1)
##' datc2 <- centerNumerics(dat, center=c("x1", "x2"))
##' head(datc2)
##' summarize(datc2)
##' attributes(datc2)
##' datc3 <- centerNumerics(dat, center = c("x1" = 30, "x2" = 40))
##' head(datc3)
##' summarize(datc3)
##' attributes(datc3)
##' datc4 <- centerNumerics(dat, center=c("x1", "x2"), standardize = TRUE)
##' head(datc3)
##' summarize(datc4)
##' attributes(datc4)
##' datc5 <- centerNumerics(dat, center=c("x1"=30, "x2"=40),
##' standardize = c("x2" = 5, "x1" = 7))
##' head(datc5)
##' summarize(datc5)
##' attributes(datc5)
centerNumerics <- function(data, center, standardize = FALSE){
    if (!is.data.frame(data))
        data <- as.data.frame(data)
    isN <- sapply(data, is.numeric)
    if (sum(isN) == 0)  return(data)

    if (missing(center)) {
        center <- TRUE
        nc <- colnames(data)[isN] ##all need centering
    } else if (is.character(center)){
        if(sum (!center %in% colnames(data)[isN]) != 0)
            stop(paste("centerNumerics failed. Argument center includes",
                       "column names that are not numeric variables in the"
                       , "data frame", deparse(substitute(data))))
        nc <- center
        center <- TRUE
    } else if (is.numeric(center)) {
        if(is.null(names(center)))
            stop(paste("centerNumerics failed.",
                       "The center vector must be a named vector so that",
                       "centerNumerics can decide which columns need centering"))
        if(sum (!names(center) %in% colnames(data)[isN]) != 0)
            stop(paste("centerNumerics failed. Argument center includes column names that are not numeric variables in the data frame", deparse(substitute(data))))
        nc <- names(center) ##names
    }

    if (is.numeric(standardize)) {
        if (!setequal(names(standardize), nc))
            stop("centerNumerics failed. Names of standardize argument must be identical to names of center argument")
        standardize <- standardize[nc] ##sorts
    }

    datas <- scale(data[ , nc], center = center, scale = standardize)
    if(!is.null(attr(datas, "scaled:center"))) centers <- attr(datas, "scaled:center")
    colnames(datas) <- paste(colnames(datas), "c", sep="")
    if (!is.null(attr(datas, "scaled:scale"))) {
        scales <- attr(datas, "scaled:scale")
        colnames(datas) <- paste(colnames(datas), "s", sep = "")
    }
    data <- as.data.frame(cbind(data, datas))
    attr(data, "centers") <- centers
    if (!is.null(attr(datas, "scaled:scale")))attr(data, "scales") <- scales

    data
}
NULL

### Paul Johnson ### Adapted from ideas in post in r-help by Dave Armstrong May 8, 2006


##' Creates a publication quality result table for
##' regression models. Works with models fitted with lm, glm, as well
##' as lme4.
##'
##' This provides "markup" that the user is will copy into a LaTeX
##' document. As of rockchalk 1.8.4, can also create HTML markup.
##' The rockchalk vignette demonstrates use of outreg in Sweave.
##' 
##' \code{outreg} returns a string vector. It is suggested that users
##' should save the outreg result and then use cat to save it. That is
##' myMod <- outreg(m1, ...)  cat(myMod, file = "myMod.html") or
##' cat(myMod, file = "myMod.tex".  In version 1.8.66, we write the
##' html file to a temporary location and display it in a web
##' browser. Many word processors will not accept a cut-and paste
##' transfer from the browser, they will, however, be able to open the
##' html file itself and automatically re-format it in the native
##' table format.
##'
##' In version 1.8.111, an argument \code{print.results} was introduced.
##' This is TRUE by default, so the marked-up table is printed into
##' the session, and it is returned as well.  If the function should
##' run silently (as suggested in the last few versions), include
##' \code{print.results = TRUE}. 
##'
##' The table includes a minimally sufficient (in my opinion) model
##' summary.  It offers parameter estimates, standard errors, and
##' minimally sufficient goodness of fit.  My tastes tend toward
##' minimal tables, but users request more features, and
##' \code{outreg}'s interface hass been generalized to allow
##' specialized requests. See \code{request} and \code{runFuns}
##' arguments.
##'
##' I don't want to write a separate table function for every
##' different kind of regression model that exists (how
##' exhausting). So I've tried to revise \code{outreg()} to work with
##' regression functions that follow the standard R framework. It is
##' known to work \code{lm} and \code{glm}, as well as \code{merMod}
##' class from \code{lme4}, but it will try to interact with other
##' kinds of regression models.  Those models should have methods
##' \code{summary()}, \code{coef()}, \code{vcov()} and \code{nobs()}.
##' Package writes should provide those, its not my job.
##'
##' Do you want "robust standard errors"? P values calculated
##' according to some alternative logic?  Go ahead, calculate them in
##' your code, outreg will now accept them as arguments. As of Version
##' 1.8.4, users can provide their own standard errors and/or p-values
##' for each model. Thus, if a model answers in the usual way to the
##' standard R request \code{coef(summary(model))}, outreg can work if
##' users supply standard errors.
##'
##' About the customizations \code{request}.  The \code{request}
##' argument supplies a list of names of summary output elements that
##' are desired. The format is a pair, a value to be retrieved from
##' \code{summary(model)}, and a pretty name to be printed for
##' it. With the \code{lm()} regression, for example, one might want
##' the output of the F test and the adjusted R-square: Include
##' \code{request = c(adj.r.squared = "adj. $R^2$", "fstatistic" =
##' "F")}. The value on the left is the name of the desired
##' information in the summary object, while the value on the right is
##' \emph{any} valid LaTeX (or HTML) markup that the user desires to
##' display in the table. \code{request} terms that generate a single
##' numerical value will generally work fine, while requests that ask
##' for more structured information, such as the F test (including the
##' 2 degrees of freedom values) may work (user feedback needed).
##'
##' The \code{runFuns} argument is inspired by a user request: could
##' this include the BIC or other summaries that can be easily
##' calculated?  Any R function, such as \code{AIC} or \code{BIC},
##' should work, as long as it returns a single value.  This is a
##' two-part specification, a function name and a pretty label to be
##' used in printing. For example, \code{runFuns = c("AIC" = "Akaike
##' Criterion", "BIC" = "Schwartz Criterion", "logLik" = "LL")}.
##'
##' About centering with dcolumn or siunitx. It appears now that
##' results are better with \code{siunitx} but \code{dcolumn} is more
##' familiar to users.  The user has the duty to make sure that the
##' document preamble includes the correct package,
##' \code{\\usepackage\{dcolumn\}} or \code{\\usepackage\{siunitx\}}.
##' In this version, I have eliminated the need for the user to
##' specify document-wide settings for \code{siunitx}. All of the
##' details are explicitly written in the header of each tabular.
##' It is done that way to more easily allow user customizations.
##'
##' @param modelList A regression model or an R list of regression
##'     models. Default model names will be M1, M2, and so forth. User
##'     specified names are allowed, such as \code{list("My Model" =
##'     m1, "Her Model" = m2)}.  This is the currently recommended way
##'     to supply model lables. This is less error prone than the use
##'     of the modelLabels argument.
##' @param type Default = "latex". The alternatives are "html" and
##'     "csv"
##' @param modelLabels This is allowed, but discouraged. A vector of
##'     character string variables, one for each element in
##'     modelList. Will override the names in modelList.
##' @param varLabels To beautify the parameter names printed.  Must be
##'     a named vector in the format c(parmname = "displayName",
##'     parmname = "displayName"). Include as many parameters as
##'     desired, it is not necessary to supply new labels for all of
##'     the parameters.
##' @param tight Table format. If TRUE, parameter estimates and
##'     standard errors are printed in a single column.  If FALSE,
##'     parameter estimates and standard errors are printed side by
##'     side.
##' @param centering Default is "none", but may be "siunitx" or
##'     "dcolumn". No centering has been the only way until this
##'     version. User feedback requested.  Don't forget to insert
##'     usepackage statment in document preamble for siunitx or
##'     dcolumn. If user specifies \code{centering=TRUE}, the
##'     \code{siunitx} method will be used. The \code{dcolumn}
##'     approach assumes that the values reported in the column use
##'     fewer than 3 integer places and 3 decimal places. Additional
##'     room is allocated for the significance stars.
##' @param showAIC This is a legacy argument, before the
##'     \code{request} argument was created.  If TRUE, the AIC
##'     estimate is included with the diagnostic values. It has the
##'     same effect as described by \code{request}.
##' @param float Default = FALSE. Include boilerplate for a LaTeX
##'     table float, with the tabular markup inside it. Not relevant
##'     if type = "html".
##' @param request Extra information to be retrieved from the
##'     summary(model) and displayed. This must be a vector of named
##'     arguments, such as c(adj.r.squared = "adj $R^2$", fstatistic =
##'     "F"). The name must be a valid name of the output object, the
##'     value should be the label the user wants printed in the
##'     table. See details.
##' @param runFuns A list of functions
##' @param digits Default = 3. How many digits after decimal sign are
##'     to be displayed.
##' @param alpha Default = c(0.05, 0.01, 0.001). I think stars are
##'     dumb, but enough people have asked me for more stars that I'm
##'     caving in.
##' @param SElist Optional. Replacement standard errors. Must be a
##'     list of named vectors. \code{outreg} uses the R \code{summary}
##'     to retrieve standard errors, but one might instead want to use
##'     robust or bootstrapped standard errors.  This argument may
##'     supply a new SE vector for each fitted regression model, but
##'     it is also allowed to supply the SE replacement for just one
##'     of the models. The format should be \code{list("A Model Label"
##'     = c(0.1, 0.3, 0.4), "Another Model Label" = c(0.4, 0.2, 0.3)}.
##'     On the left, one must use the same names that are used in the
##'     modelList argument.
##' @param PVlist Optional. A list of replacement "p values". It must
##'     be a list of named vectors, similar in format to SElist. The
##'     which the elements are the "p values" that the user wants to
##'     use for each model.
##' @param Blist Optional. This is only needed in the rare case where
##'     a model's parameters cannot be discerned from its
##'     summary. List must have names for models, and vectors slope
##'     coefficient. See discussion of SElist and PVlist.
##' @param title A LaTeX caption for the table. Not relevant if type =
##'     "html".
##' @param label A string to be used as a LaTeX label in the table to
##'     be created. Not relevant if type = "html".
##' @param gofNames Optional pretty names. R regression summaries use
##'     names like "sigma" or "r.squared" that we might want to revise
##'     for presentation. I prefer to refer to "sigma" as "RMSE", but
##'     perhaps you instead prefer something like \code{gofnames =
##'     c("sigma" = "That Estimate I don't understand", "deviance" =
##'     "Another Mystery")}. The words that you might replace are
##'     "sigma", "r.squared", "deviance", "adj.r.squared",
##'     "fstatistic".
##' @param print.results Default TRUE, marked-up table will be
##'     displayed in session.  If FALSE, same result is returned as an
##'     object.
##' @param browse Display the regression model in a browse? Defaults
##'     to TRUE if type = "html"
##' @export outreg
##' @importFrom lme4 VarCorr
##' @importFrom utils getFromNamespace
##' @import grDevices
##' @rdname outreg
##' @return A character vector, one element per row of the regression
##'     table.
##' @keywords regression
##' @note There are many R packages that can be used to create LaTeX
##'     regression tables. memisc, texreg, apsrtable, xtables, and rms
##'     are some. This "outreg" version was in use in our labs before
##'     we were aware that those packages were in development. It is
##'     not intended as a competitor, it is just a slightly different
##'     version of the same that is more suited to our needs.
##' @author Paul E. Johnson \email{<pauljohn@@ku.edu>}
##' @examples
##' set.seed(2134234)
##' dat <- data.frame(x1 = rnorm(100), x2 = rnorm(100))
##' dat$y1 <- 30 + 5 * rnorm(100) + 3 * dat$x1 + 4 * dat$x2
##' dat$y2 <- rnorm(100) + 5 * dat$x2
##' m1 <- lm(y1 ~ x1, data = dat)
##' m2 <- lm(y1 ~ x2, data = dat)
##' m3 <- lm(y1 ~ x1 + x2, data = dat)
##' gm1 <- glm(y1 ~ x1, family = Gamma, data = dat)
##' outreg(m1, title = "My One Tightly Printed Regression", float = TRUE)
##' ex1 <- outreg(m1, title = "My One Tightly Printed Regression",
##'                float = TRUE, print.results = FALSE, centering = "siunitx")
##' ## Show markup, Save to file with cat()
##' cat(ex1)
##' ## cat(ex1, file = "ex1.tex")
##'  
##' ex2 <- outreg(list("Fingers" = m1), tight = FALSE, 
##'     title = "My Only Spread Out Regressions", float = TRUE,
##'     alpha = c(0.05, 0.01, 0.001)) 
##' 
##' ex3 <- outreg(list("Model A" = m1, "Model B label with Spaces" = m2),
##'     varLabels = list(x1 = "Billie"), 
##'     title = "My Two Linear Regressions", request = c(fstatistic = "F"),
##'     print.results = TRUE)
##' cat(ex3)
##' 
##' ex4 <- outreg(list("Model A" = m1, "Model B" = m2),
##'     modelLabels = c("Overrides ModelA", "Overrides ModelB"),
##'     varLabels = list(x1 = "Billie"),
##'     title = "Note modelLabels Overrides model names")
##' cat(ex4)
##' ##'
##' ex5 <- outreg(list("Whichever" = m1, "Whatever" = m2),
##'     title = "Still have showAIC argument, as in previous versions",
##'     showAIC = TRUE, float = TRUE, centering = "siunitx")
##' 
##' ex5s <- outreg(list("Whichever" = m1, "Whatever" = m2),
##'     title = "Still have showAIC argument, as in previous versions",
##'     showAIC = TRUE, float = TRUE, centering = "siunitx")
##' 
##' \donttest{
##' ## Launches HTML browse
##' ex5html <- outreg(list("Whichever" = m1, "Whatever" = m2),
##'     title = "Still have showAIC argument, as in previous versions",
##'     showAIC = TRUE, type = "html")
##' ## Could instead, make a file:
##' ## fn <- "some_name_you_choose.html"
##' ## cat(ex5html, file = fn)
##' ## browseURL(fn)
##' ## Open that HTML file in LibreOffice or MS Word
##' }
##' 
##' ex6 <- outreg(list("Whatever" = m1, "Whatever" =m2),
##'     title = "Another way to get AIC output",
##'     runFuns = c("AIC" = "Akaike IC"))
##' cat(ex6)
##' 
##' ex7 <- outreg(list("Amod" = m1, "Bmod" = m2, "Gmod" = m3),
##'        title = "My Three Linear Regressions", float = FALSE)
##' cat(ex7)
##' 
##' ## A new feature in 1.85 is ability to provide vectors of beta estimates
##' ## standard errors, and p values if desired. 
##' ## Suppose you have robust standard errors!
##' if (require(car)){
##'    newSE <- sqrt(diag(car::hccm(m3)))
##'    ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, "Model C w Robust SE" = m3),
##'         SElist= list("Model C w Robust SE" = newSE))
##'    cat(ex8)
##' }
##' 
##' ex11 <- outreg(list("I Love Long Titles" = m1,
##'           "Prefer Brevity" = m2,
##'           "Short" = m3), tight = FALSE, float = FALSE)
##' cat(ex11)
##' ##'
##' ex12 <- outreg(list("GLM" = gm1), float = TRUE)
##' cat(ex12)
##' 
##' ex13 <- outreg(list("OLS" = m1, "GLM" = gm1), float = TRUE,
##'         alpha = c(0.05, 0.01))
##' cat(ex13)
##' ##'
##' ex14 <- outreg(list(OLS = m1, GLM = gm1), float = TRUE,
##'     request = c(fstatistic = "F"), runFuns = c("BIC" = "BIC"))
##' cat(ex14)

##' ex15 <- outreg(list(OLS = m1, GLM = gm1), float = TRUE,
##'     request = c(fstatistic = "F"), runFuns = c("BIC" = "BIC"),
##'     digits = 5, alpha = c(0.01))
##'
##' ex16 <- outreg(list("OLS 1" = m1, "OLS 2" = m2,  GLM = gm1), float = TRUE,
##'     request = c(fstatistic = "F"),
##'     runFuns = c("BIC" = "BIC", logLik = "ll"),
##'     digits = 5, alpha = c(0.05, 0.01, 0.001))
##'
##' ex17 <- outreg(list("Model A" = gm1, "Model B label with Spaces" = m2),
##'     request = c(fstatistic = "F"),
##'     runFuns = c("BIC" = "Schwarz IC", "AIC" = "Akaike IC",
##'     "nobs" = "N Again?"))
##'
##' ## Here's a fit example from lme4.
##' if (require(lme4) && require(car)){
##'   fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
##'   ex18 <- outreg(fm1)
##'   cat(ex18)
##'   ## Fit same with lm for comparison
##'   lm1 <- lm(Reaction ~ Days, sleepstudy)
##'   ## Get robust standard errors
##'   lm1rse <- sqrt(diag(car::hccm(lm1)))
##' 
##'   if(interactive()){
##'   ex19 <- outreg(list("Random Effects" = fm1, 
##'        "OLS" = lm1, "OLS Robust SE" = lm1),
##'        SElist = list("OLS Robust SE" = lm1rse), type = "html")
##'   }
##'   ## From the glmer examples
##'   gm2 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
##'                    data = cbpp, family = binomial)
##'   lm2 <- lm(incidence/size ~ period,  data = cbpp)
##'   lm2rse <- sqrt(diag(car::hccm(lm2)))
##'   ## Lets see what MASS::rlm objects do? Mostly OK
##'   rlm2 <- MASS::rlm(incidence/size ~ period, data = cbpp)
##'   \donttest{
##'   ex20 <- outreg(list("GLMER" = gm2, "lm" = lm2, "lm w/robust se" = lm2,
##'             "rlm" = rlm2), SElist = list("lm w/robust se" = lm2rse),
##'             type = "html")
##'   } 
##' }
outreg <-
    function(modelList, type = "latex", modelLabels = NULL,  varLabels = NULL,
             tight = TRUE, centering = c("none", "siunitx", "dcolumn"),
             showAIC = FALSE, float = FALSE, request,
             runFuns, digits = 3, alpha = c(0.05, 0.01, 0.001),  SElist = NULL,
             PVlist = NULL,  Blist = NULL, title, label,
             gofNames, print.results = TRUE, 
             browse = identical(type, "html") && interactive())
{

    if (is.character(centering)) {
        centering <- match.arg(tolower(centering), c("none", "siunitx", "dcolumn"))
    } else if (is.logical(centering)){
        centering <- if(isTRUE(centering)) "siunitx" else "none"
    }
    
    myGofNames <- c(sigma = "RMSE",
                    r.squared = paste("_R2_"),
                    adj.r.squared = paste("adj", "_R2_"),
                    fstatistic = "_Fmarkup_",
                    deviance = "Deviance")
 
    if (missing(gofNames)) {
        gofNames <- myGofNames
    } else {
        myGofNames[names(gofNames)] <- gofNames
        gofNames <- myGofNames
    }

    
    if (!missing(request)) gofNames <- c(gofNames, request)

    if (type == "html") float = FALSE
    
    ## Required methods
    req <- (c("coef", "nobs", "vcov", "summary"))

    checkFunctions <- function(obj, req){
        for (i in req){
            if (inherits(get(i)(obj), "try-error")) stop(paste("A model has no", i, "method"))
        }
    }
    
    checkReg <- function(modlist){
        ##Pre-approved model classes
        knownTypes <- c("lm", "glm", "merMod")
        approved <- sapply(modlist, inherits, knownTypes)
        ## Ask trouble modl for a list of all methods that apply to it
        problematic <-  modlist[!approved]
        errors <- lapply(problematic, checkFunctions, req)
    }

   latex.markup <- c(
        "_LB_" = "\\\n",                                                       
        "_EOC_" =  "",                                                                              
        "_BOC_" = "&",                                                                          
        "_EOMC_" = "}",                                                                       
        "_EOR_" = "\\\\tabularnewline",  
        "_BRU_" = "",
        "_BRT_" = "",                                                                       
        "_BOCU_" = "&",                             
        "_BR_" = "",   
        "_EOL_" = "\n",                                                               
        "_HL_" = "\\\\hline", 
        "_UL_" = "\\\\underline{",                                                   
        "_EOUL_" = "}",                                                                   
        "_SEPU_" = " &",                                                              
       "_SEP_" = " &",
       "_ETABULAR_" = "\\\\end{tabular}",  
        "_BOMR1_" = "& \\\\multirow{1}{c}{",
        "_BOMR2_" = "& \\\\multirow{2}{c}{",
        "_BOMC1_" = "\\\\multicolumn{1}{c}{",
        "_BOMC2_" = "\\\\multicolumn{2}{c}{",
        "_BOMC3_" = "\\\\multicolumn{3}{c}{",
        "_BOMC4_" = "\\\\multicolumn{4}{c}{",
        "_BOMC5_" = "\\\\multicolumn{5}{c}{",
        "_BOMC6_" = "\\\\multicolumn{6}{c}{",
        "_BOMC7_" = "\\\\multicolumn{7}{c}{",
        "_BOMC8_" = "\\\\multicolumn{8}{c}{",
        "_BOMC9_" = "\\\\multicolumn{9}{c}{",
        "_BOML1_" = "\\\\multicolumn{1}{l}{",
        "_BOML2_" = "\\\\multicolumn{2}{l}{",
        "_BOML3_" = "\\\\multicolumn{3}{l}{",
        "_BOML4_" = "\\\\multicolumn{4}{l}{",
        "_BOML5_" = "\\\\multicolumn{5}{l}{",
        "_BOML6_" = "\\\\multicolumn{6}{l}{",
        "_BOML7_" = "\\\\multicolumn{7}{l}{",
        "_BOML8_" = "\\\\multicolumn{8}{l}{",
        "_BOML9_" = "\\\\multicolumn{9}{l}{",
        "_BOMCT1_" = "\\\\multicolumn{1}{c}{",
        "_BOMCT2_" = "\\\\multicolumn{2}{c}{",
        "_BOMCT3_" = "\\\\multicolumn{3}{c}{",
        "_BOMCT4_" = "\\\\multicolumn{4}{c}{",
        "_HTMLHL_" = "",
        "_CHI2_" = "$\\\\chi^{2}(\\\\mathrm{df})$",
        "_R2_" = "$R^2$",
        "_X2_" = "$-2LLR (Model \\\\chi^2)$",
        "_Fmarkup_" = "F($df_{num}$,$df_{denom}$)",
        "_DOT_" = paste0("\\\\_"),
        "_LEQ_" = "$\\\\leq$",
        "_SIGMA_" = "$\\\\sigma$",
        "_NBSP_" = " ",
        "_FIXED_" = "$^+$",
        "_STAR0_" = "$\\\\phantom{{^{***}}}$",
        "_STAR1_" = "$^{*}\\\\phantom{{^{**}}}$",
        "_STAR2_" = "$^{**}\\\\phantom{{^{*}}}$",
        "_STAR3_" = "$^{***}$"
    )

    ## Replacement strings for HTML output
    ## TODO: 20171102: refactor abbreviations
    ## Problem in output is duplicate <td><td ..>, workaround in last item"
    html.markup <- c(                                                                                          
        "_LB_" = "<br>",  
        "_EOC_" = "</td>",   
        "_BOC_" = "<td>",   
        "_EOMC_" = "</td>",
        "_EOR_" = "</tr>", 
        "_BRU_" = paste("<tr><td style=\"border-bottom: solid thin black; border-collapse:collapse;\">"), 
        "_BRT_" = paste("<tr><td style=\"border-top: solid thin black; border-collapse:collapse;\">"),
        "_BOCU_" = paste("<td style=\"border-bottom: solid thin black; border-collapse:collapse;\">"),  
        "_BR_" = "<tr><td>",      
        "_BTABULAR_" =  "<table style=\"padding-right:20px;padding-left:20px;\">\n",                           
        "_BT_" =  "<table>\n",
        "_EOL_" = "\n",                                                                                        
        "_HL_" =  "",                                                                                          
        "_UL_" =  "<span style=\"text-decoration: underline;\">",                                              
        "_EOUL_" = "</span>",                                                                                  
        "_SEPU_" = "</td><td style=\"border-bottom: solid thin black; border-collapse:collapse;\">&nbsp;",     
        "_SEP_" = "</td><td>",                                                                                 
        "_ETABULAR_" = "</table>",                                                                             
        "_BOMR1_" = "<td rowspan = '1'>",                                                                      
        "_BOMR2_" = "<td rowspan = '2'>",                                                                      
        "_BOMC1_" = "<td colspan = '1'; align = 'center'>",                                                    
        "_BOMC2_" = "<td colspan = '2'; align = 'center'>",                                                    
        "_BOMC3_" = "<td colspan = '3'; align = 'center'>",                                                    
        "_BOMC4_" = "<td colspan = '4'; align = 'center'>",
        "_BOMC5_" = "<td colspan = '5'; align = 'center'>",
        "_BOMC6_" = "<td colspan = '6'; align = 'center'>",
        "_BOMC7_" = "<td colspan = '7'; align = 'center'>",
        "_BOMC8_" = "<td colspan = '8'; align = 'center'>",
        "_BOMC9_" = "<td colspan = '9'; align = 'center'>",
        "_BOML1_" = "<td colspan = '1'; align = 'left'>",
        "_BOML2_" = "<td colspan = '2'; align = 'left'>",
        "_BOML3_" = "<td colspan = '3'; align = 'left'>",
        "_BOML4_" = "<td colspan = '4'; align = 'left'>",
        "_BOML5_" = "<td colspan = '5'; align = 'left'>",
        "_BOML6_" = "<td colspan = '6'; align = 'left'>",
        "_BOML7_" = "<td colspan = '7'; align = 'left'>",
        "_BOML8_" = "<td colspan = '8'; align = 'left'>",
        "_BOML9_" = "<td colspan = '9'; align = 'left'>",
        "_BOMCT1_" = "<td colspan = '1'; style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;",
        "_BOMCT2_" = "<td colspan = '2'; style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;",
        "_BOMCT3_" = "<td colspan = '3'; style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;",
        "_BOMCT4_" = "<td colspan = '4'; align = 'center'; ; style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;",
        "_HTMLHL_" = "<tr><td colspan = '5'; align = 'center'; ; style=\"border-top: solid thin black; border-collapse:collapse;\">&nbsp;</tr>",
        "_DOT_" = "_",    
        "_CHI2_" = "&chi;<sup>2</sup>",
         "_X2_" =  "&chi;<sup>2</sup>",  
        "_R2_" = "R<sup>2</sup>",
        "_LEQ_" = "&le;",
        "_Fmarkup_" = "F(df1,df2)",                   
        "_SIGMA_" = "&sigma;",
        "_NBSP_" = "&nbsp;",
        "_FIXED_" = "<sup>+</sup>",
        "_STAR0_" = "&nbsp;",
        "_STAR1_" = "<sup>*</sup>",
        "_STAR2_" = "<sup>**</sup>",
        "_STAR3_" = "<sup>***</sup>",
        "<td>\\s*<td" = "<td"
    )
    
    ## Replacement strings for CSV output
    csv.markup <- c(
        "_LB_" = "\n",                           
        "_EOC_" =  ",",       
        "_BOC_" = "",                             
        "_EOMC_" = ",",                               
        "_EOR_" = "\n",                              
        "_BRU_" = "",                             
        "_BRT_" = "",                              
        "_BOCU_" = ",",                          
        "_BR_" = "",                               
        "_BTABULAR_" = "",                        
        "_EOL_" = "\n",                             
        "_HL_" = "",                          
        "_UL_" = "",                           
        "_EOUL_" = "",                           
        "_SEPU_" = "",                      
        "_SEP_" = ",",                            
        "_ETABULAR_" = "",                        
        "_BOMR1_" = "",                          
        "_BOMR2_" = "",                              
        "_BOMC1_" = "",                         
        "_BOMC2_" = "",                       
        "_BOMC3_" = "",                             
        "_BOMC4_" = "",
        "_BOMC5_" = "",
        "_BOMC6_" = "",
        "_BOMC7_" = "",
        "_BOMC8_" = "",
        "_BOMC9_" = "",
        "_BOML1_" = "",
        "_BOML2_" = "",
        "_BOML3_" = "",
        "_BOML4_" = "",
        "_BOML5_" = "",
        "_BOML6_" = "",
        "_BOML7_" = "",
        "_BOML8_" = "",
        "_BOML9_" = "",
        "_BOMCT1_" = "",
        "_BOMCT2_" = "",
        "_BOMCT3_" = "",
        "_BOMCT4_" = "",
        "_HTMLHL_" = "",
         "_DOT_" = ".",   
        "_CHI2_" = "chi^2",
        "_R2_" = "R^2",
        "_LEQ_" = "<=",
        "_SIGMA_" = "sigma",
         "_Fmarkup_" = "F",  
        "_NBSP_" = " ",
        "_FIXED_" = "+",
        "_STAR0_" = "", 
        "_STAR1_" = "*",
        "_STAR2_" = "**",
        "_STAR3_" = "**"
    )

   
    markup <- function(x, type){
        if (type == "latex"){
            markup <- latex.markup
        } else if (type == "html"){
            markup <- html.markup
        } else {
            markup <- csv.markup
        }
        
        for(i in names(markup)){
            x <- gsub(i, markup[i], x)
        }
        x
    }

    ## cs = column start
    ## uses environment for tight, defaults as centered
    ## 20190214: TODO figure out the underlining
    bomc <- function(ctr = TRUE, n, underline = FALSE){
        if(missing(n)) n <- if(tight)1 else 2
        if(ctr) {
            return(paste0("_BOMC", n, "_"))
        } else {
            return(paste0("_BOML", n, "_"))
        }
    }
    
    ## TESTME: grabs param from object by name, rounds, simplifies
    ## returns text. For getting r.square, adj.r.square, fstatistic.
    harvest <- function(sl, name) {
        res <- vector("character", length = length(sl))
        names(res) <- names(sl)
        
        for(i in seq_along(sl)) {
            y <- sl[[i]][[name]]
            
            if (length(y) <= 1 && (is.null(y) || is.null(y[1]) ||  is.na(y))) {
                y <- "_BOC__EOC_"
            } else if (name == "fstatistic"){                
                staty <- paste(format(c(y["value"]), digits = digits),
                               "(", format(y["numdf"], digits = digits),
                               ",", format(y["dendf"], digits = digits), ")", sep = "")

                nstars <- sum(pf(y["value"], df1 = y["numdf"], df2 = y["dendf"], lower.tail = FALSE) < alpha)
                y <- paste0("_BOC_", bomc(), staty, paste(rep("*", nstars), collapse = ""), "_EOMC_")
            } else if (is.numeric(y)) {
                if (length(y) > 1){
                    messg <- paste0("outreg: ", 
                                    "asked for: ", paste0("\"", name,"\"."),
                                    "These values were received: \n ",
                                    paste(y, collapse = " "), "\n",  
                                    "Just the first returned value is used for ", name)
                    warning(messg)
                    y <- y[1]
                }
                y <- paste0("_BOC_", format(round(y, digits), nsmall = digits), "_EOC_")
            } 
            if (is.null(y) ||is.na(y)) res[i] <- "" else res[i] <- y
        }
        if (any(res != "")) nonNull <- TRUE else nonNull <- FALSE
        attr(res, "nonNull") <- nonNull
        res
    }

    gofRow <- function(x, xname = "fixme") {
        ## x has markup already! don't re-insert
        zline <- c("_BR_", xname, "_EOC_")
        zline <- paste0(paste0(zline, collapse="") , paste0(x, collapse = paste0(if(!tight) "_BOC__EOC_" else " ")), "_EOR__EOL_")
        zline
    }


    gofPrint <- function(sl, name){
        y <- harvest(sl, name)
        xname <- ifelse(is.na(gofNames[name]), name, gofNames[name])
        if (attr(y, "nonNull")) {
            res <- gofRow(y, xname)
        } else {
            res <- ""
        }
    }

    ## Insert a horizontal line, or as close as we can get in an html table
    SL <- function(n, type) {
        if (type == "latex") x <- "\\hline\n"
        else if (type == "latex") x <- paste0("<tr><td colspan=\'", n, "\'", " style=\"border-bottom:solid thin black;\">&nbsp;</td></tr>\n")
        else x <- "\n"
        x
    }

    ## Double line
    DL <- function(n, type) {
        if (type == "latex") x <- "\\hline\n\\hline\n"
        else if (type == "html") x <- paste0("<tr  style=\"height:5px;\"><td colspan=\'", n, "\'", " style=\"border-bottom:double thin black;\">&nbsp;</td></tr>\n")
        else x <- "\n"
        x
    }
    
    ##Problem: some models have class output like maxLik:
    ## > class(res)
    ## [1] "maxLik" "maxim"  "list"
    ## So we can't just ask modelList if it is a list or an object.
   
    ## So as if the thing is ONLY a list with setequal
    if ( !setequal(class(modelList), "list") ){
        ## modelList is not a list only, so put it in a list
        modelList <- list(modelList)
    }
    checkReg(modelList) ###PJPJP FIXME. OK 20140109
    
    
    nmodels <- length(modelList)
    if (missing(modelLabels) || is.null(modelLabels)) {
        if(!is.null(names(modelList))){
            modelLabels = names(modelList)
        } else {
            modelLabels <- paste0("M", 1:nmodels)
        }
        for (i in seq_along(modelLabels)) {
            modlname <- modelLabels[i]
            if ((modlname == "" || is.na(modlname))) {
                modelLabels[i] <- if(names(modelList[i]) == "") paste0("M", i) else names(modelList[i])
            }
        }
    } else {
        if(length(modelList) != length(modelLabels)) {
            MESSG <- "modelLabels must be provided for each model"
            stop(MESSG)
        }
    }
    
    ##Ugh. nonunique labels. brute force fix
    names(modelList) <- make.unique(modelLabels)

    parmnames <- vector()
    myModelClass <- vector()

    getBSE <- function(modl, alpha, modLab = NULL) {
        if (is.null(best <- tryCatch(Blist[[modLab]], error = function(e) NULL))) {
            if (!is.null(estTable <- coef(summaryList[[modLab]], digits = 11))) {
                validColNum <-  which(colnames(estTable) %in% c("Estimate", "Value", "Param"))
                if (length(validColNum) > 1) stop(paste("Model ", modLab,
                      " has a summary table with unusual column names. They are ",
                      colnames(estTable)))
                if (length(validColNum) == 0) {
                    warning(paste("Model ", modLab, " summary table does not have a column named Estimate, Value, or Param, so we are guessing on column 1"))
                    validColNum <- 1
                }
                best <- estTable[ , validColNum]
            } else {
                best <- coef(modl)
            }
        }
        
        if (is.null(se <- tryCatch(SElist[[modLab]], error = function(e) NULL))) {
            if (!is.null(estTable <- coef(summaryList[[modLab]], digits = 11))) {
                se <- estTable[ , "Std. Error"]
            } else if (!is.null(vcov <- vcov(modl))){
                se <- sqrt(diag(vcov))
            } else {
                stop("outreg:getBSE can't get standard errors")
            }
        }
         
        if (!is.null(DF <- tryCatch(df.residual(modl), error = function(e) NULL))
            | !is.null(DF <- tryCatch(nobs(modl) - length(best), error = function(e) NULL))
            | !is.null(DF <- tryCatch(NROW(model.matrix(modl)), error = function(e) NULL))
            ) {##diagnostic## print("whew, there is a DF")
        } else {
            stop("Sorry, can't find the degrees of freedom on a a model")
        }
            
        if (is.null(PT <- tryCatch(PVlist[[modLab]], error = function(e) NULL))) {
            if (!is.null(estTable <- coef(summaryList[[modLab]], digits = 11)) & is.null(SElist[[modLab]])) {
                if (!is.na(x <-pmatch("Pr", colnames(estTable)))) {
                    PT <- estTable[ , colnames(estTable)[x]]
                } else if ("t value" %in% colnames(estTable)) {
                    PT <- pt(abs(estTable[ , "t value"]), lower.tail = FALSE, df = DF) * 2
                } else if ( "z value" %in% colnames(estTable)) {
                    PT <- pt(abs(estTable[ , "z value"]), lower.tail = FALSE, df = DF) * 2
                    ##fixme. Use Normal?
                }
            } else if(!is.null(best) & !is.null(se) & !is.null(DF)) {
                T <- best/se
                PT <- pt(abs(T), lower.tail = FALSE, df = DF) * 2
            } else {
                print(paste("Sorry, outreg can't figure a way to guess the p values for model", modLab))
            }
        }

       
        stars <- function(x, alpha) {xxx <- sum(abs(x) < alpha)
                                     paste0("", rep("*", xxx), collapse = "")}
        nstars <- sapply(PT, stars, alpha)
        addStar <- function(b, nstar) paste0(format(round(b, digits), nsmall = digits), nstar)
        BSTAR <- best
        for (i in seq_along(best)) BSTAR[i] <- addStar(best[i], nstars[i])
        names(BSTAR) <- names(best)
        se <- paste0("(", format(round(se, digits), nsmall = digits), ")")
       
        res <- data.frame(B = BSTAR, SE = se, stringsAsFactors = FALSE)
        rownames(res) <- names(best) 
        res
    }


    myModelClass <- lapply(modelList, function(x) {class(x)[1]})
    
    ## Get a regression summary object for each fitted model
    summaryList <- lapply(modelList, summary)
    ##    summaryList <- lapply(modelList, function(x) tryCatch(summary(x), error = NULL))
    
    ##    BSEs <- lapply(modelList, getBSE, alpha)
    BSEs <- vector("list", length = length(modelList))
    for(i in seq_along(modelList)){
        BSEs[[i]] <- getBSE(modelList[[i]], alpha, names(modelList)[i])
    }
    
    names(BSEs) <- modelLabels
  
    parmnames <- unique(unlist(lapply(BSEs, function(bse) rownames(bse))))
    B <- SE <- matrix(NA, nrow = length(parmnames), ncol =
                length(modelList), dimnames = list(parmnames, modelLabels))
    for (j in seq_along(modelList)){
        modl <- BSEs[[j]]
        modlnames <- rownames(BSEs[[j]])
        B[modlnames, j] <-  modl[modlnames, "B"]
        SE[modlnames, j] <- modl[modlnames, "SE"]
    }

    displayNames <- as.character(parmnames)
    names(displayNames) <- as.character(parmnames)
    displayNames[names(varLabels)] <- varLabels

    ## 20190214: new idea to fiddle lines HTML
    fudgeLength <- (1 + !tight) * length(modelList)
    blankline <- paste0("_BRU__EOC_", paste0(rep(paste0("_BOCU__EOC_"), fudgeLength), collapse=""), "_EOR__EOL_")
    
    getVC.merMod <- function(modl){
        if(inherits(modl, "merMod")){
            vc <- lme4::VarCorr(modl)
            formatVC <- getFromNamespace("formatVC", "lme4")
            vcfmt <- formatVC(vc, 3, "Std.Dev.")
            vcfmt[ ,2] <- gsub("\\(Intercept\\)", "", vcfmt[ ,2])
            
            for ( i in seq_along(vcfmt[ ,1])){
                if (i == 1) next
                if (vcfmt[i, 1] == "") vcfmt[i, 1] <- vcfmt[ (i-1), 1]
            }
            vcfmt[ ,1] <- paste0(vcfmt[ ,1], ":", vcfmt[ ,2])
            vcfmt[ ,1] <- gsub(":$", "", vcfmt[ ,1])
            vcfmt <- vcfmt[ , c("Groups", "Std.Dev."), drop = FALSE]
        } else {
           ## vcfmt <- cbind("Groups" = "", "Std.Dev." = "")
            vcfmt <- NULL
        }
        vcfmt
    }


    getVCmat <- function(modelList, modelLabels){
        VCs <-lapply(modelList, getVC.merMod)
       
        ##names(VCs) <- modelLabels
        vcnames <- unique(unlist(lapply(modelLabels, function(bsen) VCs[[bsen]][, "Groups", drop = FALSE])))
        
        if(any("Residual" %in% vcnames)) {
            if(!is.na(indx <- match("Residual" , vcnames))) {
                vcnames <- vcnames[ -indx ]
                vcnames <- c("Residual", vcnames)
        }}
        
        VCmat <- matrix("   ", nrow = length(vcnames), ncol =
                        length(modelList), dimnames = list(vcnames, modelLabels))
        for (i in seq_along(VCs)) {
            vc <- VCs[[i]]
            labl <- modelLabels[i]
            VCmat[vc[ , "Groups", drop = F], labl] <- vc[ , "Std.Dev."] 
        }
        VCmat
    }
    
    VCmat <- getVCmat(modelList, modelLabels)

    printVC <- function(VCmat){
        if (!any(VCmat != "")) return()
        aline <- paste0("_BR_", "Random Effects (_SIGMA_)", "_EOC__EOR__EOL_")
        if (tight) hereSep <- " _SEP_ " else hereSep <-  "_SEP_ _SEP_ "
 
        bline1 <- paste0("_BR__NBSP_", paste0(rownames(VCmat)))
        bline2 <- paste(rep(" ", max(2, (5 - nchar(rownames(VCmat))))), collapse = "")
        bline3 <- paste(apply(VCmat, 1, paste, collapse = hereSep), "_EOC__EOR__EOL_")
        bline <- paste(bline1, bline2, "_SEP_ ",  bline3)
        c(aline, bline)
    }
    
    z <- c()
    ## If you want a LaTeX table float...
    if (type == "latex") {
        if (float == TRUE || !missing(title) || !missing(label)){
            float <- TRUE
            aline <- "\\begin{table}\n"
            if (missing(title)) title <- "A Regression"
            if (missing(label)) label <- "regrlabl"
            z <- paste0(aline, "\\caption{", title, "}\\label{", label,"}\n")
        }
    }
    
    nColumns <- ifelse(tight, 1 + nmodels, 1 + 2*nmodels)
    ## siunitxmarkup
    ## Smarkup <- paste0("S[table-format=1.", digits, ", table-align-text-post=false]")
    Smarkup <- paste0("{S[
                         input-symbols = ( ),   
                         group-digits = false,   
                         table-number-alignment = center,   
                         %table-space-text-pre = (, 
                         table-align-text-pre = false,
                         table-align-text-post = false,
                         table-space-text-post = {", paste0(rep("*", length(alpha)), collapse=""),"},   
                         parse-units = false]}")
    ## Dcolumn treat all columns same with digits, not quite perfect
    ## Here we guess on 3.6, seems only way out of bad mis-alignment
    Dmarkup  <- paste0("{D{.}{.}{3.", digits + length(alpha), "}}")
    
    BT <- function(n, type = "latex"){
        if (type == "latex") {
            if(centering == "dcolumn"){
                return(paste0("\\begin{tabular}{@{}l*{",n-1,"}", Dmarkup, "@{}}\n", SL(n, type)))
            } else if (centering == "siunitx"){
                return(paste0("\\begin{tabular}{@{}l*{", n-1,"}", Smarkup, "@{}}\n", SL(n, type)))
            }
            else {
                return(paste0("\\begin{tabular}{@{}l*{",n,"}{l}@{}}\n", SL(n, type)))
            }
        }
        if (type == "html")  return(paste("<table>\n", SL(n, type)))
    }

    aline <- paste0(BT(nColumns, type = type))
    z <- c(z, aline)
 
    ## Put model labels on top of each model column, if modelLabels were given
    if (!is.null(modelLabels)){
        aline <- paste0("_BR_","_EOC_", collapse = "")
        for (modelLabel in modelLabels){
                 aline <- c(aline, paste0("_NBSP__BOC_", bomc(centering %in% c("dcolumn", "siunitx")), modelLabel, "_NBSP_ _EOMC_"))
        }
        aline <- c(aline, "_EOR__EOL_")
        z <- c(z, paste0(aline, collapse = ""))
    }

    ## Print the headers "Estimate" and "(S.E.)", output depends on tight or other format
    if (tight == TRUE) {
        aline <- paste0("_BR__EOC_", paste0(rep(paste0("_BOC_", bomc(centering %in% c("dcolumn", "siunitx")), "Estimate_EOMC_"), nmodels), collapse = ""),
                       "_EOR__EOL_", collapse = "") 
        z <- c(z, paste0(aline, collapse = ""))
        ##aline <- c("_BRU_", sprintf("%2s", " "), paste(rep ("_EOC__BOCU_ (S.E.)", nmodels, collapse = "")), "_EOR__EOL_")
        aline <- c("_BR__EOC_", paste0(rep(paste0("_BOC_", bomc(centering %in% c("dcolumn", "siunitx")), "(S.E.)_EOMC_"), nmodels, collapse = "")), "_EOR__EOL_")
        z <- c(z, paste0(aline, collapse = ""))
    } else {
        aline1 <- paste0("_BR__EOC_")
        aline2 <- paste0(rep ("_BOC__BOMC1_Estimate_EOMC__BOC__BOMC1_(S.E.)_EOMC_", nmodels), collapse = "")
        aline3 <- paste0("_EOR__EOL_")
        z <- c(z, paste0(aline1, aline2, aline3, collapse = ""))
    }

    if (type == "latex") z <- c(z, SL(1, "latex"), SL(1, "latex"))
    if (type == "html") z <- c(z, blankline)
   
    ## Here come the regression coefficients
    for (regname in parmnames){
        aline <- paste(paste("_BR_", displayNames[regname], "_EOC_"), collapse = "")
        for (model in modelLabels) {
            est <- B[regname, model]
            se <- SE[regname, model]
            if (!is.na(est)) {
                aline <- c(aline, paste("_BOC_", est, "_EOC_", collapse = " "))
                if (tight == FALSE) {
                    aline <- c(aline, paste("_BOC_", se, "_EOC_", collapse = " "))
                }
            } else {
                aline <- c(aline, paste0("_BOC_", bomc(centering %in% c("dcolumn", "siunitx"), 1), "_DOT_ _EOMC_", if(tight==FALSE) "_BOC__EOC_"))
            }
        }
        aline <- c(aline, "_EOR__EOL_")
        z <- c(z, paste(aline, collapse = ""))

        if (tight == TRUE){
            aline <- "_BR__EOC_"
            for (model in modelLabels) {
                est <- B[regname, model]
                se <- SE[regname, model]
                aline2 <- if (!is.na(est)) c("_BOC_", se, "_EOC_")  else c(" _BOC__EOC_")
                aline <- c(aline, paste(aline2, collapse = ""))
            }
            aline <- c(aline, "_EOR__EOL_")
            z <- c(z, paste(aline, collapse = ""))
        }
    }

    aline <- SL(nColumns, type)
    z <- c(z, aline)

  
    ## Print a row for the number of cases
    ##fiddline here to get line above N
    aline <- c("_BR_", "N", "_EOC_")
    for (model in modelList) {
        myN <- stats::nobs(model)
        columnFudge <- if(centering %in% c("dcolumn", "siunitx")) "_BOMC1_" else "_BOML1_"
        aline <- c(aline, "_BOC_", columnFudge,  myN, "_EOMC_", if(tight==FALSE) "  _BOC__EOC_" else "")
    }
    ##    if (type == "html") z <- c(z, blankline)
    aline <- c(if(type == "html") blankline else "", aline, " _EOR__EOL_")
    z <- c(z, paste(aline, collapse = ""))

    ## The new way
    z <- c(z, gofPrint(summaryList, "sigma"))
   
    ## The new way
    z <- c(z, gofPrint(summaryList, "r.squared"))

    ##"adj.r.squared" if there is more than 1 predictor

    ## Print a row for the adj-R-square
    if (length(parmnames) > 2) {
        z <- c(z, gofPrint(summaryList, "adj.r.squared"))
    }

    if (!missing(request)){
        for (extra in names(request)){
            z <- c(z, gofPrint(summaryList, extra))
        }
    }

    z <- c(z, printVC(VCmat))


    ## Print a row for the model residual deviance
    if ("glm" %in% myModelClass) {
        z <- c(z, gofPrint(summaryList, "deviance"))
    }

    ## Print a row for the model's fit, as -2LLR
    if ("glm" %in% myModelClass) {
        aline <- "_BR__X2_ "
        for (model in modelList) {
            if (is.numeric(model$deviance)){
                n2llr <- model$null.deviance - model$deviance
                aline <- c(aline, paste("_SEP_  ", format(round(n2llr, digits), nsmall = digits)))
                gmdf <- model$df.null - model$df.residual + 1
                nstars <- sum(pchisq(n2llr, df = gmdf, lower.tail = FALSE) < alpha)
                aline <-  paste0(paste(aline, collapse = ""), paste0(rep("*", nstars), collapse=""))
            } else {
                aline <- c(aline, "_SEP_ ")
            }
            if (tight == FALSE) aline <- c(aline, "_SEP_  ")
        }
        aline <- paste(paste(aline, collapse = ""), "_EOR__EOL_")
        z <- c(z, paste(aline))
    }

    ## Print a row for the model's fit, as -2 LLR
    ## Can't remember why I was multiplying by -2
    if (showAIC == TRUE) {
        aicList <- lapply(modelList, function(x) {
            aic.x <- AIC(x)
            y <- if(is.numeric(aic.x)) format(aic.x, digits=digits, nsmall=3) else ""
            paste0("_BOC_", y, "_EOC_")
        })
        z <- c(z, gofRow(aicList, "AIC"))
    }

    ## TODO: round the following output
    if (!missing(runFuns)){
        elist <- vector("list", length = length(runFuns))
        runFunsFn <- names(runFuns)
        for (i in seq_along(runFuns)){
            myfn <- runFunsFn[i]
            if (myfn == "logLik") {
                myresult <- lapply(modelList, function(x) {
                    y <- do.call(myfn, list(x))
                    fstaty <- paste(format(y[1], digits = digits), collapse = ", ",
                                    "(", format(attr(y, "df")), ")", sep = "")
                    fstaty <- paste0(if(tight)"_BOC__BOMC1_" else "_BOMC2_", fstaty, "_EOMC_")
                    invisible(fstaty)
                })
                elist[[i]] <- myresult
            } else {
                myresult <- lapply(modelList, function(x){
                    y <- do.call(myfn, list(x))
                    fstaty <- format(c(y), digits = digits, nsmall = 2)
                    fstaty <- paste0(if(tight)"_BOC__BOMC1_" else "_BOMC2_", fstaty, "_EOMC_")
                })
                elist[[i]] <- myresult
            }
        }
        names(elist) <- runFunsFn

        for(i in seq_along(runFuns)){
            z <- c(z, gofRow(elist[[i]], runFuns[i]))
        }
    }

    aline <- DL(nColumns, type)
    z <- c(z, aline)
    z <- c(z, "_EOL_")

    

    pline <- function(type, alpha){
        aline <- "_NBSP__NBSP_"
        if (type == "latex"){
            for ( i in seq_along(alpha)){
                aline <- paste0(aline, "${", paste0(rep("*", i), collapse = "\\!\\!"), "\  p}",  "\\le ", alpha[i], "$", sep = "")
            }
            aline <- paste0("\\multicolumn{", nColumns, "}{l}{", aline, "_EOMC__EOR__EOL_")
        } else if (type == "html"){
            aline <- paste0("<tr>\n",
                            "<td colspan=\"", nColumns, "\">")
            for ( i in seq_along(alpha)){
                aline <- paste0(aline,  paste0(rep("*", i), collapse = ""), " <it>p</it> &#8804;", alpha[i], sep = "")
            }
            aline <- paste0(aline, "_EOR__EOL_")
        } else {
            for ( i in seq_along(alpha)){
                aline <- paste0(aline,  paste0(rep("*", i), collapse = ""), "p <", alpha[i], sep = ",")
            }
            aline <- paste0(aline, "\n")
        }
        aline
    }

        
    z <- c(z, pline(type, alpha))

    aline <- "_ETABULAR__EOL_"
    z <- c(z, aline)
    if (float == TRUE && type == "latex"){
        aline <- "\\end{table}_EOL_\n"
        z <- c(z, aline)
    }
    
    zresult <- markup(z, type = type)
    matchCall <- match.call()
    matchCall[["type"]] <- "html"
               
    if (type == "latex" || type == "csv") {
        if (print.results) cat(zresult)
        return(invisible(zresult))
    } else if (browse) {
        fn <- tempfile(pattern = "file", tmpdir = tempdir(), fileext = ".html")
        cat(zresult, file = fn)
        cat(paste("HTML file : ", fn, "\n"))
        browseURL(fn)
        attr(zresult, "file") <- fn
        return(invisible(zresult))
    } else {
        if (print.results) cat(zresult)
        return(invisible(zresult))
    }
}

NULL



##' Convert LaTeX output from outreg to HTML markup
##'
##' This function is deprecated. Instead, please use \code{outreg(type = "html")}
##'
##' This will write the html on the screen, but if a filename argument is
##' supplied, it will write a file. One can
##' then open or insert the file into Libre Office or other popular
##' "word processor" programs.
##'
##' @param outreg output from outreg
##' @param filename A file name into which the regression markup is to be saved. Should end in .html.
##' @return A vector of strings
##' @export
##' @author Paul E. Johnson \email{<pauljohn@@ku.edu>}
##' @examples
##' dat <- genCorrelatedData2(means = c(50,50,50,50,50,50),
##'     sds = c(10,10,10,10,10,10), rho = 0.2, beta = rnorm(7), stde = 50)
##' m1 <- lm(y ~ x1 + x2 + x3 + x4 + x5 + x6 + x1*x2, data = dat)
##' summary(m1)
##'
##' m1out <- outreg(list("Great Regression" = m1), alpha = c(0.05, 0.01, 0.001),
##'          request = c("fstatistic" = "F"), runFuns = c(AIC = "AIC"),
##'          float = TRUE)
##' ##html markup will appear on screen
##' outreg2HTML(m1out)
##  ## Run this for yourself to create an output file
##' ## outreg2HTML(m1out, filename = "funky.html")
##' ## I'm not running that for you because you
##' ## need to be in the intended working directory
##'
##' m2 <- lm(y ~ x1 + x2, data = dat)
##'
##' m2out <- outreg(list("Great Regression" = m1, "Small Regression" = m2),
##'                alpha = c(0.05, 0.01, 0.01),
##'                 request = c("fstatistic" = "F"), runFuns = c(BIC = "BIC"))
##' outreg2HTML(m2out)
##' ## Run this for yourself, it will create the output file funky2.html
##' ## outreg2HTML(m2out, filename = "funky2.html")
##' ## Please inspect the file "funky2.html
##'
outreg2HTML <-
    function(outreg, filename)
{
    myz2 <- gsub("^\\n$", "</tr></td>\n", outreg)
    myz2 <- gsub("^", "<tr><td>", myz2)
    myz2 <- gsub(".*\\\\begin\\{tabular\\}.*$", "<table>\n", myz2)
    myz2 <- gsub("\\\\\\\\","</td></tr>", myz2)
    myz2 <- gsub("^.*\\\\hline.*", "", myz2)
    myz2 <- gsub("&", "</td><td>", myz2)
    myz2 <- gsub(".*end\\{tabular\\}", "</table>", myz2)
    myz2 <- gsub("\\\\le", "&#8804;", myz2)
        ## Emacs indentation fooled by previous
        myz2 <- gsub("\\$R\\^2\\$", "R<sup>2</sup>", myz2)

    myz2 <- sub("<td>\\\\mul(.*?)\\$\\{", "<td colspan = '3'>",  myz2)
    myz2 <- gsub("<td>\\\\multicolumn\\{(\\d+)\\}\\{.*?\\}\\{(.*?)\\}", "<td colspan = '\\1'> \\2", myz2)


    myz2 <- gsub("\\$\\{", "", myz2)
    myz2 <- gsub("(\\**)}", "\\1", myz2)
    myz2 <- gsub("\\$\ *", " ", myz2)
    myz2 <- gsub("\\\\chi\\^2", "&chi;<sup>2</sup>", myz2)

    if (!missing(filename)){
        if (!checkIntFormat(filename))
            stop("invalid 'file'")
        cat(myz2, file = filename)
        cat(paste("Saved to ", filename, "\n"))
        } else {
            cat(myz2)
        }
    invisible(myz2)
}

##' Estimate leave-one-variable-out regressions
##'
##' This is a convenience for analysis of multicollinearity in regression.
##'
##' @param model a fitted regression model
##' @return a list of fitted regressions, one for each omitted variable.
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @export
lmAuxiliary <- function(model){
  dat <- as.data.frame(model.matrix(model))
  ## ivnames <- attr(delete.response(terms(model)), "term.labels")
  ## previous does not work with transforms like poly
  hasIntercept <- attr(terms(model), "intercept")
  if (hasIntercept) dat <- dat[ , -1] # removes intercept. assumes intercept in column 1 
  ivnames <- colnames(dat)
  cat("The following auxiliary models are being estimated and returned in a list:\n")

  results <- list()
  colnames(dat) <- gsub("`","",  colnames(dat)) #rm ticks on varnames
  
  for (i in ivnames) {
    fmla <- paste( "`",i,"`", " ~ ." , sep = "") #add ticks in fmla
    fmla <- gsub("``","`",  fmla) # remove double ticks
    lmcall <- call("lm", formula(fmla, data = dat), quote(dat))
    results[[ i ]] <- maux <- eval(lmcall)
    maux.fmla <- formula(maux)
    print(maux.fmla, showEnv = FALSE)
  }
 results
}
NULL

##' retrieves estimates of the coefficient of determination from a list of regressions
##'
##' Asks each regression model in a list for a summary and then reports the R-squares.
##' @param auxRegs a list of fitted regression objects
##' @return a numeric vector of the same length as auxRegs.
##' @author Paul E. Johnson <pauljohn@@ku.edu>
getAuxRsq <- function(auxRegs){
  auxRsq <- numeric(length=length(auxRegs))
  j <- 0
  for ( i in auxRegs ){
    j <- j + 1
    sumry <- summary(i)
    auxRsq[j] <- sumry$r.squared
  }

  names(auxRsq) <- names(auxRegs)

  invisible(auxRsq)
}
NULL


##' Converts the R-square to the variance inflation factor
##'
##' calculates vif = 1/(1-R-square)
##'
##' @param rsq a vector of real values, presumably fitted R-squares
##' @return a vector of vif estimates
##' @author Paul E. Johnson <pauljohn@@ku.edu>
getVIF <- function(rsq){
  vif <- 1/(1-rsq)

  invisible(vif)
}

##' Calculates the delta R-squares, also known as squared
##' semi-partial correlation coefficients.
##'
##' The change in the R-square when a variable is removed from a
##' regression is called delta R-square. It is sometimes suggested as
##' a way to determine whether a variable has a substantial effect on
##' an outcome. This is also known as the squared semi-partial correlation
##' coefficient.
##'
##' @export
##' @importFrom stats drop1
##' @param model a fitted regression model
##' @return a vector of estimates of the delta R-squares
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @examples
##' dat1 <- genCorrelatedData(N=250, means=c(100,100),
##' sds=c(30,20), rho=0.0,  stde = 7, beta=c(1.1, 2.4, 4.1, 0))
##' m1 <- lm(y ~ x1 + x2, data=dat1)
##' getDeltaRsquare(m1)
##' ## more problematic in presence of collinearity
##' dat2 <- genCorrelatedData(N=250, means=c(100,100),
##' sds=c(30,20), rho=0.6,  stde = 7, beta=c(1.1, 2.4, 4.1, 0))
##' m2 <- lm(y ~ x1 + x2, data=dat2)
##' getDeltaRsquare(m2)
getDeltaRsquare <- function(model){
  modeldrop1 <- drop1(model)
  RSS <- modeldrop1[1, "RSS"] ##Residual Sum of Squares
  deltaSS <- modeldrop1[ , "Sum of Sq"]
  SST = sum((model$model[ , 1] - mean(model$model[, 1]))^2)
  deltaRsquare <- deltaSS/SST
  names(deltaRsquare) <- row.names(modeldrop1)
  omit <- is.na(deltaRsquare)
  deltaRsquare <- deltaRsquare[-omit]
  cat("The deltaR-square values: the change in the R-square
      observed when a single term is removed.", fill = TRUE)
  cat("Same as the square of the 'semi-partial correlation coefficient'", fill = TRUE)
  print(as.data.frame(deltaRsquare))
  invisible(deltaRsquare)
}
NULL

##' Multi-collinearity diagnostics
##'
##' Conducts a series of checks for multicollinearity.
##' @param model a fitted regression model
##' @return a list of the "auxiliary regressions" that were
##' fitted during the analysis
##' @export
##' @importFrom stats model.matrix
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @example inst/examples/mcDiagnose-ex.R
mcDiagnose <- function(model){
  if (any(is.na(coef(model)))) stop("There are redundant variables in the model. Fix the specification before diagnosing multicollinearity")
  auxRegs <- lmAuxiliary(model)
  auxRsq <- getAuxRsq(auxRegs)
  vif <- getVIF(auxRsq)
  cat("\n")
  cat("R_j Squares of auxiliary models\n")
  print(auxRsq)
  cat("The Corresponding VIF, 1/(1-R_j^2)\n")
  print(vif)
  cat("Bivariate Pearson Correlations for design matrix \n")
  mm <- model.matrix(model)[ , -1] ## data, omit intercept
  print(round(cor(mm[,]), 2))
  invisible(auxRegs)
}
NULL

#TODO 1. Find out if there is a more numerically stable/recommended method
#TODO 2. figure smarter way to get rid of intercept variable than -1
##' Calculates partial correlation coefficients after retrieving data matrix froma fitted regression model
##'
##' The input is a fitted regression model, from which the design
##' matrix is retrieved, along with the dependent variable. The
##' partial correlation is calculated using matrix algebra that
##' has not been closely inspected for numerical precision.  That is to
##' say, it is in the stats book style, rather than the numerically
##' optimized calculating format that functions like \code{lm()} have adopted.
##'
##' I often criticize partial correlations because they change in a
##' very unstable way as terms are added or removed in regression
##' models. Nevertheless, I teach with books that endorse them, and in
##' order to have something to criticize, I need to have a function
##' like this. There are other packages that offer partial correlation
##' calculations, but they are either 1) not easy to install from CRAN
##' because of dependencies or 2) do not directly calculate the values
##' we want to see.
##'
##' To students. 1) This gives the same result as the function
##' \code{cov2pcor} in \code{gRbase}, so far as I can tell. Why use
##' this?  Simply for convenenience.  We have found that installing
##' gRbase is a problem because it depends on packages in
##' Bioconductor.  2) By default, I show only one column of output,
##' the partial correlations involving the dependent variable as
##' something being explained. The other columns that would depict the
##' dependent variable as a predictor of the independent variables
##' have been omitted. You can let me know if you think that's wrong.
##'
##' Please note I have not gone out of my way to make this calculation
##' "numerically stable."  It does not use any orthogonal matrix
##' calculations; it is using the same textbook theoretical stats
##' formula that is used by cov2pcor in gRbase and in every other
##' package or online source I could find. I prepared a little
##' WorkingExample file matrix-partial-correlations-1.R that
##' discusses this, in case you are interested (http://pj.freefaculty.org/R).
##'
##' @param model A fitted regression model, such as output from
##' lm(). Any object that has methods model.matrix and model.frame
##' will be sufficient.
##' @param dvonly Default = TRUE. Only show first column of the full
##' partial correlation matrix. That corresponds to the partial correlation of each predictor with y. I mean, r[yx].[others]
##' @return A column or matrix of partial correlation coefficients
##' @export
##' @author Paul E. Johnson <pauljohn@@ku.edu>
getPartialCor <- function(model, dvonly = TRUE){
    x <- model.matrix(model)
    modelterms <- terms(model)
    hasIntercept <- attr(modelterms, "intercept")
    if (hasIntercept) x <- x[ , -1] # removes intercept. assumes intercept in column 1
    yname <- as.character(modelterms[[2]])
    y <- model.response(model.frame(model))
    x <- cbind(y, x)
    colnames(x)[1] <- yname
    R <- cor(x)
    Rinv <- solve(R)
    Id <- diag(1/sqrt(diag(Rinv)))
    p <- - Id %*% Rinv %*% Id
    dimnames(p) <- list(colnames(x), colnames(x))
    if (dvonly) p[ ,1, drop = FALSE] else p
}

## There's got to be a better way.

## Center X, then proceed via QR decomposition?
## S-1 * R'R * S-1

## Solving that:
## S *  inv(R) inv(R') S

##' Calculates a "residual-centered" interaction regression.
##'
##' Given a fitted \code{lm}, this function scans for coefficients
##' estimated from "interaction terms" by checking for colon
##' symbols. The function then calculates the "residual centered"
##' estimate of the interaction term and replaces the interaction term
##' with that residual centered estimate. It works for any order of
##' interaction, unlike other implementations of the same
##' approach. The function \code{lmres} in the now-archived package
##' pequod was a similar function.
##' @param model A fitted lm object
##' @export 
##' @rdname residualCenter
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @references  Little, T. D., Bovaird, J. A.,
##' & Widaman, K. F. (2006). On the Merits of Orthogonalizing
##' Powered and Product Terms: Implications for Modeling
##' Interactions Among Latent Variables.
##' Structural Equation Modeling, 13(4), 497-519.
residualCenter <- function(model)
{
    UseMethod("residualCenter")
}

##' @rdname residualCenter
##' @export
##' @return a regression model of the type as the input model, with
##' the exception that the residualCentered predictor is used in place
##' of the original interaction. The return model includes new
##' variable centeringRegressions: a list including each of the
##' intermediate regressions that was calculated in order to create
##' the residual centered interaction terms. These latter objects may
##' be necessary for diagnostics and to calculate predicted values for
##' hypothetical values of the inputs. If there are no interactive
##' terms, then NULL is returned.
##' @method residualCenter default
##' @export
##' @example inst/examples/residualCenter-ex.R
residualCenter.default <- function (model)
{
    createRCinteraction <- function(x, data) {
        nterms <- length(strsplit( x, ":")[[1]])
        if (nterms == 1) return(NULL)
        dv <- paste("I(", gsub(":", "*", x), ")", sep = "")
        iv <- paste("(", gsub(":"," + ",x),")")
        if(nterms >= 3) iv <- paste(iv, "^", nterms-1, sep = "")
        myformula <- paste(dv, "~", iv)
        myformula <- as.formula(myformula)
        rcterms <-  attr(terms(myformula), "term.labels")
        rcterms <- gsub(":", ".X.", rcterms)
        finalFormula <- paste(dv, "~", paste(rcterms, collapse = " + "))
        aReg <- lm(finalFormula, as.data.frame(data))
        list(residuals = as.numeric(resid(aReg)), reg = aReg)
    }

    dat <- model.matrix(model)

    if (colnames(dat)[1] == "(Intercept)")  dat <- dat[ , -1] # remove intercept
    datn <- colnames(dat)

    frmla <- paste(datn, collapse = " + ")

    rcRegressions <- list()
    for (i in seq_along(datn)) {
        newname <- gsub(":", ".X.", datn[i])
        res <- createRCinteraction( datn[i] , data = dat)
        if (!is.null(res)){
            dat[ , datn[i]] <- res[["residuals"]]
            rcRegressions[[newname]] <- res[["reg"]]
            dimnames(dat)[[2]][i] <- newname
        }
    }

    modelt <- terms(model)
    frmla <- paste(colnames(dat), collapse = " + ")
    frmlanew <- as.formula(paste( modelt[[2L]], "~" , frmla))

    mfnew <- model.frame(frmlanew, data=cbind(dat, model$model))

    mc <- model$call
    mc$formula <- frmlanew
    mc$data <- quote(mfnew)
    res <- eval(mc)

    class(res) <- c("rcreg", class(model))
    res$rcRegressions <- rcRegressions
    res
}
NULL

##' predict method for rcreg objects
##'
##' Calculates predicted values of
##' residual centered interaction regressions estimated in
##' any type of regression framework (lm, glm, etc).
##' @method predict rcreg
##' @export 
##' @rdname residualCenter
##' @example inst/examples/predict.rcreg-ex.R
##' @param object Fitted residual-centered regression from residualCenter
##' @param ... Other named arguments. May include newdata, a dataframe of 
##' predictors. That should include values for individual predictor, need 
##' not include interactions that are constructed by residualCenter. 
##' These parameters that will be passed to the predict method of the model.
predict.rcreg <-
    function (object, ...)
{
    if ( ! c("rcreg") %in% class(object) ) stop("predict.rcreg is intended for rcreg objects, which are created by residualCenter in the rockchalk package")

    dots <- list(...)
    newdata <- NULL
    if (!is.null(dots[["newdata"]])) {
        newdata <- dots[["newdata"]]
        dots[["newdata"]] <- NULL
    } else {
        newdata <- model.frame(object)
    }

    objectTerms <- terms(object)
    dvname <- names(attr(objectTerms, "dataClasses"))[1]
    rcRegs <- object$rcRegressions
    nams <- names(rcRegs)
    ## remove user's residual centered variables, if any
    ## for(i in nams)  newdata[[i]] <- NULL
    for(i in seq_along(rcRegs)){
        aReg <- rcRegs[[i]]
        prodVars <-  unlist(strsplit(nams[i], ".X."))
        predvals <-  predict.lm(aReg, newdata = newdata )
        actualProduct <- apply(newdata[ ,prodVars], 1, prod)
        myresids <-  actualProduct - predvals
        newdata[[nams[i]]] <- myresids
    }
    NextMethod(object, newdata = newdata, dots)
}

##' Creates a cross tabulation with counts and percentages
##'
##' @description
##' This function is pronounced "presentable"!  The original purpose
##' was to create a particular kind of cross tabulation that I ask for
##' in class: counts with column percentages. Requests from users have
##' caused a bit more generality to be built into the function. Now,
##' optionally, it will provide row percents. This is a generic function.
##' Most users will find the formula method most convenient. Use the
##' colpct and rowpct arguments to indicate if column or row percentages
##' are desired.
##'
##' I suggest most users will use the formula method for this. Running
##' a command like this will, generally, do the right thing:
##'
##' \code{tab <- pctable(y ~ x, data = dat)}
##'
##' There is also a method that will work with characters representing
##' variable names.
##' 
##' \code{tab <- pctable("y", "x", data = dat)}
##'  
##' Running the function should write a table in the output console,
##' but it also creates an object (\code{tab}). That object
##' can be displayed in a number of ways.
##'
##' A summary method is provided, so one could look at different
##' representations of the same table.
##'
##' \code{summary(tab, rowpct = TRUE, colpct = FALSE)}
##'
##' or
##'
##' \code{summary(tab, rowpct = TRUE, colpct = TRUE)}
##'
##' Tables that include only row or column percentages will be
##' compatible with the html and latex exporters in the
##' excellent \code{tables} package.
##' 
##' @details
##' Please bear in mind the following. The output object is a
##' list of tables of partial information, which are then assembled in
##' various ways by the print method (print.pctable). A lovely table
##' will appear on the screen, but the thing itself has more
##' information and a less beautiful structure.
##' 
##' A print method is supplied.
##' For any \code{pctable} object, it is possible to run follow-ups like
##'
##' print(tab, rowpct = TRUE, colpct = FALSE)
##' 
##' The method \code{print.pctable(tab)} assembles the object into (my
##' opinion of) a presentable form. The print method has argumnets
##' \code{rowpct} and \code{colpct} that determine which percentages
##' are included in the presentation.
##' 
##' @name pctable
NULL

##' @param rv A row variable name
##' @param ... Other arguments. So far, the most likely additional
##' arguments are to be passed along to the table function, such as
##' "exclude", "useNA", or "dnn" (which will override the rvlab and
##' cvlab arguments provided by some methods). Some methods will
##' also pass along these arguments to model.frame, "subset" 
##' "xlev", "na.action", "drop.unused.levels".
##' @seealso \code{\link[tables]{tabular}} and the CrossTable function
##' in \code{gmodels} package.
##' @rdname pctable
##' @export
##' @return A list with tables (count, column percent, row percent) as
##' well as a copy of the call.
##' @author Paul Johnson <pauljohn@@ku.edu>
pctable <- function(rv, ...)
{
    UseMethod("pctable")
}
NULL




##' The method pctable.default is the calculator, I don't expect
##' many users will need to call it directly.
##' 
##' @param cv Column variable
##' @param rvlab Optional: row variable label
##' @param cvlab Optional: col variable label
##' @param colpct Default TRUE: are column percentags desired in the
##' presentation of this result?
##' @param rowpct Default FALSE: are row percentages desired in the
##' presentation of this result
##' @param rounded Default FALSE, rounds to 10's for privacy purposes.
##' @rdname pctable
##' @method pctable default
##' @rdname pctable
##' @export
pctable.default <- function(rv, cv,
                            rvlab = NULL, cvlab = NULL,
                            colpct = TRUE, rowpct = FALSE,
                            rounded = FALSE, ...)
{
    rvlabel <- if (!missing(rv)) deparse(substitute(rv))
    cvlabel <- if (!missing(cv)) deparse(substitute(cv))
    rvlab <- if (is.null(rvlab)) rvlabel else rvlab
    cvlab <- if (is.null(cvlab)) cvlabel else cvlab

    dots <- list(...)
    dotnames <- names(dots)
    ## altargs <- list()
   
    ## if ("dnn" %in% dotnames) altargs$dnn <- dots[["dnn"]]
    ## if ("deparse.level" %in% dotnames)
    ##     altargs$deparse.level <- dots[["deparse.level"]]
    
    tableargs <- list(rv, cv, dnn = c(rvlab, cvlab))
    newargs <- modifyList(tableargs, dots, keep.null = TRUE)

    t1 <- do.call("table", newargs)
    rownames(t1)[is.na(rownames(t1))] <- "NA" ## symbol to letters
    colnames(t1)[is.na(colnames(t1))] <- "NA"
    if (rounded) t1 <- round(t1, -1)
    t2 <- addmargins(t1, c(1,2))
    t1cpct <- round(100*prop.table(t1, 2), 1)
    t1rpct <- round(100*prop.table(t1, 1), 1)
    t1cpct <- apply(t1cpct, c(1,2), function(x) gsub("NaN", "", x))
    t1rpct <- apply(t1rpct, c(1,2), function(x) gsub("NaN", "", x))
    res <- list("count" = t2, "cpct" = t1cpct, "rpct" = t1rpct,
                call = match.call())
    class(res) <- "pctable"
    print.pctable(res, colpct = colpct, rowpct = rowpct)
    invisible(res)
}
NULL




##' Creates a cross tabulation with counts and column percents
##'
##' The formula method is the recommended method for users. Run
##' \code{pctable(myrow ~ mycol, data = dat)}. In an earlier version,
##' I gave different advice, so please adjust your usage.
##' 
##' @param formula A two sided formula.  
##' @param data A data frame.
##' @examples
##' dat <- data.frame(x = gl(4, 25),
##'                   y = sample(c("A", "B", "C", "D", "E"), 100, replace= TRUE))
##' pctable(y ~ x, dat)
##' pctable(y ~ x, dat, exclude = NULL)
##' pctable(y ~ x, dat, rvlab = "My Outcome Var", cvlab = "My Columns")
##' pctable(y ~ x, dat, rowpct = TRUE, colpct = FALSE)
##' pctable(y ~ x, dat, rowpct = TRUE, colpct = TRUE)
##' pctable(y ~ x, dat, rowpct = TRUE, colpct = TRUE, exclude = NULL)
##' tab <- pctable(y ~ x, dat, rvlab = "Outcome", cvlab = "Predictor")
##' @rdname pctable
##' @method pctable formula
##' @export
pctable.formula <- function(formula, data = NULL,  rvlab = NULL,
                            cvlab = NULL, colpct = TRUE, rowpct = FALSE,
                            rounded = FALSE, ...)
    
{
    if (missing(formula) || (length(formula) != 3L))
        stop("pctable requires a two sided formula")
    dots <- list(...)
    dotnames <- names(dots)
    
    mt <- terms(formula, data = data)
    if (attr(mt, "response") == 0L) stop("response variable is required")
    mf <- match.call(expand.dots = TRUE)
    mfnames <- c("formula", "data", "subset", "xlev", "na.action", "drop.unused.levels")
    keepers <- match(mfnames, names(mf), 0L)
    mf <- mf[c(1L, keepers)]
    
    ## mf$drop.unused.levels <- FALSE
    if (!"na.action" %in% dotnames) mf$na.action <- na.pass
    mf[[1L]] <- quote(stats::model.frame)

    ## remove used arguments from dots, otherwise errors happen
    ## when unexpected arguments pass through. Don't know why
    for (i in c("subset", "xlev", "na.action", "drop.unused.levels")) dots[[i]] <- NULL
        
    mf <- eval(mf, parent.frame())
    mfnames <- names(mf)
    response <- attr(attr(mf, "terms"), "response")
    ## response is column 1
    rvname <- mfnames[response]
    cvname <- mfnames[-response][1] ##just take 2?
    rvlab <- if (missing(rvlab)) rvname else rvlab
    cvlab <- if (missing(cvlab)) cvname else cvlab

    arglist <- list(rv = mf[[rvname]], cv = mf[[cvname]],
                    rvlab = rvlab, cvlab = cvlab,
                    colpct = colpct, rowpct = rowpct,
                    rounded = rounded)
    arglist <- modifyList(arglist, dots, keep.null = TRUE)
    ##keep.null needed because exclude = NULL can be a valid
    ## (meaningful) argument to table.
    
    res <- do.call(pctable.default, arglist)
    invisible(res)
}
NULL





##' Method for variable names as character strings 
##'
##' The character method exists only for variety.  It accepts
##' character strings rather than a formula to define the columns that
##' should be plotted.  The method used most often for most users should
##' be the formula method.
##' 
##' When using character arguments, the row variable rv rowvar must be
##' a quoted string if the user intends the method pctable.character
##' to be dispatched. The column variable cv may be a string or just a
##' variable name (which this method will coerce to a string).
##' @examples
##' dat <- data.frame(x1 = gl(4, 25, labels = c("Good", "Bad", "Ugly", "Indiff")),
##'                 x2 = gl(5, 20, labels = c("Denver", "Cincy", "Baltimore", "NY", "LA")), 
##'                 y = sample(c("A", "B", "C", "D", "E"), 100, replace= TRUE))
##' tab <- pctable(y ~ x1, data = dat, rvlab = "my row label",
##'     subset = dat$x1 %in% c("Good", "Bad"),
##'     drop.unused.levels = TRUE)
##' tab <- pctable(y ~ x1, data = dat, rvlab = "my row label",
##'     subset = dat$x1 %in% c("Good", "Bad"))
##' pctable("y", "x1", dat)
##' pctable("y", x1, dat)
##' tab <- pctable(y ~ x2, data = dat, rvlab = "A Giant Variable")
##' summary(tab, rowpct = TRUE, colpct = FALSE)
##' tabsum <- summary(tab)
##' \donttest{
##' ## if user has tables package, can push out to latex or html
##' if (require(tables) & require(Hmisc)){
##'     tabsumtab <- tables::as.tabular(tabsum)
##'     Hmisc::html(tabsumtab)
##'     fn <- tempfile(pattern = "file", tmpdir = tempdir(), 
##'             fileext = ".html")
##'     Hmisc::html(tabsumtab, file = fn)
##'     print(paste("The file saved was named", fn, "go get it."))
##'     if (interactive()) browseURL(fn)
##'     ## go get the fn file if you want to import it in document
##'     ## Now LaTeX output
##'     ## have to escape the percent signs
##'     tabsumtab <- apply(tabsumtab, 1:2, function(x) {gsub("%", "\\\\%", x) })
##'     fn2 <- tempfile(pattern = "file", tmpdir = tempdir(), 
##'                    fileext = ".tex")
##'     Hmisc::latex(tabsumtab, file = fn2)
##'     print(paste("The file saved was named", fn2, "go get it."))
##' }
##' }
##' @rdname pctable
##' @method pctable character
##' @export
pctable.character <- function(rv, cv, data = NULL, rvlab = NULL,
                              cvlab = NULL, colpct = TRUE,
                              rowpct = FALSE,
                              rounded = FALSE, ...)
{
    if (missing(data) || !is.data.frame(data)) stop("pctable requires a data frame")
    cv <- as.character(substitute(cv))[1L]
    
    rvlab <- if (missing(rvlab)) rv else rvlab
    cvlab <- if (missing(cvlab)) cv else cvlab
    res <- pctable.formula(formula(paste(rv, " ~ ", cv)), data = data,
                           rvlab = rvlab, cvlab = cvlab, colpct = colpct,
                           rowpct = rowpct, rounded = rounded, ...)
  
    invisible(res)
}
NULL


##' Extract presentation from a pctable object
##'
##' Creates a column and/or row percent display of a pctable
##' result
##' @param object A pctable object 
##' @param colpct Default TRUE: should column percents be included
##' @param rowpct Default FALSE: should row percents be included
##' @param ... Other arguments, currently unused
##' @return An object of class summary.pctable
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @method summary pctable
##' @export
summary.pctable <- function(object, ..., colpct = TRUE, rowpct = FALSE)
{
    colpct <- if (missing(colpct)) object$call[["colpct"]] else colpct 
    rowpct <- if (missing(rowpct)) object$call[["rowpct"]] else rowpct
    
    count <- object[["count"]]
   
    t3 <- count
    attr(t3, which = "colpct") <- colpct
    attr(t3, which = "rowpct") <- rowpct
    class(t3) <- c("summary.pctable", "table")
    if (colpct && !rowpct) {
        cpct <- object[["cpct"]]
        for(j in rownames(cpct)){
            for(k in colnames(cpct)){
                if(j != "" & k != ""){
                    t3[j, k] <- paste0(count[j, k], "(", cpct[j, k], "%)")
                }
            }
        }
        return(t3)
    }
    
    ## rowpct == TRUE else would have returned
    rpct <- object[["rpct"]]
    for(j in rownames(rpct)){
        for(k in colnames(rpct)){
            if (j != "" & k != ""){
                t3[j, k] <- paste0(count[j, k], "(", rpct[j, k], "%)")
            }
        }
    }
    
    if (!colpct) {
        return(t3)
    } else {
        cpct <- object[["cpct"]]
        t4 <- array("", dim = c(1, 1) + c(2,1)*dim(object$cpct))
        t4[seq(1, NROW(t4), 2), ] <- t3
        rownames(t4)[seq(1, NROW(t4), 2)] <- rownames(t3)
        rownames(t4)[is.na(rownames(t4))] <- "" 
        colnames(t4) <- colnames(t3)
        for(j in rownames(object[["cpct"]])) {
            for(k in colnames(object[["cpct"]])){
                if(j != "" & k != "") {
                    t4[1 + which(rownames(t4) == j) ,k] <- paste0(cpct[j, k], "%")
                }
            }
        }
        t4 <- as.table(t4)
        names(dimnames(t4)) <- names(dimnames(count))
        attr(t4, which = "colpct") <- colpct
        attr(t4, which = "rowpct") <- rowpct
        class(t4) <- c("summary.pctable", "table")
        return(t4)
    }
}

##' print method for summary.pctable objects 
##'
##' prints pctab objects. Needed only to deal properly with quotes
##' 
##' @param x a summary.pctable object
##' @param ... Other arguments to print method
##' @return Nothing is returned
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @method print summary.pctable
##' @export
print.summary.pctable <- function(x, ...){
    colpct <- attr(x, "colpct")
    rowpct <- attr(x, "rowpct")
    
    if (colpct && !rowpct) {
        cat("Count (column %)\n")
    } else if (!colpct && rowpct) {
        cat("Count (row %)\n")
    } else {
        cat("Count (row %)\n")
        cat("column %\n")
    }
    NextMethod(generic = "print", object = x, quote = FALSE, ...)
    ## Error in NextMethod(generic = "print", object = x, quote = FALSE, ...) :
    ## no calling generic was found: was a method called directly?
    ## Calls: pctable ... print.pctable -> print.summary.pctable -> NextMethod
    ## Execution halted
    ## print.table(x, quote = FALSE, ...)
}


##' Display pctable objects
##'
##' This is not very fancy. Note that the saved pctable object
##' has the information inside it that is required to write both
##' column and row percentages. The arguments colpct and rowpct
##' are used to ask for the two types.
##' 
##' @param x A pctable object
##' @param colpct Default TRUE: include column percentages?
##' @param rowpct Default FALSE: include row percentages?
##' @param ... Other arguments passed through to print method
##' @return A table object for the final printed table.
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @method print pctable
##' @export
print.pctable <- function(x, colpct = TRUE, rowpct = FALSE, ...)
{
    colpct <- if (missing(colpct)) x$call[["colpct"]] else colpct 
    rowpct <- if (missing(rowpct)) x$call[["rowpct"]] else rowpct
    tab <- summary(x, colpct = colpct, rowpct = rowpct)
    print(tab, ...)
    invisible(tab)
}
NULL

##' makeVec for checking or creating vectors
##'
##' This is a convenience for handling function arguments. If x is a
##' single value, it makes a vector of length d in which all values
##' are equal to x. If x is a vector, check that its length is d.
##'
##' @param x A single value or a vector
##' @param d An integer, the desired size of the vector
##' @return A vector of length d
##' @author Paul E. Johnson <pauljohn@@ku.edu>
makeVec <-
    function(x = NULL, d = NULL)
{
    if (length(x) == 1) {
        x <- rep(x, d) #assign same for all
    } else if (length(x) != d){
        msg <- paste(deparse(substitute(x)), "has too",
                     if(length(x) > d) "many" else "few",
                     "elements. Please specify", d, "elements.
                     Or just specify 1, we will use that one value to",
                     "manufacture a vector for you.")
        stop(msg)
    }
    x
}
NULL

##' Create sequences for plotting
##'
##' \code{plotSeq} is a convenience for the creation of sequence
##' across the range of a variable.
##' By default, the length of the plotting
##' sequence will be equal to the length of the original sequence.
##' In that case, the only effect is to create an evenly-spaced
##' set of values. If \code{length.out} is specified, the user
##' determines the number of elements in plotSeq.
##'
##' The primary intended usage is for the creation of
##' plotting sequences of numeric variables.  It takes
##' a variable's range and the fills in evenly spaced steps.
##' If x is a factor variable, the levels will be returned.
##' Uses of this functionality are planned in the future.
##' @usage plotSeq(x, length.out = length(x))
##' @param x an R vector variable
##' @param length.out the number of elements in the desired plotting sequence.
##' @export plotSeq
##' @seealso \code{pretty}
##' @examples
##' #Create a quadratic regression
##'
##' stde <- 14
##' x <- rnorm(100, m = 50, s = 10)
##' y <- 0.2 - 02*x + 0.2*x^2 + stde*rnorm(100)
##' mod1 <- lm (y ~ poly(x, 2))
##'
##' plot(x, y, main="The Quadratic Regression")
##' seqx <- plotSeq(x, length.out = 10)
##' seqy <- predict(mod1, newdata = data.frame(x = seqx))
##' lines(seqx, seqy, col = "red")
##'
##' # Notice the bad result when a plotting sequence is
##' # not used.
##' plot(x, y, main = "Bad Plot Result")
##' seqy <- predict(mod1)
##' lines(x, seqy, col = "green")
##'
plotSeq <-
    function (x, length.out = length(x))
{
  if (is.numeric(x)) {
    xr <- range(x, na.rm = TRUE)
    pseq <- seq(xr[1], xr[2], length.out = length.out)
    return(pseq)
  } else {
    if (is.factor(x)) {
      pseq <- levels(x)
      return(pseq)
    } else {
      stop("plotSeq can only create plotting sequences for numeric or factor variables")
    }
  }
}
NULL


##' Select most frequently occurring values from numeric or categorical variables.
##'
##' The "n" most frequently occurring values are returned, sorted by
##' frequency of occurrence (in descending order). The names attribute
##' includes information about the percentage of cases that have the
##' indicated values.
##'
##' This is used by plotSlopes, plotCurves, and other "newdata" making
##' functions.
##' @param x A numeric or character variable
##' @param n The maximum number of values that may be returned.
##' @param pct Default = TRUE. Include percentage of responses within each category
##' @return A named vector.
##' @export
##' @author Paul E. Johnson <pauljohn@@ku.edu>
cutByTable <-
    function(x, n = 5, pct = TRUE)
{
    table1 <- table(x)
    n <- min(n, length(names(table1)))
    table1 <- sort(table1, decreasing = T)
    table1cut <- table1[1:n]
    tabNames <- names(table1cut)
    res <- if(is.factor(x)){
        as.factor(tabNames)
    } else if (is.numeric(x)){
        as.numeric(tabNames)
    } else {
        tabNames
    }
    if (pct) {
        freq <- 100*round(table1cut/sum(table1), 2)
        names(res) <- paste(tabNames," (",freq,"%)", sep="")
    } else {
        names(res) <- tabNames
    }
    if (is.factor(x)) res <- factor(res, levels = levels(x))
    res
}
NULL


##' Calculates the "center" quantiles, always including the median, when n is odd.
##'
##' If the numeric variable has fewer than 6 unique observed values,
##' this will send the data to \code{cutByTable}.  The default return
##' will find dividing points at three quantiles: c(0.25, 0.50, 0.75)
##' If n=4, the dividing points will be c(0.20, 0.40, 0.60, 0.80) If
##' n=5, c(0.0, 0.25, 0.50, 0.75, 1.0) Larger n that are odd will
##' include 0.5 and evenly spaced points out to proportions 0 and
##' 1.0. Larger n that is even will return evenly spaced points
##' calculated by R's \code{pretty} function.
##' @param x A numeric vector.
##' @param n The number of quantile points. See details.
##' @return A vector
##' @export
##' @importFrom stats quantile
##' @author Paul E. Johnson <pauljohn@@ku.edu>
cutByQuantile <-
    function(x, n = 3)
{
    uniqueVals <- unique(x)
    n <- as.integer(n) ##defend self against fractions
    if (length(uniqueVals) < 6) {
        qs <- cutByTable(x, n)
        invisible(qs)
    } else {
        cutVector <- if(n == 1) {
            c(0.50)
        } else if(n == 2) {
            c(0.25, 0.75)
        } else if(n == 3) {
            c(0.25, 0.50, 0.75)
        } else if(n == 4) {
            c(0.20, 0.40, 0.60, 0.80)
        } else if(n == 5) {
            c(0.00, 0.25, 0.50, 0.75, 1.0)
        } else if (n > 5) {
            if (n %% 2 == 0) {
                g <- 0.5 / n %/% 2
                c(seq(0, 0.5-g, by = g), 0.5, seq(0.5+g, 1.0, by = g))
            } else {
                pretty(c(0, 1), n = n-1)
            }
        }
        qs <- quantile(x, probs = cutVector, na.rm = TRUE)
        invisible(qs)
    }
}
NULL

##' Returns center values of x, the mean, mean-std.dev, mean+std.dev
##'
##' If the numeric variable has fewer than 6 unique observed values,
##' this will send the data to cutByTable.
##' @return A named vector
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @param x A numeric variable
##' @param n Should be an odd number 1, 3, 5, or 7. If 2 < n < 5,
##' values that divide the data at c(m-sd, m, m+sd) are returned. If n
##' > 4, the returned values are c(m-2sd, m-sd, m, m+sd, m+2sd).
##' @export
##' @importFrom stats sd
##' @examples
##' x <- rnorm(100, m = 100, s = 20)
##' cutBySD (x, n = 3)
##' cutBySD (x, n = 5)
cutBySD <-
    function(x, n = 3)
{
    uniqueVals <- unique(x)
    if (length(uniqueVals) < 6) {
        qs <- cutByTable(x, n)
        invisible(qs)
    } else {
        mx <- round(mean(x, na.rm = TRUE),2)
        sdx <- round(sd(x, na.rm = TRUE),2)
        if (n == 1) {
            qs <- c( mx )
            suffix <- c("(m)")
        } else if (n == 2) {
            qs <- c(mx - sdx, mx + sdx)
            suffix <- c("(m-sd)", "(m+sd)")
        } else if (n <= 4) {
            qs <- c(mx - sdx, mx, mx + sdx)
            suffix <- c("(m-sd)", "(m)", "(m+sd)")
        } else if (n <= 6){
            qs <- c(mx - 2*sdx, mx - sdx, mx, mx + sdx, mx + 2*sdx)
            suffix <- c("(m-2sd)","(m-sd)","(m)","(m+sd)","(m+2sd)")
        }  else {
            qs <- c(mx - 3*sdx, mx - 2*sdx, mx - sdx, mx, mx + sdx, mx + 2*sdx, mx + 3*sdx)
            suffix <- c("(m-3sd)", "(m-2sd)", "(m-sd)", "(m)", "(m+sd)", "(m+2sd)", "(m+3sd)")
        }
        names(qs) <-  paste(suffix)
        invisible(qs)
    }
}
NULL




##' Select focal values from an observed variable.
##'
##' This is a generic function with 2 methods, getFocal.default
##' handles numeric variables, while getFocal.factor handles
##' factors. No other methods have been planned for preparation.
##'
##' This is used in functions like \code{plotSlopes} or
##' \code{plotCurves}.
##'
##' @param x Required. A variable
##' @param ... Other arguments that will be passed to the
##' user-specified xvals function.
##' @return A vector.
##' @rdname getFocal
##' @export
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##'
getFocal <-
    function(x, ...)
{
    UseMethod("getFocal")
}
NULL

##' Select focal values from a numeric variable
##'
##' Many plotting functions need to select "focal" values from
##' a variable. There is a family of functions that are used
##' to do that.  User requests can be accepted in a number
##' of ways. Numeric and character variables will be treated
##' differently. Please see details.
##'
##' If \code{xvals} is not provided, a default divider for numeric
##' variables will be the algorithm "quantile". The divider algorithms
##' provided with rockchalk are c("quantile", "std.dev.", "table",
##' "seq").  \code{xvals} can also be the name of a user-supplied
##' function, such as R's \code{pretty()}. If the user supplies a
##' vector of possible values, that selection will be checked to make
##' sure all elements are within a slightly expanded range of
##' \code{x}. If a value out of range is requested, a warning is
##' issued. Maybe that should be an outright error?
##' 
##' With factor variables, \code{xvals} is generally not used because
##' the only implemented divider algorithm is "table" (see
##' \code{cutByTable}), which selects the \code{n} most frequently
##' observed values. That is the default algorithm. It is legal to
##' specify xvals = "table", but there is no point in doing
##' that. However, xvals may take two other formats. It may be a
##' user-specified function that can select levels values from
##' \code{x} or it may be a vector of labels (or, names of
##' levels). The purpose of the latter is to check that the requested
##' levels are actually present in the supplied data vector
##' \code{x}. If the levels specified are not in the observed
##' variable, then this function stops with an error message.
##'
##' @param xvals A function name (either "quantile", "std.dev.",
##' "table", or "seq") or a user-supplied function that can receive
##' x and return a selection of values.
##' @param n Number of values to be selected.
##' @param pct Default TRUE. Include percentage of observed cases
##' in variable name? (used in legends)
##' @return A named vector of values.
##' @export
##' @method getFocal default
##' @rdname getFocal
##' @examples
##' x <- rnorm(100)
##' getFocal(x)
##' getFocal(x, xvals = "quantile")
##' getFocal(x, xvals = "quantile", n = 5)
##' getFocal(x, xvals = "std.dev")
##' getFocal(x, xvals = "std.dev", n = 5)
##' getFocal(x, xvals = c(-1000, 0.2, 0,5))
getFocal.default <- function(x, xvals = NULL, n = 3, pct = TRUE, ...)
{
    xRange <- magRange(range(x, na.rm = TRUE), 1.1)

    if (is.null(xvals))
        return(xfocal <- cutByQuantile(x, n))

    if (is.numeric(xvals)) {
        if ((max(xvals) > max(xRange)) || (min(xvals) < min(xRange))){
            warning("values requested out of observed range in getFocal")
        }
        xvals <- sort(xvals)
        return(xvals)
    }

    if (is.character(xvals)) {
        xvals <- match.arg(tolower(xvals),
                           c("quantile", "std.dev.","table", "seq"))
        xfocal <- switch(xvals,
                         table = cutByTable(x, n, pct = pct),
                         quantile = cutByQuantile(x, n),
                         "std.dev." = cutBySD(x, n),
                         seq = plotSeq(x, n),
                         stop("unknown 'xvals' algorithm in getFocal"))
        return(xfocal)
    }

    if (is.function(xvals)) {
        return(xvals(x, n, ...))
    }

    stop("getFocal received unexpected input for xvals")
}
NULL

##' @export
##' @method getFocal factor
##' @rdname getFocal
##' @examples
##' x <- factor(c("A","B","A","B","C","D","D","D"))
##' getFocal(x)
##' getFocal(x, n = 2)
##'
getFocal.factor <- function(x, xvals = NULL, n = 3, pct = TRUE, ...)
{
    ## drop unused levels from x
    x <- droplevels(x)
    if (is.null(xvals)) {
        xvals <- cutByTable(x, n, pct = pct)
        return(xvals)
    }
    if (is.vector(xvals)) {
       x <- factor(x) ## drop unused levels
       if (!all(xvals %in% levels(x))) stop("xvals includes non-observed levels of x")
       return(xvals)
    }
    if (is.function(xvals)) {
        xvals <- xvals(x, n, ...)
        return(xvals)
    }

    if (is.character(x)) {
        xvals <- match.arg(tolower(xvals),
                           c("table"))
        xvals <- switch(xvals,
                        table = cutByTable(x, n),
                        stop("Sorry, only known algorithm for factors is 'table'"))
        return(xvals)
    }

    if (is.function(xvals)) {
        return(xvals(x, n, ...))
    }

    stop("getFocal received unexpected input xvals")
}
NULL

##' @export
##' @method getFocal character
##' @rdname getFocal
##' @examples
##' x <- c("A","B","A","B","C","D","D","D")
##' getFocal(x)
##' getFocal(x, n = 2)
##'
getFocal.character <- function(x, xvals = NULL, n = 3, pct = TRUE, ...)
{
    if (is.null(xvals)) {
        xvals <- cutByTable(x, n, pct = pct)
        return(xvals)
    }
    if (is.vector(xvals)) {
       if (!all(xvals %in% unique(x))) stop("xvals includes non-observed values of x")
       return(xvals)
    }
    if (is.function(xvals)) {
        xvals <- xvals(x, n, ...)
        return(xvals)
    }

    xvals <- match.arg(tolower(xvals),
                       c("table"))
    xvals <- switch(xvals,
                    table = cutByTable(x, n, pct = pct),
                    stop("Sorry, only known algorithm for character variables is 'table'"))
    xvals
}
NULL

##' Pad with 0's.
##'
##' Sometimes we receive this c(1, 22, 131) and we need character
##' variables of the same size, such as c("001", "022", "131").  This
##' happens if a user has mistakenly converted a zip code (US regional
##' identifier) like "00231" to a number. This function converts the
##' number back to a 0 padded string.
##'
##' This works differently if the number provided is an integer, or a
##' character string.  Integers are left padded with the character
##' "0".  A character string will be left-padded with blanks.
##' @param x a numeric  variable.
##' @return A character string vector padded with 0's
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' x <- c(1 , 11, 22, 121, 14141, 31)
##' (xpad <- padW0(x))
##' x <- rpois(7, lambda = 11)
##' (xpad <- padW0(x))
##' x <- c("Alabama", "Iowa", "Washington")
padW0 <- function(x)
{
    vtype <- "d"
    if (is.character(x)) vtype <- "s"
    sprintf(paste("%0", max(nchar(as.character(x))), vtype, sep=""), x)
}
NULL

##' A way of checking if a string is a valid file name.
##'
##' A copy from R's grDevices:::checkIntFormat because it is not
##' exported there
##' @param s An integer
##' @return logical: TRUE or FALSE
##' @author R Core Development Team
checkIntFormat <- function(s)
{
    s <- gsub("%%", "", s)
    if (length(grep("%", s)) == 0L)
        return(TRUE)
    s <- sub("%[#0 ,+-]*[0-9.]*[diouxX]", "", s)
    length(grep("%
", s)) == 0L
}
NULL

##' Generates a data frame for regression analysis
##'
##' The output is a data frame (x1, x2, y) with user-specified
##' correlation between x1 and x2. The y (output) variable is created
##' according to the equation\cr
##' \deqn{
##' y = beta1 + beta2 * x1 + beta3 * x2 + beta4 * x1 * x2 + e.
##' }
##' The arguments determine the scales of the X matrix, the random
##' error, and the slope coefficients.
##'
##' The vector (x1,x2) is drawn from a multivariate normal
##' distribution in which the expected value (argument \code{means}).
##' The covariance matrix of X is
##' built from the standard deviations (\code{sds})
##' and the specified correlation between x1 and x2 (\code{rho}).
##' It is also necessary to specify the standard deviation
##' of the error term (\code{stde}) and the coefficients
##' of the regression equation (\code{beta}).
##'
##' @param N Number of cases desired
##' @param means 2-vector of means for x1 and x2
##' @param sds 2-vector of standard deviations for x1 and x2
##' @param rho Correlation coefficient for x1 and x2
##' @param stde standard deviation of the error term in the data
##' generating equation
##' @param beta beta vector of at most 4 coefficients for intercept,
##' slopes, and interaction
##' @export genCorrelatedData
##' @importFrom stats rnorm
##' @examples
##' ## 1000 observations of uncorrelated x1 and x2 with no
##' ## interaction between x1 and x2
##' dat <- genCorrelatedData(N=1000, rho=0, beta=c(1, 1.0, -1.1, 0.0))
##'   mcGraph1(dat$x1, dat$x2, dat$y, theta=20, phi=8,
##'   ticktype="detailed", nticks=10)
##' m1 <- lm(y ~ x1 + x2, data = dat)
##' plotPlane(m1, plotx1 = "x1", plotx2 = "x2")
##'
genCorrelatedData <-
    function(N = 100, means = c(50,50), sds = c(10,10), rho = 0.0, stde = 1,
             beta = c(0, 0.2, 0.2, 0.0))
{
  if (length(beta)> 4) stop("beta vector can have at most 4 values")
  corr.mat <- matrix(c(1,rho,rho,1), nrow = 2)
  sigma <- diag(sds) %*% corr.mat %*% diag(sds)
  x.mat <-  rockchalk::mvrnorm(n = N, mu = means, Sigma = sigma)
  y = beta[1] + beta[2] * x.mat[,1] + beta[3] * x.mat[,2] + beta[4] * x.mat[,1] * x.mat[ ,2] +  stde*rnorm (N, mean = 0, sd = 1)
  dat <- data.frame(x.mat, y)
  names(dat) <- c("x1", "x2", "y")
  dat
}
NULL

##' Generates a data frame for regression analysis.
##'
##' Unlike \code{genCorrelatedData}, this new-and-improved
##' function can generate a data frame with as many predictors
##' as the user requests along with an arbitrarily complicated
##' regression formula.  The result will be a data frame with
##' columns named (y, x1, x2, ..., xp).
##'
##' Arguments supplied must have enough information so that an
##' N x P matrix of predictors can be constructed.
##' The matrix X is drawn from a multivariate normal
##' distribution, the expected value vector (mu vector) is given by
##' the \code{means} and the var/covar matrix (Sigma) is
##' built from user supplied standard deviations \code{sds}
##' and the correlations between the columns of X, given by \code{rho}.
##' The user can also set the standard deviation
##' of the error term (\code{stde}) and the coefficients
##' of the regression equation (\code{beta}).
##'
##' If called with no arguments, this creates a data frame with
##' X ~ MVN(mu = c(50,50,50), Sigma = diag(c(10,10,10))).
##' y = X %*% c(0.15, 0.1, -0.1) + error, where error
##' is N(m = 0, sd = 200). All of these details can be
##' changed by altering the arguments.
##'
##' The y (output) variable is created according to the
##' equation\cr
##' \deqn{
##' y = b1 + b2 * x1 + ...+ bk * xk + b[k+1] * x1 * ...interactions.. + e}
##' \cr
##' For shorthand, I write b1 for beta[1], b2 for beta[2], and so forth.\cr
##'
##' The first P+1 arguments in the argument beta are the coefficients
##' for the intercept and the columns of the X matrix.  Any additional
##' elements in beta are the coefficients for nonlinear and interaction terms.\cr
##'
##' Those additional values in the beta vector are completely
##' optional. Without them, the true model is a linear
##' regression. However, one can incorporate the effect of squared terms
##' (conceptualize that as x1 * x1, for example) or interactions
##' (x1 * x2) easily.  This is easier to illustrate than describe.
##' Suppose there are 4 columns in X. Then a beta
##' vector like beta = c(0, 1, 2, 3, 4, 5, 6, 7, 8) would amount to
##' asking for\cr
##' \deqn{
##' y = 0 + 1 x1 + 2 x2 + 3 x3 + 4 x4 + 5 x1^2 + 6 x1 x2 + 7 x1 x3 + 8 x1 x4 + error
##' }
##' \cr
##' If beta supplies more coefficients, they are interpeted as additional
##' interactions.\cr
##'
##' When there are a many predictors and the beta vector is long, this
##' can become confusing. I think of this as a vech for the lower
##' triangle of a coefficient matrix. In the example with 4
##' predictors, beta[1:5] are used for the intercepts and slopes. The
##' rest of the  beta elements lay in like so:\cr
##'
##'    X1   X2  X3  X4\cr
##' X1 b6   .    .\cr
##' X2 b7   b10  .\cr
##' X3 b8   b11  b13\cr
##' X4 b9   b12  b14 b15\cr
##'
##' If the user only supplies b6 and b7, the rest are assumed  to  be 0.\cr
##'
##' To make this clear, the formula used to calculate y is printed to
##' the console when genCorrelatedData2 is called.
##'
##' @param N Number of cases desired
##' @param means P-vector of means for X. Implicitly sets the dimension of
##' the predictor matrix as N x P.
##' @param sds Values for standard deviations for columns of X. If
##' less than P values are supplied, they will be recycled.
##' @param rho Correlation coefficient for X. Several input formats
##' are allowed (see \code{lazyCor}). This can be a single number (common
##' correlation among all variables), a full matrix of correlations
##' among all variables, or a vector that is interpreted as the
##' strictly lower triangle (a vech).
##' @param stde standard deviation of the error term in the data
##' generating equation
##' @param beta beta vector of coefficients for intercept, slopes, and
##' interaction terma.  The first P+1 values are the
##' intercept and slope coefficients for the predictors. Additional
##' elements in beta are interpreted as coefficients for nonlinear and
##' interaction coefficients.  I have decided to treat these as a
##' column (vech) that fills into a lower triangular matrix. It
##' is easy to see what's going on if you run the examples. There
##' is also explanation in Details.
##' @param intercept Default FALSE. Should the predictors include an
##' intercept?
##' @param verbose TRUE or FALSE. Should information about the data
##' generation be reported to the terminal?
##' @return A data matrix that has columns c(y, x1, x2, ..., xP)
##' @export
##' @examples
##' ## 1000 observations of uncorrelated X with no interactions
##' set.seed(234234)
##' dat <- genCorrelatedData2(N = 10, rho = 0.0, beta = c(1, 2, 1, 1),
##'     means = c(0,0,0), sds = c(1,1,1), stde = 0)
##' summarize(dat)
##' ## The perfect regression!
##' m1 <- lm(y ~ x1 + x2 + x3, data = dat)
##' summary(m1)
##'
##' dat <- genCorrelatedData2(N = 1000, rho = 0,
##'     beta = c(1, 0.2, -3.3, 1.1), stde = 50)
##' m1 <- lm(y ~ x1 + x2 + x3, data = dat)
##' summary(m1)
##' predictOMatic(m1)
##' plotCurves(m1, plotx = "x2")
##'
##' ## interaction between x1 and x2
##' dat <- genCorrelatedData2(N = 1000, rho = 0.2,
##'     beta = c(1, 1.0, -1.1, 0.1, 0.0, 0.16), stde = 1)
##' summarize(dat)
##' ## Fit wrong model? get "wrong" result
##' m2w <- lm(y ~ x1 + x2 + x3, data = dat)
##' summary(m2w)
##' ## Include interaction
##' m2 <- lm(y ~ x1 * x2 + x3, data = dat)
##' summary(m2)
##'
##' dat <- genCorrelatedData2(N = 1000, rho = 0.2,
##'     beta = c(1, 1.0, -1.1, 0.1, 0.0, 0.16), stde = 100)
##' summarize(dat)
##' m2.2 <- lm(y ~ x1 * x2 + x3, data = dat)
##' summary(m2.2)
##'
##' dat <- genCorrelatedData2(N = 1000, means = c(100, 200, 300, 100),
##'     sds = 20,  rho = c(0.2, 0.3, 0.1, 0, 0, 0),
##'     beta = c(1, 1.0, -1.1, 0.1, 0.0, 0.16, 0, 0, 0.2, 0, 0, 1.1, 0, 0, 0.1),
##'     stde = 200)
##' summarize(dat)
##' m2.3w <- lm(y ~ x1 + x2 + x3, data = dat)
##' summary(m2)
##'
##' m2.3 <- lm(y ~ x1 * x2 + x3, data = dat)
##' summary(m2.3)
##'
##' predictOMatic(m2.3)
##' plotCurves(m2.3, plotx = "x1", modx = "x2", modxVals = "std.dev.", n = 5)
##'
##' simReg <- lapply(1:100, function(x){
##'     dat <- genCorrelatedData2(N = 1000, rho = c(0.2),
##'         beta = c(1, 1.0, -1.1, 0.1, 0.0, 0.46), verbose = FALSE)
##'     mymod <- lm (y ~ x1 * x2 + x3, data = dat)
##'     summary(mymod)
##' })
##'
##' x3est <- sapply(simReg, function(reg) {coef(reg)[4 ,1] })
##' summarize(x3est)
##' hist(x3est,  breaks = 40, prob = TRUE,
##'     xlab = "Estimated Coefficients for column x3")
##'
##' r2est <- sapply(simReg, function(reg) {reg$r.square})
##' summarize(r2est)
##' hist(r2est,  breaks = 40, prob = TRUE, xlab = "Estimates of R-square")
##'
##' ## No interaction, collinearity
##' dat <- genCorrelatedData2(N = 1000, rho = c(0.1, 0.2, 0.7),
##'     beta = c(1, 1.0, -1.1, 0.1), stde = 1)
##' m3 <- lm(y ~ x1 + x2 + x3, data = dat)
##' summary(m3)
##'
##' dat <- genCorrelatedData2(N=1000, rho=c(0.1, 0.2, 0.7),
##'     beta = c(1, 1.0, -1.1, 0.1), stde = 200)
##' m3 <- lm(y ~ x1 + x2 + x3, data = dat)
##' summary(m3)
##' mcDiagnose(m3)
##'
##' dat <- genCorrelatedData2(N = 1000, rho = c(0.9, 0.5, 0.4),
##'     beta = c(1, 1.0, -1.1, 0.1), stde = 200)
##' m3b <- lm(y ~ x1 + x2 + x3, data = dat)
##' summary(m3b)
##' mcDiagnose(m3b)
##'
genCorrelatedData2 <-
    function(N = 100, means = c(50,50,50), sds = c(10,10,10),
             rho = c(0.0, 0.0, 0.0), stde = 100,
             beta = c(0, 0.15, 0.1, -0.1), intercept = FALSE, 
             verbose = TRUE)
{
    d <- length(means)
    x.mat <- as.matrix(genX(N, means, sds, rho, intercept = TRUE))
    ## Get rid of Intercept if it is in there
    x.mat.noint <- x.mat[ , -grep("Intercept", colnames(x.mat))]
    
    beta1 <- beta[1:(d+1)]
    beta2 <- beta[-(1:(d+1))]

    ## pad beta2 with 0's on back end so it is right size to go into
    ## triangular matrix
    beta2 <- c(beta2, rep(0, times =(d*(d+1)/2) - length(beta2)))

    Bmat2 <- matrix(0, nrow = d, ncol = d)
    Bmat2[lower.tri(Bmat2, diag = TRUE)] <- beta2

    ##Would have been easier if I studied sparse matrix entry and usage.
    ## TODO 2013-05-16: This does a lot of calculations on zeroes.
    ## May enhance effiency to select columns before multiplying.
    intEffects <-  sapply(1:d, function(j) {
        if(!any( Bmat2[ ,j] != 0)) {
            return()
        }
        intEff <-  (x.mat.noint * x.mat.noint[ , j]) %*% Bmat2[ , j]
    })

    intEffects <- do.call("cbind", intEffects)

    y = x.mat %*% beta1 +  stde*rnorm (N, mean = 0, sd = 1)

    if (!is.null(intEffects)) y = y + rowSums(intEffects)

    dat <- if (!intercept){
               data.frame(y, x.mat.noint)
           } else {
               data.frame(y, x.mat)
           }
    
    if (verbose == TRUE){
        x.names <- colnames(x.mat.noint)  
        print("The equation that was calculated was")
        cat("y =",  beta1[1], "+", paste(beta1[2:(d+1)] , c(x.names), collapse = " + ", sep = "*"), "\n" ,
            "+ ")
        for(i in seq(d)){
            cat(paste(Bmat2[, i], x.names, x.names[i], collapse = " + ", sep = "*"), "\n",
                "+ ")
        }
        cat(paste("N(0,", stde, ") random error \n", sep = ""))
    }

    dat
}
NULL

##' Generate correlated data (predictors) for one unit
##'
##' This is used to generate data for one unit. It is recently
##' re-designed to serve as a building block in a multi-level data
##' simulation exercise.  The new arguments "unit" and "idx" can be
##' set as NULL to remove the multi-level unit and row naming
##' features.  This function uses the rockchalk::mvrnorm function, but
##' introduces a convenience layer by allowing users to supply
##' standard deviations and the correlation matrix rather than the
##' variance.
##'
##' Today I've decided to make the return object a data frame.  This
##' allows the possibility of including a character variable "unit"
##' within the result.  For multi-level models, that will help.  If
##' unit is not NULL, its value will be added as a column in the data
##' frame. If unit is not null, the rownames will be constructed by
##' pasting "unit" name and idx. If unit is not null, then idx will
##' be included as another column, unless the user explicitly sets
##' idx = FALSE. 
##'
##' @param N Number of cases desired
##' @param means A vector of means for p variables. It is optional to
##'     name them.  This implicitly sets the dimension of the
##'     predictor matrix as N x p. If no names are supplied, the
##'     automatic variable names will be "x1", "x2", and so forth. If
##'     means is named, such as c("myx1" = 7, "myx2" = 13, "myx3" =
##'     44), those names will be come column names in the output
##'     matrix.
##' @param sds Standard deviations for the variables.  If less than p
##'     values are supplied, they will be recycled.
##' @param rho Correlation coefficient for p variables. Several input
##'     formats are allowed (see \code{lazyCor}). This can be a single
##'     number (common correlation among all variables), a full matrix
##'     of correlations among all variables, or a vector that is
##'     interpreted as the strictly lower triangle (a vech).
##' @param Sigma P x P variance/covariance matrix.
##' @param intercept Default = TRUE, do you want a first column filled
##'     with 1?
##' @param col.names Names supplied here will override column names
##'     supplied with the means parameter. If no names are supplied
##'     with means, or here, we will name variables x1, x2, x3,
##'     ... xp, with Intercept at front of list if intercept =
##'     TRUE.
##' @param unit A character string for the name of the unit being
##'     simulated. Might be referred to as a "group" or "district" or
##'     "level 2" membership indicator.
##' @param idx If set TRUE, a column "idx" is added, numbering the
##'     rows from 1:N. If the argument unit is not NULL, then idx is
##'     set to TRUE, but that behavior can be overridded by
##'     setting idx = FALSE.
##' @export
##' @return A data frame with rownames to specify unit and
##'     individual values, including an attribute "unit" with the
##'     unit's name.
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' X1 <- genX(10, means = c(7, 8), sds = 3, rho = .4)
##' X2 <- genX(10, means = c(7, 8), sds = 3, rho = .4, unit = "Kansas")
##' head(X2)
##' X3 <- genX(10, means = c(7, 8), sds = 3, rho = .4, idx = FALSE, unit = "Iowa")
##' head(X3)
##' X4 <- genX(10, means = c("A" = 7, "B" = 8), sds = c(3), rho = .4)
##' head(X4)
##' X5 <- genX(10, means = c(7, 3, 7, 5), sds = c(3, 6),
##'             rho = .5, col.names = c("Fred", "Sally", "Henry", "Barbi"))
##' head(X5)
##' Sigma <- lazyCov(Rho = c(.2, .3, .4, .5, .2, .1), Sd = c(2, 3, 1, 4))
##' X6 <- genX(10, means = c(5, 2, -19, 33), Sigma = Sigma, unit = "Winslow_AZ")
##' head(X6)
##'
genX <-
    function(N, means, sds, rho, Sigma = NULL, intercept = TRUE,
             col.names = NULL, unit = NULL, idx = FALSE)
{
    d <- length(means)
    if (!missing(rho) & !is.null(Sigma))
        stop ("Please provide rho or Sigma, not both")
    if (!is.null(Sigma) & !missing(sds))
        warning(paste("Note, if you provide Sigma we are ignoring",
                      " your input on the sds argument"))
    if ((!is.null(Sigma)) && (d != dim(Sigma)[1]))
        stop (paste("Sigma must have same number of rows",
                    "& columns as there are means"))
    if (is.null(Sigma)){
        R <- lazyCor(rho, d)
        if (length(sds) < d) sds <- rep(sds, length.out = d)
        Sigma <- lazyCov(Rho = R, Sd = sds)
    }
    if (missing(col.names) && is.null(names(means))) {
        col.names <- paste0("x", 1:d)
    } else if (missing(col.names) && !is.null(names(means))){
        col.names <- names(means)
    } else {
        if (length(col.names) != d)
            stop(paste("If you provide column names, please provide one",
                       "for each column in the means input"))
    }
    col.names <- make.names(col.names, unique = TRUE)

    ## If unit is meaningful, set idx TRUE. Defending against
    ## user values like unit = NULL and idx = NULL
    if (missing(idx) || (!identical(idx, FALSE) && !is.null(idx))){ 
        if (!missing(unit) & !is.null(unit))
            idx <- TRUE
    }

    x.mat <- rockchalk::mvrnorm(N, means, Sigma)
    if (is.null(unit)) {
        rownames <- 1:N
    } else {
        if (length(unit) > 1) stop("genX: just supply 1 unit name")
        rownames <- paste0(unit, "_", 1:N)
    }
    dimnames(x.mat) <- list(rownames, col.names)
    x.mat <- as.data.frame(x.mat)
    if (intercept) x.mat <- cbind(Intercept = 1L, x.mat)
    
    if (!is.null(unit)) x.mat[ , "unit"] <- rep(unit, N)
    if (isTRUE(idx)) x.mat[ , "idx"] <- seq(1L, N, by = 1L)
    attr(x.mat, "unit") <- unit
    x.mat
}
NULL
    



##' Generate correlated data for simulations (third edition)
##'
##' This is a revision of \code{genCorrelatedData2}.  The output is a
##' data frame that has columns for the predictors along with an error
##' term, the linear predictor, and the observed value of the outcome
##' variable.  The new features are in the user interface. It has a
##' better way to specify beta coefficients. It is also more flexible
##' in the specification of the names of the predictor columns.
##'
##' The enhanced methods for authors to specify a data-generating
##' process are as follows. Either way will work and the choice
##' between the methods is driven by the author's convenience.\cr
##' \itemize{
##' \item 1. Use the formula argument as a quoted string:
##' \code{"1 + 2.2 * x1 + 2.3 * x2 + 3.3 * x3 + 1.9 * x1:x2"}.
##' The "*" represents multiplication of coefficient times variable,
##' and the colon ":" has same meaning but it is used for products of variables.
##' \item 2. Use the beta argument with parameter names, \code{beta =
##' c("Intercept" = 1, x1 = 2.2, x2 = 2.3, x3 = 3.3, "x1:x2" = 1.9)}
##' where the names are the same as the names of the variables in the
##' formula. Names of the variables in the formula or the beta vector
##' should be used also in either the means parameter or the col.names
##' parameter.
##' }
##' 
##' The error distribution can be specified. Default is normal, with
##' draws provided by R's \code{rnorm}. All error models assume
##' \eqn{E[e] = 0} and the scale coefficient is the parameter
##' \code{stde}. Thus, the default setup's error will be drawn from
##' \code{rnorm(N, 0, stde)}. Any two parameter "location" and "scale"
##' distribution should work as well, as long as the first coefficient
##' is location (because we set that as 0 in all cases) and the second
##' argument is scale. For example, \code{distrib=rlogis}, will lead
##' to errors drawn from \code{rlogis(N, 0, stde)}. Caution: in rlogis,
##' the scale parameter is not the same as standard deviation.
##'
##' The only one parameter distribution currently supported is the T
##' distribution.  If user specifies \code{distrib=rt}, then the
##' \code{stde} is passed through to the parameter \code{df}. Note
##' that if increasing the stde parameter will cause the standard
##' deviation of \code{rt} to get smaller. \code{df=1} implies sd =
##' 794.6; \code{df=2} implies sd = 3.27; \code{df=3} implies 1.7773.
##'
##' Methods to specify error distributions in a more flexible way need
##' to be considered.
##' 
##' @param formula a text variable, e.g., \code{"y ~ 1 + 2*x1"}. Use
##'     ":" to create squared and interaction terms,
##'     \code{"y ~ 1 + 2*x1 + 1.1*x1:x1 + 0.2*x1:x2".} Multi-way
##'     interactions are allowed, eg
##'     \code{"y ~ 1 + 2*x1 + .4*x2 + .1*x3 + 1.1*x1:x1 + 0.2*x1:x2:x3".}
##'     Note author can specify any order of interation.
##' @param N sample size
##' @param means averages of predictors, can include names c(x1 = 10,
##'     x2 = 20) that will be used in the data.frame result.
##' @param sds standard deviations, 1 (common value for all variables)
##'     or as many elements as in \code{means}.
##' @param rho correlations, can be 1 or a vech for a correlation
##'     matrix
##' @param stde The scale of the error term. If \code{distrib=rnorm},
##'     stde is the standard deviation of the error term. If the user
##'     changes the distribution, this is a scale parameter that may
##'     not be equal to the standard deviation. For example,
##'     \code{distrib=rlogist} has a scale parameter such that a value
##'     of stde implies the error's standard deviation will be
##'     \eqn{stde * pi / sqrt(3)}.
##' @param beta slope coefficients, use either this or \code{formula},
##'     not both. It is easier (less error prone) to use named
##'     coefficients, but (for backwards compatability with
##'     \code{genCorrelatedData2}) names are not required. If named,
##'     use "Intercept" for the intercept coefficient, and use
##'     variable names that match the \code{xmeans} vector.  Un-named
##'     coefficients follow same rules as in
##'     \code{\link{genCorrelatedData2}}. The first (1 + p) values are
##'     for the intercept and p main effects.  With 3 predictors and
##'     no squares or interactions, specify four betas corresponding
##'     to \code{c(Intercept, x1, x2, x3)}. The squared and
##'     interaction terms may follow.  The largest possible model
##'     would correspond to \code{c(Intercept, x1, x2, x3, x1:x1,
##'     x1:x2, x1:x3, x2:x2, x2:x3, x3:x3)}.  Squares and interactions
##'     fill in a "lower triangle". The unnamed beta vector can be
##'     terminated with the last non-zero coefficient, the function
##'     will insert 0's for the coefficients at the end of the vector.
##' @param intercept TRUE or FALSE. Should the output data set include
##'     a column of 1's. If beta is an unnamed vector, should the
##'     first element be treated as an intercept?
##' @param col.names Can override names in means vector
##' @param verbose TRUE for diagnostics
##' @param ... extra arguments, ignored for now. We use that to ignore
##'     unrecognized parameters.
##' @param distrib An R random data generating function.  Default is
##'     \code{rnorm}. Also \code{rlogis} or any other two-parameter
##'     location/scale distribution will work. Special configuration
##'     allows \code{rt}. See details.
##' @return a data frame
##' @name genCorrelatedData3
##' @export genCorrelatedData3
##' @importFrom stats rnorm
##' @importFrom stats rt
##' @importFrom stats rlogis
##' @author Paul Johnson <pauljohn@@ku.edu> and Gabor Grothendieck <ggrothendieck@gmail.com>
##' @examples
##' set.seed(123123)
##' ## note: x4 is an unused variable in formula
##' X1a <-
##'     genCorrelatedData3("y ~ 1.1 + 2.1 * x1 + 3 * x2 + 3.5 * x3 + 1.1 * x1:x3",
##'                        N = 1000, means = c(x1 = 1, x2 = -1, x3 = 3, x4 = 1),
##'                        sds = 1, rho = 0.4, stde = 5)
##' lm1a <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X1a)
##' ## note that normal errors have std.error. close to 5
##' summary(lm1a)
##' attr(X1a, "beta") 
##' attr(X1a, "formula")
##' ## Demonstrate name beta vector method to provide named arguments
##' set.seed(123123)
##' X2 <- genCorrelatedData3(N = 1000, means = c(x1 = 1, x2 = -1, x3 = 3, x4 = 1),
##'           sds = 1, rho = 0.4, 
##'           beta = c("Intercept" = 1.1, x1 = 2.1, x2 = 3,
##'                     x3 = 3.5, "x1:x3" = 1.1),
##'                     intercept = TRUE, stde = 5)
##' attr(X2, c("beta"))
##' attr(X2, c("formula"))
##' head(X2)
##' lm2 <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X2)
##' summary(lm2)
##'
##' ## Equivalent with unnamed beta vector. Must carefully count empty
##' ## spots, fill in 0's when coefficient is not present. This
##' ## method was in genCorrelated2. Order of coefficents is
##' ## c(intercept, x1, ..., xp, x1:x1, x1:x2, x1:xp, x2:x2, x2:x3, ..., )
##' ## filling in a lower triangle.
##' set.seed(123123)
##' X3 <- genCorrelatedData3(N = 1000, means = c(x1 = 1, x2 = -1, x3 = 3, x4 = 1),
##'           sds = 1, rho = 0.4, 
##'           beta = c(1.1, 2.1, 3, 3.5, 0, 0, 0, 1.1),
##'                     intercept = TRUE, stde = 5)
##' attr(X3, c("beta"))
##' attr(X3, c("formula"))
##' head(X3)
##' lm3 <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X3)
##' summary(lm3)
##'
##' ## Same with more interesting variable names in the means vector
##' X3 <- genCorrelatedData3(N = 1000,
##'           means = c(friend = 1, enemy = -1, ally = 3, neutral = 1),
##'           sds = 1, rho = 0.4, 
##'           beta = c(1.1, 2.1, 3, 3.5, 0, 0, 0, 1.1),
##'                     intercept = TRUE, stde = 5)
##' head(X3)
##' attr(X3, c("beta"))
##' 
##' 
##' X3 <- genCorrelatedData3(N = 1000, means = c(x1 = 50, x2 = 50, x3 = 50),
##'           sds = 10, rho = 0.4,
##'           beta = c("Intercept" = .1, x1 = .01, x2 = .2, x3 = .5,
##'                    "x1:x3" = .1))
##' lm3 <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X3)
##' 
##' 
##' ## Names via col.names argument: must match formula
##' X2 <- genCorrelatedData3("y ~ 1.1 + 2.1 * educ + 3 * hlth + 3 * ses + 1.1 * educ:ses",
##'          N = 100, means = c(50, 50, 50, 20),
##'          sds = 10, rho = 0.4, col.names = c("educ", "hlth", "ses", "wght"))
##' str(X2) 
##'
##' X3 <- genCorrelatedData3("y ~ 1.1 + 2.1 * educ + 3 * hlth + 3 * ses + 1.1 * educ:ses",
##'          N = 100, means = c(50, 50, 50, 20),
##'          sds = 10, rho = 0.4, col.names = c("educ", "hlth", "ses", "wght"),
##'          intercept = TRUE)
##' str(X3)
##' 
##' ## note the logistic errors have residual std.error approximately 5 * pi/sqrt(3)
##' X1b <-
##'     genCorrelatedData3("y ~ 1.1 + 2.1 * x1 + 3 * x2 + 3.5 * x3 + 1.1 * x1:x3",
##'                        N = 1000, means = c(x1 = 1, x2 = -1, x3 = 3),
##'                        sds = 1, rho = 0.4, stde = 5, distrib = rlogis)
##' lm1b <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X1b)
##' summary(lm1b)
##'
##' ## t distribution is very sensitive for fractional df between 1 and 2 (recall
##' ## stde parameter is passed through to df in rt.
##' X1c <-
##'     genCorrelatedData3("y ~ 1.1 + 2.1 * x1 + 3 * x2 + 3.5 * x3 + 1.1 * x1:x3",
##'                        N = 1000, means = c(x1 = 1, x2 = -1, x3 = 3),
##'                        sds = 1, rho = 0.4, stde = 1.2, distrib = rt)
##' lm1c <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X1c)
##' summary(lm1c)
##' 
genCorrelatedData3 <- function (formula, N = 100,
                                means = c("x1" = 50, "x2" =  50, "x3" = 50),
                                sds = 10, rho = 0,
                                stde = 1,
                                beta = c(0, 0.15, 0.1, -0.1),
                                intercept = FALSE, col.names,
                                verbose = FALSE, ..., distrib = rnorm)
{

    ## From Gabor Grothendieck, r-help August 24, 2018:
    isChar <- function(e, ch) identical(e, as.symbol(ch))
    ## From Gabor Grothendieck, r-help August 24, 2018:
    ## Only works if formula is written out, as in x1 + x2 + x1:x2,
    ## not if it is abbreviated 
    Parse <- function(e) {
        if (length(e) == 1) {
            if (is.numeric(e)) return(e)
            else setNames(1, as.character(e))
        } else {
            if (isChar(e[[1]], "*")) {
                x1 <- Recall(e[[2]])
                x2 <- Recall(e[[3]])
                setNames(unname(x1 * x2), paste0(names(x1), names(x2)))
            } else if (isChar(e[[1]], "+")) c(Recall(e[[2]]), Recall(e[[3]]))
            else if (isChar(e[[1]], "-")) {
                if (length(e) == 2) -1 * Recall(e[[2]])
                else c(Recall(e[[2]]), -Recall(e[[3]]))
            } else if (isChar(e[[1]], ":")) setNames(1, paste(e[-1], collapse = ":"))
        }
    }

    ## If col.names does not include all elements of varnames, stop
    ## varnames comes from names(beta)
    ## Allow "Intercept" exception
    checkVarnames <- function(col.names, varnames){
        uniquenames <- unique(unlist(strsplit(varnames, "[:+*]")))
        col.names <- unique(c("Intercept", col.names))
        if (any(!uniquenames %in% col.names)){
            MESSG <- paste("formula uses variable not present in matrix:",
                           paste(uniquenames[!uniquenames %in% col.names], collapse = ","))
            stop(MESSG)
        } ## else do nothing
        TRUE
    }

    ## creates text (character variable) like a formula, not an R formula
    betanamestoformula <- function(beta){
        namefix <- gsub(":", "*", names(beta))
        namefix <- gsub("^\\s*$", "1", namefix)
        paste(beta, namefix, collapse = " + ", sep = "*")
    }

    
    d <- length(means)
    if (missing(col.names) && is.null(names(means))) {
        col.names <- paste0("x", 1:d)
    }
    else if (missing(col.names) && !is.null(names(means))) {
        col.names <- names(means)
    }
    else {
        if (length(col.names) != d)
            stop(paste("If you provide column names, please provide one",
                "for each column in the means input"))
    }

    ldots <- list(...)
    
    x.mat <- as.matrix(genX(N, means, sds, rho, intercept = intercept,
                            col.names = col.names))
    ## update col.names to match generated data
    ## may be one more name than col.names, b/c intercept
    x.mat.col.names <- colnames(x.mat)
    if(!missing(formula)){
        if(!missing(beta)) stop("Don't provide both beta and formula arguments")
        if(!inherits(formula, "formula")) formula <- as.formula(formula)
        beta <- Parse(formula[[3]])
        checkVarnames(x.mat.col.names, names(beta)) ## will stop if fail
    } else {
        ## using user-provided beta
        if(!is.null(names(beta))){
            if(any(names(beta) == "")){
                MESSG <- "All elements in beta should have variable names if any have names"
                stop(MESSG)
            }
            checkVarnames(x.mat.col.names, names(beta)) ## will stop if fail
        } else {
            beta1 <- beta[1:(d + as.integer(intercept))]
            names(beta1) <- x.mat.col.names
            beta2 <- beta[-(1:(d + as.integer(intercept)))]
            beta2 <- c(beta2, rep(0, times = (d * (d + 1)/2) - length(beta2)))
            intnames.mat <- outer(col.names, col.names, FUN=function(x,y) paste(x, y, sep=":"))
            beta2.names <- intnames.mat[lower.tri(intnames.mat, diag=TRUE)]
            names(beta2) <- beta2.names
            beta <- c(beta1, beta2)
        }
    }

    x.mat <- as.data.frame(x.mat)
    x.mat$error <- if (deparse(substitute(distrib)) == "rt"){
                       rt(n = N, df = stde)
                   } else {
                       distrib(N, 0, stde)
                   }

    newformula <- betanamestoformula(beta)
    ## If "Intercept" appears, change it to 1
    newformula <- gsub("Intercept", "1", newformula)
    x.mat$eta <- with(x.mat, eval(parse(text = newformula)))
    x.mat$y  <-  x.mat$eta + x.mat$error
    attr(x.mat, "beta") <- beta
    attr(x.mat, "formula") <- newformula
    attr(x.mat, "stde") <- stde
    x.mat
}

##' Convert a half-vector (vech) into a matrix.
##'
##' Fills a matrix from a vector that represents the lower triangle.
##' If user does not supply a value for diag, then the vech will fill
##' in the diagonal as well as the strictly lower triangle.  If diag
##' is provided (either a number or a vector), then vech is for the
##' strictly lower triangular part.  The default value for lowerOnly
##' is FALSE, which means that a symmetric matrix will be created. See
##' examples for a demonstration of how to fill in the lower triangle
##' and leave the diagonal and the upper triangle empty.
##'
##' @param vech A vector
##' @param diag Optional. A single value or a vector for the
##' diagonal. A vech is a strictly lower triangluar vech, it
##' does not include diagonal values. diag can be either a single
##' value (to replace all elements along the diagonal) or a vector of
##' the correct length to replace the diagonal.
##' @param lowerOnly Default = FALSE. 
##' @seealso Similar functions exist in many packages, see
##' \code{vec2sm} in corpcor, \code{xpnd} in MCMCpack
##' @export
##' @examples
##' x <- 1:6
##' vech2mat(x)
##' vech2mat(x, diag = 7)
##' vech2mat(x, diag = c(99, 98, 97, 96))
##' vech2mat(x, diag = 0, lowerOnly = TRUE)
vech2mat <-
    function(vech, diag = NULL, lowerOnly = FALSE)
{
    ## Must calculate correct number of rows from vech, if
    ## vech implies a non-square, stop.
    ## If no diag, then vech provides the diagonal values
    if (!is.null(diag)){
        d <- (sqrt(1 + 8 * length(vech)) + 1)/2
        if (!as.integer(d) == d)
            stop(deparse(substitute(vech)), " must have the correct number of elements to fill a stricly lower triangle.")
        X <- matrix(0, nrow = d, ncol = d)
        X[lower.tri(X, diag = FALSE)] <- vech
        diag(X) <- makeVec(diag, d)
        if (!lowerOnly) X[upper.tri(X)] <- t(X)[upper.tri(X)]
    } else {
        d <- (sqrt(1 + 8 * length(vech)) - 1)/2
        if (!as.integer(d) == d)
            stop(paste("You supplied diag. So ", deparse(substitute(vech)), " must have the correct number of elements to fill in a lower triangle, including the diagonal.."))
        X <- matrix(0, nrow = d, ncol = d)
        X[lower.tri(X, diag = TRUE)] <- vech
        if (!lowerOnly) X[upper.tri(X)] <- t(X)[upper.tri(X)]
    }
    X
}
NULL

##Paul Johnson
## 2012-05-04

## This is a proposed new & improved back end for regression users who
## want to calculate predicted values for selected values of inputs in
## regression. The end user is most likely to use the main function
## "predictOMatic", but the support functions newdata and model.data
## may be helpful in some applications.  Similar in spirit to the
## packages Effects, Zelig, rms, and others that attempt to do the
## same. This is distinguished mainly because it is more flexible for
## end users and works with a broader array of model formulas. Does
## (or will) work with troublesome formulas like log(10+x) + sin(x) +
## poly(x,2).

## I believe "model.data" may be suitable for widespread use in R
## packages like termplot that attempt to re-construct input data
## sets from fitted models.

##' Create a newdata frame for usage in predict methods
##'
##' This is a generic function. The default method covers almost all
##' regression models.
##'
##' It scans the fitted model, discerns the names of the predictors,
##' and then generates a new data frame.  It can guess values of the
##' variables that might be substantively interesting, but that
##' depends on the user-supplied value of predVals.  If not supplied
##' with a predVals argument, newdata returns a data frame with one
##' row -- the central values (means and modes) of the variables in
##' the data frame that was used to fit the model. The user can supply
##' a keyword "auto" or "margins". The function will try to do the
##' "right thing."
##'
##' The \code{predVals} can be a named list that supplies specific
##' values for particular predictors.  Any legal vector of values is
##' allowed. For example, \code{predVals = list(x1 = c(10, 20, 30), x2
##' = c(40, 50), xcat = levels(xcat)))}. That will create a newdata
##' object that has all of the "mix and match" combinations for those
##' values, while the other predictors are set at their central
##' values.
##'
##' If the user declares a variable with the "default" keyword, then
##' the default divider algorithm is used to select focal values.  The
##' default divider algorithm is an optional argument of this
##' function. If the default is not desired, the user can specify a
##' divider algorithm by character string, either "quantile",
##' "std.dev.", "seq", or "table".  The user can mix and match
##' algorithms along with requests for specific focal values, as in
##' \code{predVals = list(x1 = "quantile", x2 = "std.dev.", x3 = c(10,
##' 20, 30), xcat1 <- levels(xcat1))}
##' 
##' @param model Required. Fitted regression model
##' @param predVals Predictor Values that deserve investigation.
##' Previously, the argument was called "fl".  This can be 1) a
##' keyword, one of c("auto", "margins") 2) a vector
##' of variable names, which will use default methods for all named
##' variables and the central values for non-named variabled, 3) a
##' named vector with predictor variables and divider algorithms, or
##' 4) a full list that supplies variables and possible values. Please
##' see details and examples.
##' @param n Optional. Default = 3. How many focal values are desired?
##' This value is used when various divider algorithms are put to use
##' if the user has specified keywords "default", "quantile", "std.dev."
##' "seq", and "table".

##' @param ... Other arguments.
##' @return A data frame of x values that could be used as the
##' data = argument in the original regression model. The attribute
##' "varNamesRHS" is a vector of the predictor variable names.
##' @rdname newdata
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @export
##' @seealso \code{predictOMatic}
newdata <- function(model, predVals, n,  ...) UseMethod("newdata")

NULL

##' Creates a new data frame from which a model could be re-estimated.
##' 
##' @param divider Default is "quantile". Determines the method of
##' selection. Should be one of c("quantile", "std.dev", "seq", "table").
##' @param emf Optional. data frame used to fit model (not a model
##' frame, which may include transformed variables like
##' log(x1). Instead, use output from function \code{model.data}). It
##' is UNTRANSFORMED variables ("x" as opposed to poly(x,2).1 and
##' poly(x,2).2).
##' @export
##' @method newdata default
##' @export 
##' @rdname newdata
##' @example inst/examples/predictOMatic-ex.R
newdata.default <-
    function (model = NULL, predVals = NULL, n = 3, emf = NULL,
              divider = "quantile", ...){
   
    if (is.null(emf)){
        emf <- model.data(model = model)
        ##20140417 Danger: missings. Doublecheck later on model.data change
    }
    if (is.character(divider)) {
        divider <- match.arg(tolower(divider),
                             c("quantile", "std.dev.","table","seq"))
    }

    ## 20140115: why did i do this, what was I cleaning up by trying
    ## to remove variables from the model frame? First appraoch
    ## emf[ , varNamesRHS, drop = FALSE]
    ## Destroys the attributes.
    ##
    ## If this is truly necessary, there is a way to remove columns that does
    ## not kill attributes. Get the "reject names" and Null them out.
    ## This is only necesssary if the emf argument is given, since
    ## result from model.data should be OK. But I cant' remember why
    ## it seemed important to me to clear out columns. And then it is
    ## tricky to get varNamesRHS
    
    if (inherits(emf, "model.data"))  {
        varNamesRHS <- attr(emf, "varNamesRHS")
    } else {
        Terms <- terms(model)
        varNames <- all.vars(Terms) ## all variable names
        Terms[[2L]] <- 0 ## removes DV
        varNamesRHS <- all.vars(Terms)
    }

    ## 20140115: check varNamesRHS are all in emf?
    if (!missing(emf)){
        rejectNames <- setdiff(colnames(emf),  varNamesRHS)
        for (x in rejectNames) emf[ , x] <- NULL
    }
    modelcv <- centralValues(emf)
    if (is.null(predVals)) return(modelcv)
    
    auto <- FALSE   ## 20141015 brain freeze on how to do this more elegantly
    margins <- FALSE

    if (length(predVals) == 1 && is.character(predVals) && isTRUE(predVals == "auto")) {
        predVals <- vector("list", length = length(varNamesRHS))
        names(predVals) <- varNamesRHS
        auto <- TRUE
    } else if (length(predVals) == 1 && is.character(predVals) && isTRUE(predVals == "margins")) {
        predVals <- vector("list", length = length(varNamesRHS))
        names(predVals) <- varNamesRHS
        margins <- TRUE
    } else if (!is.list(predVals)) {
        flnames <- names(predVals) 
        if (is.null(flnames)) {  ##no names
            flnames <- predVals ## values become names
            predVals <- vector("list", length = length(flnames))
            names(predVals) <- flnames
        } else {
            predVals <- as.list(predVals)
        }
    }

    errMessg <- paste("Error. The focus list:\n", predVals,
                      "requests variables that are not included in the original model.\n",
                      "The names of the variables in predVals must be drawn from this list: ",
                      varNamesRHS, "\n")
    
    if (sum(!names(predVals) %in% varNamesRHS) > 0) stop(errMessg)
    flnames <- names(predVals)

    for (x in flnames) {
        if (is.null(predVals[[x]])){
            predVals[[x]] <- focalVals(emf[ ,x], divider, n)
        } else if (length(predVals[[x]]) == 1)
            if (is.character(predVals[[x]]) & (predVals[[x]] == "default")) {
                predVals[[x]] <- focalVals(emf[ ,x], divider, n)
            } else if (is.character(predVals[[x]]) & (predVals[[x]] %in% c("quantile", "std.dev.", "table", "seq"))){
            predVals[[x]] <- focalVals( emf[ ,x],  divider = predVals[[x]], n)
        }
    }

    if (margins) {
        newdf <- lapply(flnames, function(x) {
            newList <- list(predVals[[x]])
            names(newList) <- x
            ndsub <- newdata(model, predVals = newList)
        })
        names(newdf) <- flnames
    } else {
        mixAndMatch <- expand.grid(predVals)
        ## TODO: Its OK to select columns this way, but no better way add names?
        unames <- colnames(modelcv)[!colnames(modelcv) %in% colnames(mixAndMatch)]
        newdf <- cbind(mixAndMatch, modelcv[  , unames])
        colnames(newdf) <- c(colnames(mixAndMatch), unames)
        newdf <- newdf[ , colnames(emf), drop = FALSE] ## 20140115: Doublecheck me
    }
    newdf
}
NULL


##' Create a "raw" (UNTRANSFORMED) data frame equivalent to the input
##' data that would be required to fit the given model.
##'
##' This is a generic method. Unlike model.frame and
##' model.matrix, this does not return transformed variables. It deals
##' with regression formulae that have functions like poly(x, d) in
##' them. It differentiates x from d in those expressions. And it also
##' manages log(x + 10). The default method works for standarad R
##' regression models like lm and glm. 
##'
##' @param model A fitted regression model in which the data argument
##' is specified. This function will fail if the model was not fit
##' with the data option.
##' @param ... Arguments passed to implementing methods.
##' @return A data frame
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
model.data <-
    function(model, ...)
{
    UseMethod("model.data")
}

NULL

##' Create a data frame suitable for estimating a model
##'
##' This is the default method. Works for lm and glm fits.
##' 
##' @param model A fitted model
##' @param na.action Defaults to na.omit, so model as it would appear
##' in user workspace is re-created, except that rows with missing
##' values are deleted.  Changing this argument to na.pass will
##' provide the data as it was in the workspace.
##' @param ...  Place holder for other arguments, not used at present
##' @return A data frame
##' @export
##' @method model.data default
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @example inst/examples/model.data-ex.R
model.data.default <- function(model, na.action = na.omit, ...){
    ## from nls, returns -1 for missing variables
    lenVar <- function(var, data) tryCatch(NROW(eval(as.name(var),
                         data, env)), error = function(e) -1)
    fmla <- formula(model)
    varNames <- all.vars(fmla) ## all variable names
    fmla2 <- fmla
    fmla2[[2L]] <- 0
    varNamesRHS <- all.vars(fmla2)
    varNamesLHS <- setdiff(varNames, varNamesRHS)
    env <- environment(fmla)
    if (is.null(env))
        env <- parent.frame() 
    if (isS4(model)) {
        dataOrig <- eval(model@call$data, env)
    } else {
        dataOrig <-  eval(model$call$data, env)
    }
    if (is.null(dataOrig)) dataOrig <- model.frame(model)

    dataOrigRN <- row.names(dataOrig)
    n <- sapply(varNames, lenVar, data = dataOrig)

    targetLength <- max(n) ## suppose the longest-length is the number
    ## of rows in input data, before missings removed.

    varNames <- varNames[n == targetLength]

    varNamesRHS <- varNamesRHS[varNamesRHS %in% varNames]
    varNamesLHS <- varNamesLHS[varNamesLHS %in% varNames]

    mylhs <- eval(varNamesLHS)
    myrhs <- eval(paste(varNamesRHS, collapse = " + "))
    myfmla <- as.formula(paste(mylhs, " ~", myrhs), env = environment(formula(model)))
    data <- model.frame(myfmla, dataOrig, na.action = na.action)

    ## FIXME 20140421: commenting out again because lme4 objects don't have
    ## model$na.action, instead (model@frame)$na.action. And I don't want
    ## to keep messing with this.
    ## 2013:remove rows listed in model's na.action
    ## TODO: double check. Following not needed because model.frame does it
    ## 20140417: Previous conjecture false. commenting this out causes hell
    ## with NAs floating into analysis. It hits meanCenter, standardize.
    ## Fixed there, but now putting this back in.
    ## if (!is.null(model$na.action)){
    ##      data <- data[ -as.vector(model$na.action), , drop = FALSE]
    ## }

    ## keep varNamesRHS
    attr(data, "varNamesRHS") <- setdiff(colnames(data), varNamesLHS)
    class(data) <- c("model.data", "data.frame")
    invisible(data)
}
NULL


##' Create a focal value vector.
##'
##' This selects some values of a variable and creates a new "focal vector"
##' from them. Can use one "divider" algorithm, to be selected by name.
##'
##' This is a "wrapper" (or convenience) function that re-directs work
##' to other functions. The functions that do the work to select the
##' focal values for types ("table", "quantile", "std.dev.", "seq") are
##' (cutByTable(), cutByQuantile(), cutBySD(), and plotSeq())
##'
##' The built-in R function \code{pretty()}
##' works as of rockchalk 1.7.2. Any function that accepts an argument
##' n will work, as long as it creates a vector of values.
##'
##' @param x The input variable may be numeric or a factor.
##' @param divider Either a quoted string name of an algorithm or a
##' function. Default = "quantile" for numeric variables, "table" for
##' factors. Other valid values: "seq" for an evenly spaced sequence
##' from minimum to maximum, "std.dev." for a sequence that has the
##' mean at the center and values on either side that are proportional
##' to the standard deviation.
##' @param n Desired number of focal values.
##' @return A named vector of focal values selected from a variable. The
##' values of the names should be informative and useful for plotting or
##' other diagnostic work.
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @export
##' @seealso \code{predictOMatic} \code{newdata}
focalVals <-
    function(x, divider = "quantile", n = 3)
{
    if (is.function(divider)) {
        res <-  divider(x, n)
        return(res)
    }

    if (is.numeric(x)) {
        divider <- match.arg(tolower(divider),
                             c("quantile", "std.dev.","table", "seq"))
        res <- switch(divider,
                      table = cutByTable(x, n),
                      quantile = cutByQuantile(x, n),
                      "std.dev." = cutBySD(x, n),
                      "seq" = plotSeq(x, n),
                      stop("unknown 'divider' algorithm"))
    }  else {
        res <- cutByTable(x, n)
    }
    res
}
NULL



##' Create predicted values after choosing values of predictors.  Can
##' demonstrate marginal effects of the predictor variables.
##'
##' It creates "newdata" frames which are passed
##' to predict. The key idea is that each predictor has certain focal
##' values on which we want to concentrate. We want a more-or-less
##' easy way to spawn complete newdata objects along with fitted values.
##' The \code{newdata} function creates those objects, its documentation
##' might be helpful in understanding some nuances.
##'
##' If no predVals argument is supplied (same as
##' \code{predVals = "margins"}, predictOMatic creates a list of new
##' data frames, one for each predictor variable. It uses the default
##' divider algorithm (see the divider argument) and it estimates
##' predicted values for \code{n} different values of the predictor. A
##' model with formula \code{y ~ x1 + x2 + x3} will cause 3 separate
##' output data frames, one for each predictor. They will be named
##' objects in the list.
##'
##' The default approach will have marginal tables, while the setting
##' \code{predVals = "auto"} will create a single large newdata frame
##' that holds the Cartesian product of the focal values of each predictor.
##' 
##' \code{predVals} may be a vector of variable names, or it may be a
##' list of names and particular values. Whether a vector or a list is supplied,
##' \code{predVals} must name only predictors that are fitted in the
##' model. \code{predictOMatic} will choose the mean or mode for
##' variables that are not explicitly listed, and selected values of
##' the named variables are "mixed and matched" to make a data set.
##
##' There are many formats in which it can be supplied.  Suppose a
##' regression formula is \code{y1 ~ sex + income + health +
##' height}. The simplest format for predVals will be a vector of
##' variable names, leaving the selection of detailed values to the
##' default algorithms. For example, \code{predVals = c("income",
##' "height")} will cause sex and health to be set at central values
##' and income and height will have target values selected according
##' to the divider algorithm (see the argument \code{divider}).
##'
##' The user can spcecify divider algoriths to choose focal values,
##' \code{predvals = c(income = "quantile", height = "std.dev.")}. The
##' dividers provided by the rockchalk package are "quantile",
##' "std.dev.", "seq" and "table".  Those are discussed more
##' completely in the help for \code{focalVals}.  The appropriate
##' algorithms will select focal values of the predictors and they
##' will supply \code{n} values for each in a "mix and match" data
##' frame. After rockchalk 1.7.2, the divider argument can also be the
##' name of a function, such as R's pretty.
##'
##' Finally, users who want very fine grained control over
##' predictOMatic can supply a named list of predictor
##' values. For example,
##' \code{predVals = list(height = c(5.5, 6.0, 6.5),
##' income = c(10, 20, 30, 40, 50), sex = levels(dat$sex))}. One can
##' also use algorithm names, \code{predVals = list(height =
##' c(5.5, 6.0, 6.5), income = "quantile")} and so forth. Examples are
##' offered below.
##'
##' The variables named in the \code{predVals} argument should be the names
##' of the variables in the raw data frame, not the names that R
##' creates when it interprets a formula. We want "x", not the
##' transformation in the functions (not \code{log(x)}, or
##' \code{as.factor(x)} or \code{as.numeric(x)}). If a formula has a
##' predictor \code{poly(height, 3)}, then the predVals argument
##' should refer to height, not \code{poly(height, 3)}.  I've invested
##' quite a bit of effort to make sure this "just works" (many
##' alternative packages that calculate predicted values do not).
##'
##' It it important to make sure that diagnostic plots and summaries
##' of predictions are calculated with the exact same data that was
##' used to fit the model. This is surprisingly difficult because
##' formulas can include things like log(income + d) and so forth. The
##' function \code{model.data} is the magic bullet for that part of
##' the problem.
##'
##' Here is one example sequence that fits a model, discerns some
##' focal values, and then uses predictOMatic.
##'
##' \code{
##' d <- 3
##' alpha <- 13
##' m1 <- lm(yout ~ xin + xout + poly(xother,2) + log(xercise + alpha), data = dat)
##' m1dat <- model.data(m1)
##' }
##' 
##' Now, when you are thinking about which values you might like to
##' specify in predVals, use m1dat to decide. Try
##'
##' \code{summarize(m1dat)}
##'
##' Then run something like
##'
##' \code{predictOMatic( m1, predVals = list(xin = median(m1dat$xin), xout =
##' c(1,2,3), xother = quantile(m1dat$xother))}
##'
##' Get the idea?
##'
##' @param model Required. A fitted regression model. A \code{predict}
##' method must exist for that model.
##' @param predVals Optional. How to choose predictor values?  Can be
##' as simple as a keyword "auto" or "margins". May also be very
##' fine-grained detail, including 1) a vector of variable names (for
##' which values will be automatically selected) 2) a named
##' vector of variable names and divider functions, or 3) a list
##' naming variables and values. See details and examples.
##' @param divider An algorithm name from c("quantile", "std.dev",
##' "seq", "table") or a user-provided function.  This sets the method
##' for selecting values of the predictor. Documentation for the
##' rockchalk methods can be found in the functions
##' \code{cutByQuantile}, \code{cutBySD}, \code{plotSeq}, and
##' \code{cutByTable},.
##' @param n Default = 5. The number of values for which
##' predictions are sought.
##' @param ... Optional arguments to be passed to the predict
##' function. In particular, the arguments se.fit and interval are
##' extracted from ... and used to control the output.
##' @return A data frame or a list of data frames.
##' @export
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @example inst/examples/predictOMatic-ex.R
predictOMatic <-
    function(model = NULL, predVals = "margins", divider = "quantile", n = 5, ...)
{

    dots <- list(...)
    dotnames <- names(dots)

    interval <- "none"
    if (!is.null(dots[["interval"]])){
        interval <- dots[["interval"]]
        dots[["interval"]] <- NULL
    }

    se.fit <- FALSE
    if (!is.null(dots[["se.fit"]])){
        se.fit <- dots[["se.fit"]]
        dots[["se.fit"]] <- NULL
    }

    ## 2013-05-12 TODO, or FIXME 1) no smarter way than previous?  2)
    ## Why not trust modifyList below to do the work for us. 3) Why
    ## did I only attend to "interval" 4) will any of the other
    ## predict arguments cause trouble??

    ## 2013-04-18: What magic was I planning with dots here? Can't recall :(
    ##  ## next should give c('digits', 'alphaSort')
    ##  nnames <- names(formals(rockchalk::summarizeNumerics))[-1L]
    ##  ## names that need keeping if in dots:
    ##  keepnames <- dotnames %in% nnames

    emf <- model.data(model = model)
    varNamesRHS <- attr(emf, "varNamesRHS")

    ## Cover the bases
    ## User gives in a length 1 character vector "auto" or "margins"
    ## FIXME ??? 2019-02-18 R 3.5.3 fails here, now can't remember why I had last stanza here ## (!any(predVals %in% varNamesRHS)
    ##if ( (length(predVals) == 1 && is.character(predVals) && predVals == "auto") && (!any(predVals %in% varNamesRHS)) ||
    ##    ((length(predVals) == 1 && is.character(predVals) && predVals == "margins") && (!any(predVals %in% varNamesRHS))) )  {
    if ( (length(predVals) == 1 && is.character(predVals) && predVals == "auto")  ||
         (length(predVals) == 1 && is.character(predVals) && predVals == "margins") )  {   
        ndat <- newdata(model, predVals = predVals, emf = emf, divider = divider, n = n)
        ## auto returns a data frame, margins returns a list of data frames
        ## put the single df in a list
        if (!setequal(class(ndat), "list")) {
            ndat <- list(ndat)
        }
        
        nd <- lapply(ndat, function(ndsub){
            pargs <- list(model, newdata = ndsub, type = "response", interval = interval)
            pargs <-  modifyList(pargs, dots)
            fit <-  do.call("predictCI", pargs)
            if (interval != "none") ndsub <- cbind(ndsub, fit$fit)
            else ndsub <- cbind(ndsub, fit = fit$fit[ ,"fit"])
            if (se.fit) ndsub <- cbind(ndsub, fit$se.fit)
            attr(ndsub, "residual.scale") <- fit$residual.scale
            ndsub
        })
    } else {
        if (!is.list(predVals) && is.vector(predVals)) {
            if (is.null(names(predVals))){ ##no names
                pnames <- predVals
                predVals <- lapply(pnames, function(x) "default")
                names(predVals) <- pnames
            } else {  ## names and options
                predVals <- as.list(predVals)
            }
        }
        pnames <- names(predVals)
        if (any(! pnames %in% varNamesRHS))
            stop(paste("predictOMatic predVals variables must be included in the regression model.",
                       paste(varNamesRHS, collapse = " "), "\n"))

        for(x in pnames) {
            if (is.character(predVals[[x]]) && length(predVals[[x]]) == 1)
                if (predVals[[x]] == "default") {
                    predVals[[x]] <- focalVals(emf[ ,x], divider, n)
                } else if (is.function(predVals[[x]]) | predVals[[x]] %in% c("quantile", "std.dev.","table", "seq")){
                    predVals[[x]] <- focalVals(emf[ ,x],  divider = predVals[[x]], n)
                }
        }

        nd <- newdata(model, predVals, emf = emf, divider = divider, n = n)
        attr(nd, "pnames") <- pnames ## 20140114. doublecheck
        pargs <- list(model, newdata = nd, type = "response", interval = interval)
        pargs <-  modifyList(pargs, dots)
        fit <-  do.call("predictCI", pargs)
        if (interval != "none") nd <- cbind(nd, fit$fit)
        else nd <- cbind(nd, fit = fit$fit[ ,"fit"])
        if (se.fit) nd <- cbind(nd, fit$se.fit)
        attr(nd, "residual.scale") <- fit$residual.scale
    }
    nd
}
NULL



##' Calculate a predicted value matrix (fit, lwr, upr) for a
##' regression, either lm or glm, on either link or response scale.
##'
##' This adapts code from predict.glm and predict.lm. I eliminated
##' type = "terms" from consideration.
##'
##' R's predict.glm does not have an interval argument. There are
##' about 50 methods to calculate CIs for predicted values of GLMs,
##' that's a major worry. This function takes the simplest route, calculating the
##' (fit, lwr, upr) in the linear predictor scale, and then if type=
##' "response", those 3 columns are put through linkinv().  This is
##' the same method that SAS manuals suggest they use, same as Ben
##' Bolker suggests in r-help (2010).  I'd rather use one of the
##' fancy tools like Edgeworth expansion, but that R code is not
##' available (but is promised).
##'
##' Use predict.lm with se.fit = TRUE to calculate fit and se.fit.
##' Then calculate lwr and upr as fit +/- tval * se.fit. If model is
##' lm, the model df.residual will be used to get tval. If glm, this
##' is a normal approximation, so we thugishly assert tval = 1.98.
##'
##' There's some confusing term translation. I wish R lm and glm would
##' be brought into line. For lm, residual.scale = sigma. For glm,
##' residual.scale = sqrt(dispersion)
##'
##' @param object Regression object, class must include glm or lm.
##' @param newdata Data frame including focal values for predictors
##' @param type One of c("response", "link"), defaults to former.
##' @param interval One of c("none", "confidence",
##' "prediction"). "prediction" is defined only for lm objects, not
##' for glm.
##' @param dispersion Will be estimated if not provided. The variance
##' coefficient of the glm, same as scale squared. Dispersion is
##' allowed as an argument in predict.glm.
##' @param scale The square root of dispersion. In an lm, this is the
##' RMSE, called sigma in summary.lm.
##' @param na.action What to do with missing values
##' @param level Optional. Default = 0.95.  Specify whatever
##' confidence level one desires.
##' @param ... Other arguments to be passed to predict
##' @return c(fit, lwr, upr), and possibly more.
##'
predictCI <-
  function(object, newdata = NULL, type = c("response", "link"),
           interval = c("none", "confidence", "prediction"),
           dispersion = NULL, scale = NULL,
           na.action = na.pass, level = 0.95, ...)
{
    dots <- list(...)
    interval <- match.arg(interval)
    type <- if(!missing(type)) match.arg(type) else "response"
    
    if (inherits(object, "merMod")){
        na.action <- attr(object@frame, "na.action")
        dots[["REform"]] <- NA
        interval <- "none"
    } else {
        na.act <- object$na.action
    }
    ## commented out 2014-01-10 bc not compatible with S4
    ##object$na.action <- NULL ## Why did predict.glm do this?
    
    ## if interval = none, figure if the predict method gives us a vector of numbers,
    ## or a matrix, or a list with fit as an argument. Oh, this is
    ## frustrating.
    if (interval == "none") {
        if (inherits(object, "merMod")){
            pargs <- list(object, newdata = newdata, type = type, na.action = na.action)
        } else {
            pargs <- list(object, newdata = newdata, type = type, se.fit = TRUE, na.action = na.action)
        }
        pargs <-  modifyList(pargs, dots)
        predtry <-  try(do.call("predict", pargs))
        if (inherits(predtry, "try-error")) {
            stop("rockchalk:::predCI: predict(object) gave an error. Maybe this regression model does not have a predict method? \n")
        }
        ## vectors or matrices are atomic
        if (is.atomic(predtry)) {
            if (is.vector(predtry)) {
                predtry <- as.matrix(predtry, ncol = 1)
                dimnames(predtry) = list(NULL, c("fit"))
            } else if (!is.matrix(predtry)) {
                stop("atomic, but not a vector or matrix. What are you?")
            }
            pred <- list(fit = predtry, se.fit = NULL)
            return(pred)
        }
        if (is.list(predtry)) {
            fit <- predtry[["fit"]]
            if (is.null(fit)) {
                stop(paste("rockchalk:predictCI: return object from predict",
                           "does not have a fit element.",
                           "We cannot understand that."))
            } else if (is.vector(fit)) {
                fit <- as.matrix(fit, ncol = 1)
                dimnames(fit) = list(NULL, c("fit"))
            } else { ##gamble, try to cast it as a matrix
                fit <- as.matrix(fit)
                ##TODO: rethink. if there is no fit[["fit"]] inside there, other stuff will break.
            }
            predtry[["fit"]] <- fit
            if (is.null(predtry[["se.fit"]])) predtry[["se.fit"]] <- NULL
            ## previous silly? if se.fit is undefined, we want to set it NULL.
            ## or do we care? Won't future accessors get NULL automatically?
            return(predtry)
        } else {
            stop(paste("rockchalk:predictCI: return object from predict is not",
                       "a vector or a matrix or a list.",
                       "fit element. We can't understand that."))
        }
    }
    ## First, try the object's predict method. Need to know if
    ## a model's predict method answers in an understandable way
    ## to predict(object, type = "response", interval = "confidence").
    ##
    ## Frustratingly, R glm objects don't give error or warning
    ## from that, instead they just ignore it.

    ## Lets try, see what we get. If a model has no predict
    ## method, this should alert us.
    intervaltry <- interval
    if (interval == "none") intervaltry <- "confidence"
    ## TODO previous "none" should be impossible, please verify that
    ## can never happen.
    predtry <- try(predict(object, newdata = newdata, se.fit = TRUE,
                           type = type, interval = intervaltry,
                           na.action = na.action, level = level))

    if (inherits(predtry, "try-error")) {
        cat(paste("rockchalk:::predCI: There was an error from predict(object).",
                  "We will see if we can improvize.\n"))
    } else {
        ## Perplexing, Must weed out return from  predict.glm or
        ## other non-standard predict methods.
        if (is.list(predtry)) {
            if (is.vector(predtry$fit)) {
                cat(paste("rockchalk:::predCI: model's predict",
                          "method does not return an interval.\n",
                          "We will improvize with a Wald type approximation",
                          "to the confidence interval \n"))
            } else if (is.null(dimnames(predtry[["fit"]]))) {
                warning(paste("rockchalk:::predCI. The object's predict method",
                              "did not return a $fit component that has dimnames,\n",
                              "so I'll ignore that output and improvize.\n"))
            } else {
                if (all(dimnames(predtry[["fit"]])[[2]] == c("fit", "lwr", "upr")))
                    return(predtry)
                ## if this test is passed, the work is done, the model's
                ## predict method returned the expected structure
            }
        } else if (is.vector(predtry)) {
            cat(paste("rockchalk:::predCI: model's predict method does not return an interval.",
                      "We will try to improvize with a Wald type approximation to the",
                      "confidence interval \n"))
        }
    }
    ## If model was from lm, previous should have returned already.

    ## Previous will not return for glm, hopefully. glm has no interval
    ## argument, but using one doesn't cause error. It just returns
    ## what we don't want.

    ## Now we are wrestling with glm and other models who claim their
    ## class is glm or lm. It is possible a model with class "lm" could
    ## end up here because user has not implemented a method we expect
    ## but still says model is in class "lm".

    ## I'm adapting code from R predict.lm and predict.glm
    if (inherits(object, c("glm", "lm"))) {
        ## summary.survreg has no ... argument.
        if(inherits(object, "survreg")) dispersion <- 1.

        if (inherits(object, "glm")) {
            if (is.null(dispersion) || dispersion == 0) {
                dispersion <- summary(object, dispersion=dispersion)$dispersion
            }

            if (interval == "prediction") {
                stop("rockchalk::predictCI, prediction intervals not defined
            for glm in general. Try confidence intervals instead.")
            }
            ## scale = residual.scale in predict.glm
            if (is.null(scale) || scale == 0) {
                scale <- as.vector(sqrt(dispersion))
            }
        } else if (inherits(object, "lm")) {
            if (is.null(scale) || scale == 0) {
                scale <- summary(object)$sigma
            }
        }

        ## set se.fit = TRUE always, even if user doesn't want it.
        ## make them have it! Trying to make return structure more understandable.

        ## 20181109 type for lm must be response
        pred <- predict.lm(object, newdata = newdata, se.fit = TRUE, scale = scale,
                           type = "response", interval = "none", na.action = na.action, level = level)

        ## pred insides, at least:
        ## pred$fit      on linear predictor (==link) scale
        ## pred$se.fit   on linear predictor (==link) scale

        ## glm: use normal approximation. Calculate fit +/- on link scale first
        ## lm: use Wald-type CI with correct t value
        if (inherits(object, "glm")) {
            tval <-  qnorm((1 - level)/2, lower.tail = FALSE)
            fit <- cbind(fit = pred$fit,
                         lwr = pred$fit - tval * pred$se.fit,
                         upr = pred$fit + tval * pred$se.fit)
        } else if (inherits(object, "lm")) {
            tval <- qt((1 - level)/2, lower.tail = FALSE, object$df.residual)
            fit <- cbind(fit = pred$fit,
                         lwr = pred$fit - tval * pred$se.fit,
                         upr = pred$fit + tval * pred$se.fit)
        }
        ## following does nothing to lm output, but does invert glm
        if (type == "response") fit <- family(object)$linkinv(fit)

        if(missing(newdata) && !is.null(na.act)) {
            fit <- napredict(na.act, fit)
            se.fit <- napredict(na.act, pred$se.fit)
        }
        pred <- list(fit = fit, se.fit = pred$se.fit, residual.scale = scale)
        return(pred)
    }

    ## Could add stanzas for model types here. This is commented out
    ## now because I don't know if the predict method for betareg works
    ## the way I expect.
    ## if (inherits(object, "betareg")){
    ##     pred <- predict(object, newdata = newdata , type = type)
    ##     se   <- sqrt(predict(object, newdata = newdata, type = "variance"))
    ##     fit <- cbind(fit = pred,
    ##                  lwr = pred - 1.96 * se,
    ##                  upr = pred + 1.96 * se)
    ##     pred <- list(fit = fit, se.fit = se)
    ##     return(pred)
    ## }


    msg <-
        paste("rockchalk:::predictCI. The regression object was not a lm, or glm,",
              "so I can't tell how to calculate a confidence interval.",
              "May I politely suggest you rewrite your predict function to accept",
              "the argument \"interval\" in a manner similar to predict.lm, or",
              "email the rockchalk maintainer with information about this unfamiliar",
              "regression you are trying to run.\n")
    ## TODO: Consider the stone age methods.
    ## object.b <- coef(object)
    ## object.linkPred <- newdata %*% object.b
    stop(msg)
}



## ## Other approaches I've wrestled with for model.data

## ## model.data.1: how its done in termplot, using carrier functions
## ## I used this in "plotCurves" of rockchalk
## ## Problem: cant handle log(10+x), can handle log(x+10)
## ## termplot has same trouble, observe:
## ## x <- rpois(100,l=6)
## ## y <- rpois(100, l=6)
## ## m1 <- lm(log(y) ~ log(10+x))
## ## termplot(m1)

## model.data.1 <- function(model = NULL) {
##     carrier <- function(term, data, enc = NULL) {
##         if (length(term) > 1L)
##             carrier(term[[2L]])
##         else eval(term, envir = data, enclos = enc)
##     }
##     carrier.name <- function(term) {
##         if (length(term) > 1L)
##             carrier.name(term[[2L]])
##         else as.character(term)
##     }
##     mt <- terms(model)
##     mt <- delete.response(mt)
##     mf <- model.frame(model) ##[ ,-1] ## -1 gets rid of DV
##     cn <- parse(text = colnames(mf))
##     varnames <- unique(unlist(lapply(cn, carrier.name)))
##     print(varnames)
##     emf <- get_all_vars(mt, data = expand.model.frame(model, varnames, na.expand=TRUE))
##     emf <- emf[ , varnames]
##     attr(emf, "varnames") <- varnames
##     emf
## }


## ## Bill Dunlap suggests this approach, r-help 2012-04-22.
## ## mt is a terms object:
## ## unique(unlist(lapply(attr(mt, "variables")[-1], all.vars)))
## ## Solves log(10+x) problem
## ## EXCEPT, as Bill warns, it
## ## includes d in poly(x2,d). That includes "d" as a variable
## ## name.
## model.data.2 <- function(model = NULL) {
##     mt <- terms(model)
##     varnames <- unique(unlist(lapply(attr(mt, "variables")[-1], all.vars)))
##     print(varnames)
##     emf <- get_all_vars(mt, data = expand.model.frame(model, varnames, na.expand=TRUE))
##     emf <- emf[ , varnames1]
##     attr(emf, "varnames") <- varnames1
##     emf
## }


## This is what I was using until rockchalk-1.7.6.0

## model.data <- function(model){
##     ## from nls, returns -1 for missing variables
##     lenVar <- function(var, data) tryCatch(length(eval(as.name(var),
##                          data, env)), error = function(e) -1)
##     fmla <- formula(model)
##     varNames <- all.vars(fmla) ## all variable names
##     fmla2 <- fmla
##     fmla2[[2L]] <- 0
##     varNamesRHS <- all.vars(fmla2)
##     varNamesLHS <- setdiff(varNames, varNamesRHS)
##     env <- environment(fmla)
##     if (is.null(env))
##         env <- parent.frame()
##     if (is.null(model$call$data)) stop("rockchalk:::model.data. \nPlease refit your regression model using the \ndata argument. Otherwise, I don't know how to re-construct the data structure.\n")
##     dataOrig <-  eval(model$call$data, environment(formula(model)))
##     dataOrigRN <- row.names(dataOrig)
##     n <- sapply(varNames, lenVar, data = dataOrig)
##     targetLength <- length(eval(as.name(varNamesRHS[1]), dataOrig, env))
##     varNames <- varNames[n == targetLength]
##     ldata <- lapply(varNames, function(x) {
##         myv <- eval(as.name(x), dataOrig, env)
##         row.names(myv) <- NULL
##         myv
##     }
##                     )

##         names(ldata) <- varNames
##     data <- data.frame(ldata[varNames])
##     if (!is.null(dataOrigRN)) row.names(data) <- dataOrigRN
##     ## remove rows listed in model's na.action
##     ## TODO: question: what else besides OMIT might be called for?
##     if ( !is.null(model$na.action)){
##         data <- data[ -as.vector(model$na.action), , drop = FALSE]
##     }
##     ## keep varNamesRHS that exist in datOrig
##     attr(data, "varNamesRHS") <- setdiff(colnames(data), varNamesLHS)
##     invisible(data)
## }
## NULL

##' Create Symmetric Matrices, possibly covariance or correlation
##' matrices, or check a matrix for symmetry and serviceability.
##'
##' Check X and do the right thing. If X is a matrix, check that it is
##' a valid for the intended purpose (symmetric or correlation or
##' covariance).  If X a single value, use that to fill up a
##' matrix. If it is a vector, try to use it as a vech to fill the
##' lower triangle. If d is supplied as an integer, use that as desired size.
##'
##' @param X A single value, a vector (a vech), or a matrix
##' @param d Optional. An integer, the desired number of rows (or
##'     columns). Don't specify this argument if X is already a
##'     matrix.  Only required if X is an integer and diag is not
##'     supplied. Otherwise, the function tries to deduce desired size
##'     of output from X (as a vech) and diag.
##' @param diag Values for the diagonal. This is important because it
##'     alters the way X is interpreted.  If diag is not provided,
##'     then X is understood to include diagonal elements.
##' @param corr TRUE or FALSE: Should we construct a correlation
##'     matrix
##' @param cov TRUE or FALSE: Should this be a covariance matrix?
##' @return A d x d matrix
##' @export
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @examples
##' makeSymmetric(X = 3, d = 4)
##' makeSymmetric(X = 3, d = 4, diag = c(99, 98, 97, 96))
##' makeSymmetric(c(1,2,3))
##' makeSymmetric(c(1,2,3), d = 5)
##' makeSymmetric(c(0.8,0.4, 0.2), cov = TRUE)
##' makeSymmetric(c(0.8,0.4, 0.2), cov = TRUE, diag = c(44, 55, 66))
##'
makeSymmetric <-
    function(X, d = NULL, diag = NULL, corr = FALSE, cov = FALSE)
{
    if (is.matrix(X)) {
        dims <- dim(X)
        if (dims[1] != dims[2]) stop("X not square")
        stopifnot (isSymmetric(X))
        if (corr | cov) stopifnot(checkPosDef(X))
    } else if (length(X) == 1) {
        if (is.null(d) & is.null(diag)) stop("One of diag or d is required if X is singleton")
        if (corr & (X < -1 | X > 1)) stop(paste("The value of of a correlation should be in [-1,1]"))
        X <- matrix(X, nrow = d, ncol = d)
        if (corr == TRUE) diag(X) <- 1.0
        if (!is.null(diag)) diag(X) <- diag
    } else if (is.vector(X)) {
        if (corr == TRUE) {
            if(!is.null(diag)) stop("If you want a correlation matrix, just provide X as a strictly lower triangle")
            X <- vech2Corr(X)
            if(!is.null(d)) {
                if (dim(X)[1] != d) stop("d parameter inconsisent with size implied by X as a strictly lower triangular vech")
            }
        } else if (cov == TRUE) {
            X <- vech2mat(X, diag)
            stopifnot(checkPosDef(X))
        } else {
            X <- vech2mat(X, diag)
        }
    }  else {
        stop(paste("makeSquare cannot understand the value supplied for argument", deparse(substitute(X))))
    }
    X
}
NULL

##' Illustrate multicollinearity in regression, part 1.
##'
##' @description
##' This is a set of functions that faciliates the examination
##' of multicollinearity. Suppose the "true" relationship is
##'  y[i] = 0.2 * x1[i] + 0.2 * x2[i] + e
##' where e is Normal(0, stde^2).
##'
##' mcGraph1 draws the 3D regression space, but all of the points
##' are illustrated "in" the flat plane of x1-x2 variables.
##'
##' @details
##' These functions are specialized to a particular purpose. If you
##' just want to draw 3D regressions, look at plotPlane.
##' @name mcGraph1
##' @param x1 a predictor vector
##' @param x2 a predictor vector
##' @param y  the dependent variable
##' @param x1lab label for the x1 axis, (the one called "xlab" inside persp)
##' @param x2lab label for the x2 axis, (the one called "ylab" inside persp)
##' @param ylab  label for the y (vertical) axis (the one called "zlab" inside persp)
##' @param ... additional parameters passed to persp
##' @export
##' @rdname mcGraph
##' @return mcGraph1 and mcGraph2 return only the perspective matrix
##' from persp. It is returned so that users can add additional embellishments on the 3D plot (can be used with trans3d)
##' @keywords regression hplot
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @examples
##' set.seed(12345)
##' ## Create data with x1 and x2 correlated at 0.10
##' dat <- genCorrelatedData(rho=.1, stde=7)
##'
##' mcGraph1(dat$x1, dat$x2, dat$y, theta=20, phi=8, ticktype="detailed", nticks=10)
##'
mcGraph1 <- function (x1, x2, y, x1lab, x2lab, ylab, ...){
    x1range <- magRange(x1, 1.25)
    x2range <- magRange(x2, 1.25)
    yrange <- magRange(y, 1.5)
    
    if (missing(x1lab)) x1lab <- gsub(".*\\$", "", deparse(substitute(x1)))
    if (missing(x2lab)) x2lab <- gsub(".*\\$", "", deparse(substitute(x2)))
    if (missing(ylab)) ylab  <- gsub(".*\\$", "", deparse(substitute(y)))
    
    res <- perspEmpty(x1 = plotSeq(x1range, 5),
                      x2 = plotSeq(x2range,5),
                      y = yrange, x1lab = x1lab,
                      x2lab = x2lab, ylab = ylab, ...)
    
    yMinimum <- rep(yrange[1] , length(x1))
    mypoints1 <- trans3d(x1, x2, yMinimum, pmat = res)
    points( mypoints1, pch = 16, col = "blue")
    invisible(res)
}
NULL

 
##' mcGraph2 draws a 3-D representation of a scatterplot with shadows
##' in the x1-x2 plane.  The observations are represented by blue
##' points floating above the x1-x2 plane. If scaley=1, the end result
##' is a scatterplot "cloud" of the y points above the x1-x2 plane,
##' and gray shadows of the points are cast down from the cloud onto
##' the x1-x2 plane itself. This uses persp to make the actual
##' drawing.
##'
##' @param rescaley a single scalar value or a vector of the same
##' length as y.
##' @export
##' @rdname mcGraph
##' @examples
##' set.seed(12345)
##' ## Create data with x1 and x2 correlated at 0.10
##' dat <- genCorrelatedData(rho=.1, stde=7)
##' ## This will "grow" the "cloud" of points up from the
##' ## x1-x2 axis
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.0, theta = 0)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.1, theta = 0)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.2, theta = 0)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.3, theta = 0)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.4, theta = 0)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.5, theta = 0)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.6, theta = 0)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.7, theta = 0)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.8, theta = 0)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.9, theta = 0)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 1, theta = 0)
##'
##' ##rotate this
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 1, theta = 20)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 1, theta = 40)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 1, theta = 60)
##' mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 1, theta = 80)
##'
##' ## once they reach the top, make them glitter a while
##' for(i in 1:20){
##'   mcGraph2(dat$x1, dat$x2, dat$y, rescaley = runif(length(dat$x1), .9,1.1), theta = 0)
##' }
##'
mcGraph2 <- function(x1, x2, y, rescaley = 1, drawArrows = TRUE, x1lab, x2lab, ylab, ...){
    x1range <- magRange(x1, 1.25)
    x2range <- magRange(x2, 1.25)
    yrange <- magRange(y, 1.5)

    if (missing(x1lab)) x1lab <- gsub(".*\\$", "", deparse(substitute(x1)))
    if (missing(x2lab)) x2lab <- gsub(".*\\$", "", deparse(substitute(x2)))
    if (missing(ylab)) ylab  <- gsub(".*\\$", "", deparse(substitute(y)))

    res <- perspEmpty(x1 = plotSeq(x1range,5), x2 = plotSeq(x2range,5), y = yrange, x1lab = x1lab, x2lab = x2lab, ylab = ylab, ...)

    mypoints1 <- trans3d ( x1, x2 ,yrange[1], pmat = res )
    newy <- rescaley * (y - yrange[1]) + yrange[1]
    mypoints2 <- trans3d ( x1 , x2 , newy , pmat = res )

    points( mypoints1, pch = 16, col=gray(0.8))
    points( mypoints2, pch = 1, col= "blue")
    mypoints2s <- trans3d(x1, x2, (0.8)*newy, pmat = res)
    if (drawArrows) arrows(mypoints1$x , mypoints1$y , mypoints2s$x , mypoints2s$y , col="red" , lty = 2, lwd = 0.3, length = 0.1)
    invisible(res)
}
NULL

##' With mcGraph3, the observations are scattered in 3-dimensions, the
##' fitted values are represented by a mesh, and their shadows in the
##' x1-x2 plane are also represented.
##'
##' @param interaction a TRUE or FALSE request for inclusion of the x1-x2 interaction in the regression calculation
##' @param drawArrows TRUE or FALSE, do you want arrows from the plane to observed y?
##' @return mcGraph3 returns a list of 2 objects. 1) the fitted
##' regression model2) the perspective matrix used with persp to draw
##' the image.
##' @export
##' @rdname mcGraph
##' @examples
##' set.seed(12345)
##' ## Create data with x1 and x2 correlated at 0.10
##' dat <- genCorrelatedData(rho=.1, stde=7)
##'
##' mcGraph3(dat$x1, dat$x2, dat$y, theta = 0)
##'
##' dat2 <- genCorrelatedData(rho = 0, stde = 7)
##'
##' mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = 0, phi = 10)
##' mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = 30, phi = 10)
##' mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = -30, phi = 10)
##' mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = -30, phi = -10)
##' mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = -30, phi = -15)
##'
##' ## Run regressions with not-strongly correlated data
##' modset1 <- list()
##' for(i in 1:20){
##'   dat2 <- genCorrelatedData(rho = .1, stde = 7)
##'   summary(lm( y ~ x1 + x2 , data = dat2))
##'   modset1[[i]] <- mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = -30)
##' }
##'
##'
##' ## Run regressions with strongly correlated data
##' modset2 <- list()
##' for(i in 1:20){
##'   dat2 <- genCorrelatedData(rho = .981, stde = 7)
##'   summary(lm( y ~ x1 + x2 , data = dat2))
##'   modset2[[i]] <- mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = -30)
##' }
##'
##' dat3 <- genCorrelatedData(rho = .981, stde = 100, beta=c(0.1, 0.2, 0.3, -0.1))
##' mcGraph3(dat3$x1, dat3$x2, dat3$y, theta=-10, interaction = TRUE)
mcGraph3 <- function(x1, x2, y, interaction = FALSE, drawArrows = TRUE, x1lab, x2lab, ylab, ...){
    x1range <- magRange(x1, 1.25)
    x2range <- magRange(x2, 1.25)
    yrange <- magRange(y, 1.5)

    if (missing(x1lab)) x1lab <- gsub(".*\\$", "", deparse(substitute(x1)))
    if (missing(x2lab))  x2lab <- gsub(".*\\$", "", deparse(substitute(x2)))
    if (missing(ylab)) ylab  <- gsub(".*\\$", "", deparse(substitute(y)))

    res <- perspEmpty(x1 = plotSeq(x1range, 5), x2 = plotSeq(x2range, 5),  y = yrange, x1lab = x1lab, x2lab = x2lab, ylab = ylab, ...)

    mypoints1 <- trans3d( x1, x2, yrange[1], pmat = res )
    points(mypoints1, pch = 16, col = gray(0.8))

    mypoints2 <- trans3d( x1, x2, y, pmat = res )
    points(mypoints2, pch = 1, col = "blue")

    if (interaction) m1 <- lm(y ~ x1 * x2)
    else m1 <- lm(y ~ x1 + x2)

    x1seq <- plotSeq (x1range, length.out = 20)
    x2seq <- plotSeq (x2range, length.out = 20)

    zplane <- outer (x1seq, x2seq, function(a, b) { predict(m1,
                      newdata = data.frame(x1 = a, x2 = b))})

    for( i in 1:length(x1seq) ){
        lines(trans3d(x1seq[i], x2seq, zplane[i,], pmat = res), lwd = 0.3)
    }
    for( j in 1:length(x2seq) ){
        lines(trans3d(x1seq, x2seq[j], zplane[,j], pmat = res), lwd = 0.3)
    }

    mypoints4 <- trans3d (x1 , x2 , fitted(m1) , pmat = res)
    ##  points(mypoints4)
    newy <- ifelse(fitted(m1) < y, fitted(m1)+ 0.8*(y-fitted(m1)),
                   fitted(m1) + 0.8 * (y-fitted(m1)))
    mypoints2s <- trans3d(x1, x2, newy, pmat = res)

    if (drawArrows) arrows(mypoints4$x , mypoints4$y , mypoints2s$x , mypoints2s$y , col = "red" , lty = 4, lwd = 0.3, length = 0.1)
    invisible(list(lm = m1, res = res))
}
NULL

##' recode a factor by "combining" levels
##'
##' This makes it easy to put levels together and create a new factor
##' variable. If a
##' factor variable is currently coded with levels
##' c("Male","Female","Man", "M"), and the user needs to combine the
##' redundant levels for males, this is the function to use!  This is a surprisingly difficult problem in R.
##'
##' If the factor is an ordinal factor, then levels may be combined
##' only if they are adjacent. A factor with levels
##' c("Lo","Med","Hi","Extreme") allows us to combine responses "Lo"
##' and "Med", while it will NOT allow us to combine "Lo" with "Hi".
##'
##' A non-ordered factor can be reorganized to combine any values, no
##' matter what positions they occupy in the levels vector.
##'
##' @param fac An R factor variable, either ordered or not.
##' @param levs The levels to be combined. Users may specify either a
##' numerical vector of level values, such as c(1,2,3), to combine the
##' first three elements of level(fac), or they may specify level
##' names.  This can be done as a character vector of *correctly
##' spelled* factor values, such as c("Yes","Maybe","Always") or it
##' may be provided as a subset of the output from levels, such as
##' levels(fac)[1:3].
##' @param newLabel A character string that represents the label of
##' the new level to be created when \code{levs} values are combined.
##' @name combineLevels
##' @export combineLevels
##' @return A new factor variable, with unused levels removed.
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @examples
##' x <- c("M","A","B","C","A","B","A","M")
##' x <- factor(x)
##' levels(x)
##' x2a <- combineLevels(x, levs = c("M","A"), newLabel = "M_or_A")
##' addmargins(table(x2a, x, exclude=NULL))
##' x2b <- combineLevels(x, c(1,4), "M_or_A")
##' addmargins(table(x2b, x, exclude=NULL))
##' x3 <- combineLevels(x, levs = c("M","A","C"), newLabel = "MAC")
##' addmargins(table(x3, x, exclude=NULL))
##' ## Now an ordinal factor
##' z <- c("M","A","B","C","A","B","A","M")
##' z <- ordered(z)
##' levels(z)
##' table(z, exclude=NULL)
##' z2a <-  combineLevels(z, levs = c(1,2), "Good")
##' addmargins(table(z2a, z, exclude = NULL))
##' z2b <- combineLevels(z, levs = c("A","B"), "AorB")
##' addmargins(table(z2b, z, exclude = NULL))
## Should fail:
## z2 <- combineLevels(z, levs=c("A","C"), "Whoops!")
##'
combineLevels <-
    function(fac, levs, newLabel = "combinedLevels")
{
    ##internal fn to discern connected sequence
    adjacent <- function(x){
        xfull <- seq(min(x),max(x))
        identical(xfull, as.integer(x))
    }

    facl <- levels(fac)
    if (is.character(levs)){
        if(!identical(sum(levs %in% facl), length(levs))){
            stop(cat("Error: requested levels: \"",levs,"\" are not in the legal list of factor levels:\"", facl, "\""))
        }
    } else {
        if( sum(!levs %in% 1:length(facl)) > 0) stop(cat("Error: requested levs don't exist in the factor"))
  }

    ##convert levs to numeric indices
    if (is.character(levs)) {
        levsNum <- which( facl %in% levs )
    } else {
        levsNum <- levs     ##levsNum: Numerical Positions
        levs <- facl[levs]  ##levs: NAMES
    }

    ##for NOT ORDINAL factors, easy. Put new level on end

    if (! "ordered" %in% class(fac)) {
        faclnew  <- c(facl, "pjtempfacname")
        facnew <- factor(fac, levels = faclnew)
        facnew[ facnew %in% levs ] <- "pjtempfacname"
    } else {
        if ("ordered" %in% class(fac)) {  ## levels must be adjacent
            if (!adjacent(levsNum)) {
                stop("fac is ordered. The levels to be combined must be adjacent")
            }
            faclnew <- c(facl[1:min(levsNum)], "pjtempfacname", facl[(1+min(levsNum)):length(facl)])
            facnew <- factor(fac, levels=faclnew)
            facnew[ facnew %in% levs ] <- "pjtempfacname"
        }
    }

    facnew <- facnew[ , drop = TRUE]
    levels(facnew)[ levels(facnew) == "pjtempfacname" ] <- newLabel
    cat("The original levels", facl, "\nhave been replaced by", levels(facnew),"\n")
    facnew
}

##' magRange
##' Magnify the range of a variable.
##'
##' By default, R's range function returns the minimum and maximum
##' values of a variable. This returns a magnified range. It is used
##' for some plotting functions in the rockchalk package
##'
##' @usage magRange(x, mult = 1.25)
##' @param x an R vector variable
##' @param mult a multiplier by which to magnify the range of the
##' variable.  A value of 1 leaves the range unchanged. May be a
##' scalar, in which case both ends of the range are magnified by the
##' same amount.  May also be a two valued vector, such as c(minMag,
##' maxMag), in which case the magnification applied to the minimum is
##' minMag and the magnification of the maximum is maxMag.  After
##' version 1.5.5, mult may be smaller than 1, thus shrinking the
##' range. Setting mult to values closer to 0 causes the range to
##' shrink to the center point from both sides.
##' @name magRange
##' @export magRange
##' @examples
##' x1 <- c(0, 0.5, 1.0)
##' range(x1)
##' magRange(x1, 1.1)
##' magRange(x1, c(1.1, 1.4))
##' magRange(x1, 0.5)
##' magRange(x1, c(0.1, 0.1))
##' x1 <- rnorm(100)
##' range(x1)
##' magRange(x1)
##' magRange(x1, 1.5)
##' magRange(x1, c(1,1.5))

magRange <- function(x, mult = 1.25){
  xr <- range(x, na.rm = TRUE)
  xm <- 0.5*(xr[1] + xr[2])
  magXr <- vector(mode = "numeric", length = 2)

  if (length(mult) > 2) stop("magRange: supply only 2 multipliers")
  if (length(mult) == 1) mult <- c(mult, mult)

  if ( mult[2] > 1 ) {
      magXr[2] <- xr[2] + 2 * (mult[2] - 1)*(xr[2] - xm)
  } else {
      magXr[2] <- xm + 0.5 * mult[2] * (xr[2] -xr[1])
  }

  if ( mult[1] > 1 ) {
      magXr[1] <- xr[1] - 2 * (mult[1] - 1) *( xm - xr[1])
  } else {
      magXr[1] <- xm - 0.5 * mult[1] * (xr[2] -xr[1])
  }
  magXr
}

##' Create an ordinal variable by grouping numeric data input.
##'
##' This is a convenience function for usage of R's \code{cut}
##' function. Users can specify cutpoints or category labels or
##' desired proportions of groups in various ways. In that way, it has
##' a more flexible interface than \code{cut}. It also tries to notice
##' and correct some common user errors, such as omitting the outer
##' boundaries from the probs argument. The returned values are
##' labeled by their midpoints, rather than cut's usual boundaries.
##'
##' The dividing points, thought of as "thresholds" or "cutpoints",
##' can be specified in several ways.  \code{cutFancy} will
##' automatically create equally-sized sets of observations for a
##' given number of categories if neither \code{probs} nor
##' \code{cutpoints} is specified. The bare minimum input needed is
##' \code{categories=5}, for example, to ask for 5 equally sized
##' groups. More user control can be had by specifying either
##' \code{cutpoints} or \code{probs}. If \code{cutpoints} is not
##' specified at all, or if \code{cutpoints="quantile"}, then
##' \code{probs} can be used to specify the proportions of the data
##' points that are to fall within each range. On the other hand, one
##' can specify \code{cutpoints = "quantile"} and then \code{probs} will
##' be used to specify the proportions of the data points that are to
##' fall within each range.
##' 
##' If \code{categories} is not specified, the category names will be
##' created. Names for ordinal categories will be the numerical
##' midpoints for the outcomes.  Perhaps this will deviate from your
##' expectation, which might be ordinal categories name "0", "1", "2",
##' and so forth.  The numerically labeled values we provide can be
##' used in various ways during the analysis process. Read "?factor"
##' to learn ways to convert the ordinal output to other
##' formats. Examples include various ways of converting the ordinal
##' output to numeric.
##' 
##' The \code{categories} parameter works together with
##' \code{cutpoints}.  \code{cutpoints} allows a character string
##' "quantile". If \code{cutpoints} is not specified, or if the user
##' specifies a character string \code{cutpoints="quantile"}, then the
##' \code{probs} would be used to determine the cutpoints.  However,
##' if \code{probs} is not specified, then the \code{categories}
##' argument can be used. If \code{cutpoints="quantile"}, then
##' 
##' \itemize{
##' \item if \code{categories} is one integer, then it is interpreted
##' as the number of "equally sized" categories to be created, or
##'
##' \item \code{categories} can be a vector of names. The length
##' of the vector is used to determine the number of categories, and
##' the values are put to use as factor labels.
##' }
##' 
##' @param y The input data from which the categorized variable will
##'     be created.
##' @param cutpoints Optional paramter, a vector of thresholds at
##'     which to cut the data. If it is not supplied, the default
##'     value \code{cutpoints="quantile"} will take effect. Users can
##'     supplement with \code{probs} and/or \code{categories} as shown
##'     in examples.
##' @param probs This is an optional parameter, relevant only when the
##'     R function \code{\link{quantile}} function is used to
##'     calculate cutpoints. The length should be number of desired
##'     categories PLUS ONE, as in \code{c(0, .3, .6, 1)}. That will
##'     create categories that represent 1) less than .3, between .3
##'     and .6, and above .6.  A common user error is to specify only
##'     the internal divider values, such as \code{probs = c(.3,
##'     .6)}. To anticipate and correct that error, this function will
##'     insert the lower limit of 0 and the upper limit of 1 if they
##'     are not already present in \code{probs}.
##' @param categories Can be a number to designate the number of
##'     sub-groups created, or it can be a vector of names used. If
##'     \code{cutpoints} and \code{probs} are not specified, the
##'     parameter \code{categories} should be an integer to specify
##'     how many data groups to create.It is required if
##'     cutpoints="quantile" and probs is not specified. Can also be a
##'     vector of names to be used for the categories that are
##'     created. If category names are not provided, the names for the
##'     ordinal variable will be the midpoint of the numeric range
##'     from which they are constructed.
##' @return an ordinal vector with attributes "cutpoints" and "props"
##'     (proportions)
##' @importFrom stats embed
##' @name cutFancy
##' @export cutFancy
##' @examples
##' set.seed(234234)
##' y <- rnorm(1000, m = 35, sd = 14)
##' yord <- cutFancy(y, cutpoints = c(30, 40, 50))
##' table(yord)
##' attr(yord, "props")
##' attr(yord, "cutpoints")
##' yord <- cutFancy(y, categories = 4L)
##' table(yord, exclude = NULL)
##' attr(yord, "props")
##' attr(yord, "cutpoints")
##' yord <- cutFancy(y, probs = c(0, .1, .3, .7, .9, 1.0),
##'                   categories = c("A", "B", "C", "D", "E"))
##' table(yord, exclude = NULL)
##' attr(yord, "props")
##' attr(yord, "cutpoints")
##' yord <- cutFancy(y, probs = c(0, .1, .3, .7, .9, 1.0))
##' table(yord, exclude = NULL)
##' attr(yord, "props")
##' attr(yord, "cutpoints")
##' yasinteger <- as.integer(yord)
##' table(yasinteger, yord)
##' yasnumeric <- as.numeric(levels(yord))[yord]
##' table(yasnumeric, yord)
##' barplot(attr(yord, "props"))
##' hist(yasnumeric)
##' X1a <-
##'    genCorrelatedData3("y ~ 1.1 + 2.1 * x1 + 3 * x2 + 3.5 * x3 + 1.1 * x1:x3",
##'                        N = 10000, means = c(x1 = 1, x2 = -1, x3 = 3),
##'                        sds = 1, rho = 0.4)
##' ## Create cutpoints from quantiles
##' probs <- c(.3, .6)
##' X1a$yord <- cutFancy(X1a$y, probs = probs)
##' attributes(X1a$yord)
##' table(X1a$yord, exclude = NULL)
cutFancy <- function(y, cutpoints = "quantile", probs, categories){
    if(!missing(cutpoints) && !is.null(cutpoints)){
        if(is.numeric(cutpoints)){
            ## Use dividers provided by user
            yrange <- range(y, na.rm=TRUE)
            ## need outer boundaries to get centers:
            breaks <- c(yrange[1], cutpoints, yrange[2]) 
            centers <- rowMeans(embed(breaks, 2))
            if(missing(categories) || is.null(categories)){
                ## create category names: centernames
                centernames <- formatC(centers, 2)
            } else {
                centernames <- categories
            }
            ## Use input category labels
            yord <- cut(y, breaks = breaks, labels = centernames,
                        include.lowest = TRUE, ordered_result = TRUE)
            props <- prop.table(table(yord))
            attr(yord, "cutpoints") <- cutpoints
            attr(yord, "props") <- props
            return(invisible(yord))
        } else if(cutpoints != "quantile") {
            MESSG <- "only `quantile` is allowed if cutpoints is not numeric thresolds"
            stop(MESSG)
        }
    }

    ## pad probs with 0, and 1 one edges to help with common user error
    if(!missing(probs) && !is.null(probs)){
        if(abs(zapsmall(probs[1] - 0)) > 0) {probs <- c(0, probs)}
        if(abs(zapsmall(1 - probs[length(probs)])) > 0) {probs <- c(probs, 1)}
    }
    
    ## categories = 1 number, not vector of names
    if(!missing(categories) && !is.null(categories) && length(categories) == 1) {
        categories <- as.integer(categories)
        if(is.null(categories)){
            MESSG <- "categories cannot be coerced to an integer"
            stop(MESSG)
        }
        ## we will make up names by midpoints of data
        if(missing(probs) || is.null(probs)){
            ## categories is an integer:
            ## cutpoints is even spaced sections with n of groups = "categories"
            probs <- seq(0, 1, length.out = categories + 1)
        } else {
            MESSG <- "single-value categories parameter is ignored if probs is specified"
            warning(MESSG)
            breaks <- quantile(y, probs =  probs)
        }
        breaks <- quantile(y, probs =  probs)
        centers <- rowMeans(embed(breaks, 2))
        centernames <- formatC(centers, 2)
    } else if(!missing(categories) && !is.null(categories) && length(categories) > 1){
        ## categories has more values
        if(!missing(probs) && !is.null(probs)){
            if (length(categories) +1 != length(probs)){
                MESSG <- "Number of categories should be 1 less than probs vector"
                stop(MESSG)
            }
        }
        if(missing(probs) || is.null(probs)){
            ## Use categories to figure how many levels needed
            ## cutpoints is even spaced sections with n of groups = "categories"
            probs <- seq(0, 1, length.out = length(categories) + 1)
        }
        breaks <- quantile(y, probs = probs)
        centernames <- categories
    } else if(missing(categories) || is.null(categories)) {
        if(is.null(probs) || missing(probs)){
              MESSG <- "Must specify either categories or probs"
              stop(MESSG)
        } else {
            breaks <- quantile(y, probs =  probs)
            centernames <- formatC(rowMeans(embed(breaks, 2)))
        }
    }
    yord <- cut(y, breaks = breaks, labels = centernames,
                include.lowest = TRUE, ordered_result = TRUE)
    ## exclude outer edges
    cutpoints <- unname(breaks[-c(1, length(breaks))])
    props <- prop.table(table(yord))
    attr(yord, "cutpoints") <- cutpoints
    attr(yord, "props") <- props
    invisible(yord)
}

##' perspEmpty
##'
##' Creates a persp plot without drawing anything in the interior.
##' Does equivalent \code{of plot( type="n")} for persp.
##'
##' Regression demonstrations require a blank slate in which
##' points and planes can be drawn. This function creates that
##' blank persp canvas for those projects. It is not necessary
##' that x1, x2 and y be vectors of the same length, since this
##' function's only purpose is to plot an empty box with ranges
##' determined by the input variables. persp calls the 3 axes
##' x, y, and z, but here they are called x1, x2, and y.
##'
##' @param x1 data for the first horizontal axis, an R vector
##' @param x2 data for the second horizontal axis, an R vector
##' @param y data for the vertical axis, an R vector
##' @param x1lab label for the x1 axis, (the one called "xlab" inside persp)
##' @param x2lab label for the x2 axis, (the one called "ylab" inside persp)
##' @param ylab label for the y (vertical) axis (the one called "zlab" inside persp)
##' @param x1lim Optional: limits for x1 axis (should be a vector with 2 elements)
##' @param x2lim Optional: limits for x2 axis (should be a vector with 2 elements)
##' @param ... further arguments that are passed to persp. Please note
##' Please remember that y is the vertical axis, but for persp, that
##' is the one I call x2.  Thus dot-dot-dot arguments including xlab,
##' ylab, zlab, xlim, ylim, and zlim are going to be ignored.
##' @return The perspective matrix that is returned by persp
##' @name perspEmpty
##' @export perspEmpty
##' @examples
##' x1 <- 1:10
##' x2 <- 41:50
##' y <-  rnorm(10)
##' perspEmpty(x1, x2, y)
##' res <- perspEmpty(x1, x2, y, ticktype="detailed", nticks=10)
##' mypoints1 <- trans3d ( x1, x2, y, pmat = res )
##' points( mypoints1, pch = 16, col= "blue")
perspEmpty <-
    function(x1, x2, y, x1lab = "x1", x2lab = "x2", ylab = "y", x1lim, x2lim, ... )
{
    x1range <- range(x1, na.rm = TRUE)
    x2range <- range(x2, na.rm = TRUE)
    yrange <- range(y, na.rm = TRUE)
    zZero <- outer(x1, x2, function(a,b) { a*b*0 + yrange[1] })

    dotargs <- list(...)
    dotargs[["xlab"]] <- x1lab
    dotargs[["ylab"]] <- x2lab
    dotargs[["zlab"]] <- ylab
    if (!missing(x1lim)) dotargs[["xlim"]] <- x1lim
    if (!missing(x2lim)) dotargs[["ylim"]] <- x2lim
    myDefaults <- list(x = x1, y = x2, z = zZero, zlim = yrange, lwd = 1, theta = -20, phi = 15)

    myargs <- modifyList(myDefaults, dotargs)
    res <- do.call("persp", myargs)
}

##' Stack together data frames
##' 
##' In the end of the code for plyr::rbind.fill, the author explains
##' that is uses an experimental function to build the data.frame.  I
##' would rather not put any weight on an experimental function, so I
##' sat out to create a new rbindFill. This function uses no
##' experimental functions. It does not rely on any functions from
##' packages that are not in base of R, except, of course, for functions
##' in this package.
##'
##' Along the way, I noticed a feature that seems to be a flaw in both
##' rbind and rbind.fill.  In the examples, there is a demonstration
##' of the fact that base R rbind and plyr::rbind.fill both have
##' undesirable properties when data sets containing factors and
##' ordered variables are involved. This function introduces a
##' "data consistency check" that prevents corruption of variables
##' when data frames are combined.  This "safe" version will notice
##' differences in classes of variables among data.frames and stop
##' with an error message to alert the user to the problem.
##' @param ... Data frames
##' @return A stacked data frame
##' @author Paul Johnson
##' @name rbindFill
##' @export rbindFill
##' @examples
##' set.seed(123123)
##' N <- 10000
##' dat <- genCorrelatedData2(N, means = c(10, 20, 5, 5, 6, 7, 9), sds = 3,
##'            stde = 3, rho = .2,  beta = c(1, 1, -1, 0.5))
##' dat1 <- dat
##' dat1$xcat1 <- factor(sample(c("a", "b", "c", "d"), N, replace=TRUE))
##' dat1$xcat2 <- factor(sample(c("M", "F"), N, replace=TRUE),
##'                     levels = c("M", "F"), labels = c("Male", "Female"))
##' dat1$y <- dat$y +
##'           as.vector(contrasts(dat1$xcat1)[dat1$xcat1, ] %*% c(0.1, 0.2, 0.3))
##' dat1$xchar1 <- rep(letters[1:26], length.out = N)
##' dat2 <- dat
##' dat1$x3 <- NULL
##' dat2$x2 <- NULL
##' dat2$xcat2 <- factor(sample(c("M", "F"), N, replace=TRUE),
##'                      levels = c("M", "F"), labels = c("Male", "Female"))
##' dat2$xcat3 <- factor(sample(c("K1", "K2", "K3", "K4"), N, replace=TRUE))
##' dat2$xchar1 <- "1"
##' dat3 <- dat
##' dat3$x1 <- NULL
##' dat3$xcat3 <-  factor(sample(c("L1", "L2", "L3", "L4"), N, replace=TRUE)) 
##' dat.stack <- rbindFill(dat1, dat2, dat3)
##' str(dat.stack)
##'
##' ## Possible BUG alert about base::rbind and plyr::rbind.fill
##' ## Demonstrate the problem of a same-named variable that is factor in one and
##' ## an ordered variable in the other
##' dat5 <- data.frame(ds = "5", x1 = rnorm(N),
##'                    xcat1 = gl(20, 5, labels = LETTERS[20:1]))
##' dat6 <- data.frame(ds = "6", x1 = rnorm(N),
##'                    xcat1 = gl(20, 5, labels = LETTERS[1:20], ordered = TRUE))
##' ## rbind reduces xcat1 to factor, whether we bind dat5 or dat6 first.
##' stack1 <- base::rbind(dat5, dat6)
##' str(stack1)
##' ## note xcat1 levels are ordered T, S, R, Q
##' stack2 <- base::rbind(dat6, dat5)
##' str(stack2)
##' ## xcat1 levels are A, B, C, D
##' ## stack3 <- plyr::rbind.fill(dat5, dat6)
##' ## str(stack3)
##' ## xcat1 is a factor with levels T, S, R, Q ...
##' ## stack4 <- plyr::rbind.fill(dat6, dat5)
##' ## str(stack4)
##' ## oops, xcat1 is ordinal with levels A < B < C < D
##' ## stack5 <- rbindFill(dat5, dat6)
rbindFill <- function(...) 
{
    ## small copy removeNULL
    removeNULL <- function (aList){ 
        Filter(Negate(is.null), aList)
    }

    dfs <- list(...)
    if (length(dfs) == 0) 
        stop("no data frame found")
    if (is.list(dfs[[1]]) && !is.data.frame(dfs[[1]])) {
        dfs <- dfs[[1]]
    }
    dfs <- removeNULL(dfs)
    if (length(dfs) == 0) 
        stop("no data frame found")
    if (length(dfs) == 1) 
        return(dfs[[1]])
    is_df <- vapply(dfs, is.data.frame, logical(1))
    if (any(!is_df)) {
        stop("All inputs to rbindFill must be data.frames", 
            call. = FALSE)
    }

    allclasses <- lapply(dfs, function(x) lapply(x, class))
    allnames <- unique(unlist(lapply(allclasses, names)))

    ## scan data frames and accumulate the levels of factor variables
    agglevels <- setNames(vector("list", length = length(allnames)), allnames)
    for(k in dfs){
        for(j in allnames){
            kjlevels <- if(any(j %in% names(k))) levels(k[[j]]) else NULL
            if(is.null(kjlevels)) next()
            agglevels[[j]] <- union(agglevels[[j]], kjlevels)
        }
    }
    agglevels <- removeNULL(agglevels)

    ## Classes of variables must be identical!
    ## if different data frames use same variable name for data of
    ## different classes, STOP with error
    aggclasses <- setNames(vector("list", length = length(allnames)), allnames)
    for(k in dfs){
        for(j in allnames){
            kjclasses <- if(any(j %in% names(k))) class(k[[j]]) else NULL
            if(is.null(kjclasses)) next()
            
            if (!is.null(aggclasses[[j]]) && !identical(aggclasses[[j]], kjclasses)){
                MESSG <- paste("Class mis-match between data.frames\n",
                               "Variable: ", j, "\n",
                               "(", paste(aggclasses[[j]], collapse = ", "), ")",
                               "versus",
                               "(", paste(kjclasses, collapse = ", "), ")")
                stop(MESSG)
            }
            aggclasses[[j]] <-kjclasses
        }
    }
        
    rowcount <- sapply(dfs, NROW)
    rowcountsum <- sum(rowcount)

    ## I have tested methods to allocate space for new stacked data
    ## Choose by if() statement here
    if(TRUE){
        ## Allocate a big block matrix, then
        ## coerce to a data.frame, then set the factor/ordered vars levels
        datastack <- as.data.frame(matrix(NA, nrow = rowcountsum,
                                        ncol = length(allnames),
                                        dimnames = list(NULL, allnames)))
        ## factor variables must have expanded level list
        for(k in names(agglevels)){
            datastack[ , k] <- factor(NA, levels = agglevels[[k]],
                                    ordered = "ordered" %in% aggclasses[k])
        }
    } else {
        ## Allocates storage for vectors of correct type in first
        ## step, then casts that a data frame. 
        datastack <- setNames(vector("list", length = length(allnames)), allnames)
        for(j in allnames) {
            varclass <- aggclasses[[j]][1]
            if (!varclass %in% c("factor", "ordered")){
                datastack[[j]] <- vector(aggclasses[[j]][1], length = rowcountsum)
            } else {
                datastack[[j]] <- factor(NA, levels = agglevels[[j]],
                                       ordered = "ordered" %in% aggclasses[j])
            }
        }
        datastack <- as.data.frame(datastack, stringsAsFactors = FALSE)
    }    

    ## place the data frames into the datastack object, using row and column indexing.
    for(i in seq_along(dfs)){
        start <- if(i == 1) 1 else start + rowcount[i-1]
        end <- if(i == 1) rowcount[i] else end + rowcount[i]
        if(i > 1){
            commonnames <- intersect(names(allclasses[[i-1]]),
                                     names(allclasses[[i]]))
            ## This is a duplicate column-class check, was written first, but
            ## previous check should make it unnecessary. If we trust previous check
            if(!identical(allclasses[[i-1]][commonnames], allclasses[[i]][commonnames])){
                MESSG <- paste("Classes of data frames are not identical:\n",
                               paste(allclasses[i-1], collapse = " "),
                               paste(allclasses[i], collapse = " "))        
                stop(MESSG)
            }
        }
        datastack[start: end, colnames(dfs[[i]])] <- dfs[[i]]
    }
    datastack
}
NULL

##' Assists creation of predicted value curves for regression models.
##'
##'
##' Creates a predicted value plot that includes a separate predicted
##' value line for each value of a focal variable. The x axis variable
##' is specified by the \code{plotx} argument. As of rockchalk 1.7.x,
##' the moderator argument, modx, is optional. Think of this a new
##' version of R's \code{termplot}, but it allows for
##' interactions. And it handles some nonlinear transformations more
##' gracefully than termplot.
##'
##' This is similar to \code{plotSlopes}, but it accepts regressions
##' in which there are transformed variables, such as "log(x1)".
##' It creates a plot of the predicted dependent
##' variable against one of the numeric predictors, \code{plotx}. It
##' draws a predicted value line for each value of \code{modx}, a
##' moderator variable. The moderator may be a numeric or categorical
##' moderator variable.
##'
##' The user may designate which particular values of the moderator
##' are used for calculating the predicted value lines.  That is,
##' \code{modxVals = c( 12,22,37)} would draw lines for values 12, 22,
##' and 37 of the moderator. User may instead supply a character
##' string to choose one of the built in algorithms. The default
##' algorithm is "quantile", which will select \code{n} values that
##' are evenly spaced along the \code{modx} axis. The algorithm
##' "std.dev" will select the mean of \code{modx} (m) and then it will
##' select values that step away from the mean in standard deviation
##' sd units. For example, if \code{n = 3}, the focal
##' values will \code{m, m - sd, am + sd}.
##'
##' @param model Required. Fitted regression object. Must have a
##' predict method
##' @param plotx Required. String with name of predictor for the x axis
##' @param plotxRange Optional. If not specified, the observed
##' range of plotx will be used to determine the axis range.
##' @param nx Number of values of plotx at which to calculate the predicted
##' value.  Default = 40.
##' @param modx Optional. String for moderator variable name. May be
##' either numeric or factor.
##' @param n Optional.  Number of focal values of \code{modx}, used by
##' algorithms specified by modxVals; will be ignored if modxVals
##' supplies a vector of focal values.
##' @param modxVals Optional. A vector of focal values for which
##' predicted values are to be plotted. May also be a character string
##' to select an algorithm ("quantile","std.dev." or "table"), or a
##' user-supplied function to select focal values (a new method
##' similar to \code{getFocal}). If modx is a factor, currently, the
##' only available algorithm is "table" (see \code{getFocal.factor}.
##' @param interval Optional. Intervals provided by the
##' \code{predict.lm} may be supplied, either "conf" (95% confidence
##' interval for the estimated conditional mean) or "pred" (95%
##' interval for observed values of y given the rest of the model).
##' @param plotPoints Optional. TRUE or FALSE: Should the plot include
##' the scatterplot points along with the lines.
##' @param plotLegend Optional. TRUE or FALSE: Should the default
##' legend be included?
##' @param legendTitle Optional. You'll get an automatically generated
##' title, such as "Moderator: modx", but if you don't like that,
##' specify your own string here.
##' @param legendPct Default = TRUE. Variable labels print with sample percentages.
##' @param col I offer my preferred color vector as default.
##'     Replace if you like. User may supply a vector of valid
##'     color names, or  \code{rainbow(10)} or
##'     \code{gray.colors(5)}. Color names will be recycled if there
##'     are more focal values of \code{modx} than colors provided.
##' @param envir environment to search for variables.
##' @param llwd Optional. Line widths for predicted values. Can be
##' single value or a vector, which will be recycled as necessary.
##' @param opacity Optional, default = 100. A number between 1 and
##' 255. 1 means "transparent" or invisible, 255 means very dark.  the
##' darkness of confidence interval regions
##' @param ... further arguments that are passed to plot or
##' predict. The arguments that are monitored to be sent to predict
##' are c("type", "se.fit", "dispersion", "interval", "level",
##' "terms", "na.action").
##' @export
##' @import carData
##' @return A plot is created as a side effect, a list is returned
##' including 1) the call, 2) a newdata object that includes
##' information on the curves that were plotted, 3) a vector modxVals,
##' the values for which curves were drawn.
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @example  inst/examples/plotCurves-ex.R
plotCurves <-
    function (model, plotx, nx = 40,  modx,  plotxRange = NULL, n, modxVals = NULL,
              interval = c("none", "confidence", "prediction"),
              plotPoints = TRUE, plotLegend = TRUE, legendTitle = NULL,
              legendPct = TRUE, 
              col = c("black", "blue", "darkgreen", "red", "orange", "purple", "green3"),
              llwd = 2, opacity = 100,
              envir = environment(formula(model)), ...)
{
    if (missing(model))
        stop("plotCurves requires a fitted regression model.")
    if (missing(plotx))
        stop("plotCurves requires the name of the variable to be drawn on the x axis")

    cl <- match.call()
    interval <- match.arg(interval)
    mf <- model.frame(model)
    mm <- model.data(model)

    zzz <- as.character(substitute(plotx))
    plotx <- zzz[1L]
    
    if (!missing(modx)) {
        zzz <- as.character(substitute(modx))
        modx <- zzz[1L]
    }
        
    plotxVar <- mm[ , plotx]
    if (!is.numeric(plotxVar))
        stop(paste("plotCurves: The variable", plotx, "should be a numeric variable"))

    ## Gamble on row names to select which cases are nonmissing
    depVar <- model.response(mf)[row.names(mm)]

    ylab <- names(mf)[1]  ## returns transformed DV
    plotxRange <- if(is.null(plotxRange)) range(plotxVar, na.rm = TRUE) else plotxRange
    plotxVals <- plotSeq(plotxRange, length.out = nx)

    ##Bug noticed 2013-09-22
    ## focalVals for a nonlinear model needs to be long set,
    ## not just end points as I imagined before. See 3 lines below
    if (missing(modx) || is.null(modx)) {
        modxVar <- rep(1, nobs(model))
        if (interval == "none") {
            ## focalVals <- list(plotxRange)
            focalVals <- list(plotxVals)
        } else {
            focalVals <- list(plotxVals)
        }
        names(focalVals) <- c(plotx)
        modx <- NULL
        modxVals <- 1
    } else {
        modxVar <- mm[ , modx]
        if (is.factor(modxVar)) { ## modxVar is a factor
            n <- ifelse(missing(n), nlevels(modxVar), n)
            modxVals <- getFocal(modxVar, xvals = modxVals, n, pct = legendPct)
        } else {
            n <- ifelse(missing(n), 3, n)
            modxVals <- getFocal(modxVar, xvals = modxVals, n, pct = legendPct)
        }

        focalVals <- list(modxVals, plotxVals)
        names(focalVals) <- c(modx, plotx)
    }

    newdf <- newdata(model, predVals = focalVals, emf = mm)

    dotargs <- list(...)
    dotnames <- names(dotargs)
    ## scan dotargs for predict keywords. Remove from dotargs
    ## the ones we only want going to predict. Leave
    ## others.

    level <- if (!is.null(dotargs[["level"]])) dotargs[["level"]] else 0.95

    parms <- list(model, newdata = newdf, type = "response" , interval = interval)
    ## type requires special handling because it may go to predict or plot
    if (!is.null(dotargs[["type"]])) {
        if (dotargs[["type"]] %in% c("response", "link", "none")) {
            parms[["type"]] <- dotargs[["type"]]
            dotargs[["type"]] <- NULL
        }
    }
    predArgs <- list()
    validForPredict <- c("se.fit", "dispersion", "level", "terms", "na.action")
    dotsForPredict <- dotnames[dotnames %in% validForPredict]

    if (length(dotsForPredict) > 0){
        parms <- modifyList(parms, dotargs[dotsForPredict])
        dotargs[[dotsForPredict]] <- NULL
    }
    np <- do.call("predictCI", parms)
    newdf <- cbind(newdf, np$fit)

    if ((!is.null(parms[["se.fit"]])) && (parms[["se.fit"]] == TRUE)) newdf <- cbind(newdf, np$se.fit)

    if ("ylim" %in% dotargs) {
        plotyRange <- dotargs$ylim
        dotargs$ylim <- NULL
    } else if (is.logical(depVar) || (is.factor(depVar) && length(levels(depVar)) == 2)) {
        plotyRange <- c(0, 1.2)
        ## Rescale depVar as 0 or 1 score
        depVar <- ifelse(depVar %in% levels(depVar)[1], 0, 1)
    } else if (is.numeric(depVar)) {
        plotyRange <- magRange(depVar, mult = c(1, 1.2))
    } else {
        stop("plotCurves: The dependent variable is neither numeric nor logical. What kind of model is this?")
    }
    
    parms <- list(newdf = newdf, olddf = mm,
                  plotx = plotx, modx = modx, modxVals = modxVals,
                  interval = interval, level = level, plotPoints = plotPoints,
                  plotLegend = plotLegend, legendTitle = legendTitle,
                  col = col,  opacity = opacity, xlim = plotxRange,
                  ylab = ylab, ylim = plotyRange, llwd = llwd)
    
    parms <- modifyList(parms, dotargs)
    plotArgs <- do.call("plotFancy", parms)

    z <- list(call = cl, newdata = newdf, modxVals = modxVals,
              col = plotArgs$col, lty = plotArgs$lty)

    class(z) <- "rockchalk"

    invisible(z)
}

##' Summary stats table-maker for regression users
##'
##' rockchalk::summarize does the numerical calculations
##' 
##' This is, roughly speaking, doing the right thing, but
##' not in a clever way. For the categorical variables, the
##' only summary is proportions.
##'
##' @param object A fitted regression or an R data.frame, or any
##'      other object type that does not fail in code{model.frame(object)}.
##' @param stats Default is a vector c("mean", "sd", "min", "max"). Other
##'      stats reported by rockchalk::summarize should work fine as well
##' @param digits 2 decimal points is default
##' @param probs Probability cut points to be used in the calculation
##'      of summaries of numeric variables.  Default is c(0, 0.5, 1), meaning
##'      \code{min, median, max}.
##' @param varLabels A named vector of variables labels, as in outreg function.
##'      Format is c("oldname"="newlabel").
##' @param ... Other arguments passed to rockchalk::summarizeNumerics and
##'      summarizeFactors.
##' @importFrom kutils mgsub
##' @export
##' @return a character matrix
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' dat <- genCorrelatedData2(1000, means=c(10, 10, 10), sds = 3, 
##'                           stde = 3, beta = c(1, 1, -1, 0.5))
##' dat$xcat1 <- factor(sample(c("a", "b", "c", "d"), 1000, replace=TRUE))
##' dat$xcat2 <- factor(sample(c("M", "F"), 1000, replace=TRUE), levels = c("M", "F"),
##' labels = c("Male", "Female"))
##' dat$y <- dat$y + contrasts(dat$xcat1)[dat$xcat1, ] %*% c(0.1, 0.2, 0.3)
##' m4 <- lm(y ~ x1 + x2  + x3 + xcat1 + xcat2, dat)
##' m4.desc <- descriptiveTable(m4)
##' m4.desc
##' ## Following may cause scientific notation, want to avoid.
##' dat <- genCorrelatedData2(1000, means=c(10, 100, 400), 
##'                  sds = c(3, 10, 20), stde = 3, beta = c(1, 1, -1, 0.5))
##' m5 <- lm(y ~ x1 + x2  + x3, dat)
##' m5.desc <- descriptiveTable(m5, digits = 4)
##' m5.desc
##' 
descriptiveTable <- function(object, stats = c("mean", "sd", "min", "max"), 
                            digits = 4, probs = c(0, .5, 1), varLabels, ...){
    mc <- match.call(expand.dots = TRUE)
    dots <- list(...)
    options.orig <- options()
    options(scipen=20)
    on.exit(options(options.orig))
    
    dat <- model.frame(object)
    arglist <- list(dat = dat, stats = stats, digits = digits, maxLevels = 20)
    arglist <- modifyList(arglist, dots)
    summ.dat <- do.call(rockchalk::summarize, arglist)
    reslt <- data.frame(variable = rownames(summ.dat[["numerics"]]), 
                                    summ.dat[["numerics"]][stats[stats %in% names(summ.dat[["numerics"]])]],
                        stringsAsFactors = FALSE)
    numbers <- names(which(sapply(reslt, is.numeric)))
    for(j in numbers) reslt[ , j] <- formatC(reslt[ , j], digits = digits)
    
    reslt2 <- vector("list", length = length(summ.dat[["factors"]]))
    names(reslt2) <- names(summ.dat[["factors"]])
    for(j in names(summ.dat[["factors"]])){
        tab <- summ.dat[["factors"]][[j]]
        tab.prop <- prop.table(tab[["table"]])
        reslt2[[j]] <- data.frame(variable=names(tab.prop), 
                                  mean = formatC(tab.prop, digits = digits), 
                                  stringsAsFactors = FALSE)
        reslt2[[j]] <- rbind(data.frame(variable = j, mean = "", 
                                        stringsAsFactors = FALSE), 
                             reslt2[[j]])
    }
   
    reslt3 <- do.call(rbind, reslt2)
    
    reslt4 <- rbindFill(reslt, reslt3)
    reslt4[is.na(reslt4)] <- ""
    if(!missing(varLabels) && !is.null(varLabels)){
        reslt4$variable <- kutils::mgsub(names(varLabels), varLabels, reslt4$variable)
    }
    reslt4
}

##' Create covariance matrix from correlation and standard deviation
##' information
##'
##' This is a flexible function that allows lazy R programmers to
##' create covariance matrix. The user may be lazy because the
##' correlation and standard deviation infomation may be supplied in a
##' variety of formats.
##'
##' @param Rho Required. May be a single value (correlation common
##' among all variables), a vector of the lower triangular values
##' (vech) of a correlation matrix, or a symmetric matrix of
##' correlation coefficients.
##' @param Sd Required. May be a single value (standard deviation
##' common among all variables) or a vector of standard deviations,
##' one for each variable.
##' @param d Optional. Number of rows or columns. lazyCov may be able
##' to deduce the required dimension of the final matrix from the
##' input. However, when the user supplies only a single value for
##' both Rho and Sd, d is necessary.
##' @return covariance matrix.
##' @author <pauljohn@@ku.edu>
##' @export
##' @examples
##' ##correlation 0.8 for all pairs, standard deviation 1.0 of each
##' lazyCov(Rho = 0.8, Sd = 1.0, d = 3)
##' ## supply a vech (lower triangular values in a column)
##' lazyCov(Rho = c(0.1, 0.2, 0.3), Sd = 1.0)
##' ## supply vech with different standard deviations
##' lazyCov(Rho = c(0.1, 0.2, 0.3), Sd = c(1.0, 2.2, 3.3))
##' newRho <- lazyCor(c(0.5, 0.6, 0.7, -0.1, 0.1, 0.2))
##' lazyCov(Rho = newRho, Sd = 1.0)
##' lazyCov(Rho = newRho, Sd = c(3, 4, 5, 6))
lazyCov <-
    function(Rho, Sd, d)
{
    if (missing(Sd)){
        MESSG <- paste("lazyCov requires user to specify either a vector",
                       "or a single common value for all standard deviations")
        stop(MESSG)
    }
    if (missing(Rho)){
        MESSG <- paste("lazyCov requires a symmstric correlation matrix",
                       "or enough information to create one, either a vech",
                       "of lower triangular values or a single common",
                       "correlation value")
        stop(MESSG)
    }
    if (!missing(d) && (length(Sd) > 1) && (length(Sd) != d)){
        MESSG <- paste("lazyCov doesn't require a d argument,",
                       "but if you provide one, it must be consistent",
                       "with the length of a supplied Sd vector")
        stop(MESSG)
    }
    if (missing(d)){
        if (length(Sd) > 1) d <- length(Sd)
        else if (is.matrix(Rho)) d <- NROW(Rho)
        else if (is.vector(Rho)) {
            d <- (sqrt(1 + 8 * length(Rho)) + 1)/2
            if (!isTRUE(all.equal(as.integer(d)- d, 0))){
                MESSG <- paste(deparse(substitute(vech)),
                               " must have the correct number of elelemnts",
                               "to fill in a strictly lower triangle in a",
                               "square matrix.")
                stop(MESSG)
            }
        }
    }
    if (length(Sd) == 1) Sd <- rep(Sd, d)
    Rho <- lazyCor(Rho, d)

    covMat <- diag(Sd) %*% Rho %*% diag(Sd)
    covSVD <- svd(covMat, nu = 0, nv = 0)
    eS <- eigen(covMat, symmetric = TRUE, only.values = TRUE)
    ev <- eS$values
    tol <- 1e-6
    if(!all(ev >= -tol*abs(ev[1L]))) stop("'covMat' is not positive definite")
    covMat
}

NULL


    

##' Draw a 3-D regression plot for two predictors from any linear or nonlinear lm or glm object
##'
##' This allows user to fit a regression model with many variables and
##' then plot 2 of its predictors and the output plane for those
##' predictors with other variables set at mean or mode (numeric or
##' factor).  This is a front-end (wrapper) for R's persp function.
##' Persp does all of the hard work, this function reorganizes the
##' information for the user in a more readily understood way.  It
##' intended as a convenience for students (or others) who do not
##' want to fight their way through the details needed to use persp to
##' plot a regression plane.  The fitted model can have any number of
##' input variables, this will display only two of them. And, at least
##' for the moment, I insist these predictors must be numeric
##' variables. They can be transformed in any of the usual ways, such
##' as poly, log, and so forth.
##'
##' Besides a fitted model object, plotPlane requires two additional
##' arguments, plotx1 and plotx2. These are the names of the plotting
##' variables. Please note, that if the term in the regression is
##' something like poly(fish,2) or log(fish), then the argument to
##' plotx1 should be the quoted name of the variable "fish".
##' plotPlane will handle the work of re-organizing the information so
##' that R's predict functions can generate the desired information.
##' This might be thought of as a 3D version of "termplot", with a
##' significant exception. The calculation of predicted values depends
##' on predictors besides plotx1 and plotx2 in a different ways. The
##' sample averages are used for numeric variables, but for factors
##' the modal value is used.
##'
##' This function creates an empty 3D drawing and then fills in the
##' pieces. It uses the R functions \code{lines}, \code{points}, and
##' \code{arrows}. To allow customization, several parameters are
##' introduced for the users to choose colors and such. These options
##' are prefixed by "l" for the lines that draw the plane, "p" for the
##' points, and "a" for the arrows. Of course, if plotPoints=FALSE or
##' drawArrows=FALSE, then these options are irrelevant.
##'
##' @param model an lm or glm fitted model object
##' @param plotx1 name of one variable to be used on the x1 axis
##' @param plotx2 name of one variable to be used on the x2 axis
##' @param drawArrows draw red arrows from prediction plane toward observed values TRUE or FALSE
##' @param plotPoints Should the plot include scatter of observed scores?
##' @param npp number of points at which to calculate prediction
##' @param x1lab optional label
##' @param x2lab optional label
##' @param ylab optional label
##' @param x1lim optional lower and upper bounds for x1, as vector like c(0,1)
##' @param x2lim optional lower and upper bounds for x2, as vector like c(0,1)
##' @param x1floor Default=5. Number of "floor" lines to be drawn for variable x1
##' @param x2floor Default=5. Number of "floor" lines to be drawn for variable x2
##' @param pch plot character, passed on to the "points" function
##' @param pcol color for points, col passed to "points" function
##' @param plwd line width, lwd passed to "points" function
##' @param pcex character expansion, cex passed to "points" function
##' @param llwd line width, lwd passed to the "lines" function
##' @param lcol line color, col passed to the "lines" function
##' @param llty line type, lty passed to the "lines" function
##' @param acol color for arrows, col passed to "arrows" function
##' @param alty arrow line type, lty passed to the "arrows" function
##' @param alwd arrow line width, lwd passed to the "arrows" function
##' @param alength arrow head length, length passed to "arrows" function
##' @param linesFrom object with information about "highlight" lines to be added to the 3d plane (output from plotCurves or plotSlopes)
##' @param lflwd line widths for linesFrom highlight lines
##' @param envir environment from whence to grab data
##' @param ... additional parameters that will go to persp
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @rdname plotPlane
##' @export plotPlane
##' @importFrom graphics lines
##' @seealso \code{\link[graphics]{persp}}, \code{\link[scatterplot3d]{scatterplot3d}}, \code{\link[HH]{regr2.plot}}
##' @example inst/examples/plotPlane-ex.R

plotPlane <-
function(model = NULL,  plotx1 = NULL, plotx2 = NULL, drawArrows = FALSE,
         plotPoints = TRUE, npp = 20, x1lab, x2lab, ylab, x1lim, x2lim,
         x1floor = 5, x2floor = 5,  pch = 1, pcol = "blue", plwd = 0.5,
         pcex = 1, llwd = 0.3, lcol = 1, llty = 1, acol = "red", alty = 4,
         alwd = 0.3, alength = 0.1, linesFrom,
         lflwd = 3, envir = environment(formula(model)),  ...){
    UseMethod("plotPlane")
}


##' @return The main point is the plot that is drawn, but for record
##' keeping the return object is a list including 1) res: the
##' transformation matrix that was created by persp 2) the call that
##' was issued, 3) x1seq, the "plot sequence" for the x1 dimension, 4)
##' x2seq, the "plot sequence" for the x2 dimension, 5) zplane, the
##' values of the plane corresponding to locations x1seq and x2seq.
##' @rdname plotPlane
##' @method plotPlane default
##' @export

plotPlane.default <-
function(model = NULL, plotx1 = NULL, plotx2 = NULL, drawArrows = FALSE,
         plotPoints = TRUE, npp = 20, x1lab, x2lab, ylab, x1lim, x2lim, 
         x1floor = 5, x2floor = 5, pch = 1, pcol = "blue", plwd = 0.5,
         pcex = 1, llwd = 0.3, lcol = 1, llty = 1, acol = "red", alty = 4,
         alwd = 0.3, alength = 0.1, linesFrom, lflwd = 3,
         envir = environment(formula(model)), ...){


    if (is.null(model))
        stop("plotPlane requires a fitted regression model.")
    if (is.null(plotx1) | is.null(plotx2))
        stop("plotPlane requires the name of the variable to be drawn on the x axis")
    if (plotx1 == plotx2) stop("the two plotting variables should not be the same")
    cl <- match.call()
    mf <- model.frame(model) ##y first, no intercept

    ## The dependent variable data column
    y <- model.response(mf)
    ## Need a dataframe of raw data
    emf <- model.data(model)
    varnames <- attr(emf, "varNamesRHS")

    if (!plotx1 %in% varnames) stop(paste("plotx1 variable,", plotx1, ", is not in fitted model"))
    if (!plotx2 %in% varnames) stop(paste("plotx2 variable,", plotx2, ", is not in fitted model"))
                                    
    if (!missing(x1lim))
        emf <- subset(emf, emf[ , plotx1] >= x1lim[1] & emf[ , plotx1] <= x1lim[2])
    if (!missing(x2lim))
        emf <- subset(emf, emf[ , plotx2] >= x2lim[1] & emf[ , plotx2] <= x2lim[2])
    
    x1 <- emf[, plotx1]
    x2 <- emf[, plotx2]
    if (!is.numeric(x1))
        stop(paste("plotPlane: The variable", plotx1, "should be a numeric variable"))
    if (!is.numeric(x2))
        stop(paste("plotPlane: The variable", plotx2, "should be a numeric variable"))
    x1range <- magRange(x1, 1.15)
    x2range <- magRange(x2, 1.15)
  
    if (missing(ylab)) ylab <- colnames(model$model)[1]
    if (missing(x1lab)) x1lab <- plotx1
    if (missing(x2lab)) x2lab <- plotx2

   
    if (missing(x2lim)) x2range <- magRange(x2, 1.15) else x2range <- magRange(x2lim, 1.15)

    ##TODO must double check effect of function predictors
    otherPredictors <- setdiff(varnames, c(plotx2, plotx1)) #remove x1 x2
    if (length(otherPredictors) > 0) {
        otherPredictorValues <- centralValues(emf[ , otherPredictors, drop = FALSE])
    }

    myPredict <- function(a,b){
        ndf <- data.frame(a, b) #ndf = new data frame
        colnames(ndf) <- c(plotx1, plotx2)
        if (length(otherPredictors) > 0) {
            ndf <- cbind(ndf, otherPredictorValues)
            colnames(ndf) <- c(plotx1, plotx2, otherPredictors)
        }
        if ("glm" %in% class(model)) {
            if ("mcreg" %in% class(model)) attr(ndf, "isCentered") <- TRUE
            predict(model, newdata = ndf, type = "response")
        } else {
            if ("mcreg" %in% class(model)) attr(ndf, "isCentered") <- TRUE
            predict(model, newdata = ndf)
        }
    }

    x1seq <- plotSeq(x1range, length.out = npp)
    x2seq <- plotSeq(x2range, length.out = npp)
    zplane <- outer(x1seq, x2seq, function(a, b) { myPredict(a,b) } )

    yrange <- magRange(c(zplane,y), 1.15)

    ## must block users from trying to send wrong messages to persp.
    ## this is brutish, but don't have time figure more eloquent way.
    dotargs <- list(...)
    dotargs[["xlim"]] <- NULL
    dotargs[["ylim"]] <- NULL
    dotargs[["zlim"]] <- NULL

    argList <- list(x1 = plotSeq(x1range, x1floor),
                      x2 = plotSeq(x2range, x2floor), y = yrange,
                      x1lab = x1lab, x2lab = x2lab, ylab = ylab)
    newArgList <- modifyList(dotargs, argList)
    
    res <- do.call("perspEmpty", newArgList)

    ##for arrows. NEEDS reworking to be more general
    if ("glm" %in% class(model)) {
        fits <-  predict(model, type = "response")
    }else{
        fits <-  fitted(model)
    }

    mypoints4 <- trans3d(x1, x2, fits, pmat = res)
    newy <- ifelse(fits < y, fits + 0.8 * (y - fits),
                   fits + 0.8 * (y - fits))
    mypoints2s <- trans3d(x1, x2, newy, pmat = res)
    if (drawArrows){
        lmp <- length(fits)
        if (length(acol) > 1 && length(acol) < lmp) acol <- rep(acol, length.out = lmp)
        if (length(alty) > 1 && length(alty) < lmp) acol <- rep(acol, length.out = lmp)
        if (length(alwd) > 1 && length(alwd) < lmp) alwd <- rep(alwd, length.out = lmp)
        if (length(alength) > 1 && length(alength) < lmp) alwd <- rep(alength, length.out = lmp)
        arrows(mypoints4$x, mypoints4$y, mypoints2s$x, mypoints2s$y,
               col = acol, lty = alty, lwd = alwd, length = alength)
    }

    if (plotPoints){
        mypoints2 <- trans3d(x1, x2, y, pmat = res)
        lmp <- length(y)
        if (length(pch) > 1 && length(pch) < lmp) pch <- rep(pch, length.out = lmp)
        if (length(pcol) > 1 && length(pcol) < lmp) pcol <- rep(pcol, length.out = lmp)
        if (length(plwd) > 1 && length(plwd) < lmp) plwd <- rep(plwd, length.out = lmp)
        if (length(pcex) > 1 && length(pcex) < lmp) pcex <- rep(pcex, length.out = lmp)

        points(mypoints2, pch = pch, col = pcol, lwd = plwd, cex = pcex)
    }

    ##recycles parameters as necessary
    if (length(llwd) > 1 && length(llwd) < npp) llwd <- rep(llwd, length.out = npp)
    if (length(lcol) > 1 && length(lcol) < npp) lcol <- rep(lcol, length.out = npp)
    if (length(llty) > 1 && length(llty) < npp) llty <- rep(llty, length.out = npp)

    for (i in 1:npp) {
        lines(trans3d(x1seq[i], x2seq, zplane[i, ], pmat = res),
              lwd = llwd, col = lcol, lty= llty)
    }

    for (j in 1:npp) {
        lines(trans3d(x1seq, x2seq[j], zplane[, j], pmat = res),
              lwd = llwd, col = lcol, lty = llty)
    }


    if (!missing(linesFrom)) {
        dataSplits <- split(linesFrom$newdata, f = linesFrom$newdata[[linesFrom$call[["modx"]]]])
        drawLine <- function(nd, mycol, mylty){
            lines(trans3d(nd[[plotx1]], nd[[plotx2]], nd$fit, pmat=res), col = mycol, lwd = lflwd, lty = mylty)
        }
        mapply(drawLine, dataSplits, linesFrom$col, linesFrom$lty)
    }

    retval <- list(res = res, call = cl, "x1seq" = x1seq, "x2seq" = x2seq, "zplane" = zplane)
    class(retval) <- "rockchalk3d"
    invisible(retval)
}
NULL


## New for version 1.7.x

##' Superimpose regression lines on a plotted plane
##'
##' The examples will demonstrate the intended usage.
##'
##' From an educational stand point, the objective is to assist with the
##' student's conceptualization of the two and three dimensional regression
##' relationships.
##' @param to a 3d plot object produced by plotPlane
##' @param from output from a plotSlopes or plotCurves function (class="rockchalk")
##' @param col color of plotted lines (default: "red")
##' @param lwd line width of added lines (default: 2)
##' @param lty line type of added lines (default: 1)
##' @return NULL, nothing, nicht, nada.
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @export
##' @example inst/examples/addLines-ex.R

addLines <- function(to = NULL, from = NULL, col, lwd = 2, lty = 1){
    if (!inherits(from, "rockchalk"))
        stop("addLines: from must be an output object from plotSlopes or plotCurves, of class rockchalk")
    if (!inherits(to, "rockchalk3d"))
        stop("addLines: to must be a 3d plot object created by plotPlane or such")
    if ( !(from$call[["modx"]] %in% to$call[["plotx1"]] | from$call[["modx"]] %in% to$call[["plotx2"]]) )
        stop("Mismatched plotPlanes and plotSlopes objects")

    dataSplits <- split(from$newdata, f = from$newdata[[from$call[["modx"]]]])
    if(missing(col)) {
        col = from$col
    } else {
        if (length(col) < length(dataSplits)) col <- rep(col, length.out = length(dataSplits))
    }

    if (length(lwd) < length(dataSplits)) lwd <- rep(lwd, length.out = length(dataSplits))
    i <- 0
    for (i in seq_along(dataSplits)) {
        nd <- dataSplits[[i]]
        lines(trans3d(nd[[to$call$plotx1]], nd[[to$call$plotx2]],
                      nd$fit, pmat = to$res), col = col[i], lwd = lwd[i], lty = lty);
    }
    NULL
}

##' Remove NULL values variables from a list
##'
##' Unlike vectors, lists can hold objects with
##' value NULL. This gets rid of them.
##'
##' This version is NOT recursive
##'
##' plyr::rbind.fill uses an experimental function
##' that I choose to avoid. This is the "safe" version.
##' 
##' @param aList A list
##' @return Same list with NULL's removed
##' @author Paul Johnson
##' @export removeNULL
##' @examples
##' ## Note it is non-recursive, NULL remains in e
##' x <- list(a = rnorm(5), b = NULL, c = rnorm(5), d = NULL,
##'      e = list(f = rnorm(2), g = NULL))
##' x
##' removeNULL(x)
removeNULL <- function (aList){ 
    Filter(Negate(is.null), aList)
}

##' Generic function for plotting regressions and interaction effects
##'
##' This is a function for plotting regression
##' objects. So far, there is an implementation for \code{lm()} objects.
##' I've been revising plotSlopes so that it should handle the work
##' performed by plotCurves.  As sure as that belief is verified,
##' the plotCurves work will be handled by plotSlopes. Different
##' plot types are created, depending on whether the x-axis predictor
##' \code{plotx} is numeric or categorical.
##' ##'
##' @param model Required. A fitted Regression
##' @param plotx Required. Name of one predictor from the fitted
##' model to be plotted on horizontal axis. May be numeric or factor.
##' @param ... Additional arguments passed to methods. Often includes
##' arguments that are passed to plot. Any arguments that customize
##' plot output, such as lwd, cex, and so forth, may be
##' supplied. These arguments intended for the predict method will be
##' used: c("type", "se.fit", "interval", "level", "dispersion",
##' "terms", "na.action")
##' @export plotSlopes
##' @importFrom graphics abline arrows legend lines mtext plot points polygon text
##' @importFrom methods as
##' @importFrom stats AIC addmargins as.formula coef cor
##'             df.residual drop1 family fitted formula lm
##'             model.frame model.matrix model.response na.omit na.pass
##'             napredict nobs pchisq pf predict
##'             predict.lm pt qnorm qt quantile resid
##'             rnorm sd setNames terms var vcov
##' @importFrom utils browseURL methods modifyList 
##' @rdname plotSlopes
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @seealso \code{\link[rockchalk]{testSlopes}} \code{\link[rockchalk]{plotCurves}}
##' @return Creates a plot and an output object that summarizes it.
plotSlopes <- function(model, plotx, ...) UseMethod("plotSlopes")

##' Plot predicted values for focal values of a moderator variable.
##'
##' This is a "simple slope" plotter for regression objects created by
##' \code{lm()} or similar functions that have capable predict methods
##' with newdata arguments. The term "simple slopes" was coined by
##' psychologists (Aiken and West, 1991; Cohen, et al 2002) for
##' analysis of interaction effects for particular values of a
##' moderating variable. The moderating variable may be continuous or
##' categorical, lines will be plotted for focal values of that
##' variable.
##'
##' The original \code{plotSlopes} did not work well with nonlinear
##' predictors (log(x) and poly(x)). The separate function
##' \code{plotCurves()} was created for nonlinear predictive equations
##' and generalized linear models, but the separation of the two
##' functions was confusing for users. I've been working to make
##' plotSlopes handle everything and \code{plotCurves} will disappear
##' at some point.  \code{plotSlopes} can create an object which
##' is then tested with \code{testSlopes()} and that can be graphed
##' by a plot method.
##'
##' The argument \code{plotx} is the name of the horizontal plotting
##' variable. An innovation was introduced in Version 1.8.33 so that
##' \code{plotx} can be either  numeric or categorical.
##'
##' The argument \code{modx} is the moderator variable. It may be
##' either a numeric or a factor variable. As of version 1.7, the modx
##' argument may be omitted. A single predicted value line will be
##' drawn. That version also introduced the arguments interval and n.
##'
##' There are many ways to specify focal values using the arguments
##' \code{modxVals} and \code{n}. This changed in rockchalk-1.7.0.  If
##' \code{modxVals} is omitted, a default algorithm for the variable
##' type will be used to select \code{n} values for
##' plotting. \code{modxVals} may be a vector of values (for a numeric
##' moderator) or levels (for a factor).  If modxVals is a vector of
##' values, then the argument \code{n} is ignored.  However, if
##' modxVals is one of the name of one of the algorithms, "table",
##' "quantile", or "std.dev.", then the argument \code{n} sets number
##' of focal values to be selected.  For numeric \code{modx}, n
##' defaults to 3, but for factors \code{modx} will be the number of
##' observed values of \code{modx}. If modxVals is omitted, the
##' defaults will be used ("table" for factors, "quantile" for numeric
##' variables).
##'
##' For the predictors besides \code{modx} and \code{plotx} (the ones
##' that are not explicitly included in the plot), predicted values
##' are calculated with variables set to the mean and mode, for numeric
##' or factor variables (respectively). Those values can be reviewed
##' in the newdata object that is created as a part of the output from
##' this function
##'
##' @param plotxRange Optional. If not specified, the observed range
##'     of plotx will be used to determine the axis range.
##' @param modx Optional. String for moderator variable name. May be
##'     either numeric or factor. If omitted, a single predicted value
##'     line will be drawn.
##' @param n Optional. Number of focal values of \code{modx}, used by
##'     algorithms specified by modxVals; will be ignored if modxVals
##'     supplies a vector of focal values.
##' @param modxVals Optional. Focal values of \code{modx} for which
##'     lines are desired. May be a vector of values or the name of an
##'     algorithm, "quantile", "std.dev.", or "table".
##' @param interval Optional. Intervals provided by the
##'     \code{predict.lm} may be supplied, either "confidence"
##'     (confidence interval for the estimated conditional mean) or
##'     "prediction" (interval for observed values of y given the rest
##'     of the model).  The level can be specified as an argument
##'     (which goes into ...  and then to the predict method)
##' @param plotPoints Optional. TRUE or FALSE: Should the plot include
##'     the scatterplot points along with the lines.
##' @param legendPct Default = TRUE. Variable labels print with sample
##'     percentages.
##' @param legendArgs Set as "none" if no legend is
##'     desired. Otherwise, this can be a list of named arguments that
##'     will override the settings I have for the legend.
##' @param col Optional.I offer my preferred color vector as default.
##'     Replace if you like. User may supply a vector of valid
##'     color names, or  \code{rainbow(10)} or
##'     \code{gray.colors(5)}. Color names will be recycled if there
##'     are more focal values of \code{modx} than colors provided.
##' @param llwd Optional, default = 2. Line widths for predicted
##'     values. Can be single value or a vector, which will be
##'     recycled as necessary.
##' @param opacity Optional, default = 100. A number between 1 and
##'     255.  1 means "transparent" or invisible, 255 means very dark.
##'     Determines the darkness of confidence interval regions
##' @param type Argument passed to the predict function. If model is glm,
##'     can be either "response" or "link". For lm, no argument of this
##'     type is needed, since both types have same value.
##' @param gridArgs Only used if plotx (horizontal axis) is a factor
##'     variable. Designates reference lines between values. Set as
##'     "none" if no grid lines are needed.  Default will be
##'     \code{gridArgs = list(lwd = 0.3, lty = 5)}
##' @param width Only used if plotx (horizontal axis) is a factor. Designates
##'     thickness of shading for bars that depict confidence intervals.
##' @export
##' @method plotSlopes lm
##' @rdname plotSlopes
##' @import carData
##' @return The return object includes the "newdata" object that was
##'     used to create the plot, along with the "modxVals" vector, the
##'     values of the moderator for which lines were drawn, and the
##'     color vector. It also includes the call that generated the
##'     plot.
##' @references
##'
##' Aiken, L. S. and West, S.G. (1991). Multiple
##' Regression: Testing and Interpreting Interactions. Newbury
##' Park, Calif: Sage Publications.
##'
##' Cohen, J., Cohen, P., West, S. G., and Aiken, L. S. (2002).
##' Applied Multiple Regression/Correlation Analysis for the Behavioral
##' Sciences (Third.). Routledge Academic.
##' @example inst/examples/plotSlopes-ex.R
plotSlopes.lm <-
    function (model, plotx, modx = NULL, n = 3, modxVals = NULL ,
              plotxRange = NULL, interval = c("none", "confidence", "prediction"),
              plotPoints = TRUE, legendPct = TRUE, legendArgs,  
              llwd = 2, opacity = 100, ...,
              col = c("black", "blue", "darkgreen", "red", "orange", "purple", "green3"),
              type = c("response", "link"),  gridArgs, width = 0.2)
{
    if (missing(model))
        stop("plotSlopes requires a fitted regression model.")
    if (missing(plotx)) stop("plotSlopes: plotx argument is required")
    else if(!is.character(plotx)) plotx <-as.character(substitute(plotx))

    ## Explanation of variable names
    ## "Vals" means focal values
    ## "Var"  means observed values
    ## "plotx" and "modx" will be reduced to character for variable names
    
    cl <- match.call()
    dotargs <- list(...)
    dotnames <- names(dotargs)
    level <- if (!is.null(dotargs[["level"]])) dotargs[["level"]] else 0.95
    
    mm <- model.data(model)
    plotxVar <- mm[, plotx]

    ## ylab is name of dependent variable, 
    ylab <- as.character(terms(model)[[2]])

    interval <- match.arg(interval)
    type <- match.arg(type)
    
    if (!missing(modx)) {
        zzz <- as.character(substitute(modx))
        if(!is.character(modx)) modx <- as.character(substitute(modx))
    }

    ## gets "raw" untransformed data
    ## depVar <- model.data[ , ylab]
    ## try to get the logged or transformed value???
    ## should be same as model.response(model.frame())??
    depName <- as.character(terms(model)[[2]])
    depVar <- mm[[depName]]
    
    if(is.null(plotxRange)){
        if (is.numeric(plotxVar)){
            plotxRange <- range(plotxVar, na.rm = TRUE)
        }
    }

    ## We are not allowing user to select categories (yet)
    ## TODO Check what character variable does here!
    if (is.factor(plotxVar)){
        plotxVals <- levels(droplevels(plotxVar))
    } else {
        plotxVals <- plotSeq(plotxRange, length.out = 40)
    }

    if (!(missing(modx) || is.null(modx))) {
        if((missing(modxVals) || is.null(modxVals)) || (length(modxVals) == 1 && is.character(modxVals))) {
            modxVar <- mm[, modx]
            if (is.factor(modxVar)) { ## modxVar is a factor
                n <- ifelse(missing(n), nlevels(modxVar), n)
                modxVar <- droplevels(modxVar)
                modxVals <- getFocal(modxVar, xvals = modxVals, n, pct = legendPct)
            } else {
                modxVals <- getFocal(modxVar, xvals = modxVals, n, pct = legendPct)
            }
        }
    }
            
    predVals <- list(plotxVals)
    names(predVals) <- plotx
    if(!is.null(modx)) {
        if(is.null(modxVals)) modxVals <- "table"
        TEXT <- paste0("predVals[[\"", modx, "\"]] <- modxVals")
        eval(parse(text = TEXT))
    }
    
    parms.pred <- list(model, predVals = predVals, type = type, interval = interval)
    validForPredict <- c("se.fit", "dispersion", "terms", "na.action",
                         "level", "pred.var", "weights")
    dotsForPredict <- dotnames[dotnames %in% validForPredict]
    if (length(dotsForPredict) > 0) {
        parms.pred <- modifyList(parms.pred, dotargs[dotsForPredict])
        dotargs[[dotsForPredict]] <- NULL
    }
  
    newdf <-  do.call("predictOMatic", parms.pred)

    ## ignores confidence interval height here. Why?
    plotyRange <- if(is.numeric(depVar)){
        magRange(depVar, mult = c(1, 1.2))
    } else {
        stop(paste("plotSlopes: use plotCurves if this is a glm (logit, probit, count)"))
    }

    if(is.numeric(plotxVals)){
        
        parms <- list(newdf = newdf, olddf = mm, plotx = plotx, modx = modx,
                       modxVals = modxVals, depName = depName, 
                       interval = interval, level = level, plotPoints = plotPoints,
                       col = col, opacity = opacity, xlim = plotxRange,
                       ylim = plotyRange, ylab = ylab, llwd = llwd)
        if(!missing(legendArgs) && !is.null(legendArgs)) parms[["legendArgs"]] <- legendArgs
        parms <- modifyList(parms, dotargs)
        plotret <- do.call("plotFancy", parms)
    } else {
       
        plotyRange <- magRange(range(c(newdf[["fit"]], if(!is.null(newdf[["lwr"]])) min(newdf[["lwr"]])),
                                   if(!is.null(newdf[["upr"]])) max(newdf[["upr"]])), 1.25)
               
        parms <- list(newdf, olddf = mm,  plotx = plotx, modx = modx,
                      modxVals = modxVals, depName = depName,
                      xlim = plotxRange, interval = interval, level = level,
                      xlab = plotx, ylab = ylab,  ylim = plotyRange,
                      main = "", col = col, width = width)
        if(!missing(legendArgs) && !is.null(legendArgs)) parms[["legendArgs"]] <- legendArgs
        if(!missing(gridArgs) && !is.null(gridArgs)) parms[["gridArgs"]] <- gridArgs
        parms <- modifyList(parms, dotargs)
        plotret <- do.call("plotFancyCategories", parms)
    }
        
    z <- list(call = cl, newdata = newdf, modxVals = modxVals, col = plotret$col, lty = plotret$lty, fancy = plotret)
    class(z) <- c("plotSlopes", "rockchalk")

    invisible(z)
}
NULL

##' Regression plots with predicted value lines, confidence intervals, color coded interactions
##'
##' This is the back-end for the functions plotSlopes and plotCurves. Don't use it directly.
##'
##' @param newdf The new data frame with predictors and fit, lwr, upr
##'     variables
##' @param olddf A data frame with variables(modxVar, plotxVar,
##'     depVar)
##' @param plotx Character string for name of variable on horizontal
##'     axis
##' @param modx Character string for name of moderator variable.
##' @param modxVals Values of moderator for which lines are desired
##' @param interval TRUE or FALSE: want confidence intervals?
##' @param plotPoints TRUE or FALSE: want to see observed values in
##'     plot?
##' @param legendArgs Set as "none" for no legend. Otherwise, a list
##'     of arguments for the legend function
##' @param col requested color scheme for lines and points. One per
##'     value of modxVals.
##' @param llwd requested line width, will re-cycle.
##' @param opacity Value in 0, 255 for darkness of interval shading
##' @param ... Other arguments passed to plot function.
##' @return col, lty, and lwd information
##' @importFrom methods formalArgs
##' @importFrom graphics par
##' @importFrom graphics plot.default
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##'
plotFancy <-
    function(newdf, olddf, plotx, modx, modxVals, interval, plotPoints,
             legendArgs, col = NULL, llwd = 2, opacity, ...)
{
    dotargs <- list(...)
    
    ## Damn. Need ylab from dotargs explicitly
    ylab <- dotargs[["depName"]]
    if (!is.null(dotargs[["ylab"]])) ylab <- dotargs[["ylab"]]
    ## Also need level
    if (!is.null(dotargs[["level"]])) {
        level <- dotargs[["level"]]
        dotargs[["level"]] <- NULL
    } else {
        level <- 0.95
    }

    ## sort rows
    newdf <- if(is.null(modx)) newdf[order(newdf[[plotx]]), ]
                   else newdf[order(newdf[[plotx]], newdf[[modx]]), ]
    
    modxVar <- if(is.null(modx)) rep(1, NROW(olddf)) else olddf[ , modx]
    if(is.factor(modxVar)) modxVar <- droplevels(modxVar)
    plotxVar <- olddf[ , plotx]
    depVar <- olddf[ , 1] ## column 1 is observed dv fix me here with name

    ## Now begin the plotting work.
    if (missing(modx) || is.null(modx)) {
        modxVals <- 1
        lmx <- 1
    } else {
        lmx <- length(modxVals)
    }

  
    ## if modx is a factor's name, we want to use all the levels
    ## to set the color scheme, even if some are not used in this
    ## particular plot.
    if (is.factor(modxVar)) {
        modxLevels <- levels(modxVar)
    } else {
        modxLevels <- modxVals
        if (is.null(names(modxVals))) names(modxVals) <- modxVals
    }
 
  

    
    ## Deal w colors
    if (is.null(col)) {
        if (is.factor(modxVar)) {
            col <- seq_along(modxLevels)
            names(col) <- modxLevels
        }  else {
            col <- 1:lmx
            names(col) <- names(modxVals)
        }
    } else {
        if (is.null(names(col))){
            col <- rep(col, length.out = length(modxLevels))
            if (is.factor(modxVar)) names(col) <- modxLevels
            else names(col) <- names(modxVals)
        } else if (length(col) < lmx) {
            stop("plotFancy: named color vector does not match data.")
        } else if (length(col) >= length(modxLevels)) {
            col <- rep(col, length.out = length(modxLevels))
            if (is.null(names(col))) {
                names(col) <- names(modxLevels[1:length(col)])
            }
        }
    }

    ## We have colors set for all levels, now
    ## drop colors to values actually used
    col <-  if (is.factor(modxVar))  col[names(col) %in% modxVals] else col[1:length(modxVals)]
    ## Now only modxLevels remaining should be in modxVals
    modxLevels <- modxLevels[modxLevels %in% modxVals]
    
    if (length(modxLevels) != length(modxVals)) {stop("goof")}
    
    ## Deal w line widths
    if (length(llwd) < length(col)) {
        llwd <- rep(llwd, length.out = length(col))
    }
    names(llwd) <- names(col)

    lty <- seq_along(col)
    if (!is.null(dotargs[["lty"]])) {
        lty <-  rep(dotargs[["lty"]], length.out = lmx)
        dotargs[["lty"]] <- NULL ## erase
    }
    names(lty) <- names(col)
    parms <- list(x = plotxVar, y = depVar, xlab = plotx, ylab = ylab,
                  type = "n")
    dots.plot <- dotargs[names(dotargs)[names(dotargs) %in% c(names(par()), formalArgs(plot.default))]]
    parms <- modifyList(parms, dots.plot)
    do.call("plot", parms)

    ## iCol: rgb color matrix. Why does rgb insist the columns be
    iCol <- col2rgb(col)
    ### bCol: border color

    bCol <- mapply(rgb, red = iCol[1,], green = iCol[2,],
                   blue = iCol[3,], alpha = opacity, maxColorValue = 255)
    ### sCol: shade color
    sCol <-  mapply(rgb, red = iCol[1,], green = iCol[2,],
                    blue = iCol[3,], alpha = opacity/3, maxColorValue = 255)

    if (interval != "none") {
        for (j in modxVals) {
            k <- match(j, modxVals)   ##integer index
            if (is.factor(modxVar)) i <- j  ## level names
            else i <- k  ## i integer

            if (missing(modx) || is.null(modx)) {
                pdat <- newdf
            } else {
                pdat <- newdf[newdf[ , modx] %in% j, ]
            }
            parms <- list(x = c(pdat[, plotx], pdat[NROW(pdat):1 , plotx]),
                          y = c(pdat$lwr, pdat$upr[NROW(pdat):1]), lty = lty[i])
            dots.plot <- dotargs[names(dotargs)[names(dotargs) %in% c(names(par()), formalArgs(polygon))]]
            parms <- if(!is.null(dots.plot)) modifyList(parms, dots.plot)
            parms <- modifyList(parms, list(border = bCol[i],
                                            col = sCol[i], lwd = 0.3* llwd[k]))
            
            do.call("polygon", parms)
        }
    }

    for (j in modxVals) {
        if (is.factor(modxVar)) i <- j  ## level names
        else i <- match(j, modxVals)   ##integer index
        if (missing(modx) || is.null(modx)) {
            pdat <- newdf
        } else {
            pdat <- newdf[newdf[ , modx] %in% j, ]
        }
        parms <- list(x = pdat[, plotx], y = pdat$fit, lty = lty[i])
        dots.plot <- dotargs[names(dotargs)[names(dotargs) %in% c(names(par()), formalArgs(lines))]]
        parms <- if(!is.null(dots.plot)) modifyList(parms, dots.plot)
        parms <- modifyList(parms, list(col = col[i], lwd = llwd[i]))
        do.call("lines", parms)
    }


    if (plotPoints) {
        parms <- list(xlab = plotx, ylab = ylab,
                      cex = 0.6, lwd = 0.75)
        if (is.factor(modxVar)) {
            parms[["col"]] <- col[as.vector(modxVar[modxVar %in% modxVals])]
            parms[["x"]] <- plotxVar[modxVar %in% modxVals]
            parms[["y"]] <- depVar[modxVar %in% modxVals]
        } else {
            parms[["col"]] <- 1
            parms[["x"]] <- plotxVar
            parms[["y"]] <- depVar
        }
        dots.plot <- dotargs[names(dotargs)[names(dotargs) %in% c(names(par()), formalArgs(points))]]
        parms <- if(!is.null(dots.plot)) modifyList(parms, dots.plot)
        do.call("points", parms)
    }

    if (!missing(legendArgs) && is.character(legendArgs) && (legendArgs == "none")) {
        ## "do nothing"
    } else if (!is.null(modx)){
        ## guess legend parms from modx information
       if (is.factor(modxVar)){ ## level names
            col <- col[as.vector(modxVals)]
            lty <- lty[as.vector(modxVals)]
            llwd <- llwd[as.vector(modxVals)]
        } else {
            col <- col[names(modxVals)]
            lty <- lty[names(modxVals)]
            llwd <- llwd[names(modxVals)]
        }
        titl <- paste("Moderator:", modx)
        if (is.null(names(modxVals))) {
            legnd <- paste(modxVals, sep = "")
        } else {
            legnd <- paste(names(modxVals), sep = "")
        }
        
        legend.parms <- list(x = "topleft", legend = legnd, lty = lty, col = col,
                             lwd = llwd, bg = "white", title = titl)
        if(!missing(legendArgs) && !is.null(legendArgs)) legend.parms <- modifyList(legend.parms, legendArgs)
        do.call(legend, legend.parms)
    } else {
        legend.parms <- list()
        legend.parms[["x"]] <- "topleft"
        legend.parms[["title"]] <-  "Regression analysis"
        legnd <- c("Predicted values")
        col <- col[1]
        lty <- lty[1]
        llwd <- llwd[1]
        if (interval != "none") {
            legnd[2] <- paste(level, interval, "interval")
            col <- c(col, 0)
            lty <- c(lty, 0)
            llwd <- c(llwd, 0)
        }
        legend.parms[["legend"]] <- legnd
        legend.parms[["col"]] <- col
        legend.parms[["lty"]] <- lty
        legend.parms[["lwd"]] <- llwd
        if(!missing(legendArgs) && !is.null(legendArgs)) legend.parms <- modifyList(legend.parms, legendArgs)
        ## Give the author what they asked for 
        do.call(legend, legend.parms)
    }

    invisible(list(col = col, lty = lty, lwd = llwd))
}

NULL



##' Draw standard error bar for discrete variables
##'
##' Used with plotSlopes if plotx is discrete. This is not currently
##' exported.
##' 
##' @param x The x center point
##' @param y The fitted "predicted" value
##' @param lwr The lower confidence interval bound
##' @param upr The upper confidence interval bound
##' @param width Thickness of shaded column
##' @param col Color for a bar
##' @param opacity Value in c(0, 254). 120 is default, that's partial see through.
##' @param lwd line width, usually 1
##' @param lty line type, usually 1
##' @return NULL
##' @author Paul Johnson
##' 
se.bars <- function(x, y, lwr, upr, width = 0.20, col, opacity = 120, lwd = 1, lty = 1) {
    h <- min(1, 2.5 * width) #half of fit line
    q <- min(1, width)  #half with of fit line quarter width
    d <- 0.5 * width #half width of shaded area
    iCol <- col2rgb(col)
    bCol <- mapply(rgb, red = iCol[1,], green = iCol[2,],
                   blue = iCol[3,], alpha = .8 * opacity, maxColorValue = 255)
    ### sCol: shade color
    sCol <-  mapply(rgb, red = iCol[1,], green = iCol[2,],
                    blue = iCol[3,], alpha = opacity/3, maxColorValue = 255)
    lCol <- mapply(rgb, red = iCol[1,], green = iCol[2,],
                   blue = iCol[3,], alpha = min(255, 2*opacity), maxColorValue = 255)
    if (!is.null(lwr)){
        polygon(c(x-d, x+d, x+d,  x-d), c(rep(lwr, 2), rep(upr, 2)), col = sCol, border = bCol)
        lines(c(x, x), c(lwr, upr), col = lCol, cex=0.5, lwd = lwd, lty = lty, lend = 1, ljoin = 1)
        lines(c(x, x) + c(-q, q), c(lwr, lwr), col = lCol, lwd = lwd, lty = lty, lend = 1, ljoin = 1, lmitre = 2)
        lines(c(x, x) + c(-q, q), c(upr, upr), col = lCol, lwd = lwd, lty = lty, lend = 1, ljoin = 1, lmitre = 2)
    }
    lines(c(x, x) + c(-h, h), c(y, y), col = lCol, lwd = lwd + 1)
    NULL
}

NULL


##' Draw display for discrete predictor in plotSlopes
##'
##' There's plotFancy for numeric predictor. This is for discrete
##' 
##' @param newdf The new data object, possibly from predictOMatic
##' @param olddf The model data matrix
##' @param plotx Name of horizontal axis variable
##' @param modx Name of moderator
##' @param modxVals values for modx
##' @param xlab X axis label
##' @param xlim x axis limits. Don't bother setting this, the internal
##'     numbering is too complicated.
##' @param ylab y axis label
##' @param ylim y axis limits
##' @param col color pallet for values of moderator variable
##' @param opacity Value in 0, 255 for darkness of interval shading
##' @param main main title
##' @param space same as space in barplot, vector c(0, 1) is
##'     c(space_between, space_before_first)
##' @param width width of shaded bar area, default is 0.2. Maximum is 1.
##' @param llwd requested line width, will re-cycle.
##' @param offset Shifts display to right (not tested)
##' @param gridArgs A list of values to control printing of reference grid.
##'     Set as "none" if no grid is desired. 
##' @param ... Arguments sent to par
##' @param legendArgs Arguments to the legend function. Set as "none"
##'     if no legend is needed. Otherwise, provide a list
##' @importFrom graphics axis
##' @return None
##' @author Paul Johnson <pauljohn@@ku.edu>
##' 
plotFancyCategories <- function(newdf, olddf, plotx, modx=NULL,
                                modxVals, xlab, xlim, ylab, ylim,
                                col = c("black", "blue", "darkgreen", "red", "orange", "purple", "green3"),
                                opacity = 120, main,
                                space=c(0,1), width = 0.2, llwd = 1, offset=0, 
                                ..., gridArgs = list(lwd = 0.3, lty = 5),
                                legendArgs)
{
    call.orig <- match.call()
    dotargs <- list(...)
    dotnames <- names(dotargs)
    lty <- if("lty" %in% dotnames)  dotargs[["lty"]] else 1
    interval <-  if("interval" %in% dotnames)  dotargs[["interval"]] else "none"
    
    plotxval <- newdf[[plotx]]
    plotxVals <- levels(plotxval)

    if(is.null(modx)){
        modxVals <-  1
    }

    ##Caution, modxVals comes out in wrong order compared to levels.
    ## Why? well put back in order according to levels(newdf)
    if(!is.null(modx)){
        if(is.null(names(modxVals))) names(modxVals) <- as.character(modxVals)
        if(is.factor(modxVals)){
            modxVal.names <- names(modxVals)
            names(modxVal.names) <- modxVals
            modxVal.names <- modxVal.names[levels(newdf[[modx]])]
            modxVals <- names(modxVal.names)
            names(modxVals) <- modxVal.names
            modx.levels <- modxVals
        } else {
            modx.levels <- modxVals
        }
    }
    ## sort rows
    newdf <- if(is.null(modx)) newdf[order(newdf[[plotx]]), ]
             else newdf[order(newdf[[plotx]], newdf[[modx]]), ]
 
    if(!is.null(modx) && is.factor(modxVals) && any(levels(newdf[[modx]]) != modxVals) )stop("levels fail")
    
    if(missing(ylim)){
        ylim <- magRange(range(c(newdf[["fit"]], if(!is.null(newdf[["lwr"]])) min(newdf[["lwr"]])),
                                if(!is.null(newdf[["upr"]])) max(newdf[["upr"]])), 1.25)
    }
    if(missing(xlim) || is.null(xlim)){
        xlim <- range(seq_along(plotxVals)) + c(-0.5, 0.5)
    }
    if(missing(xlab) || is.null(xlab)){
        xlab <- plotx
    }
    if(missing(ylab) || is.null(ylab)){
        ylab <- attr(mm, "terms")[[2]]
    }
    ##if(!is.null(modx) && missing(legend.title)){
    ##    legend.title <- modx
    ##}
        
    if(missing(main)){
        main <- "rockchalk discrete predictor plot"
    }

    ## lnc: levels needing colors; I got tired if if/then is.null(modx), see if this helps
    lnc <- if(is.null(modx)) plotxVals  else  modx.levels
    ## can we simplify by having a name selector color/line/ 
    xname <- if(is.null(modx)) plotx else modx

    lwd <- rep(llwd, length.out = length(lnc))
    names(lwd) <- lnc
    
    lty <-  rep(lty, length.out = length(lnc))
    names(lty) <- lnc
    
    if(!is.null(names(col))){
        if (any(!lnc %in% names(col))) stop("names in color vector incorrect")
    }
    
    if(missing(col) || is.null(col) || length(col) < length(lnc)) col <- col
    col <- rep(col, length.out = length(lnc))
    names(col) <- lnc                   
  
    newdf$col <- col[as.character(newdf[[xname]])] 
        
    width <- if(length(width) > 0 && length(width) < length(lnc)) rep(width, length.out = length(lnc))
    names(width) <- lnc
     
    newdf$lwd <-  lwd[as.character(newdf[[xname]])]
    newdf$lty <-  lty[as.character(newdf[[xname]])]
    ## Previous were done like this before
    if(is.null(modx)){
        newdf$width <- width[as.character(newdf[[plotx]])]
    } else {
        newdf$width <- width[as.character(newdf[[modx]])]
    }
   

    ## In a data subset, now many rows and columns are there?
    NR <- if(is.null(modx)) 1 else length(unique(newdf[[modx]]))
    NC <- length(unique(newdf[[plotx]]))
    width.internal <- 1
    space <- space * mean(width.internal)
  
    
    if (length(space) == 2) {
        space <- rep.int(c(space[2L], rep.int(space[1L], NR - 1)), NC)
    }
   
    width.internal <- rep_len(width.internal, NR) ## width of a bar
    offset <- rep_len(as.vector(offset), length(width.internal))
    delta <- width.internal/2
    w.r <- cumsum(space + width.internal)
    w.m <- w.r - delta
    w.l <- w.m - delta
    newdf$w.m <- w.m
    newdf$w.r <- w.r
    newdf$w.l <- w.l
    ## if (horiz) {
    ##     if (is.null(xlim)) 
    ##         xlim <- range(rAdj, height + offset, na.rm = TRUE)
    ##     if (is.null(ylim)) 
    ##         ylim <- c(min(w.l), max(w.r))
    ## }
    ## else {
    ##if (is.null(xlim)) 
    xlim <- c(min(w.l), max(w.r))
    ##if (is.null(ylim)) 
    ##    ylim <- range()
    ##   }
    ## w.m <- matrix(w.m, ncol = NC)


    plot(1, 0, type = "n", ## xlab = xlabs[i], ylab = ylabs[i], 
         xlim = xlim, ylim = ylim, main = main, xaxt = "n",
         xlab = xlab, ylab = ylab)
    
    if (is.null(modx)) {
        axis(1, at = newdf$w.m, labels = newdf[[plotx]])
    }else{
        ## TODO: Tricky, better test this
        xloc <- aggregate(newdf$w.m, by = list(plotx = newdf[[plotx]]),  mean)
        axis(1, at = xloc[ , "x"], labels = xloc[ , "plotx"])
    }
    
    for(mm in 1:(NROW(newdf))){
        ##if(!is.null(newdf[["lwr"]])){
            se.bars(newdf[mm , "w.m"], newdf[mm, "fit"], newdf[mm , "lwr"], newdf[mm , "upr"],
                    col = newdf[mm , "col"], width = newdf[mm , "width"], lty = newdf[mm, "lty"])
        #}
    }

    if((is.character(gridArgs) && gridArgs == "none") || interval == "none"){
        ## do nothing
    } else {
        gridlwd <- rep(gridArgs[["lwd"]], length.out = length(lnc))
        gridlty <- rep(gridArgs[["lty"]], length.out = length(lnc))        
        if(is.null(modx)){
            rownames(newdf) <- newdf[[plotx]]
            plotxList <- list(newdf)
            names(plotxList) <- plotx
        } else {
            plotxList <- split(newdf, f = newdf[ , plotx], drop = TRUE)
            for(i in names(plotxList)) rownames(plotxList[[i]]) <- plotxList[[i]][[xname]]
        }
        
        if(length(plotxList) > 1) {
            for(i in 1:(length(plotxList) - 1)){
                set1 <- plotxList[[i]]
                set2 <- plotxList[[i+1]]
                arrows(set1[["w.m"]], set1[["fit"]], set2[["w.l"]] + 0.3 * (1 - width), set1[["fit"]], col = col, lwd = gridlwd,
                       lty = gridlty, length = 0.1) 
            }
        }
    }
    ## if (plotPoints) {
    ##     parms <- list(xlab = plotx, ylab = ylab, cex = 0.6, lwd = 0.75)
    ##     if (is.factor(modxVar)) {
    ##         parms[["col"]] <- col[as.vector(modxVar[modxVar %in% modxVals])]
    ##         parms[["x"]] <- plotxVar[modxVar %in% modxVals]
    ##         parms[["y"]] <- depVar[modxVar %in% modxVals]
    ##     } else {
    ##         parms[["col"]] <- 1
    ##         parms[["x"]] <- olddf[[plotxVar
    ##         parms[["y"]] <- olddf[[depVar]]
    ##     }
    ##     parms <- modifyList(parms, dotargs)
    ##     do.call("points", parms)
    ## }

    if(!missing(legendArgs) && is.character(legendArgs) && legendArgs == "none") {
                                        # "do nothing"
    } else if (!is.null(modx)){
        ## guess legend parms from modx information
        legend.parms <- list(x = "bottomleft",
                             fill = col[as.character(lnc)],
                             col = col[as.character(lnc)],
                             border = NA, bty = "n")
        if(length(unique(lty)) > 1){
            legend.parms[["lty"]] <- lty[as.character(lnc)]
            legend.parms[["lwd"]] <- lwd[as.character(lnc)]
        }
        
        if(any(modxVals != lnc)){ 
            stop("plotFancyCategories: fatal error")
        }
        if (is.null(names(modxVals))) {
            legend.parms[["title"]] <- paste("Moderator:", modx)
            legend.parms[["legend"]] <- modxVals
        } else {
            legend.parms[["title"]] <-  paste("Moderator:", modx)
            legend.parms[["legend"]] <- names(modxVals)
        }
        
        if(!missing(legendArgs) && !is.null(legendArgs)) legend.parms <- modifyList(legend.parms, legendArgs)
        do.call(legend, legend.parms)
    } else {
        ## Give the author what they asked for 
         if(!missing(legendArgs) && !is.null(legendArgs)) do.call(legend, legendArgs)
    }

    
    invisible(list(newdf = newdf, col = col, lwd = lwd, call.orig=call.orig))
}

NULL

##' draw a normal distribution with beautiful illustrations
##'
##' This was developed for the R Working Example collection
##' in my website, pj.freefaculty.org/R/WorkingExamples
##' @param mu The mu parameter
##' @param sigma The sigma parameter
##' @param xlab Label for x axis
##' @param ylab Label for Y axis
##' @param main Title for plot. OK to ignore this, we'll make a nice one for you
##' @param ps pointsize of text
##' @param ... arguments passed to par
##' @return NULL
##' @importFrom stats dnorm
##' @importFrom stats pnorm
##' @importFrom stats median
##' @export
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @examples
##' drawnorm(mu = 10, sigma = 20)
##' drawnorm(mu= 0, sigma = 1)
##' drawnorm(mu = 102, sigma = 313)
##' drawnorm(mu = 0, sigma = 1, main = "A Standard Normal Distribution, N(0,1)",
##'          xlab = "X", ylab = "Density", ps = 7)
##' drawnorm(mu = 0, sigma = 1, ylab = "Density", ps = 14) 
drawnorm <- function(mu = 0, sigma = 1, xlab = "A Normally Distributed Variable",
                     ylab = "Probability Density", main, ps = par("ps"), ...){
    dotargs <- list(...)
    dotnames <- names(dotargs)
    dots.par <- dotargs[names(dotargs)[names(dotargs) %in% c(names(par()), formalArgs(plot.default))]]
        
    sigma.rounded <- if(1 == sigma[1]) 1 else round(sigma[1],2)
    mu.rounded <- round(mu, 2)
  
    myx <- seq( mu - 3.5*sigma,  mu+ 3.5*sigma, length.out=500)
    myDensity <- dnorm(myx,mean=mu,sd=sigma)
    if(missing(main)) {
        main <- bquote(x %~% Normal~group("(", list(mu == .(mu.rounded),
                       sigma^2 == .(if(sigma[1] == 1) 1 else sigma.rounded^2)),")"))
    }
    ## xpd needed to allow writing outside strict box of graph
    par.orig <-  par(xpd=TRUE, ps = ps)
    on.exit(par(par.orig))

    plot.parms <- list(x = myx, y = myDensity, type = "l", xlab = xlab, ylab = ylab, main = main, axes = FALSE)
    plot.parms <- modifyList(plot.parms, dots.par)
    do.call(plot, plot.parms)
    axis(2, pos = mu - 3.6*sigma)
    ticksat <- c(mu - 2.5 * sigma, mu,  mu - sigma, mu + sigma, mu + 2.5 * sigma)
    axis(1, pos = 0, at = ticksat)
    lines(c(myx[1],myx[length(myx)]),c(0,0)) ### closes off axes

    ## bquote creates an expression that text plotters can use
    t1 <-  bquote(mu== .(mu))
    ## Find a value of myx that is "very close to" mu
    centerX <- max(which (myx <= mu))
    ## plot light vertical line under peak of density
    lines( c(mu, mu), c(0, myDensity[centerX]), lty= 14, lwd=.2)
    ## label the mean
    text(mu, 0.4 * max(myDensity), labels = bquote( mu == .(mu.rounded)), pos = 2)

    ### find position 20% "up" vertically, to use for arrow coordinate
    ss = 0.2 * max(myDensity)
    ## Insert interval to represent width of one sigma
    arrows( x0=mu, y0= ss, x1=mu+sigma, y1=ss, code=3, angle=90, length=0.1)
    ## Write the value of sigma above that interval
    t2 <-  bquote( sigma== .(round(sigma,2)))
    text( mu+0.5*sigma, 1.15*ss, t2)

    ## Create a formula for the Normal
    normalFormula <- expression (f(x) == frac (1, sigma* sqrt(2*pi)) * ~~ e^{~-~frac(1,2)~bgroup("(", frac(x-mu,sigma),")")^2})
    ## Draw the Normal formula
    text ( mu + 0.5*sigma, max(myDensity)- 0.10 * max(myDensity),  normalFormula, pos=4)
    
    ## Theory says we should have 2.5% of the area to the left of: -1.96 * sigma.
    ## Find the X coordinate of that "critical value"
    criticalValue <- mu -1.96 * sigma
    ## Then grab all myx values that are "to the left" of that critical value.
    specialX <-  myx[myx <= criticalValue]
    
    ## mark the critical value in the graph
    text ( criticalValue, 0 , label= paste(round(criticalValue,2)), pos=1, cex = .7)
    ## Take sequence parallel to values of myx inside critical region
    specialY <- myDensity[myx < criticalValue]
    ##  Polygon makes a nice shaded illustration
    polygon(c(specialX[1], specialX, specialX[length(specialX )]), c(0, specialY, 0), density=c(-110),col="lightgray" )
    shadedArea <- round(pnorm(mu - 1.96 * sigma, mean=mu, sd=sigma), 4)
    criticalValue.rounded <- round(criticalValue, 3)
    ## I want to insert annotation about area on left side.
    al1 <- bquote(atop(Prob(x <= .(criticalValue.rounded)),
                        F(.(criticalValue.rounded)) == .(shadedArea)))
    ## Get center position in shaded area
    medX <- median(specialX) 
    indexMed <- max(which(specialX < medX))
    ## density at that center point:
    medY <- specialY[indexMed]
    denMax <- max(specialY)
    text(medX, denMax, labels=al1, pos = 3, cex = 0.7)
    indexMed <- max(which(specialX < medX))
    ## left side arrow
    x1 <-  medX + .1 *abs(max(specialX) - min(specialX))
    y1 <- 1.2 * myDensity[max(which(specialX < x1))]
    arrows(x0=medX, y0=denMax, x1= x1, y1 = y1,   length=0.1)
    ss <- 0.1 * max(myDensity)
    ## Mark interval from mu to mu-1.96*sigma
    arrows( x0=mu, y0= ss, x1=mu-1.96*sigma, y1=ss, code=3, angle=90, length=0.1)
    ## Put text above interval
    text( mu - 2.0*sigma, 1.15*ss, bquote(paste(.(criticalValue.rounded)==mu-1.96 * sigma,sep="")),pos=4 )

    ## Now work on right side critical value and area
    criticalValue <- mu +1.96 * sigma
    criticalValue.rounded <- round(criticalValue, 3)
    ## Then grab all myx values that are "to the left" of that critical value.
    specialX <-  myx[myx >= criticalValue]
    ## mark the critical value in the graph
    text ( criticalValue, 0 , label= paste(criticalValue.rounded), pos=1, cex = 0.7)
    ## Take sequence parallel to values of myx inside critical region
    specialY <- myDensity[myx >= criticalValue]
    ##  Polygon for shaded illustration
    polygon(c(specialX[1], specialX, specialX[length(specialX )]), c(0, specialY, 0), density=c(-110), col="lightgray" )
    shadedArea <- round(pnorm(mu + 1.96 * sigma, mean=mu, sd=sigma, lower.tail=F),4)
    ## Insert comment on right side.
    al2 <- bquote(atop(1 - F( .(criticalValue.rounded)),
                       phantom(0) == .(shadedArea)))
    medX <- median(specialX)
    ## denAtMedX <- myDensity[indexMed <- max(which(specialX < medX))]
    denMax <-  max(specialY)
    text(medX, denMax, labels=al2, pos = 3, cex = 0.7)
    ## point from text toward shaded area
    x1 <-  medX - .1 *abs(max(specialX) - min(specialX))
    y1  <- 1.2 * specialY[max(which(specialX < x1))]
    ## arrows( x0=medX, y0=denMax, x1= medX - 0.1*abs(max(specialX) - min(specialX)), y1= 1.02 * myDensity[indexMed] ,   length=0.1)
    arrows( x0=medX, y0=denMax, x1= x1, y1 = y1,  length=0.1)
    
    ss <- 0.05 * max(myDensity)
    ## Mark interval from mu to mu+1.96*sigma
    arrows( x0=mu, y0= ss, x1=mu+1.96*sigma, y1=ss, code=3, angle=90, length=0.1)
    ## Put text above interval
    text( mu + 1.96*sigma,1.15*ss, bquote(paste(.(criticalValue.rounded)==mu+1.96 * sigma,sep="")), pos=2 )
}

##' Check a matrix for positive definitness
##'
##' Uses eigen to check positive definiteness. Follows example used
##' in \code{MASS} package by W. N. Venables and Brian D. Ripley
##'
##' @param X A matrix
##' @param tol Tolerance (closeness to 0 required to declare failure)
##' @return TRUE or FALSE
##' @author Paul E. Johnson <pauljohn@@ku.edu>
checkPosDef <-
    function(X, tol = 1e-6)
{
    evalues <- eigen(X, only.values = TRUE)$values
    res <- if(!all(evalues >= -tol*abs(evalues[1L]))) FALSE else TRUE
    res
}
 
NULL



##' Minor revision of mvrnorm (from \code{MASS}) to facilitate replication
##'
##' This is the \code{\link[MASS]{mvrnorm}} function from the MASS
##' package (Venables and Ripley, 2002), with one small modification
##' to facilitate replication of random samples. The aim is to make
##' sure that, after the seed is reset, the first rows of generated
##' data are identical no matter what value is chosen for n.  The one
##' can draw 100 observations, reset the seed, and then draw 110
##' observations, and the first 100 will match exactly. This is done
##' to prevent unexpected and peculiar patterns that are observed
##' when n is altered with MASS package's mvrnorm.
##'
##' To assure replication, only a very small change is made. The code
##' in \code{MASS::mvrnorm} draws a random sample and fills a matrix
##' by column, and that matrix is then decomposed.  The change
##' implemented here fills that matrix by row and the problem is
##' eliminated.
##'
##' Some peculiarities are noticed when the covariance matrix changes
##' from a diagonal matrix to a more general symmetric matrix
##' (non-zero elements off-diagonal).  When the covariance is strictly
##' diagonal, then just one column of the simulated multivariate
##' normal data will be replicated, but the others are not. This has
##' very troublesome implications for simulations that draw samples of
##' various sizes and then base calculations on the separate simulated
##' columns (i.e., some columns are identical, others are completely
##' uncorrelated).
##'
##' @seealso For an alternative multivariate normal generator
##' function, one which has had this fix applied to it,
##' consider using the new versions of \code{\link[mvtnorm]{rmvnorm}} in the
##' package \code{mvtnorm}.
##' @param n the number of samples ("rows" of data) required.
##' @param mu a vector giving the means of the variables.
##' @param Sigma positive-definite symmetric matrix specifying the
##'    covariance matrix of the variables.
##' @param tol tolerance (relative to largest variance) for numerical lack
##'    of positive-definiteness in \code{Sigma}
##' @param empirical logical. If true, mu and Sigma specify the empirical
##'    not population mean and covariance matrix.
##' @import MASS
##' @export
##' @return If \code{n = 1} a vector of the same length as \code{mu}, otherwise an
##'  \code{n} by \code{length(mu)} matrix with one sample in each row.
##' @author Ripley, B.D. with revision by Paul E. Johnson
##' @references
##' Venables, W. N. & Ripley, B. D. (2002) Modern Applied Statistics with
##' S. Fourth Edition. Springer, New York. ISBN 0-387-95457-0
##' @examples
##' library(MASS)
##' library(rockchalk)
##'
##' set.seed(12345)
##' X0 <- MASS::mvrnorm(n=10, mu = c(0,0,0), Sigma = diag(3))
##' ## create a smaller data set, starting at same position
##' set.seed(12345)
##' X1 <- MASS::mvrnorm(n=5, mu = c(0,0,0), Sigma = diag(3))
##' ## Create a larger data set
##' set.seed(12345)
##' X2 <- MASS::mvrnorm(n=15, mu = c(0,0,0), Sigma = diag(3))
##' ## The first 5 rows in X0, X1, and X2 are not the same
##' X0
##' X1
##' X2
##' set.seed(12345)
##' Y0 <- mvrnorm(n=10, mu = c(0,0,0), Sigma = diag(3))
##' set.seed(12345)
##' Y1 <- mvrnorm(n=5, mu = c(0,0,0), Sigma = diag(3))
##' set.seed(12345)
##' Y2 <- mvrnorm(n=15, mu = c(0,0,0), Sigma = diag(3))
##' # note results are the same in the first 5 rows:
##' Y0
##' Y1
##' Y2
##' identical(Y0[1:5, ], Y1[1:5, ])
##' identical(Y1[1:5, ], Y2[1:5, ])
##'
##' myR <- lazyCor(X = 0.3, d = 5)
##' mySD <- c(0.5, 0.5, 0.5, 1.5, 1.5)
##' myCov <- lazyCov(Rho = myR, Sd = mySD)
##'
##' set.seed(12345)
##' X0 <- MASS::mvrnorm(n=10, mu = rep(0, 5), Sigma = myCov)
##' ## create a smaller data set, starting at same position
##' set.seed(12345)
##' X1 <- MASS::mvrnorm(n=5, mu = rep(0, 5), Sigma = myCov)
##' X0
##' X1
##' ##' set.seed(12345)
##' Y0 <- rockchalk::mvrnorm(n=10, mu = rep(0, 5), Sigma = myCov)
##' ## create a smaller data set, starting at same position
##' set.seed(12345)
##' Y1 <- rockchalk::mvrnorm(n=5, mu = rep(0, 5), Sigma = myCov)
##' Y0
##' Y1
##'
mvrnorm <-
    function(n = 1, mu, Sigma, tol=1e-6, empirical = FALSE)
{
    p <- length(mu)
    if(!all(dim(Sigma) == c(p,p))) stop("incompatible arguments")
    eS <- eigen(Sigma, symmetric = TRUE)
    ev <- eS$values
    if(!all(ev >= -tol*abs(ev[1L]))) stop("'Sigma' is not positive definite")
    X <- matrix(rnorm(p * n), n, byrow = TRUE)
    if(empirical) {
        X <- scale(X, TRUE, FALSE) # remove means
        X <- X %*% svd(X, nu = 0)$v # rotate to PCs
        X <- scale(X, FALSE, TRUE) # rescale PCs to unit variance
    }
    X <- drop(mu) + eS$vectors %*% diag(sqrt(pmax(ev, 0)), p) %*% t(X)
    nm <- names(mu)
    if(is.null(nm) && !is.null(dn <- dimnames(Sigma))) nm <- dn[[1L]]
    dimnames(X) <- list(nm, NULL)
    if(n == 1)
        drop(X)
    else t(X)
}

##' Group Mean Center: Generate group summaries and individual deviations within groups
##'
##' Multilevel modelers often need to include predictors like the
##' within-group mean and the deviations of individuals around the
##' mean. This function makes it easy (almost foolproof) to calculate
##' those variables.
##'
##' This was originally just for "group mean-centered" data, but now
##' is more general, can accept functions like median to calculate
##' center and then deviations about that center value within the
##' group.
##'
##' Similar to Stata egen, except more versatile and fun! Will create
##' 2 new columns for each variable, with suffixes for the summary and
##' deviations (default suffixes are "_mn" and "_dev". Rows will match
##' the rows of the original data frame, so it will be easy to merge
##' or cbind them back together.
##'
##' @param dframe a data frame.
##' @param x Variable names or a vector of variable names. Do NOT
##'     supply a variable like dat$x1, do supply a quoted variable
##'     name "x1" or a vector c("x1", "x2")
##' @param by A grouping variable name or a vector of grouping
##'     names. Do NOT supply a variable like dat$xfactor, do supply a
##'     name "xfactor", or a vector c("xfac1", "xfac2").
##' @param FUN Defaults to the mean, have not tested alternatives
##' @param suffix The suffixes to be added to column 1 and column 2
##' @param fulldataframe Default TRUE. original data frame is returned
##'     with new columna added (which I would call "Stata style"). If
##'     FALSE, this will return only newly created columns, the
##'     variables with suffix[1] and suffix[2] appended to names.
##'     TRUE is easier (maybe safer), but also wastes memory.
##' @return Depending on \code{fulldataframe}, either a new data frame
##'     with center and deviation columns, or or original data frame
##'     with "x_mn" and "x_dev" variables appended (Stata style).
##' @author Paul Johnson
##' @importFrom stats aggregate
##' @export
##' @examples
##' ## Make a data frame out of the state data collection (see ?state)
##' data(state)
##' statenew <- as.data.frame(state.x77)
##' statenew$region <- state.region
##' statenew$state <- rownames(statenew)
##' head(statenew.gmc1 <- gmc(statenew, c("Income", "Population"), by = "region"))
##' head(statenew.gmc2 <- gmc(statenew, c("Income", "Population"), by = "region",
##'    fulldataframe = FALSE))
##' ## Note dangerous step: assumes row alignment is correct.
##' ## return has rownames from original set to identify danger
##' head(statenew2 <- cbind(statenew, statenew.gmc2))
##' if(!all.equal(rownames(statenew), rownames(statenew.gmc2))){
##'    warning("Data row-alignment probable error")
##' }
##' ## The following box plots should be identical
##' boxplot(Income ~ region, statenew.gmc1)
##' boxplot((Income_mn + Income_dev) ~ region, statenew.gmc1)
##' ## Multiple by variables
##' fakedat <- data.frame(i = 1:200, j = gl(4, 50), k = gl(20, 10),
##'                       y1 = rnorm(200), y2 = rnorm(200))
##' head(gmc(fakedat, "y1", by = "k"), 20)
##' head(gmc(fakedat, "y1", by = c("j", "k"), fulldataframe = FALSE), 40)
##' head(gmc(fakedat, c("y1", "y2"), by = c("j", "k"), fulldataframe = FALSE))
##' ## Check missing value management
##' fakedat[2, "k"] <- NA
##' fakedat[4, "j"] <- NA##' head(gmc(fakedat, "y1", by = "k"), 20)
##' head(gmc(fakedat, "y1", by = c("j", "k"), fulldataframe = FALSE), 40)
gmc <- function(dframe, x, by, FUN = mean, suffix = c("_mn", "_dev"),
                fulldataframe = TRUE) {
    xmean <- aggregate(dframe[ , x, drop = FALSE],
                       dframe[ , by, drop = FALSE], FUN,
                       na.rm = TRUE)
    dframe.orignrow <- NROW(dframe)
    dframe.origrownames <- rownames(dframe)
    dframe$rownames <-  dframe.origrownames
    dframe$index <- 1:NROW(dframe)
    dframenamez <- setdiff(colnames(dframe), by)
    xmeannamez <- setdiff(colnames(xmean), by)
    xmeannamez_new <- paste0(xmeannamez, suffix[1])
    for (i in seq_along(xmeannamez)){
        colnames(xmean) <-  gsub(xmeannamez[i], xmeannamez_new[i],
                                 colnames(xmean), fixed = TRUE)
    }
    
    df2 <- merge(dframe, xmean, by = by, all.x = TRUE,  sort = FALSE)
    ## put rows back in original order!
    df2 <- df2[order(df2$index), ]
    ## Calculate deviations
    for(i in x){
        df2[ , paste0(i, suffix[2])] <- df2[ , i] - df2[ , paste0(i, suffix[1])]
    }
    rownames(df2) <- df2$rownames
    df2$index <- NULL
    if (dframe.orignrow != NROW(df2)) stop("wrong data frame row count in gmc")
    if (!all.equal(rownames(df2), dframe.origrownames)) stop("wrong data frame row names in gmc")
    if(!isTRUE(fulldataframe)){
        df3 <- df2[c(by, colnames(df2)[!colnames(df2) %in% c(by, dframenamez)])]
        attr(df3, "meanby") <- by
        return(df3)
    }
    ## else
    df2$rownames <- NULL
    attr(df2, "meanby") <- by
    return(df2)
}

##' T-test for the difference in 2 regression parameters
##'
##' This is the one the students call the "fancy t test". It is just
##' the simplest, most easy to use version of the t test to decide if
##' 2 coefficients are equal. It is not as general as other functions
##' in other packages. This is simpler to use for beginners.  The
##' \code{car} package's function \code{linearHypothesis} is more
##' general, but its documentation is much more difficult to understand.
##' It gives statistically identical results, albeit phrased as an F
##' test.
##'
##' I did this because we have trouble understanding terminology in
##' documentation for more abstract functions in other R
##' packages.
##' 
##' It has an additional feature, it can import robust standard errors
##' to conduct the test.
##' 
##' @param parm1 A parameter name, in quotes!
##' @param parm2 Another parameter name, in quotes!
##' @param model A fitted regression model
##' @param model.cov Optional, another covariance matrix to use while
##'     calculating the test. Primarily used for robust (or otherwise
##'     adjusted) standard errors
##' @param digits How many digits to print? This affects only the on-screen
##'     printout. The return object is numeric, full precision.
##' @return A vector with the difference, std. err., t-stat, and p
##'     value. Prints a formatted output statement.
##' @author Paul Johnson <pauljohn@@ku.edu>
##' @export
##' @examples
##' mdat <- data.frame(x1 = rnorm(100), x2 = rnorm(100))
##' stde <- 2
##' mdat$y <- 0.2 * mdat$x1 + 0.24 * mdat$x2 + stde * rnorm(100)
##' m1 <- lm(y ~ x1 + x2, data = mdat)
##' waldt("x1", "x2", m1)
##' waldt("x1", "x2", m1, digits = 2)
##' ## Returned object is not "rounded characters". It is still numbers
##' stillnumeric <-  waldt("x1", "x2", m1, digits = 2)
##' stillnumeric
##' ## Equivalent to car package linearHypothesis:
##' if(require(car)){
##'     linearHypothesis(m1, "x1 = x2")
##' }
##' ## recall t = sqrt(F) for a 1 degree of freedom test.
##' ## If we could understand instructions for car, we probably
##' ## would not need this function, actually.
waldt <- function(parm1, parm2, model, model.cov = NULL, digits = getOption("digits")){
    V <- function(mat, parm1, parm2 = NULL) {
        if(is.null(parm2)) return (mat[parm1, parm1])
        else return(mat[parm1, parm2])
    }
    if(is.null(model.cov)) model.cov <- vcov(model)
    ## following does not succeed with lme4 object
    ## model.coef <- coef(model)[c(parm1, parm2)]
    model.coef <- coef(summary(model))[c(parm1, parm2), "Estimate"]
    se <- sqrt(V(model.cov, parm1) + V(model.cov, parm2) - 2*V(model.cov, parm1, parm2))
    diff <- model.coef %*% c(1, -1)
    t <-  diff/se
    df <- df.residual(model)
    pval <- pt(abs(t), lower.tail = FALSE, df = df) * 2
    reslt <- c(diff = diff, Std.Err. = se, t = t, p = pval)
    print(formatC(reslt, digits = digits))
    invisible(reslt)
}

NULL

##' Convert the vech (column of strictly lower trianglar values from a matrix) into a correlation matrix.
##'
##' vech2Corr is a convenience function for creating correlation matrices
##' from a vector of the lower triangular values. It checks the arguments
##' to make sure they are consistent with the requirements of a
##' correlation matrix. All values must be in [-1, 1], and the number
##' of values specified must be correct for a lower triangle.
##'
##' Use this in combination with the \code{lazyCov} function to
##' convert a vector of standard deviations and the correlation matrix
##' into a covariance matrix.
##'
##' @export
##' @seealso Similar functions exist in many packages, see
##'     \code{vec2sm} in corpcor, \code{xpnd} in MCMCpack
##' @param vech A vector of values for the strictly lower triangle of
##'     a matrix. All values must be in the [0,1] interval (because
##'     they are correlations) and the matrix formed must be positive
##'     definite.
##' @return A symmetric correlation matrix, with 1's on the diagonal.
##' @author Paul E. Johnson <pauljohn@@ku.edu>
##' @examples
##' v <- c(0.1, 0.4, -0.5)
##' vech2Corr(v)
##' v <- c(0.1, 0.4, -0.4, 0.4, 0.5, 0.1)
##' vech2Corr(v)
vech2Corr <-
    function(vech)
{
    ##compute number of rows from vech. diag not in the vech!
    n <- (sqrt(1 + 8 * length(vech)) + 1)/2
    if (!as.integer(n) == n) stop(deparse(substitute(vech)),
                                  " must have the correct number of elements to fill",
                                  "in a strictly lower triangle in a square matrix.")
    if(any(vech > 1 | vech < -1)) {
        stop(paste("All values in ", deparse(substitute(vech)),
                   " must be in the interval [-1,1]"))
    }
    X <- matrix(NA, nrow = n, ncol = n)
    X[lower.tri(X, diag = FALSE)] <- vech
    X[upper.tri(X)] <- t(X)[upper.tri(X)]
    diag(X) <- 1
    stopifnot(checkPosDef(X))
    X
}
NULL

#!/bin/bash

lyx -e pdf2 Rstyle.lyx
lyx -e sweave Rstyle.lyx

lyx -e pdf2 Rchaeology.lyx
lyx -e sweave Rchaeology.lyx


lyx -e pdf2 rockchalk.lyx
lyx -e sweave rockchalk.lyx

#mkdir Rchaeology.html.LyXconv
#cp Rchaeology.Rnw Rchaeology.html.LyXconv
#cd Rchaeology.html.LyXconv
#R CMD Sweave Rchaeology.Rnw
#../sw


NOW=$(date +"%Y%m%d")
dir="backup/backup-$NOW"
budir="freefaculty.org/R/backup/backup-$NOW"

ssh freefaculty mkdir -p $budir

rsync -e ssh  -b --suffix="-bak2" --backup-dir=$dir Rchaeology.pdf  freefaculty:freefaculty.org/R

rsync -e ssh -b --suffix="-bak2" --backup-dir=$dir rockchalk.pdf  freefaculty:freefaculty.org/R

rsync -e ssh -b --suffix="-bak2" --backup-dir=$dir Rstyle.pdf  freefaculty:freefaculty.org/R


##rsync -e ssh -b --suffix="-bak2" --backup-dir=$dir  Rchaeology.html.LyXconv/Rchaeology.html  freefaculty:freefaculty.org/R

%% LyX 2.4.4 created this file.  For more info, see https://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[american,noae]{scrartcl}
\usepackage{lmodern}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{cmtt}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage{color}
\usepackage{babel}
\usepackage{url}
\usepackage{enumitem}
\usepackage{geometry}
\geometry{verbose,tmargin=1in,bmargin=1in,lmargin=1in,rmargin=1in}
\usepackage[authoryear]{natbib}
\usepackage[pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=true,pdfborder={0 0 0},pdfborderstyle={},backref=section,colorlinks=true]
 {hyperref}
\hypersetup{
 colorlinks=true, linkcolor=darkblue, urlcolor=darkblue, citecolor=darkblue}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
<<echo=F>>=
  if(exists(".orig.enc")) options(encoding = .orig.enc)
@
\newlength{\lyxlabelwidth}      % auxiliary length 
\newenvironment{lyxcode}
	{\par\begin{list}{}{
		\setlength{\rightmargin}{\leftmargin}
		\setlength{\listparindent}{0pt}% needed for AMS classes
		\raggedright
		\setlength{\itemsep}{0pt}
		\setlength{\parsep}{0pt}
		\normalfont\ttfamily}%
	 \item[]}
	{\end{list}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
%\VignetteIndexEntry{Rstyle}

\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage{babel}
\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%

 \ifthenelse{\equal{#1}{}}{%
   \@float{figure}
 }{%
   \@float{figure}[#1]%
 }%
 \centering
}{%
 \end@float
}
\renewenvironment{table}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{table}
 }{%
   \@float{table}[#1]%
 }%
 \centering
%  \setlength{\@tempdima}{\abovecaptionskip}%
%  \setlength{\abovecaptionskip}{\belowcaptionskip}%
% \setlength{\belowcaptionskip}{\@tempdima}%
}{%
 \end@float
}

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\normalsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.95}}
\def\Routbackground{\color{white}}
\def\Routcolor{\color{black}}


\usepackage{listings}% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, style=Rstyle}

\usepackage{xcolor}
\definecolor{darkblue}{HTML}{1e2277}
% Added by lyx2lyx
\setlength{\parskip}{\smallskipamount}
\setlength{\parindent}{0pt}

\makeatother

\usepackage{listings}
\renewcommand{\lstlistingname}{\protect\inputencoding{latin9}Listing}

\begin{document}
\title{R Style. An Rchaeological Commentary. }
\author{Paul E. Johnson <pauljohn @ ku.edu>}
\maketitle

\section{Introduction: Ugly Code that Runs}

Because there is no comprehensive official R style manual, students
and package writers seem to think that there is no style whatsoever
to be followed. While it may be true that ``ugly code runs,'' it
is also 1) difficult to read and 2) frustrating to extend, and 3)
tiring to debug. Code is a language, a medium of communication, and
one should not feel free no ignore its customs. 

After students have finished a semester of statistics with R, they
may be ready to start preparing functions or packages. Those R users
are the ones I'm trying to address with this note. It is important
to realize that the readability of code makes a difference. It sometimes
difficult to know that there is a ``right way'' and a ``wrong way''
because there are so many examples to study on CRAN.

This note describes R style from an Rchaeological\footnote{Definitions:
\begin{description}
\item [{Rchaeology:}] The study of R programming by investigation of R
source code. It is the effort to discern the programming strategies,
idioms, and style of R programmers in order to better communicate
with them.
\item [{Rchaeologist:}] One who practices Rchaeology.
\end{description}
} perspective. By examining the work of the R Core Development Team
\citep{RCore} and other notable package writers, we are able to discern
an implicit style guide. However, this note is not ``official''
or endorsed from R Core.\footnote{Yet :)} With one exception at the
end of this note, none of the advice here is ``my'' advice. Instead,
it is my best description of the standards followed by the leading
R programmers. 

At one point, the only guide was the Google R style guide,\footnote{\url{https://google.github.io/styleguide/Rguide.xml}}
which was used as a policy for R-related ``Google Summer of Code''
projects. There are many excellent suggestions in Hadley Wickham's
Style Guide.\footnote{\url{http://adv-r.had.co.nz/Style.html}} In
what follows, I'll try to explain why there are some variations among
these projects and offer some advice about how we (the users) should
sort through their advice.

<<echo=F>>=
dir.create("plots", showWarnings=F)
@

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\SweaveOpts{prefix.string=plots/plot,ae=F,height=4,width=6}

<<Roptions, echo=F>>=
options(width=100, continue="+ ")
options(useFancyQuotes = FALSE) 
set.seed(12345)
pdf.options(onefile=F,family="Times",pointsize=12)
@

\section{Rchaeological Methodology}

I am a student of R as a programming language. I am also student of
the R community as an international success that created a working
open source computer program. One of the most interesting differences
between R and other open source projects I have observed is that R
attracts non-programmers. There is an abundance of statistical novices
and untrained computer programmers in the R user community. Many students
begin with R as a way of learning about computer programming. In contrast,
the developers of R are world-class software engineers. They have
formal training in computer programming and years of experience in
a variety of computer languages. The diversity creates a healthy tension
that is easy to see in the r-help email list or on Web forums for
R users. 

\subsection{\textquotedblleft Use the Source, Luke,\textquotedblright{} said Obi-Wan}

What should R code look like? Stop guessing. The implicit style guide
for R is the R source code itself. If users want to communicate with
R Core developers, they ought to communicate using the style that
developers use.

I'm often surprised to find that R users--even experienced ones--have
never looked at the R source code. Before going any further,
\begin{quote}
Open the source code for R. I mean, literally, download R-3.5.2.tar.gz
(or whatever is current when you read this). Unpack that, navigate
to the directory src/library/stats/R. Open the file ``lm.R''. 
\end{quote}
That's what R code should look like. 

Browse other R files in the source code. Notice the files are suffixed
by R, not r! 

Then go read a lot of R packages. Begin with the recommended packages
(in the R source code under src/library/Recommended). Then draw some
samples from CRAN. Choose packages that are prepared by members of
R Core, and then sample a few packages that are widely installed,
such as John Fox's car package \citep{fox_r_2011}. 

After that, pick a random sample of packages on CRAN. Don't be surprised
by ugly code in a randomly chosen R package. 

\subsection{Notice How R Describes its Own Style}

Type the name of a function at the R command prompt. That is the same
as using the function called \bgroup\inputencoding{latin9}\lstinline!print.function()!\egroup
 to review the contents of a function from an R package. For example,
try ``lm''. The first few lines are\bgroup\inputencoding{latin9}
\begin{lstlisting}
> lm
function (formula, data, subset, weights, na.action, method = "qr",
    model = TRUE, x = FALSE, y = FALSE, qr = TRUE, singular.ok = TRUE,
    contrasts = NULL, offset, ...)
{
    ret.x <- x
    ret.y <- y
    cl <- match.call()
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "subset", "weights", "na.action",
        "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    if (method == "model.frame")
        return(mf)
    else if (method != "qr")
        warning(gettextf("method = '%s' is not supported. Using 'qr'",
            method), domain = NA)
\end{lstlisting}
\leavevmode\egroup

That's quite a bit like the code in the file lm.R, but it is not exactly
the same. Even if the code in lm.R were an ugly, horrible mess, its
output in the terminal would be indented and spaced just right. That
is an important Rchaeological finding!

Why can there be a difference between the code for a function in a
file (like ``lm.R'') and the output of the command (like ``lm'')?
Admittedly, this is difficult to understand. The on-screen output
is not (by default, anyway) the source that went into R, but rather
it is R's rendition of the internal structure of the function. I recently
had an epiphany while reading a section in the \emph{Writing R Extensions}
manual called ``Tidying R code''. That title is a bit misleading.
It is not about tidying R source code; rather, it is about beautifying
the rendition of internal structures for the terminal. ``R treats
function code loaded from packages and code entered by users differently.
By default code entered by users has the source code stored internally,
and when the function is listed, the original source is reproduced.
Loading code from a package (by default) discards the source code,
and the function listing is re-created from the parse tree of the
function.'' That is to say, if ugly code is syntatically valid, R
can parse it and structure it according to the internal dictates of
the R runtime system, and when we ask to see the function, we get
a nice looking result.

\subsection{Formulate SEA estimates.}

As already noted, there is no mandatory style for R code. The \emph{R
Internals} manual has a section ``R coding standards,'' but it is
quite brief. The main point that most readers take away concerns indentation:
subsections in code should be preceded by 4 blank spaces, not a tab
character. 

But there is a larger point in \emph{R Internals}, but novices don't
recognize the importance of it. R is a GNU project, and there are
GNU coding standards.\footnote{\url{http://www.gnu.org/prep/standards/standards.html}}
The R project's C code follows the standard closely. In the entire
body of the R source code, we find the GNU thumb print. The importance
of that fact is missed by untrained readers, who mistake the lack
of a comprehensive discussion of style for an encouragement to ``do
anything you want.'' 

In the following, I will try to point out the areas of greatest agreement
by assigning an SEA score to each point. SEA stands for ``Subjective
and completely unscientific personal Estimate of Agreement.'' These
are my Bayesian priors. If I could survey my favorite R programmers,
I'd find some variety, and I am trying to make it clear where the
disagreements might lie. But, then again, I may have been fooling
myself. It has recently been suggested to me that these recommendations
are not descriptions of the Rchaeological community I'm studying,
they are rather my personal litmus test for admirable R programmers. 

\section{Nearly Universally accepted standards.}

\subsection{(SEA 1.0) Indentation of code sections is required. }

This is explicitly spelled out in the R documentation. No tabs! Insert
4 blank spaces. Personally, I prefer 2 spaces, which has been the
default in Emacs. But I'm changing my code to use 4 spaces. If you
find my code with 2 spaces, please accept this apology and believe
that it is an oversight. 

\subsection{(SEA .95). Use \textquotedblleft <-\textquotedblright , not \textquotedblleft =\textquotedblright ,
for assignments. }

One cannot find the equal sign used for assignments in any file in
the R source code. Nor can one find it in any of the Recommended packages
(so far as I can tell). 

Students who have learned R in introductory textbooks are sometimes
shocked to learn that they were taught wrong. I'm sympathetic to their
outrage. How can this be? 

The equal sign was used by mistake so frequently that the R system
was re-designed to tolerate that mistake. \emph{Most} usages of the
equal sign for assignments do not cause runtime errors. Not all possible
problems were eliminated, however. Thus the equal sign is not recommended,
it is tolerated. Nevertheless, A horrible profusion of textbooks and
packages ensued using the equal sign for assignment. 

\subsection{(SEA .98) Blank spaces around symbols are required. }

This is a general GNU coding standard. 
\begin{enumerate}
\item Insert spaces before and after 

\begin{enumerate}
\item mathematical symbols like: ``='', ''<-'', ``<'', ``{*}'',
''+''
\item R binary operations like: ``\%{*}\%'', ``\%o\%'', and ``\%in\%''.
\end{enumerate}
\item Put one space after commas. 
\item Insert one space before the opening squiggly braces ``\{''.
\item Put one space after the closing parenthesis ``)'' and the closing
squiggly brace ``\}''.
\end{enumerate}
This is purely a matter of convention and legibility, it does not
affect the ``rightness'' of code. 

Other observations about spaces,
\begin{enumerate}
\item Do not insert spaces between function names and their opening parentheses.
\item After reviewing the R source code, I was uncertain about whether one
ought to insert one space after ``if'' and ``for''. From an Rchaeological
perspective, this is a little bit perplexing. In the help page for
those terms (see help(``for'')), there is no space after ``if''
or ``for''. In the R-3.0.0 source code folder src/library/base/R,
I count 1741 instances of ``if(`` and 683 instances of ``if (``.
The former style seemed right to me, at least at first, because people
often say that R's ``if'' and ``for'' are functions. I asked for
clarification in the R-devel email list, and Peter Dalgaard explained
that the space should be used because those terms are

\begin{quote}
language constructs (and they \emph{are} keywords, not names, that's
why ?for won't work). The function calls are `if`(fee, \{foo\}, \{fie\})
and something rebarbative for `for`(....).

Besides, both constructs are harder to read without the spaces. (r-devel,
April 18, 2013)
\end{quote}
For me, that settles the question. For R code, as in C, ``if'' and
``for'' should be treated as keywords, and there would be a space
after them, as in ``\bgroup\inputencoding{latin9}\lstinline!if (x < 7)!\egroup
''.
\item Do not insert ``extra spaces'' inside parentheses. 

Programmers who have written in the BASH scripting language may recall
that a space inside brackets is required. That training causes me
to think that R code is a little bit ``jammed together.'' This is
pleasant to my eye:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
if ( (x == 1) & (y == 2) ) {
\end{lstlisting}
\leavevmode\egroup

but, from an Rchaeological point of view, more the correct style is:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
if((x == 1) & (y == 2)) {
\end{lstlisting}
\leavevmode\egroup

The insertion of the interior parentheses for the smaller conditions
inside the if statement is consistent with the GNU standard for C.
\end{enumerate}

\subsubsection*{Is there an \textquotedblleft argument exception\textquotedblright{}
to the space rule for equal signs?}

Package writers are not entirely consistent, and Rchaeologically speaking,
we cannot be sure if these variations are accidental. We sometimes
find no spaces, as in

\bgroup\inputencoding{latin9}
\begin{lstlisting}
plot(x, y, lwd=4, col=green, main="My Title")
\end{lstlisting}
\leavevmode\egroup

It would surely be more correct like so:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
plot(x, y, lwd = 4, col = green, main = "My Title")
\end{lstlisting}
\leavevmode\egroup

Spaces may sometimes be omitted in an effort to keep code on one line.
Especially where publishers are concerned about the use of scarce
paper, the omission of spaces around equal signs is not uncommon.
Please note, however, that it is NEVER acceptable to omit the spaces
after commas!

\subsubsection*{What about indentation of long function declarations?}

One of the interesting space related questions is the indentation
of function declarations when there are many arguments. Consider the
R source code for the function lm():

\bgroup\inputencoding{latin9}
\begin{lstlisting}
lm <- function (formula, data, subset, weights, na.action,
                method = "qr", model = TRUE, x = FALSE, y = FALSE,
                qr = TRUE, singular.ok = TRUE, contrasts = NULL,
                offset, ...)
\end{lstlisting}
\leavevmode\egroup

Note that lines 2-4 are indented under the letter ``f'' in formula.
If the function's name were longer, it would push all of that indented
code to the right, probably causing line wraps. The solution is to
put the function's name and the assignment symbol on separate line.
This is the format of R's function plot.lm(). 

\bgroup\inputencoding{latin9}
\begin{lstlisting}
plot.lm <-
function (x, which = c(1L:3L,5L), ## was which = 1L:4L,
          caption = list("Residuals vs Fitted", "Normal Q-Q",
          "Scale-Location", "Cook's distance",
          "Residuals vs Leverage",
          expression("Cook's dist vs Leverage  " * h[ii] / (1 - h[ii]))),
          panel = if(add.smooth) panel.smooth else points,
          sub.caption = NULL, main = "",
          ask = prod(par("mfcol")) < length(which) && dev.interactive(), ...,
          id.n = 3, labels.id = names(residuals(x)), cex.id = 0.75,
          qqline = TRUE, cook.levels = c(0.5, 1.0),
          add.smooth = getOption("add.smooth"),
          label.pos = c(4,2), cex.caption = 1)
{
\end{lstlisting}
\leavevmode\egroup

The continuation is indented to be below the first argument. The benefit
of this ``declaration by itself'' approach is that the additional
lines are always re-formatted with consistent indentation and we are
not creating a huge empty white space due to indentation.

\subsubsection*{Try formatR::tidy.source()}

The advice so far mostly concerns ``white space''. We would like
a programmer's text editor to handle automatically as much of that
as possible. 

The R package ``formatR'' \citep{formatr} has a function called
tidy.source() which can often (but not always) clean up code. Below
I've pasted in part of an Emacs session. I wrote a badly formatted
function, myfn(), and copied it to the clipboard, and then tidy.source()
reads the clipboard. It works like magic. 

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> myfn <- function(x){ if (x < 7) {i = 77; print(paste("x is less than 7 but i is", i))} else {print("x is excessive") }} 
> library(formatR)
> tidy.source()
function(x) {
    if (x < 7) {
        i = 77
        print(paste("x is less than 7 but i is", i))
    } else {
        print("x is excessive")
    }
}
\end{lstlisting}
\leavevmode\egroup

The tidy.source() function can get rid of equals sign assignments
if we ask it to. (In my opinion, it should do that by default.)

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> tidy.source(source = "clipboard", replace.assign = TRUE)
function(x) {
    if (x < 7) {
        i <- 77
        print(paste("x is less than 7 but i is", i))
    } else {
        print("x is excessive")
    }
} 
\end{lstlisting}
\leavevmode\egroup

The tidy.source() function can receive input as files or whole directories.

There are two reasons why tidy.source() is not a panacea. First, by
design, tidy.source() will fail if there are programming errors in
the original source code. That leads to a Catch-22. I want to clean
up the code to find out why it does not run, but tidy.source() cannot
clean it up because it does not run. Second, quite often it happens
that tidy.source() chokes on unexpected user code. Especially problematic
is code that has comments inserted in unexpected places. For example,
I recently ran tidy.source() on the file emb.r in the package Amelia
\citep{Amelia}.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> library(formatR)
> tidy.source("emb.r")
Error in base::parse(text = text, srcfile = NULL) :
  152:88: unexpected SPECIAL
151: }
152: if (ncol(as.matrix(startvals)) == AMp+1 && nrow(as.matrix(startvals)) == AMp+1)        %InLiNe_IdEnTiFiEr%
                                                                                           ^
\end{lstlisting}
\leavevmode\egroup

I would estimate that tidy.source() fails on about one-third of the
R code I randomly select from CRAN.

\subsection{(SEA .70) The \textquotedblleft\} else \{\textquotedblright{} policy. }

Did you notice ``\bgroup\inputencoding{latin9}\lstinline!} else {!\egroup
'' in the \bgroup\inputencoding{latin9}\lstinline!tidy.source()!\egroup
 output for \bgroup\inputencoding{latin9}\lstinline!myfn()!\egroup
? That's the correct style. We should not have the left squiggly brace
``\bgroup\inputencoding{latin9}\lstinline!}!\egroup
'' on a separate line from the ``\bgroup\inputencoding{latin9}\lstinline!else!\egroup
,'' and the right squiggly brace ``\bgroup\inputencoding{latin9}\lstinline!{!\egroup
'' should be on that same line. This is, well, obviously good (in
my opinion).

Why? Try this at the command line. 

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> if (x < 10) print("hello") 
[1] "hello" 
> else print("goodbye") 
Error: unexpected 'else' in "else"
\end{lstlisting}
\leavevmode\egroup

R does not realize that it is not yet finished with the if keyword's
work. The keyword else appears to begin a new thought, which is illegal.
The if's help page (run \bgroup\inputencoding{latin9}\lstinline!help("if")!\egroup
 or \bgroup\inputencoding{latin9}\lstinline!?"if"!\egroup
) is referring to this problem when it says, 
\begin{quote}
In particular, you should not have a newline between ‘\}’ and ‘else’
to avoid a syntax error in entering a ‘if ... else’ construct at the
keyboard or via ‘source’. For that reason, one (somewhat extreme)
attitude of defensive programming is to always use braces, e.g., for
‘if’ clauses.
\end{quote}
I agree with the somewhat extreme attitude, but will compromise: If
one uses squiggly braces, always follow the ``\bgroup\inputencoding{latin9}\lstinline!} else {!\egroup
'' policy. 

Some might follow a soft line on this, suggesting only that \textbf{users
should not} \textbf{begin a line with the word else}. That does not
go quite far enough for me. I'd add, \textbf{always use squiggles
after else.} This is simply a way of avoiding a very common coding
error. This code is OK:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
if (x < 7) print("so far, so good") else
print("this is else")
\end{lstlisting}
\leavevmode\egroup

But it invites a coding error like so:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
if (x < 7) print("so far, so good") else
print("this is else")
print("and we want this also to be with else, but it is not")
\end{lstlisting}
\leavevmode\egroup

To be perfectly clear, and to protect ourselves against editing errors
in the future, we could follow the ``somewhat extreme'' advice and
write this:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
if (x < 7) {
	print("so far, so good") 
} else {
    print("this is else")
    print("and we want this also to be with else")
}
\end{lstlisting}
\leavevmode\egroup


\subsubsection*{Counter-argument based on the R source code}

This would be a completely closed case if not for the fact that the
``\bgroup\inputencoding{latin9}\lstinline!} else {!\egroup
'' policy is ignored in vast expanses of the R source code. In the
R source code, scan for the keyword else and in almost every file,
one finds:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
}
  else
\end{lstlisting}
\leavevmode\egroup

A naked else! This is frustrating for writers of style guides. It
ignores the advice in the ``if'' help page. We cannot run this code
line-by-line. 

On the other hand, the function that includes that apparently runs!
Why doesn't that code crash? When an if/else statement is enclosed
in a larger area that is demarcated by squiggly braces, then R will
understand the naked else when it finds it. Observe the fix at the
command line:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> x <- 1
> {
+ if (x < 10) print("hello")
+ else
+ print("My dangling else")
+ }
[1] "hello"
\end{lstlisting}
\leavevmode\egroup

I don't think I'm going to have any luck persuading the R Core Development
Team that their naked elses need to be fixed. The best I can do is
to urge code writers to use ``\bgroup\inputencoding{latin9}\lstinline!} else {!\egroup
'' and make them responsible for errors that result from ignoring
that rule. 

One will note another interesting anomaly while reviewing R source
code. Unlike programs written in C, where a consistent style for the
placement of squiggly braces will be followed, in R we observe files
that do not follow a particular rule. In src/library/src/logLik.R,
we find functions in both the K\&R (\citealp{kernighan_c_1988}) C
style

\bgroup\inputencoding{latin9}
\begin{lstlisting}
nobs.logLik <- function(object, ...) {
    res <- attr(object, "nobs")
    if (is.null(res)) stop("no \"nobs\" attribute is available")
    res
}
\end{lstlisting}
\leavevmode\egroup

and we also find the vertically aligned squiggly braces approach:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
print.logLik <- function(x, digits = getOption("digits"), ...)
{
    cat("'log Lik.' ", paste(format(c(x), digits=digits), collapse=", "),
        " (df=",format(attr(x,"df")),")\n",sep="")
    invisible(x)
}
\end{lstlisting}
\leavevmode\egroup

I am at a loss to explain these stylistic variations, so I conclude
that R users can follow either style, while keeping in mind the ``\bgroup\inputencoding{latin9}\lstinline!} else {!\egroup
'' policy, which strongly pushes us toward the K\&R style.

\section{How to name functions.}

Now we begin to consider some issues that are more subjective. Many
styles are legal, but some are more easily understood. R syntax has
changed over the years, and some things that were illegal are now
allowed. And some styles that were standard might now be discouraged. 

\subsection{(.98 SEA) Avoid using names that are already in use by R, especially
common ones.}

Don't write functions named ``\bgroup\inputencoding{latin9}\lstinline!rep()!\egroup
'', ``\bgroup\inputencoding{latin9}\lstinline!seq()!\egroup
'', ``\bgroup\inputencoding{latin9}\lstinline!c()!\egroup
'', and so forth. Notice that my new function \bgroup\inputencoding{latin9}\lstinline!lm()!\egroup
 does not obliterate the one from the stats package, but it sure does
make it harder to use it.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> lm <- function(z) print("Hi, I'm z where lm was")
> x <- rnorm(100)
> y <- rnorm(100)
> lm (y ~ x)
[1] "Hi, I'm z where lm was"
> stats::lm(y ~ x)

Call:
stats::lm(formula = y ~ x)

Coefficients:
(Intercept)            x
    0.02688      0.01796
\end{lstlisting}
\leavevmode\egroup

As long as we remember that \bgroup\inputencoding{latin9}\lstinline!lm()!\egroup
 is in the namespace stats, we can find it. 

Similarly, packages can declare namespaces of their own. (Since R
version 2.14, all packages \emph{must} do so.) We are allowed to place
a new function like \bgroup\inputencoding{latin9}\lstinline!seq()!\egroup
 or \bgroup\inputencoding{latin9}\lstinline!lm()!\egroup
 into a package if we want to. Nevertheless, almost everybody will
hate to read code like that. 

The danger that user functions might interfere with core functionality
was at one time very serious. Now it is, for the most part, a historical
footnote. It is still possible to obliterate a function that is embedded
within a namespace, but doing so requires a bit of effort and mischief.\footnote{In case you wonder, here's how to cause the worst case scenario.
\begin{lyxcode}
nseq~<-~function(x)~print(\textquotedbl Hello,~good~to~see~you\textquotedbl )

assignInNamespace(\textquotedbl seq.default\textquotedbl ,~nseq,~\textquotedbl base\textquotedbl )
\end{lyxcode}
}

When we say that a namespace is imported, it means that all of the
functions in that namespace can be accessed by the function's name,
without the namespace name as a prefix. We might write \bgroup\inputencoding{latin9}\lstinline!base::seq(1, 10, length.out = 40)!\egroup
 to be clear, but we need only write \bgroup\inputencoding{latin9}\lstinline!seq(1, 10, length.out = 40)!\egroup
 because an R session imports the namespace base. I notice a trend
in R to suggest that one should not import whole namespaces unless
that is truly necessary, and even if a namespace is imported, we should
strive for clarity by using syntax that includes the namespace name.
In the source code for many R examples, one will find syntax like
\bgroup\inputencoding{latin9}\lstinline!graphics::par()!\egroup
 where, until recently, that would have simply been \bgroup\inputencoding{latin9}\lstinline!par()!\egroup
.

\subsection{(.65 SEA)Use periods to indicate classes, otherwise don't use periods
in function names. }

Instead, use camel case to name functions. This function name \bgroup\inputencoding{latin9}\lstinline!mySuperThing()!\egroup
 is better than \bgroup\inputencoding{latin9}\lstinline!my.super.thing()!\egroup
. 

The period in a function name has a special meaning in the S3 object-oriented
framework. A ``generic function'' (such as print() or summary())
is accompanied by methods that implement its work for particular kinds
of objects, such as \bgroup\inputencoding{latin9}\lstinline!print.function()!\egroup
 or \bgroup\inputencoding{latin9}\lstinline!print.lm()!\egroup
. Before the period, we have a function's name, and after the period,
we have the class name of the object being managed. The function name
\bgroup\inputencoding{latin9}\lstinline!my.super.thing()!\egroup
 suggests the user might have an object of class ``thing'' and that
\bgroup\inputencoding{latin9}\lstinline!my.super(x)!\egroup
 would diagnose the class of x and send the work to \bgroup\inputencoding{latin9}\lstinline!my.super.thing()!\egroup
. A camel cased function name \bgroup\inputencoding{latin9}\lstinline!mySuperThing()!\egroup
 will not convey the wrong meaning. 

If we were starting with a clean slate, I believe many R functions
would be re-named for the purposes of consistency. Since we do not
have a clean slate, we live with an accumulation of function names
from olde S and R. Changes in computer science--the growth of object-oriented
programming--cause new naming conventions. Consider some of the traditional
S function names that are still used in R, like \bgroup\inputencoding{latin9}\lstinline!read.table!\egroup
 and \bgroup\inputencoding{latin9}\lstinline!read.csv!\egroup
. Those are not method implementations of a generic function read().
The period is simply part of a shorthand of the form ``action.qualifier''.
Otherwise, if one had an object of type table, then read(x) would
call read.table(x). But it does not:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> example(table)
> class(tab) 
[1] "xtabs" "table"
> read(tab) 
Error: could not find function "read" 
\end{lstlisting}
\leavevmode\egroup

I believe that, if these functions were being created today, they
would be named \bgroup\inputencoding{latin9}\lstinline!readTable()!\egroup
 and \bgroup\inputencoding{latin9}\lstinline!readCSV()!\egroup
. 

In the R source code, there are some very confusing function names
and I have a hard time believing we would use them if we were re-designing
everything today. The file src/library/base/readhttp.R has a function
called \bgroup\inputencoding{latin9}\lstinline!url.show()!\egroup
, which follows none of the styles that I recognize. There's no class
\bgroup\inputencoding{latin9}\lstinline!show!\egroup
 and \bgroup\inputencoding{latin9}\lstinline!url()!\egroup
 is not a generic function. In the ``action.qualifier'' tradition,
it would be \bgroup\inputencoding{latin9}\lstinline!show.url()!\egroup
. And why not \bgroup\inputencoding{latin9}\lstinline!showURL()!\egroup
? I hasten to point out that the same file includes some camel cased
functions like \bgroup\inputencoding{latin9}\lstinline!defaultUserAgent()!\egroup
.

I like camel cased function names. They are common in Objective-C
and Java. Some programmers vigorously disagree. Programmers trained
in C++ seem to hate camel case names, almost at a visceral level.
As a result, we find a division of opinion on function names. As a
spot check, consider two of my favorite packages, MASS \citep{venables_modern_2002}
and car \citep{fox_r_2011}. There are not many camel case function
names in the MASS, where we find brief names in lower case letters
(such as \bgroup\inputencoding{latin9}\lstinline!boxcox()!\egroup
). In contrast, car calls that \bgroup\inputencoding{latin9}\lstinline!boxCox()!\egroup
. When I started using R, Professor Fox used function names with periods,
but he has been systematically weeding them out and replacing them
with camel case names. If those two packages are counterbalancing
each other in my mind (for and against camel case functions), the
leading packages for mixed effects models, nlme \citep{pinheiro_nlme:_2012}
and lme4 \citep{lme4}, weigh in on the camel case side of the ledger. 

In conclusion, users should avoid gratuitous periods in function names
because, after S3, the period has special meaning in R. When a function
has been declared as a generic, then that function's name followed
by a period has an object-oriented meaning. A period is not merely
word separation. New functions introduced in R tend to use either
camel case names (\bgroup\inputencoding{latin9}\lstinline!browseVignettes()!\egroup
) or underscores (\bgroup\inputencoding{latin9}\lstinline!get_all_vars()!\egroup
). Considering recent additions to R, I believe that the chance of
finding a decorative period in a new function name is almost zero.
But we are still living with an awful lot of older counter-examples.

\section{How to name variables (and objects).}

\subsection{(1.0 SEA) Follow the \textquotedblleft letters and numbers\textquotedblright{}
rule.}

R variable names must 
\begin{enumerate}
\item begin with an alphabetical character
\item include only letters, numbers and the symbols ``\_'' and ``.''. 
\end{enumerate}
They must not include ``{*}'',''?'',''!'',''\&'' or other
special symbols. They must not include spaces.

One peculiar side effect of this rule is that the ellipsis symbol,
three periods, ``...'', is actually a legal object name. That's
three periods, which is just as legal as aaa or bbb. Many R functions
allow the argument ``...'', most users don't realize it literally
is a word. When that is listed as a function argument, then any argument
that the user includes is gobbled up by ``...''. 

\subsection{(1.0 SEA) Never name a variable T or F. }

Almost everybody (99.999\%) will agree with this. These are too easily
mistaken for TRUE and FALSE values. Since R uses TRUE and FALSE as
vital elements of almost all commands and functions, and since users
are allowed to abbreviate those as T or F, a horrible confusion can
develop if variables are named T or F. 

Here's some good news. R will not allow users to name variables TRUE
or FALSE:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> TRUE <- 7
Error in TRUE <- 7 : invalid (do_set) left-hand side to assignment
\end{lstlisting}
\leavevmode\egroup

But R will not prevent the usage of T and F for variable names.

\subsection{(.75 SEA) Avoid declaring variables that have the same names as widely
used functions.}

This is just a handy rule of thumb now, but it used to be a ``watch
out for that tree!'' warning. In 2001, I created a variable ``rep''
(for Republican party members) and nothing worked in my program. In
exasperation, I wrote to the r-help list, and learned that I had obliterated
R's own function \bgroup\inputencoding{latin9}\lstinline!rep()!\egroup
 with my variable. \bgroup\inputencoding{latin9}\lstinline!rep()!\egroup
 is used inside many R functions and thus obliterating it was a very
serious mistake. In 2002 or so, the R system was revised so that user-declared
variables cannot ``step on'' R system functions. Nevertheless, it
is disconcerting to me (probably others) when users create variables
with names like ``lm'', ``rep'', ``seq'', and so forth. 

\subsection{(0.50 SEA) Use long names for infrequently used variables. }

And use short names for variables that will be used very often.

If a variable is going to be used twice, we might as well be verbose
about it. ``xlog'' is better than ``xl'', if we are only writing
it a few times. If we are going to use a name 50 times in a 5 line
program, we should choose a short one. For abbreviations, include
a comment to remind the reader what the thing stands for.

\subsection{(0.10 SEA) Suggested naming scheme: keep related objects in an alphabetically
sorted scheme.}

This is my personal naming scheme. Nobody but me follows this policy
now, but I like it so much I'm tacking it onto the end of this essay.
I believe that R code is much more readable if objects that ``go
together'' begin with the common series of letters. As seen by ls(),
the related pieces should always be together. From now on, when I
work with a variable named ``x'', then all transformations will
begin with ``x''. I will use ``xlog'' rather than ``logx'' and
so forth. 

Example 1. Create a numeric variable, recode it as a factor, then
create the ``dummy'' variables that correspond. I include the output
in order to emphasize the clarity due to the alphabetical emphasis:

<<>>=
x <- runif(1000, min = 0, max = 100)
xf <- cut(x, breaks = c(-1, 20, 50, 80, 101), labels = c("cold", "luke", "warm", "hot"))
xfdummies <- contrasts(xf, contrasts = FALSE )[xf,]
colnames(xfdummies) <-  paste("xf", c("cold", "luke", "warm", "hot"), sep="")
rownames(xfdummies) <- names(x)
dat <- data.frame(x, xf, xfdummies)
head(dat)
@

I have not included the output of these code chunks, but the alphabetical
emphasis is demonstrated in them.

Example 2. Estimate a regression, calculate summary information.

<<echo=T, eval=F>>=
set.seed(12345)
x1 <- rnorm(200, m = 300, s = 140)
x2 <- rnorm(200, m = 80, s = 30)
y <- 3 + 0.2 * x1 + 0.4 * x2 + rnorm(200, s=400)
dat <- data.frame(x1, x2, y); rm(x1,x2,y)
m1 <- lm (y ~ x1 + x2, data = dat)
m1summary <- summary(m1)
m1se <- m1summary$sigma
m1rsq <- m1summary$r.squared
m1coef <- m1summary$coef
m1aic <- AIC(m1)
@

Example 3. Run a regression, collect mean-centered and residual centered
variants of it.

<<ps10, fig=T, eval=F, height=5, width=9>>=
library(rockchalk)
dat$y2 = with(dat, 3 + 0.02 * x1 + 0.05 * x2 + 2.65 * x1 *x2 + rnorm(200, s=4000))
par(mfcol=c(1,2))
m1 <- lm(y2 ~ x1 + x2, data = dat)
m1i <- lm(y2 ~ x1 * x2, data = dat)
m1ps <- plotSlopes(m1, plotx = "x1", modx = "x2")
m1ips <- plotSlopes(m1i, plotx = "x1", modx = "x2")
m1imc <- meanCenter(m1i)
m1irc <- residualCenter(m1i)	
@

\section{Conclusion}

R can be understood at several levels, varying in sophistication from
an elementary statistics course or to an advanced platform for the
development of computer programming concepts. In the future, I will
be more cautious to teach new R users about coding style. I intend
to prevent the accumulation of bad habits that result in code that
is difficult to read and hard to debug. 

Users who ask for help in the r-help email list \footnote{\url{https://www.r-project.org/mail.html}}
or on web forums \footnote{e.g., \url{https://stackoverflow.com/questions/tagged/r}}
are well advised to remember the importance of style. Most newcomers
believe that the experts will understand what they write, but that's
not true. Experts will find it much easier to spot errors in code
that has the correct indentation and uses a proper naming scheme for
variables and functions. In my experience, the most likely source
of trouble in R code is not actually the style, but rather poor compartmentalization
of separate calculations. The potential to compartmentalize, however,
is obscured by bad style. 

When users throw together 2000 lines of spaghetti code with no indentation
(I can point to examples on CRAN), there's almost no chance than anyone
except the author will be able to understand and extend that kind
of code. Ugly code writers will respond, ``my ugly code runs!''
That misses the point. Coding style is not about making things ``work,''
it is about making them work in a way that is understood by the widest
possible audience. And where possible, the code should be re-usable
and extended to other purposes. 

\bibliographystyle{chicago}
\bibliography{rockchalk}

\end{document}

%% LyX 2.4.4 created this file.  For more info, see https://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[11pt,letterpaper,english]{extarticle}
\usepackage{lmodern}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage{babel}
\usepackage{float}
\usepackage[authoryear]{natbib}
\usepackage[pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
\special{papersize=\the\paperwidth,\the\paperheight}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
<<echo=F>>=
  if(exists(".orig.enc")) options(encoding = .orig.enc)
@
\providecommand*{\code}[1]{\texttt{#1}}

\@ifundefined{date}{}{\date{}}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
%\VignetteIndexEntry{outreg}

% Following comment is required. Do not delete it.
%\usepackage{Sweave}

\usepackage[includehead,includefoot]{geometry}
\geometry{
lmargin=1in,
rmargin=1in,
tmargin=0.75in,
bmargin=1.0in,
headheight=0pt,
headsep=0pt,
marginparwidth=0pt,
footskip=1.5\baselineskip,
}

\usepackage{booktabs}
\usepackage{dcolumn}

% paste in
%%\input{theme/guidePreambleHeader.tex}

% $ This nullifies emacs subscripting in following%
%%BEGIN: This begins the custom header
\usepackage{array}
\usepackage{xparse}
\usepackage{babel}
\usepackage{datetime}
\usepackage{xcolor}
\usepackage{colortbl}
\usepackage{graphicx}
\usepackage{multicol}

\definecolor{darkblue}{HTML}{1e2277}
\definecolor{kublue81186}{RGB}{0,81,186}
\newdateformat{monddyyyy}{\shortmonthname[\THEMONTH].~\THEDAY,~\THEYEAR}


%% BEGIN custom header
\ExplSyntaxOn
\NewDocumentCommand \guidesetup { m }
 {
  \keys_set:nn { guide } { #1 }
 }

\int_new:N \g_guide_authors_int
\prop_new:N \g_guide_authors_prop
\tl_new:N \l__guide_authors_tl

\keys_define:nn { guide }
 {
  author .code:n =
   {
    \int_gincr:N \g_guide_authors_int
    \keys_set:nn { guide / author } { #1 }
   },
  author/firstname .code:n = { \guide_add_author:nn { firstname } { #1 } },
  author/lastname .code:n = { \guide_add_author:nn { lastname } { #1 } },
  author/affiliation .code:n = { \guide_add_author:nn { affiliation } { #1 } },
  author/email .code:n = { \guide_add_author:nn { email } { #1 } },
  leftlogo .tl_set:N = \l_guide_leftlogo_tl,
  rightlogo .tl_set:N = \l_guide_rightlogo_tl,
  number .tl_set:N = \l_guide_number_tl,
  title .tl_set:N = \l_guide_title_tl,
  subtitle .tl_set:N = \l_guide_subtitle_tl,
  keywords .tl_set:N = \l_guide_keywords_tl,
  url .tl_set:N = \l_guide_url_tl,
  title .initial:n = {TITLE~NEEDED},
}
\NewDocumentCommand \guidehdr { +O{} }
 {
  \group_begin:
  \keys_set:nn { guide } { #1 }
  \guide_hdr:
  \group_end:
 }
\cs_new_protected:Nn \guide_hdr:
 {
   \centering
   \noindent\fcolorbox{kublue81186}{white}{
  \begin{tabular}
   {
    @{}c
    >{\centering}m{ \dim_eval:n { \textwidth - 2.5in - 6\tabcolsep - 2\arrayrulewidth } }
    c@{}
   }
   &&\tabularnewline[-\dim_eval:n { -\normalbaselineskip + 3ex }]
  \begin{tabular}{@{}>{\centering}p{1.25in}@{}}
  \includegraphics[width=.75in]{\l_guide_leftlogo_tl}
  \end{tabular}
  &
  \centering
    {\textsc{\large{}\l_guide_title_tl}}
    \tl_if_empty:NF \l_guide_subtitle_tl
    {
      {\\\textsc{\large{}\l_guide_subtitle_tl}}
    }
  \\
  \textcolor{kublue81186}{\rule[0.5ex]{1\linewidth}{0.7pt}}\\
  \guide_print_authors:
  &
  \begin{tabular}{@{}>{\centering}p{1.25in}@{}}
  \includegraphics[width=1.25in]{\l_guide_rightlogo_tl}
  \end{tabular}
  \tabularnewline
  \tl_if_empty:NTF \l_guide_number_tl
   { \leavevmode\vphantom{G}}
   {\small\bfseries { Guide~No: ~ \tl_use:N \l_guide_number_tl }} & 
   \tl_if_empty:NF \l_guide_keywords_tl
   {\centering
    \textbf{Keywords}: ~ \l_guide_keywords_tl}
   & 
   \small\bfseries\monddyyyy\today \tabularnewline
   \tl_if_empty:NF \l_guide_url_tl
   {
   \multicolumn{3}{c}{
    See ~ \url{\l_guide_url_tl} ~ for ~ updates.}
     }
  \tabularnewline
  \end{tabular}
  }
}

\cs_new_protected:Nn \guide_add_author:nn
 {
  \prop_gput:Nxn \g_guide_authors_prop
   { #1 \int_to_arabic:n { \g_guide_authors_int } }
   { #2 }
 }
\cs_generate_variant:Nn \prop_gput:Nnn { Nx }
\cs_new_protected:Nn \guide_print_authors:
 {
  \tl_clear:N \l__guide_authors_tl
  \int_step_inline:nnnn { 1 } { 1 } { \g_guide_authors_int }
   {
    \tl_put_right:Nn \l__guide_authors_tl
     {
      \textbf
       {
        \prop_item:Nn \g_guide_authors_prop { firstname ##1 }~
        \prop_item:Nn \g_guide_authors_prop { lastname ##1 },~
       }
       \prop_item:Nn \g_guide_authors_prop { affiliation##1 },~
       \href{mailto:\prop_item:Nn \g_guide_authors_prop { email##1 } }
       {\url{\prop_item:Nn \g_guide_authors_prop { email##1 }}}
       \\
     }
   }
   \tl_use:N \l__guide_authors_tl
 }
\ExplSyntaxOff
%%END This ends the custom header


%%Now the hyperref colors

\hypersetup{
 unicode=true,pdfusetitle,bookmarks=true,
 bookmarksnumbered=false, bookmarksopen=false,
 breaklinks=true, pdfborder={0 0 0},
 pdfborderstyle={}, backref=false,
 colorlinks=true, linkcolor=darkblue,
 urlcolor=darkblue, citecolor=darkblue}

%% Was previously in "preambleFloatCenter.tex"
%%BEGIN: graphics and such inside floats are centered by default
\renewenvironment{figure}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{figure}
 }{%
   \@float{figure}[#1]%
 }%
 \centering
}{%
 \end@float
}
\renewenvironment{table}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{table}
 }{%
   \@float{table}[#1]%
 }%

 \centering
}{%
 \end@float
}
%%END: graphics and such inside floats are centered by default

% paste
%%\input{theme/preambleFooter.tex}

\usepackage{fancyhdr}
\usepackage{lastpage}
\usepackage{xparse}

\ExplSyntaxOn
\NewDocumentCommand \footersetup { +m }
{
\keys_set:nn { footer } { #1 }
}
\keys_define:nn { footer }
{	
    leftside .code:n = {
\keys_set:nn { footer / leftside } { #1 }
},
rightside .code:n = {
\keys_set:nn { footer / rightside } { #1 }
},
    leftside / lone .tl_set:N = \l_footer_leftside_lone_tl,
    leftside / ltwo .tl_set:N = \l_footer_leftside_ltwo_tl,
    leftside / lthree .tl_set:N = \l_footer_leftside_lthree_tl,
    rightside / rone .tl_set:N = \l_footer_rightside_rone_tl,
    rightside / rtwo .tl_set:N = \l_footer_rightside_rtwo_tl,
    rightside / rthree .tl_set:N = \l_footer_rightside_rthree_tl,
    leftside .initial:n = {lone = , ltwo = ~, lthree = ~ },
    rightside .initial:n = {rone = ~, rtwo = ~,  rthree = ~},
    leftside .default:n = {lone = , ltwo = ~, lthree = ~ },
    rightside .default:n = {rone = ~, rtwo = ~,  rthree = ~},
}
\NewDocumentCommand \guideftr { +o }
{
  \IfValueT { #1 } 
  {
    \keys_set:nn { footer } { #1 }
  }
  \guide_ftr:
}
\cs_new_protected:Nn \guide_ftr:
{
  % Now fancy header/footer settings
  \lhead{}%
  \chead{}%
  \rhead{}%
  \renewcommand{\headrulewidth}{0pt}%
  \lfoot{}%
  \cfoot{{Page} ~ \thepage {} ~ { of } ~ {} \pageref{LastPage}}%
  \rfoot{}%

  \fancypagestyle{firststyle}%
  {%
    \lhead{}%
    \chead{}%
    \rhead{}%
    \lfoot{}%
    \cfoot{}%
    \rfoot{}%
    \renewcommand{\headrulewidth}{0pt}%
    \fancyfoot[L]{\l_footer_leftside_lone_tl {}\\%
      \l_footer_leftside_ltwo_tl {}\\%
      \l_footer_leftside_lthree_tl {}}%
    \fancyfoot[R]{\l_footer_rightside_rone_tl {}\\%
      \l_footer_rightside_rtwo_tl {}\\%
      \l_footer_rightside_rthree_tl {}}%
      \renewcommand{\footrulewidth}{.2pt}%
  }%
  \thispagestyle{firststyle}
}
\ExplSyntaxOff

\pagestyle{fancy}
%% End of fancy footer setup

%% Now the addressFooter file
\urlstyle{tt}
\footersetup{
  leftside={
    lone={Data~Adjacent~LLC}, 
    ltwo={Oletha{,}~KS~66061},
    lthree={}},
  rightside={
    rone=Web: \url{https://pj.freefaculty.org},
    rtwo=Email: \href{mailto:pauljohn32@freefaculty.org}{\url{pauljohn32@freefaculty.org}},
    rthree={}    
}
}
\guideftr




% paste from
%%\input{theme/guidePreambleSweavel.tex}
%%% From beamer slide:
%% \usepackage{Sweave}
%% 
%% This controls display of code chunks
\usepackage{ae,fancyvrb,relsize,listings}

\providecommand{\Sweavesize}{\normalsize}
\providecommand{\Rsize}{}
\renewcommand{\Rsize}{\normalsize}
\providecommand{\Routsize}{\scriptsize}

\providecommand{\Rcolor}{\color[rgb]{0.1, 0.1, 0.1}}
\providecommand{\Routcolor}{\color[rgb]{0.2, 0.2, 0.2}}
\providecommand{\Rcommentcolor}{\color[rgb]{0.101, 0.43, 0.432}}

\providecommand{\Rbackground}{\color[gray]{0.91}}
\providecommand{\Routbackground}{\color[gray]{0.935}}
% Can specify \color[gray]{1} for white background or just \color{white}

\lstdefinestyle{Rinput}{
  language=R,
  escapechar=`,
  fancyvrb=false,%
  tabsize=2,%
  breaklines=true,
  breakatwhitespace=true,%
  captionpos=b,%
  frame=single,%
  framerule=0.2pt,%
  framesep=1pt,%
  showstringspaces=false,%
  basicstyle=\Rsize\Rcolor\ttfamily,%
  columns=fixed%,
  \lst@ifdisplaystyle\scriptsize\fi,%,
  commentstyle=\Rcommentcolor\ttfamily,%
  identifierstyle=,%
  keywords=\bfseries,%
  keywordstyle=\color[rgb]{0, 0.5, 0.5},
  escapeinside={(*}{*)},
  literate={~}{{$\sim$}}1{==}{{=\,=}}2{--}{{-\,-}}2,
  alsoother={$},
  alsoletter={.<-},%
  otherkeywords={!,!=,~,$$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},%
  backgroundcolor=\Rbackground,%
  numbers=left,%
  %numberblanklines=false,%
  stepnumber=5,
  firstnumber=1,
  numberstyle={\tiny}
}%

% Other options of interest:
% frame=single,framerule=0.1pt,framesep=1pt,rulecolor=\color{blue},
% numbers=left,numberstyle=\tiny,stepnumber=1,numbersep=7pt,
% keywordstyle={\bf\Rcolor}

\lstdefinestyle{Routput}{fancyvrb=false,
  literate={~}{{$\sim$}}1{R^2}{{$R^{2}$}}2{^}{{$^{\scriptstyle\wedge}$}}1{R-squared}{{$R^{2}$}}2,%
  basicstyle=\Routcolor\Routsize\ttfamily,%
  backgroundcolor=\Routbackground,
  language=R,
  escapechar=`,
  fancyvrb=false,%
  tabsize=2,%
  breaklines=true,
  breakatwhitespace=true,%
  captionpos=b,%
  frame=single,%
  framerule=0.2pt,%
  framesep=1pt,%
  showstringspaces=false,%
  columns=fixed%,
  \lst@ifdisplaystyle\scriptsize\fi,%
  identifierstyle=,%
  keywords=\bfseries,%
  keywordstyle=\color[rgb]{0, 0.5, 0.5},
  escapeinside={(*}{*)},
  literate={~}{{$\sim$}}1 {==}{{=\,=}}2,
  alsoother={$},
  alsoletter={.<-},%
  otherkeywords={!,!=,~,$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},
  numbers=left,
  %numberblanklines=false,%
  stepnumber=5,
  firstnumber=1,
  numberstyle={\tiny}
}

\newenvironment{Schunk}{}{}
\newenvironment{Sinput}{}{}
\let\Sinput\relax
\let\Scode\relax
\let\Soutput\relax
\lstnewenvironment{Sinput}{\lstset{style=Rinput}}{}
\lstnewenvironment{Scode}{\lstset{style=Rinput}}{}
\lstnewenvironment{Soutput}{\lstset{style=Routput}}{}
%%end paste in from guidePreambleSweavel.tex


\lstset{tabsize=2, breaklines=true, style=Rinput, breakatwhitespace=true}

\fvset{listparameters={\setlength{\topsep}{0em}}}

\usepackage{xcolor}
\definecolor{light-gray}{gray}{0.90}
\usepackage{realboxes}
\providecommand*{\code}[1]{\texttt{#1}}
\renewcommand{\code}[1]{%
\Colorbox{light-gray}{#1}%
}%


\usepackage{siunitx}
%following now in each table
\sisetup{
  input-symbols = ( )
}
%,
%  group-minimum-digits = 9,
%  table-number-alignment = center,
%  table-space-text-pre = (,
%  table-align-text-pre = false,
%  table-align-text-post = false,
%  table-space-text-post = {***},
%  parse-units = false
%}
% Added by lyx2lyx
\setlength{\parskip}{\medskipamount}
\setlength{\parindent}{0pt}

\makeatother

\usepackage{listings}
\renewcommand{\lstlistingname}{\protect\inputencoding{latin9}Listing}

\begin{document}
\noindent %% Fill in values of the arguments here, 
%% If blanks are needed, must insert value " ~ "
%% If comma needed inside value, wrap in {}.
%% Delete secondauthor and thirdauthor if not needed
\guidesetup{%
  author={
    lastname=Johson, 
    firstname=Paul, 
    affiliation=Data~Adjacent~LLC,
    email=pauljohn32@freefaculty.org},
  url={https://pj.freefaculty.org/guides/},
  keywords={R, rockchalk, outreg, LaTeX, regression~tables},
  title={Regression~Table~Survey~with~outreg},
  leftlogo={theme/logoleft.pdf},
  rightlogo={theme/logo-vert.pdf},
  number=49,
}
\guidehdr

% tmpout directory must exist first
<<tmpout, echo=FALSE, include=FALSE, results=hide>>=
if(!dir.exists("tmpout")) dir.create("tmpout", showWarnings=FALSE)
@

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\SweaveOpts{prefix.string=tmpout/t, split=F, ae=FALSE, height=3, width=5}

<<Roptions, echo=FALSE, include=FALSE, results=hide>>=
opts.orig <- options()
options(width=100, prompt=" ", continue="  ")
options(useFancyQuotes = FALSE) 
set.seed(12345)
options(SweaveHooks=list(fig=function() par(ps=10)))
pdf.options(onefile=FALSE,family="Times",pointsize=10)
@

<<echo=FALSE,results=hide>>=
fn1 <- "theme/logoleft.pdf"
fn2 <- "theme/logo-vert.pdf"
if(!file.exists("theme")) dir.create("theme")

blankpdf <- function(file, height=3, width=3, pointsize=20, replace = FALSE){
    if(file.exists(file) && !replace) return(TRUE)
    pdf(file=file, width=width, height=height, paper="special", 
        onefile=FALSE, pointsize=pointsize)
    par(mar=c(1,1,1,1))
    plot(1:2, 1:2, type = "n", axes=FALSE, xlab="", ylab="")
    ##text(1.5, 1.5, "left\n logo", axes=FALSE) 
    ##box(which="plot")
    dev.off()
    if(file.exists(file)) TRUE else FALSE
}
blankpdf(fn1)
blankpdf(fn2)
@

%The listings class used here allows within-document style
%changes. R input boxes are governed by 
% "Rsize", "Rbackground" and "Rcolor", while R output boxes depend on
% "Routsize", "Routbackground", and "Routcolor". Colors
% can be specified in many ways, as shown here
%\def\Rsize{\huge\ttfamily}
%\def\Routsize{\huge}
%\def\Rbackground{\color[gray]{0.90}}
%\def\Routbackground{\color[gray]{0.40}}
%\def\Rcolor{\color[gray]{0.60}
%\def\Routcolor{\color[rgb]{0.9, 0.1, 0.1}]}
%\def\Rcommentcolor{\color{green}}
\begin{abstract}
\noindent This is a comprehensive demonstration of the \code{outreg()}
function in \code{rockchalk}. This vignette demonstrates the ability
to integrate model estimates, to assign labels for models and variables,
and to customize the selection of summary statistics. A major innovation
in recent versions is the introduction of the ability to generate
``decimal-centered'' columns. This has been the most widely requested
feature for a while.
\end{abstract}
\tableofcontents{}

The \code{rockchalk} package for R \citep{RCore} was created to
facilitate authors in stats classes who need to make presentable tables
and graphs for regression models. It does not aspire to be a comprehensive
table-maker, mainly because R package writers are not consistent in
their methodology. Procedures called ``regression'' within R's base
have a certain internal consistency, but user-contributed packages
do not. The \code{outreg} function works for regression models that
I use regularly, usually in the base of R, like \code{lm}, \code{glm},
and for \code{lme4} objects. Other regression packages may work.
I've tried to be somewhat general in the design.

\noindent The major new feature is ``decimal-centered'' columns.
In the implementation of centering, we tested approaches based on
two LaTeX packages, \code{dcolumn} and \code{siunitx}. \code{dcolumn}
output it is mostly adequate, but it leaves problems in column headers
and non-numeric content alignment. \code{siunitx} is a much more
massive framework. There is much more danger of outright failure because
of the immensity of \code{siunitx} itself. However, at the current
time, all of the examples we have tested indicate that the \code{siunitx}-based
tables are superior. there are hopeful signs it can deal with some
very tough examples. See Table \ref{tab:ex16s} I have a solution
using \code{siunitx} for a troublesome table that failed badly with
\code{dcolumn} (Table \ref{tab:ex16d}). The user can designate which
method should be used by assigning the parameter \code{centering = \textquotedbl dcolumn\textquotedbl}
or \code{centering = \textquotedbl siunitx\textquotedbl}. NOTE: it
is necessary for the user to insert either \code{\textbackslash usepackage\{dcolumn\}}
or \code{\textbackslash usepackage\{siunitx\}} in the preamble! The
aim is for this whole thing to ``just work'' without any more attention
than that. Please let me know if there is trouble.

\section{Briefly: outreg()}

First, run regression models. Second, give the fitted regression object(s)
to \code{outreg}. When \code{outreg} runs, it will have 2 effects.
First, it will write out LaTeX code to the screen (unless \code{print.results=FALSE}).
A user might ``copy/paste'' that code into a LaTeX document. (Or,
write it in a file and import it in the future. (Or, as in the case
of this Sweaved document, the output goes directly into the result
file.)

\section{Briefly: table vs tabular}

This is the most confusing thing for LaTeX beginners. \emph{Put bluntly,
a ``table'' is not a table}. 

Terminology
\begin{description}
\item [{tabular}] In LaTeX, a ``\code{tablular}'' object is a grid,
a display like a ``spreadsheet''. You'd call that a ``table'',
I believe.
\item [{table}] In LaTeX, a ``\code{table}'' is something else. It is
a document ``subsection'' that ``floats'' around in the document.
It is a container. A tabular is placed inside one of these table containers.
Table objects are numbered, can be used in cross references (if they
have labels)
\end{description}
If we are writing LaTeX code by hand, a tabular inside a table will
look like 

\bgroup\inputencoding{latin9}
\begin{lstlisting}
\begin{table}
    \caption{The title of the floating object is specified in caption}
    \label{tab:ex1}% cross referencing use \ref{tab:ex1}
\begin{tabular}
  ...code to create tablar defined here
\end{tabular}
\end{table}
\end{lstlisting}
\leavevmode\egroup

The big question is ``should my outreg function write the tabular
only, or also the table container''?

I used to think the only reasonable answer was ``tabular only'',
but now I see reasons why we might want to do both. So it is an option,
as explained next.

\subsection*{rockchalk lets you choose, float or no float}

The rockchalk outreg function has a parameter, \code{float=FALSE}
or \code{float=TRUE}, to determine whether the output should include
the floating table part along with the tabular object. If outreg users
specify the argument title, then they are implicitly setting \code{float=TRUE}
and the value for title is used as the LaTeX caption. 

The default for \code{float} is \code{FALSE}, because I have usually
preferred to control the floating table from my document. However,
I recognize that many people who are better at LaTeX than I am disagree,
they want to write the table and tabular structures into one file. 

\subsection*{Why would a user prefer float=TRUE?}

If you are not using LyX, then it may be convenient to let float=TRUE
so the floating table can be handled from the outreg function. outreg
allows the author to specify the caption and the label. However, I
did not allow for any more fine-grained customizations, especially
table placement details.

\subsection*{Why I don't generally use float=TRUE}

LaTeX has options to control the placement of floating table structures
that the rockchalk outreg function does not adjust. I think that if
the LaTeX author wants to adjust those things, it is easier for the
author to control them in the document itself, rather than the outreg
function that writes a tabular object. 

In LyX, the user interface has a pull down menu to create floating
objects and the GUI includes a feature to set the title (sorry, the
'caption'), and it also includes a way to set a label. If I use this
method, then LyX is aware of this thing and the LyX cross-referencing
system is available. 

\subsection*{Another reason to use float=FALSE}

In an instructional document using Sweave, the code chunks will print
out where they ``are'' in the input file. 

In LyX, there are some cross-referencing tools. It is more desirable
to create the table container in LyX, so then cross-references work
correctly. If one creates a table (floating container), and then puts
the tabular-creating code chunk inside, then the code will print out
with the table. And cross references succeed using the LyX pull down
menus. That is demonstrated in Tables \ref{tab:ex11}, \ref{tab:ex11t},
\ref{tab:ex8}, \ref{tab:ex17}.

\section{Torture test of \textquotedblleft rockchalk\textquotedblright{} outreg
function}

In most of these examples, I've taken the one-step ``float=TRUE''
option, but I have some examples where I've manually created the LaTeX
float and placed an outreg tabular inside it.

<<>>=
set.seed(2134234)
dat <- data.frame(x1 = rnorm(100), x2 = rnorm(100))
dat$y1 <- 30 + 5 * rnorm(100) + 3 * dat$x1 + 4 * dat$x2
dat$y2 <- rnorm(100) + 5 * dat$x2
m1 <- lm(y1 ~ x1, data = dat)
m2 <- lm(y1 ~ x2, data = dat)
m3 <- lm(y1 ~ x1 + x2, data = dat)
gm1 <- glm(y1 ~ x1, family = Gamma, data = dat)
@

\subsection{tight and wide styles for regression tables}

In my terminology, a tight table has one ``narrow'' column for a
regression (parameter and standard error stacked on top of each other).
A table that is not in the tight format is wide, it has 2 columns
for each table (parameter and standard error side by side).

A tight table is the default, it seems to be what many political science,
sociology, and economics students will need. See Table \ref{tab:ex1}.
A wide format table is, in my opinion, more pleasant for the eyes
and seems to be more popular in psychology. See Table \ref{tab:ex1w}. 

<<ex1, echo=F, results=tex>>=
library(rockchalk)
vl <- c("(Intercept)" = "Intercept")
ex1 <- outreg(m1, title = "My One Tightly Printed Regression (uncentered)",
                    label = "tab:ex1",
                    float = TRUE, print.results = FALSE, varLabels=vl)
# cat that, don't print it
cat(ex1)
@

<<ex1w,results=tex>>=
library(rockchalk)
ex1w <- outreg(m1, title = "My Wide Format \"side-by-side\" columns (uncentered)", label = "tab:ex1w", tight = FALSE, float = TRUE, print.results = FALSE)
cat(ex1w)
@

\subsection*{Decimal-centered test case 1}

A decimal-centered version of the tight column format can be seen
in Table \ref{tab:ex2d} and the decimal-centered version of the wide
format is seen in Table \ref{tab:ex2wd}

<<ex1d, results=tex>>=
library(rockchalk)
ex2d <- outreg(m1, title = 'Tight column with centering = "dcolumn"', label = "tab:ex2d", centering = "dcolumn", float = TRUE, print.results=FALSE)
cat(ex2d)
@

<<ex1s, results=tex>>=
library(rockchalk)
ex2s <- outreg(m1, title = 'Tight column with centering = "siunitx"', label = "tab:ex2s", centering = "siunitx", float = TRUE)
@

<<ex2wd, results=tex>>=
library(rockchalk)
ex1wd <- outreg(m1, title = 'Wide (not tight) format with centering = "dcolumn"',  label = "tab:ex2wd", tight = FALSE, centering = "dcolumn",  float = TRUE, print.results = FALSE)
cat(ex1wd)
@

<<ex2ws, results=tex>>=
ex1ws <- outreg(m1, title = 'Wide (not tight) format with centering = "siunitx"',  label = "tab:ex2ws", tight = FALSE, centering = "siunitx", float = TRUE, print.results = FALSE)
cat(ex1ws)
@

\subsection*{alpha level adjustments}

In Table \ref{tab:ex2p}, I demonstrate that the alpha parameter can
be used to select different values for the critical region. In Tables
\ref{tab:ex2pd} and \ref{tab:ex2ps}, the centered-with-dcolumn and
centered-with-siunitx are presented. I see some trouble here because
the stars are not evenly spaced in the dcolumn-based Table {\ref{tab:ex2pd}}.
However, the siunitx based Table {\ref{tab:ex2ps}} may be adequate.
This is one of the reasons I now \emph{strongly} lean toward use of
\code{siunitx} if centering is desired. Several alpha adjustments
are scattered about in the examples in this essay, just to make sure
they work.

<<results=tex>>=
ex2p <- outreg(list("Fingers" = m1), tight = FALSE, title = "Ability to change p values (not centered)", label = "tab:ex2p",  float = TRUE, alpha = c(0.1, 0.05, 0.01)) 
@

<<results=tex>>=
ex2pd <- outreg(list("Fingers" = m1), tight = FALSE,  title = "Ability to change p values (dcolumn)", label = "tab:ex2pd", centering = "dcolumn", float = TRUE, alpha = c(0.1, 0.05, 0.01)) 
@

<<results=tex>>=
ex2ps <- outreg(list("Fingers" = m1), tight = FALSE,  title = "Ability to change p values (siunitx)", label = "tab:ex2ps", centering = "siunitx", float = TRUE, alpha = c(0.1, 0.05, 0.01)) 
@

\subsection*{Several models in same table}

One of the most valuable features of rockchalk is that one can align
several models side by side and compare them. The rows are matched
by the variable name.

About model names: If a list of regression fits is not named, then
the names will be bland, ``M1'', ``M2''. 

It is highly recommended that authors should name the regression models. 

Currently, I recommend that we provide the model names \textbf{in}
the list that provides the fitted regressions, as you see here. That
produces Table \ref{tab:ex3}.

<<results=tex>>=
ex3 <- outreg(list("Model A" = m1, "Model B has a longer heading" = m2), varLabels = list(x1 = "Billie"),   title = "My Two Linear Regressions (uncentered)", label = "tab:ex3", request = c(fstatistic = "F"),  print.results = FALSE)
cat(ex3)
@

In the original rockchalk, I had it differently. It is possible to
provide model names in a separate argument, \code{modelLabels}. That
is shown in the code below, and in output Table \ref{tab:ex3b}. The
modelLabels parameter takes precedence, it will replace the labels
in the first argument. However, I found it confusing to write tables
that way, so I made it work the other way too.

<<results=tex>>=
ex3b <- outreg(list("Model A" = m1, "Model B" = m2),   modelLabels = c("Overrides ModelA", "Overrides ModelB"), varLabels = list(x1 = "Billie"), title = "Note modelLabels Overrides model names (uncentered)",  label = "tab:ex3b"
)
@

The column-centered versions are in Table \ref{tab:ex3bd} and Table
\ref{tab:ex3bs}.

<<results=tex>>=
ex3bd <- outreg(list("Model A" = m1, "Model B" = m2), modelLabels = c("Overrides ModelA", "Overrides ModelB"), varLabels = list(x1 = "Billie"), title = "Note modelLabels Overrides model names (dcolumn)", label = "tab:ex3bd", centering = "dcolumn")
@

<<results=tex>>=
ex3bs <- outreg(list("Model A" = m1, "Model B" = m2), modelLabels = c("Overrides ModelA", "Overrides ModelB"),  varLabels = list(x1 = "Billie"), title = "Note modelLabels Overrides model names (siunitx)", label = "tab:ex3bs", centering = "siunitx")
@

\subsection*{Specifying additional summary information}

In the first version, I was thinking that everybody would be happy
enough if the table included N, the standard error of the regression
(which I dubbed RMSE in the old SAS style), and R-square. There were
requests for other summaries. 

At first, I was thinking I needed to add arguments for each request.
I created argument ``\code{showAIC=TRUE}'' to request Akaike's
information criterion. For example, see Table \ref{tab:ex5d} produced
by the following

<<ex5d, results=tex>>=
ex5d <- outreg(list("Whichever" = m1, "Whatever" = m2), title = "Still have showAIC argument (uncentered)", label = "tab:ex5d", showAIC = TRUE, float = TRUE)
@

The column-centered versions are in Table \ref{tab:ex5dd} and \ref{tab:ex5ds}.

<<ex5dd, results=tex>>=
ex5dd <- outreg(list("Whichever" = m1, "Whatever" = m2),  title = "Still have showAIC argument (dcolumn)", label = "tab:ex5dd", showAIC = TRUE, float = TRUE, centering = "dcolumn")
@

<<ex5ds, results=tex>>=
ex5ds <- outreg(list("Whichever" = m1, "Whatever" = m2), title = "Still have showAIC argument (siunitx)", label = "tab:ex5ds", showAIC = TRUE, float = TRUE, centering = "siunitx")
@

However, I anticipated that way might lead me down a bad path of writing
a parameter for every possible summary statistic.

My first idea was to create a recipe book of particular summary items
and make them available for requests by users. For example, I wrote
a customized reporter for F statistics and a parameter to ask for
that was called ``request''. For example, \code{request = c(fstatistic = \textquotedbl F\textquotedbl )}
asks for my special fstatistic and the label to be used for it in
the table would be ``F''. I ended up not making very many of those
fancy object, but in the semTable in the kutils package there is a
similar approach for the model Chi-Square and now it seems to me I
should come back and do more like that in outreg.

Because I expected that I would never have time to keep up with requests
for specialized summary items, I created a ``back door'' though
which users could use functions and include them in the summary. 

However, I shot myself in the foot by creating that general purpose
ability with a horrible, ungainly name for the function's argument,
\code{runFuns}. I admit that is a horrible name, but I had good intentions.
``runFuns'' is short for ``run functions''. It will run a function
named as the first argument and then label it with the right hand
side argument. So, if one has a lot of request like AIC, BIC, and
so forth, the R functions can be used without too much effort. 

<<ex6d,results=tex>>=
ex6d <- outreg(list("Whatever" = m1, "Whatever" =m2), title = "Another way to get AIC output", label="ex6d",  runFuns = c("AIC" = "Akaike IC"), centering = "dcolumn", print.results=FALSE)
cat(ex6d)
@

\subsection*{Insert more regressions in one table}

This code produces Table \ref{tab:ex7}, which is NOT decimal aligned

<<results=tex>>=
ex7 <- outreg(list("Amod" = m1, "Bmod" = m2, "Gmod" = m3), title = "My Three Linear Regressions", label="tab:ex7")
@

The column-aligned version of the same is found in Table \ref{tab:ex7d},
produced by the following code.

<<results=tex>>=
ex7d <- outreg(list("Amod" = m1, "Bmod" = m2, "Gmod" = m3), centering = "dcolumn", title = "My Three Linear Regressions (decimal aligned)", label="tab:ex7d")
@

I worried that users who are verbose might break the function. In
Table \ref{tab:ex11}, I show that some very long names, even ones
with periods, do not seem to cause horrible trouble. They WILL run
off the edge of the page if they get much longer. Author will need
to change to the tight columns if that is an issue, as seen in Table
\ref{tab:ex11t}.

\begin{table}[H]
\caption{Stress test very long titles (uncentered) \label{tab:ex11}}

<<results=tex>>=
ex11 <- outreg(list("I Love Long Titles" = m1, "Prefer Brevity" = m2, "Captain. Kirk. Named. This." = m3), tight = FALSE, float = FALSE, centering = "dcolumn")
@
\end{table}

\begin{table}[H]
\caption{Stress test very long titles (dcolumn) \label{tab:ex11t}}

<<results=tex>>=
ex11td <- outreg(list("I Love Long Titles" = m1, "Prefer Brevity" = m2, "Captain. Kirk. Named. This" = m3), float = FALSE,  centering = "dcolumn")
@
\end{table}

\begin{table}[H]
\caption{Stress test very long titles (dcolumn) \label{tab:ex11ts}}

<<results=tex>>=
ex11ts <- outreg(list("I Love Long Titles" = m1, "Prefer Brevity" = m2, "Captain. Kirk. Named. This" = m3), float = FALSE, centering = "siunitx")
@
\end{table}


\subsection*{Alternative standard errors}

The original rockchalk took the standard errors from the fitted model.
A student in Brazil wrote and asked me to make it possible for the
author to supply ``robust'' standard errors. That was a good idea.
The code to demonstrate how to create an alternative vector of standard
errors (in that case, Huber-White robust standard errors) will appear
with the output in Table \ref{tab:ex8}. The code is displayed there
because I created the outreg table with float=FALSE, and then I manually
created a table container into which I typed the R code chunk.

\begin{table}[H]
\caption{Robust Standard Errors (uncentered)\label{tab:ex8}}

<<results=tex>>=
if (require(car)){
   newSE <- sqrt(diag(car::hccm(m3)))
   ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, 
             "Model C w Robust SE" = m3),
             SElist= list("Model C w Robust SE" = newSE))
}
@
\end{table}

\begin{table}[H]
\caption{Robust Standard Errors (dcolumn)\label{tab:ex8d}}

<<results=tex>>=
if (require(car)){
   newSE <- sqrt(diag(car::hccm(m3)))
   ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, 
             "Model C w Robust SE" = m3),
             SElist= list("Model C w Robust SE" = newSE),
             centering = "dcolumn")
}
@
\end{table}

\begin{table}[H]
\caption{Robust Standard Errors (siunitx)\label{tab:ex8s}}

<<results=tex>>=
if (require(car)){
   newSE <- sqrt(diag(car::hccm(m3)))
   ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, 
             "Model C w Robust SE" = m3),
             SElist= list("Model C w Robust SE" = newSE),
             centering = "siunitx")
}
@
\end{table}


\subsection*{Aligning different kinds of fits}

The output from lm and glm fits may sometimes be usefully compared.
The parameter display from rockchalk will align same-named variables.
The difference in the available summary statistics is apparent because
the rows do not ``line up''. See Table \ref{tab:ex13} for the output
from the following.

<<results=tex>>=
ex13 <- outreg(list("OLS" = m1, "GLM" = gm1), float = TRUE,
               title = "OLS and Logit in same table (dcolumn)", 
               label="tab:ex13", alpha = c(0.05, 0.01), centering = "dcolumn")
@

As seen in Table \ref{tab:ex14}, we check if the supplemental parameter
requests hold up with dcolumn

<<results=tex>>=
ex14 <- outreg(list(OLS = m1, GLM = gm1), float = TRUE,
         title = "OLS and Logit with summary report features (dcolumn)",
         label = "tab:ex14",
         request = c(fstatistic = "F"), runFuns = c("BIC" = "BIC"),
         centering = "dcolumn")
@

What if the number of digits is dialed up and alpha is altered? See
Table \ref{tab:ex15}. 

<<results=tex>>=
ex15 <- outreg(list(OLS = m1, GLM = gm1), float = TRUE,
         title="OLS and GLM with more digits (digits)", 
         label="tab:ex15", 
         request = c(fstatistic = "F"), runFuns = c("BIC" = "BIC"),
         digits = 5, alpha = c(0.01), centering = "dcolumn")
@

In Table \ref{tab:ex16d}, output shows result when several runFuns
are requested. Again, I'm very sorry that parameter name is so awful.
I adjusted the alpha stars as well.

<<results=tex>>=
ex16d <- outreg(list("OLS 1" = m1, "OLS 2" = m2,  GLM = gm1), float = TRUE,
           title = "2 OLS and 1 Logit (dcolumn), additional runFuns", 
           label="tab:ex16d",
           request = c(fstatistic = "F"),
           runFuns = c("BIC" = "BIC", "logLik" = "ll"),
           digits = 5, alpha = c(0.1, 0.05, 0.01), centering = "dcolumn")
@

<<results=tex>>=
ex16s <- outreg(list("OLS 1" = m1, "OLS 2" = m2,  GLM = gm1), float = TRUE,
           title = "2 OLS and 1 Logit (siunitx), additional runFuns", 
           label="tab:ex16s",
           request = c(fstatistic = "F"),
           runFuns = c("BIC" = "BIC", "logLik" = "ll"),
           digits = 5, alpha = c(0.1, 0.05, 0.01), centering = "siunitx")
@

After a while, I noticed that the left hand side of runFuns need not
be in quotation marks. So my examples are not always consistent. For
example, in Table \ref{tab:ex17} I have some in quotes, some not.
And I also show that if authors want to create redundant rows, they
are allowed to do so (N, for example).

\begin{table}[H]
\caption{Additional test on summary stats (dcolumn)\label{tab:ex17}}

<<results=tex>>=
ex17 <- outreg(list("Model A" = gm1, "Model B label with Spaces" = m2),
         request = c(fstatistic = "F"),
         runFuns = c("BIC" = "Schwarz IC", "AIC" = "Akaike IC", "logLik" = "ll",
         "nobs" = "N Again?"), centering = "dcolumn")
@
\end{table}

\begin{table}[H]
\caption{Additional test on summary stats (siunitx)\label{tab:ex17s}}

<<results=tex>>=
ex17s <- outreg(list("Model A" = gm1, "Model B label with Spaces" = m2),
         request = c(fstatistic = "F"),
         runFuns = c("BIC" = "Schwarz IC", "AIC" = "Akaike IC", "logLik" = "ll",
         "nobs" = "N Again?"), centering = "siunitx")
@
\end{table}


\section{About examples here and your long-run work flow}

In this document, I'm using a lot of short cuts to make it easier
to produce examples with and without centering. I'm using the \code{float=TRUE}
option (or, equivalently, giving a title) often here to cut down on
the amount of work I'm doing. 

If you are learning how to make tables, I usually suggest doing it
in a different way. I would suggest instead that authors should create
an outreg output file, with a sequence like this

\bgroup\inputencoding{latin9}
\begin{lstlisting}
lm1 <- lm(... regression commands ...)
lm1.out <- outreg(lm1, print.results=FALSE, ...parameters like float=FALSE, tight = FALSE ...)
cat(lm1.out, file = "lm1.out.tex")
\end{lstlisting}
\leavevmode\egroup

I'd usually have an output directory where the tex file would be placed,
but aside from this detail, that's my workflow. Then, when I want
to use that file in my document, I use the LaTeX code ``\textbackslash input\{lm1.out.tex\}''. 

One reason for using that 2-step workflow is that the \emph{automatically
produced outreg table may not be exactly perfect}. Perhaps a variable
label does not look right. Personally speaking, I wish the automatic
table were always perfect. Practically, I accept it is not. So I leave
open the opportunity that it might need to be revised by hand. 

\pagebreak{}

\bibliographystyle{apalike2}
\phantomsection\addcontentsline{toc}{section}{\refname}\bibliography{rockchalk}

\pagebreak{}

\section*{Replication Information\label{sec:Session-Info}}

<<session, echo=F>>=
sessionInfo()
if(!is.null(warnings())){
    print("Warnings:")
    warnings()}
@

<<opts20, include=F>>=
## Don't delete this. It puts the interactive session options
## back the way they were. If this is compiled within a session
## it is vital to do this.
options(opts.orig)
@
\end{document}

%% LyX 2.4.4 created this file.  For more info, see https://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[american,noae]{scrartcl}
\usepackage{lmodern}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage{float}
\usepackage{url}
\usepackage{amsmath}
\usepackage{geometry}
\geometry{verbose,tmargin=1in,bmargin=1in,lmargin=1in,rmargin=1in}
\usepackage[authoryear]{natbib}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
<<echo=F>>=
  if(exists(".orig.enc")) options(encoding = .orig.enc)
@
\providecommand*{\code}[1]{\texttt{#1}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
%\VignetteIndexEntry{Using rockchalk}

\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{figure}
 }{%
   \@float{figure}[#1]%
 }%
 \centering
}{%
 \end@float
}

\renewenvironment{table}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{table}
 }{%
   \@float{table}[#1]%
 }%
 \centering
%  \setlength{\@tempdima}{\abovecaptionskip}%
%  \setlength{\abovecaptionskip}{\belowcaptionskip}%
% \setlength{\belowcaptionskip}{\@tempdima}%
}{%
 \end@float
}


%\usepackage{listings}
% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, %,style=Rstyle}
                        fancyvrb=false,escapechar=`,language=R,%
                        %%basicstyle={\Rcolor\Sweavesize},%
                        backgroundcolor=\Rbackground,%
                        showstringspaces=false,%
                        keywordstyle=\Rcolor,%
                        commentstyle={\Rcommentcolor\ttfamily\itshape},%
                        literate={<<-}{{$\twoheadleftarrow$}}2{~}{{$\sim$}}1{<=}{{$\leq$}}2{>=}{{$\geq$}}2{^}{{$^{\scriptstyle\wedge}$}}1{==}{{=\,=}}1,%
                        alsoother={$},%
                        alsoletter={.<-},%
                        otherkeywords={!,!=,~,$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},%
                        escapeinside={(*}{*)}}%


% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\scriptsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.90}}

\usepackage{dcolumn}
\usepackage{siunitx}

\makeatother

\usepackage{babel}
\usepackage{listings}
\renewcommand{\lstlistingname}{\protect\inputencoding{latin9}Listing}

\begin{document}
\title{Using rockchalk for Regression Analysis\thanks{In case you want to be involved in development, the source code is
available on GitHub. Please browse http://github.com/pauljohn32. The
read-only git archive address is \protect\url{git://github.com/pauljohn32/rockchalk.git}.
Once you learn how to ``clone'' the repository with a git client,
then we can discuss ways for you to contribute.}}
\author{Paul E. Johnson}
\maketitle
\begin{abstract}
The rockchalk package includes functions for estimating regressions,
extracting diagnostic information from them, preparing presentable
tables, and creating helpful two and three dimensional plots. It is
primarily intended to facilitate teachers and students who are conducting
ordinary least squares and elementary generalized models. Compared
to other packages that help with regression analysis, this one offers
more-than-the-usual amount of emphasis on the analysis of regression
models that include interaction terms. It includes functions that
can receive a fitted model and then return standardized, mean-centered,
and residual-centered regression results. The plotting functions address
some limitations of R's \code{termplot} function. Version 1.8 introduces
a consistent framework for the creation of ``\code{newdata}'' objects
and the calculation of predicted values so that the marginal effects
of separate predictors can be easily explored.
\end{abstract}
% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\SweaveOpts{ae=F,nogin=T}

<<Roptions, echo=F>>=
options(device = pdf)
options(width=80, prompt=" ", continue="  ")
options(useFancyQuotes = FALSE) 
@

\section{Introduction}

This is the Spring, 2019 update of the rockchalk package. I offer
a course in regression analysis for social and behavioral scientists
every year. As the course goes on, I keep track of the difficulties
that the students experience with R and I craft functions that facilitate
their work on particular assignments. The new features in this edition
are the functions \code{descriptiveTables()} and \code{waldt()}
(for regression reports), tools for regression simulation, including
\code{genCorrelatedData3()} and \code{cutFancy()}, some support
for common recoding support for multi-level modeling. Finally, the
regression table-writing function the \code{outreg()} can now generate
decimal-aligned columns. A new vignette demonstrating the \code{outreg()}
function is introduced with the package.

As in previous versions, a core of the effort is development of an
internally coherent framework that allows students to estimate regressions
(usually with lm and glm) and then create useful summary tables and
diagnostic reports. It is easy to create ``newdata'' objects and
interact with fitted regressions. The \code{newdata()} function will
be handy for just about any kind of regression. For the common kinds
of regression in base R and lme4, there is also a convenience function
called \code{predictOMatic()} that allows one to specify a regression
model and then receive predicted values for a range of input values.
The default output will be similar to other regression software tools
that provide ``marginal effect'' estimates for regression analysis. 

I have exerted a great deal of effort to improve the consistency of
predicted value plots for regression analysis. The \code{plotSlopes()}
function is a plotter for linear models with interaction terms. It
draws a plot on the screen, but it also creates an output object that
can be subjected to a follow-up hypothesis test (by the \code{testSlopes()}
function). There is a plot method for testSlopes objects to help with
analysis of interactions. 

For people who want to plot nonlinear regressions, the function \code{plotCurves()}
is a replacement for \code{plotSlopes()}. It can handle any sort
of nonlinearity and interaction on the right hand side. Where some
regression formula can cause R's \code{termplot()} function to fail,
\code{plotCurves()} will succeed.

The core functionality of many functions has not been altered. Functions
to provide standardized regression coefficients, mean-centered, or
residual-centered regression coefficients have not changed much, although
the plots of them are improved. 

\section{Facilitating Collection of Summary Information}

\subsection{summarize: A replacement for summary}

When an R function provides output that is not suitable for a term
paper, the student must cobble together some code and assemble a customized
table. In my opinion, R's summary() function for data frames is not
adequate. It lacks summaries for diversity of observations. In addition,
the output is not formatted in a way that is conducive to the creation
of plots or tables. As a result, I offer the function summarize(),
which separates the numeric from factor variables and provides an
alphabetized summary. 

<<echo=F>>=
library(rockchalk)
@

Consider the results of applying summarize() to Fox's Chile data set
from the carData package:

<<echo=T>>=
library(carData)
data(Chile)
(summChile <- summarize(Chile))
@

The result object is a list that includes a data frame for the numeric
variables, with named rows and (optionally) alphabetized columns,
as well as a separate report for each factor variable. Users who wish
to summarize only the numeric variables can run summarizeNumerics()
instead, while others who want to summarize only factors can run summarizeFactors().
The output from summarizeFactors() is a list of factor summaries. 

A companion function is centralValues(), which will provide only one
number for each variable in a data frame. For numerics, it returns
the mean, while for factor variables, it returns the mode. 

<<echo=T>>=
centralValues(Chile)
@

\subsection{Easier predictions and newdata objects}

Students struggle with R's predict() methods. The primary challenge
is in the creation of a newdata object of interesting values of the
predictors. If the newdata argument, here myNewDF in \bgroup\inputencoding{latin9}\lstinline!predict(m1, newdata = myNewDF)!\egroup
 is not exactly right, the command will fail. If the regression formula
uses functions like ``as.numeric()'' or ``as.factor()'', its almost
impossible for first-time R users to get this right. In version 1.8,
I believe I have solved the problem entirely with the functions newdata()
and predictOMatic(). 

Let's fit an example regression with the Chile data set from the carData
package.

<<echo=T>>=
m1 <- lm(statusquo ~ age + income + population + region + sex, data = Chile)
@

\noindent The default output of predictOMatic() will cycle through
the predictors, one by one.

<<echo=T>>=
m1pred <- predictOMatic(m1)
m1pred
@

The newdata() and predictOMatic() functions handle the details and
allow the user to adjust their requests to allow for very fine grained
control. Here are the key arguments for these functions.
\begin{enumerate}
\item divider. The name of an algorithm to select among observed values
for which predictions are to be calculated. These are the possibilities.

\begin{description}
\item [{``seq''}] an evenly spaced sequence of values from low to high
across the variable.
\item [{``quantile''}] quantile values that eminate from the center of
the variable ``outward''. 
\item [{``std.dev.''}] the mean plus or minus the standard deviation,
or 2 standard deviations, and so forth.
\item [{``table''}] when a variable has only a small set of possible
values, this selects the most frequently observed values.
\end{description}
\item n. The number of values to be selected.
\item predVals. Where the divider argument sets the default algorithm to
be used, the predVals argument can choose variables for focus and
select divider algorithms separately for the variables. It is also
allowed to declare particular values.
\end{enumerate}
The user can request that particular values of the predictors are
used, or can declare one of several algorithms for selection of focal
values. All of these details are managed by the argument called predVals,
which is described in the help pages (with plenty of examples). 

<<echo=T>>=
mypred2 <- predictOMatic(m1, predVals = c("age", "region"), n = 3)
mypred2
mypred3 <- predictOMatic(m1, predVals = c(age = "std.dev.", region = "table"), n = 3)
mypred3
mypred4 <- predictOMatic(m1, predVals = list(age = c(18, 30, 45), region = c("SA", "C","N")), n = 3)
mypred4
@

I've invested quite a bit of effort to make sure this works dependably
with complicated regression formulae. All formulae, such as y \textasciitilde{}
x1 + log(x2 + alpha) + ploy(x3, d) will just work. (In case one is
interested to know \emph{why} this works, the secret recipe is a new
function called model.data(). Under the hood, this required some hard
work, a frustrating chain of trial and error that is discussed in
the vignette \emph{Rchaeology}, which is distributed with this package).

The \code{predictOMatic()} function will work when the regression
model provides a predict function that can handle the newdata argument.
If the regression package does not provide such a function, the user
should step back and use the newdata function to create the examplar
predictor data frames and then calculate predicted values manually.
The newdata() function will set all variables to center values and
then it will create a ``mix and match'' combination of the ones
that the user asks for. This sequence shows ways to ask for various
``mix and match'' combinations of values from age and region. 

<<echo=T>>=
mynewdf <- newdata(m1, predVals = c("age","region"), n = 3)
mynewdf
@

<<echo=T>>=
mynewdf2 <- newdata(m1, predVals = list(age = "std.dev.", region = c("SA", "C","N")))
mynewdf2
@

<<echo=T>>=
mynewdf3 <- newdata(m1, predVals = list(age = c(20, 30, 40), region = c("SA", "C","N")))
mynewdf3
@\\
Of course, functions from rockchalk or any other R package can be
placed into the process for choosing focal values. 

<<echo=T>>=
mynewdf <- newdata(m1, predVals = list(age = getFocal(Chile$age, n = 3), region = getFocal(Chile$region, n = 3)))
mynewdf
@\\
The function getFocal() is a generic function; it will receive variables
of different types and ``do the right thing.'' By default, focal
values of numeric variables are quantile values. For factor values,
the most frequently observed values are selected. These are customizable,
as explained in the documentation. The \code{newdata()} output can
be used in a predict() function call as demonstrated above. 

It would be nice if every regression model's predicted values were
accompanied by 95\% confidence intervals. Models fit by \code{lm()}
can supply confidence intervals, but not \code{glm()}. At the current
time, there are many competing methods that might be used to calculate
those intervals; predict.glm() in R's stats package avoids the issue
entirely by not calculating intervals. In rockchalk-1.8, I crafted
some code to calculate confidence intervals for glm objects using
the (admittedly crude) Wald-based approximation. In the scale of the
linear predictor, we calculate a 95\% confidence interval, and then
use the inverse link function to transform that onto the space of
the observed response. In this example, I replicate an example that
is an R classic, from the help page of predict.glm. The reader will
note that the output includes a warning about the construction of
the confidence interval.

<<echo=T>>=
df <- data.frame(ldose = rep(0:5, 2), sex = factor(rep(c("M", "F"), c(6, 6))), 
	SF.numdead = c(1, 4, 9, 13, 18, 20, 0, 2, 6, 10, 12, 16))      
df$SF.numalive <-  20 - df$SF.numdead
budworm.lg <- glm(cbind(SF.numdead, SF.numalive) ~ sex*ldose, data = df,  family = binomial)
predictOMatic(budworm.lg, predVals = c(ldose = "std.dev.", sex = "table"), interval = "confidence")  
@

\subsection{Descriptive Tables}

In regression-based reports, it is common for authors to present a
brief summary table to represent the data. The \code{descriptiveTable()}
function was created to make that process easier and more predictable.
The aim is to provide the usual summary statistics for the numeric
predictors and a table of proportions for the categorical predictors.
See Table \ref{tab:descriptiveTable}. The user can customize the
requested statistical summaries for numeric predictors.

\begin{table}[H]
\caption{The descriptiveTable function\label{tab:descriptiveTable}}

<<>>=
descriptiveTable(m1, digits=6)
@
\end{table}

\noindent I my opinion, the output is, well, simple and understandable.
More importantly, it will be sufficient for a term paper, once it
is converted into a presentable final format. It can be converted
into HTML or LaTeX with various R packages, such as xtable. Note that
because this descriptive table is derived from the fitted regression,
it summarizes \code{only the cases} that were actually used in the
regression analysis. If cases were lost by listwise deletion, the
deleted cases are not taken into account here.

\section{Better Regression Tables: Some outreg Examples.}

On May 8, 2006, Dave Armstrong, who was a political science PhD student
at University of Maryland, posted a code snippet in r-help that demonstrated
one way to use the ``cat'' function from R to write \LaTeX{} markup.
That gave me the idea to write a \LaTeX{} output scheme that would
help create some nice looking term and research papers. I wanted ``just
enough'' information, but not too much.

Since 2006, many new R packages have been introduced for the creation
of regression tables, but I still prefer to maintain \code{outreg}.
I fight to keep this simple, but have added some features in response
to user requests. For me, the biggest question is whether the format
should be ``wide'' or ``tight''. A tight format stacks estimates
and standard errors into a single column, while a wide format has
them in separate columns. In response to user requests, stars for
statistical significance were made customizable. It is now allowed
to directly insert vectors of parameter estimates and standard errors.
In this version, a new parameter \code{centering} is introduced for
decimal-centered columns. 

In the following, I will demonstrate some tables for lm and glm fits
on a simulated data set. This new simulation function, genCorrelatedData3(),
is a convenient way to create multiple-predictor regression data sets
of arbitrary size and complexity, allowing for interactions and nonlinearity. 

<<createdata1, echo=T>>=
set.seed(1234)
dat <- genCorrelatedData3(N = 100, means = c(x1 = 0, x2 = 10, x3 = 0), sds = c(1, 2, 1), rho = c(0, 0, 0), stde = 10, beta = c(0, -3, 4, 0), verbose = FALSE)
@\\
That creates a new matrix with variables $x1,$ $x2$, $x3$, and
y. We run some linear regressions and then create a categorical output
variable for a logistic regression.

<<createdata1, echo=T>>=
m1 <- lm(y ~ x1 + x2, data = dat)
m2 <- lm(y ~ x2, data = dat)
m3 <- lm(y ~ x1 + x2 + x3, data = dat)
## Create categorical variant
myilogit <- function(x) exp(x)/(1 + exp(x))
dat$y3 <- rbinom(100, size = 1, p = myilogit(scale(dat$y)))
gm1 <- glm(y3 ~ x1 + x2, data = dat)
@

The outreg examples are offered in Tables \ref{tab:Tab1} through
\ref{tab:Combined-OLSGLM}. Table \ref{tab:Tab1} is the ``tight
format output for three models, obtained from \code{outreg(list(m1, m2, m3), centering=\textquotedbl siunitx\textquotedbl )}.
On the other hand, as illustrated in Table \ref{tab:Tab2}, the author
can request a wide table (tight = FALSE) and provide more elaborate
model labels and adjust the significance stars. In Table \ref{tab:Combined-OLSGLM},
observe that the linear and generalized linear model output peacefully
co-exist, side-by-side. This output is, in my opinion, completely
acceptable for inclusion in a professional presentation or conference
paper. There are some warts in this output. 

The default output from \code{outreg} will have left-aligned columns.
Because users requested decimal-centered columns, the argument centering
has been introduced. If \code{centering=\textquotedbl siunitx\textquotedbl}
or \code{centering=TRUE}, the \code{outreg} table will be adjusted
to use features in the LaTeX package ``\code{siunitx}''. The LaTeX
package dcolumn is probably more familiar to users, and \code{\textquotedbl dcolumn\textquotedbl}
is also a legal value for centering, but the results are not quite
as good. As a \textbf{caution}, I hasten to mention that if a user
asks for decimal-centered columns, the user has the duty to insert
into the document preamble either ``\code{\textbackslash usepackage\{siunitx\}}''
or ``\code{\textbackslash usepackage\{dcolumn\}}''.

\begin{table}
\caption{Decimal-centered columns in the ``tight'' style\label{tab:Tab1}}

<<outreg10, echo = F, results=tex>>="
or <- outreg(list(m1, m2, m3), centering = "siunitx")
@
\end{table}

\begin{table}
\caption{The wide format obtained with tight = FALSE\label{tab:Tab2}}

<<outreg20, results=tex, echo=F>>=
or1 <- outreg(list("The First Model with a Long Title" = m1, "Another Model" = m3), tight = FALSE, alpha = c(0.05, 0.01, 0.001), centering = "siunitx")
@
\end{table}

\begin{table}
\caption{Combined OLS and GLM Estimates\label{tab:Combined-OLSGLM}}

<<outreg70, results=tex, echo=F>>=
or2 <- outreg(list(m1,gm1), modelLabels = c("OLS:y","GLM: Categorized y"))
@
\end{table}

I understand that some authors need to include regression tables in
documents that are not prepared with LaTeX. The parameter type = ``HTML''
will change the output format to HTML. An HTML file can be displayed
in a web browser and it can be imported in traditional ``word processor''
programs.

\section{Plotting Regressions with Interactions }

\subsection{Interaction in Linear Regression. }

One of the most fundamental skills in regression analysis is the interpretation
of interactive predictors. It is much easier for students to understand
the effect of an interaction if they can create a nice plot to show
how the predicted value depends on various values of a predictor.
The \code{plotSlopes()} function was introduced in 2010 when I was
teaching a large first-year graduate course (more than 50 students)
and it became apparent that about 20 percent of them would not be
able to manage the R coding required to draw several lines on a single
plot. Unfortunately, R's termplot() function will not draw regressions
involving interactions.

The rockchalk package has two functions to help with this, \code{plotSlopes()}
and \code{plotCurves()}. \code{plotCurves()} is more general, it
can handle any kind of formula that the user estimates. \code{plotSlopes()}
is more limited, it is only for lm objects. In return for that limitation,
\code{plotSlopes()} creates an output object which can be used to
conduct post-hoc hypothesis tests. 

\begin{figure}
<<ps05, fig=T, echo=F, include=F, height=5, width=6>>=
m1ps <- plotSlopes(m1, plotx = "x2", xlab = "x2 from model m1", interval = "confidence", opacity = 80, col = "red",
ylim = c(-70, 15), legendArgs = list(x="topright"))
@

\includegraphics[height=4in]{rockchalk-ps05}

\caption{plotSlopes: Linear Model with Confidence Interval\label{fig:ps05}}
\end{figure}

At its most elementary level, \code{plotSlopes()} is a ``one step''
regression line plotter. If the regression model includes more than
one predictor, then a single predictor is displayed on the horizontal
axis and the other predictors are set on their central values. A plot
for the model m1, that was illustrated above, is presented in Figure
\ref{fig:ps05}. In rockchalk-1.8, new arguments were added to allow
the ``see though'' confidence region. The command to generate Figure
\ref{fig:ps05} was

<<echo=T, eval=F>>=
<<ps05>>
@

\noindent I've adjusted the color and opacity to illustrate the usage
of those arguments. The y range is adjusted to make a little extra
room for the legend. The \code{plotSlopes()} function is very flexible.
All of the label, color, and scale arguments of a plot function are
also available. The plotSlopes function also works well if the moderator
is a categorical variable. 

It is important to note that the output object, m1ps, has the information
necessary to re-create the plotted line in the form of a \code{newdata}
data frame. The first few lines in the \code{newdata} object are

<<>>=
m1ps$newdata[1:3, ]
@

<<ps09, fig=F, echo=F, height=9, width=6>>=
dat$y4 <- 1 + 0.1 * dat$x1 - 6.9 * dat$x2 + 0.5 * dat$x1*dat$x2 + 0.2 * dat$x3 + rnorm(100, m = 0, sd = 10)
m4 <- lm(y4 ~ x1*x2 + x3, data = dat)
@

\begin{figure}
<<ps10, fig=T, echo=F, height=9, width=6>>=
par(mfcol=c(2,1))
m4psa <- plotSlopes(m4, plotx = "x1", modx = "x2", xlab = "x1 is a fun plotx")
m4psb <- plotSlopes(m4, plotx = "x2", modx = "x1", modxVals = "std.dev.", xlab = "x2 is plotx", ylim = c(-120, 15),
legendArgs = list(x = "topright"))
par(mfcol=c(1,1))
@

\caption{plotSlopes Illustrated\label{fig:ps10}}
\end{figure}

This is more interesting if we fit a regression with an interaction
term, such as 

\bgroup\inputencoding{latin9}
\begin{lstlisting}
m4 <- lm(y4 ~ x1*x2 + x3, data = dat)
\end{lstlisting}
\leavevmode\egroup

\noindent We then ask \code{plotSlopes} to draw the predicted values
using one numeric variable as the horizontal axis and values of another
variable (a moderator) are set at particular values. Either x1 or
x2 can be viewed as the ``moderator'' predictor, the one on which
the effect of the other depends. In rockchalk version 1.8, the selection
of values of the moderator was generalized, so that the user can specify
either a function that selects values, or a vector of values, or the
name of an algorithm. The default algorithm will choose quantile values,
but Figure \ref{fig:ps10} demonstrates also the ``std.dev.'' divider
algorithm. The code to produce that figure was

<<eval=F, echo=T>>=
<<ps10>>
@

When \code{modx} is a numeric variable, then some particular values
must be selected for calculation of predicted value lines. The \code{modxVals}
argument is used to either specify moderator values or an algorithm
to select focal values. By default, three hypothetical values of \code{plotx}
are selected (the quantiles 25\%, 50\%, and 75\%). 

If \code{modx} is a factor variable, then the most frequently observed
scores will be selected for consideration. The default display will
include the regression line as well as color-coded points for the
subgroups represented by values of the moderator.

\begin{figure}
<<echo=F>>=
fourCat <- gl(4,25, labels=c("East","West","South", "Midwest"))
dat$x4 <- sample(fourCat, 100, replace = TRUE)
dat$y5 <- 1 + 0.1 * dat$x1 + contrasts(dat$x4)[dat$x4, ] %*% c(-1,1,2) + rnorm(100,0, sd=10)
m5 <- lm (y5 ~ x1*x4 + x3, data=dat)
@

<<ps20, fig=T, echo=F>>=
m5psa <- plotSlopes(m5, plotx = "x1", modx = "x4", xlab = "x1 is a Continuous Predictor", xlim = magRange(dat$x1, c(1.2,1)))
@

\caption{plotSlopes with a Categorical Moderator\label{fig:ps20}}
\end{figure}

\noindent{}
\begin{figure}
<<ps21, fig=T, echo=F>>=
m5psb <- plotSlopes(m5, plotx = "x1", modx = "x4", modxVals = c("West","East"), xlab = "x1 is a Continuous Predictor", xlim=magRange(dat$x1, c(1.2,1)), interval = "conf")
@

\caption{plotSlopes: the interval argument\label{fig:ps21}}
\end{figure}
Suppose we have a four-valued categorical variable, ``West'',''Midwest'',
``South'', and ``East''. If that variable is used in an interaction
in the regression model, then the \code{plotSlopes} output will include
four lines, one for each region. For example, consider Figure \ref{fig:ps20},
which is created by

<<eval=F>>=
<<ps20>>
@

\noindent The categorical variable is x4. 

It is possible to superimpose confidence intervals for many subgroups,
but sometimes these plots start to look a little bit ``busy''. The
mixing of shades in overlapping intervals may help with that problem.
A plot that focuses on just two subgroups is presented in Figure \ref{fig:ps21},
which is produced by 

<<eval=F>>=
<<ps21>>
@ 

In rockchalk version 1.8, I've exerted quite a bit of effort to make
sure that colors are chosen consistently when users remove or insert
groups in these plots. The same value of the moderator should always
be plotted in the same way–the line, points, and interval colors should
not change. Note, for example, in Figures \ref{fig:ps20} and \ref{fig:ps21},
the line for East is black in both plots, while the line for West
is red in both. 

\subsection{testSlopes, a companion of plotSlopes}

The students in psychology and political science are usually interested
in conducting some diagnostic analysis of the interactive terms. \citet{Aiken1991}
(and later \citealp{Cohen2002}Cohen, Cohen, West, and Aiken) propose
using the t test to find out if the effect of the ``plotx'' variable
is statistically significantly different from zero for each particular
value of the moderator variable. The new version of rockchalk declares
a method plot.testSlopes that handles the work of plotting the interaction.

The usual case would be the following. First, carry out the regression
analysis. Then run plotSlopes, and run testSlopes, and then pass that
result object to the plot method.

\begin{figure}
<<ts10, fig=T, echo=F, height=6, width=6>>=
m4psats <- testSlopes(m4psa)
plot(m4psats)
@

\caption{testSlopes for an Interactive Model\label{fig:ts10}}
\end{figure}

\bgroup\inputencoding{latin9}
\begin{lstlisting}
m4 <- lm(y ~ x1*x2 + x3, data = dat)
m4ps <- plotSlopes(m4, plotx = "x1", modx ="x2", xlab = "x1 is a Continuous Predictor")
m4psats <- testSlopes(m4ps)
plot(m4psats)
\end{lstlisting}
\leavevmode\egroup

\noindent The output from testSlopes will differ, depending on whether
modx is numeric or a factor. If it is a factor, then the slope of
the lines and the associated t-test for each will be presented. My
psychology students call these ``simple-slopes'', following the
terminology of Aiken and West. The general idea is that we want to
know if the ``combined'' effect of plotx is not zero. For a model
stated with predictors $plotx_{i}$ and $modx_{i}$as
\begin{equation}
y_{i}=b_{0}+b_{plotx}plotx_{i}+b_{modx}modx_{i}+b_{plotx:modx}plotx_{i}\cdot modx_{i}+\ldots+e_{i}
\end{equation}

\noindent the null hypothesis would be

\noindent
\begin{equation}
H_{0}:0=\hat{b}_{simple\,slope}=\hat{b}_{plotx}+\hat{b}_{plotx:modx}modx
\end{equation}

\noindent If modx is a factor, then we simply calculate the slope
of each line and the test is straight-forward. If modx is a numeric
variable, then we confront a problem that is a bit more interesting.
We don't really want to say that the simple slope is different from
0 for particular values of $modx$, but instead we want to answer
the question, ``for which values of the moderator would the effect
of the plotx variable be statistically significant?''. This necessitates
the calculation of the so-called Johnson-Neyman interval (\citeyear{JohnsonNeyman1936}),
a plot of which is presented in Figure \ref{fig:ts10}.

The method of calculation is outlined in \citet{PreacherCurren2006}.
The values of $modx$ associated with a statistically significant
effect of $plotx$ on the outcome is determined from the computation
of a T statistic for $\hat{b}_{simple\,slope}$. The J-N interval
is the set of values of $modx$ for which the following (quadratic
equation) holds:
\begin{align}
\hat{t} & =\frac{\hat{b}_{simple\,slope}}{std.err(\hat{b}_{simple\,slope})}\nonumber \\
 & =\frac{\hat{b}_{simple\,slope}}{\sqrt{\widehat{Var(\hat{b}_{plotx})}+modx^{2}\widehat{Var(\hat{b}_{plotx\cdot modx})}+2modx\widehat{Cov(\hat{b}_{plotx},\hat{b}_{plotx\cdot modx})}}}\geq T_{\frac{\alpha}{2},df}
\end{align}

\noindent Suppose there are two real roots, $root1$ and $root2$.
The values of $modx$ for which the slope is statistically significant
may be a compact interval, $[root1,root2]$, as demonstrated in Figure
\ref{fig:ts10}, or it may two open intervals, $(-\infty,root1]$
and $[root2,\infty)$. I had expected almost all applications to result
in that latter case, but the somewhat surprising case illustrated
in Figure \ref{fig:ts10} is not too infrequently observed. 

\subsection{plotCurves for nonlinear predictor formulae}

In the most recent revision of this package, I believe I have made
the plotCurves() function redundant. Before I delete the function
entirely, I'm waiting for feedback. There was a problem in the past.
Some students used plotSlopes() (which drew straight lines) when they
intended to draw curves (and should have used plotcCurves).

plotCurves() was developed to generalize the plotting capability for
regression formulas that include nonlinear transformations. It was
for models that have polynomials or terms that are logged (or otherwise
transformed). In that sense, plotCurves() is rather similar to R's
own termplot() function, but plotCurves() has two major advantages.
First, it allows interactions, and second, it handles some complicated
formulae that termplot() is not able to manage. 

Suppose a dependent variable y5 is created according to a nonlinear
process.
\begin{equation}
y5_{i}=-3x1_{i}+7*log(x2)+1.1x2_{i}+2.2x1_{i}\times x2_{i}+e_{i}
\end{equation}

<<echo=F,include=F>>=
dat$y5 <- with(dat, -3*x1 + 15*log(0.1 + x2 - min(x2)) + 1.1*x2 + 8.2 *x1 * x2 + 10*rnorm(100))
@

The estimated model is

<<>>=
m5 <- lm(y5 ~ log(x2) + x1 * x2, data = dat)
@

In the new version, the function calls for plotSlopes() and plotCurves()
are the same:

<<pcps20a, fig=T, eval=F, height=4, width=6>>=
m5pc <- plotCurves(m5, plotx = "x2", modx = "x1", main = "plotCurves output", ylim = c(-210, 330),
legendArgs = list(x = "topleft", title = "x1", cex = 0.8))
@

<<psps20b, fig=T, eval=T, eval=F, echo = T, height = 4, width = 6>>=
m5ps <- plotSlopes(m5, plotx = "x2", modx = "x1", main = "plotSlopes outout", ylim = c(-210, 330),
legendArgs = list(x = "topleft", title = "x1", cex = 0.8))
@

So far as I have been able to tell, the results are the same. See
Figure \ref{fig:pcps20}.

\begin{figure}

<<eval=T, fig=T,echo=F,height = 4, width = 6>>=
<<pcps20a>>
@

<<eval=T, fig=T, echo=F,height = 4, width = 6>>=
<<psps20b>>
@

\caption{Illustrations of m5 with plotSlopes and plotCurves\label{fig:pcps20}}

\end{figure}


\subsection{plotPlane}

The persp() function in R works well, but its interface is too complicated
for most elementary and intermediate R users. To facilitate its use
for regression users, the plotPlane() function is offered.

The plotPlane function offers a visualization of the mutual effect
of two predictors, whether or not the regression model is linear.
plotPlane() is designed to work like plotCurves(), to tolerate nonlinear
components in the regression formula. plotPlane() allows the depiction
of a 3 dimensional curving plane that ``sits'' in the cloud of data
points. The variables that are not explicitly pictured in the plotPlane()
figure are set to central reference values. Recall model m4, which
used the formula \bgroup\inputencoding{latin9}\lstinline!y4 ~ x1*x2 + x3!\egroup
. As illustrated in Figure \ref{fig:pp100}, plotCurves() presents
a reasonable view of the predicted values.

\begin{figure}
<<pp100, fig=T, echo=F>>=
p100 <- plotPlane(m4, plotx1 = "x1", plotx2 = "x2", phi = 10, theta = -80, lcol = gray(.70))
@

\caption{plotPlane for the Interactive Model m4\label{fig:pp100}}
\end{figure}

Because plotPlane() is a simple convenience wrapper for R's persp()
function, it responds to the same customizing arguments that perp
would allow. The arguments phi and theta will rotate the figure, for
example. The output in Figure \ref{fig:pp100} is produced by the
following.

<<echo=T,eval=F>>=
<<pp100>>
@

\noindent One of the major educational benefits of the 3-D figure
is that students can easily see that a model with a simple interaction
effect is \emph{not a linear model any more. }We will return to that
point in the discussion of mean centering in regression analysis.

Recall that model m5 is the rather complicated nonlinear formula \bgroup\inputencoding{latin9}\lstinline!log(x2*x2) + x1 * x2!\egroup
. The plotCurves() output for that was already presented in Figure
\ref{fig:pcps20}. The three dimensional view of the same is presented
in Figure \ref{fig:pp110}, but with an added twist. The twist is
that the predicted value lines from the 2-D plot functions can be
superimposed on the plane. The function addLines() does the work for
translating 2-D plot object onto the regression plane. 

\begin{figure}
<<pp111, fig = T, echo = F, height = 5, results = hide>>=
ppm5 <- plotPlane(m5, plotx1 = "x2", plotx2 = "x1", phi = 0, npp = 15, lcol = gray(.80))
addLines(from = m5pc, to = ppm5, col = m5pc$col)
@

\caption{Making plotSlopes and plotPlane Work Together\label{fig:pp110}}
\end{figure}


\section{Standardized, Mean-Centered, and Residual-Centered Regressions }

\subsection{Standardized regression}

Many of us learned to conduct regression analysis with SPSS, which
reported both the ordinary (unstandardized) regression coefficients
as well as a column of ``beta weights'', the output of a ``standardized''
regression analysis. Each variable, for example $x1_{i}$, was replaced
by an estimated $Z-score:$ $(x1_{i}-\overline{x1})/std.dev.(x1_{i}$).
Some people think these coefficients are easier to interpret (but,
for a strong cautionary argument against them, see \citealp{King1986}).
R offers no such thing as standardized regression, probably because
this practice is thought to be mistaken. The automatic standardization
of all predictors, no matter whether they are categorical, interaction
terms, or transformed values (such as logs) is dangerous. 

To illustrate that, the rockchalk introduces a function called standardize().
Each column of the design matrix is scaled to a new variable with
mean 0 and standard deviation 1. The result from standardize() will
be an R lm object, which will respond to any follow-up analysis commands.
For example:

<<>>=
m4 <- lm (y4 ~ x1 * x2, data = dat)
m4s <- standardize(m4)
@

I doubt that a reasonable person would actually want a standardized
regression and have tried to warn users in the output.

<<>>=
summary(m4s)
@

\begin{table}
\caption{Comparing Ordinary and Standardized Regression\label{tab:stdreg10}}

<<stdreg10, results=tex, echo=F>>=
or10 <- outreg(list(m4, m4s), tight = F, modelLabels = c("Not Standardized","Standardized"))
@
\end{table}


\subsection{Mean-centered Interaction Models}

Sometimes people will fit a model like this
\begin{equation}
y_{i}=b_{o}+b_{1}x1_{i}+b_{2}x2_{i}+e_{i}
\end{equation}

\noindent and then wonder, ``is there an interaction between $x1_{i}$
and $x2_{i}$?'' The natural inclination is to run this model, 

\bgroup\inputencoding{latin9}
\begin{lstlisting}
m1 <- lm(y ~ x1*x2)
\end{lstlisting}
\leavevmode\egroup

\noindent or its equivalent

\bgroup\inputencoding{latin9}
\begin{lstlisting}
m2 <- lm(y ~ x1 + x2 + x1:x2)
\end{lstlisting}
\leavevmode\egroup

Researchers have been advised that they should not run the ordinary
interaction model without ``mean-centering'' the predictors (Aiken
and West, 1991). They are advised to replace $x1_{i}$ with $(x1_{i}-\overline{x1})$
and $x2_{i}$ with $(x2_{i}-\overline{x2})$, so that the fitted model
will
\begin{equation}
y_{i}=b_{o}+b_{1}(x1_{i}-\overline{x1})+b_{2}(x2_{i}-\overline{x2})+b_{3}(x1_{i}-\overline{x1})(x2_{i}-\overline{x2})+e_{i}
\end{equation}

This is a little tedious to do in R, so I provide a function meanCenter()
that can handle the details. meanCenter() will receive a model, scan
it for interaction terms, and then center the variables that are involved
in interactions. We previously fit the model m4, and now we center
it.

<<>>=
m4mc <- meanCenter(m4)
summary(m4mc)
@

By default, meanCenter() will only center the variables involved in
an interaction, and it leaves the others unchanged. The user can request
a different treatment of the variables. Version 1.8 introduces the
argument ``terms'', which allows the user to list the names of the
predictors that should be centered. If the user wants all of the numeric
predictors to be mean-centered, the usage of the argument centerOnlyInteractors
would be appropriate:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
m4mc <- meanCenter(m4, centerOnlyInteractors = FALSE)
\end{lstlisting}
\leavevmode\egroup

By default, it does not standardize while centering (but the user
can request standardization with the argument standardize = TRUE.
The option centerDV causes the dependent variable to be centered as
well.

\subsection{Residual-centered Models}

Residual-centering \citep{LittleBovaird2006} is another adjustment
that has been recommended for models that include interactions or
squared terms. Like mean-centering, it is often recommended as a way
to obtain smaller standard errors or to make estimates more numerically
stable. Like mean centering, it causes a superficial change in the
estimated coefficients, but the predicted values and the regression
relationship is not actually changed. Nothing of substance is altered.

The residualCenter() function is used in the same manner as meanCenter().
The user fits an interactive model and the result object is passed
to residualCenter() like so:

<<>>=
m4rc <- residualCenter(m4)
summary(m4rc)
@

I would explain residual-centering as follows. Suppose we fit the
linear model, with no interaction (note, I'm calling the coefficients
$c_{j}$, not $b_{j}$ as usual):

\begin{equation}
y=c_{0}+c_{1}x1+c_{2}x2+e{}_{i}.\label{eq:rc20}
\end{equation}

\noindent Let's proceed as if those parameter estimates, $\hat{c}_{1}$,
$\hat{c}_{2}$, are the ``right ones'' for our analytical purpose.
We'd like to fit an interactive model, but protect the linear parts
from fluctuation. In R, when we run the model \bgroup\inputencoding{latin9}\lstinline!lm(y ~ x1 * x2)!\egroup
, we are allowing all of the coefficients of all variables to fluctuate
\begin{equation}
y_{i}=b_{o}+b_{1}x1_{i}+b_{2}x2_{i}+b_{3}x1_{i}\times x2_{i}+e_{i}\label{eq:rcwant}
\end{equation}
Residual centering is one way to stabilize the estimation by assuring
that $\hat{b}_{1}=\hat{c}_{1}$ and $\hat{b}_{2}=\hat{c}_{2}$. Only
the coefficient $\hat{b}_{3}$ floats freely.

One of the reasons that residual-centering is so appealing is that
its stabilizing benefit is obtained almost by accident. Here is the
gist of the calculations. First, estimate a regression in which the
left hand side is the interaction product term:
\begin{equation}
(x1_{i}\times x2_{i})=d_{0}+d_{1}x1_{i}+d_{2}x2+u_{i}\label{eq:residCentered}
\end{equation}

\noindent The residuals from that regression are, by definition, orthogonal
to both $x1$ and $x2$. Call those fitted residuals $\widehat{u_{i}}$.
The we run the interactive regression, replacing the column of the
predictor $x1_{i}\times x2_{i}$, with $\widehat{u}_{i}$. That is
to say, the model we want, equation (\ref{eq:rcwant}), is estimated
as: 
\begin{equation}
y_{i}=b_{0}+b_{1}x1_{i}+b_{2}x2_{i}+b3\widehat{u_{i}}+e_{i},\label{eq:rc10-1}
\end{equation}

\noindent In essence, we have taken the interaction $(x1_{i}\times x2_{i})$,
and purged it of its parts that are linearly related to $x1_{i}$
and $x2_{i}$. 

rockchalk 1.6 included summary, print, and predict methods for the
residual-centered regression objects. It is worth mentioning that
the code can handle interactions of arbitrarily many predictors. If
the formula has \bgroup\inputencoding{latin9}\lstinline!lm(y ~ x1 * x2 * x3 * x4)!\egroup
, for example, this implies many separate interactions must be calculated.
We need to calculate residual-centered residuals for $x1\cdot x2$,
$x1\cdot x3$, $x1\cdot x4$, $x2\cdot x3$ and so forth, and then
use them as predictors to get centered estimates of terms $x1\cdot x2\cdot x3$,
and then their centered values are predictors four term interactions.
Aspiring R programmers who want to learn about programming with R
formula objects might benefit from the study of the function residualCenter.R
in the rockchalk source code. 

\section{A Brief Analysis of Mean-Centering}

We can put the tools together by making a little detour into the question
that seems to plague every regression analyst at one time or another:
What does that interaction term \emph{really mean}? Along the way,
we will try to dispel the idea that centering somehow makes estimates
``better'' or more numerically precise. The primary advocates of
centering as a way to deal with numerical instability are \citet{Aiken1991},
who integrated that advice into the very widely used regression textbook,
\emph{Applied Multiple Regression/Correlation for the Behavioral Sciences}
\citep{Cohen2002}. They claim that the inclusion of interactions
causes ``inessential multicollinearity'' that is alleviated by centering.
The advice is widely followed. One statistics book intended for biologists
observed, for example, ``We support the recommendation of Aiken \&
West (1991) and others that multiple regression with interaction terms
should be fitted to data with centered predictor values'' (\citealp{QuinnKeough2002},
Chapter 6). 

Technical rebuttals have been published already (\citealp{Kromrey1998}),
but the matter still seems not widely understood. The argument is
not that mean-centering (or residual-centering) is wrong, but rather
that it is unnecessary. It is irrelevant, and possibly, misleading.

At the core of the matter is the fact that our uncertainty about regression
estimates depends on our point of view. Please review the confidence
interval in Figure \ref{fig:ps05}. The $y$ axis is not even ``in
the picture.'' Would one's appreciation of the regression's predictive
line be enhanced if the y axis were moved into the picture? 

We can move the $y$ axis by centering the predictor variable. Suppose
we replace $x_{i}$ with $x_{i}-5$ and then re-estimate the model.
That has the simple effect of moving the $y$ axis 5 units to the
right. The slope is unchanged, and the reported intercept is changed
in a completely superficial way. The predicted value line, the slope
estimates, the residual standard error, and so forth, are not changed
in any substantial way. This is simply a matter of user convenience.
I believe that no reasonable person can say the regression is ``better''
after centering $x_{i}$.

However, there is a superficial difference that has deceived many
authors. Notice that the confidence interval is hourglass shaped.
\emph{At the new $y$ axis}, our prediction is more precise. If we
move the y axis further to the right, into the center of the data,
say by mean-centering ($x_{i}-10$), we move to the position that
allows an even more precise prediction. The estimate of the intercept's
standard error will be smaller for the obvious reason. We are not
actually gaining certainty, we are simply reporting our most favorable
``snapshot'' of it. The predicted value, and the confidence interval
for any observed value of $x_{i}$ is completely unchanged by centering.

It should not surprise the reader to learn that mean-centering interactive
predictors enhances the standard errors in the same illusory way.
Let's work though an example to see why this is so tempting. Suppose
the true data generating mechanism is an interaction like so
\begin{equation}
y_{i}=2+0.1x1_{i}+0.1x2_{i}+0.2\cdot(x1_{i}\times x2_{i})+e_{i},
\end{equation}

\noindent where $e_{i}\sim N(0,300^{2})$ and $\rho_{x1,x2}=0.4$. 

A regression analysis that ignores the interaction, 

\bgroup\inputencoding{latin9}
\begin{lstlisting}
lm(y ~ x1 + x2, data = dat2)
\end{lstlisting}
\leavevmode\egroup

\noindent is reported in the first column of Table \ref{tab:meancenter10-1}.
I've used outreg's new alpha argument to emphasize the ``really good''
estimates with more stars. Notice that everything is ``statistically
significant!''

Unable to leave well enough alone, the researcher wonders, ``is there
an interaction between $x1$ and $x2$?'' Run \bgroup\inputencoding{latin9}
\begin{lstlisting}
lm(y ~ x1 * x2, data = dat2)
\end{lstlisting}
\leavevmode\egroup

\noindent The second column in Table \ref{tab:meancenter10-1} summarizes
that regression. Be prepared for a shock when you scan the estimates.
Almost everybody I know has said ``what the heck?'' or ``Holy Cow!''
or ``Oh My God, my great result went to Hell, I'll never get tenure!''
Neither of the key variables is ``statistically significant'' any
more. 

<<echo=F, include=F>>=
dat2 <- genCorrelatedData3(N=400, rho=.4, stde=300, beta=c(2,0.1,0.1,0.2))
m6linear <- lm (y ~ x1 + x2, data=dat2)
m6int <- lm (y ~ x1 * x2, data=dat2)
m6mc <- meanCenter(m6int)
m6rc <- residualCenter(m6int)
@

\begin{table}
\caption{Comparing Regressions\label{tab:meancenter10-1}}

<<mcenter10, results=tex, echo=F>>=
or11 <- outreg(list(m6linear, m6int, m6mc, m6rc), tight=F, modelLabels=c("Linear", "Interaction","Mean-centered","Residual-centered"), alpha = c(0.05, 0.01))
@
\end{table}

Cohen, et al. claim that the apparent instability of the coefficients
is a reflection of ``inessential collinearity,'' due to the fact
that $x1$ and $x2$ are correlated with the new term, $x1\times x2$.
They advised their readers to ``mean-center'' their predictors and
run the regression again. 

Remember the hourglass shape of the confidence interval. By mean-centering,
we are re-positioning ourself for a much more favorable snapshot.
The welcoming effect of the centered estimates is found in the third
column of Table \ref{tab:meancenter10-1}. The point estimates in
that snapshot are ``significant again.'' It appears we have ``solved''
the problem of inessential collinearity. 

The first hint of trouble is in the fact that the coefficients of
the interactive effects in columns 2 and 3 are identical. Those coefficients
are the same because they are estimates of the cross partial derivative
$\partial^{2}y/\partial x1\partial x2$. That particular value is
the same, no matter where we position the $y$ axis, as it should
be. Note as well that the root mean square and $R^{2}$ estimates
are identical. Everything that we expect to remain the same is the
same. Except for the fact that the slopes and their hypothesis tests
seem better in the centered model, we would think there is nothing
interesting here.

Here's a puzzle for you. Consider Figure \ref{fig:pscentering}, which
shows the predicted values and confidence intervals from the centered
and uncentered regressions. Is there any substantively important difference
between these two regressions?

\begin{figure}
<<pscenter, fig=T, echo=F, include = F, height = 10, width = 5, results = hide>>=
par(mfcol = c(2, 1))
plotSlopes(m6int, plotx = "x1", modx = "x2", modxVals = "std.dev.",  n = 2, interval = "confidence", main= "Not Centered")
plotSlopes(m6mc, plotx = "x1c", modx = "x2c", modxVals = "std.dev.", n = 2, interval = "confidence", main = "Mean Centered")
par(mfcol = c(1, 1))
@

\includegraphics[height=8in]{rockchalk-pscenter.pdf}

\caption{plotSlopes for the centered and non-centered regressions\label{fig:pscentering}}
\end{figure}

Perhaps the 2-D plots are not persuasive. Don't stop yet. In the 3-D
plots will help quite a bit. We have not yet grasped the most fundamental
changed caused by the insertion of the interaction term. When we insert
$x1_{i}\times x2_{i}$, we change the fudamental nature of the regression
surface. The surface of the fitted model is no longer a ``flat''
plane, but rather it is a curving surface. 

I've assembled 3-D plots in Figure \ref{fig:mcenter30}. We see that
the ordinary interaction, mean-centered, and residual-centered models
produce identical predicted values! The only difference in the figures
is that the axes of the predictors have been re-scaled, so that the
$y$ axis is (implicitly) re-positioned. Now that we understand the
situation, we could play around with the data and move the axis back
and forth until we arrive at a position that is most favorable to
our interpretation. 

\begin{figure}
<<mcenter50, fig=T,echo=FALSE, height=5, width=7>>=
op <- par(no.readonly = TRUE)
par(mfcol=c(2,2))
par(mar=c(2,2,2,1))
plotPlane(m6linear, plotx1="x1", plotx2="x2", plotPoints=FALSE, main="Linear", ticktype="detailed")
plotPlane(m6int, plotx1="x1", plotx2="x2", plotPoints=FALSE, main="Interaction: Not Centered", ticktype="detailed")
plotPlane(m6mc, plotx1="x1c", plotx2="x2c", plotPoints=FALSE, main="Mean-centered", ticktype="detailed")
plotPlane(m6rc, plotx1="x1", plotx2="x2", plotPoints=FALSE, main="Residual-centered", ticktype="detailed")
par(op)
@

\caption{Predicted Planes from Centered and Uncentered Fits Identical\label{fig:mcenter30}}
\end{figure}

The regression coefficient estimates are snapshots, each summarizing
the curvature at one particular point in a curving surface. It seems
quite apparent in Figure \ref{fig:mcenter30} that the models are
identical, and yet we receive different regression reports from different
spots. The non-centered data offers us the slope estimate from the
``front-left'' part of the graph. Mean-centered estimates report
on the slope in the middle of the graph. In the rockchalk examples
folder, one can find a file called ``centeredRegression.R'' that
walks through this argument step by step. 

What about residual-centering? Because the transformation that it
employs is more abstract, I initially thought it was actually a different
model. And yet it is not. The residual-centered model is completely
equivalent to the ordinary interaction model and the mean-centered
model. For a given combination of the input values, the predicted
values and confidence intervals are the same. The predicted values
of the ordinary interactive model, the mean-centered model, and the
residual-centered models are illustrated in Figure \ref{fig:rcenter40}. 

\begin{figure}
<<rcenter40, fig=T,echo=FALSE, include=F, height=4, width=8>>=
dat3 <- centerNumerics(dat2)
##m6mcpred <- fitted(m6mc) ##
m6mcpred <- predict(m6mc, newdata=dat3)
##m6rcpred <- fitted(m6rc) ##
m6rcpred <- predict(m6rc, newdata=dat3)
##m6intpred <- fitted(m6int) ##
m6intpred <- predict(m6int, newdata=dat3)
op <- par(no.readonly = TRUE)
par(mfcol=c(1,2))
plot(m6intpred, m6rcpred, main="", xlab="Predictions of Uncentered Interaction", ylab="Residual-centered Predictions")
predcor <- round(cor(m6intpred, m6rcpred),3)
legend("topleft", legend=c(paste0("Correlation=", predcor)))
plot(m6mcpred, m6rcpred, main="", xlab="Mean-centered Predictions", ylab = "Residual-centered Predictions")
predcor <- round(cor(m6mcpred, m6rcpred),3)
legend("topleft", legend=c(paste0("Correlation=", predcor)))
par(op)
@

\includegraphics[width = 6.5in]{rockchalk-rcenter40}

\caption{Predicted Values of Mean and Residual-centered Models\label{fig:rcenter40}}
\end{figure}

The take-away point from this is that there is no free lunch in regression
analysis. If re-scaling a variable by adding or subtracting a constant
seems to change a result, one should be cautious and suspect an error. 

In order to drive the point home, I'd like to show that it is possible
to translate between the estimates of any one of these fitted models
and the estimates of the others. The ordinary model is
\begin{equation}
y_{i}=b_{0}+b_{1}x1_{i}+b_{2}x2_{i}+b_{3}(x1_{i}\times x2_{i})+e1_{i}\label{eq:int}
\end{equation}

The mean-centered model is 
\begin{equation}
y_{i}=c_{0}+c_{1}(x1_{i}-\overline{x1})+c_{2}(x2_{i}-\overline{x2})+c_{3}(x1_{i}-\overline{x1})\cdot(x2_{i}-\overline{x2})+e2_{i}\label{eq:mc1}
\end{equation}

In order to compare with equation \ref{eq:int}, we would re-arrange
like so

\begin{equation}
y_{i}=c_{0}+c_{1}(x1_{i})-c_{1}\overline{x1}+c_{2}(x2_{i})-c_{2}\overline{x2}+c_{3}(x1_{i}x2_{i}+\overline{x1}\overline{x2}-\overline{x1}x2_{i}-\overline{x2}x1_{i})+e2_{i}
\end{equation}

\begin{equation}
y_{i}=c_{0}+c_{1}(x1_{i})-c_{1}\overline{x1}+c_{2}(x2_{i})-c_{2}\overline{x2}+c_{3}(x1_{i}x2_{i})+c_{3}\overline{x1}\overline{x2}-c_{3}\overline{x1}x2_{i}-c_{3}\overline{x2}x1_{i})+e2_{i}
\end{equation}

\begin{equation}
y_{i}=\{c_{0}-c_{1}\overline{x1}-c_{2}\overline{x2}+c_{3}\overline{x1}\overline{x2}\}+\{c_{1}-c_{3}\overline{x2}\}x1_{i}+\{c_{2}-c_{3}\overline{x1}\}x2_{i}+c_{3}(x1_{i}x2_{i})+e2_{i}\label{eq:mc3}
\end{equation}

One can then compare the parameter estimates from equations \ref{eq:int}
and \ref{eq:mc3}. Both \ref{eq:int} and \ref{eq:mc3} include a
single parameter times $(x1_{i}x2_{i}),$ leading one to expect that
the estimate $\hat{b}_{3}$ should be equal to the estimate of $\hat{c}_{3}$,
and they are! Less obviously, one can use the fitted coefficients
from either model to deduce the fitted coefficients from the other.
The following equalities describe that relationship.
\begin{eqnarray}
\hat{b}_{0} & = & \hat{c}_{0}-\hat{c}_{1}\overline{x1}-\hat{c}_{2}\overline{x2}+\hat{c}_{3}\overline{x1}\overline{x2}\\
\hat{b}_{1} & = & \hat{c}_{1}-\hat{c}_{3}\overline{x2}\\
\hat{b}_{2} & = & \hat{c}_{2}-\hat{c}_{3}\overline{x1}\\
\hat{b}_{3} & = & \hat{c}_{3}
\end{eqnarray}
The estimated fit of equation \ref{eq:mc1} would provide estimated
coefficients $\hat{c}_{j}$, $j=0,...,3$, which would then be used
to calculate the estimates from the non-centered model.

The estimation of the residual-centered model requires two steps.
The residual from this regression model

\noindent
\begin{equation}
\widehat{(x1_{i}\times x2_{i})}=\hat{d}_{0}+\hat{d}_{1}x1_{i}+\hat{d}_{2}x2_{i}.
\end{equation}

\noindent is 
\begin{equation}
\widehat{u}_{i}=(x1_{i}\times x2_{i})-\widehat{(x1_{i}\times x2_{i})},
\end{equation}

\noindent which is inserted into equation \ref{eq:int}. 

\begin{equation}
y_{i}=h_{0}+h_{1}x1_{i}+h_{2}x2_{i}+h_{3}\{x1_{i}\times x2_{i}-\widehat{x1_{i}\times x2_{i}}\}+e3_{i}\label{eq:rc1}
\end{equation}

Replacing $\widehat{x1_{i}\times x2_{i}}$ with $\hat{d}_{0}+\hat{d}_{1}x1_{i}+\hat{d}_{2}x2_{i}$,
\ref{eq:rc1} becomes

\begin{eqnarray}
y_{i} & = & h_{0}+h_{1}x1_{i}+h_{2}x2_{i}+h_{3}\{x1_{i}\times x2_{i}-\hat{d}_{0}-\hat{d}_{1}x1_{i}-\hat{d}_{2}x2_{i}\}+e3_{i}\\
 & = & h_{0}+h_{1}x1_{i}+h_{2}x2_{i}+h_{3}\{x1_{i}\times x2_{i}\}-h_{3}\hat{d}_{0}-h_{3}\hat{d}_{1}x1_{i}-h_{3}\hat{d}_{2}x2_{i}\}+e3_{i}\\
 &  & \{h_{0}-h_{3}\hat{d}_{0}\}+\{h_{1}-h_{3}\hat{d}_{1}\}x1_{i}+\{h_{2}-h_{3}\hat{d}_{2}\}x2_{i}+h_{3}\{x1_{i}\times x2_{i}\}+e3_{i}
\end{eqnarray}

As in the previous comparison, we can translate coefficient estimates
between the ordinary specification and the residual-centered model.
The coefficient estimated for the product term, $\hat{h}_{3}$, should
be equal to $\hat{b}_{3}$ and $\hat{c}_{3}$ (and it is!). If we
fit the residual centered model, \ref{eq:rc1}, we can re-generate
the coefficients of the other models like so: 
\begin{eqnarray}
\hat{b}_{0} & =\hat{c}_{0}-\hat{c}_{1}\overline{x1}-\hat{c}_{2}\overline{x2}+\hat{c}_{3}\overline{x1}\overline{x2}= & h_{0}-h_{3}\hat{d}_{0}\\
\hat{b}_{1} & =\hat{c}_{1}-\hat{c}_{3}\overline{x2}= & h_{1}-h_{3}\hat{d}_{1}\\
\hat{b}_{2} & =\hat{c}_{2}-\hat{c}_{3}\overline{x1}= & h_{2}-h_{3}\hat{d}_{2}
\end{eqnarray}

From the preceding, it should be clear enough that the three models
are equivalent.

\section{Conclusion}

The rockchalk package is offered as a system of support for teachers
and students of regression analysis. It should help with the preparation
of plots, summary tables, and other diagnostics. 

A number of functions are currently offered in this package that have
not been emphasized in this writeup. I would draw the reader to the
help pages for these functions
\begin{description}
\item [{combineLevels}] a recoder for factor variables
\item [{mcDiagnose}] a one stop shop for multicollinearity diagnostic information
\item [{getDeltaRsquare}] calculate the change in the $R^{2}$ that results
from the omission of each variable. This is the squared semi-partial
correlation coefficient.
\item [{getPartialCor}] calculates the partial correlation matrix of the
predictors in a regression model
\item [{lazyCor~and~lazyCov}] convenient ways to create correlation and
covariance matrices that are needed in many simulation exercises
\item [{mvrnorm}] a slightly improved version of the MASS package's multivariate
normal generator.
\end{description}
\bibliographystyle{chicago}
\bibliography{rockchalk}

\end{document}

#LyX 2.3 created this file. For more info see http://www.lyx.org/
\lyxformat 544
\begin_document
\begin_header
\save_transient_properties true
\origin unavailable
\textclass extarticle
\begin_preamble
%\VignetteIndexEntry{outreg}

% Following comment is required. Do not delete it.
%\usepackage{Sweave}

\usepackage[includehead,includefoot]{geometry}
\geometry{
lmargin=1in,
rmargin=1in,
tmargin=0.75in,
bmargin=1.0in,
headheight=0pt,
headsep=0pt,
marginparwidth=0pt,
footskip=1.5\baselineskip,
}

\usepackage{booktabs}
\usepackage{dcolumn}

% paste in
%%\input{theme/guidePreambleHeader.tex}

% $ This nullifies emacs subscripting in following%
%%BEGIN: This begins the custom header
\usepackage{array}
\usepackage{xparse}
\usepackage{babel}
\usepackage{datetime}
\usepackage{xcolor}
\usepackage{colortbl}
\usepackage{graphicx}
\usepackage{multicol}

\definecolor{darkblue}{HTML}{1e2277}
\definecolor{kublue81186}{RGB}{0,81,186}
\newdateformat{monddyyyy}{\shortmonthname[\THEMONTH].~\THEDAY,~\THEYEAR}


%% BEGIN custom header
\ExplSyntaxOn
\NewDocumentCommand \guidesetup { m }
 {
  \keys_set:nn { guide } { #1 }
 }

\int_new:N \g_guide_authors_int
\prop_new:N \g_guide_authors_prop
\tl_new:N \l__guide_authors_tl

\keys_define:nn { guide }
 {
  author .code:n =
   {
    \int_gincr:N \g_guide_authors_int
    \keys_set:nn { guide / author } { #1 }
   },
  author/firstname .code:n = { \guide_add_author:nn { firstname } { #1 } },
  author/lastname .code:n = { \guide_add_author:nn { lastname } { #1 } },
  author/affiliation .code:n = { \guide_add_author:nn { affiliation } { #1 } },
  author/email .code:n = { \guide_add_author:nn { email } { #1 } },
  leftlogo .tl_set:N = \l_guide_leftlogo_tl,
  rightlogo .tl_set:N = \l_guide_rightlogo_tl,
  number .tl_set:N = \l_guide_number_tl,
  title .tl_set:N = \l_guide_title_tl,
  subtitle .tl_set:N = \l_guide_subtitle_tl,
  keywords .tl_set:N = \l_guide_keywords_tl,
  url .tl_set:N = \l_guide_url_tl,
  title .initial:n = {TITLE~NEEDED},
}
\NewDocumentCommand \guidehdr { +O{} }
 {
  \group_begin:
  \keys_set:nn { guide } { #1 }
  \guide_hdr:
  \group_end:
 }
\cs_new_protected:Nn \guide_hdr:
 {
   \centering
   \noindent\fcolorbox{kublue81186}{white}{
  \begin{tabular}
   {
    @{}c
    >{\centering}m{ \dim_eval:n { \textwidth - 2.5in - 6\tabcolsep - 2\arrayrulewidth } }
    c@{}
   }
   &&\tabularnewline[-\dim_eval:n { -\normalbaselineskip + 3ex }]
  \begin{tabular}{@{}>{\centering}p{1.25in}@{}}
  \includegraphics[width=.75in]{\l_guide_leftlogo_tl}
  \end{tabular}
  &
  \centering
    {\textsc{\large{}\l_guide_title_tl}}
    \tl_if_empty:NF \l_guide_subtitle_tl
    {
      {\\\textsc{\large{}\l_guide_subtitle_tl}}
    }
  \\
  \textcolor{kublue81186}{\rule[0.5ex]{1\linewidth}{0.7pt}}\\
  \guide_print_authors:
  &
  \begin{tabular}{@{}>{\centering}p{1.25in}@{}}
  \includegraphics[width=1.25in]{\l_guide_rightlogo_tl}
  \end{tabular}
  \tabularnewline
  \tl_if_empty:NTF \l_guide_number_tl
   { \leavevmode\vphantom{G}}
   {\small\bfseries { Guide~No: ~ \tl_use:N \l_guide_number_tl }} & 
   \tl_if_empty:NF \l_guide_keywords_tl
   {\centering
    \textbf{Keywords}: ~ \l_guide_keywords_tl}
   & 
   \small\bfseries\monddyyyy\today \tabularnewline
   \tl_if_empty:NF \l_guide_url_tl
   {
   \multicolumn{3}{c}{
    See ~ \url{\l_guide_url_tl} ~ for ~ updates.}
     }
  \tabularnewline
  \end{tabular}
  }
}

\cs_new_protected:Nn \guide_add_author:nn
 {
  \prop_gput:Nxn \g_guide_authors_prop
   { #1 \int_to_arabic:n { \g_guide_authors_int } }
   { #2 }
 }
\cs_generate_variant:Nn \prop_gput:Nnn { Nx }
\cs_new_protected:Nn \guide_print_authors:
 {
  \tl_clear:N \l__guide_authors_tl
  \int_step_inline:nnnn { 1 } { 1 } { \g_guide_authors_int }
   {
    \tl_put_right:Nn \l__guide_authors_tl
     {
      \textbf
       {
        \prop_item:Nn \g_guide_authors_prop { firstname ##1 }~
        \prop_item:Nn \g_guide_authors_prop { lastname ##1 },~
       }
       \prop_item:Nn \g_guide_authors_prop { affiliation##1 },~
       \href{mailto:\prop_item:Nn \g_guide_authors_prop { email##1 } }
       {\url{\prop_item:Nn \g_guide_authors_prop { email##1 }}}
       \\
     }
   }
   \tl_use:N \l__guide_authors_tl
 }
\ExplSyntaxOff
%%END This ends the custom header


%%Now the hyperref colors

\hypersetup{
 unicode=true,pdfusetitle,bookmarks=true,
 bookmarksnumbered=false, bookmarksopen=false,
 breaklinks=true, pdfborder={0 0 0},
 pdfborderstyle={}, backref=false,
 colorlinks=true, linkcolor=darkblue,
 urlcolor=darkblue, citecolor=darkblue}

%% Was previously in "preambleFloatCenter.tex"
%%BEGIN: graphics and such inside floats are centered by default
\renewenvironment{figure}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{figure}
 }{%
   \@float{figure}[#1]%
 }%
 \centering
}{%
 \end@float
}
\renewenvironment{table}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{table}
 }{%
   \@float{table}[#1]%
 }%

 \centering
}{%
 \end@float
}
%%END: graphics and such inside floats are centered by default

% paste
%%\input{theme/preambleFooter.tex}

\usepackage{fancyhdr}
\usepackage{lastpage}
\usepackage{xparse}

\ExplSyntaxOn
\NewDocumentCommand \footersetup { +m }
{
\keys_set:nn { footer } { #1 }
}
\keys_define:nn { footer }
{	
    leftside .code:n = {
\keys_set:nn { footer / leftside } { #1 }
},
rightside .code:n = {
\keys_set:nn { footer / rightside } { #1 }
},
    leftside / lone .tl_set:N = \l_footer_leftside_lone_tl,
    leftside / ltwo .tl_set:N = \l_footer_leftside_ltwo_tl,
    leftside / lthree .tl_set:N = \l_footer_leftside_lthree_tl,
    rightside / rone .tl_set:N = \l_footer_rightside_rone_tl,
    rightside / rtwo .tl_set:N = \l_footer_rightside_rtwo_tl,
    rightside / rthree .tl_set:N = \l_footer_rightside_rthree_tl,
    leftside .initial:n = {lone = , ltwo = ~, lthree = ~ },
    rightside .initial:n = {rone = ~, rtwo = ~,  rthree = ~},
    leftside .default:n = {lone = , ltwo = ~, lthree = ~ },
    rightside .default:n = {rone = ~, rtwo = ~,  rthree = ~},
}
\NewDocumentCommand \guideftr { +o }
{
  \IfValueT { #1 } 
  {
    \keys_set:nn { footer } { #1 }
  }
  \guide_ftr:
}
\cs_new_protected:Nn \guide_ftr:
{
  % Now fancy header/footer settings
  \lhead{}%
  \chead{}%
  \rhead{}%
  \renewcommand{\headrulewidth}{0pt}%
  \lfoot{}%
  \cfoot{{Page} ~ \thepage {} ~ { of } ~ {} \pageref{LastPage}}%
  \rfoot{}%

  \fancypagestyle{firststyle}%
  {%
    \lhead{}%
    \chead{}%
    \rhead{}%
    \lfoot{}%
    \cfoot{}%
    \rfoot{}%
    \renewcommand{\headrulewidth}{0pt}%
    \fancyfoot[L]{\l_footer_leftside_lone_tl {}\\%
      \l_footer_leftside_ltwo_tl {}\\%
      \l_footer_leftside_lthree_tl {}}%
    \fancyfoot[R]{\l_footer_rightside_rone_tl {}\\%
      \l_footer_rightside_rtwo_tl {}\\%
      \l_footer_rightside_rthree_tl {}}%
      \renewcommand{\footrulewidth}{.2pt}%
  }%
  \thispagestyle{firststyle}
}
\ExplSyntaxOff

\pagestyle{fancy}
%% End of fancy footer setup

%% Now the addressFooter file
\urlstyle{tt}
\footersetup{
  leftside={
    lone={CRMDA{,}~1425~Jayhawk~BLVD}, 
    ltwo={University~of~Kansas{,}~Waston~Library~Suite~470}, 
    lthree={Lawrence{,}~KS~66045}},
  rightside={
    rone=Web: \url{https://crmda.ku.edu},
    rtwo=Email: \href{mailto:pauljohn@ku.edu}{\url{pauljohn@ku.edu}},
    rthree=Phone: 785-864-3353}
}
\guideftr




% paste from
%%\input{theme/guidePreambleSweavel.tex}
%%% From beamer slide:
%% \usepackage{Sweave}
%% 
%% This controls display of code chunks
\usepackage{ae,fancyvrb,relsize,listings}

\providecommand{\Sweavesize}{\normalsize}
\providecommand{\Rsize}{}
\renewcommand{\Rsize}{\normalsize}
\providecommand{\Routsize}{\scriptsize}

\providecommand{\Rcolor}{\color[rgb]{0.1, 0.1, 0.1}}
\providecommand{\Routcolor}{\color[rgb]{0.2, 0.2, 0.2}}
\providecommand{\Rcommentcolor}{\color[rgb]{0.101, 0.43, 0.432}}

\providecommand{\Rbackground}{\color[gray]{0.91}}
\providecommand{\Routbackground}{\color[gray]{0.935}}
% Can specify \color[gray]{1} for white background or just \color{white}

\lstdefinestyle{Rinput}{
  language=R,
  escapechar=`,
  fancyvrb=false,%
  tabsize=2,%
  breaklines=true,
  breakatwhitespace=true,%
  captionpos=b,%
  frame=single,%
  framerule=0.2pt,%
  framesep=1pt,%
  showstringspaces=false,%
  basicstyle=\Rsize\Rcolor\ttfamily,%
  columns=fixed%,
  \lst@ifdisplaystyle\scriptsize\fi,%,
  commentstyle=\Rcommentcolor\ttfamily,%
  identifierstyle=,%
  keywords=\bfseries,%
  keywordstyle=\color[rgb]{0, 0.5, 0.5},
  escapeinside={(*}{*)},
  literate={~}{{$\sim$}}1{==}{{=\,=}}2{--}{{-\,-}}2,
  alsoother={$},
  alsoletter={.<-},%
  otherkeywords={!,!=,~,$$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},%
  backgroundcolor=\Rbackground,%
  numbers=left,%
  %numberblanklines=false,%
  stepnumber=5,
  firstnumber=1,
  numberstyle={\tiny}
}%

% Other options of interest:
% frame=single,framerule=0.1pt,framesep=1pt,rulecolor=\color{blue},
% numbers=left,numberstyle=\tiny,stepnumber=1,numbersep=7pt,
% keywordstyle={\bf\Rcolor}

\lstdefinestyle{Routput}{fancyvrb=false,
  literate={~}{{$\sim$}}1{R^2}{{$R^{2}$}}2{^}{{$^{\scriptstyle\wedge}$}}1{R-squared}{{$R^{2}$}}2,%
  basicstyle=\Routcolor\Routsize\ttfamily,%
  backgroundcolor=\Routbackground,
  language=R,
  escapechar=`,
  fancyvrb=false,%
  tabsize=2,%
  breaklines=true,
  breakatwhitespace=true,%
  captionpos=b,%
  frame=single,%
  framerule=0.2pt,%
  framesep=1pt,%
  showstringspaces=false,%
  columns=fixed%,
  \lst@ifdisplaystyle\scriptsize\fi,%
  identifierstyle=,%
  keywords=\bfseries,%
  keywordstyle=\color[rgb]{0, 0.5, 0.5},
  escapeinside={(*}{*)},
  literate={~}{{$\sim$}}1 {==}{{=\,=}}2,
  alsoother={$},
  alsoletter={.<-},%
  otherkeywords={!,!=,~,$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},
  numbers=left,
  %numberblanklines=false,%
  stepnumber=5,
  firstnumber=1,
  numberstyle={\tiny}
}

\newenvironment{Schunk}{}{}
\newenvironment{Sinput}{}{}
\let\Sinput\relax
\let\Scode\relax
\let\Soutput\relax
\lstnewenvironment{Sinput}{\lstset{style=Rinput}}{}
\lstnewenvironment{Scode}{\lstset{style=Rinput}}{}
\lstnewenvironment{Soutput}{\lstset{style=Routput}}{}
%%end paste in from guidePreambleSweavel.tex


\lstset{tabsize=2, breaklines=true, style=Rinput, breakatwhitespace=true}

\fvset{listparameters={\setlength{\topsep}{0em}}}

\usepackage{xcolor}
\definecolor{light-gray}{gray}{0.90}
\usepackage{realboxes}
\providecommand*{\code}[1]{\texttt{#1}}
\renewcommand{\code}[1]{%
\Colorbox{light-gray}{#1}%
}%


\usepackage{siunitx}
%following now in each table
\sisetup{
  input-symbols = ( )
}
%,
%  group-minimum-digits = 9,
%  table-number-alignment = center,
%  table-space-text-pre = (,
%  table-align-text-pre = false,
%  table-align-text-post = false,
%  table-space-text-post = {***},
%  parse-units = false
%}
\end_preamble
\use_default_options false
\begin_modules
logicalmkup
sweave
\end_modules
\maintain_unincluded_children false
\language english
\language_package default
\inputencoding utf8
\fontencoding global
\font_roman "lmodern" "default"
\font_sans "lmss" "default"
\font_typewriter "lmtt" "default"
\font_math "auto" "auto"
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100 100
\font_tt_scale 100 100
\use_microtype false
\use_dash_ligatures true
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\float_placement H
\paperfontsize 11
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered false
\pdf_bookmarksopen false
\pdf_bookmarksopenlevel 1
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref false
\pdf_pdfusetitle true
\papersize letterpaper
\use_geometry false
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 0
\use_package esint 1
\use_package mathdots 0
\use_package mathtools 0
\use_package mhchem 0
\use_package stackrel 0
\use_package stmaryrd 0
\use_package undertilde 0
\cite_engine natbib
\cite_engine_type authoryear
\biblio_style plainnat
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date true
\justification true
\use_refstyle 0
\use_minted 0
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 1in
\topmargin 1in
\rightmargin 1in
\bottommargin 1in
\secnumdepth 3
\tocdepth 3
\paragraph_separation skip
\defskip medskip
\is_math_indent 0
\math_numbering_side default
\quotes_style english
\dynamic_quotes 0
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Standard
\noindent
\begin_inset ERT
status open

\begin_layout Plain Layout

%% Fill in values of the arguments here, 
\end_layout

\begin_layout Plain Layout

%% If blanks are needed, must insert value " ~ "
\end_layout

\begin_layout Plain Layout

%% If comma needed inside value, wrap in {}.
\end_layout

\begin_layout Plain Layout

%% Delete secondauthor and thirdauthor if not needed
\end_layout

\begin_layout Plain Layout


\backslash
guidesetup{%
\end_layout

\begin_layout Plain Layout

  author={
\end_layout

\begin_layout Plain Layout

    lastname=Johson, 
\end_layout

\begin_layout Plain Layout

    firstname=Paul, 
\end_layout

\begin_layout Plain Layout

    affiliation=CRMDA,
\end_layout

\begin_layout Plain Layout

    email=pauljohn@ku.edu},
\end_layout

\begin_layout Plain Layout

  url={https://crmda.ku.edu/guides},
\end_layout

\begin_layout Plain Layout

  keywords={R, rockchalk, outreg, LaTeX, regression tables},
\end_layout

\begin_layout Plain Layout

  title={Regression Table Survey with outreg},
\end_layout

\begin_layout Plain Layout

  leftlogo={theme/logoleft.pdf},
\end_layout

\begin_layout Plain Layout

  rightlogo={theme/logo-vert.pdf},
\end_layout

\begin_layout Plain Layout

  number=49,
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout


\backslash
guidehdr
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% tmpout directory must exist first
\end_layout

\begin_layout Plain Layout

<<tmpout, echo=FALSE, include=FALSE, results=hide>>=
\end_layout

\begin_layout Plain Layout

if(!dir.exists("tmpout")) dir.create("tmpout", showWarnings=FALSE)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% In document Latex options:
\end_layout

\begin_layout Plain Layout


\backslash
fvset{listparameters={
\backslash
setlength{
\backslash
topsep}{0em}}}
\end_layout

\begin_layout Plain Layout


\backslash
SweaveOpts{prefix.string=tmpout/t, split=F, ae=FALSE, height=3, width=5}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<Roptions, echo=FALSE, include=FALSE, results=hide>>=
\end_layout

\begin_layout Plain Layout

opts.orig <- options()
\end_layout

\begin_layout Plain Layout

options(width=100, prompt=" ", continue="  ")
\end_layout

\begin_layout Plain Layout

options(useFancyQuotes = FALSE) 
\end_layout

\begin_layout Plain Layout

set.seed(12345)
\end_layout

\begin_layout Plain Layout

options(SweaveHooks=list(fig=function() par(ps=10)))
\end_layout

\begin_layout Plain Layout

pdf.options(onefile=FALSE,family="Times",pointsize=10)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=FALSE,results=hide>>=
\end_layout

\begin_layout Plain Layout

fn1 <- 
\begin_inset Quotes eld
\end_inset

theme/logoleft.pdf
\begin_inset Quotes erd
\end_inset


\end_layout

\begin_layout Plain Layout

fn2 <- 
\begin_inset Quotes eld
\end_inset

theme/logo-vert.pdf
\begin_inset Quotes erd
\end_inset


\end_layout

\begin_layout Plain Layout

if(!file.exists(
\begin_inset Quotes eld
\end_inset

theme
\begin_inset Quotes erd
\end_inset

)) dir.create(
\begin_inset Quotes eld
\end_inset

theme
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

blankpdf <- function(file, height=3, width=3, pointsize=20, replace = FALSE){
\end_layout

\begin_layout Plain Layout

    if(file.exists(file) && !replace) return(TRUE)
\end_layout

\begin_layout Plain Layout

    pdf(file=file, width=width, height=height, paper=
\begin_inset Quotes erd
\end_inset

special
\begin_inset Quotes erd
\end_inset

, 
\end_layout

\begin_layout Plain Layout

        onefile=FALSE, pointsize=pointsize)
\end_layout

\begin_layout Plain Layout

    par(mar=c(1,1,1,1))
\end_layout

\begin_layout Plain Layout

    plot(1:2, 1:2, type = 
\begin_inset Quotes eld
\end_inset

n
\begin_inset Quotes erd
\end_inset

, axes=FALSE, xlab=
\begin_inset Quotes erd
\end_inset


\begin_inset Quotes erd
\end_inset

, ylab=
\begin_inset Quotes erd
\end_inset


\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

    ##text(1.5, 1.5, 
\begin_inset Quotes eld
\end_inset

left
\backslash
n logo
\begin_inset Quotes erd
\end_inset

, axes=FALSE) 
\end_layout

\begin_layout Plain Layout

    ##box(which=
\begin_inset Quotes erd
\end_inset

plot
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

    dev.off()
\end_layout

\begin_layout Plain Layout

    if(file.exists(file)) TRUE else FALSE
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

blankpdf(fn1)
\end_layout

\begin_layout Plain Layout

blankpdf(fn2)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

%The listings class used here allows within-document style
\end_layout

\begin_layout Plain Layout

%changes.
 R input boxes are governed by 
\end_layout

\begin_layout Plain Layout

% "Rsize", "Rbackground" and "Rcolor", while R output boxes depend on
\end_layout

\begin_layout Plain Layout

% "Routsize", "Routbackground", and "Routcolor".
 Colors
\end_layout

\begin_layout Plain Layout

% can be specified in many ways, as shown here
\end_layout

\begin_layout Plain Layout

%
\backslash
def
\backslash
Rsize{
\backslash
huge
\backslash
ttfamily}
\end_layout

\begin_layout Plain Layout

%
\backslash
def
\backslash
Routsize{
\backslash
huge}
\end_layout

\begin_layout Plain Layout

%
\backslash
def
\backslash
Rbackground{
\backslash
color[gray]{0.90}}
\end_layout

\begin_layout Plain Layout

%
\backslash
def
\backslash
Routbackground{
\backslash
color[gray]{0.40}}
\end_layout

\begin_layout Plain Layout

%
\backslash
def
\backslash
Rcolor{
\backslash
color[gray]{0.60}
\end_layout

\begin_layout Plain Layout

%
\backslash
def
\backslash
Routcolor{
\backslash
color[rgb]{0.9, 0.1, 0.1}]}
\end_layout

\begin_layout Plain Layout

%
\backslash
def
\backslash
Rcommentcolor{
\backslash
color{green}}
\end_layout

\end_inset


\end_layout

\begin_layout Abstract
\noindent
This is a comprehensive demonstration of the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
outreg()
\end_layout

\end_inset

 function in 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
rockchalk
\end_layout

\end_inset

.
 This vignette demonstrates the ability to integrate model estimates, to
 assign labels for models and variables, and to customize the selection
 of summary statistics.
 A major innovation in recent versions is the introduction of the ability
 to generate 
\begin_inset Quotes eld
\end_inset

decimal-centered
\begin_inset Quotes erd
\end_inset

 columns.
 This has been the most widely requested feature for a while.
\end_layout

\begin_layout Standard
\begin_inset CommandInset toc
LatexCommand tableofcontents

\end_inset


\end_layout

\begin_layout Standard
The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
rockchalk
\end_layout

\end_inset

 package for R 
\begin_inset CommandInset citation
LatexCommand citep
key "RCore"
literal "true"

\end_inset

 was created to facilitate authors in stats classes who need to make presentable
 tables and graphs for regression models.
 It does not aspire to be a comprehensive table-maker, mainly because R
 package writers are not consistent in their methodology.
 Procedures called 
\begin_inset Quotes eld
\end_inset

regression
\begin_inset Quotes erd
\end_inset

 within R's base have a certain internal consistency, but user-contributed
 packages do not.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
outreg
\end_layout

\end_inset

 function works for regression models that I use regularly, usually in the
 base of R, like 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
lm
\end_layout

\end_inset

, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
glm
\end_layout

\end_inset

, and for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
lme4
\end_layout

\end_inset

 objects.
 Other regression packages may work.
 I've tried to be somewhat general in the design.
\end_layout

\begin_layout Standard
\noindent
The major new feature is 
\begin_inset Quotes eld
\end_inset

decimal-centered
\begin_inset Quotes erd
\end_inset

 columns.
 In the implementation of centering, we tested approaches based on two LaTeX
 packages, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
dcolumn
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
siunitx
\end_layout

\end_inset

.
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
dcolumn
\end_layout

\end_inset

 output it is mostly adequate, but it leaves problems in column headers
 and non-numeric content alignment.
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
siunitx
\end_layout

\end_inset

 is a much more massive framework.
 There is much more danger of outright failure because of the immensity
 of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
siunitx
\end_layout

\end_inset

 itself.
 However, at the current time, all of the examples we have tested indicate
 that the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
siunitx
\end_layout

\end_inset

-based tables are superior.
 there are hopeful signs it can deal with some very tough examples.
 See Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex16s}
\end_layout

\end_inset

 I have a solution using 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
siunitx
\end_layout

\end_inset

 for a troublesome table that failed badly with 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
dcolumn
\end_layout

\end_inset

 (Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex16d}
\end_layout

\end_inset

).
 The user can designate which method should be used by assigning the parameter
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
centering = "dcolumn"
\end_layout

\end_inset

 or 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
centering = "siunitx"
\end_layout

\end_inset

.
 NOTE: it is necessary for the user to insert either 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent

\backslash
usepackage{dcolumn}
\end_layout

\end_inset

 or 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent

\backslash
usepackage{siunitx}
\end_layout

\end_inset

 in the preamble! The aim is for this whole thing to 
\begin_inset Quotes eld
\end_inset

just work
\begin_inset Quotes erd
\end_inset

 without any more attention than that.
 Please let me know if there is trouble.
\end_layout

\begin_layout Section
Briefly: outreg()
\end_layout

\begin_layout Standard
First, run regression models.
 Second, give the fitted regression object(s) to 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
outreg
\end_layout

\end_inset

.
 When 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
outreg
\end_layout

\end_inset

 runs, it will have 2 effects.
 First, it will write out LaTeX code to the screen (unless 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
print.results=FALSE
\end_layout

\end_inset

).
 A user might 
\begin_inset Quotes eld
\end_inset

copy/paste
\begin_inset Quotes erd
\end_inset

 that code into a LaTeX document.
 (Or, write it in a file and import it in the future.
 (Or, as in the case of this Sweaved document, the output goes directly
 into the result file.)
\end_layout

\begin_layout Section
Briefly: table vs tabular
\end_layout

\begin_layout Standard
This is the most confusing thing for LaTeX beginners.
 
\emph on
Put bluntly, a 
\begin_inset Quotes eld
\end_inset

table
\begin_inset Quotes erd
\end_inset

 is not a table
\emph default
.
 
\end_layout

\begin_layout Standard
Terminology
\end_layout

\begin_layout Description
tabular In LaTeX, a 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout
tablular
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 object is a grid, a display like a 
\begin_inset Quotes eld
\end_inset

spreadsheet
\begin_inset Quotes erd
\end_inset

.
 You'd call that a 
\begin_inset Quotes eld
\end_inset

table
\begin_inset Quotes erd
\end_inset

, I believe.
\end_layout

\begin_layout Description
table In LaTeX, a 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout
table
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 is something else.
 It is a document 
\begin_inset Quotes eld
\end_inset

subsection
\begin_inset Quotes erd
\end_inset

 that 
\begin_inset Quotes eld
\end_inset

floats
\begin_inset Quotes erd
\end_inset

 around in the document.
 It is a container.
 A tabular is placed inside one of these table containers.
 Table objects are numbered, can be used in cross references (if they have
 labels)
\end_layout

\begin_layout Standard
If we are writing LaTeX code by hand, a tabular inside a table will look
 like 
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout


\backslash
begin{table}
\end_layout

\begin_layout Plain Layout

    
\backslash
caption{The title of the floating object is specified in caption}
\end_layout

\begin_layout Plain Layout

    
\backslash
label{tab:ex1}% cross referencing use 
\backslash
ref{tab:ex1}
\end_layout

\begin_layout Plain Layout


\backslash
begin{tabular}
\end_layout

\begin_layout Plain Layout

  ...code to create tablar defined here
\end_layout

\begin_layout Plain Layout


\backslash
end{tabular}
\end_layout

\begin_layout Plain Layout


\backslash
end{table}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The big question is 
\begin_inset Quotes eld
\end_inset

should my outreg function write the tabular only, or also the table container
\begin_inset Quotes erd
\end_inset

?
\end_layout

\begin_layout Standard
I used to think the only reasonable answer was 
\begin_inset Quotes eld
\end_inset

tabular only
\begin_inset Quotes erd
\end_inset

, but now I see reasons why we might want to do both.
 So it is an option, as explained next.
\end_layout

\begin_layout Subsection*
rockchalk lets you choose, float or no float
\end_layout

\begin_layout Standard
The rockchalk outreg function has a parameter, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
float=FALSE
\end_layout

\end_inset

 or 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
float=TRUE
\end_layout

\end_inset

, to determine whether the output should include the floating table part
 along with the tabular object.
 If outreg users specify the argument title, then they are implicitly setting
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
float=TRUE
\end_layout

\end_inset

 and the value for title is used as the LaTeX caption.
 
\end_layout

\begin_layout Standard
The default for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
float
\end_layout

\end_inset

 is 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
FALSE
\end_layout

\end_inset

, because I have usually preferred to control the floating table from my
 document.
 However, I recognize that many people who are better at LaTeX than I am
 disagree, they want to write the table and tabular structures into one
 file.
 
\end_layout

\begin_layout Subsection*
Why would a user prefer float=TRUE?
\end_layout

\begin_layout Standard
If you are not using LyX, then it may be convenient to let float=TRUE so
 the floating table can be handled from the outreg function.
 outreg allows the author to specify the caption and the label.
 However, I did not allow for any more fine-grained customizations, especially
 table placement details.
\end_layout

\begin_layout Subsection*
Why I don't generally use float=TRUE
\end_layout

\begin_layout Standard
LaTeX has options to control the placement of floating table structures
 that the rockchalk outreg function does not adjust.
 I think that if the LaTeX author wants to adjust those things, it is easier
 for the author to control them in the document itself, rather than the
 outreg function that writes a tabular object.
 
\end_layout

\begin_layout Standard
In LyX, the user interface has a pull down menu to create floating objects
 and the GUI includes a feature to set the title (sorry, the 'caption'),
 and it also includes a way to set a label.
 If I use this method, then LyX is aware of this thing and the LyX cross-referen
cing system is available.
 
\end_layout

\begin_layout Subsection*
Another reason to use float=FALSE
\end_layout

\begin_layout Standard
In an instructional document using Sweave, the code chunks will print out
 where they 
\begin_inset Quotes eld
\end_inset

are
\begin_inset Quotes erd
\end_inset

 in the input file.
 
\end_layout

\begin_layout Standard
In LyX, there are some cross-referencing tools.
 It is more desirable to create the table container in LyX, so then cross-refere
nces work correctly.
 If one creates a table (floating container), and then puts the tabular-creating
 code chunk inside, then the code will print out with the table.
 And cross references succeed using the LyX pull down menus.
 That is demonstrated in Tables 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex11}
\end_layout

\end_inset

, 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex11t}
\end_layout

\end_inset

, 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex8}
\end_layout

\end_inset

, 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex17}
\end_layout

\end_inset

.
\end_layout

\begin_layout Section
Torture test of 
\begin_inset Quotes eld
\end_inset

rockchalk
\begin_inset Quotes erd
\end_inset

 outreg function
\end_layout

\begin_layout Standard
In most of these examples, I've taken the one-step 
\begin_inset Quotes eld
\end_inset

float=TRUE
\begin_inset Quotes erd
\end_inset

 option, but I have some examples where I've manually created the LaTeX
 float and placed an outreg tabular inside it.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

set.seed(2134234)
\end_layout

\begin_layout Plain Layout

dat <- data.frame(x1 = rnorm(100), x2 = rnorm(100))
\end_layout

\begin_layout Plain Layout

dat$y1 <- 30 + 5 * rnorm(100) + 3 * dat$x1 + 4 * dat$x2
\end_layout

\begin_layout Plain Layout

dat$y2 <- rnorm(100) + 5 * dat$x2
\end_layout

\begin_layout Plain Layout

m1 <- lm(y1 ~ x1, data = dat)
\end_layout

\begin_layout Plain Layout

m2 <- lm(y1 ~ x2, data = dat)
\end_layout

\begin_layout Plain Layout

m3 <- lm(y1 ~ x1 + x2, data = dat)
\end_layout

\begin_layout Plain Layout

gm1 <- glm(y1 ~ x1, family = Gamma, data = dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
tight and wide styles for regression tables
\end_layout

\begin_layout Standard
In my terminology, a tight table has one 
\begin_inset Quotes eld
\end_inset

narrow
\begin_inset Quotes erd
\end_inset

 column for a regression (parameter and standard error stacked on top of
 each other).
 A table that is not in the tight format is wide, it has 2 columns for each
 table (parameter and standard error side by side).
\end_layout

\begin_layout Standard
A tight table is the default, it seems to be what many political science,
 sociology, and economics students will need.
 See Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex1}
\end_layout

\end_inset

.
 A wide format table is, in my opinion, more pleasant for the eyes and seems
 to be more popular in psychology.
 See Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex1w}
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ex1, echo=F, results=tex>>=
\end_layout

\begin_layout Plain Layout

library(rockchalk)
\end_layout

\begin_layout Plain Layout

vl <- c(
\begin_inset Quotes eld
\end_inset

(Intercept)
\begin_inset Quotes erd
\end_inset

 = 
\begin_inset Quotes eld
\end_inset

Intercept
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

ex1 <- outreg(m1, title = "My One Tightly Printed Regression (uncentered)",
\end_layout

\begin_layout Plain Layout

                    label = 
\begin_inset Quotes eld
\end_inset

tab:ex1
\begin_inset Quotes erd
\end_inset

,
\end_layout

\begin_layout Plain Layout

                    float = TRUE, print.results = FALSE, varLabels=vl)
\end_layout

\begin_layout Plain Layout

# cat that, don't print it
\end_layout

\begin_layout Plain Layout

cat(ex1)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ex1w,results=tex>>=
\end_layout

\begin_layout Plain Layout

library(rockchalk)
\end_layout

\begin_layout Plain Layout

ex1w <- outreg(m1, title = "My Wide Format 
\backslash

\begin_inset Quotes erd
\end_inset

side-by-side
\backslash

\begin_inset Quotes erd
\end_inset

 columns (uncentered)", label = 
\begin_inset Quotes eld
\end_inset

tab:ex1w
\begin_inset Quotes erd
\end_inset

, tight = FALSE, float = TRUE, print.results = FALSE)
\end_layout

\begin_layout Plain Layout

cat(ex1w)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection*
Decimal-centered test case 1
\end_layout

\begin_layout Standard
A decimal-centered version of the tight column format can be seen in Table
 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex2d}
\end_layout

\end_inset

 and the decimal-centered version of the wide format is seen in Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex2wd}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ex1d, results=tex>>=
\end_layout

\begin_layout Plain Layout

library(rockchalk)
\end_layout

\begin_layout Plain Layout

ex2d <- outreg(m1, title = 'Tight column with centering = 
\begin_inset Quotes eld
\end_inset

dcolumn
\begin_inset Quotes erd
\end_inset

', label = 
\begin_inset Quotes eld
\end_inset

tab:ex2d
\begin_inset Quotes erd
\end_inset

, centering = 
\begin_inset Quotes eld
\end_inset

dcolumn
\begin_inset Quotes erd
\end_inset

, float = TRUE, print.results=FALSE)
\end_layout

\begin_layout Plain Layout

cat(ex2d)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ex1s, results=tex>>=
\end_layout

\begin_layout Plain Layout

library(rockchalk)
\end_layout

\begin_layout Plain Layout

ex2s <- outreg(m1, title = 'Tight column with centering = 
\begin_inset Quotes eld
\end_inset

siunitx
\begin_inset Quotes erd
\end_inset

', label = 
\begin_inset Quotes eld
\end_inset

tab:ex2s
\begin_inset Quotes erd
\end_inset

, centering = "siunitx", float = TRUE)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ex2wd, results=tex>>=
\end_layout

\begin_layout Plain Layout

library(rockchalk)
\end_layout

\begin_layout Plain Layout

ex1wd <- outreg(m1, title = 'Wide (not tight) format with centering = 
\begin_inset Quotes eld
\end_inset

dcolumn"',  label = 
\begin_inset Quotes eld
\end_inset

tab:ex2wd
\begin_inset Quotes erd
\end_inset

, tight = FALSE, centering = 
\begin_inset Quotes erd
\end_inset

dcolumn
\begin_inset Quotes erd
\end_inset

,  float = TRUE, print.results = FALSE)
\end_layout

\begin_layout Plain Layout

cat(ex1wd)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ex2ws, results=tex>>=
\end_layout

\begin_layout Plain Layout

ex1ws <- outreg(m1, title = 'Wide (not tight) format with centering = 
\begin_inset Quotes eld
\end_inset

siunitx"',  label = 
\begin_inset Quotes eld
\end_inset

tab:ex2ws
\begin_inset Quotes erd
\end_inset

, tight = FALSE, centering = 
\begin_inset Quotes erd
\end_inset

siunitx
\begin_inset Quotes erd
\end_inset

, float = TRUE, print.results = FALSE)
\end_layout

\begin_layout Plain Layout

cat(ex1ws)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection*
alpha level adjustments
\end_layout

\begin_layout Standard
In Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex2p}
\end_layout

\end_inset

, I demonstrate that the alpha parameter can be used to select different
 values for the critical region.
 In Tables 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex2pd}
\end_layout

\end_inset

 and 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex2ps}
\end_layout

\end_inset

, the centered-with-dcolumn and centered-with-siunitx are presented.
 I see some trouble here because the stars are not evenly spaced in the
 dcolumn-based Table 
\begin_inset ERT
status open

\begin_layout Plain Layout

{
\backslash
ref{tab:ex2pd}}
\end_layout

\end_inset

.
 However, the siunitx based Table 
\begin_inset ERT
status open

\begin_layout Plain Layout

{
\backslash
ref{tab:ex2ps}}
\end_layout

\end_inset

 may be adequate.
 This is one of the reasons I now 
\emph on
strongly
\emph default
 lean toward use of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
siunitx
\end_layout

\end_inset

 if centering is desired.
 Several alpha adjustments are scattered about in the examples in this essay,
 just to make sure they work.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex2p <- outreg(list("Fingers" = m1), tight = FALSE, title = "Ability to
 change p values (not centered)", label = 
\begin_inset Quotes eld
\end_inset

tab:ex2p
\begin_inset Quotes erd
\end_inset

,  float = TRUE, alpha = c(0.1, 0.05, 0.01)) 
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex2pd <- outreg(list("Fingers" = m1), tight = FALSE,  title = "Ability to
 change p values (dcolumn)", label = 
\begin_inset Quotes eld
\end_inset

tab:ex2pd
\begin_inset Quotes erd
\end_inset

, centering = 
\begin_inset Quotes eld
\end_inset

dcolumn
\begin_inset Quotes erd
\end_inset

, float = TRUE, alpha = c(0.1, 0.05, 0.01)) 
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex2ps <- outreg(list("Fingers" = m1), tight = FALSE,  title = "Ability to
 change p values (siunitx)", label = 
\begin_inset Quotes eld
\end_inset

tab:ex2ps
\begin_inset Quotes erd
\end_inset

, centering = 
\begin_inset Quotes eld
\end_inset

siunitx
\begin_inset Quotes erd
\end_inset

, float = TRUE, alpha = c(0.1, 0.05, 0.01)) 
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection*
Several models in same table
\end_layout

\begin_layout Standard
One of the most valuable features of rockchalk is that one can align several
 models side by side and compare them.
 The rows are matched by the variable name.
\end_layout

\begin_layout Standard
About model names: If a list of regression fits is not named, then the names
 will be bland, 
\begin_inset Quotes eld
\end_inset

M1
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

M2
\begin_inset Quotes erd
\end_inset

.
 
\end_layout

\begin_layout Standard
It is highly recommended that authors should name the regression models.
 
\end_layout

\begin_layout Standard
Currently, I recommend that we provide the model names 
\series bold
in
\series default
 the list that provides the fitted regressions, as you see here.
 That produces Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex3}
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex3 <- outreg(list("Model A" = m1, "Model B has a longer heading" = m2),
 varLabels = list(x1 = "Billie"),   title = "My Two Linear Regressions (uncenter
ed)", label = 
\begin_inset Quotes eld
\end_inset

tab:ex3
\begin_inset Quotes erd
\end_inset

, request = c(fstatistic = "F"),  print.results = FALSE)
\end_layout

\begin_layout Plain Layout

cat(ex3)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
In the original rockchalk, I had it differently.
 It is possible to provide model names in a separate argument, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
modelLabels
\end_layout

\end_inset

.
 That is shown in the code below, and in output Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex3b}
\end_layout

\end_inset

.
 The modelLabels parameter takes precedence, it will replace the labels
 in the first argument.
 However, I found it confusing to write tables that way, so I made it work
 the other way too.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex3b <- outreg(list("Model A" = m1, "Model B" = m2),   modelLabels = c("Override
s ModelA", "Overrides ModelB"), varLabels = list(x1 = "Billie"), title =
 "Note modelLabels Overrides model names (uncentered)",  label = 
\begin_inset Quotes eld
\end_inset

tab:ex3b
\begin_inset Quotes erd
\end_inset


\end_layout

\begin_layout Plain Layout

)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The column-centered versions are in Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex3bd}
\end_layout

\end_inset

 and Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex3bs}
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex3bd <- outreg(list("Model A" = m1, "Model B" = m2), modelLabels = c("Overrides
 ModelA", "Overrides ModelB"), varLabels = list(x1 = "Billie"), title =
 "Note modelLabels Overrides model names (dcolumn)", label = 
\begin_inset Quotes eld
\end_inset

tab:ex3bd
\begin_inset Quotes erd
\end_inset

, centering = 
\begin_inset Quotes eld
\end_inset

dcolumn
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex3bs <- outreg(list("Model A" = m1, "Model B" = m2), modelLabels = c("Overrides
 ModelA", "Overrides ModelB"),  varLabels = list(x1 = "Billie"), title =
 "Note modelLabels Overrides model names (siunitx)", label = 
\begin_inset Quotes eld
\end_inset

tab:ex3bs
\begin_inset Quotes erd
\end_inset

, centering = 
\begin_inset Quotes eld
\end_inset

siunitx
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection*
Specifying additional summary information
\end_layout

\begin_layout Standard
In the first version, I was thinking that everybody would be happy enough
 if the table included N, the standard error of the regression (which I
 dubbed RMSE in the old SAS style), and R-square.
 There were requests for other summaries.
 
\end_layout

\begin_layout Standard
At first, I was thinking I needed to add arguments for each request.
 I created argument 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout
showAIC=TRUE
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 to request Akaike's information criterion.
 For example, see Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex5d}
\end_layout

\end_inset

 produced by the following
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ex5d, results=tex>>=
\end_layout

\begin_layout Plain Layout

ex5d <- outreg(list("Whichever" = m1, "Whatever" = m2), title = "Still have
 showAIC argument (uncentered)", label = 
\begin_inset Quotes eld
\end_inset

tab:ex5d
\begin_inset Quotes erd
\end_inset

, showAIC = TRUE, float = TRUE)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The column-centered versions are in Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex5dd}
\end_layout

\end_inset

 and 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex5ds}
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ex5dd, results=tex>>=
\end_layout

\begin_layout Plain Layout

ex5dd <- outreg(list("Whichever" = m1, "Whatever" = m2),  title = "Still
 have showAIC argument (dcolumn)", label = 
\begin_inset Quotes eld
\end_inset

tab:ex5dd
\begin_inset Quotes erd
\end_inset

, showAIC = TRUE, float = TRUE, centering = 
\begin_inset Quotes eld
\end_inset

dcolumn
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ex5ds, results=tex>>=
\end_layout

\begin_layout Plain Layout

ex5ds <- outreg(list("Whichever" = m1, "Whatever" = m2), title = "Still
 have showAIC argument (siunitx)", label = 
\begin_inset Quotes eld
\end_inset

tab:ex5ds
\begin_inset Quotes erd
\end_inset

, showAIC = TRUE, float = TRUE, centering = 
\begin_inset Quotes eld
\end_inset

siunitx
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
However, I anticipated that way might lead me down a bad path of writing
 a parameter for every possible summary statistic.
\end_layout

\begin_layout Standard
My first idea was to create a recipe book of particular summary items and
 make them available for requests by users.
 For example, I wrote a customized reporter for F statistics and a parameter
 to ask for that was called 
\begin_inset Quotes eld
\end_inset

request
\begin_inset Quotes erd
\end_inset

.
 For example, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
request = c(fstatistic = "F")
\end_layout

\end_inset

 asks for my special fstatistic and the label to be used for it in the table
 would be 
\begin_inset Quotes eld
\end_inset

F
\begin_inset Quotes erd
\end_inset

.
 I ended up not making very many of those fancy object, but in the semTable
 in the kutils package there is a similar approach for the model Chi-Square
 and now it seems to me I should come back and do more like that in outreg.
\end_layout

\begin_layout Standard
Because I expected that I would never have time to keep up with requests
 for specialized summary items, I created a 
\begin_inset Quotes eld
\end_inset

back door
\begin_inset Quotes erd
\end_inset

 though which users could use functions and include them in the summary.
 
\end_layout

\begin_layout Standard
However, I shot myself in the foot by creating that general purpose ability
 with a horrible, ungainly name for the function's argument, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
runFuns
\end_layout

\end_inset

.
 I admit that is a horrible name, but I had good intentions.
 
\begin_inset Quotes eld
\end_inset

runFuns
\begin_inset Quotes erd
\end_inset

 is short for 
\begin_inset Quotes eld
\end_inset

run functions
\begin_inset Quotes erd
\end_inset

.
 It will run a function named as the first argument and then label it with
 the right hand side argument.
 So, if one has a lot of request like AIC, BIC, and so forth, the R functions
 can be used without too much effort.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ex6d,results=tex>>=
\end_layout

\begin_layout Plain Layout

ex6d <- outreg(list("Whatever" = m1, "Whatever" =m2), title = "Another way
 to get AIC output", label=
\begin_inset Quotes erd
\end_inset

ex6d
\begin_inset Quotes erd
\end_inset

,  runFuns = c("AIC" = "Akaike IC"), centering = "dcolumn", print.results=FALSE)
\end_layout

\begin_layout Plain Layout

cat(ex6d)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection*
Insert more regressions in one table
\end_layout

\begin_layout Standard
This code produces Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex7}
\end_layout

\end_inset

, which is NOT decimal aligned
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex7 <- outreg(list("Amod" = m1, "Bmod" = m2, "Gmod" = m3), title = "My Three
 Linear Regressions", label=
\begin_inset Quotes erd
\end_inset

tab:ex7
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The column-aligned version of the same is found in Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex7d}
\end_layout

\end_inset

, produced by the following code.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex7d <- outreg(list("Amod" = m1, "Bmod" = m2, "Gmod" = m3), centering =
 "dcolumn", title = "My Three Linear Regressions (decimal aligned)", label=
\begin_inset Quotes erd
\end_inset

tab:ex7d
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I worried that users who are verbose might break the function.
 In Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex11}
\end_layout

\end_inset

, I show that some very long names, even ones with periods, do not seem
 to cause horrible trouble.
 They WILL run off the edge of the page if they get much longer.
 Author will need to change to the tight columns if that is an issue, as
 seen in Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex11t}
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Stress test very long titles (uncentered) 
\begin_inset CommandInset label
LatexCommand label
name "tab:ex11"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex11 <- outreg(list("I Love Long Titles" = m1, "Prefer Brevity" = m2, "Captain.
 Kirk.
 Named.
 This." = m3), tight = FALSE, float = FALSE, centering = "dcolumn")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Stress test very long titles (dcolumn) 
\begin_inset CommandInset label
LatexCommand label
name "tab:ex11t"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex11td <- outreg(list("I Love Long Titles" = m1, "Prefer Brevity" = m2,
 
\begin_inset Quotes eld
\end_inset

Captain.
 Kirk.
 Named.
 This" = m3), float = FALSE,  centering = "dcolumn")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Stress test very long titles (dcolumn) 
\begin_inset CommandInset label
LatexCommand label
name "tab:ex11ts"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex11ts <- outreg(list("I Love Long Titles" = m1, "Prefer Brevity" = m2,
 "Captain.
 Kirk.
 Named.
 This" = m3), float = FALSE, centering = "siunitx")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Subsection*
Alternative standard errors
\end_layout

\begin_layout Standard
The original rockchalk took the standard errors from the fitted model.
 A student in Brazil wrote and asked me to make it possible for the author
 to supply 
\begin_inset Quotes eld
\end_inset

robust
\begin_inset Quotes erd
\end_inset

 standard errors.
 That was a good idea.
 The code to demonstrate how to create an alternative vector of standard
 errors (in that case, Huber-White robust standard errors) will appear with
 the output in Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex8}
\end_layout

\end_inset

.
 The code is displayed there because I created the outreg table with float=FALSE
, and then I manually created a table container into which I typed the R
 code chunk.
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Robust Standard Errors (uncentered)
\begin_inset CommandInset label
LatexCommand label
name "tab:ex8"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

if (require(car)){
\end_layout

\begin_layout Plain Layout

   newSE <- sqrt(diag(car::hccm(m3)))
\end_layout

\begin_layout Plain Layout

   ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, 
\end_layout

\begin_layout Plain Layout

             "Model C w Robust SE" = m3),
\end_layout

\begin_layout Plain Layout

             SElist= list("Model C w Robust SE" = newSE))
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Robust Standard Errors (dcolumn)
\begin_inset CommandInset label
LatexCommand label
name "tab:ex8d"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

if (require(car)){
\end_layout

\begin_layout Plain Layout

   newSE <- sqrt(diag(car::hccm(m3)))
\end_layout

\begin_layout Plain Layout

   ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, 
\end_layout

\begin_layout Plain Layout

             "Model C w Robust SE" = m3),
\end_layout

\begin_layout Plain Layout

             SElist= list("Model C w Robust SE" = newSE),
\end_layout

\begin_layout Plain Layout

             centering = "dcolumn")
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Robust Standard Errors (siunitx)
\begin_inset CommandInset label
LatexCommand label
name "tab:ex8s"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

if (require(car)){
\end_layout

\begin_layout Plain Layout

   newSE <- sqrt(diag(car::hccm(m3)))
\end_layout

\begin_layout Plain Layout

   ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, 
\end_layout

\begin_layout Plain Layout

             "Model C w Robust SE" = m3),
\end_layout

\begin_layout Plain Layout

             SElist= list("Model C w Robust SE" = newSE),
\end_layout

\begin_layout Plain Layout

             centering = "siunitx")
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Subsection*
Aligning different kinds of fits
\end_layout

\begin_layout Standard
The output from lm and glm fits may sometimes be usefully compared.
 The parameter display from rockchalk will align same-named variables.
 The difference in the available summary statistics is apparent because
 the rows do not 
\begin_inset Quotes eld
\end_inset

line up
\begin_inset Quotes erd
\end_inset

.
 See Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex13}
\end_layout

\end_inset

 for the output from the following.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex13 <- outreg(list("OLS" = m1, "GLM" = gm1), float = TRUE,
\end_layout

\begin_layout Plain Layout

               title = 
\begin_inset Quotes eld
\end_inset

OLS and Logit in same table (dcolumn)
\begin_inset Quotes erd
\end_inset

, 
\end_layout

\begin_layout Plain Layout

               label=
\begin_inset Quotes erd
\end_inset

tab:ex13
\begin_inset Quotes erd
\end_inset

, alpha = c(0.05, 0.01), centering = "dcolumn")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
As seen in Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex14}
\end_layout

\end_inset

, we check if the supplemental parameter requests hold up with dcolumn
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex14 <- outreg(list(OLS = m1, GLM = gm1), float = TRUE,
\end_layout

\begin_layout Plain Layout

         title = 
\begin_inset Quotes eld
\end_inset

OLS and Logit with summary report features (dcolumn)
\begin_inset Quotes erd
\end_inset

,
\end_layout

\begin_layout Plain Layout

         label = 
\begin_inset Quotes eld
\end_inset

tab:ex14
\begin_inset Quotes erd
\end_inset

,
\end_layout

\begin_layout Plain Layout

         request = c(fstatistic = "F"), runFuns = c("BIC" = "BIC"),
\end_layout

\begin_layout Plain Layout

         centering = "dcolumn")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
What if the number of digits is dialed up and alpha is altered? See Table
 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex15}
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex15 <- outreg(list(OLS = m1, GLM = gm1), float = TRUE,
\end_layout

\begin_layout Plain Layout

         title=
\begin_inset Quotes erd
\end_inset

OLS and GLM with more digits (digits)
\begin_inset Quotes erd
\end_inset

, 
\end_layout

\begin_layout Plain Layout

         label=
\begin_inset Quotes erd
\end_inset

tab:ex15
\begin_inset Quotes erd
\end_inset

, 
\end_layout

\begin_layout Plain Layout

         request = c(fstatistic = "F"), runFuns = c("BIC" = "BIC"),
\end_layout

\begin_layout Plain Layout

         digits = 5, alpha = c(0.01), centering = "dcolumn")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
In Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex16d}
\end_layout

\end_inset

, output shows result when several runFuns are requested.
 Again, I'm very sorry that parameter name is so awful.
 I adjusted the alpha stars as well.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex16d <- outreg(list("OLS 1" = m1, "OLS 2" = m2,  GLM = gm1), float = TRUE,
\end_layout

\begin_layout Plain Layout

           title = 
\begin_inset Quotes eld
\end_inset

2 OLS and 1 Logit (dcolumn), additional runFuns
\begin_inset Quotes erd
\end_inset

, 
\end_layout

\begin_layout Plain Layout

           label=
\begin_inset Quotes erd
\end_inset

tab:ex16d
\begin_inset Quotes erd
\end_inset

,
\end_layout

\begin_layout Plain Layout

           request = c(fstatistic = "F"),
\end_layout

\begin_layout Plain Layout

           runFuns = c("BIC" = "BIC", 
\begin_inset Quotes eld
\end_inset

logLik
\begin_inset Quotes erd
\end_inset

 = "ll"),
\end_layout

\begin_layout Plain Layout

           digits = 5, alpha = c(0.1, 0.05, 0.01), centering = "dcolumn")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex16s <- outreg(list("OLS 1" = m1, "OLS 2" = m2,  GLM = gm1), float = TRUE,
\end_layout

\begin_layout Plain Layout

           title = 
\begin_inset Quotes eld
\end_inset

2 OLS and 1 Logit (siunitx), additional runFuns
\begin_inset Quotes erd
\end_inset

, 
\end_layout

\begin_layout Plain Layout

           label=
\begin_inset Quotes erd
\end_inset

tab:ex16s
\begin_inset Quotes erd
\end_inset

,
\end_layout

\begin_layout Plain Layout

           request = c(fstatistic = "F"),
\end_layout

\begin_layout Plain Layout

           runFuns = c("BIC" = "BIC", 
\begin_inset Quotes eld
\end_inset

logLik
\begin_inset Quotes erd
\end_inset

 = "ll"),
\end_layout

\begin_layout Plain Layout

           digits = 5, alpha = c(0.1, 0.05, 0.01), centering = "siunitx")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
After a while, I noticed that the left hand side of runFuns need not be
 in quotation marks.
 So my examples are not always consistent.
 For example, in Table 
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
ref{tab:ex17}
\end_layout

\end_inset

 I have some in quotes, some not.
 And I also show that if authors want to create redundant rows, they are
 allowed to do so (N, for example).
\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Additional test on summary stats (dcolumn)
\begin_inset CommandInset label
LatexCommand label
name "tab:ex17"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex17 <- outreg(list("Model A" = gm1, "Model B label with Spaces" = m2),
\end_layout

\begin_layout Plain Layout

         request = c(fstatistic = "F"),
\end_layout

\begin_layout Plain Layout

         runFuns = c("BIC" = "Schwarz IC", "AIC" = "Akaike IC", 
\begin_inset Quotes eld
\end_inset

logLik
\begin_inset Quotes erd
\end_inset

 = "ll",
\end_layout

\begin_layout Plain Layout

         "nobs" = "N Again?"), centering = "dcolumn")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Additional test on summary stats (siunitx)
\begin_inset CommandInset label
LatexCommand label
name "tab:ex17s"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<results=tex>>=
\end_layout

\begin_layout Plain Layout

ex17s <- outreg(list("Model A" = gm1, "Model B label with Spaces" = m2),
\end_layout

\begin_layout Plain Layout

         request = c(fstatistic = "F"),
\end_layout

\begin_layout Plain Layout

         runFuns = c("BIC" = "Schwarz IC", "AIC" = "Akaike IC", 
\begin_inset Quotes eld
\end_inset

logLik
\begin_inset Quotes erd
\end_inset

 = "ll",
\end_layout

\begin_layout Plain Layout

         "nobs" = "N Again?"), centering = "siunitx")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Section
About examples here and your long-run work flow
\end_layout

\begin_layout Standard
In this document, I'm using a lot of short cuts to make it easier to produce
 examples with and without centering.
 I'm using the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
float=TRUE
\end_layout

\end_inset

 option (or, equivalently, giving a title) often here to cut down on the
 amount of work I'm doing.
 
\end_layout

\begin_layout Standard
If you are learning how to make tables, I usually suggest doing it in a
 different way.
 I would suggest instead that authors should create an outreg output file,
 with a sequence like this
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

lm1 <- lm(...
 regression commands ...)
\end_layout

\begin_layout Plain Layout

lm1.out <- outreg(lm1, print.results=FALSE, ...parameters like float=FALSE, tight
 = FALSE ...)
\end_layout

\begin_layout Plain Layout

cat(lm1.out, file = 
\begin_inset Quotes eld
\end_inset

lm1.out.tex
\begin_inset Quotes erd
\end_inset

)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I'd usually have an output directory where the tex file would be placed,
 but aside from this detail, that's my workflow.
 Then, when I want to use that file in my document, I use the LaTeX code
 
\begin_inset Quotes eld
\end_inset


\backslash
input{lm1.out.tex}
\begin_inset Quotes erd
\end_inset

.
 
\end_layout

\begin_layout Standard
One reason for using that 2-step workflow is that the 
\emph on
automatically produced outreg table may not be exactly perfect
\emph default
.
 Perhaps a variable label does not look right.
 Personally speaking, I wish the automatic table were always perfect.
 Practically, I accept it is not.
 So I leave open the opportunity that it might need to be revised by hand.
 
\end_layout

\begin_layout Standard
\begin_inset Newpage pagebreak
\end_inset


\end_layout

\begin_layout Standard
\begin_inset CommandInset bibtex
LatexCommand bibtex
btprint "btPrintCited"
bibfiles "rockchalk"
options "bibtotoc,apalike2"

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Newpage pagebreak
\end_inset


\end_layout

\begin_layout Section*
Replication Information
\begin_inset CommandInset label
LatexCommand label
name "sec:Session-Info"

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<session, echo=F>>=
\end_layout

\begin_layout Plain Layout

sessionInfo()
\end_layout

\begin_layout Plain Layout

if(!is.null(warnings())){
\end_layout

\begin_layout Plain Layout

    print("Warnings:")
\end_layout

\begin_layout Plain Layout

    warnings()}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<opts20, include=F>>=
\end_layout

\begin_layout Plain Layout

## Don't delete this.
 It puts the interactive session options
\end_layout

\begin_layout Plain Layout

## back the way they were.
 If this is compiled within a session
\end_layout

\begin_layout Plain Layout

## it is vital to do this.
\end_layout

\begin_layout Plain Layout

options(opts.orig)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_body
\end_document

%% LyX 2.4.4 created this file.  For more info, see https://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[american,noae]{scrartcl}
\usepackage{lmodern}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage{babel}
\usepackage{url}
\usepackage{enumitem}
\usepackage{geometry}
\geometry{verbose,tmargin=1in,bmargin=1in,lmargin=1in,rmargin=1in}
\usepackage[authoryear]{natbib}
\usepackage[pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=false,pdfborder={0 0 1},backref=section,colorlinks=false]
 {hyperref}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
<<echo=F>>=
  if(exists(".orig.enc")) options(encoding = .orig.enc)
@
\newlength{\lyxlabelwidth}      % auxiliary length 
\providecommand*{\code}[1]{\texttt{#1}}
\providecommand*{\strong}[1]{\textbf{#1}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
%\VignetteIndexEntry{Rchaeology}

\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage{babel}
\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{figure}

 }{%

   \@float{figure}[#1]%

 }%

 \centering

}{%

 \end@float

}

\renewenvironment{table}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{table}

 }{%

   \@float{table}[#1]%

 }%

 \centering

%  \setlength{\@tempdima}{\abovecaptionskip}%

%  \setlength{\abovecaptionskip}{\belowcaptionskip}%

% \setlength{\belowcaptionskip}{\@tempdima}%

}{%

 \end@float

}


%\usepackage{listings}
% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true,style=Rstyle}

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\scriptsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.90}}



\usepackage{babel}

\usepackage[samesize]{cancel}



\usepackage{ifthen}



\usepackage{listings}% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, style=Rstyle}
% Added by lyx2lyx
\setlength{\parskip}{\smallskipamount}
\setlength{\parindent}{0pt}

\makeatother

\usepackage{listings}
\renewcommand{\lstlistingname}{\protect\inputencoding{latin9}Listing}

\begin{document}
\title{Rchaeology: Idioms of R Programming}
\author{Paul E. Johnson <pauljohn @ ku.edu>}

\maketitle
This document was initiated on May 31, 2012. The newest copy will
always be available at \url{https://pj.freefaculty.org/R/} and as
a vignette in the R package ``rockchalk''.
\begin{description}
\item [{Rchaeology:}] The study of R programming by investigation of R
source code. It is the effort to discern the programming strategies,
idioms, and style of R programmers in order to better communicate
with them.
\item [{Rchaeologist:}] One who practices Rchaeology.
\end{description}
These are Rcheological observations about the style and mannerisms
of R programmers in their native habitats. Almost all of the insights
here are gathered from the r-help and r-devel emails lists, the stackoverflow
website pages for R, and the R source code itself. These are lessons
from the ``school of hard knocks.''

How is this different from Rtips(\url{https://pj.freefaculty.org/R/Rtips.pdf})?
\begin{enumerate}
\item This is oriented toward programming R, rather than using R.
\item It is more synthetic, aimed more at finding ``what's right'' rather
than ``what works.''
\item It is written with Sweave (using Harrell's Sweavel style) so that
code examples work.
\end{enumerate}
Where did the ``R Style'' section go? It was removed to a separate
vignette, RStyle, in the rockchalk package.

\tableofcontents{}

<<echo=F>>=
dir.create("plots", showWarnings=F)
@

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\SweaveOpts{prefix.string=plots/plot,ae=F,height=4,width=6}

<<Roptions, echo=F>>=
options(width=100, continue="  ")
options(useFancyQuotes = FALSE) 
set.seed(12345)
pdf.options(onefile=F,family="Times",pointsize=12)
@

\section{Introduction: R Idioms.}

This vignette is about the R idioms I have learned while working on
the rockchalk package. At the current time, I don't understand all
of the R idioms that are common in the advanced R programmers' conversation,
but I am getting some traction.

There is a language gap between an R user and an R programmer. Users
write ``scripts'' that use functions from R packages. Users don't
write (many) functions. Users don't make packages. And many users
are happy to keep it that way. For users who want to become programmers,
there is usually a harsh awakening. R for development is a different
language. Well, that's wrong. It is a different dialect. 

A transitional R user will have to learn a lot of terminology and
undergo a change of paradigm. There are resources available! Everybody
should subscribe to the email lists for the R project (\url{https://www.r-project.org/mail.html}),
especially r-help (for user questions) and r-devel. Rchaeologically
speaking, that is the native habitat of the R programmers. There is
also a burgeoning collection of blogs and Web forums, perhaps most
notably the R section on StackExchange (\url{https://stackoverflow.com/questions/tagged/r}).
Excellent books have been published. As an Rcheologist, I am drawn
to the books that are written by the R insiders. \emph{S Programming},
by William Venables and Brian Ripley \citeyearpar{venables_s_2000},
is a classic. Books by pioneering developers Robert Gentleman, \emph{R
Programming for Bioinformatics} (2009), and John Chambers, \emph{Software
for Data Analysis} \citeyearpar{chambers_software_2008}, are, well,
awesome. In 2012, I started teaching R programming using Norman Matloff's,
\emph{The Art of R Programming} (2012), which I think is great and
recommend strongly (even though Professor Matloff is not one of the
R Core Team members, so far as I know). 

There are now three types of object-oriented programming in R (S3,
S4, and reference classes) and the programmer is apparently free to
select among them without prejudice. The best brief explanation of
S3 that I've found is in Friedrich Leisch's brief note about R packaging,
``Creating R Packages: A Tutorial'' (2009). One should be mindful
of the fact that R is provided with several manuals, one of which
is the \emph{R Language Definition}. I find that one difficult to
understand, and I usually can't understand it until I search through
the R source code and packages for usage examples.

I usually approach R coding in three phases. I write code that ``works'',
however tedious and slow it might be. There will be a lot of ``copy
and paste'' constructions with tedious, manual editing of commands.
Then I read through the code and look for repetitious elements, and
I re-organize to make functions that abstract that work. If there
are any stanzas that look mostly the same, except that ``x1'' is
repalced by ``x2'', I know the work is not done. Finally, I look
through the code to find ``dumb'' constructions that can be cleaned
up and completed with fewer lines. A dumb construction is one that
I would be ashamed to show to an expert R programmer.

Beginning R programmers, the ones who have never studied C or Java
or Fortran, will often become ``stuck'' in stage one of that process.
Experienced programmers with more formal training will usually have
the ``little voice'' in the back of their minds saying ``there's
a better way,'' and, unfortunately, R novices don't hear that voice.
Lacking some of the discipline imposed by those other languages, the
usual ``rambling'' R script will seem ``good enough.'' The first
step for the transitional R programmer, the one who does not want
to be a novice any more, is to take a harsh look at the code that
has already been written.

The rockchalk package has many functions that receive fitted regression
models and re-arrange their components for re-fitting or plotting.
A good deal of the advice I have to offer is about the process of
managing R formula, but I have also accumulated some lessons in argument
handling. 

\section{do.call(), eval(), substitute(), formula().}

If the transitional programmer understands these four functions, she/he
will be well on the way to escaping the category of novice R programmer.
The other benefit is that the code written by the experts, including
the R source code itself, will become much more understandable.

\subsection{Rewriting Formulas. My Introductory Puzzle.}

On May 29, 2012, I was working on a regression problem in the rockchalk
package. I have a number of functions that receive elementary regressions
and then change them. I need to receive a fitted model, extract the
formula, change some variables, and then revise the formula to match
the new variables. And then run the model over again. 

The functions meanCenter() and residualCenter() receive a fitted regression
model, transform some variables, and then fit a new regression. Suppose
a regression has been fitted as ``\code{y \textasciitilde{} x1{*}x2}''.
The R result will estimate a predictive formula such as $\hat{y}_{i}=\hat{\beta}_{0}+\hat{\beta}_{1}x1_{i}+\hat{\beta}_{2}x2_{i}++\hat{\beta}_{3}x1_{i}\cdot x2_{i}$.
meanCenter() will replace the non-centered variables ``x1'' and
``x2'' with mean-centered variables ``x1c'' and ``x2c''. The
original formula y \textasciitilde{} x1{*}x2 must be replaced with
y \textasciitilde{} x1c{*}x2c. I found this to be a very complicated
problem with a very satisfying answer. 

My first effort used R's update function. That is the most obvious
approach. I learned very quickly that update() is not sufficient.
It is fairly easy to replace x1 with x1c in the formula, but not when
x1 is logged or otherwise transformed. Here is the runable example
code that demonstrates the problem. 

<<include=T, results=hide>>=
dat <- data.frame(x1 = rnorm(100, m = 50), x2 = rnorm(100, m = 50),
  x3 = rnorm(100, m = 50), x4 = rnorm(100, m=50), y = rnorm(100))
m2 <- lm(y ~ log(x1) + x2*x3, data = dat)
suffixX <- function(fmla, x, s){
    upform <- as.formula(paste(". ~ .", "-", x, "+", paste(x, s, sep = ""), sep="", collapse=" "))
    update.formula(fmla, upform)
}
newFmla <- formula(m2)
newFmla
suffixX(newFmla, "x2", "c")
suffixX(newFmla, "x1", "c")
@

Run that and check the last few lines of the output. See how the update
misses x1 inside log(x1) or in the interaction?

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> newFmla <- formula(m2)
> newFmla
y ~ log(x1) + x2 * x3
> suffixX(newFmla, "x2", "c")
y ~ log(x1) + x3 + x2c + x2:x3
> suffixX(newFmla, "x1", "c")
y ~ log(x1) + x2 + x3 + x1c + x2:x3
\end{lstlisting}
\leavevmode\egroup

I asked the members of r-help for assistance. Lately I've had very
good luck with r-help. Gabor Grothendieck wrote an answer to r-help
on May 29, 2012. He said simply, ``Try substitute,'' with this example.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> do.call("substitute", list(newFmla, setNames(list(as.name("x1c")), "x1")))
y ~ log(x1c) + x2 * x3
\end{lstlisting}
\leavevmode\egroup

Problem solved, in a single line. 

That's very clever, I think. It packs together a half-dozen very deep
thoughts. It has most of the essential secrets of R's guts, laid out
in a single line. It has do.call(), substitute(), it interprets a
formula as a list, and it shows that every command in R is, when it
comes down to brass tacks, a list. 

I would like to take up these separate pieces in order.

\subsection{A Formula Object is a List.}

While struggling with this, I noticed this really interesting pattern.
The solution depends on it. The object ``newFmla'' is not just a
text string. It prints out as if it were text, but it is actually
an R list object. Its parts can be probed recursively, to eventually
reveal all of the individual pieces:

<<newFla10>>=
newFmla
newFmla[[1]]
newFmla[[2]]
newFmla[[3]]
newFmla[[3]][[2]]
newFmla[[3]][[2]][[2]]
@

How could I put that information to use? Read on.

\subsection{do.call() and eval()}

In my early work as an Rchaeologist, I had noticed eval() and do.call(),
but did not understand their significance. Coming to grips with these
ideas is a critical step in the R programmer's growth, because they
separate the ``script writer'' from the ``language programmer.''
Whenever difficult problems arise in r-help, the answer almost invariably
involves do.call() or eval(). 

\subsubsection{do.call()}

Let's concentrate on do.call first. The syntax is like this

\bgroup\inputencoding{latin9}
\begin{lstlisting}
do.call("someRFunction", aListOfArgumentsToGoInTheParentheses)
\end{lstlisting}
\leavevmode\egroup

It is as if we were telling R to run this:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
someRFunction(aListOfArgumentsToGoInTheParentheses)
\end{lstlisting}
\leavevmode\egroup

We use do.call() because it is much more flexible than calling someRFunction()
directly.

Let's consider an example that runs a regression the ordinary way,
and then with do.call. In this example, the role of ``someRFunction''
will be played by lm() and the list of arguments will be the parameters
of the regression. The regression m1 will be constructed the ordinary
way, while m2 is constructed with do.call().

<<>>=
m1 <- lm(y ~ x1*x2, data = dat)
coef(m1)
regargs <- list(formula = y ~ x1*x2, data = quote(dat))
m2 <- do.call("lm", regargs)
coef(m2)
all.equal(m1, m2)
@

The object regargs is a list of arguments that R can understand when
they are supplied to the lm function. 

do.call() is a powerful, mysterious symbol. It holds flexibility;
we can calculate ``on the language'' to create commands and then
run them. I first needed do.call() when we had a simulation project
that ran very slowly. There's a writeup in the working examples distributed
with rockchalk called stackListItems-01.R. I was using rbind() over
and over to join the results of simulation runs. Basically, the code
was like this

\bgroup\inputencoding{latin9}
\begin{lstlisting}
for (i in 1:10000){
   dat <- someHugeSimulation(i)
   result <- rbind(result, dat)
}
\end{lstlisting}
\leavevmode\egroup

That will call rbind() 10000 times. I had not realized that rbind()
is time-consuming. It accesses a new chunk of memory each time it
is run. On the other hand, we could collect those results in a list,
then we can call rbind() one time to smash together all of the results.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
for (i in 1:10000){
   mylist[[i]] <- someHugeSimulation(i)
}
result <- do.call("rbind", mylist)
\end{lstlisting}
\leavevmode\egroup

It is much faster to run rbind() only once. It would be OK if we typed
it all out like this:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
result <- rbind(mylist[[1]], mylist[[2]], mylist[[3]], ..., mylist[[10000])
\end{lstlisting}
\leavevmode\egroup

But who wants to do all of that typing? How tiresome! Thanks to Erik
Iverson in r-help, I understand that

\bgroup\inputencoding{latin9}
\begin{lstlisting}
result <- do.call("rbind", mylist)
\end{lstlisting}
\leavevmode\egroup

is doing the EXACT same thing. ``mylist'' is a list of arguments.
do.call is \emph{constructing} a function call from the list of arguments.
It is \emph{as if} I had actually typed rbind with 10000 arguments.

The beauty in this is that we could design a program that can assemble
the list of arguments, and also choose the function to be run, on
the fly. We are not required to literally write the function in quotes,
as in ``rbind''. We could instead have a variable that is calculated
to select one function among many, and then use do.call on that. In
a very real sense, we could write a program that can write itself
as it runs. 

From all of this (and a peek at ?call), I arrive at an Rchaeological
eureka! A call object is a quoted command plus a list of arguments
for that command. 

\subsubsection{eval()}

Where does eval() fit into the picture? do.call() manufactures a call
and executes it immediately. It is possible to arrest the process
mid stream, to capture the call without evaluating it with eval().
The terminology in the R documentation on this is difficult, mainly
because there are several different ways to create a piece of not-yet-evaluated
code. By \strong{not-yet-evaluated code}, I mean an ``unevaluated
expression'' or a ``call object'', anything that can be handled
by the eval() function. A not-yet-evaluated object holds some syntax
that can we need later. I think of do.call() as a contraction of ``eval''
and ``call''. As far as I can tell, do.call(\textquotedbl lm\textquotedbl ,
list(arguments)) is the same as eval(call(``lm'', arguments)). 

There are many ways to ask R to create the not-yet-evaluated object,
that's one of the confusing things (in my opinion). Most obviously,
call() function can be used for that. But functions like quote() and
expression() can also create not-yet-evaluated code.

Everybody has experienced an error like this:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> x1 + x2
Error: object 'x1' not found
\end{lstlisting}
\leavevmode\egroup

Whenever we use a variable, the R interpreter expects to find it.
We know those variables are in the data frame dat, but the interpreter
does not know that's what we mean. But we can ask R to trust us with
the quote() function, which, basically, means ``here's some code
we will evaluate later'':

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> mycall <- quote(x1 + x2)
\end{lstlisting}
\leavevmode\egroup

The object ``mycall'' is an R call object, something that might
be evaluated in the future. 

When we want to evaluate ``x1 + x2'', we ask R do to so:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
eval(mycall, dat)
\end{lstlisting}
\leavevmode\egroup
The second argument supplies the place where x1 and x2 are to be found.
R users may never have noticed the eval() function, but most will
have used the with() function, which is simply a clever re-phrasing
of the eval function's interface. It does the same thing:\bgroup\inputencoding{latin9}
\begin{lstlisting}
with(dat, x1 + x2)
\end{lstlisting}
\leavevmode\egroup

and so does \bgroup\inputencoding{latin9}
\begin{lstlisting}
with(dat, eval(mycall)).
\end{lstlisting}
\leavevmode\egroup

Now, back to the regression example we were working on before. We
can create the call object using the call() function, which requires
the name of a function and a comma-separated list of quoted arguments.
I quote each of the bits of syntax in the arguments because I don't
want R to replace ``dat'' with the actual data frame, I only want
it to remember the name.

<<>>=
m3 <- lm(y ~ x1*x2, data = dat)
coef(m3)
mycall <- call("lm", quote(y ~ x1*x2), data = quote(dat))
m4 <- eval(mycall)
coef(m4)
@

The main reason for using eval is that we can ``piece together''
commands and then run them after we have assembled all the pieces.
The nickname for this seems to be ``computing on the language.''
There is a discussion of it in the R Language Definition. It is also
surveyed in Hadley Wickham's new book, \emph{Advanced R} \citeyearpar{Wickham2015}.

Did you ever notice that you get an error if you run ``x1 + x2'',
but if you run ``y $\sim$ x1 + x2'', then there is no error? It
is as if R is wrapping your code in protective tissue, something like
quote(). However, the protective wrapper in this case is actually
the function called formula(). The R interpreter notices the symbol
$\sim$ in the syntax, and so it assumes you meant to run

\bgroup\inputencoding{latin9}
\begin{lstlisting}
formula(y ~ x1 + x2)
\end{lstlisting}
\leavevmode\egroup

A formula is another kind of not-yet-evaluated thing. It is something
that we can fiddle with before evaluating. 

Create a complicated formula. Note that the lm function receives that
formula object without trouble.

<<>>=
f1 <- y ~ x1 + x2 + x3 + log(x4)
class(f1)
m5 <- lm(f1, data = dat)
coef(m5)
@

The object f1 is a formula object. Its not just a text string. Observe
it has separate pieces, just like newFmla in the example problem that
started this section.

<<>>=
f1[[1]]
f1[[2]]
f1[[3]]
f1[[3]][[1]]
f1[[3]][[2]]
f1[[3]][[3]]
@\\
Note that f1 created in this way must be a syntactically valid R formula;
it cannot include any other regression options.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> f1 <- y ~ x1 + x2 + x3 + log(x4), data=dat 
Error: unexpected ',' in "f1 <- y ~ x1 + x2 + x3 + log(x4),"
\end{lstlisting}
\leavevmode\egroup

If I declare f1exp as an expression, then R does not re-interpret
it as a formula (f1exp is an unevaluated expression, the R parser
has not translated it yet). To use that as a formula in the regression,
we have to evaluate it.

<<>>=
f1exp <- expression(y ~ x1 + x2 + x3 + log(x4))
class(f1exp)
m6 <- lm(eval(f1exp), data = dat)
@

When f1exp is evaluated, what do we have? Here's the answer.

<<>>=
f1expeval <- eval(f1exp)
class(f1expeval)
all.equal(f1expeval, f1)
m7 <- lm(f1expeval, data=dat)
all.equal(coef(m6), coef(m7))	
@

The point here is that the pieces of an ordinary use command can be
separated and put back together again before the work of doing calculations
begins. We can edit the formula. Suppose we replace a part:

<<>>=
f1[[3]][[2]] <- quote(x1 + log(x2))
m8 <- lm(f1, data = dat)
coef(m8)
@

Now we turn back to the main theme. How is eval() used in functions?
Some functions take a lot of arguments. We may want to do a lot of
fine tuning on the arguments, before they are assembled and evaluated.
Hence, it is very important that we can keep some bits of R that are
symbolic, rather than calculated, until we need them. 

If you are looking for examples of eval() in the R source code, there
is a nice one in the beginning of the lm() function. Suppose a user
submits a command like ``lm(y \textasciitilde{} x, data = dat, x
= TRUE, y = TRUE).'' Inside lm(), the import is re-organized. Here
are the first lines of the lm() function

\bgroup\inputencoding{latin9}
\begin{lstlisting}[breaklines=true,numbers=left,numberstyle={\footnotesize},tabsize=4]
lm <- function (formula, data, subset, weights, na.action, method = "qr",
    model = TRUE, x = FALSE, y = FALSE, qr = TRUE, singular.ok = TRUE,
    contrasts = NULL, offset, ...)
{
    ret.x <- x
    ret.y <- y
    cl <- match.call()
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "subset", "weights", "na.action",
        "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
\end{lstlisting}
\leavevmode\egroup

Now suppose we turn on the debugger and run a regression in R with
a command like this.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
debug(lm)
m1 <- lm(y ~ x1 * x2, data = dat, x = TRUE, y = TRUE)
\end{lstlisting}
\leavevmode\egroup

After that, we are ``in'' the function, stepping through line-by-line.
In line 8, the match.call() function is used to grab a copy of the
command that I typed. We can see that mf is exactly the same as my
command, except R has named the arguments:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> mf
lm(formula = y ~ x1 * x2, data = dat, x = TRUE, y = TRUE)
\end{lstlisting}
\leavevmode\egroup

That's not just a string of letters, however. It is a call object,
a list with individual pieces that can be revised. Recall that the
first element in a call object is the name of a function, and the
following elements are the arguments. Lines 10 and 11 check the names
of mf for the presence of certain arguments, and throw away the rest.
It only wants the arguments we would be needed to run the function
model.frame. Line 12 adds an argument to the list, drop.unused.levels.
Up to that point, then, we can look at the individual pieces of mf:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> names(mf) 
[1] ""      "formula"       "data" [4] "drop.unused.levels"
> mf[[1]]
lm
> mf[[2]]
y ~ x1 * x2
> mf[[3]]
dat
> mf[[4]] 
[1] TRUE
\end{lstlisting}
\leavevmode\egroup

The object mf has separate pieces that can be revised and then evaluated.
Line 13 replaces the element 1 in mf with the symbol ``model.frame''.
That's the function that will be called. Line 14 is the coup de grâce,
when the revised call ``mf'' is sent to eval. In the end, it is
\emph{as if} lm had directly submitted the command

\bgroup\inputencoding{latin9}
\begin{lstlisting}
mf <- model.frame(y ~ x1 * x2, data = dat, drop.unused.levels = TRUE)
\end{lstlisting}
\leavevmode\egroup

It would not do to simply write that into the lm function, however,
because some people use variables that have names different from y,
x1, and x2, and their data objects may not be called dat. lm allows
users to input whatever they want for a formula and data, and then
lm takes what it needs to build a model frame.

\subsection{substitute()}

Most R users I know have not used substitute, except possibly if they
try to use plotmath. In the context of plotmath, the problem is as
follows. Plotmath causes the R plot functions to convert expressions
into mathematical symbols in a way that is reminiscent of \LaTeX .
For example, a command like this:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
text(4, 4, expression(gamma))
\end{lstlisting}
\leavevmode\egroup
will draw the gamma symbol at the position (4,4). We can use paste
to combine symbolic commands and text like so:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
text(4, 4, expression(paste(gamma, " = 7")))
\end{lstlisting}
\leavevmode\egroup
The number 7 is a nice number, but what if we want to calculate something
and insert it into the expression? Your first guess might be to insert
a function that makes a calculation, such as the mean, but this fails:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
text(4, 4, expression(paste(gamma, mean(x))))
\end{lstlisting}
\leavevmode\egroup
In order to smuggle the result of a calculation into an expression,
some fancy footwork is required. In the help page for plotmath, examples
using the functions bquote and substitute are offered. I have found
this to be quite frustrating and difficult to do, and if one examines
my collection of examples on \url{https://pj.freefaculty.org/R/WorkingExamples/},
one will find quite a few plotmath exercises.

For the particular purpose of blending expressions with calculation
results, I find the bquote function to be more easily understandable.
However, when going beyond plotmath, I expect most R programmers will
need to use substitute() instead, so I will discuss that in this section.
The plotmath help page points to syntax like this:

<<fig=T, height=3, width=4>>=
plot(1:10, seq(1,5, length.out=10), type = "n", main="Illustrating Substitute with plotmath", xlab="x", ylab="y")
text(5, 4, substitute(gamma + x1mean, list(x1mean = mean(dat$x1))))
text(5, 2, expression(paste(gamma, " plus the mean of x1")))
@

Run ?substitute and one is brought to a famous piece of Rchaeological
pottery: 
\begin{quote}
‘substitute’ returns the parse tree for the (unevaluated) expression
‘expr’, substituting any variables bound in ‘env’. 
\end{quote}
Pardon me. parse tree? We've seen expressions already, that part is
not so off putting. But ``parse tree''? Really?

This is one of those points at which being an Rchaeologist has real
benefits. We have to dig deeper, to try to understand not only what
the R programmer says, but what she is actually trying to do. The
manual page gives us some insights into the R programmer, and it is
his or her view of his or her own actions, but it doesn't necessarily
speak to how we should understand substitute(). 

For me, the only workable approach is to build up a sequence of increasingly
complicated examples. I start by creating the list of replacements.
This replacement list can have a format like this: 

<<>>=
sublist <- list(x1 = "alphabet", x2 = "zoology")
@

Suppose I have some other object in which x1 and x2 need to be replaced.
Specifically, when an expression has ``x1'', I want ``alphabet'',
and ``x2'' should become ``zoology.'' The quotes in the R code
indicate that alphabet and zoology are character strings, not other
objects that already exist. Consider replacing x1 and x2 in x1 + x2
+ log(x1) + x3:

<<>>=
substitute(expression(x1 + x2 + log(x1) + x3), sublist)
@

Note that the substitution 1) leaves other variables alone (since
they are not named in sublist) and 2) it finds all valid uses of the
symbols x1 and x2 and replaces them.

This isn't quite what I wanted, however, because the strings have
been inserted into the middle of my expression. I don't want text.
I just want symbols. I've seen the conversion from character to symbol
done with as.symbol() and with as.name(), recently I realized that
they are synonymous. I usually use as.symbol() because that name has
more intuition for me, but in Gabor's answer to my question, as.name()
is used. Using as.name(), my example would become:

<<>>=
sublist <- list(x1 = as.name("alphabet"), x2 = as.name("zoology"))
substitute(expression(x1 + x2 + log(x1) + x3), sublist)
@

\subsection{setNames and names}

Almost every R user has noticed that the elements of R lists can have
names. In a data frame, the names of the list elements are thought
of as variable names, or column names. If dat is a data frame, the
names and colnames functions return the same thing, but that's not
true for other types of objects.

<<>>=
dat <- data.frame(x1=1:10, x2=10:1, x3=rep(1:5,2), x4=gl(2,5))
colnames(dat)
names(dat)
@

After dat is created, we can change the names inside it with a very
similar approach:

<<>>=
newnames <- c("whatever","sounds","good","tome")
colnames(dat) <- newnames
colnames(dat)
@

While used interactively, this is convenient, but it is a bit tedious
because we have to create dat first, and then set the names. The setNames()
function allows us to do this in one shot. I'll paste the data frame
creating commands and the name vector in for a first try:

<<>>=
dat2 <- setNames(data.frame(x1 = rnorm(10), x2 = rnorm(10),
    x3 = rnorm(10), x4 = gl(2,5)), c("good", "names", "tough", "find"))
head(dat2, 2)
@

In order to make this more generally useful, the first step is to
take the data-frame-creating code and set it into an expression that
is not immediately evaluated (that's datcommand in what follows).
When I want the data frame to be created, I use eval(), and then the
newnames vector is put to use. 

<<>>=
newnames <- c("iVar", "uVar", "heVar", "sheVar")
datcommand <- expression(data.frame(x1=1:10, x2=10:1, x3=rep(1:5,2), x4=gl(2,5)))
eval(datcommand)
dat3 <- setNames(eval(datcommand), newnames)
@

The whole point of this exercise is that we can write code that creates
the names, and creates the data frame, and then they all come together.

What if we have just one element in a list? In Gabor's answer to my
question, there is this idiom

\bgroup\inputencoding{latin9}
\begin{lstlisting}
setNames(list(as.name("x1c")), "x1")))
\end{lstlisting}
\leavevmode\egroup

Consider this from the inside out. 
\begin{enumerate}
\item as.name(``x1c'') is an R symbol object,
\item list(as.name(``x1c'')) is a list with just one object, which is
that symbol object. 
\item Use setNames(). The object has no name! We would like to name it ``x1''. 
\end{enumerate}
It is as if we had run the command list(x1 = x1c). The big difference,
of course, is that this way is much more flexible because we can calculate
replacements. 

\subsection{The Big Finish}

In the meanCenter() function in rockchalk, some predictors are mean-centered
and their names are revised. A variable named ``age'' becomes ``agec''
or ``x1'' becomes ``x1c''. So the user's regression formula that
uses variables age or x1 must be revised. This is a function that
takes a formula ``fmla'' and replaces a symbol xname with newname.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
formulaReplace <- function(fmla, xname, newname){ 
    do.call("substitute", list(fmla, setNames(list(as.name(newname)), xname))) 
}
\end{lstlisting}
\leavevmode\egroup

This is put to use in meanCenter(). Suppose a vector of variable names
called nc (stands for ``needs centering'') has already been calculated.
The function std() creates a centered variable.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
newFmla <- mc$formula
for (i in seq_along(nc)){
    icenter <- std(stddat[, nc[i]])
    newname <- paste(as.character(nc[i]), "c", sep = "")
    newFmla <- formulaReplace(newFmla,  as.character(nc[i]), newname)
    nc[i] <- newname
}
\end{lstlisting}
\leavevmode\egroup

If one has a copy of rockchalk 1.6 or newer, the evidence of the success
of this approach should be evident in the output of the command example(meanCenter). 

\section{Make Re-Usable Tools (Rather than Cutting and Pasting)}

We seek concise solutions that are generalizable. It is almost never
right to cut and paste and then make minor edits in each copy to achieve
particular purposes. This advice goes against the grain of the graduate
students that I work with. They will almost always use cut and paste
solutions. 

Here is an example of the problem. A person needed ``dummy variables''.
The code had several pages like this:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
  if(setcorr == 1){
        corr.10 <-1
        corr.20 <-0
        corr.30 <-0
        corr.40 <-0
        corr.50 <-0
        corr.60 <-0
        corr.70 <-0
        corr.80 <-0
    }
    if(setcorr == 2){
        corr.10 <-1
        corr.20 <-1
        corr.30 <-0
        corr.40 <-0
        corr.50 <-0
        corr.60 <-0
        corr.70 <-0
        corr.80 <-0
    }
    if(setcorr == 3){
        corr.10 <-1
        corr.20 <-1
        corr.30 <-1
        corr.40 <-0
        corr.50 <-0
        corr.60 <-0
        corr.70 <-0
        corr.80 <-0
    }
\end{lstlisting}
\leavevmode\egroup

Well, that's understandable, but a little bit embarrassing. I asked
``why do you declare these separate variables, why not make a vector?''
and ``can't you see a more succinct way to declare those things?''
The answer is ``we do it that way in SAS'' or ``this runs''. One
might suspect that the author was eager to say, ``my project is based
on 10,000 lines of R code'' in order to impress a lay audience.

If the mission is to create a vector with a certain number of 1's
at the beginning, surely either of these functions would be better:

<<>>=
biVec1 <- function(n = 3, n1s = 1) {
    c(rep(1, n1s), rep(0, n - n1s))
}

biVec2 <- function(n = 3, n1s = 1){
    x <- numeric(length = n)
    x[1:n1s] <- 1
    x
}

(corr <- biVec1(n = 8, n1s = 3))
(corr <- biVec2(n = 8, n1s = 3))
@

The cut and paste way is not wrong, exactly. Its tedious. 

I think any reasonable user should want a vector, rather than the
separate variables. But suppose the user is determined, and really
wants the individual variables named corr.10, corr.20, and so forth.
We can re-design this so that those variables will be sitting out
in the workspace after the function is run. Read help(``assign'')
and try

<<>>=
biVec3 <- function(n = 3, n1s = 1) {
    X <- c(rep(1, n1s), rep(0, n - n1s))
    xnam <- paste("corr.", 1:8, "0", sep = "")
    for(i in 1:n) assign(xnam[i], X[i], envir = .GlobalEnv) 
}
ls()
biVec3(n = 8, n1s = 3)
ls()
@

I don't think most people will actually want that kind of result,
but if they do, there is a way to get it.

If we want to turn biVec1() or biVec2() into general purpose functions,
we need to start thinking about the problem of unexpected user input.
Notice what happens if we ask for more 1's than the result vector
is supposed to contain:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> biVec1(n = 3, n1s = 7)
Error in rep(0, n - n1s) (from #2) : invalid 'times' argument
> biVec2(n = 3, n1s = 7)
[1] 1 1 1 1 1 1 1
\end{lstlisting}
\leavevmode\egroup

Perhaps I prefer biVec1() because it throws an error when there is
unreasonable input, while biVec2() returns a longer vector than expected
with no error. 

If the user mistakenly enters non-integers, neither function generates
an error, and they give us unexpected output.

<<>>=
biVec1(3.3, 1.8)
biVec2(3.3, 1.8)
@

To my surprise, it is not an easy thing to ask a number if it is a
whole number (see help(``is.integer'')). For that, we introduce
a new function, is.wholenumber(), and put it to use in our new and
improved function

<<>>=
is.wholenumber <- function(x, tol = .Machine$double.eps^0.5){
 abs(x - round(x)) < tol
}

biVec1 <- function(n = 3, n1s = 1) {
    if(!(is.wholenumber(n) & is.wholenumber(n1s)))
        stop("Both n and n1s must be whole numbers (integers)")
    if(n1s > n)
        stop("n must be greater than or equal to n1s")
    c(rep(1, n1s), rep(0, n - n1s))
}
@

In all of the test cases I've tried, that works, although the real
numbers 7.0 and 4.0 are able to masquerade as integers.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> biVec1(3, 7)
Error in biVec1(3, 7) (from #4) : n must be greater than or equal to n1s
> biVec1(7, 3)
[1] 1 1 1 0 0 0 0
> biVec1(3.3, 4.4)
Error in biVec1(3.3, 4.4) (from #2) : 
  Both n and n1s must be whole numbers (integers)
> biVec1(7.0, 4.0)
[1] 1 1 1 1 0 0 0
> biVec1(7.0, 4.0)
[1] 1 1 1 1 0 0 0
\end{lstlisting}
\leavevmode\egroup


\section{Function Arguments.}

While developing functions for the rockchalk package, one of the most
important learning experiences I've had is in argument management. 

One of the reasons that it is difficult to design functions is that
there is no ``mandatory manual'' on this. The R framework allows
programmer creativity, which is a good thing. But it is also a bad
thing, since every programmer has his/her own opinion about what ought
to be done. To state the obvious, argument names should be clear,
unambiguous, convenient, short, and minimally necessary. 

Beyond that, what can we say? I urge new R programmers to study the
R source code. This is part of the Rchaeological method. Study the
most frequently used functions in the R distribution. I've learned
the most from the way arguments are handled in the functions \code{hist}(),
\code{lm()}, \code{plot()} and \code{termplot()}. These functions
have had the most eyes on them and, therefore, they are most likely
to use the best approach. I wrote several functions that eventually
went into rockchalk before I realized that important fact and I regret
it (on a weekly basis).

In this section, I offer this advice.
\begin{enumerate}
\item Protect your function's calculations from the user's workspace.
\item Check argument values.
\item Design the function so that it runs with a minimum number of arguments.
\end{enumerate}

\subsection{A few tidits about arguments}

R includes several functions that are very helpful in dealing with
arguments.
\begin{description}
\item [{missing()}] Inside a function, such as \code{plotme()} we often
need to check if the user supplied a value for an argument, such as
\code{col}. If the user did not supply a value, we might construct
one from the other input

\bgroup\inputencoding{latin9}
\begin{lstlisting}
if (missing(col)) col <- someTediousCalculation(y, z)
\end{lstlisting}
\leavevmode\egroup

This is usually good enough, but there is danger that the user might
have called the function like so:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
plotme(x, y, z, col = NULL)
\end{lstlisting}
\leavevmode\egroup

This outsmarts missing(col), which will return FALSE. If our goal
is to make sure col is something, some non-NULL thing, a more careful
argument check would be

\bgroup\inputencoding{latin9}
\begin{lstlisting}
if (missing(col) | is.null(col)) col <- someTediousCalculation(y, z)
\end{lstlisting}
\leavevmode\egroup

\item [{stop()}] Halts a function and returns a message to the user. Usage
example 

\bgroup\inputencoding{latin9}
\begin{lstlisting}
if (isTRUE(x)) stop(paste("I'm sorry, you failed to supply the right information.")))
\end{lstlisting}
\leavevmode\egroup

\item [{stopifnot()}] It is so frequent to run ``\code{if(isTRUE(x)) stop()}''
that they created a wrapper, \code{stopifnot(x)}.
\item [{warning()}] If you don't want the function to fail, but you need
to warn the user about a condition that seems dubious, run

\bgroup\inputencoding{latin9}
\begin{lstlisting}
if (isTRUE(x)) warning(paste("Are you sure the value of x ought to be", x, "?"))
\end{lstlisting}
\leavevmode\egroup

\item [{deparse(substitute())}] Inside a function, it is often important
to find out the symbolic value of the argument that the user supplied
to the function. I don't mean the \emph{value}, but rather the symbolic
name. To see what I mean, run this

<<eval=F>>=
dat <- data.frame(myx = c(1, 2, 3), myy = c(10, 5, 1))
debug(plot.default)
plot(dat$myx, dat$myy)
@

After you hit enter, the R debugger/browser will come open and the
prompt will allow you to inspect the situation. First, type ``n''
and hit return. That takes the first step into the calculations. After
that, run \code{match.call()}. The function is somewhat ``self aware''.
It knows how it was called. You should see this

\bgroup\inputencoding{latin9}
\begin{lstlisting}
Browse[2]> n 
debug: localAxiis <- function(..., col, bg, pch, cex, lty, lwd) Axis(...)
Browse[2]> match.call()
plot.default(x = dat$myx, y = dat$myy)
\end{lstlisting}
\leavevmode\egroup

You asked for \code{plot()}, R sends the work to \code{plot.default()}.
All is well.

Still inside the debugger now, type ``x'' and then ``myx'':

\bgroup\inputencoding{latin9}
\begin{lstlisting}
Browse[2]> x
[1] 1 2 3
Browse[2]> myx
Error: object 'myx' not found
\end{lstlisting}
\leavevmode\egroup

Inside the function, the vector value that we passed in, which was
referred to as dat\$myx, is now called x. The thing x is a data holder,
it is used in the following calculations. \code{x} does not remember
that its passed-in value was named \code{dat\$myx} in a previous
life. It is as if \code{x} is the new identity of the same old data
vector. However, we can ask \code{x} ``what name were you known
by in your previous life?'' with this interesting idiom.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
Browse[2]> deparse(substitute(x))
[1] "dat$myx"
\end{lstlisting}
\leavevmode\egroup

The substitute() function, as we have already seen, is a way to replace
a letter x with a value from somewhere. Maybe it would be more obvious
to run this in 2 steps.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
Browse[2]> xsubs <- substitute(x)
Browse[2]> xsubs
dat$myx
Browse[2]> deparse(xsubs)
[1] "dat$myx"
\end{lstlisting}
\leavevmode\egroup

\code{substitute()} grabs the object name from whence \code{x} was
calculated. We need to convert that back to a character string so
that we can know the symbolic name of the thing that eventually became
\code{x}. This is at the outer boundary of my R comprehension, so
I better stop here.

Incidentally, you can go exploring in the debugger. Run some standards
like \code{ls()} to see what's available. If you keep hitting return,
the function will run line by line. If you get tired of hitting return,
type ``c'' and the function will run to its conclusion. To let R
know that you don't want to debug that function any more, run:

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> undebug(plot.default)
\end{lstlisting}
\leavevmode\egroup

If you don't do that, then the function browser will pop open every
time you try to run plot.
\item [{list(...)}] The three periods, ``...'', are a valid word. (A
period is a valid character). R scans user syntax for function arguments
and it uses whatever it can understand (formal named arguments, positional
matching), and then the other arguments are thrown into a thing called
``...''.

In a simple case, we could just pass those additional arguments to
another function, as in

\bgroup\inputencoding{latin9}
\begin{lstlisting}
plotme <- function(x, y, ...){
    plot(x, y, ...)
}
\end{lstlisting}
\leavevmode\egroup

Often, it is not sufficient to simply pass on the ``...'' variables,
and we instead need to scan them and sort them for various purposes.
Inside the function, we access those variables by turning them into
a list: 

\begin{Sinput}
dots <- list(...)
\end{Sinput}

We can still pass on the dots to other functions, but the usual problem
is that some of the arguments in dots may be intended for several
different functions. Suppose we are writing a plotter for categorical
variables. We might need to receive some arguments, then run \code{table()},
then run \code{layout()}, then run \code{mosaicplot()}. The user
may have put in arguments intended for those separate functions. There's
a chance of an error, or at least a warning, if irrelevant arguments
are passed to those arguments. 

The rockchalk functions \code{plotSlopes()} has examples that do
this work. They separate the dot arguments that are needed for the
\code{predict()} function and the \code{plot()} function. Because
I was learning about arguments when I wrote those functions, I did
not take the cleanest or most direct route to do this, but they do
stand as working examples that users can review.

A function may set a number of defaults that we want to use when the
users do not supply a lot of details. In a case like that, it is useful
to remember a very handy function in R called \code{modifyList()}.
The basic idea is that we can build a list of default arguments for
a function, and then we can scan the \code{dots} list and it will
replace the defaults in the first with changes in the second. There's
an example in rockchalk's \code{pctable()} function.

\begin{Sinput}
dots <- list(...)
dotnames <- names(dots)
tableargs <- list(rv, cv, dnn = c(rvlab, cvlab))
newargs <- modifyList(tableargs, dots, keep.null = TRUE)
t1 <- do.call("table", newargs)
\end{Sinput}The dots list, by definition, is named arguments, and it is often
handy to have a copy of their names for checking whether an argument
is included in the list. In the plotSlopes() function, here's what
I ended up with:

\begin{Sinput}
validForPredict <- c("se.fit", "dispersion", "terms", "na.action",
                     "level", "pred.var", "weights")
dotsForPredict <- dotnames[dotnames %in% validForPredict]
if (length(dotsForPredict) > 0) {
    parms <- modifyList(parms, dotargs[dotsForPredict])
    dotargs[[dotsForPredict]] <- NULL
}
np <- do.call("predictCI", parms)
\end{Sinput}This scans dotnames for elements in the list of arguments that are
valid for the predict method, and after they are put onto the parms
list, then those arguments are set to NULL in the dotargs collection.
Looking at that now, I see it would have been more clear to use \code{setdiff()}
on dotargs to clean that up.

The central challenge in that work is to find out the names of the
arguments that are intended for one function, then remove them from
\code{dots} so that they are not passed to a subsequent function
that needs them. How can one know what arguments are needed? This
requires some experience, but the first step is to read the interface
documentation of the function being considered. A shortcut is to run
the function \code{formals()}, which prints out the names of the
function being considered. This can sometimes be deceptive for implementations
of generic methods. Observe that the return from the generic is not
so informative as the return from the method:

<<>>=
formals(plot)
formals(plot.default)
@

If a method is not exported, if it does not show as ``visible''
in the output from \code{methods(plot)}, then a little more fancy
footwork is required to see all of the legal arguments.

<<>>=
formals(graphics:::plot.histogram)
@
\end{description}

\subsection{Protect your function's calculations from the user's workspace.}

In the early days of using R, around 2000, it was possible for a user
to obliterate base functions. I recall working on a model about Democratic
and Republican voters. I created a variable named \code{rep} and,
after that, the \code{lm()} and \code{plot()} functions would not
work. I wrote to r-help about this and learned that my \code{rep}
had obliterated the built-in function \code{rep()}, which is vital
in many calculations. Shortly after that, R was improved to protect
functions from obliteration in that way. Today I tried to cause mischief
by re-defining rep in a number of ways and it appears to me the problem
is completely solved.

There has also been a problem that users sometime use the variable
names \code{T} and \code{F}. Recall that R allows \code{T} as an
abbreviation for \code{TRUE} and \code{F} for \code{FALSE}. R will
allow the user to re-define T and F in ways that will break calculations.
However, R will not allow the user to re-define TRUE or FALSE. Observe

\bgroup\inputencoding{latin9}
\begin{lstlisting}
> TRUE <- 2342 
Error in TRUE <- 2342 : invalid (do_set) left-hand side to assignment
> FALSE <- function(x,y){x + y}
Error in FALSE <- function(x, y) { :
  invalid (do_set) left-hand side to assignment
\end{lstlisting}
\leavevmode\egroup

In order to protect our function's calculations, we are urged to fully
write out the symbols TRUE and FALSE inside functions. This way, users
cannot cause damage. 

Another danger we need to be concerned about is that our functions
might have undefined variables in them. When R encounters these undefined
variables, the lexical scoping process looks outward for values. It
may find something we don't expect. Another danger is that a function
seems to work right when we practice it in our session, but another
user gets an error because something in our workspace is missing from
theirs. The lexical scope can be convenient, but it is also dangerous.

Here the example function relies on variables \code{y} and \code{z}
that are not passed in as arguments.

\bgroup\inputencoding{latin9}
\begin{lstlisting}
x <- c(1, 2, 3, 4)
y <- c(8, 9, 10, 11)
z <- c(1, 1, 2, 2)
plotme <- function(x){
    plot(x, y, col = z)
}
plotme(x)
\end{lstlisting}
\leavevmode\egroup

That lets the function go and find \code{y} and \code{z}. This will
run, but it is pretty risky. It depends on R's willingness to reach
outside the boundaries of the function to find \code{y}, and \code{z}.
If I have inadvertently altered \code{y} in the workspace, this will
end badly.

If I'm writing a function for somebody else to use, as in a package,
I believe the function should to NOT rely on variables from the environment.
I want to prevent R from filling in the gaps for users. We'd rather
give the user an error ``variable not found'' than an erroneous
calculation that ``seems'' adequate. The best case scenario is that
\code{y} and \code{z} do not exist in the workspace, so the user
will receive an error message:

\begin{Sinput}
> plotme(x)
\end{Sinput}
\begin{Schunk}
   \begin{Soutput}
Error in xy.coords(x, y, xlabel, ylabel, log) : object 'y' not found
  \end{Soutput}
\end{Schunk}

The worst case scenario is that the user has some other variables
\code{y} and z\code{} sitting about in the workspace. Those ``found''
variables are put to use with a completely unintentional result. 

As a first step toward fixing this, I propose:
\begin{quote}
Every piece of input upon which we rely should be named in the list
of arguments. 
\end{quote}
If we define the function as follows,

\bgroup\inputencoding{latin9}
\begin{lstlisting}
plotme <- function(x, y, z){
    plot(x, y, col = z)
}
\end{lstlisting}
\leavevmode\egroup

the user's use of \code{plotme(x)} will result in the error

\begin{Schunk}
   \begin{Soutput}
Error in xy.coords(x, y, xlabel, ylabel, log) :
    argument "y" is missing, with no default
   \end{Soutput}
\end{Schunk}Naming all of the arguments effectively protects the function against
the accidental importation of data. 

There's a function called \code{checkUsage()} in the \code{codetools}
package that can help check functions for reliance on global variables.
First, we'll make sure the workspace is clean by removing \code{x},
\code{y}, and \code{z}.

\begin{Schunk}
   \begin{Soutput}
> rm(x, y, z)
> library(codetools) 
> checkUsage(plotme) 
<anonymous>: no visible binding for global variable 'y' (:2) 
<anonymous>: no visible binding for global variable 'z' (:2)
   \end{Soutput}
\end{Schunk}

Note that, if the variables \code{x}, \code{y}, an \code{z} exist
in the user workspace, they are global and \code{checkUsage} will
not offer a warning about them. Hence, it is very important to run
\code{checkUsage()} after cleaning up the workspace. 

checkUsage() will also offer warnings about unused variables. This
can be helpful in cleaning up a function.

\subsection{Check argument values.}

We want users to provide input arguments for all information that
the calculations will rely upon. The next step is to check their inputs
to make sure they are reasonable. 

In some cases this is very easy. If we want the user to supply a data.frame,
we can stop when the user provides something else.

<<>>=
plotme <- function(x, y, data, ...){
    if(missing(data) | !is.data.frame(data)) 
        stop(paste0("plotme: the object you supplied as data: '", 
        deparse(substitute(data)) , "' is not a data.frame"))
    plot(as.formula(paste(y,  "~",  x, collapse=" ")), data = data, ...)
}
myx <- rnorm(10)
myy <- rnorm(10)
dat <- data.frame(myx, myy)
@

This works properly

<<eval=F>>=
plotme("myx", "myy", data = dat)
@

However, if the user supplies something else where data ought to be,
an error is returned.

\begin{Sinput}
plotme("myx", "myy", rnorm(10))
\end{Sinput}
\begin{Soutput}
Error in plotme("myx", "myy", rnorm(10)) :
  plotme: the object you supplied as data: 'rnorm(10)' is not a data.frame
\end{Soutput}

The main point here is that if we definitely know what we expect the
user to supply, then we can add a filter that checks the input's type.
That's what is.character, is.vector, is.integer, and so forth are
intended for. (In an R session, type ``is'' and then hit the TAB
key a couple of times. You'll see all the completions.

The problem is not so easy to manage when we can't be sure what the
argument type must be. Sometimes, it appears the best we can do is
pass through what the users give us and hope R can make sense out
of it. I've wrestled with R's flexibility for the naming of colors.
Suppose our plotme() function displays x and y, and we allow the user
to specify colors for the points.

<<>>=
plotme <- function(x, y, z){
    plot(x, y, col = z)
}
myx <- rnorm(10)
myy <- rnorm(10)
@

R colors can be supplied in many formats, as integers, character strings,
or more elaborate HTML-style color designators. Any of these will
work.

<<eval=F>>=
mycol <- 1:10
plotme(myx, myy, z = mycol)
@

<<eval=F>>=
mycol <- rainbow(10)
plotme(myx, myy, z = mycol)
@

<<eval=F>>=
mycol <- gray.colors(10)
plotme(myx, myy, z = mycol)
@

R does not throw an error if the user runs this, but the result is
not what we wanted, probably.

<<eval=F>>=
mycol <- rnorm(10)
plotme(myx, myy, z = mycol)
@

Suppose the user's code ends up supplying a NULL value for \code{z},
as in \code{plotme(x, y, z = NULL)}. They would not do that on purpose,
but it might be an accident. If \code{z} is calculated in their code
and something has gone wrong. To protect against that, we find this
idiom is used in the R source code quite often:

<<eval=F>>=
plotme <- function(x, y, z){
    if (missing(z) | is.null(z)) z <- rep(2, length(x))   
    plot(x, y, col = z)
}
@

If \code{z} is either not supplied by the user's command, or if the
user's command explicitly sets it to NULL, then we'll fabricate \code{z}.

\subsection{Design the function so that it runs with a minimum number of arguments.}

Take a look at the most commonly used functions in R, such as lm()
or plot.default(). These functions allow a great many arguments, but
they will deliver what the users minimally need if the they run

<<eval=F>>=
m1 <- lm(y ~ x, data = dat)
plot(y ~ x, data = dat)
@

Put the arguments that users will use most often at the front of the
argument list, and then design the function so that it will work even
if all of the other arguments are not included. There are two elements
in making this work.

\subsubsection{Specify defaults}

A graph can use any color scheme, but we might put in something colorful
for users who like a little zest in life.

<<eval=F>>=
plotme <- function(x, y, z = rainbow(length(x))){     
    plot(x, y, col = z)
}
plotme(x = rnorm(100), y = rnorm(100))
@

We could as well have put the rainbow call inside the function if
we want to keep a cleaner function declaration. 

<<eval=F>>=
plotme <- function(x, y, z){     
  if(missing(z)) mycol <- rainbow(length(x))
  plot(x, y, col = mycol)
}
plotme(x = rnorm(100), y = rnorm(100))
@

It seems to me that the R source itself tends to have more elaborate
specifications of defaults within the declaration, rather than inside
the function.

In the R source code, I noticed a peculiar-looking example in which
an argument's default depends on a function that is within the function
itself. Observe that this runs fine:

<<eval=F>>=
plotme <- function(x, y, z = getAColor(length(x)))
{
    getAColor <- function(n){
             gray.colors(n)
    }
    plot(x, y, col = z, cex = 5)
}
plotme(rnorm(20), rnorm(20))
@

That was startling to me because the nested function, \code{getAColor},
does not exist outside the function, and yet it can be used in the
definition of a default. I merely promise that \code{getAColor} will
exist when the value of \code{z} is needed and all is well. 

\subsubsection{Extract or construct what else is needed from the user input}

We can deduce information from the arguments that are passed in, we
can eliminate the need for the user to pass in other arguments. We
would never need to require both arguments x and xlab in the plot
function, for example. xlab is optional because, if the user does
not supply it, then deparse(substitute()) can find something to use
as a label. The 2010 version of plotSlopes in rockchalk required the
user to pass through many parameters that (I've since learned) can
be deduced from the other arguments.

A key element in this is getting an understanding of the information
that comes in with a standard R object. Quite a few--almost all--functions
that generate interesting objects in R provide, as a part of their
output, a copy of the call that ran the function. Inside our functions,
we can inspect that call argument to see what the users ran, and from
that we can often deduce what we need without requiring users to supply
a lot of information. 

\subsection{Return values versus attributes}

If you review the R regression functions, like lm() or glm(), you
notice that they build a list of things and pass it back to the user.
That is the most obvious way to return information.

There is another way, however, which is a little interesting. One
can mark an attribute on a return object. The R usage of attributes
comes to mind at this point because, when we are scanning for inputs,
we may need to inspect not only their values, but also their attributes.
The most commonly inspected attribute, of course, is the class of
the argument. So, if we are writing a package and we need to make
sure that the objects being passed around among functions are correct
for the purpose, then the object attributes are a very useful tool.

I don't want to write a lot about this so as to display my ignorance,
but I would point user to \code{pctable()} as an example of a rockchalk
function that uses both a return list and attributes to get the job
done. Maybe I'll fill in this example at a later time, once I figure
out the most direct way to explain it.

The only problem I find with attributes is that the printing functions
for R objects generally ``splat out'' the attributes when we don't
want them. If one puts attributes on an object, then the default printer
will show them. I've not learned all of the ins-and-outs of preventing
that, but in 2015 I became aware of it and learned that one way to
deal with it can be found in the R source code for the method \code{print.table()}. 

\section{Do This, Not That (Stub) }

R novices sometimes use Google to search for R advice and they find
it, good or bad. They may find their way to the r-help email list,
where advice is generally good, or to the StackOverflow pages for
R, which may be better. A lot of advice is offered by people like
me, who may have good intentions, but are simply not qualified to
offer advice. 

One of the few bits of advice that seems to grab widespread support
is that ``for loops are bad.'' One can write an lapply statement
in one line, while a for loop can take 3 lines. The code is shorter,
but it won't necessarily run more quickly. I recall being jarred by
this revelation in John Chambers's book, \emph{Software for Data Analysis}.
The members of the apply family (apply, lapply, sapply, etc) can make
for more readable code, but they aren't always faster. ``However,
none of the apply mechanisms changes the number of times the supplied
function is called, so serious improvements will be limited to iterating
simple calculations many times. Otherwise, the n evaluations of the
function can be expected to be the dominant fraction of the computation''\citep[213]{chambers_software_2008}.

Todo: insert discussion of stackListItems-001.

Insert alternative methods of measuring executation time and measuring
performance

Balance time spent optimizing code versus time spent running program.

\section{Suggested Chores}

I suggest the would be programmer should take on some basic challenges. 
\begin{enumerate}
\item Try to create a generic function and several methods to handle classes
of various types. If the term ``generic function'' and ``method''
cause disorientation, that means the reader is a user, not a programmer
yet. 
\item Consider developing a routine (or package) for statistical estimation
or presentation. Find several R packages and compare the R code in
them. Let me know if you agree with me about these points.
\end{enumerate}
\begin{itemize}
\item Good code is compartmentalized. Separate pieces of work are handled
by separated functions and results are returned in a well organized
way.
\item Functions should not be HUGE. 

The components of a project should be small enough so that we can
comprehend them. Generally speaking, if we are reading code and we
come to a line that uses a variable that we cannot find on the screen,
that's a problem. I used to correspond with a programmer at the University
of Michigan who said he began to feel uncomfortable when a function
filled up the entire terminal screen. 
\item People who copy and paste sections over and over in order to handle
slightly different cases are causing trouble for themselves and others.
They should think harder on ways to separate that work into re-usable
functions. 
\end{itemize}
\bibliographystyle{chicago}
\bibliography{rockchalk}

\end{document}

#LyX 2.3 created this file. For more info see http://www.lyx.org/
\lyxformat 544
\begin_document
\begin_header
\save_transient_properties true
\origin unavailable
\textclass scrartcl
\begin_preamble
%\VignetteIndexEntry{Rstyle}

\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage{babel}
\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%

 \ifthenelse{\equal{#1}{}}{%
   \@float{figure}
 }{%
   \@float{figure}[#1]%
 }%
 \centering
}{%
 \end@float
}
\renewenvironment{table}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{table}
 }{%
   \@float{table}[#1]%
 }%
 \centering
%  \setlength{\@tempdima}{\abovecaptionskip}%
%  \setlength{\abovecaptionskip}{\belowcaptionskip}%
% \setlength{\belowcaptionskip}{\@tempdima}%
}{%
 \end@float
}

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\normalsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.95}}
\def\Routbackground{\color{white}}
\def\Routcolor{\color{black}}


\usepackage{listings}% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, style=Rstyle}

\usepackage{xcolor}
\definecolor{darkblue}{HTML}{1e2277}
\end_preamble
\options noae
\use_default_options false
\begin_modules
sweave
enumitem
logicalmkup
\end_modules
\maintain_unincluded_children false
\language american
\language_package default
\inputencoding utf8
\fontencoding global
\font_roman "lmodern" "default"
\font_sans "lmss" "default"
\font_typewriter "cmtt" "default"
\font_math "auto" "auto"
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100 100
\font_tt_scale 100 100
\use_microtype false
\use_dash_ligatures true
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered false
\pdf_bookmarksopen false
\pdf_bookmarksopenlevel 1
\pdf_breaklinks true
\pdf_pdfborder true
\pdf_colorlinks true
\pdf_backref section
\pdf_pdfusetitle true
\pdf_quoted_options "colorlinks=true, linkcolor=darkblue, urlcolor=darkblue, citecolor=darkblue"
\papersize default
\use_geometry true
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 1
\use_package esint 1
\use_package mathdots 0
\use_package mathtools 1
\use_package mhchem 0
\use_package stackrel 1
\use_package stmaryrd 1
\use_package undertilde 1
\cite_engine natbib
\cite_engine_type authoryear
\biblio_style plainnat
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\justification true
\use_refstyle 0
\use_minted 0
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 1in
\topmargin 1in
\rightmargin 1in
\bottommargin 1in
\secnumdepth 3
\tocdepth 3
\paragraph_separation skip
\defskip smallskip
\is_math_indent 0
\math_numbering_side default
\quotes_style english
\dynamic_quotes 0
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Title
R Style.
 An Rchaeological Commentary.
 
\end_layout

\begin_layout Author
Paul E.
 Johnson <pauljohn @ ku.edu>
\end_layout

\begin_layout Section
Introduction: Ugly Code that Runs
\end_layout

\begin_layout Standard
Because there is no comprehensive official R style manual, students and
 package writers seem to think that there is no style whatsoever to be followed.
 While it may be true that 
\begin_inset Quotes eld
\end_inset

ugly code runs,
\begin_inset Quotes erd
\end_inset

 it is also 1) difficult to read and 2) frustrating to extend, and 3) tiring
 to debug.
 Code is a language, a medium of communication, and one should not feel
 free no ignore its customs.
 
\end_layout

\begin_layout Standard
After students have finished a semester of statistics with R, they may be
 ready to start preparing functions or packages.
 Those R users are the ones I'm trying to address with this note.
 It is important to realize that the readability of code makes a difference.
 It sometimes difficult to know that there is a 
\begin_inset Quotes eld
\end_inset

right way
\begin_inset Quotes erd
\end_inset

 and a 
\begin_inset Quotes eld
\end_inset

wrong way
\begin_inset Quotes erd
\end_inset

 because there are so many examples to study on CRAN.
\end_layout

\begin_layout Standard
This note describes R style from an Rchaeological
\begin_inset Foot
status collapsed

\begin_layout Plain Layout
Definitions:
\end_layout

\begin_layout Description
Rchaeology: The study of R programming by investigation of R source code.
 It is the effort to discern the programming strategies, idioms, and style
 of R programmers in order to better communicate with them.
\end_layout

\begin_layout Description
Rchaeologist: One who practices Rchaeology.
\end_layout

\end_inset

 perspective.
 By examining the work of the R Core Development Team 
\begin_inset CommandInset citation
LatexCommand citep
key "RCore"
literal "true"

\end_inset

 and other notable package writers, we are able to discern an implicit style
 guide.
 However, this note is not 
\begin_inset Quotes eld
\end_inset

official
\begin_inset Quotes erd
\end_inset

 or endorsed from R Core.
\begin_inset Foot
status open

\begin_layout Plain Layout
Yet :)
\end_layout

\end_inset

 With one exception at the end of this note, none of the advice here is
 
\begin_inset Quotes eld
\end_inset

my
\begin_inset Quotes erd
\end_inset

 advice.
 Instead, it is my best description of the standards followed by the leading
 R programmers.
 
\end_layout

\begin_layout Standard
At one point, the only guide was the Google R style guide,
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status open

\begin_layout Plain Layout

https://google.github.io/styleguide/Rguide.xml
\end_layout

\end_inset


\end_layout

\end_inset

 which was used as a policy for R-related 
\begin_inset Quotes eld
\end_inset

Google Summer of Code
\begin_inset Quotes erd
\end_inset

 projects.
 There are many excellent suggestions in Hadley Wickham's Style Guide.
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status open

\begin_layout Plain Layout

http://adv-r.had.co.nz/Style.html
\end_layout

\end_inset


\end_layout

\end_inset

 In what follows, I'll try to explain why there are some variations among
 these projects and offer some advice about how we (the users) should sort
 through their advice.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=F>>=
\end_layout

\begin_layout Plain Layout

dir.create("plots", showWarnings=F)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% In document Latex options:
\end_layout

\begin_layout Plain Layout


\backslash
fvset{listparameters={
\backslash
setlength{
\backslash
topsep}{0em}}}
\end_layout

\begin_layout Plain Layout


\backslash
SweaveOpts{prefix.string=plots/plot,ae=F,height=4,width=6}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<Roptions, echo=F>>=
\end_layout

\begin_layout Plain Layout

options(width=100, continue="+ ")
\end_layout

\begin_layout Plain Layout

options(useFancyQuotes = FALSE) 
\end_layout

\begin_layout Plain Layout

set.seed(12345)
\end_layout

\begin_layout Plain Layout

pdf.options(onefile=F,family="Times",pointsize=12)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Rchaeological Methodology
\end_layout

\begin_layout Standard
I am a student of R as a programming language.
 I am also student of the R community as an international success that created
 a working open source computer program.
 One of the most interesting differences between R and other open source
 projects I have observed is that R attracts non-programmers.
 There is an abundance of statistical novices and untrained computer programmers
 in the R user community.
 Many students begin with R as a way of learning about computer programming.
 In contrast, the developers of R are world-class software engineers.
 They have formal training in computer programming and years of experience
 in a variety of computer languages.
 The diversity creates a healthy tension that is easy to see in the r-help
 email list or on Web forums for R users.
 
\end_layout

\begin_layout Subsection
\begin_inset Quotes eld
\end_inset

Use the Source, Luke,
\begin_inset Quotes erd
\end_inset

 said Obi-Wan
\end_layout

\begin_layout Standard
What should R code look like? Stop guessing.
 The implicit style guide for R is the R source code itself.
 If users want to communicate with R Core developers, they ought to communicate
 using the style that developers use.
\end_layout

\begin_layout Standard
I'm often surprised to find that R users–even experienced ones–have never
 looked at the R source code.
 Before going any further,
\end_layout

\begin_layout Quote
Open the source code for R.
 I mean, literally, download R-3.5.2.tar.gz (or whatever is current when you
 read this).
 Unpack that, navigate to the directory src/library/stats/R.
 Open the file 
\begin_inset Quotes eld
\end_inset

lm.R
\begin_inset Quotes erd
\end_inset

.
 
\end_layout

\begin_layout Standard
That's what R code should look like.
 
\end_layout

\begin_layout Standard
Browse other R files in the source code.
 Notice the files are suffixed by R, not r! 
\end_layout

\begin_layout Standard
Then go read a lot of R packages.
 Begin with the recommended packages (in the R source code under src/library/Rec
ommended).
 Then draw some samples from CRAN.
 Choose packages that are prepared by members of R Core, and then sample
 a few packages that are widely installed, such as John Fox's car package
 
\begin_inset CommandInset citation
LatexCommand citep
key "fox_r_2011"
literal "true"

\end_inset

.
 
\end_layout

\begin_layout Standard
After that, pick a random sample of packages on CRAN.
 Don't be surprised by ugly code in a randomly chosen R package.
 
\end_layout

\begin_layout Subsection
Notice How R Describes its Own Style
\end_layout

\begin_layout Standard
Type the name of a function at the R command prompt.
 That is the same as using the function called 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

print.function()
\end_layout

\end_inset

 to review the contents of a function from an R package.
 For example, try 
\begin_inset Quotes eld
\end_inset

lm
\begin_inset Quotes erd
\end_inset

.
 The first few lines are
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> lm
\end_layout

\begin_layout Plain Layout

function (formula, data, subset, weights, na.action, method = "qr",
\end_layout

\begin_layout Plain Layout

    model = TRUE, x = FALSE, y = FALSE, qr = TRUE, singular.ok = TRUE,
\end_layout

\begin_layout Plain Layout

    contrasts = NULL, offset, ...)
\end_layout

\begin_layout Plain Layout

{
\end_layout

\begin_layout Plain Layout

    ret.x <- x
\end_layout

\begin_layout Plain Layout

    ret.y <- y
\end_layout

\begin_layout Plain Layout

    cl <- match.call()
\end_layout

\begin_layout Plain Layout

    mf <- match.call(expand.dots = FALSE)
\end_layout

\begin_layout Plain Layout

    m <- match(c("formula", "data", "subset", "weights", "na.action",
\end_layout

\begin_layout Plain Layout

        "offset"), names(mf), 0L)
\end_layout

\begin_layout Plain Layout

    mf <- mf[c(1L, m)]
\end_layout

\begin_layout Plain Layout

    mf$drop.unused.levels <- TRUE
\end_layout

\begin_layout Plain Layout

    mf[[1L]] <- as.name("model.frame")
\end_layout

\begin_layout Plain Layout

    mf <- eval(mf, parent.frame())
\end_layout

\begin_layout Plain Layout

    if (method == "model.frame")
\end_layout

\begin_layout Plain Layout

        return(mf)
\end_layout

\begin_layout Plain Layout

    else if (method != "qr")
\end_layout

\begin_layout Plain Layout

        warning(gettextf("method = '%s' is not supported.
 Using 'qr'",
\end_layout

\begin_layout Plain Layout

            method), domain = NA)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
That's quite a bit like the code in the file lm.R, but it is not exactly
 the same.
 Even if the code in lm.R were an ugly, horrible mess, its output in the
 terminal would be indented and spaced just right.
 That is an important Rchaeological finding!
\end_layout

\begin_layout Standard
Why can there be a difference between the code for a function in a file
 (like 
\begin_inset Quotes eld
\end_inset

lm.R
\begin_inset Quotes erd
\end_inset

) and the output of the command (like 
\begin_inset Quotes eld
\end_inset

lm
\begin_inset Quotes erd
\end_inset

)? Admittedly, this is difficult to understand.
 The on-screen output is not (by default, anyway) the source that went into
 R, but rather it is R's rendition of the internal structure of the function.
 I recently had an epiphany while reading a section in the 
\emph on
Writing R Extensions
\emph default
 manual called 
\begin_inset Quotes eld
\end_inset

Tidying R code
\begin_inset Quotes erd
\end_inset

.
 That title is a bit misleading.
 It is not about tidying R source code; rather, it is about beautifying
 the rendition of internal structures for the terminal.
 
\begin_inset Quotes eld
\end_inset

R treats function code loaded from packages and code entered by users differentl
y.
 By default code entered by users has the source code stored internally,
 and when the function is listed, the original source is reproduced.
 Loading code from a package (by default) discards the source code, and
 the function listing is re-created from the parse tree of the function.
\begin_inset Quotes erd
\end_inset

 That is to say, if ugly code is syntatically valid, R can parse it and
 structure it according to the internal dictates of the R runtime system,
 and when we ask to see the function, we get a nice looking result.
\end_layout

\begin_layout Subsection
Formulate SEA estimates.
\end_layout

\begin_layout Standard
As already noted, there is no mandatory style for R code.
 The 
\emph on
R Internals
\emph default
 manual has a section 
\begin_inset Quotes eld
\end_inset

R coding standards,
\begin_inset Quotes erd
\end_inset

 but it is quite brief.
 The main point that most readers take away concerns indentation: subsections
 in code should be preceded by 4 blank spaces, not a tab character.
 
\end_layout

\begin_layout Standard
But there is a larger point in 
\emph on
R Internals
\emph default
, but novices don't recognize the importance of it.
 R is a GNU project, and there are GNU coding standards.
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.gnu.org/prep/standards/standards.html
\end_layout

\end_inset


\end_layout

\end_inset

 The R project's C code follows the standard closely.
 In the entire body of the R source code, we find the GNU thumb print.
 The importance of that fact is missed by untrained readers, who mistake
 the lack of a comprehensive discussion of style for an encouragement to
 
\begin_inset Quotes eld
\end_inset

do anything you want.
\begin_inset Quotes erd
\end_inset

 
\end_layout

\begin_layout Standard
In the following, I will try to point out the areas of greatest agreement
 by assigning an SEA score to each point.
 SEA stands for 
\begin_inset Quotes eld
\end_inset

Subjective and completely unscientific personal Estimate of Agreement.
\begin_inset Quotes erd
\end_inset

 These are my Bayesian priors.
 If I could survey my favorite R programmers, I'd find some variety, and
 I am trying to make it clear where the disagreements might lie.
 But, then again, I may have been fooling myself.
 It has recently been suggested to me that these recommendations are not
 descriptions of the Rchaeological community I'm studying, they are rather
 my personal litmus test for admirable R programmers.
 
\end_layout

\begin_layout Section
Nearly Universally accepted standards.
\end_layout

\begin_layout Subsection
(SEA 1.0) Indentation of code sections is required.
 
\end_layout

\begin_layout Standard
This is explicitly spelled out in the R documentation.
 No tabs! Insert 4 blank spaces.
 Personally, I prefer 2 spaces, which has been the default in Emacs.
 But I'm changing my code to use 4 spaces.
 If you find my code with 2 spaces, please accept this apology and believe
 that it is an oversight.
 
\end_layout

\begin_layout Subsection

\series bold
(SEA .95).
 Use 
\begin_inset Quotes eld
\end_inset

<-
\begin_inset Quotes erd
\end_inset

, not 
\begin_inset Quotes eld
\end_inset

=
\begin_inset Quotes erd
\end_inset

, for assignments.

\series default
 
\end_layout

\begin_layout Standard
One cannot find the equal sign used for assignments in any file in the R
 source code.
 Nor can one find it in any of the Recommended packages (so far as I can
 tell).
 
\end_layout

\begin_layout Standard
Students who have learned R in introductory textbooks are sometimes shocked
 to learn that they were taught wrong.
 I'm sympathetic to their outrage.
 How can this be? 
\end_layout

\begin_layout Standard
The equal sign was used by mistake so frequently that the R system was re-design
ed to tolerate that mistake.
 
\emph on
Most
\emph default
 usages of the equal sign for assignments do not cause runtime errors.
 Not all possible problems were eliminated, however.
 Thus the equal sign is not recommended, it is tolerated.
 Nevertheless, A horrible profusion of textbooks and packages ensued using
 the equal sign for assignment.
 
\end_layout

\begin_layout Subsection

\series bold
(
\series default
SEA .98) Blank spaces around symbols are required.
 
\end_layout

\begin_layout Standard
This is a general GNU coding standard.
 
\end_layout

\begin_layout Enumerate
Insert spaces before and after 
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Enumerate
mathematical symbols like: 
\begin_inset Quotes eld
\end_inset

=
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes erd
\end_inset

<-
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

<
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

*
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes erd
\end_inset

+
\begin_inset Quotes erd
\end_inset


\end_layout

\begin_layout Enumerate
R binary operations like: 
\begin_inset Quotes eld
\end_inset

%*%
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

%o%
\begin_inset Quotes erd
\end_inset

, and 
\begin_inset Quotes eld
\end_inset

%in%
\begin_inset Quotes erd
\end_inset

.
\end_layout

\end_deeper
\begin_layout Enumerate
Put one space after commas.
 
\end_layout

\begin_layout Enumerate
Insert one space before the opening squiggly braces 
\begin_inset Quotes eld
\end_inset

{
\begin_inset Quotes erd
\end_inset

.
\end_layout

\begin_layout Enumerate
Put one space after the closing parenthesis 
\begin_inset Quotes eld
\end_inset

)
\begin_inset Quotes erd
\end_inset

 and the closing squiggly brace 
\begin_inset Quotes eld
\end_inset

}
\begin_inset Quotes erd
\end_inset

.
\end_layout

\begin_layout Standard
This is purely a matter of convention and legibility, it does not affect
 the 
\begin_inset Quotes eld
\end_inset

rightness
\begin_inset Quotes erd
\end_inset

 of code.
 
\end_layout

\begin_layout Standard
Other observations about spaces,
\end_layout

\begin_layout Enumerate
Do not insert spaces between function names and their opening parentheses.
\end_layout

\begin_layout Enumerate
After reviewing the R source code, I was uncertain about whether one ought
 to insert one space after 
\begin_inset Quotes eld
\end_inset

if
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

for
\begin_inset Quotes erd
\end_inset

.
 From an Rchaeological perspective, this is a little bit perplexing.
 In the help page for those terms (see help(
\begin_inset Quotes eld
\end_inset

for
\begin_inset Quotes erd
\end_inset

)), there is no space after 
\begin_inset Quotes eld
\end_inset

if
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

for
\begin_inset Quotes erd
\end_inset

.
 In the R-3.0.0 source code folder src/library/base/R, I count 1741 instances
 of 
\begin_inset Quotes eld
\end_inset

if(
\begin_inset Quotes eld
\end_inset

 and 683 instances of 
\begin_inset Quotes eld
\end_inset

if (
\begin_inset Quotes eld
\end_inset

.
 The former style seemed right to me, at least at first, because people
 often say that R's 
\begin_inset Quotes eld
\end_inset

if
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

for
\begin_inset Quotes erd
\end_inset

 are functions.
 I asked for clarification in the R-devel email list, and Peter Dalgaard
 explained that the space should be used because those terms are
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Quote
language constructs (and they 
\emph on
are
\emph default
 keywords, not names, that's why ?for won't work).
 The function calls are `if`(fee, {foo}, {fie}) and something rebarbative
 for `for`(....).
\end_layout

\begin_layout Quote
Besides, both constructs are harder to read without the spaces.
 (r-devel, April 18, 2013)
\end_layout

\begin_layout Standard
For me, that settles the question.
 For R code, as in C, 
\begin_inset Quotes eld
\end_inset

if
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

for
\begin_inset Quotes erd
\end_inset

 should be treated as keywords, and there would be a space after them, as
 in 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

if (x < 7)
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

.
\end_layout

\end_deeper
\begin_layout Enumerate
Do not insert 
\begin_inset Quotes eld
\end_inset

extra spaces
\begin_inset Quotes erd
\end_inset

 inside parentheses.
 
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Standard
Programmers who have written in the BASH scripting language may recall that
 a space inside brackets is required.
 That training causes me to think that R code is a little bit 
\begin_inset Quotes eld
\end_inset

jammed together.
\begin_inset Quotes erd
\end_inset

 This is pleasant to my eye:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

if ( (x == 1) & (y == 2) ) {
\end_layout

\end_inset


\end_layout

\begin_layout Standard
but, from an Rchaeological point of view, more the correct style is:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

if((x == 1) & (y == 2)) {
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The insertion of the interior parentheses for the smaller conditions inside
 the if statement is consistent with the GNU standard for C.
\end_layout

\end_deeper
\begin_layout Subsubsection*
Is there an 
\begin_inset Quotes eld
\end_inset

argument exception
\begin_inset Quotes erd
\end_inset

 to the space rule for equal signs?
\end_layout

\begin_layout Standard
Package writers are not entirely consistent, and Rchaeologically speaking,
 we cannot be sure if these variations are accidental.
 We sometimes find no spaces, as in
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

plot(x, y, lwd=4, col=green, main="My Title")
\end_layout

\end_inset


\end_layout

\begin_layout Standard
It would surely be more correct like so:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

plot(x, y, lwd = 4, col = green, main = "My Title")
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Spaces may sometimes be omitted in an effort to keep code on one line.
 Especially where publishers are concerned about the use of scarce paper,
 the omission of spaces around equal signs is not uncommon.
 Please note, however, that it is NEVER acceptable to omit the spaces after
 commas!
\end_layout

\begin_layout Subsubsection*
What about indentation of long function declarations?
\end_layout

\begin_layout Standard
One of the interesting space related questions is the indentation of function
 declarations when there are many arguments.
 Consider the R source code for the function lm():
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

lm <- function (formula, data, subset, weights, na.action,
\end_layout

\begin_layout Plain Layout

                method = "qr", model = TRUE, x = FALSE, y = FALSE,
\end_layout

\begin_layout Plain Layout

                qr = TRUE, singular.ok = TRUE, contrasts = NULL,
\end_layout

\begin_layout Plain Layout

                offset, ...)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Note that lines 2-4 are indented under the letter 
\begin_inset Quotes eld
\end_inset

f
\begin_inset Quotes erd
\end_inset

 in formula.
 If the function's name were longer, it would push all of that indented
 code to the right, probably causing line wraps.
 The solution is to put the function's name and the assignment symbol on
 separate line.
 This is the format of R's function plot.lm().
 
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

plot.lm <-
\end_layout

\begin_layout Plain Layout

function (x, which = c(1L:3L,5L), ## was which = 1L:4L,
\end_layout

\begin_layout Plain Layout

          caption = list("Residuals vs Fitted", "Normal Q-Q",
\end_layout

\begin_layout Plain Layout

          "Scale-Location", "Cook's distance",
\end_layout

\begin_layout Plain Layout

          "Residuals vs Leverage",
\end_layout

\begin_layout Plain Layout

          expression("Cook's dist vs Leverage  " * h[ii] / (1 - h[ii]))),
\end_layout

\begin_layout Plain Layout

          panel = if(add.smooth) panel.smooth else points,
\end_layout

\begin_layout Plain Layout

          sub.caption = NULL, main = "",
\end_layout

\begin_layout Plain Layout

          ask = prod(par("mfcol")) < length(which) && dev.interactive(),
 ...,
\end_layout

\begin_layout Plain Layout

          id.n = 3, labels.id = names(residuals(x)), cex.id = 0.75,
\end_layout

\begin_layout Plain Layout

          qqline = TRUE, cook.levels = c(0.5, 1.0),
\end_layout

\begin_layout Plain Layout

          add.smooth = getOption("add.smooth"),
\end_layout

\begin_layout Plain Layout

          label.pos = c(4,2), cex.caption = 1)
\end_layout

\begin_layout Plain Layout

{
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The continuation is indented to be below the first argument.
 The benefit of this 
\begin_inset Quotes eld
\end_inset

declaration by itself
\begin_inset Quotes erd
\end_inset

 approach is that the additional lines are always re-formatted with consistent
 indentation and we are not creating a huge empty white space due to indentation.
\end_layout

\begin_layout Subsubsection*
Try formatR::tidy.source()
\end_layout

\begin_layout Standard
The advice so far mostly concerns 
\begin_inset Quotes eld
\end_inset

white space
\begin_inset Quotes erd
\end_inset

.
 We would like a programmer's text editor to handle automatically as much
 of that as possible.
 
\end_layout

\begin_layout Standard
The R package 
\begin_inset Quotes eld
\end_inset

formatR
\begin_inset Quotes erd
\end_inset

 
\begin_inset CommandInset citation
LatexCommand citep
key "formatr"
literal "true"

\end_inset

 has a function called tidy.source() which can often (but not always) clean
 up code.
 Below I've pasted in part of an Emacs session.
 I wrote a badly formatted function, myfn(), and copied it to the clipboard,
 and then tidy.source() reads the clipboard.
 It works like magic.
 
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> myfn <- function(x){ if (x < 7) {i = 77; print(paste("x is less than 7
 but i is", i))} else {print("x is excessive") }} 
\end_layout

\begin_layout Plain Layout

> library(formatR)
\end_layout

\begin_layout Plain Layout

> tidy.source()
\end_layout

\begin_layout Plain Layout

function(x) {
\end_layout

\begin_layout Plain Layout

    if (x < 7) {
\end_layout

\begin_layout Plain Layout

        i = 77
\end_layout

\begin_layout Plain Layout

        print(paste("x is less than 7 but i is", i))
\end_layout

\begin_layout Plain Layout

    } else {
\end_layout

\begin_layout Plain Layout

        print("x is excessive")
\end_layout

\begin_layout Plain Layout

    }
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The tidy.source() function can get rid of equals sign assignments if we ask
 it to.
 (In my opinion, it should do that by default.)
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> tidy.source(source = "clipboard", replace.assign = TRUE)
\end_layout

\begin_layout Plain Layout

function(x) {
\end_layout

\begin_layout Plain Layout

    if (x < 7) {
\end_layout

\begin_layout Plain Layout

        i <- 77
\end_layout

\begin_layout Plain Layout

        print(paste("x is less than 7 but i is", i))
\end_layout

\begin_layout Plain Layout

    } else {
\end_layout

\begin_layout Plain Layout

        print("x is excessive")
\end_layout

\begin_layout Plain Layout

    }
\end_layout

\begin_layout Plain Layout

} 
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The tidy.source() function can receive input as files or whole directories.
\end_layout

\begin_layout Standard
There are two reasons why tidy.source() is not a panacea.
 First, by design, tidy.source() will fail if there are programming errors
 in the original source code.
 That leads to a Catch-22.
 I want to clean up the code to find out why it does not run, but tidy.source()
 cannot clean it up because it does not run.
 Second, quite often it happens that tidy.source() chokes on unexpected user
 code.
 Especially problematic is code that has comments inserted in unexpected
 places.
 For example, I recently ran tidy.source() on the file emb.r in the package
 Amelia 
\begin_inset CommandInset citation
LatexCommand citep
key "Amelia"
literal "true"

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> library(formatR)
\end_layout

\begin_layout Plain Layout

> tidy.source("emb.r")
\end_layout

\begin_layout Plain Layout

Error in base::parse(text = text, srcfile = NULL) :
\end_layout

\begin_layout Plain Layout

  152:88: unexpected SPECIAL
\end_layout

\begin_layout Plain Layout

151: }
\end_layout

\begin_layout Plain Layout

152: if (ncol(as.matrix(startvals)) == AMp+1 && nrow(as.matrix(startvals))
 == AMp+1)        %InLiNe_IdEnTiFiEr%
\end_layout

\begin_layout Plain Layout

                                                                       
                    ^
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I would estimate that tidy.source() fails on about one-third of the R code
 I randomly select from CRAN.
\end_layout

\begin_layout Subsection
(SEA .70) The 
\begin_inset Quotes eld
\end_inset

} else {
\begin_inset Quotes erd
\end_inset

 policy.
 
\end_layout

\begin_layout Standard
Did you notice 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

} else {
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 in the 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

tidy.source()
\end_layout

\end_inset

 output for 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

myfn()
\end_layout

\end_inset

? That's the correct style.
 We should not have the left squiggly brace 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

}
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 on a separate line from the 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

else
\end_layout

\end_inset

,
\begin_inset Quotes erd
\end_inset

 and the right squiggly brace 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

{
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 should be on that same line.
 This is, well, obviously good (in my opinion).
\end_layout

\begin_layout Standard
Why? Try this at the command line.
 
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> if (x < 10) print("hello") 
\end_layout

\begin_layout Plain Layout

[1] "hello" 
\end_layout

\begin_layout Plain Layout

> else print("goodbye") 
\end_layout

\begin_layout Plain Layout

Error: unexpected 'else' in "else"
\end_layout

\end_inset


\end_layout

\begin_layout Standard
R does not realize that it is not yet finished with the if keyword's work.
 The keyword else appears to begin a new thought, which is illegal.
 The if's help page (run 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

help("if")
\end_layout

\end_inset

 or 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

?"if"
\end_layout

\end_inset

) is referring to this problem when it says, 
\end_layout

\begin_layout Quote
In particular, you should not have a newline between ‘}’ and ‘else’ to avoid
 a syntax error in entering a ‘if ...
 else’ construct at the keyboard or via ‘source’.
 For that reason, one (somewhat extreme) attitude of defensive programming
 is to always use braces, e.g., for ‘if’ clauses.
\end_layout

\begin_layout Standard
I agree with the somewhat extreme attitude, but will compromise: If one
 uses squiggly braces, always follow the 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

} else {
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 policy.
 
\end_layout

\begin_layout Standard
Some might follow a soft line on this, suggesting only that 
\series bold
users should not
\series default
 
\series bold
begin a line with the word else
\series default
.
 That does not go quite far enough for me.
 I'd add, 
\series bold
always use squiggles after else.

\series default
 This is simply a way of avoiding a very common coding error.
 This code is OK:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

if (x < 7) print("so far, so good") else
\end_layout

\begin_layout Plain Layout

print("this is else")
\end_layout

\end_inset


\end_layout

\begin_layout Standard
But it invites a coding error like so:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

if (x < 7) print("so far, so good") else
\end_layout

\begin_layout Plain Layout

print("this is else")
\end_layout

\begin_layout Plain Layout

print("and we want this also to be with else, but it is not")
\end_layout

\end_inset


\end_layout

\begin_layout Standard
To be perfectly clear, and to protect ourselves against editing errors in
 the future, we could follow the 
\begin_inset Quotes eld
\end_inset

somewhat extreme
\begin_inset Quotes erd
\end_inset

 advice and write this:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

if (x < 7) {
\end_layout

\begin_layout Plain Layout

	print("so far, so good") 
\end_layout

\begin_layout Plain Layout

} else {
\end_layout

\begin_layout Plain Layout

    print("this is else")
\end_layout

\begin_layout Plain Layout

    print("and we want this also to be with else")
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\begin_layout Subsubsection*
Counter-argument based on the R source code
\end_layout

\begin_layout Standard
This would be a completely closed case if not for the fact that the 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

} else {
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 policy is ignored in vast expanses of the R source code.
 In the R source code, scan for the keyword else and in almost every file,
 one finds:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

  else
\end_layout

\end_inset


\end_layout

\begin_layout Standard
A naked else! This is frustrating for writers of style guides.
 It ignores the advice in the 
\begin_inset Quotes eld
\end_inset

if
\begin_inset Quotes erd
\end_inset

 help page.
 We cannot run this code line-by-line.
 
\end_layout

\begin_layout Standard
On the other hand, the function that includes that apparently runs! Why
 doesn't that code crash? When an if/else statement is enclosed in a larger
 area that is demarcated by squiggly braces, then R will understand the
 naked else when it finds it.
 Observe the fix at the command line:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> x <- 1
\end_layout

\begin_layout Plain Layout

> {
\end_layout

\begin_layout Plain Layout

+ if (x < 10) print("hello")
\end_layout

\begin_layout Plain Layout

+ else
\end_layout

\begin_layout Plain Layout

+ print("My dangling else")
\end_layout

\begin_layout Plain Layout

+ }
\end_layout

\begin_layout Plain Layout

[1] "hello"
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I don't think I'm going to have any luck persuading the R Core Development
 Team that their naked elses need to be fixed.
 The best I can do is to urge code writers to use 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

} else {
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 and make them responsible for errors that result from ignoring that rule.
 
\end_layout

\begin_layout Standard
One will note another interesting anomaly while reviewing R source code.
 Unlike programs written in C, where a consistent style for the placement
 of squiggly braces will be followed, in R we observe files that do not
 follow a particular rule.
 In src/library/src/logLik.R, we find functions in both the K&R (
\begin_inset CommandInset citation
LatexCommand citealp
key "kernighan_c_1988"
literal "true"

\end_inset

) C style
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

nobs.logLik <- function(object, ...) {
\end_layout

\begin_layout Plain Layout

    res <- attr(object, "nobs")
\end_layout

\begin_layout Plain Layout

    if (is.null(res)) stop("no 
\backslash
"nobs
\backslash
" attribute is available")
\end_layout

\begin_layout Plain Layout

    res
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
and we also find the vertically aligned squiggly braces approach:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

print.logLik <- function(x, digits = getOption("digits"), ...)
\end_layout

\begin_layout Plain Layout

{
\end_layout

\begin_layout Plain Layout

    cat("'log Lik.' ", paste(format(c(x), digits=digits), collapse=", "),
\end_layout

\begin_layout Plain Layout

        " (df=",format(attr(x,"df")),")
\backslash
n",sep="")
\end_layout

\begin_layout Plain Layout

    invisible(x)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I am at a loss to explain these stylistic variations, so I conclude that
 R users can follow either style, while keeping in mind the 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

} else {
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 policy, which strongly pushes us toward the K&R style.
\end_layout

\begin_layout Section
How to name functions.
\end_layout

\begin_layout Standard
Now we begin to consider some issues that are more subjective.
 Many styles are legal, but some are more easily understood.
 R syntax has changed over the years, and some things that were illegal
 are now allowed.
 And some styles that were standard might now be discouraged.
 
\end_layout

\begin_layout Subsection
(.98 SEA) Avoid using names that are already in use by R, especially common
 ones.
\end_layout

\begin_layout Standard
Don't write functions named 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

rep()
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

seq()
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset


\begin_inset listings
inline true
status open

\begin_layout Plain Layout

c()
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

, and so forth.
 Notice that my new function 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

lm()
\end_layout

\end_inset

 does not obliterate the one from the stats package, but it sure does make
 it harder to use it.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> lm <- function(z) print("Hi, I'm z where lm was")
\end_layout

\begin_layout Plain Layout

> x <- rnorm(100)
\end_layout

\begin_layout Plain Layout

> y <- rnorm(100)
\end_layout

\begin_layout Plain Layout

> lm (y ~ x)
\end_layout

\begin_layout Plain Layout

[1] "Hi, I'm z where lm was"
\end_layout

\begin_layout Plain Layout

> stats::lm(y ~ x)
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

Call:
\end_layout

\begin_layout Plain Layout

stats::lm(formula = y ~ x)
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

Coefficients:
\end_layout

\begin_layout Plain Layout

(Intercept)            x
\end_layout

\begin_layout Plain Layout

    0.02688      0.01796
\end_layout

\end_inset


\end_layout

\begin_layout Standard
As long as we remember that 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

lm()
\end_layout

\end_inset

 is in the namespace stats, we can find it.
 
\end_layout

\begin_layout Standard
Similarly, packages can declare namespaces of their own.
 (Since R version 2.14, all packages 
\emph on
must
\emph default
 do so.) We are allowed to place a new function like 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

seq()
\end_layout

\end_inset

 or 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

lm()
\end_layout

\end_inset

 into a package if we want to.
 Nevertheless, almost everybody will hate to read code like that.
 
\end_layout

\begin_layout Standard
The danger that user functions might interfere with core functionality was
 at one time very serious.
 Now it is, for the most part, a historical footnote.
 It is still possible to obliterate a function that is embedded within a
 namespace, but doing so requires a bit of effort and mischief.
\begin_inset Foot
status collapsed

\begin_layout Plain Layout
In case you wonder, here's how to cause the worst case scenario.
\end_layout

\begin_layout LyX-Code
nseq <- function(x) print("Hello, good to see you")
\end_layout

\begin_layout LyX-Code
assignInNamespace("seq.default", nseq, "base")
\end_layout

\end_inset


\end_layout

\begin_layout Standard
When we say that a namespace is imported, it means that all of the functions
 in that namespace can be accessed by the function's name, without the namespace
 name as a prefix.
 We might write 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

base::seq(1, 10, length.out = 40)
\end_layout

\end_inset

 to be clear, but we need only write 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

seq(1, 10, length.out = 40)
\end_layout

\end_inset

 because an R session imports the namespace base.
 I notice a trend in R to suggest that one should not import whole namespaces
 unless that is truly necessary, and even if a namespace is imported, we
 should strive for clarity by using syntax that includes the namespace name.
 In the source code for many R examples, one will find syntax like 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

graphics::par()
\end_layout

\end_inset

 where, until recently, that would have simply been 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

par()
\end_layout

\end_inset

.
\end_layout

\begin_layout Subsection
(.65 SEA)Use periods to indicate classes, otherwise don't use periods in
 function names.
 
\end_layout

\begin_layout Standard
Instead, use camel case to name functions.
 This function name 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

mySuperThing()
\end_layout

\end_inset

 is better than 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

my.super.thing()
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Standard
The period in a function name has a special meaning in the S3 object-oriented
 framework.
 A 
\begin_inset Quotes eld
\end_inset

generic function
\begin_inset Quotes erd
\end_inset

 (such as print() or summary()) is accompanied by methods that implement
 its work for particular kinds of objects, such as 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

print.function()
\end_layout

\end_inset

 or 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

print.lm()
\end_layout

\end_inset

.
 Before the period, we have a function's name, and after the period, we
 have the class name of the object being managed.
 The function name 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

my.super.thing()
\end_layout

\end_inset

 suggests the user might have an object of class 
\begin_inset Quotes eld
\end_inset

thing
\begin_inset Quotes erd
\end_inset

 and that 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

my.super(x)
\end_layout

\end_inset

 would diagnose the class of x and send the work to 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

my.super.thing()
\end_layout

\end_inset

.
 A camel cased function name 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

mySuperThing()
\end_layout

\end_inset

 will not convey the wrong meaning.
 
\end_layout

\begin_layout Standard
If we were starting with a clean slate, I believe many R functions would
 be re-named for the purposes of consistency.
 Since we do not have a clean slate, we live with an accumulation of function
 names from olde S and R.
 Changes in computer science–the growth of object-oriented programming–cause
 new naming conventions.
 Consider some of the traditional S function names that are still used in
 R, like 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

read.table
\end_layout

\end_inset

 and 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

read.csv
\end_layout

\end_inset

.
 Those are not method implementations of a generic function read().
 The period is simply part of a shorthand of the form 
\begin_inset Quotes eld
\end_inset

action.qualifier
\begin_inset Quotes erd
\end_inset

.
 Otherwise, if one had an object of type table, then read(x) would call
 read.table(x).
 But it does not:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> example(table)
\end_layout

\begin_layout Plain Layout

> class(tab) 
\end_layout

\begin_layout Plain Layout

[1] "xtabs" "table"
\end_layout

\begin_layout Plain Layout

> read(tab) 
\end_layout

\begin_layout Plain Layout

Error: could not find function "read" 
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I believe that, if these functions were being created today, they would
 be named 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

readTable()
\end_layout

\end_inset

 and 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

readCSV()
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Standard
In the R source code, there are some very confusing function names and I
 have a hard time believing we would use them if we were re-designing everything
 today.
 The file src/library/base/readhttp.R has a function called 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

url.show()
\end_layout

\end_inset

, which follows none of the styles that I recognize.
 There's no class 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

show
\end_layout

\end_inset

 and 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

url()
\end_layout

\end_inset

 is not a generic function.
 In the 
\begin_inset Quotes eld
\end_inset

action.qualifier
\begin_inset Quotes erd
\end_inset

 tradition, it would be 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

show.url()
\end_layout

\end_inset

.
 And why not 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

showURL()
\end_layout

\end_inset

? I hasten to point out that the same file includes some camel cased functions
 like 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

defaultUserAgent()
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
I like camel cased function names.
 They are common in Objective-C and Java.
 Some programmers vigorously disagree.
 Programmers trained in C++ seem to hate camel case names, almost at a visceral
 level.
 As a result, we find a division of opinion on function names.
 As a spot check, consider two of my favorite packages, MASS 
\begin_inset CommandInset citation
LatexCommand citep
key "venables_modern_2002"
literal "true"

\end_inset

 and car 
\begin_inset CommandInset citation
LatexCommand citep
key "fox_r_2011"
literal "true"

\end_inset

.
 There are not many camel case function names in the MASS, where we find
 brief names in lower case letters (such as 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

boxcox()
\end_layout

\end_inset

).
 In contrast, car calls that 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

boxCox()
\end_layout

\end_inset

.
 When I started using R, Professor Fox used function names with periods,
 but he has been systematically weeding them out and replacing them with
 camel case names.
 If those two packages are counterbalancing each other in my mind (for and
 against camel case functions), the leading packages for mixed effects models,
 nlme 
\begin_inset CommandInset citation
LatexCommand citep
key "pinheiro_nlme:_2012"
literal "true"

\end_inset

 and lme4 
\begin_inset CommandInset citation
LatexCommand citep
key "lme4"
literal "true"

\end_inset

, weigh in on the camel case side of the ledger.
 
\end_layout

\begin_layout Standard
In conclusion, users should avoid gratuitous periods in function names because,
 after S3, the period has special meaning in R.
 When a function has been declared as a generic, then that function's name
 followed by a period has an object-oriented meaning.
 A period is not merely word separation.
 New functions introduced in R tend to use either camel case names (
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

browseVignettes()
\end_layout

\end_inset

) or underscores (
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

get_all_vars()
\end_layout

\end_inset

).
 Considering recent additions to R, I believe that the chance of finding
 a decorative period in a new function name is almost zero.
 But we are still living with an awful lot of older counter-examples.
\end_layout

\begin_layout Section

\series bold
How to name variables
\series default
 (and objects).
\end_layout

\begin_layout Subsection
(1.0 SEA) Follow the 
\begin_inset Quotes eld
\end_inset

letters and numbers
\begin_inset Quotes erd
\end_inset

 rule.
\end_layout

\begin_layout Standard
R variable names must 
\end_layout

\begin_layout Enumerate
begin with an alphabetical character
\end_layout

\begin_layout Enumerate
include only letters, numbers and the symbols 
\begin_inset Quotes eld
\end_inset

_
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

.
\begin_inset Quotes erd
\end_inset

.
 
\end_layout

\begin_layout Standard
They must not include 
\begin_inset Quotes eld
\end_inset

*
\begin_inset Quotes erd
\end_inset

,
\begin_inset Quotes erd
\end_inset

?
\begin_inset Quotes erd
\end_inset

,
\begin_inset Quotes erd
\end_inset

!
\begin_inset Quotes erd
\end_inset

,
\begin_inset Quotes erd
\end_inset

&
\begin_inset Quotes erd
\end_inset

 or other special symbols.
 They must not include spaces.
\end_layout

\begin_layout Standard
One peculiar side effect of this rule is that the ellipsis symbol, three
 periods, 
\begin_inset Quotes eld
\end_inset

...
\begin_inset Quotes erd
\end_inset

, is actually a legal object name.
 That's three periods, which is just as legal as aaa or bbb.
 Many R functions allow the argument 
\begin_inset Quotes eld
\end_inset

...
\begin_inset Quotes erd
\end_inset

, most users don't realize it literally is a word.
 When that is listed as a function argument, then any argument that the
 user includes is gobbled up by 
\begin_inset Quotes eld
\end_inset

...
\begin_inset Quotes erd
\end_inset

.
 
\end_layout

\begin_layout Subsection
(1.0 SEA) Never name a variable T or F.
 
\end_layout

\begin_layout Standard
Almost everybody (99.999%) will agree with this.
 These are too easily mistaken for TRUE and FALSE values.
 Since R uses TRUE and FALSE as vital elements of almost all commands and
 functions, and since users are allowed to abbreviate those as T or F, a
 horrible confusion can develop if variables are named T or F.
 
\end_layout

\begin_layout Standard
Here's some good news.
 R will not allow users to name variables TRUE or FALSE:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> TRUE <- 7
\end_layout

\begin_layout Plain Layout

Error in TRUE <- 7 : invalid (do_set) left-hand side to assignment
\end_layout

\end_inset


\end_layout

\begin_layout Standard
But R will not prevent the usage of T and R for variable names.
\end_layout

\begin_layout Subsection
(.75 SEA) Avoid declaring variables that have the same names as widely used
 functions.
\end_layout

\begin_layout Standard
This is just a handy rule of thumb now, but it used to be a 
\begin_inset Quotes eld
\end_inset

watch out for that tree!
\begin_inset Quotes erd
\end_inset

 warning.
 In 2001, I created a variable 
\begin_inset Quotes eld
\end_inset

rep
\begin_inset Quotes erd
\end_inset

 (for Republican party members) and nothing worked in my program.
 In exasperation, I wrote to the r-help list, and learned that I had obliterated
 R's own function 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

rep()
\end_layout

\end_inset

 with my variable.
 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

rep()
\end_layout

\end_inset

 is used inside many R functions and thus obliterating it was a very serious
 mistake.
 In 2002 or so, the R system was revised so that user-declared variables
 cannot 
\begin_inset Quotes eld
\end_inset

step on
\begin_inset Quotes erd
\end_inset

 R system functions.
 Nevertheless, it is disconcerting to me (probably others) when users create
 variables with names like 
\begin_inset Quotes eld
\end_inset

lm
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

rep
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

seq
\begin_inset Quotes erd
\end_inset

, and so forth.
 
\end_layout

\begin_layout Subsection
(0.50 SEA) Use long names for infrequently used variables.
 
\end_layout

\begin_layout Standard
And use short names for variables that will be used very often.
\end_layout

\begin_layout Standard
If a variable is going to be used twice, we might as well be verbose about
 it.
 
\begin_inset Quotes eld
\end_inset

xlog
\begin_inset Quotes erd
\end_inset

 is better than 
\begin_inset Quotes eld
\end_inset

xl
\begin_inset Quotes erd
\end_inset

, if we are only writing it a few times.
 If we are going to use a name 50 times in a 5 line program, we should choose
 a short one.
 For abbreviations, include a comment to remind the reader what the thing
 stands for.
\end_layout

\begin_layout Subsection
(0.10 SEA) Suggested naming scheme: keep related objects in an alphabetically
 sorted scheme.
\end_layout

\begin_layout Standard
This is my personal naming scheme.
 Nobody but me follows this policy now, but I like it so much I'm tacking
 it onto the end of this essay.
 I believe that R code is much more readable if objects that 
\begin_inset Quotes eld
\end_inset

go together
\begin_inset Quotes erd
\end_inset

 begin with the common series of letters.
 As seen by ls(), the related pieces should always be together.
 From now on, when I work with a variable named 
\begin_inset Quotes eld
\end_inset

x
\begin_inset Quotes erd
\end_inset

, then all transformations will begin with 
\begin_inset Quotes eld
\end_inset

x
\begin_inset Quotes erd
\end_inset

.
 I will use 
\begin_inset Quotes eld
\end_inset

xlog
\begin_inset Quotes erd
\end_inset

 rather than 
\begin_inset Quotes eld
\end_inset

logx
\begin_inset Quotes erd
\end_inset

 and so forth.
 
\end_layout

\begin_layout Standard
Example 1.
 Create a numeric variable, recode it as a factor, then create the 
\begin_inset Quotes eld
\end_inset

dummy
\begin_inset Quotes erd
\end_inset

 variables that correspond.
 I include the output in order to emphasize the clarity due to the alphabetical
 emphasis:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

x <- runif(1000, min = 0, max = 100)
\end_layout

\begin_layout Plain Layout

xf <- cut(x, breaks = c(-1, 20, 50, 80, 101), labels = c("cold", "luke",
 "warm", "hot"))
\end_layout

\begin_layout Plain Layout

xfdummies <- contrasts(xf, contrasts = FALSE )[xf,]
\end_layout

\begin_layout Plain Layout

colnames(xfdummies) <-  paste("xf", c("cold", "luke", "warm", "hot"), sep="")
\end_layout

\begin_layout Plain Layout

rownames(xfdummies) <- names(x)
\end_layout

\begin_layout Plain Layout

dat <- data.frame(x, xf, xfdummies)
\end_layout

\begin_layout Plain Layout

head(dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I have not included the output of these code chunks, but the alphabetical
 emphasis is demonstrated in them.
\end_layout

\begin_layout Standard
Example 2.
 Estimate a regression, calculate summary information.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T, eval=F>>=
\end_layout

\begin_layout Plain Layout

set.seed(12345)
\end_layout

\begin_layout Plain Layout

x1 <- rnorm(200, m = 300, s = 140)
\end_layout

\begin_layout Plain Layout

x2 <- rnorm(200, m = 80, s = 30)
\end_layout

\begin_layout Plain Layout

y <- 3 + 0.2 * x1 + 0.4 * x2 + rnorm(200, s=400)
\end_layout

\begin_layout Plain Layout

dat <- data.frame(x1, x2, y); rm(x1,x2,y)
\end_layout

\begin_layout Plain Layout

m1 <- lm (y ~ x1 + x2, data = dat)
\end_layout

\begin_layout Plain Layout

m1summary <- summary(m1)
\end_layout

\begin_layout Plain Layout

m1se <- m1summary$sigma
\end_layout

\begin_layout Plain Layout

m1rsq <- m1summary$r.squared
\end_layout

\begin_layout Plain Layout

m1coef <- m1summary$coef
\end_layout

\begin_layout Plain Layout

m1aic <- AIC(m1)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Example 3.
 Run a regression, collect mean-centered and residual centered variants
 of it.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ps10, fig=T, eval=F, height=5, width=9>>=
\end_layout

\begin_layout Plain Layout

library(rockchalk)
\end_layout

\begin_layout Plain Layout

dat$y2 = with(dat, 3 + 0.02 * x1 + 0.05 * x2 + 2.65 * x1 *x2 + rnorm(200, s=4000))
\end_layout

\begin_layout Plain Layout

par(mfcol=c(1,2))
\end_layout

\begin_layout Plain Layout

m1 <- lm(y2 ~ x1 + x2, data = dat)
\end_layout

\begin_layout Plain Layout

m1i <- lm(y2 ~ x1 * x2, data = dat)
\end_layout

\begin_layout Plain Layout

m1ps <- plotSlopes(m1, plotx = "x1", modx = "x2")
\end_layout

\begin_layout Plain Layout

m1ips <- plotSlopes(m1i, plotx = "x1", modx = "x2")
\end_layout

\begin_layout Plain Layout

m1imc <- meanCenter(m1i)
\end_layout

\begin_layout Plain Layout

m1irc <- residualCenter(m1i)	
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Conclusion
\end_layout

\begin_layout Standard
R can be understood at several levels, varying in sophistication from an
 elementary statistics course or to an advanced platform for the development
 of computer programming concepts.
 In the future, I will be more cautious to teach new R users about coding
 style.
 I intend to prevent the accumulation of bad habits that result in code
 that is difficult to read and hard to debug.
 
\end_layout

\begin_layout Standard
Users who ask for help in the r-help email list 
\begin_inset Foot
status open

\begin_layout Plain Layout
\begin_inset Flex URL
status open

\begin_layout Plain Layout

http://www.r-project.org/mail.html
\end_layout

\end_inset


\end_layout

\end_inset

 or on web forums 
\begin_inset Foot
status open

\begin_layout Plain Layout
e.g., 
\begin_inset Flex URL
status open

\begin_layout Plain Layout

http://stackoverflow.com/questions/tagged/r
\end_layout

\end_inset


\end_layout

\end_inset

 are well advised to remember the importance of style.
 Most newcomers believe that the experts will understand what they write,
 but that's not true.
 Experts will find it much easier to spot errors in code that has the correct
 indentation and uses a proper naming scheme for variables and functions.
 In my experience, the most likely source of trouble in R code is not actually
 the style, but rather poor compartmentalization of separate calculations.
 The potential to compartmentalize, however, is obscured by bad style.
 
\end_layout

\begin_layout Standard
When users throw together 2000 lines of spaghetti code with no indentation
 (I can point to examples on CRAN), there's almost no chance than anyone
 except the author will be able to understand and extend that kind of code.
 Ugly code writers will respond, 
\begin_inset Quotes eld
\end_inset

my ugly code runs!
\begin_inset Quotes erd
\end_inset

 That misses the point.
 Coding style is not about making things 
\begin_inset Quotes eld
\end_inset

work,
\begin_inset Quotes erd
\end_inset

 it is about making them work in a way that is understood by the widest
 possible audience.
 And where possible, the code should be re-usable and extended to other
 purposes.
 
\end_layout

\begin_layout Standard
\begin_inset CommandInset bibtex
LatexCommand bibtex
bibfiles "rockchalk"
options "chicago"

\end_inset


\end_layout

\end_body
\end_document

%% LyX 2.3.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[american,noae]{scrartcl}
\usepackage{lmodern}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage{geometry}
\geometry{verbose,tmargin=1in,bmargin=1in,lmargin=1in,rmargin=1in}
\usepackage{float}
\usepackage{url}
\usepackage{amsmath}
\usepackage[authoryear]{natbib}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
<<echo=F>>=
  if(exists(".orig.enc")) options(encoding = .orig.enc)
@
\providecommand*{\code}[1]{\texttt{#1}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
%\VignetteIndexEntry{Using rockchalk}

\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{figure}
 }{%
   \@float{figure}[#1]%
 }%
 \centering
}{%
 \end@float
}

\renewenvironment{table}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{table}
 }{%
   \@float{table}[#1]%
 }%
 \centering
%  \setlength{\@tempdima}{\abovecaptionskip}%
%  \setlength{\abovecaptionskip}{\belowcaptionskip}%
% \setlength{\belowcaptionskip}{\@tempdima}%
}{%
 \end@float
}


%\usepackage{listings}
% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, %,style=Rstyle}
                        fancyvrb=false,escapechar=`,language=R,%
                        %%basicstyle={\Rcolor\Sweavesize},%
                        backgroundcolor=\Rbackground,%
                        showstringspaces=false,%
                        keywordstyle=\Rcolor,%
                        commentstyle={\Rcommentcolor\ttfamily\itshape},%
                        literate={<<-}{{$\twoheadleftarrow$}}2{~}{{$\sim$}}1{<=}{{$\leq$}}2{>=}{{$\geq$}}2{^}{{$^{\scriptstyle\wedge}$}}1{==}{{=\,=}}1,%
                        alsoother={$},%
                        alsoletter={.<-},%
                        otherkeywords={!,!=,~,$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},%
                        escapeinside={(*}{*)}}%


% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\scriptsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.90}}

\usepackage{dcolumn}
\usepackage{siunitx}

\makeatother

\usepackage{babel}
\usepackage{listings}
\renewcommand{\lstlistingname}{\inputencoding{latin9}Listing}

\begin{document}
\title{Using rockchalk for Regression Analysis\thanks{In case you want to be involved in development, the source code is
available on GitHub. Please browse http://github.com/pauljohn32. The
read-only git archive address is \protect\url{git://github.com/pauljohn32/rockchalk.git}.
Once you learn how to ``clone'' the repository with a git client,
then we can discuss ways for you to contribute.}}
\author{Paul E. Johnson}
\maketitle
\begin{abstract}
The rockchalk package includes functions for estimating regressions,
extracting diagnostic information from them, preparing presentable
tables, and creating helpful two and three dimensional plots. It is
primarily intended to facilitate teachers and students who are conducting
ordinary least squares and elementary generalized models. Compared
to other packages that help with regression analysis, this one offers
more-than-the-usual amount of emphasis on the analysis of regression
models that include interaction terms. It includes functions that
can receive a fitted model and then return standardized, mean-centered,
and residual-centered regression results. The plotting functions address
some limitations of R's \code{termplot} function. Version 1.8 introduces
a consistent framework for the creation of ``\code{newdata}'' objects
and the calculation of predicted values so that the marginal effects
of separate predictors can be easily explored.
\end{abstract}
% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\SweaveOpts{ae=F,nogin=T}

<<Roptions, echo=F>>=
options(device = pdf)
options(width=80, prompt=" ", continue="  ")
options(useFancyQuotes = FALSE) 
@

\section{Introduction}

This is the Spring, 2019 update of the rockchalk package. I offer
a course in regression analysis for social and behavioral scientists
every year. As the course goes on, I keep track of the difficulties
that the students experience with R and I craft functions that facilitate
their work on particular assignments. The new features in this edition
are the functions \code{descriptiveTables()} and \code{waldt()}
(for regression reports), tools for regression simulation, including
\code{genCorrelatedData3()} and \code{cutFancy()}, some support
for common recoding support for multi-level modeling. Finally, the
regression table-writing function the \code{outreg()} can now generate
decimal-aligned columns. A new vignette demonstrating the \code{outreg()}
function is introduced with the package.

As in previous versions, a core of the effort is development of an
internally coherent framework that allows students to estimate regressions
(usually with lm and glm) and then create useful summary tables and
diagnostic reports. It is easy to create ``newdata'' objects and
interact with fitted regressions. The \code{newdata()} function will
be handy for just about any kind of regression. For the common kinds
of regression in base R and lme4, there is also a convenience function
called \code{predictOMatic()} that allows one to specify a regression
model and then receive predicted values for a range of input values.
The default output will be similar to other regression software tools
that provide ``marginal effect'' estimates for regression analysis. 

I have exerted a great deal of effort to improve the consistency of
predicted value plots for regression analysis. The \code{plotSlopes()}
function is a plotter for linear models with interaction terms. It
draws a plot on the screen, but it also creates an output object that
can be subjected to a follow-up hypothesis test (by the \code{testSlopes()}
function). There is a plot method for testSlopes objects to help with
analysis of interactions. 

For people who want to plot nonlinear regressions, the function \code{plotCurves()}
is a replacement for \code{plotSlopes()}. It can handle any sort
of nonlinearity and interaction on the right hand side. Where some
regression formula can cause R's \code{termplot()} function to fail,
\code{plotCurves()} will succeed.

The core functionality of many functions has not been altered. Functions
to provide standardized regression coefficients, mean-centered, or
residual-centered regression coefficients have not changed much, although
the plots of them are improved. 

\section{Facilitating Collection of Summary Information}

\subsection{summarize: A replacement for summary}

When an R function provides output that is not suitable for a term
paper, the student must cobble together some code and assemble a customized
table. In my opinion, R's summary() function for data frames is not
adequate. It lacks summaries for diversity of observations. In addition,
the output is not formatted in a way that is conducive to the creation
of plots or tables. As a result, I offer the function summarize(),
which separates the numeric from factor variables and provides an
alphabetized summary. 

<<echo=F>>=
library(rockchalk)
@

Consider the results of applying summarize() to Fox's Chile data set
from the carData package:

<<echo=T>>=
library(carData)
data(Chile)
(summChile <- summarize(Chile))
@

The result object is a list that includes a data frame for the numeric
variables, with named rows and (optionally) alphabetized columns,
as well as a separate report for each factor variable. Users who wish
to summarize only the numeric variables can run summarizeNumerics()
instead, while others who want to summarize only factors can run summarizeFactors().
The output from summarizeFactors() is a list of factor summaries. 

A companion function is centralValues(), which will provide only one
number for each variable in a data frame. For numerics, it returns
the mean, while for factor variables, it returns the mode. 

<<echo=T>>=
centralValues(Chile)
@

\subsection{Easier predictions and newdata objects}

Students struggle with R's predict() methods. The primary challenge
is in the creation of a newdata object of interesting values of the
predictors. If the newdata argument, here myNewDF in \inputencoding{latin9}\lstinline!predict(m1, newdata = myNewDF)!\inputencoding{utf8}
is not exactly right, the command will fail. If the regression formula
uses functions like ``as.numeric()'' or ``as.factor()'', its almost
impossible for first-time R users to get this right. In version 1.8,
I believe I have solved the problem entirely with the functions newdata()
and predictOMatic(). 

Let's fit an example regression with the Chile data set from the carData
package.

<<echo=T>>=
m1 <- lm(statusquo ~ age + income + population + region + sex, data = Chile)
@

\noindent The default output of predictOMatic() will cycle through
the predictors, one by one.

<<echo=T>>=
m1pred <- predictOMatic(m1)
m1pred
@

The newdata() and predictOMatic() functions handle the details and
allow the user to adjust their requests to allow for very fine grained
control. Here are the key arguments for these functions.
\begin{enumerate}
\item divider. The name of an algorithm to select among observed values
for which predictions are to be calculated. These are the possibilities.

\begin{description}
\item [{``seq''}] an evenly spaced sequence of values from low to high
across the variable.
\item [{``quantile''}] quantile values that eminate from the center of
the variable ``outward''. 
\item [{``std.dev.''}] the mean plus or minus the standard deviation,
or 2 standard deviations, and so forth.
\item [{``table''}] when a variable has only a small set of possible
values, this selects the most frequently observed values.
\end{description}
\item n. The number of values to be selected.
\item predVals. Where the divider argument sets the default algorithm to
be used, the predVals argument can choose variables for focus and
select divider algorithms separately for the variables. It is also
allowed to declare particular values.
\end{enumerate}
The user can request that particular values of the predictors are
used, or can declare one of several algorithms for selection of focal
values. All of these details are managed by the argument called predVals,
which is described in the help pages (with plenty of examples). 

<<echo=T>>=
mypred2 <- predictOMatic(m1, predVals = c("age", "region"), n = 3)
mypred2
mypred3 <- predictOMatic(m1, predVals = c(age = "std.dev.", region = "table"), n = 3)
mypred3
mypred4 <- predictOMatic(m1, predVals = list(age = c(18, 30, 45), region = c("SA", "C","N")), n = 3)
mypred4
@

I've invested quite a bit of effort to make sure this works dependably
with complicated regression formulae. All formulae, such as y \textasciitilde{}
x1 + log(x2 + alpha) + ploy(x3, d) will just work. (In case one is
interested to know \emph{why} this works, the secret recipe is a new
function called model.data(). Under the hood, this required some hard
work, a frustrating chain of trial and error that is discussed in
the vignette \emph{Rchaeology}, which is distributed with this package).

The \code{predictOMatic()} function will work when the regression
model provides a predict function that can handle the newdata argument.
If the regression package does not provide such a function, the user
should step back and use the newdata function to create the examplar
predictor data frames and then calculate predicted values manually.
The newdata() function will set all variables to center values and
then it will create a ``mix and match'' combination of the ones
that the user asks for. This sequence shows ways to ask for various
``mix and match'' combinations of values from age and region. 

<<echo=T>>=
mynewdf <- newdata(m1, predVals = c("age","region"), n = 3)
mynewdf
@

<<echo=T>>=
mynewdf2 <- newdata(m1, predVals = list(age = "std.dev.", region = c("SA", "C","N")))
mynewdf2
@

<<echo=T>>=
mynewdf3 <- newdata(m1, predVals = list(age = c(20, 30, 40), region = c("SA", "C","N")))
mynewdf3
@\\
Of course, functions from rockchalk or any other R package can be
placed into the process for choosing focal values. 

<<echo=T>>=
mynewdf <- newdata(m1, predVals = list(age = getFocal(Chile$age, n = 3), region = getFocal(Chile$region, n = 3)))
mynewdf
@\\
The function getFocal() is a generic function; it will receive variables
of different types and ``do the right thing.'' By default, focal
values of numeric variables are quantile values. For factor values,
the most frequently observed values are selected. These are customizable,
as explained in the documentation. The \code{newdata()} output can
be used in a predict() function call as demonstrated above. 

It would be nice if every regression model's predicted values were
accompanied by 95\% confidence intervals. Models fit by \code{lm()}
can supply confidence intervals, but not \code{glm()}. At the current
time, there are many competing methods that might be used to calculate
those intervals; predict.glm() in R's stats package avoids the issue
entirely by not calculating intervals. In rockchalk-1.8, I crafted
some code to calculate confidence intervals for glm objects using
the (admittedly crude) Wald-based approximation. In the scale of the
linear predictor, we calculate a 95\% confidence interval, and then
use the inverse link function to transform that onto the space of
the observed response. In this example, I replicate an example that
is an R classic, from the help page of predict.glm. The reader will
note that the output includes a warning about the construction of
the confidence interval.

<<echo=T>>=
df <- data.frame(ldose = rep(0:5, 2), sex = factor(rep(c("M", "F"), c(6, 6))), 
	SF.numdead = c(1, 4, 9, 13, 18, 20, 0, 2, 6, 10, 12, 16))      
df$SF.numalive <-  20 - df$SF.numdead
budworm.lg <- glm(cbind(SF.numdead, SF.numalive) ~ sex*ldose, data = df,  family = binomial)
predictOMatic(budworm.lg, predVals = c(ldose = "std.dev.", sex = "table"), interval = "confidence")  
@

\subsection{Descriptive Tables}

In regression-based reports, it is common for authors to present a
brief summary table to represent the data. The \code{descriptiveTable()}
function was created to make that process easier and more predictable.
The aim is to provide the usual summary statistics for the numeric
predictors and a table of proportions for the categorical predictors.
See Table \ref{tab:descriptiveTable}. The user can customize the
requested statistical summaries for numeric predictors.

\begin{table}[H]
\caption{The descriptiveTable function\label{tab:descriptiveTable}}

<<>>=
descriptiveTable(m1, digits=6)
@
\end{table}
\noindent I my opinion, the output is, well, simple and understandable.
More importantly, it will be sufficient for a term paper, once it
is converted into a presentable final format. It can be converted
into HTML or LaTeX with various R packages, such as xtable. Note that
because this descriptive table is derived from the fitted regression,
it summarizes \code{\noindent only the cases} that were actually
used in the regression analysis. If cases were lost by listwise deletion,
the deleted cases are not taken into account here.

\section{Better Regression Tables: Some outreg Examples.}

On May 8, 2006, Dave Armstrong, who was a political science PhD student
at University of Maryland, posted a code snippet in r-help that demonstrated
one way to use the ``cat'' function from R to write \LaTeX{} markup.
That gave me the idea to write a \LaTeX{} output scheme that would
help create some nice looking term and research papers. I wanted ``just
enough'' information, but not too much.

Since 2006, many new R packages have been introduced for the creation
of regression tables, but I still prefer to maintain \code{outreg}.
I fight to keep this simple, but have added some features in response
to user requests. For me, the biggest question is whether the format
should be ``wide'' or ``tight''. A tight format stacks estimates
and standard errors into a single column, while a wide format has
them in separate columns. In response to user requests, stars for
statistical significance were made customizable. It is now allowed
to directly insert vectors of parameter estimates and standard errors.
In this version, a new parameter \code{centering} is introduced for
decimal-centered columns. 

In the following, I will demonstrate some tables for lm and glm fits
on a simulated data set. This new simulation function, genCorrelatedData3(),
is a convenient way to create multiple-predictor regression data sets
of arbitrary size and complexity, allowing for interactions and nonlinearity. 

<<createdata1, echo=T>>=
set.seed(1234)
dat <- genCorrelatedData3(N = 100, means = c(x1 = 0, x2 = 10, x3 = 0), sds = c(1, 2, 1), rho = c(0, 0, 0), stde = 10, beta = c(0, -3, 4, 0), verbose = FALSE)
@\\
That creates a new matrix with variables $x1,$ $x2$, $x3$, and
y. We run some linear regressions and then create a categorical output
variable for a logistic regression.

<<createdata1, echo=T>>=
m1 <- lm(y ~ x1 + x2, data = dat)
m2 <- lm(y ~ x2, data = dat)
m3 <- lm(y ~ x1 + x2 + x3, data = dat)
## Create categorical variant
myilogit <- function(x) exp(x)/(1 + exp(x))
dat$y3 <- rbinom(100, size = 1, p = myilogit(scale(dat$y)))
gm1 <- glm(y3 ~ x1 + x2, data = dat)
@

The outreg examples are offered in Tables \ref{tab:Tab1} through
\ref{tab:Combined-OLSGLM}. Table \ref{tab:Tab1} is the ``tight
format output for three models, obtained from \code{outreg(list(m1, m2, m3), centering=\textquotedbl siunitx\textquotedbl )}.
On the other hand, as illustrated in Table \ref{tab:Tab2}, the author
can request a wide table (tight = FALSE) and provide more elaborate
model labels and adjust the significance stars. In Table \ref{tab:Combined-OLSGLM},
observe that the linear and generalized linear model output peacefully
co-exist, side-by-side. This output is, in my opinion, completely
acceptable for inclusion in a professional presentation or conference
paper. There are some warts in this output. 

The default output from \code{outreg} will have left-aligned columns.
Because users requested decimal-centered columns, the argument centering
has been introduced. If \code{centering=\textquotedbl siunitx\textquotedbl}
or \code{centering=TRUE}, the \code{outreg} table will be adjusted
to use features in the LaTeX package ``\code{siunitx}''. The LaTeX
package dcolumn is probably more familiar to users, and \code{\textquotedbl dcolumn\textquotedbl}
is also a legal value for centering, but the results are not quite
as good. As a \textbf{caution}, I hasten to mention that if a user
asks for decimal-centered columns, the user has the duty to insert
into the document preamble either ``\code{\textbackslash usepackage\{siunitx\}}''
or ``\code{\textbackslash usepackage\{dcolumn\}}''.

\begin{table}
\caption{Decimal-centered columns in the ``tight'' style\label{tab:Tab1}}

<<outreg10, echo = F, results=tex>>="
or <- outreg(list(m1, m2, m3), centering = "siunitx")
@
\end{table}
\begin{table}
\caption{The wide format obtained with tight = FALSE\label{tab:Tab2}}

<<outreg20, results=tex, echo=F>>=
or1 <- outreg(list("The First Model with a Long Title" = m1, "Another Model" = m3), tight = FALSE, alpha = c(0.05, 0.01, 0.001), centering = "siunitx")
@
\end{table}
\begin{table}
\caption{Combined OLS and GLM Estimates\label{tab:Combined-OLSGLM}}

<<outreg70, results=tex, echo=F>>=
or2 <- outreg(list(m1,gm1), modelLabels = c("OLS:y","GLM: Categorized y"))
@
\end{table}
I understand that some authors need to include regression tables in
documents that are not prepared with LaTeX. The parameter type = ``HTML''
will change the output format to HTML. An HTML file can be displayed
in a web browser and it can be imported in traditional ``word processor''
programs.

\section{Plotting Regressions with Interactions }

\subsection{Interaction in Linear Regression. }

One of the most fundamental skills in regression analysis is the interpretation
of interactive predictors. It is much easier for students to understand
the effect of an interaction if they can create a nice plot to show
how the predicted value depends on various values of a predictor.
The \code{plotSlopes()} function was introduced in 2010 when I was
teaching a large first-year graduate course (more than 50 students)
and it became apparent that about 20 percent of them would not be
able to manage the R coding required to draw several lines on a single
plot. Unfortunately, R's termplot() function will not draw regressions
involving interactions.

The rockchalk package has two functions to help with this, \code{plotSlopes()}
and \code{plotCurves()}. \code{plotCurves()} is more general, it
can handle any kind of formula that the user estimates. \code{plotSlopes()}
is more limited, it is only for lm objects. In return for that limitation,
\code{plotSlopes()} creates an output object which can be used to
conduct post-hoc hypothesis tests. 

\begin{figure}
<<ps05, fig=T, echo=F, include=F, height=5, width=6>>=
m1ps <- plotSlopes(m1, plotx = "x2", xlab = "x2 from model m1", interval = "confidence", opacity = 80, col = "red",
ylim = c(-70, 15), legendArgs = list(x="topright"))
@

\includegraphics[height=4in]{rockchalk-ps05}

\caption{plotSlopes: Linear Model with Confidence Interval\label{fig:ps05}}
\end{figure}
At its most elementary level, \code{plotSlopes()} is a ``one step''
regression line plotter. If the regression model includes more than
one predictor, then a single predictor is displayed on the horizontal
axis and the other predictors are set on their central values. A plot
for the model m1, that was illustrated above, is presented in Figure
\ref{fig:ps05}. In rockchalk-1.8, new arguments were added to allow
the ``see though'' confidence region. The command to generate Figure
\ref{fig:ps05} was

<<echo=T, eval=F>>=
<<ps05>>
@

\noindent I've adjusted the color and opacity to illustrate the usage
of those arguments. The y range is adjusted to make a little extra
room for the legend. The \code{\noindent plotSlopes()} function is
very flexible. All of the label, color, and scale arguments of a plot
function are also available. The plotSlopes function also works well
if the moderator is a categorical variable. 

It is important to note that the output object, m1ps, has the information
necessary to re-create the plotted line in the form of a \code{newdata}
data frame. The first few lines in the \code{newdata} object are

<<>>=
m1ps$newdata[1:3, ]
@

<<ps09, fig=F, echo=F, height=9, width=6>>=
dat$y4 <- 1 + 0.1 * dat$x1 - 6.9 * dat$x2 + 0.5 * dat$x1*dat$x2 + 0.2 * dat$x3 + rnorm(100, m = 0, sd = 10)
m4 <- lm(y4 ~ x1*x2 + x3, data = dat)
@

\begin{figure}
<<ps10, fig=T, echo=F, height=9, width=6>>=
par(mfcol=c(2,1))
m4psa <- plotSlopes(m4, plotx = "x1", modx = "x2", xlab = "x1 is a fun plotx")
m4psb <- plotSlopes(m4, plotx = "x2", modx = "x1", modxVals = "std.dev.", xlab = "x2 is plotx", ylim = c(-120, 15),
legendArgs = list(x = "topright"))
par(mfcol=c(1,1))
@

\caption{plotSlopes Illustrated\label{fig:ps10}}
\end{figure}
This is more interesting if we fit a regression with an interaction
term, such as 

\inputencoding{latin9}\begin{lstlisting}
m4 <- lm(y4 ~ x1*x2 + x3, data = dat)
\end{lstlisting}
\inputencoding{utf8}
\noindent We then ask \code{\noindent plotSlopes} to draw the predicted
values using one numeric variable as the horizontal axis and values
of another variable (a moderator) are set at particular values. Either
x1 or x2 can be viewed as the ``moderator'' predictor, the one on
which the effect of the other depends. In rockchalk version 1.8, the
selection of values of the moderator was generalized, so that the
user can specify either a function that selects values, or a vector
of values, or the name of an algorithm. The default algorithm will
choose quantile values, but Figure \ref{fig:ps10} demonstrates also
the ``std.dev.'' divider algorithm. The code to produce that figure
was

<<eval=F, echo=T>>=
<<ps10>>
@

When \code{modx} is a numeric variable, then some particular values
must be selected for calculation of predicted value lines. The \code{modxVals}
argument is used to either specify moderator values or an algorithm
to select focal values. By default, three hypothetical values of \code{plotx}
are selected (the quantiles 25\%, 50\%, and 75\%). 

If \code{modx} is a factor variable, then the most frequently observed
scores will be selected for consideration. The default display will
include the regression line as well as color-coded points for the
subgroups represented by values of the moderator.

\noindent 
\begin{figure}
<<echo=F>>=
fourCat <- gl(4,25, labels=c("East","West","South", "Midwest"))
dat$x4 <- sample(fourCat, 100, replace = TRUE)
dat$y5 <- 1 + 0.1 * dat$x1 + contrasts(dat$x4)[dat$x4, ] %*% c(-1,1,2) + rnorm(100,0, sd=10)
m5 <- lm (y5 ~ x1*x4 + x3, data=dat)
@

<<ps20, fig=T, echo=F>>=
m5psa <- plotSlopes(m5, plotx = "x1", modx = "x4", xlab = "x1 is a Continuous Predictor", xlim = magRange(dat$x1, c(1.2,1)))
@

\caption{plotSlopes with a Categorical Moderator\label{fig:ps20}}
\end{figure}
\noindent 
\begin{figure}
<<ps21, fig=T, echo=F>>=
m5psb <- plotSlopes(m5, plotx = "x1", modx = "x4", modxVals = c("West","East"), xlab = "x1 is a Continuous Predictor", xlim=magRange(dat$x1, c(1.2,1)), interval = "conf")
@

\caption{plotSlopes: the interval argument\label{fig:ps21}}
\end{figure}
Suppose we have a four-valued categorical variable, ``West'',''Midwest'',
``South'', and ``East''. If that variable is used in an interaction
in the regression model, then the \code{\noindent plotSlopes} output
will include four lines, one for each region. For example, consider
Figure \ref{fig:ps20}, which is created by

<<eval=F>>=
<<ps20>>
@

\noindent The categorical variable is x4. 

It is possible to superimpose confidence intervals for many subgroups,
but sometimes these plots start to look a little bit ``busy''. The
mixing of shades in overlapping intervals may help with that problem.
A plot that focuses on just two subgroups is presented in Figure \ref{fig:ps21},
which is produced by 

<<eval=F>>=
<<ps21>>
@ 

In rockchalk version 1.8, I've exerted quite a bit of effort to make
sure that colors are chosen consistently when users remove or insert
groups in these plots. The same value of the moderator should always
be plotted in the same way–the line, points, and interval colors should
not change. Note, for example, in Figures \ref{fig:ps20} and \ref{fig:ps21},
the line for East is black in both plots, while the line for West
is red in both. 

\subsection{testSlopes, a companion of plotSlopes}

The students in psychology and political science are usually interested
in conducting some diagnostic analysis of the interactive terms. \citet{Aiken1991}
(and later \citealp{Cohen2002}Cohen, Cohen, West, and Aiken) propose
using the t test to find out if the effect of the ``plotx'' variable
is statistically significantly different from zero for each particular
value of the moderator variable. The new version of rockchalk declares
a method plot.testSlopes that handles the work of plotting the interaction.

The usual case would be the following. First, carry out the regression
analysis. Then run plotSlopes, and run testSlopes, and then pass that
result object to the plot method.

\begin{figure}
<<ts10, fig=T, echo=F, height=6, width=6>>=
m4psats <- testSlopes(m4psa)
plot(m4psats)
@

\caption{testSlopes for an Interactive Model\label{fig:ts10}}
\end{figure}
\inputencoding{latin9}\begin{lstlisting}
m4 <- lm(y ~ x1*x2 + x3, data = dat)
m4ps <- plotSlopes(m4, plotx = "x1", modx ="x2", xlab = "x1 is a Continuous Predictor")
m4psats <- testSlopes(m4ps)
plot(m4psats)
\end{lstlisting}
\inputencoding{utf8}
\noindent The output from testSlopes will differ, depending on whether
modx is numeric or a factor. If it is a factor, then the slope of
the lines and the associated t-test for each will be presented. My
psychology students call these ``simple-slopes'', following the
terminology of Aiken and West. The general idea is that we want to
know if the ``combined'' effect of plotx is not zero. For a model
stated with predictors $plotx_{i}$ and $modx_{i}$as
\begin{equation}
y_{i}=b_{0}+b_{plotx}plotx_{i}+b_{modx}modx_{i}+b_{plotx:modx}plotx_{i}\cdot modx_{i}+\ldots+e_{i}
\end{equation}

\noindent the null hypothesis would be

\noindent 
\begin{equation}
H_{0}:0=\hat{b}_{simple\,slope}=\hat{b}_{plotx}+\hat{b}_{plotx:modx}modx
\end{equation}

\noindent If modx is a factor, then we simply calculate the slope
of each line and the test is straight-forward. If modx is a numeric
variable, then we confront a problem that is a bit more interesting.
We don't really want to say that the simple slope is different from
0 for particular values of $modx$, but instead we want to answer
the question, ``for which values of the moderator would the effect
of the plotx variable be statistically significant?''. This necessitates
the calculation of the so-called Johnson-Neyman interval (\citeyear{JohnsonNeyman1936}),
a plot of which is presented in Figure \ref{fig:ts10}.

The method of calculation is outlined in \citet{PreacherCurren2006}.
The values of $modx$ associated with a statistically significant
effect of $plotx$ on the outcome is determined from the computation
of a T statistic for $\hat{b}_{simple\,slope}$. The J-N interval
is the set of values of $modx$ for which the following (quadratic
equation) holds:
\begin{align}
\hat{t} & =\frac{\hat{b}_{simple\,slope}}{std.err(\hat{b}_{simple\,slope})}\nonumber \\
 & =\frac{\hat{b}_{simple\,slope}}{\sqrt{\widehat{Var(\hat{b}_{plotx})}+modx^{2}\widehat{Var(\hat{b}_{plotx\cdot modx})}+2modx\widehat{Cov(\hat{b}_{plotx},\hat{b}_{plotx\cdot modx})}}}\geq T_{\frac{\alpha}{2},df}
\end{align}

\noindent Suppose there are two real roots, $root1$ and $root2$.
The values of $modx$ for which the slope is statistically significant
may be a compact interval, $[root1,root2]$, as demonstrated in Figure
\ref{fig:ts10}, or it may two open intervals, $(-\infty,root1]$
and $[root2,\infty)$. I had expected almost all applications to result
in that latter case, but the somewhat surprising case illustrated
in Figure \ref{fig:ts10} is not too infrequently observed. 

\subsection{plotCurves for nonlinear predictor formulae}

In the most recent revision of this package, I believe I have made
the plotCurves() function redundant. Before I delete the function
entirely, I'm waiting for feedback. There was a problem in the past.
Some students used plotSlopes() (which drew straight lines) when they
intended to draw curves (and should have used plotcCurves).

plotCurves() was developed to generalize the plotting capability for
regression formulas that include nonlinear transformations. It was
for models that have polynomials or terms that are logged (or otherwise
transformed). In that sense, plotCurves() is rather similar to R's
own termplot() function, but plotCurves() has two major advantages.
First, it allows interactions, and second, it handles some complicated
formulae that termplot() is not able to manage. 

Suppose a dependent variable y5 is created according to a nonlinear
process.
\begin{equation}
y5_{i}=-3x1_{i}+7*log(x2)+1.1x2_{i}+2.2x1_{i}\times x2_{i}+e_{i}
\end{equation}

<<echo=F,include=F>>=
dat$y5 <- with(dat, -3*x1 + 15*log(0.1 + x2 - min(x2)) + 1.1*x2 + 8.2 *x1 * x2 + 10*rnorm(100))
@

The estimated model is

<<>>=
m5 <- lm(y5 ~ log(x2) + x1 * x2, data = dat)
@

In the new version, the function calls for plotSlopes() and plotCurves()
are the same:

<<pcps20a, fig=T, eval=F, height=4, width=6>>=
m5pc <- plotCurves(m5, plotx = "x2", modx = "x1", main = "plotCurves output", ylim = c(-210, 330),
legendArgs = list(x = "topleft", title = "x1", cex = 0.8))
@

<<psps20b, fig=T, eval=T, eval=F, echo = T, height = 4, width = 6>>=
m5ps <- plotSlopes(m5, plotx = "x2", modx = "x1", main = "plotSlopes outout", ylim = c(-210, 330),
legendArgs = list(x = "topleft", title = "x1", cex = 0.8))
@

So far as I have been able to tell, the results are the same. See
Figure \ref{fig:pcps20}.

\begin{figure}

<<eval=T, fig=T,echo=F,height = 4, width = 6>>=
<<pcps20a>>
@

<<eval=T, fig=T, echo=F,height = 4, width = 6>>=
<<psps20b>>
@

\caption{Illustrations of m5 with plotSlopes and plotCurves\label{fig:pcps20}}

\end{figure}

\subsection{plotPlane}

The persp() function in R works well, but its interface is too complicated
for most elementary and intermediate R users. To facilitate its use
for regression users, the plotPlane() function is offered.

The plotPlane function offers a visualization of the mutual effect
of two predictors, whether or not the regression model is linear.
plotPlane() is designed to work like plotCurves(), to tolerate nonlinear
components in the regression formula. plotPlane() allows the depiction
of a 3 dimensional curving plane that ``sits'' in the cloud of data
points. The variables that are not explicitly pictured in the plotPlane()
figure are set to central reference values. Recall model m4, which
used the formula \inputencoding{latin9}\lstinline!y4 ~ x1*x2 + x3!\inputencoding{utf8}.
As illustrated in Figure \ref{fig:pp100}, plotCurves() presents a
reasonable view of the predicted values.

\begin{figure}
<<pp100, fig=T, echo=F>>=
p100 <- plotPlane(m4, plotx1 = "x1", plotx2 = "x2", phi = 10, theta = -80, lcol = gray(.70))
@

\caption{plotPlane for the Interactive Model m4\label{fig:pp100}}
\end{figure}
Because plotPlane() is a simple convenience wrapper for R's persp()
function, it responds to the same customizing arguments that perp
would allow. The arguments phi and theta will rotate the figure, for
example. The output in Figure \ref{fig:pp100} is produced by the
following.

<<echo=T,eval=F>>=
<<pp100>>
@

\noindent One of the major educational benefits of the 3-D figure
is that students can easily see that a model with a simple interaction
effect is \emph{not a linear model any more. }We will return to that
point in the discussion of mean centering in regression analysis.

Recall that model m5 is the rather complicated nonlinear formula \inputencoding{latin9}\lstinline!log(x2*x2) + x1 * x2!\inputencoding{utf8}.
The plotCurves() output for that was already presented in Figure \ref{fig:pcps20}.
The three dimensional view of the same is presented in Figure \ref{fig:pp110},
but with an added twist. The twist is that the predicted value lines
from the 2-D plot functions can be superimposed on the plane. The
function addLines() does the work for translating 2-D plot object
onto the regression plane. 

\begin{figure}
<<pp111, fig = T, echo = F, height = 5, results = hide>>=
ppm5 <- plotPlane(m5, plotx1 = "x2", plotx2 = "x1", phi = 0, npp = 15, lcol = gray(.80))
addLines(from = m5pc, to = ppm5, col = m5pc$col)
@

\caption{Making plotSlopes and plotPlane Work Together\label{fig:pp110}}
\end{figure}

\section{Standardized, Mean-Centered, and Residual-Centered Regressions }

\subsection{Standardized regression}

Many of us learned to conduct regression analysis with SPSS, which
reported both the ordinary (unstandardized) regression coefficients
as well as a column of ``beta weights'', the output of a ``standardized''
regression analysis. Each variable, for example $x1_{i}$, was replaced
by an estimated $Z-score:$ $(x1_{i}-\overline{x1})/std.dev.(x1_{i}$).
Some people think these coefficients are easier to interpret (but,
for a strong cautionary argument against them, see \citealp{King1986}).
R offers no such thing as standardized regression, probably because
this practice is thought to be mistaken. The automatic standardization
of all predictors, no matter whether they are categorical, interaction
terms, or transformed values (such as logs) is dangerous. 

To illustrate that, the rockchalk introduces a function called standardize().
Each column of the design matrix is scaled to a new variable with
mean 0 and standard deviation 1. The result from standardize() will
be an R lm object, which will respond to any follow-up analysis commands.
For example:

<<>>=
m4 <- lm (y4 ~ x1 * x2, data = dat)
m4s <- standardize(m4)
@

I doubt that a reasonable person would actually want a standardized
regression and have tried to warn users in the output.

<<>>=
summary(m4s)
@

\begin{table}
\caption{Comparing Ordinary and Standardized Regression\label{tab:stdreg10}}

<<stdreg10, results=tex, echo=F>>=
or10 <- outreg(list(m4, m4s), tight = F, modelLabels = c("Not Standardized","Standardized"))
@
\end{table}

\subsection{Mean-centered Interaction Models}

Sometimes people will fit a model like this
\begin{equation}
y_{i}=b_{o}+b_{1}x1_{i}+b_{2}x2_{i}+e_{i}
\end{equation}

\noindent and then wonder, ``is there an interaction between $x1_{i}$
and $x2_{i}$?'' The natural inclination is to run this model, 

\inputencoding{latin9}\begin{lstlisting}
m1 <- lm(y ~ x1*x2)
\end{lstlisting}
\inputencoding{utf8}
\noindent or its equivalent

\inputencoding{latin9}\begin{lstlisting}
m2 <- lm(y ~ x1 + x2 + x1:x2)
\end{lstlisting}
\inputencoding{utf8}
Researchers have been advised that they should not run the ordinary
interaction model without ``mean-centering'' the predictors (Aiken
and West, 1991). They are advised to replace $x1_{i}$ with $(x1_{i}-\overline{x1})$
and $x2_{i}$ with $(x2_{i}-\overline{x2})$, so that the fitted model
will
\begin{equation}
y_{i}=b_{o}+b_{1}(x1_{i}-\overline{x1})+b_{2}(x2_{i}-\overline{x2})+b_{3}(x1_{i}-\overline{x1})(x2_{i}-\overline{x2})+e_{i}
\end{equation}

This is a little tedious to do in R, so I provide a function meanCenter()
that can handle the details. meanCenter() will receive a model, scan
it for interaction terms, and then center the variables that are involved
in interactions. We previously fit the model m4, and now we center
it.

<<>>=
m4mc <- meanCenter(m4)
summary(m4mc)
@

By default, meanCenter() will only center the variables involved in
an interaction, and it leaves the others unchanged. The user can request
a different treatment of the variables. Version 1.8 introduces the
argument ``terms'', which allows the user to list the names of the
predictors that should be centered. If the user wants all of the numeric
predictors to be mean-centered, the usage of the argument centerOnlyInteractors
would be appropriate:

\inputencoding{latin9}\begin{lstlisting}
m4mc <- meanCenter(m4, centerOnlyInteractors = FALSE)
\end{lstlisting}
\inputencoding{utf8}
By default, it does not standardize while centering (but the user
can request standardization with the argument standardize = TRUE.
The option centerDV causes the dependent variable to be centered as
well.

\subsection{Residual-centered Models}

Residual-centering \citep{LittleBovaird2006} is another adjustment
that has been recommended for models that include interactions or
squared terms. Like mean-centering, it is often recommended as a way
to obtain smaller standard errors or to make estimates more numerically
stable. Like mean centering, it causes a superficial change in the
estimated coefficients, but the predicted values and the regression
relationship is not actually changed. Nothing of substance is altered.

The residualCenter() function is used in the same manner as meanCenter().
The user fits an interactive model and the result object is passed
to residualCenter() like so:

<<>>=
m4rc <- residualCenter(m4)
summary(m4rc)
@

I would explain residual-centering as follows. Suppose we fit the
linear model, with no interaction (note, I'm calling the coefficients
$c_{j}$, not $b_{j}$ as usual):

\begin{equation}
y=c_{0}+c_{1}x1+c_{2}x2+e{}_{i}.\label{eq:rc20}
\end{equation}

\noindent Let's proceed as if those parameter estimates, $\hat{c}_{1}$,
$\hat{c}_{2}$, are the ``right ones'' for our analytical purpose.
We'd like to fit an interactive model, but protect the linear parts
from fluctuation. In R, when we run the model \inputencoding{latin9}\lstinline!lm(y ~ x1 * x2)!\inputencoding{utf8},
we are allowing all of the coefficients of all variables to fluctuate
\begin{equation}
y_{i}=b_{o}+b_{1}x1_{i}+b_{2}x2_{i}+b_{3}x1_{i}\times x2_{i}+e_{i}\label{eq:rcwant}
\end{equation}
Residual centering is one way to stabilize the estimation by assuring
that $\hat{b}_{1}=\hat{c}_{1}$ and $\hat{b}_{2}=\hat{c}_{2}$. Only
the coefficient $\hat{b}_{3}$ floats freely.

One of the reasons that residual-centering is so appealing is that
its stabilizing benefit is obtained almost by accident. Here is the
gist of the calculations. First, estimate a regression in which the
left hand side is the interaction product term:
\begin{equation}
(x1_{i}\times x2_{i})=d_{0}+d_{1}x1_{i}+d_{2}x2+u_{i}\label{eq:residCentered}
\end{equation}

\noindent The residuals from that regression are, by definition, orthogonal
to both $x1$ and $x2$. Call those fitted residuals $\widehat{u_{i}}$.
The we run the interactive regression, replacing the column of the
predictor $x1_{i}\times x2_{i}$, with $\widehat{u}_{i}$. That is
to say, the model we want, equation (\ref{eq:rcwant}), is estimated
as: 
\begin{equation}
y_{i}=b_{0}+b_{1}x1_{i}+b_{2}x2_{i}+b3\widehat{u_{i}}+e_{i},\label{eq:rc10-1}
\end{equation}

\noindent In essence, we have taken the interaction $(x1_{i}\times x2_{i})$,
and purged it of its parts that are linearly related to $x1_{i}$
and $x2_{i}$. 

rockchalk 1.6 included summary, print, and predict methods for the
residual-centered regression objects. It is worth mentioning that
the code can handle interactions of arbitrarily many predictors. If
the formula has \inputencoding{latin9}\lstinline!lm(y ~ x1 * x2 * x3 * x4)!\inputencoding{utf8},
for example, this implies many separate interactions must be calculated.
We need to calculate residual-centered residuals for $x1\cdot x2$,
$x1\cdot x3$, $x1\cdot x4$, $x2\cdot x3$ and so forth, and then
use them as predictors to get centered estimates of terms $x1\cdot x2\cdot x3$,
and then their centered values are predictors four term interactions.
Aspiring R programmers who want to learn about programming with R
formula objects might benefit from the study of the function residualCenter.R
in the rockchalk source code. 

\section{A Brief Analysis of Mean-Centering}

We can put the tools together by making a little detour into the question
that seems to plague every regression analyst at one time or another:
What does that interaction term \emph{really mean}? Along the way,
we will try to dispel the idea that centering somehow makes estimates
``better'' or more numerically precise. The primary advocates of
centering as a way to deal with numerical instability are \citet{Aiken1991},
who integrated that advice into the very widely used regression textbook,
\emph{Applied Multiple Regression/Correlation for the Behavioral Sciences}
\citep{Cohen2002}. They claim that the inclusion of interactions
causes ``inessential multicollinearity'' that is alleviated by centering.
The advice is widely followed. One statistics book intended for biologists
observed, for example, ``We support the recommendation of Aiken \&
West (1991) and others that multiple regression with interaction terms
should be fitted to data with centered predictor values'' (\citealp{QuinnKeough2002},
Chapter 6). 

Technical rebuttals have been published already (\citealp{Kromrey1998}),
but the matter still seems not widely understood. The argument is
not that mean-centering (or residual-centering) is wrong, but rather
that it is unnecessary. It is irrelevant, and possibly, misleading.

At the core of the matter is the fact that our uncertainty about regression
estimates depends on our point of view. Please review the confidence
interval in Figure \ref{fig:ps05}. The $y$ axis is not even ``in
the picture.'' Would one's appreciation of the regression's predictive
line be enhanced if the y axis were moved into the picture? 

We can move the $y$ axis by centering the predictor variable. Suppose
we replace $x_{i}$ with $x_{i}-5$ and then re-estimate the model.
That has the simple effect of moving the $y$ axis 5 units to the
right. The slope is unchanged, and the reported intercept is changed
in a completely superficial way. The predicted value line, the slope
estimates, the residual standard error, and so forth, are not changed
in any substantial way. This is simply a matter of user convenience.
I believe that no reasonable person can say the regression is ``better''
after centering $x_{i}$.

However, there is a superficial difference that has deceived many
authors. Notice that the confidence interval is hourglass shaped.
\emph{At the new $y$ axis}, our prediction is more precise. If we
move the y axis further to the right, into the center of the data,
say by mean-centering ($x_{i}-10$), we move to the position that
allows an even more precise prediction. The estimate of the intercept's
standard error will be smaller for the obvious reason. We are not
actually gaining certainty, we are simply reporting our most favorable
``snapshot'' of it. The predicted value, and the confidence interval
for any observed value of $x_{i}$ is completely unchanged by centering.

It should not surprise the reader to learn that mean-centering interactive
predictors enhances the standard errors in the same illusory way.
Let's work though an example to see why this is so tempting. Suppose
the true data generating mechanism is an interaction like so
\begin{equation}
y_{i}=2+0.1x1_{i}+0.1x2_{i}+0.2\cdot(x1_{i}\times x2_{i})+e_{i},
\end{equation}

\noindent where $e_{i}\sim N(0,300^{2})$ and $\rho_{x1,x2}=0.4$. 

A regression analysis that ignores the interaction, 

\inputencoding{latin9}\begin{lstlisting}
lm(y ~ x1 + x2, data = dat2)
\end{lstlisting}
\inputencoding{utf8}
\noindent is reported in the first column of Table \ref{tab:meancenter10-1}.
I've used outreg's new alpha argument to emphasize the ``really good''
estimates with more stars. Notice that everything is ``statistically
significant!''

Unable to leave well enough alone, the researcher wonders, ``is there
an interaction between $x1$ and $x2$?'' Run \inputencoding{latin9}
\begin{lstlisting}
lm(y ~ x1 * x2, data = dat2)
\end{lstlisting}
\inputencoding{utf8}
\noindent The second column in Table \ref{tab:meancenter10-1} summarizes
that regression. Be prepared for a shock when you scan the estimates.
Almost everybody I know has said ``what the heck?'' or ``Holy Cow!''
or ``Oh My God, my great result went to Hell, I'll never get tenure!''
Neither of the key variables is ``statistically significant'' any
more. 

<<echo=F, include=F>>=
dat2 <- genCorrelatedData3(N=400, rho=.4, stde=300, beta=c(2,0.1,0.1,0.2))
m6linear <- lm (y ~ x1 + x2, data=dat2)
m6int <- lm (y ~ x1 * x2, data=dat2)
m6mc <- meanCenter(m6int)
m6rc <- residualCenter(m6int)
@

\begin{table}
\caption{Comparing Regressions\label{tab:meancenter10-1}}

<<mcenter10, results=tex, echo=F>>=
or11 <- outreg(list(m6linear, m6int, m6mc, m6rc), tight=F, modelLabels=c("Linear", "Interaction","Mean-centered","Residual-centered"), alpha = c(0.05, 0.01))
@
\end{table}
Cohen, et al. claim that the apparent instability of the coefficients
is a reflection of ``inessential collinearity,'' due to the fact
that $x1$ and $x2$ are correlated with the new term, $x1\times x2$.
They advised their readers to ``mean-center'' their predictors and
run the regression again. 

Remember the hourglass shape of the confidence interval. By mean-centering,
we are re-positioning ourself for a much more favorable snapshot.
The welcoming effect of the centered estimates is found in the third
column of Table \ref{tab:meancenter10-1}. The point estimates in
that snapshot are ``significant again.'' It appears we have ``solved''
the problem of inessential collinearity. 

The first hint of trouble is in the fact that the coefficients of
the interactive effects in columns 2 and 3 are identical. Those coefficients
are the same because they are estimates of the cross partial derivative
$\partial^{2}y/\partial x1\partial x2$. That particular value is
the same, no matter where we position the $y$ axis, as it should
be. Note as well that the root mean square and $R^{2}$ estimates
are identical. Everything that we expect to remain the same is the
same. Except for the fact that the slopes and their hypothesis tests
seem better in the centered model, we would think there is nothing
interesting here.

Here's a puzzle for you. Consider Figure \ref{fig:pscentering}, which
shows the predicted values and confidence intervals from the centered
and uncentered regressions. Is there any substantively important difference
between these two regressions?

\begin{figure}
<<pscenter, fig=T, echo=F, include = F, height = 10, width = 5, results = hide>>=
par(mfcol = c(2, 1))
plotSlopes(m6int, plotx = "x1", modx = "x2", modxVals = "std.dev.",  n = 2, interval = "confidence", main= "Not Centered")
plotSlopes(m6mc, plotx = "x1c", modx = "x2c", modxVals = "std.dev.", n = 2, interval = "confidence", main = "Mean Centered")
par(mfcol = c(1, 1))
@

\includegraphics[height=8in]{rockchalk-pscenter.pdf}

\caption{plotSlopes for the centered and non-centered regressions\label{fig:pscentering}}
\end{figure}
Perhaps the 2-D plots are not persuasive. Don't stop yet. In the 3-D
plots will help quite a bit. We have not yet grasped the most fundamental
changed caused by the insertion of the interaction term. When we insert
$x1_{i}\times x2_{i}$, we change the fudamental nature of the regression
surface. The surface of the fitted model is no longer a ``flat''
plane, but rather it is a curving surface. 

I've assembled 3-D plots in Figure \ref{fig:mcenter30}. We see that
the ordinary interaction, mean-centered, and residual-centered models
produce identical predicted values! The only difference in the figures
is that the axes of the predictors have been re-scaled, so that the
$y$ axis is (implicitly) re-positioned. Now that we understand the
situation, we could play around with the data and move the axis back
and forth until we arrive at a position that is most favorable to
our interpretation. 

\begin{figure}
<<mcenter50, fig=T,echo=FALSE, height=5, width=7>>=
op <- par(no.readonly = TRUE)
par(mfcol=c(2,2))
par(mar=c(2,2,2,1))
plotPlane(m6linear, plotx1="x1", plotx2="x2", plotPoints=FALSE, main="Linear", ticktype="detailed")
plotPlane(m6int, plotx1="x1", plotx2="x2", plotPoints=FALSE, main="Interaction: Not Centered", ticktype="detailed")
plotPlane(m6mc, plotx1="x1c", plotx2="x2c", plotPoints=FALSE, main="Mean-centered", ticktype="detailed")
plotPlane(m6rc, plotx1="x1", plotx2="x2", plotPoints=FALSE, main="Residual-centered", ticktype="detailed")
par(op)
@

\caption{Predicted Planes from Centered and Uncentered Fits Identical\label{fig:mcenter30}}
\end{figure}
The regression coefficient estimates are snapshots, each summarizing
the curvature at one particular point in a curving surface. It seems
quite apparent in Figure \ref{fig:mcenter30} that the models are
identical, and yet we receive different regression reports from different
spots. The non-centered data offers us the slope estimate from the
``front-left'' part of the graph. Mean-centered estimates report
on the slope in the middle of the graph. In the rockchalk examples
folder, one can find a file called ``centeredRegression.R'' that
walks through this argument step by step. 

What about residual-centering? Because the transformation that it
employs is more abstract, I initially thought it was actually a different
model. And yet it is not. The residual-centered model is completely
equivalent to the ordinary interaction model and the mean-centered
model. For a given combination of the input values, the predicted
values and confidence intervals are the same. The predicted values
of the ordinary interactive model, the mean-centered model, and the
residual-centered models are illustrated in Figure \ref{fig:rcenter40}. 

\begin{figure}
<<rcenter40, fig=T,echo=FALSE, include=F, height=4, width=8>>=
dat3 <- centerNumerics(dat2)
##m6mcpred <- fitted(m6mc) ##
m6mcpred <- predict(m6mc, newdata=dat3)
##m6rcpred <- fitted(m6rc) ##
m6rcpred <- predict(m6rc, newdata=dat3)
##m6intpred <- fitted(m6int) ##
m6intpred <- predict(m6int, newdata=dat3)
op <- par(no.readonly = TRUE)
par(mfcol=c(1,2))
plot(m6intpred, m6rcpred, main="", xlab="Predictions of Uncentered Interaction", ylab="Residual-centered Predictions")
predcor <- round(cor(m6intpred, m6rcpred),3)
legend("topleft", legend=c(paste0("Correlation=", predcor)))
plot(m6mcpred, m6rcpred, main="", xlab="Mean-centered Predictions", ylab = "Residual-centered Predictions")
predcor <- round(cor(m6mcpred, m6rcpred),3)
legend("topleft", legend=c(paste0("Correlation=", predcor)))
par(op)
@

\includegraphics[width = 6.5in]{rockchalk-rcenter40}

\caption{Predicted Values of Mean and Residual-centered Models\label{fig:rcenter40}}
\end{figure}
The take-away point from this is that there is no free lunch in regression
analysis. If re-scaling a variable by adding or subtracting a constant
seems to change a result, one should be cautious and suspect an error. 

In order to drive the point home, I'd like to show that it is possible
to translate between the estimates of any one of these fitted models
and the estimates of the others. The ordinary model is
\begin{equation}
y_{i}=b_{0}+b_{1}x1_{i}+b_{2}x2_{i}+b_{3}(x1_{i}\times x2_{i})+e1_{i}\label{eq:int}
\end{equation}

The mean-centered model is 
\begin{equation}
y_{i}=c_{0}+c_{1}(x1_{i}-\overline{x1})+c_{2}(x2_{i}-\overline{x2})+c_{3}(x1_{i}-\overline{x1})\cdot(x2_{i}-\overline{x2})+e2_{i}\label{eq:mc1}
\end{equation}

In order to compare with equation \ref{eq:int}, we would re-arrange
like so

\begin{equation}
y_{i}=c_{0}+c_{1}(x1_{i})-c_{1}\overline{x1}+c_{2}(x2_{i})-c_{2}\overline{x2}+c_{3}(x1_{i}x2_{i}+\overline{x1}\overline{x2}-\overline{x1}x2_{i}-\overline{x2}x1_{i})+e2_{i}
\end{equation}

\begin{equation}
y_{i}=c_{0}+c_{1}(x1_{i})-c_{1}\overline{x1}+c_{2}(x2_{i})-c_{2}\overline{x2}+c_{3}(x1_{i}x2_{i})+c_{3}\overline{x1}\overline{x2}-c_{3}\overline{x1}x2_{i}-c_{3}\overline{x2}x1_{i})+e2_{i}
\end{equation}

\begin{equation}
y_{i}=\{c_{0}-c_{1}\overline{x1}-c_{2}\overline{x2}+c_{3}\overline{x1}\overline{x2}\}+\{c_{1}-c_{3}\overline{x2}\}x1_{i}+\{c_{2}-c_{3}\overline{x1}\}x2_{i}+c_{3}(x1_{i}x2_{i})+e2_{i}\label{eq:mc3}
\end{equation}

One can then compare the parameter estimates from equations \ref{eq:int}
and \ref{eq:mc3}. Both \ref{eq:int} and \ref{eq:mc3} include a
single parameter times $(x1_{i}x2_{i}),$ leading one to expect that
the estimate $\hat{b}_{3}$ should be equal to the estimate of $\hat{c}_{3}$,
and they are! Less obviously, one can use the fitted coefficients
from either model to deduce the fitted coefficients from the other.
The following equalities describe that relationship.
\begin{eqnarray}
\hat{b}_{0} & = & \hat{c}_{0}-\hat{c}_{1}\overline{x1}-\hat{c}_{2}\overline{x2}+\hat{c}_{3}\overline{x1}\overline{x2}\\
\hat{b}_{1} & = & \hat{c}_{1}-\hat{c}_{3}\overline{x2}\\
\hat{b}_{2} & = & \hat{c}_{2}-\hat{c}_{3}\overline{x1}\\
\hat{b}_{3} & = & \hat{c}_{3}
\end{eqnarray}
The estimated fit of equation \ref{eq:mc1} would provide estimated
coefficients $\hat{c}_{j}$, $j=0,...,3$, which would then be used
to calculate the estimates from the non-centered model.

The estimation of the residual-centered model requires two steps.
The residual from this regression model

\noindent 
\begin{equation}
\widehat{(x1_{i}\times x2_{i})}=\hat{d}_{0}+\hat{d}_{1}x1_{i}+\hat{d}_{2}x2_{i}.
\end{equation}

\noindent is 
\begin{equation}
\widehat{u}_{i}=(x1_{i}\times x2_{i})-\widehat{(x1_{i}\times x2_{i})},
\end{equation}

\noindent which is inserted into equation \ref{eq:int}. 

\begin{equation}
y_{i}=h_{0}+h_{1}x1_{i}+h_{2}x2_{i}+h_{3}\{x1_{i}\times x2_{i}-\widehat{x1_{i}\times x2_{i}}\}+e3_{i}\label{eq:rc1}
\end{equation}

Replacing $\widehat{x1_{i}\times x2_{i}}$ with $\hat{d}_{0}+\hat{d}_{1}x1_{i}+\hat{d}_{2}x2_{i}$,
\ref{eq:rc1} becomes

\begin{eqnarray}
y_{i} & = & h_{0}+h_{1}x1_{i}+h_{2}x2_{i}+h_{3}\{x1_{i}\times x2_{i}-\hat{d}_{0}-\hat{d}_{1}x1_{i}-\hat{d}_{2}x2_{i}\}+e3_{i}\\
 & = & h_{0}+h_{1}x1_{i}+h_{2}x2_{i}+h_{3}\{x1_{i}\times x2_{i}\}-h_{3}\hat{d}_{0}-h_{3}\hat{d}_{1}x1_{i}-h_{3}\hat{d}_{2}x2_{i}\}+e3_{i}\\
 &  & \{h_{0}-h_{3}\hat{d}_{0}\}+\{h_{1}-h_{3}\hat{d}_{1}\}x1_{i}+\{h_{2}-h_{3}\hat{d}_{2}\}x2_{i}+h_{3}\{x1_{i}\times x2_{i}\}+e3_{i}
\end{eqnarray}

As in the previous comparison, we can translate coefficient estimates
between the ordinary specification and the residual-centered model.
The coefficient estimated for the product term, $\hat{h}_{3}$, should
be equal to $\hat{b}_{3}$ and $\hat{c}_{3}$ (and it is!). If we
fit the residual centered model, \ref{eq:rc1}, we can re-generate
the coefficients of the other models like so: 
\begin{eqnarray}
\hat{b}_{0} & =\hat{c}_{0}-\hat{c}_{1}\overline{x1}-\hat{c}_{2}\overline{x2}+\hat{c}_{3}\overline{x1}\overline{x2}= & h_{0}-h_{3}\hat{d}_{0}\\
\hat{b}_{1} & =\hat{c}_{1}-\hat{c}_{3}\overline{x2}= & h_{1}-h_{3}\hat{d}_{1}\\
\hat{b}_{2} & =\hat{c}_{2}-\hat{c}_{3}\overline{x1}= & h_{2}-h_{3}\hat{d}_{2}
\end{eqnarray}

From the preceding, it should be clear enough that the three models
are equivalent.

\section{Conclusion}

The rockchalk package is offered as a system of support for teachers
and students of regression analysis. It should help with the preparation
of plots, summary tables, and other diagnostics. 

A number of functions are currently offered in this package that have
not been emphasized in this writeup. I would draw the reader to the
help pages for these functions
\begin{description}
\item [{combineLevels}] a recoder for factor variables
\item [{mcDiagnose}] a one stop shop for multicollinearity diagnostic information
\item [{getDeltaRsquare}] calculate the change in the $R^{2}$ that results
from the omission of each variable. This is the squared semi-partial
correlation coefficient.
\item [{getPartialCor}] calculates the partial correlation matrix of the
predictors in a regression model
\item [{lazyCor~and~lazyCov}] convenient ways to create correlation and
covariance matrices that are needed in many simulation exercises
\item [{mvrnorm}] a slightly improved version of the MASS package's multivariate
normal generator.
\end{description}
\bibliographystyle{chicago}
\bibliography{rockchalk}

\end{document}

#LyX 2.1 created this file. For more info see http://www.lyx.org/
\lyxformat 474
\begin_document
\begin_header
\textclass scrartcl
\begin_preamble
%\VignetteIndexEntry{Rchaeology}

\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage{babel}
\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{figure}

 }{%

   \@float{figure}[#1]%

 }%

 \centering

}{%

 \end@float

}

\renewenvironment{table}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{table}

 }{%

   \@float{table}[#1]%

 }%

 \centering

%  \setlength{\@tempdima}{\abovecaptionskip}%

%  \setlength{\abovecaptionskip}{\belowcaptionskip}%

% \setlength{\belowcaptionskip}{\@tempdima}%

}{%

 \end@float

}


%\usepackage{listings}
% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true,style=Rstyle}

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\scriptsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.90}}



\usepackage{babel}

\usepackage[samesize]{cancel}



\usepackage{ifthen}



\usepackage{listings}% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, style=Rstyle}
\end_preamble
\options noae
\use_default_options false
\begin_modules
sweave
enumitem
logicalmkup
\end_modules
\maintain_unincluded_children false
\language american
\language_package default
\inputencoding utf8
\fontencoding T1
\font_roman lmodern
\font_sans lmss
\font_typewriter lmtt
\font_math auto
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100
\font_tt_scale 100
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref true
\pdf_bookmarks true
\pdf_bookmarksnumbered false
\pdf_bookmarksopen false
\pdf_bookmarksopenlevel 1
\pdf_breaklinks false
\pdf_pdfborder false
\pdf_colorlinks false
\pdf_backref section
\pdf_pdfusetitle true
\papersize default
\use_geometry true
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 1
\use_package esint 1
\use_package mathdots 0
\use_package mathtools 1
\use_package mhchem 0
\use_package stackrel 1
\use_package stmaryrd 1
\use_package undertilde 1
\cite_engine natbib
\cite_engine_type authoryear
\biblio_style plainnat
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\justification true
\use_refstyle 0
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 1in
\topmargin 1in
\rightmargin 1in
\bottommargin 1in
\secnumdepth 3
\tocdepth 3
\paragraph_separation skip
\defskip smallskip
\quotes_language english
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Title
Rchaeology: Idioms of R Programming
\end_layout

\begin_layout Author
Paul E.
 Johnson <pauljohn @ ku.edu>
\end_layout

\begin_layout Standard
This document was initiated on May 31, 2012.
 The newest copy will always be available at 
\begin_inset Flex URL
status open

\begin_layout Plain Layout

http://pj.freefaculty.org/R
\end_layout

\end_inset

 and as a vignette in the R package 
\begin_inset Quotes eld
\end_inset

rockchalk
\begin_inset Quotes erd
\end_inset

.
\end_layout

\begin_layout Description
Rchaeology: The study of R programming by investigation of R source code.
 It is the effort to discern the programming strategies, idioms, and style
 of R programmers in order to better communicate with them.
\end_layout

\begin_layout Description
Rchaeologist: One who practices Rchaeology.
\end_layout

\begin_layout Standard
These are Rcheological observations about the style and mannerisms of R
 programmers in their native habitats.
 Almost all of the insights here are gathered from the r-help and r-devel
 emails lists, the stackoverflow website pages for R, and the R source code
 itself.
 These are lessons from the 
\begin_inset Quotes eld
\end_inset

school of hard knocks.
\begin_inset Quotes erd
\end_inset


\end_layout

\begin_layout Standard
How is this different from Rtips(
\begin_inset Flex URL
status open

\begin_layout Plain Layout

http://pj.freefaculty.org/R/Rtips.{pdf,html}
\end_layout

\end_inset

)?
\end_layout

\begin_layout Enumerate
This is oriented toward programming R, rather than using R.
\end_layout

\begin_layout Enumerate
It is more synthetic, aimed more at finding 
\begin_inset Quotes eld
\end_inset

what's right
\begin_inset Quotes erd
\end_inset

 rather than 
\begin_inset Quotes eld
\end_inset

what works.
\begin_inset Quotes erd
\end_inset


\end_layout

\begin_layout Enumerate
It is written with Sweave (using Harrell's Sweavel style) so that code examples
 work.
\end_layout

\begin_layout Standard
Where did the 
\begin_inset Quotes eld
\end_inset

R Style
\begin_inset Quotes erd
\end_inset

 section go? It was removed to a separate vignette, RStyle, in the rockchalk
 package.
\end_layout

\begin_layout Standard
\begin_inset CommandInset toc
LatexCommand tableofcontents

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=F>>=
\end_layout

\begin_layout Plain Layout

dir.create("plots", showWarnings=F)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% In document Latex options:
\end_layout

\begin_layout Plain Layout


\backslash
fvset{listparameters={
\backslash
setlength{
\backslash
topsep}{0em}}}
\end_layout

\begin_layout Plain Layout


\backslash
SweaveOpts{prefix.string=plots/plot,ae=F,height=4,width=6}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<Roptions, echo=F>>=
\end_layout

\begin_layout Plain Layout

options(width=100, continue="  ")
\end_layout

\begin_layout Plain Layout

options(useFancyQuotes = FALSE) 
\end_layout

\begin_layout Plain Layout

set.seed(12345)
\end_layout

\begin_layout Plain Layout

pdf.options(onefile=F,family="Times",pointsize=12)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Introduction: R Idioms.
\end_layout

\begin_layout Standard
This vignette is about the R idioms I have learned while working on the
 rockchalk package.
 At the current time, I don't understand all of the R idioms that are common
 in the advanced R programmers' conversation, but I am getting some traction.
\end_layout

\begin_layout Standard
There is a language gap between an R user and an R programmer.
 Users write 
\begin_inset Quotes eld
\end_inset

scripts
\begin_inset Quotes erd
\end_inset

 that use functions from R packages.
 Users don't write (many) functions.
 Users don't make packages.
 And many users are happy to keep it that way.
 For users who want to become programmers, there is usually a harsh awakening.
 R for development is a different language.
 Well, that's wrong.
 It is a different dialect.
 
\end_layout

\begin_layout Standard
A transitional R user will have to learn a lot of terminology and undergo
 a change of paradigm.
 There are resources available! Everybody should subscribe to the email
 lists for the R project (
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://www.r-project.org/mail.html
\end_layout

\end_inset

), especially r-help (for user questions) and r-devel.
 Rchaeologically speaking, that is the native habitat of the R programmers.
 There is also a burgeoning collection of blogs and Web forums, perhaps
 most notably the R section on StackExchange (
\begin_inset Flex URL
status collapsed

\begin_layout Plain Layout

http://stackoverflow.com/questions/tagged/r
\end_layout

\end_inset

).
 Excellent books have been published.
 As an Rcheologist, I am drawn to the books that are written by the R insiders.
 
\emph on
S Programming
\emph default
, by William Venables and Brian Ripley 
\begin_inset CommandInset citation
LatexCommand citeyearpar
key "venables_s_2000"

\end_inset

, is a classic.
 Books by pioneering developers Robert Gentleman, 
\emph on
R Programming for Bioinformatics
\emph default
 (2009), and John Chambers, 
\emph on
Software for Data Analysis
\emph default
 
\begin_inset CommandInset citation
LatexCommand citeyearpar
key "chambers_software_2008"

\end_inset

, are, well, awesome.
 In 2012, I started teaching R programming using Norman Matloff's, 
\emph on
The Art of R Programming
\emph default
 (2012), which I think is great and recommend strongly (even though Professor
 Matloff is not one of the R Core Team members, so far as I know).
 
\end_layout

\begin_layout Standard
There are now three types of object-oriented programming in R (S3, S4, and
 reference classes) and the programmer is apparently free to select among
 them without prejudice.
 The best brief explanation of S3 that I've found is in Friedrich Leisch's
 brief note about R packaging, 
\begin_inset Quotes eld
\end_inset

Creating R Packages: A Tutorial
\begin_inset Quotes erd
\end_inset

 (2009).
 One should be mindful of the fact that R is provided with several manuals,
 one of which is the 
\emph on
R Language Definition
\emph default
.
 I find that one difficult to understand, and I usually can't understand
 it until I search through the R source code and packages for usage examples.
\end_layout

\begin_layout Standard
I usually approach R coding in three phases.
 I write code that 
\begin_inset Quotes eld
\end_inset

works
\begin_inset Quotes erd
\end_inset

, however tedious and slow it might be.
 There will be a lot of 
\begin_inset Quotes eld
\end_inset

copy and paste
\begin_inset Quotes erd
\end_inset

 constructions with tedious, manual editing of commands.
 Then I read through the code and look for repetitious elements, and I re-organi
ze to make functions that abstract that work.
 If there are any stanzas that look mostly the same, except that 
\begin_inset Quotes eld
\end_inset

x1
\begin_inset Quotes erd
\end_inset

 is repalced by 
\begin_inset Quotes eld
\end_inset

x2
\begin_inset Quotes erd
\end_inset

, I know the work is not done.
 Finally, I look through the code to find 
\begin_inset Quotes eld
\end_inset

dumb
\begin_inset Quotes erd
\end_inset

 constructions that can be cleaned up and completed with fewer lines.
 A dumb construction is one that I would be ashamed to show to an expert
 R programmer.
\end_layout

\begin_layout Standard
Beginning R programmers, the ones who have never studied C or Java or Fortran,
 will often become 
\begin_inset Quotes eld
\end_inset

stuck
\begin_inset Quotes erd
\end_inset

 in stage one of that process.
 Experienced programmers with more formal training will usually have the
 
\begin_inset Quotes eld
\end_inset

little voice
\begin_inset Quotes erd
\end_inset

 in the back of their minds saying 
\begin_inset Quotes eld
\end_inset

there's a better way,
\begin_inset Quotes erd
\end_inset

 and, unfortunately, R novices don't hear that voice.
 Lacking some of the discipline imposed by those other languages, the usual
 
\begin_inset Quotes eld
\end_inset

rambling
\begin_inset Quotes erd
\end_inset

 R script will seem 
\begin_inset Quotes eld
\end_inset

good enough.
\begin_inset Quotes erd
\end_inset

 The first step for the transitional R programmer, the one who does not
 want to be a novice any more, is to take a harsh look at the code that
 has already been written.
\end_layout

\begin_layout Standard
The rockchalk package has many functions that receive fitted regression
 models and re-arrange their components for re-fitting or plotting.
 A good deal of the advice I have to offer is about the process of managing
 R formula, but I have also accumulated some lessons in argument handling.
 
\end_layout

\begin_layout Section
do.call(), eval(), substitute(), formula().
\end_layout

\begin_layout Standard
If the transitional programmer understands these four functions, she/he
 will be well on the way to escaping the category of novice R programmer.
 The other benefit is that the code written by the experts, including the
 R source code itself, will become much more understandable.
\end_layout

\begin_layout Subsection
Rewriting Formulas.
 My Introductory Puzzle.
\end_layout

\begin_layout Standard
On May 29, 2012, I was working on a regression problem in the rockchalk
 package.
 I have a number of functions that receive elementary regressions and then
 change them.
 I need to receive a fitted model, extract the formula, change some variables,
 and then revise the formula to match the new variables.
 And then run the model over again.
 
\end_layout

\begin_layout Standard
The functions meanCenter() and residualCenter() receive a fitted regression
 model, transform some variables, and then fit a new regression.
 Suppose a regression has been fitted as 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
y ~ x1*x2
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

.
 The R result will estimate a predictive formula such as 
\begin_inset Formula $\hat{y}_{i}=\hat{\beta}_{0}+\hat{\beta}_{1}x1_{i}+\hat{\beta}_{2}x2_{i}++\hat{\beta}_{3}x1_{i}\cdot x2_{i}$
\end_inset

.
 meanCenter() will replace the non-centered variables 
\begin_inset Quotes eld
\end_inset

x1
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

x2
\begin_inset Quotes erd
\end_inset

 with mean-centered variables 
\begin_inset Quotes eld
\end_inset

x1c
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

x2c
\begin_inset Quotes erd
\end_inset

.
 The original formula y ~ x1*x2 must be replaced with y ~ x1c*x2c.
 I found this to be a very complicated problem with a very satisfying answer.
 
\end_layout

\begin_layout Standard
My first effort used R's update function.
 That is the most obvious approach.
 I learned very quickly that update() is not sufficient.
 It is fairly easy to replace x1 with x1c in the formula, but not when x1
 is logged or otherwise transformed.
 Here is the runable example code that demonstrates the problem.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<include=T, results=hide>>=
\end_layout

\begin_layout Plain Layout

dat <- data.frame(x1 = rnorm(100, m = 50), x2 = rnorm(100, m = 50),
\end_layout

\begin_layout Plain Layout

  x3 = rnorm(100, m = 50), x4 = rnorm(100, m=50), y = rnorm(100))
\end_layout

\begin_layout Plain Layout

m2 <- lm(y ~ log(x1) + x2*x3, data = dat)
\end_layout

\begin_layout Plain Layout

suffixX <- function(fmla, x, s){
\end_layout

\begin_layout Plain Layout

    upform <- as.formula(paste(".
 ~ .", "-", x, "+", paste(x, s, sep = ""), sep=""))
\end_layout

\begin_layout Plain Layout

    update.formula(fmla, upform)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

newFmla <- formula(m2)
\end_layout

\begin_layout Plain Layout

newFmla
\end_layout

\begin_layout Plain Layout

suffixX(newFmla, "x2", "c")
\end_layout

\begin_layout Plain Layout

suffixX(newFmla, "x1", "c")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Run that and check the last few lines of the output.
 See how the update misses x1 inside log(x1) or in the interaction?
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> newFmla <- formula(m2)
\end_layout

\begin_layout Plain Layout

> newFmla
\end_layout

\begin_layout Plain Layout

y ~ log(x1) + x2 * x3
\end_layout

\begin_layout Plain Layout

> suffixX(newFmla, "x2", "c")
\end_layout

\begin_layout Plain Layout

y ~ log(x1) + x3 + x2c + x2:x3
\end_layout

\begin_layout Plain Layout

> suffixX(newFmla, "x1", "c")
\end_layout

\begin_layout Plain Layout

y ~ log(x1) + x2 + x3 + x1c + x2:x3
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I asked the members of r-help for assistance.
 Lately I've had very good luck with r-help.
 Gabor Grothendieck wrote an answer to r-help on May 29, 2012.
 He said simply, 
\begin_inset Quotes eld
\end_inset

Try substitute,
\begin_inset Quotes erd
\end_inset

 with this example.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> do.call("substitute", list(newFmla, setNames(list(as.name("x1c")), "x1")))
\end_layout

\begin_layout Plain Layout

y ~ log(x1c) + x2 * x3
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Problem solved, in a single line.
 
\end_layout

\begin_layout Standard
That's very clever, I think.
 It packs together a half-dozen very deep thoughts.
 It has most of the essential secrets of R's guts, laid out in a single
 line.
 It has do.call(), substitute(), it interprets a formula as a list, and it
 shows that every command in R is, when it comes down to brass tacks, a
 list.
 
\end_layout

\begin_layout Standard
I would like to take up these separate pieces in order.
\end_layout

\begin_layout Subsection
A Formula Object is a List.
\end_layout

\begin_layout Standard
While struggling with this, I noticed this really interesting pattern.
 The solution depends on it.
 The object 
\begin_inset Quotes eld
\end_inset

newFmla
\begin_inset Quotes erd
\end_inset

 is not just a text string.
 It prints out as if it were text, but it is actually an R list object.
 Its parts can be probed recursively, to eventually reveal all of the individual
 pieces:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<newFla10>>=
\end_layout

\begin_layout Plain Layout

newFmla
\end_layout

\begin_layout Plain Layout

newFmla[[1]]
\end_layout

\begin_layout Plain Layout

newFmla[[2]]
\end_layout

\begin_layout Plain Layout

newFmla[[3]]
\end_layout

\begin_layout Plain Layout

newFmla[[3]][[2]]
\end_layout

\begin_layout Plain Layout

newFmla[[3]][[2]][[2]]
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
How could I put that information to use? Read on.
\end_layout

\begin_layout Subsection
do.call() and eval()
\end_layout

\begin_layout Standard
In my early work as an Rchaeologist, I had noticed eval() and do.call(),
 but did not understand their significance.
 Coming to grips with these ideas is a critical step in the R programmer's
 growth, because they separate the 
\begin_inset Quotes eld
\end_inset

script writer
\begin_inset Quotes erd
\end_inset

 from the 
\begin_inset Quotes eld
\end_inset

language programmer.
\begin_inset Quotes erd
\end_inset

 Whenever difficult problems arise in r-help, the answer almost invariably
 involves do.call() or eval().
 
\end_layout

\begin_layout Subsubsection
do.call()
\end_layout

\begin_layout Standard
Let's concentrate on do.call first.
 The syntax is like this
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

do.call("someRFunction", aListOfArgumentsToGoInTheParentheses)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
It is as if we were telling R to run this:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

someRFunction(aListOfArgumentsToGoInTheParentheses)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
We use do.call() because it is much more flexible than calling someRFunction()
 directly.
\end_layout

\begin_layout Standard
Let's consider an example that runs a regression the ordinary way, and then
 with do.call.
 In this example, the role of 
\begin_inset Quotes eld
\end_inset

someRFunction
\begin_inset Quotes erd
\end_inset

 will be played by lm() and the list of arguments will be the parameters
 of the regression.
 The regression m1 will be constructed the ordinary way, while m2 is constructed
 with do.call().
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

m1 <- lm(y ~ x1*x2, data = dat)
\end_layout

\begin_layout Plain Layout

coef(m1)
\end_layout

\begin_layout Plain Layout

regargs <- list(formula = y ~ x1*x2, data = quote(dat))
\end_layout

\begin_layout Plain Layout

m2 <- do.call("lm", regargs)
\end_layout

\begin_layout Plain Layout

coef(m2)
\end_layout

\begin_layout Plain Layout

all.equal(m1, m2)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The object regargs is a list of arguments that R can understand when they
 are supplied to the lm function.
 
\end_layout

\begin_layout Standard
do.call() is a powerful, mysterious symbol.
 It holds flexibility; we can calculate 
\begin_inset Quotes eld
\end_inset

on the language
\begin_inset Quotes erd
\end_inset

 to create commands and then run them.
 I first needed do.call() when we had a simulation project that ran very
 slowly.
 There's a writeup in the working examples distributed with rockchalk called
 stackListItems-01.R.
 I was using rbind() over and over to join the results of simulation runs.
 Basically, the code was like this
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

for (i in 1:10000){
\end_layout

\begin_layout Plain Layout

   dat <- someHugeSimulation(i)
\end_layout

\begin_layout Plain Layout

   result <- rbind(result, dat)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
That will call rbind() 10000 times.
 I had not realized that rbind() is time-consuming.
 It accesses a new chunk of memory each time it is run.
 On the other hand, we could collect those results in a list, then we can
 call rbind() one time to smash together all of the results.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

for (i in 1:10000){
\end_layout

\begin_layout Plain Layout

   mylist[[i]] <- someHugeSimulation(i)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

result <- do.call("rbind", mylist)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
It is much faster to run rbind() only once.
 It would be OK if we typed it all out like this:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

result <- rbind(mylist[[1]], mylist[[2]], mylist[[3]], ..., mylist[[10000])
\end_layout

\end_inset


\end_layout

\begin_layout Standard
But who wants to do all of that typing? How tiresome! Thanks to Erik Iverson
 in r-help, I understand that
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

result <- do.call("rbind", mylist)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
is doing the EXACT same thing.
 
\begin_inset Quotes eld
\end_inset

mylist
\begin_inset Quotes erd
\end_inset

 is a list of arguments.
 do.call is 
\emph on
constructing
\emph default
 a function call from the list of arguments.
 It is 
\emph on
as if
\emph default
 I had actually typed rbind with 10000 arguments.
\end_layout

\begin_layout Standard
The beauty in this is that we could design a program that can assemble the
 list of arguments, and also choose the function to be run, on the fly.
 We are not required to literally write the function in quotes, as in 
\begin_inset Quotes eld
\end_inset

rbind
\begin_inset Quotes erd
\end_inset

.
 We could instead have a variable that is calculated to select one function
 among many, and then use do.call on that.
 In a very real sense, we could write a program that can write itself as
 it runs.
 
\end_layout

\begin_layout Standard
From all of this (and a peek at ?call), I arrive at an Rchaeological eureka!
 A call object is a quoted command plus a list of arguments for that command.
 
\end_layout

\begin_layout Subsubsection
eval()
\end_layout

\begin_layout Standard
Where does eval() fit into the picture? do.call() manufactures a call and
 executes it immediately.
 It is possible to arrest the process mid stream, to capture the call without
 evaluating it with eval().
 The terminology in the R documentation on this is difficult, mainly because
 there are several different ways to create a piece of not-yet-evaluated
 code.
 By 
\begin_inset Flex Strong
status collapsed

\begin_layout Plain Layout
not-yet-evaluated code
\end_layout

\end_inset

, I mean an 
\begin_inset Quotes eld
\end_inset

unevaluated expression
\begin_inset Quotes erd
\end_inset

 or a 
\begin_inset Quotes eld
\end_inset

call object
\begin_inset Quotes erd
\end_inset

, anything that can be handled by the eval() function.
 A not-yet-evaluated object holds some syntax that can we need later.
 I think of do.call() as a contraction of 
\begin_inset Quotes eld
\end_inset

eval
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

call
\begin_inset Quotes erd
\end_inset

.
 As far as I can tell, do.call("lm", list(arguments)) is the same as eval(call(
\begin_inset Quotes eld
\end_inset

lm
\begin_inset Quotes erd
\end_inset

, arguments)).
 
\end_layout

\begin_layout Standard
There are many ways to ask R to create the not-yet-evaluated object, that's
 one of the confusing things (in my opinion).
 Most obviously, call() function can be used for that.
 But functions like quote() and expression() can also create not-yet-evaluated
 code.
\end_layout

\begin_layout Standard
Everybody has experienced an error like this:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> x1 + x2
\end_layout

\begin_layout Plain Layout

Error: object 'x1' not found
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Whenever we use a variable, the R interpreter expects to find it.
 We know those variables are in the data frame dat, but the interpreter
 does not know that's what we mean.
 But we can ask R to trust us with the quote() function, which, basically,
 means 
\begin_inset Quotes eld
\end_inset

here's some code we will evaluate later
\begin_inset Quotes erd
\end_inset

:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> mycall <- quote(x1 + x2)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The object 
\begin_inset Quotes eld
\end_inset

mycall
\begin_inset Quotes erd
\end_inset

 is an R call object, something that might be evaluated in the future.
 
\end_layout

\begin_layout Standard
When we want to evaluate 
\begin_inset Quotes eld
\end_inset

x1 + x2
\begin_inset Quotes erd
\end_inset

, we ask R do to so:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

eval(mycall, dat)
\end_layout

\end_inset

The second argument supplies the place where x1 and x2 are to be found.
 R users may never have noticed the eval() function, but most will have
 used the with() function, which is simply a clever re-phrasing of the eval
 function's interface.
 It does the same thing:
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

with(dat, x1 + x2)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
and so does 
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

with(dat, eval(mycall)).
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Now, back to the regression example we were working on before.
 We can create the call object using the call() function, which requires
 the name of a function and a comma-separated list of quoted arguments.
 I quote each of the bits of syntax in the arguments because I don't want
 R to replace 
\begin_inset Quotes eld
\end_inset

dat
\begin_inset Quotes erd
\end_inset

 with the actual data frame, I only want it to remember the name.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

m3 <- lm(y ~ x1*x2, data = dat)
\end_layout

\begin_layout Plain Layout

coef(m3)
\end_layout

\begin_layout Plain Layout

mycall <- call("lm", quote(y ~ x1*x2), data = quote(dat))
\end_layout

\begin_layout Plain Layout

m4 <- eval(mycall)
\end_layout

\begin_layout Plain Layout

coef(m4)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The main reason for using eval is that we can 
\begin_inset Quotes eld
\end_inset

piece together
\begin_inset Quotes erd
\end_inset

 commands and then run them after we have assembled all the pieces.
 The nickname for this seems to be 
\begin_inset Quotes eld
\end_inset

computing on the language.
\begin_inset Quotes erd
\end_inset

 There is a discussion of it in the R Language Definition.
 It is also surveyed in Hadley Wickham's new book, 
\emph on
Advanced R
\emph default
 
\begin_inset CommandInset citation
LatexCommand citeyearpar
key "Wickham2015"

\end_inset

.
\end_layout

\begin_layout Standard
Did you ever notice that you get an error if you run 
\begin_inset Quotes eld
\end_inset

x1 + x2
\begin_inset Quotes erd
\end_inset

, but if you run 
\begin_inset Quotes eld
\end_inset

y 
\begin_inset Formula $\sim$
\end_inset

 x1 + x2
\begin_inset Quotes erd
\end_inset

, then there is no error? It is as if R is wrapping your code in protective
 tissue, something like quote().
 However, the protective wrapper in this case is actually the function called
 formula().
 The R interpreter notices the symbol 
\begin_inset Formula $\sim$
\end_inset

 in the syntax, and so it assumes you meant to run
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

formula(y ~ x1 + x2)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
A formula is another kind of not-yet-evaluated thing.
 It is something that we can fiddle with before evaluating.
 
\end_layout

\begin_layout Standard
Create a complicated formula.
 Note that the lm function receives that formula object without trouble.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

f1 <- y ~ x1 + x2 + x3 + log(x4)
\end_layout

\begin_layout Plain Layout

class(f1)
\end_layout

\begin_layout Plain Layout

m5 <- lm(f1, data = dat)
\end_layout

\begin_layout Plain Layout

coef(m5)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The object f1 is a formula object.
 Its not just a text string.
 Observe it has separate pieces, just like newFmla in the example problem
 that started this section.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

f1[[1]]
\end_layout

\begin_layout Plain Layout

f1[[2]]
\end_layout

\begin_layout Plain Layout

f1[[3]]
\end_layout

\begin_layout Plain Layout

f1[[3]][[1]]
\end_layout

\begin_layout Plain Layout

f1[[3]][[2]]
\end_layout

\begin_layout Plain Layout

f1[[3]][[3]]
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\begin_inset Newline newline
\end_inset

Note that f1 created in this way must be a syntactically valid R formula;
 it cannot include any other regression options.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> f1 <- y ~ x1 + x2 + x3 + log(x4), data=dat 
\end_layout

\begin_layout Plain Layout

Error: unexpected ',' in "f1 <- y ~ x1 + x2 + x3 + log(x4),"
\end_layout

\end_inset


\end_layout

\begin_layout Standard
If I declare f1exp as an expression, then R does not re-interpret it as
 a formula (f1exp is an unevaluated expression, the R parser has not translated
 it yet).
 To use that as a formula in the regression, we have to evaluate it.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

f1exp <- expression(y ~ x1 + x2 + x3 + log(x4))
\end_layout

\begin_layout Plain Layout

class(f1exp)
\end_layout

\begin_layout Plain Layout

m6 <- lm(eval(f1exp), data = dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
When f1exp is evaluated, what do we have? Here's the answer.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

f1expeval <- eval(f1exp)
\end_layout

\begin_layout Plain Layout

class(f1expeval)
\end_layout

\begin_layout Plain Layout

all.equal(f1expeval, f1)
\end_layout

\begin_layout Plain Layout

m7 <- lm(f1expeval, data=dat)
\end_layout

\begin_layout Plain Layout

all.equal(coef(m5), coef(m6), coef(m7))	
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The point here is that the pieces of an ordinary use command can be separated
 and put back together again before the work of doing calculations begins.
 We can edit the formula.
 Suppose we replace a part:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

f1[[3]][[2]] <- quote(x1 + log(x2))
\end_layout

\begin_layout Plain Layout

m8 <- lm(f1, data = dat)
\end_layout

\begin_layout Plain Layout

coef(m8)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Now we turn back to the main theme.
 How is eval() used in functions? Some functions take a lot of arguments.
 We may want to do a lot of fine tuning on the arguments, before they are
 assembled and evaluated.
 Hence, it is very important that we can keep some bits of R that are symbolic,
 rather than calculated, until we need them.
 
\end_layout

\begin_layout Standard
If you are looking for examples of eval() in the R source code, there is
 a nice one in the beginning of the lm() function.
 Suppose a user submits a command like 
\begin_inset Quotes eld
\end_inset

lm(y ~ x, data = dat, x = TRUE, y = TRUE).
\begin_inset Quotes erd
\end_inset

 Inside lm(), the import is re-organized.
 Here are the first lines of the lm() function
\end_layout

\begin_layout Standard
\begin_inset listings
lstparams "breaklines=true,numbers=left,numberstyle={\footnotesize},tabsize=4"
inline false
status open

\begin_layout Plain Layout

lm <- function (formula, data, subset, weights, na.action, method = "qr",
\end_layout

\begin_layout Plain Layout

    model = TRUE, x = FALSE, y = FALSE, qr = TRUE, singular.ok = TRUE,
\end_layout

\begin_layout Plain Layout

    contrasts = NULL, offset, ...)
\end_layout

\begin_layout Plain Layout

{
\end_layout

\begin_layout Plain Layout

    ret.x <- x
\end_layout

\begin_layout Plain Layout

    ret.y <- y
\end_layout

\begin_layout Plain Layout

    cl <- match.call()
\end_layout

\begin_layout Plain Layout

    mf <- match.call(expand.dots = FALSE)
\end_layout

\begin_layout Plain Layout

    m <- match(c("formula", "data", "subset", "weights", "na.action",
\end_layout

\begin_layout Plain Layout

        "offset"), names(mf), 0L)
\end_layout

\begin_layout Plain Layout

    mf <- mf[c(1L, m)]
\end_layout

\begin_layout Plain Layout

    mf$drop.unused.levels <- TRUE
\end_layout

\begin_layout Plain Layout

    mf[[1L]] <- as.name("model.frame")
\end_layout

\begin_layout Plain Layout

    mf <- eval(mf, parent.frame())
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Now suppose we turn on the debugger and run a regression in R with a command
 like this.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

debug(lm)
\end_layout

\begin_layout Plain Layout

m1 <- lm(y ~ x1 * x2, data = dat, x = TRUE, y = TRUE)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
After that, we are 
\begin_inset Quotes eld
\end_inset

in
\begin_inset Quotes erd
\end_inset

 the function, stepping through line-by-line.
 In line 8, the match.call() function is used to grab a copy of the command
 that I typed.
 We can see that mf is exactly the same as my command, except R has named
 the arguments:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> mf
\end_layout

\begin_layout Plain Layout

lm(formula = y ~ x1 * x2, data = dat, x = TRUE, y = TRUE)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
That's not just a string of letters, however.
 It is a call object, a list with individual pieces that can be revised.
 Recall that the first element in a call object is the name of a function,
 and the following elements are the arguments.
 Lines 10 and 11 check the names of mf for the presence of certain arguments,
 and throw away the rest.
 It only wants the arguments we would be needed to run the function model.frame.
 Line 12 adds an argument to the list, drop.unused.levels.
 Up to that point, then, we can look at the individual pieces of mf:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> names(mf) 
\end_layout

\begin_layout Plain Layout

[1] ""      "formula"       "data" [4] "drop.unused.levels"
\end_layout

\begin_layout Plain Layout

> mf[[1]]
\end_layout

\begin_layout Plain Layout

lm
\end_layout

\begin_layout Plain Layout

> mf[[2]]
\end_layout

\begin_layout Plain Layout

y ~ x1 * x2
\end_layout

\begin_layout Plain Layout

> mf[[3]]
\end_layout

\begin_layout Plain Layout

dat
\end_layout

\begin_layout Plain Layout

> mf[[4]] 
\end_layout

\begin_layout Plain Layout

[1] TRUE
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The object mf has separate pieces that can be revised and then evaluated.
 Line 13 replaces the element 1 in mf with the symbol 
\begin_inset Quotes eld
\end_inset

model.frame
\begin_inset Quotes erd
\end_inset

.
 That's the function that will be called.
 Line 14 is the coup de grâce, when the revised call 
\begin_inset Quotes eld
\end_inset

mf
\begin_inset Quotes erd
\end_inset

 is sent to eval.
 In the end, it is 
\emph on
as if
\emph default
 lm had directly submitted the command
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

mf <- model.frame(y ~ x1 * x2, data = dat, drop.unused.levels = TRUE)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
It would not do to simply write that into the lm function, however, because
 some people use variables that have names different from y, x1, and x2,
 and their data objects may not be called dat.
 lm allows users to input whatever they want for a formula and data, and
 then lm takes what it needs to build a model frame.
\end_layout

\begin_layout Subsection
substitute()
\end_layout

\begin_layout Standard
Most R users I know have not used substitute, except possibly if they try
 to use plotmath.
 In the context of plotmath, the problem is as follows.
 Plotmath causes the R plot functions to convert expressions into mathematical
 symbols in a way that is reminiscent of LaTeX.
 For example, a command like this:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

text(4, 4, expression(gamma))
\end_layout

\end_inset

will draw the gamma symbol at the position (4,4).
 We can use paste to combine symbolic commands and text like so:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

text(4, 4, expression(paste(gamma, " = 7")))
\end_layout

\end_inset

The number 7 is a nice number, but what if we want to calculate something
 and insert it into the expression? Your first guess might be to insert
 a function that makes a calculation, such as the mean, but this fails:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

text(4, 4, expression(paste(gamma, mean(x))))
\end_layout

\end_inset

In order to smuggle the result of a calculation into an expression, some
 fancy footwork is required.
 In the help page for plotmath, examples using the functions bquote and
 substitute are offered.
 I have found this to be quite frustrating and difficult to do, and if one
 examines my collection of examples on 
\begin_inset Flex URL
status open

\begin_layout Plain Layout

http://pj.freefaculty.org/R/WorkingExamples
\end_layout

\end_inset

, one will find quite a few plotmath exercises.
\end_layout

\begin_layout Standard
For the particular purpose of blending expressions with calculation results,
 I find the bquote function to be more easily understandable.
 However, when going beyond plotmath, I expect most R programmers will need
 to use substitute() instead, so I will discuss that in this section.
 The plotmath help page points to syntax like this:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<fig=T, height=3, width=4>>=
\end_layout

\begin_layout Plain Layout

plot(1:10, seq(1,5, length.out=10), type = "n", main="Illustrating Substitute
 with plotmath", xlab="x", ylab="y")
\end_layout

\begin_layout Plain Layout

text(5, 4, substitute(gamma + x1mean, list(x1mean = mean(dat$x1))))
\end_layout

\begin_layout Plain Layout

text(5, 2, expression(paste(gamma, " plus the mean of x1")))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Run ?substitute and one is brought to a famous piece of Rchaeological pottery:
 
\end_layout

\begin_layout Quote
‘substitute’ returns the parse tree for the (unevaluated) expression ‘expr’,
 substituting any variables bound in ‘env’.
 
\end_layout

\begin_layout Standard
Pardon me.
 parse tree? We've seen expressions already, that part is not so off putting.
 But 
\begin_inset Quotes eld
\end_inset

parse tree
\begin_inset Quotes erd
\end_inset

? Really?
\end_layout

\begin_layout Standard
This is one of those points at which being an Rchaeologist has real benefits.
 We have to dig deeper, to try to understand not only what the R programmer
 says, but what she is actually trying to do.
 The manual page gives us some insights into the R programmer, and it is
 his or her view of his or her own actions, but it doesn't necessarily speak
 to how we should understand substitute().
 
\end_layout

\begin_layout Standard
For me, the only workable approach is to build up a sequence of increasingly
 complicated examples.
 I start by creating the list of replacements.
 This replacement list can have a format like this: 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

sublist <- list(x1 = "alphabet", x2 = "zoology")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Suppose I have some other object in which x1 and x2 need to be replaced.
 Specifically, when an expression has 
\begin_inset Quotes eld
\end_inset

x1
\begin_inset Quotes erd
\end_inset

, I want 
\begin_inset Quotes eld
\end_inset

alphabet
\begin_inset Quotes erd
\end_inset

, and 
\begin_inset Quotes eld
\end_inset

x2
\begin_inset Quotes erd
\end_inset

 should become 
\begin_inset Quotes eld
\end_inset

zoology.
\begin_inset Quotes erd
\end_inset

 The quotes in the R code indicate that alphabet and zoology are character
 strings, not other objects that already exist.
 Consider replacing x1 and x2 in x1 + x2 + log(x1) + x3:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

substitute(expression(x1 + x2 + log(x1) + x3), sublist)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Note that the substitution 1) leaves other variables alone (since they are
 not named in sublist) and 2) it finds all valid uses of the symbols x1
 and x2 and replaces them.
\end_layout

\begin_layout Standard
This isn't quite what I wanted, however, because the strings have been inserted
 into the middle of my expression.
 I don't want text.
 I just want symbols.
 I've seen the conversion from character to symbol done with as.symbol()
 and with as.name(), recently I realized that they are synonymous.
 I usually use as.symbol() because that name has more intuition for me, but
 in Gabor's answer to my question, as.name() is used.
 Using as.name(), my example would become:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

sublist <- list(x1 = as.name("alphabet"), x2 = as.name("zoology"))
\end_layout

\begin_layout Plain Layout

substitute(expression(x1 + x2 + log(x1) + x3), sublist)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
setNames and names
\end_layout

\begin_layout Standard
Almost every R user has noticed that the elements of R lists can have names.
 In a data frame, the names of the list elements are thought of as variable
 names, or column names.
 If dat is a data frame, the names and colnames functions return the same
 thing, but that's not true for other types of objects.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

dat <- data.frame(x1=1:10, x2=10:1, x3=rep(1:5,2), x4=gl(2,5))
\end_layout

\begin_layout Plain Layout

colnames(dat)
\end_layout

\begin_layout Plain Layout

names(dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
After dat is created, we can change the names inside it with a very similar
 approach:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

newnames <- c("whatever","sounds","good","tome")
\end_layout

\begin_layout Plain Layout

colnames(dat) <- newnames
\end_layout

\begin_layout Plain Layout

colnames(dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
While used interactively, this is convenient, but it is a bit tedious because
 we have to create dat first, and then set the names.
 The setNames() function allows us to do this in one shot.
 I'll paste the data frame creating commands and the name vector in for
 a first try:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

dat2 <- setNames(data.frame(x1 = rnorm(10), x2 = rnorm(10),
\end_layout

\begin_layout Plain Layout

    x3 = rnorm(10), x4 = gl(2,5)), c("good", "names", "tough", "find"))
\end_layout

\begin_layout Plain Layout

head(dat2, 2)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
In order to make this more generally useful, the first step is to take the
 data-frame-creating code and set it into an expression that is not immediately
 evaluated (that's datcommand in what follows).
 When I want the data frame to be created, I use eval(), and then the newnames
 vector is put to use.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

newnames <- c("iVar", "uVar", "heVar", "sheVar")
\end_layout

\begin_layout Plain Layout

datcommand <- expression(data.frame(x1=1:10, x2=10:1, x3=rep(1:5,2), x4=gl(2,5)))
\end_layout

\begin_layout Plain Layout

eval(datcommand)
\end_layout

\begin_layout Plain Layout

dat3 <- setNames(eval(datcommand), newnames)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The whole point of this exercise is that we can write code that creates
 the names, and creates the data frame, and then they all come together.
\end_layout

\begin_layout Standard
What if we have just one element in a list? In Gabor's answer to my question,
 there is this idiom
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

setNames(list(as.name("x1c")), "x1")))
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Consider this from the inside out.
 
\end_layout

\begin_layout Enumerate
as.name(
\begin_inset Quotes eld
\end_inset

x1c
\begin_inset Quotes erd
\end_inset

) is an R symbol object,
\end_layout

\begin_layout Enumerate
list(as.name(
\begin_inset Quotes eld
\end_inset

x1c
\begin_inset Quotes erd
\end_inset

)) is a list with just one object, which is that symbol object.
 
\end_layout

\begin_layout Enumerate
Use setNames().
 The object has no name! We would like to name it 
\begin_inset Quotes eld
\end_inset

x1
\begin_inset Quotes erd
\end_inset

.
 
\end_layout

\begin_layout Standard
It is as if we had run the command list(x1 = x1c).
 The big difference, of course, is that this way is much more flexible because
 we can calculate replacements.
 
\end_layout

\begin_layout Subsection
The Big Finish
\end_layout

\begin_layout Standard
In the meanCenter() function in rockchalk, some predictors are mean-centered
 and their names are revised.
 A variable named 
\begin_inset Quotes eld
\end_inset

age
\begin_inset Quotes erd
\end_inset

 becomes 
\begin_inset Quotes eld
\end_inset

agec
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

x1
\begin_inset Quotes erd
\end_inset

 becomes 
\begin_inset Quotes eld
\end_inset

x1c
\begin_inset Quotes erd
\end_inset

.
 So the user's regression formula that uses variables age or x1 must be
 revised.
 This is a function that takes a formula 
\begin_inset Quotes eld
\end_inset

fmla
\begin_inset Quotes erd
\end_inset

 and replaces a symbol xname with newname.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

formulaReplace <- function(fmla, xname, newname){ 
\end_layout

\begin_layout Plain Layout

    do.call("substitute", list(fmla, setNames(list(as.name(newname)), xname)))
 
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This is put to use in meanCenter().
 Suppose a vector of variable names called nc (stands for 
\begin_inset Quotes eld
\end_inset

needs centering
\begin_inset Quotes erd
\end_inset

) has already been calculated.
 The function std() creates a centered variable.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

newFmla <- mc$formula
\end_layout

\begin_layout Plain Layout

for (i in seq_along(nc)){
\end_layout

\begin_layout Plain Layout

    icenter <- std(stddat[, nc[i]])
\end_layout

\begin_layout Plain Layout

    newname <- paste(as.character(nc[i]), "c", sep = "")
\end_layout

\begin_layout Plain Layout

    newFmla <- formulaReplace(newFmla,  as.character(nc[i]), newname)
\end_layout

\begin_layout Plain Layout

    nc[i] <- newname
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
If one has a copy of rockchalk 1.6 or newer, the evidence of the success
 of this approach should be evident in the output of the command example(meanCen
ter).
 
\end_layout

\begin_layout Section
Make Re-Usable Tools (Rather than Cutting and Pasting)
\end_layout

\begin_layout Standard
We seek concise solutions that are generalizable.
 It is almost never right to cut and paste and then make minor edits in
 each copy to achieve particular purposes.
 This advice goes against the grain of the graduate students that I work
 with.
 They will almost always use cut and paste solutions.
 
\end_layout

\begin_layout Standard
Here is an example of the problem.
 A person needed 
\begin_inset Quotes eld
\end_inset

dummy variables
\begin_inset Quotes erd
\end_inset

.
 The code had several pages like this:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

  if(setcorr == 1){
\end_layout

\begin_layout Plain Layout

        corr.10 <-1
\end_layout

\begin_layout Plain Layout

        corr.20 <-0
\end_layout

\begin_layout Plain Layout

        corr.30 <-0
\end_layout

\begin_layout Plain Layout

        corr.40 <-0
\end_layout

\begin_layout Plain Layout

        corr.50 <-0
\end_layout

\begin_layout Plain Layout

        corr.60 <-0
\end_layout

\begin_layout Plain Layout

        corr.70 <-0
\end_layout

\begin_layout Plain Layout

        corr.80 <-0
\end_layout

\begin_layout Plain Layout

    }
\end_layout

\begin_layout Plain Layout

    if(setcorr == 2){
\end_layout

\begin_layout Plain Layout

        corr.10 <-1
\end_layout

\begin_layout Plain Layout

        corr.20 <-1
\end_layout

\begin_layout Plain Layout

        corr.30 <-0
\end_layout

\begin_layout Plain Layout

        corr.40 <-0
\end_layout

\begin_layout Plain Layout

        corr.50 <-0
\end_layout

\begin_layout Plain Layout

        corr.60 <-0
\end_layout

\begin_layout Plain Layout

        corr.70 <-0
\end_layout

\begin_layout Plain Layout

        corr.80 <-0
\end_layout

\begin_layout Plain Layout

    }
\end_layout

\begin_layout Plain Layout

    if(setcorr == 3){
\end_layout

\begin_layout Plain Layout

        corr.10 <-1
\end_layout

\begin_layout Plain Layout

        corr.20 <-1
\end_layout

\begin_layout Plain Layout

        corr.30 <-1
\end_layout

\begin_layout Plain Layout

        corr.40 <-0
\end_layout

\begin_layout Plain Layout

        corr.50 <-0
\end_layout

\begin_layout Plain Layout

        corr.60 <-0
\end_layout

\begin_layout Plain Layout

        corr.70 <-0
\end_layout

\begin_layout Plain Layout

        corr.80 <-0
\end_layout

\begin_layout Plain Layout

    }
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Well, that's understandable, but a little bit embarrassing.
 I asked 
\begin_inset Quotes eld
\end_inset

why do you declare these separate variables, why not make a vector?
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

can't you see a more succinct way to declare those things?
\begin_inset Quotes erd
\end_inset

 The answer is 
\begin_inset Quotes eld
\end_inset

we do it that way in SAS
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

this runs
\begin_inset Quotes erd
\end_inset

.
 One might suspect that the author was eager to say, 
\begin_inset Quotes eld
\end_inset

my project is based on 10,000 lines of R code
\begin_inset Quotes erd
\end_inset

 in order to impress a lay audience.
\end_layout

\begin_layout Standard
If the mission is to create a vector with a certain number of 1's at the
 beginning, surely either of these functions would be better:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

biVec1 <- function(n = 3, n1s = 1) {
\end_layout

\begin_layout Plain Layout

    c(rep(1, n1s), rep(0, n - n1s))
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

biVec2 <- function(n = 3, n1s = 1){
\end_layout

\begin_layout Plain Layout

    x <- numeric(length = n)
\end_layout

\begin_layout Plain Layout

    x[1:n1s] <- 1
\end_layout

\begin_layout Plain Layout

    x
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

(corr <- biVec1(n = 8, n1s = 3))
\end_layout

\begin_layout Plain Layout

(corr <- biVec2(n = 8, n1s = 3))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The cut and paste way is not wrong, exactly.
 Its tedious.
 
\end_layout

\begin_layout Standard
I think any reasonable user should want a vector, rather than the separate
 variables.
 But suppose the user is determined, and really wants the individual variables
 named corr.10, corr.20, and so forth.
 We can re-design this so that those variables will be sitting out in the
 workspace after the function is run.
 Read help(
\begin_inset Quotes eld
\end_inset

assign
\begin_inset Quotes erd
\end_inset

) and try
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

biVec3 <- function(n = 3, n1s = 1) {
\end_layout

\begin_layout Plain Layout

    X <- c(rep(1, n1s), rep(0, n - n1s))
\end_layout

\begin_layout Plain Layout

    xnam <- paste("corr.", 1:8, "0", sep = "")
\end_layout

\begin_layout Plain Layout

    for(i in 1:n) assign(xnam[i], X[i], envir = .GlobalEnv) 
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

ls()
\end_layout

\begin_layout Plain Layout

biVec3(n = 8, n1s = 3)
\end_layout

\begin_layout Plain Layout

ls()
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I don't think most people will actually want that kind of result, but if
 they do, there is a way to get it.
\end_layout

\begin_layout Standard
If we want to turn biVec1() or biVec2() into general purpose functions,
 we need to start thinking about the problem of unexpected user input.
 Notice what happens if we ask for more 1's than the result vector is supposed
 to contain:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> biVec1(n = 3, n1s = 7)
\end_layout

\begin_layout Plain Layout

Error in rep(0, n - n1s) (from #2) : invalid 'times' argument
\end_layout

\begin_layout Plain Layout

> biVec2(n = 3, n1s = 7)
\end_layout

\begin_layout Plain Layout

[1] 1 1 1 1 1 1 1
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Perhaps I prefer biVec1() because it throws an error when there is unreasonable
 input, while biVec2() returns a longer vector than expected with no error.
 
\end_layout

\begin_layout Standard
If the user mistakenly enters non-integers, neither function generates an
 error, and they give us unexpected output.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

biVec1(3.3, 1.8)
\end_layout

\begin_layout Plain Layout

biVec2(3.3, 1.8)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
To my surprise, it is not an easy thing to ask a number if it is a whole
 number (see help(
\begin_inset Quotes eld
\end_inset

is.integer
\begin_inset Quotes erd
\end_inset

)).
 For that, we introduce a new function, is.wholenumber(), and put it to use
 in our new and improved function
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

is.wholenumber <- function(x, tol = .Machine$double.eps^0.5){
\end_layout

\begin_layout Plain Layout

 abs(x - round(x)) < tol
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout

biVec1 <- function(n = 3, n1s = 1) {
\end_layout

\begin_layout Plain Layout

    if(!(is.wholenumber(n) & is.wholenumber(n1s)))
\end_layout

\begin_layout Plain Layout

        stop("Both n and n1s must be whole numbers (integers)")
\end_layout

\begin_layout Plain Layout

    if(n1s > n)
\end_layout

\begin_layout Plain Layout

        stop("n must be greater than or equal to n1s")
\end_layout

\begin_layout Plain Layout

    c(rep(1, n1s), rep(0, n - n1s))
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
In all of the test cases I've tried, that works, although the real numbers
 7.0 and 4.0 are able to masquerade as integers.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> biVec1(3, 7)
\end_layout

\begin_layout Plain Layout

Error in biVec1(3, 7) (from #4) : n must be greater than or equal to n1s
\end_layout

\begin_layout Plain Layout

> biVec1(7, 3)
\end_layout

\begin_layout Plain Layout

[1] 1 1 1 0 0 0 0
\end_layout

\begin_layout Plain Layout

> biVec1(3.3, 4.4)
\end_layout

\begin_layout Plain Layout

Error in biVec1(3.3, 4.4) (from #2) : 
\end_layout

\begin_layout Plain Layout

  Both n and n1s must be whole numbers (integers)
\end_layout

\begin_layout Plain Layout

> biVec1(7.0, 4.0)
\end_layout

\begin_layout Plain Layout

[1] 1 1 1 1 0 0 0
\end_layout

\begin_layout Plain Layout

> biVec1(7.0, 4.0)
\end_layout

\begin_layout Plain Layout

[1] 1 1 1 1 0 0 0
\end_layout

\end_inset


\end_layout

\begin_layout Section
Function Arguments.
\end_layout

\begin_layout Standard
While developing functions for the rockchalk package, one of the most important
 learning experiences I've had is in argument management.
 
\end_layout

\begin_layout Standard
One of the reasons that it is difficult to design functions is that there
 is no 
\begin_inset Quotes eld
\end_inset

mandatory manual
\begin_inset Quotes erd
\end_inset

 on this.
 The R framework allows programmer creativity, which is a good thing.
 But it is also a bad thing, since every programmer has his/her own opinion
 about what ought to be done.
 To state the obvious, argument names should be clear, unambiguous, convenient,
 short, and minimally necessary.
 
\end_layout

\begin_layout Standard
Beyond that, what can we say? I urge new R programmers to study the R source
 code.
 This is part of the Rchaeological method.
 Study the most frequently used functions in the R distribution.
 I've learned the most from the way arguments are handled in the functions
 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
hist
\end_layout

\end_inset

(), 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
lm()
\end_layout

\end_inset

, 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
plot()
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
termplot()
\end_layout

\end_inset

.
 These functions have had the most eyes on them and, therefore, they are
 most likely to use the best approach.
 I wrote several functions that eventually went into rockchalk before I
 realized that important fact and I regret it (on a weekly basis).
\end_layout

\begin_layout Standard
In this section, I offer this advice.
\end_layout

\begin_layout Enumerate
Protect your function's calculations from the user's workspace.
\end_layout

\begin_layout Enumerate
Check argument values.
\end_layout

\begin_layout Enumerate
Design the function so that it runs with a minimum number of arguments.
\end_layout

\begin_layout Subsection
A few tidits about arguments
\end_layout

\begin_layout Standard
R includes several functions that are very helpful in dealing with arguments.
\end_layout

\begin_layout Description
missing() Inside a function, such as 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
plotme()
\end_layout

\end_inset

 we often need to check if the user supplied a value for an argument, such
 as 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
col
\end_layout

\end_inset

.
 If the user did not supply a value, we might construct one from the other
 input
\end_layout

\begin_deeper
\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

if (missing(col)) col <- someTediousCalculation(y, z)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This is usually good enough, but there is danger that the user might have
 called the function like so:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

plotme(x, y, z, col = NULL)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This outsmarts missing(col), which will return FALSE.
 If our goal is to make sure col is something, some non-NULL thing, a more
 careful argument check would be
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

if (missing(col) | is.null(col)) col <- someTediousCalculation(y, z)
\end_layout

\end_inset


\end_layout

\end_deeper
\begin_layout Description
stop() Halts a function and returns a message to the user.
 Usage example 
\end_layout

\begin_deeper
\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

if (isTRUE(x)) stop(paste("I'm sorry, you failed to supply the right information.
")))
\end_layout

\end_inset


\end_layout

\end_deeper
\begin_layout Description
stopifnot() It is so frequent to run 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
if(isTRUE(x)) stop()
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 that they created a wrapper, 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
stopifnot(x)
\end_layout

\end_inset

.
\end_layout

\begin_layout Description
warning() If you don't want the function to fail, but you need to warn the
 user about a condition that seems dubious, run
\end_layout

\begin_deeper
\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

if (isTRUE(x)) warning(paste("Are you sure the value of x ought to be",
 x, "?"))
\end_layout

\end_inset


\end_layout

\end_deeper
\begin_layout Description
deparse(substitute()) Inside a function, it is often important to find out
 the symbolic value of the argument that the user supplied to the function.
 I don't mean the 
\emph on
value
\emph default
, but rather the symbolic name.
 To see what I mean, run this
\end_layout

\begin_deeper
\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

dat <- data.frame(myx = c(1, 2, 3), myy = c(10, 5, 1))
\end_layout

\begin_layout Plain Layout

debug(plot.default)
\end_layout

\begin_layout Plain Layout

plot(dat$myx, dat$myy)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
After you hit enter, the R debugger/browser will come open and the prompt
 will allow you to inspect the situation.
 First, type 
\begin_inset Quotes eld
\end_inset

n
\begin_inset Quotes erd
\end_inset

 and hit return.
 That takes the first step into the calculations.
 After that, run 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
match.call()
\end_layout

\end_inset

.
 The function is somewhat 
\begin_inset Quotes eld
\end_inset

self aware
\begin_inset Quotes erd
\end_inset

.
 It knows how it was called.
 You should see this
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

Browse[2]> n 
\end_layout

\begin_layout Plain Layout

debug: localAxiis <- function(..., col, bg, pch, cex, lty, lwd) Axis(...)
\end_layout

\begin_layout Plain Layout

Browse[2]> match.call()
\end_layout

\begin_layout Plain Layout

plot.default(x = dat$myx, y = dat$myy)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
You asked for 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
plot()
\end_layout

\end_inset

, R sends the work to 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
plot.default()
\end_layout

\end_inset

.
 All is well.
\end_layout

\begin_layout Standard
Still inside the debugger now, type 
\begin_inset Quotes eld
\end_inset

x
\begin_inset Quotes erd
\end_inset

 and then 
\begin_inset Quotes eld
\end_inset

myx
\begin_inset Quotes erd
\end_inset

:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

Browse[2]> x
\end_layout

\begin_layout Plain Layout

[1] 1 2 3
\end_layout

\begin_layout Plain Layout

Browse[2]> myx
\end_layout

\begin_layout Plain Layout

Error: object 'myx' not found
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Inside the function, the vector value that we passed in, which was referred
 to as dat$myx, is now called x.
 The thing x is a data holder, it is used in the following calculations.
 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
x
\end_layout

\end_inset

 does not remember that its passed-in value was named 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
dat$myx
\end_layout

\end_inset

 in a previous life.
 It is as if 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
x
\end_layout

\end_inset

 is the new identity of the same old data vector.
 However, we can ask 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
x
\end_layout

\end_inset

 
\begin_inset Quotes eld
\end_inset

what name were you known by in your previous life?
\begin_inset Quotes erd
\end_inset

 with this interesting idiom.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

Browse[2]> deparse(substitute(x))
\end_layout

\begin_layout Plain Layout

[1] "dat$myx"
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The substitute() function, as we have already seen, is a way to replace
 a letter x with a value from somewhere.
 Maybe it would be more obvious to run this in 2 steps.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

Browse[2]> xsubs <- substitute(x)
\end_layout

\begin_layout Plain Layout

Browse[2]> xsubs
\end_layout

\begin_layout Plain Layout

dat$myx
\end_layout

\begin_layout Plain Layout

Browse[2]> deparse(xsubs)
\end_layout

\begin_layout Plain Layout

[1] "dat$myx"
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
substitute()
\end_layout

\end_inset

 grabs the object name from whence 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
x
\end_layout

\end_inset

 was calculated.
 We need to convert that back to a character string so that we can know
 the symbolic name of the thing that eventually became 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
x
\end_layout

\end_inset

.
 This is at the outer boundary of my R comprehension, so I better stop here.
\end_layout

\begin_layout Standard
Incidentally, you can go exploring in the debugger.
 Run some standards like 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
ls()
\end_layout

\end_inset

 to see what's available.
 If you keep hitting return, the function will run line by line.
 If you get tired of hitting return, type 
\begin_inset Quotes eld
\end_inset

c
\begin_inset Quotes erd
\end_inset

 and the function will run to its conclusion.
 To let R know that you don't want to debug that function any more, run:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> undebug(plot.default)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
If you don't do that, then the function browser will pop open every time
 you try to run plot.
\end_layout

\end_deeper
\begin_layout Description
list(...) The three periods, 
\begin_inset Quotes eld
\end_inset

...
\begin_inset Quotes erd
\end_inset

, are a valid word.
 (A period is a valid character).
 R scans user syntax for function arguments and it uses whatever it can
 understand (formal named arguments, positional matching), and then the
 other arguments are thrown into a thing called 
\begin_inset Quotes eld
\end_inset

...
\begin_inset Quotes erd
\end_inset

.
\end_layout

\begin_deeper
\begin_layout Standard
In a simple case, we could just pass those additional arguments to another
 function, as in
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

plotme <- function(x, y, ...){
\end_layout

\begin_layout Plain Layout

    plot(x, y, ...)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Often, it is not sufficient to simply pass on the 
\begin_inset Quotes eld
\end_inset

...
\begin_inset Quotes erd
\end_inset

 variables, and we instead need to scan them and sort them for various purposes.
 Inside the function, we access those variables by turning them into a list:
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{Sinput}
\end_layout

\begin_layout Plain Layout

dots <- list(...)
\end_layout

\begin_layout Plain Layout


\backslash
end{Sinput}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
We can still pass on the dots to other functions, but the usual problem
 is that some of the arguments in dots may be intended for several different
 functions.
 Suppose we are writing a plotter for categorical variables.
 We might need to receive some arguments, then run 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
table()
\end_layout

\end_inset

, then run 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
layout()
\end_layout

\end_inset

, then run 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
mosaicplot()
\end_layout

\end_inset

.
 The user may have put in arguments intended for those separate functions.
 There's a chance of an error, or at least a warning, if irrelevant arguments
 are passed to those arguments.
 
\end_layout

\begin_layout Standard
The rockchalk functions 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
plotSlopes()
\end_layout

\end_inset

 has examples that do this work.
 They separate the dot arguments that are needed for the 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
predict()
\end_layout

\end_inset

 function and the 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
plot()
\end_layout

\end_inset

 function.
 Because I was learning about arguments when I wrote those functions, I
 did not take the cleanest or most direct route to do this, but they do
 stand as working examples that users can review.
\end_layout

\begin_layout Standard
A function may set a number of defaults that we want to use when the users
 do not supply a lot of details.
 In a case like that, it is useful to remember a very handy function in
 R called 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
modifyList()
\end_layout

\end_inset

.
 The basic idea is that we can build a list of default arguments for a function,
 and then we can scan the 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
dots
\end_layout

\end_inset

 list and it will replace the defaults in the first with changes in the
 second.
 There's an example in rockchalk's 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
pctable()
\end_layout

\end_inset

 function.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{Sinput}
\end_layout

\begin_layout Plain Layout

dots <- list(...)
\end_layout

\begin_layout Plain Layout

dotnames <- names(dots)
\end_layout

\begin_layout Plain Layout

tableargs <- list(rv, cv, dnn = c(rvlab, cvlab))
\end_layout

\begin_layout Plain Layout

newargs <- modifyList(tableargs, dots, keep.null = TRUE)
\end_layout

\begin_layout Plain Layout

t1 <- do.call("table", newargs)
\end_layout

\begin_layout Plain Layout


\backslash
end{Sinput}
\end_layout

\end_inset

The dots list, by definition, is named arguments, and it is often handy
 to have a copy of their names for checking whether an argument is included
 in the list.
 In the plotSlopes() function, here's what I ended up with:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{Sinput}
\end_layout

\begin_layout Plain Layout

validForPredict <- c("se.fit", "dispersion", "terms", "na.action",
\end_layout

\begin_layout Plain Layout

                     "level", "pred.var", "weights")
\end_layout

\begin_layout Plain Layout

dotsForPredict <- dotnames[dotnames %in% validForPredict]
\end_layout

\begin_layout Plain Layout

if (length(dotsForPredict) > 0) {
\end_layout

\begin_layout Plain Layout

    parms <- modifyList(parms, dotargs[dotsForPredict])
\end_layout

\begin_layout Plain Layout

    dotargs[[dotsForPredict]] <- NULL
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

np <- do.call("predictCI", parms)
\end_layout

\begin_layout Plain Layout


\backslash
end{Sinput}
\end_layout

\end_inset

This scans dotnames for elements in the list of arguments that are valid
 for the predict method, and after they are put onto the parms list, then
 those arguments are set to NULL in the dotargs collection.
 Looking at that now, I see it would have been more clear to use 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
setdiff()
\end_layout

\end_inset

 on dotargs to clean that up.
\end_layout

\begin_layout Standard
The central challenge in that work is to find out the names of the arguments
 that are intended for one function, then remove them from 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
dots
\end_layout

\end_inset

 so that they are not passed to a subsequent function that needs them.
 How can one know what arguments are needed? This requires some experience,
 but the first step is to read the interface documentation of the function
 being considered.
 A shortcut is to run the function 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
formals()
\end_layout

\end_inset

, which prints out the names of the function being considered.
 This can sometimes be deceptive for implementations of generic methods.
 Observe that the return from the generic is not so informative as the return
 from the method:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

formals(plot)
\end_layout

\begin_layout Plain Layout

formals(plot.default)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
If a method is not exported, if it does not show as 
\begin_inset Quotes eld
\end_inset

visible
\begin_inset Quotes erd
\end_inset

 in the output from 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
methods(plot)
\end_layout

\end_inset

, then a little more fancy footwork is required to see all of the legal
 arguments.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

formals(graphics:::plot.histogram)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_deeper
\begin_layout Subsection
Protect your function's calculations from the user's workspace.
\end_layout

\begin_layout Standard
In the early days of using R, around 2000, it was possible for a user to
 obliterate base functions.
 I recall working on a model about Democratic and Republican voters.
 I created a variable named 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
rep
\end_layout

\end_inset

 and, after that, the 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
lm()
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
plot()
\end_layout

\end_inset

 functions would not work.
 I wrote to r-help about this and learned that my 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
rep
\end_layout

\end_inset

 had obliterated the built-in function 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
rep()
\end_layout

\end_inset

, which is vital in many calculations.
 Shortly after that, R was improved to protect functions from obliteration
 in that way.
 Today I tried to cause mischief by re-defining rep in a number of ways
 and it appears to me the problem is completely solved.
\end_layout

\begin_layout Standard
There has also been a problem that users sometime use the variable names
 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
T
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
F
\end_layout

\end_inset

.
 Recall that R allows 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
T
\end_layout

\end_inset

 as an abbreviation for 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
TRUE
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
F
\end_layout

\end_inset

 for 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
FALSE
\end_layout

\end_inset

.
 R will allow the user to re-define T and F in ways that will break calculations.
 However, R will not allow the user to re-define TRUE or FALSE.
 Observe
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

> TRUE <- 2342 
\end_layout

\begin_layout Plain Layout

Error in TRUE <- 2342 : invalid (do_set) left-hand side to assignment
\end_layout

\begin_layout Plain Layout

> FALSE <- function(x,y){x + y}
\end_layout

\begin_layout Plain Layout

Error in FALSE <- function(x, y) { :
\end_layout

\begin_layout Plain Layout

  invalid (do_set) left-hand side to assignment
\end_layout

\end_inset


\end_layout

\begin_layout Standard
In order to protect our function's calculations, we are urged to fully write
 out the symbols TRUE and FALSE inside functions.
 This way, users cannot cause damage.
 
\end_layout

\begin_layout Standard
Another danger we need to be concerned about is that our functions might
 have undefined variables in them.
 When R encounters these undefined variables, the lexical scoping process
 looks outward for values.
 It may find something we don't expect.
 Another danger is that a function seems to work right when we practice
 it in our session, but another user gets an error because something in
 our workspace is missing from theirs.
 The lexical scope can be convenient, but it is also dangerous.
\end_layout

\begin_layout Standard
Here the example function relies on variables 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
y
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

 that are not passed in as arguments.
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

x <- c(1, 2, 3, 4)
\end_layout

\begin_layout Plain Layout

y <- c(8, 9, 10, 11)
\end_layout

\begin_layout Plain Layout

z <- c(1, 1, 2, 2)
\end_layout

\begin_layout Plain Layout

plotme <- function(x){
\end_layout

\begin_layout Plain Layout

    plot(x, y, col = z)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

plotme(x)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
That lets the function go and find 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
y
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

.
 This will run, but it is pretty risky.
 It depends on R's willingness to reach outside the boundaries of the function
 to find 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
y
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

.
 If I have inadvertently altered 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
y
\end_layout

\end_inset

 in the workspace, this will end badly.
\end_layout

\begin_layout Standard
If I'm writing a function for somebody else to use, as in a package, I believe
 the function should to NOT rely on variables from the environment.
 I want to prevent R from filling in the gaps for users.
 We'd rather give the user an error 
\begin_inset Quotes eld
\end_inset

variable not found
\begin_inset Quotes erd
\end_inset

 than an erroneous calculation that 
\begin_inset Quotes eld
\end_inset

seems
\begin_inset Quotes erd
\end_inset

 adequate.
 The best case scenario is that 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
y
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

 do not exist in the workspace, so the user will receive an error message:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{Sinput}
\end_layout

\begin_layout Plain Layout

> plotme(x)
\end_layout

\begin_layout Plain Layout


\backslash
end{Sinput}
\end_layout

\begin_layout Plain Layout


\backslash
begin{Schunk}
\end_layout

\begin_layout Plain Layout

   
\backslash
begin{Soutput}
\end_layout

\begin_layout Plain Layout

Error in xy.coords(x, y, xlabel, ylabel, log) : object 'y' not found
\end_layout

\begin_layout Plain Layout

  
\backslash
end{Soutput}
\end_layout

\begin_layout Plain Layout


\backslash
end{Schunk}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The worst case scenario is that the user has some other variables 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
y
\end_layout

\end_inset

 and z
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout

\end_layout

\end_inset

 sitting about in the workspace.
 Those 
\begin_inset Quotes eld
\end_inset

found
\begin_inset Quotes erd
\end_inset

 variables are put to use with a completely unintentional result.
 
\end_layout

\begin_layout Standard
As a first step toward fixing this, I propose:
\end_layout

\begin_layout Quote
Every piece of input upon which we rely should be named in the list of arguments.
 
\end_layout

\begin_layout Standard
If we define the function as follows,
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

plotme <- function(x, y, z){
\end_layout

\begin_layout Plain Layout

    plot(x, y, col = z)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
the user's use of 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
plotme(x)
\end_layout

\end_inset

 will result in the error
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{Schunk}
\end_layout

\begin_layout Plain Layout

   
\backslash
begin{Soutput}
\end_layout

\begin_layout Plain Layout

Error in xy.coords(x, y, xlabel, ylabel, log) :
\end_layout

\begin_layout Plain Layout

    argument "y" is missing, with no default
\end_layout

\begin_layout Plain Layout

   
\backslash
end{Soutput}
\end_layout

\begin_layout Plain Layout


\backslash
end{Schunk}
\end_layout

\end_inset

Naming all of the arguments effectively protects the function against the
 accidental importation of data.
 
\end_layout

\begin_layout Standard
There's a function called 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
checkUsage()
\end_layout

\end_inset

 in the 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
codetools
\end_layout

\end_inset

 package that can help check functions for reliance on global variables.
 First, we'll make sure the workspace is clean by removing 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
x
\end_layout

\end_inset

, 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
y
\end_layout

\end_inset

, and 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{Schunk}
\end_layout

\begin_layout Plain Layout

   
\backslash
begin{Soutput}
\end_layout

\begin_layout Plain Layout

> rm(x, y, z)
\end_layout

\begin_layout Plain Layout

> library(codetools) 
\end_layout

\begin_layout Plain Layout

> checkUsage(plotme) 
\end_layout

\begin_layout Plain Layout

<anonymous>: no visible binding for global variable 'y' (:2) 
\end_layout

\begin_layout Plain Layout

<anonymous>: no visible binding for global variable 'z' (:2)
\end_layout

\begin_layout Plain Layout

   
\backslash
end{Soutput}
\end_layout

\begin_layout Plain Layout


\backslash
end{Schunk}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Note that, if the variables 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
x
\end_layout

\end_inset

, 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
y
\end_layout

\end_inset

, an 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

 exist in the user workspace, they are global and 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
checkUsage
\end_layout

\end_inset

 will not offer a warning about them.
 Hence, it is very important to run 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
checkUsage()
\end_layout

\end_inset

 after cleaning up the workspace.
 
\end_layout

\begin_layout Standard
checkUsage() will also offer warnings about unused variables.
 This can be helpful in cleaning up a function.
\end_layout

\begin_layout Subsection
Check argument values.
\end_layout

\begin_layout Standard
We want users to provide input arguments for all information that the calculatio
ns will rely upon.
 The next step is to check their inputs to make sure they are reasonable.
 
\end_layout

\begin_layout Standard
In some cases this is very easy.
 If we want the user to supply a data.frame, we can stop when the user provides
 something else.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

plotme <- function(x, y, data, ...){
\end_layout

\begin_layout Plain Layout

    if(missing(data) | !is.data.frame(data)) 
\end_layout

\begin_layout Plain Layout

        stop(paste0("plotme: the object you supplied as data: '", 
\end_layout

\begin_layout Plain Layout

        deparse(substitute(data)) , "' is not a data.frame"))
\end_layout

\begin_layout Plain Layout

    plot(as.formula(paste(y,  "~",  x)), data = data, ...)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

myx <- rnorm(10)
\end_layout

\begin_layout Plain Layout

myy <- rnorm(10)
\end_layout

\begin_layout Plain Layout

dat <- data.frame(myx, myy)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
This works properly
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

plotme("myx", "myy", data = dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
However, if the user supplies something else where data ought to be, an
 error is returned.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
begin{Sinput}
\end_layout

\begin_layout Plain Layout

plotme("myx", "myy", rnorm(10))
\end_layout

\begin_layout Plain Layout


\backslash
end{Sinput}
\end_layout

\begin_layout Plain Layout


\backslash
begin{Soutput}
\end_layout

\begin_layout Plain Layout

Error in plotme("myx", "myy", rnorm(10)) :
\end_layout

\begin_layout Plain Layout

  plotme: the object you supplied as data: 'rnorm(10)' is not a data.frame
\end_layout

\begin_layout Plain Layout


\backslash
end{Soutput}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The main point here is that if we definitely know what we expect the user
 to supply, then we can add a filter that checks the input's type.
 That's what is.character, is.vector, is.integer, and so forth are intended
 for.
 (In an R session, type 
\begin_inset Quotes eld
\end_inset

is
\begin_inset Quotes erd
\end_inset

 and then hit the TAB key a couple of times.
 You'll see all the completions.
\end_layout

\begin_layout Standard
The problem is not so easy to manage when we can't be sure what the argument
 type must be.
 Sometimes, it appears the best we can do is pass through what the users
 give us and hope R can make sense out of it.
 I've wrestled with R's flexibility for the naming of colors.
 Suppose our plotme() function displays x and y, and we allow the user to
 specify colors for the points.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

plotme <- function(x, y, z){
\end_layout

\begin_layout Plain Layout

    plot(x, y, col = z)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

myx <- rnorm(10)
\end_layout

\begin_layout Plain Layout

myy <- rnorm(10)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
R colors can be supplied in many formats, as integers, character strings,
 or more elaborate HTML-style color designators.
 Any of these will work.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

mycol <- 1:10
\end_layout

\begin_layout Plain Layout

plotme(myx, myy, z = mycol)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

mycol <- rainbow(10)
\end_layout

\begin_layout Plain Layout

plotme(myx, myy, z = mycol)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

mycol <- gray.colors(10)
\end_layout

\begin_layout Plain Layout

plotme(myx, myy, z = mycol)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
R does not throw an error if the user runs this, but the result is not what
 we wanted, probably.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

mycol <- rnorm(10)
\end_layout

\begin_layout Plain Layout

plotme(myx, myy, z = mycol)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Suppose the user's code ends up supplying a NULL value for 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

, as in 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
plotme(x, y, z = NULL)
\end_layout

\end_inset

.
 They would not do that on purpose, but it might be an accident.
 If 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

 is calculated in their code and something has gone wrong.
 To protect against that, we find this idiom is used in the R source code
 quite often:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

plotme <- function(x, y, z){
\end_layout

\begin_layout Plain Layout

    if (missing(z) | is.null(z)) z <- rep(2, length(x))   
\end_layout

\begin_layout Plain Layout

    plot(x, y, col = z)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
If 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

 is either not supplied by the user's command, or if the user's command
 explicitly sets it to NULL, then we'll fabricate 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

.
\end_layout

\begin_layout Subsection
Design the function so that it runs with a minimum number of arguments.
\end_layout

\begin_layout Standard
Take a look at the most commonly used functions in R, such as lm() or plot.defaul
t().
 These functions allow a great many arguments, but they will deliver what
 the users minimally need if the they run
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

m1 <- lm(y ~ x, data = dat)
\end_layout

\begin_layout Plain Layout

plot(y ~ x, data = dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Put the arguments that users will use most often at the front of the argument
 list, and then design the function so that it will work even if all of
 the other arguments are not included.
 There are two elements in making this work.
\end_layout

\begin_layout Subsubsection
Specify defaults
\end_layout

\begin_layout Standard
A graph can use any color scheme, but we might put in something colorful
 for users who like a little zest in life.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

plotme <- function(x, y, z = rainbow(length(x))){     
\end_layout

\begin_layout Plain Layout

    plot(x, y, col = z)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

plotme(x = rnorm(100), y = rnorm(100))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
We could as well have put the rainbow call inside the function if we want
 to keep a cleaner function declaration.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

plotme <- function(x, y, z){     
\end_layout

\begin_layout Plain Layout

  if(missing(z)) mycol <- rainbow(length(x))
\end_layout

\begin_layout Plain Layout

  plot(x, y, col = mycol)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

plotme(x = rnorm(100), y = rnorm(100))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
It seems to me that the R source itself tends to have more elaborate specificati
ons of defaults within the declaration, rather than inside the function.
\end_layout

\begin_layout Standard
In the R source code, I noticed a peculiar-looking example in which an argument'
s default depends on a function that is within the function itself.
 Observe that this runs fine:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

plotme <- function(x, y, z = getAColor(length(x)))
\end_layout

\begin_layout Plain Layout

{
\end_layout

\begin_layout Plain Layout

    getAColor <- function(n){
\end_layout

\begin_layout Plain Layout

             gray.colors(n)
\end_layout

\begin_layout Plain Layout

    }
\end_layout

\begin_layout Plain Layout

    plot(x, y, col = z, cex = 5)
\end_layout

\begin_layout Plain Layout

}
\end_layout

\begin_layout Plain Layout

plotme(rnorm(20), rnorm(20))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
That was startling to me because the nested function, 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
getAColor
\end_layout

\end_inset

, does not exist outside the function, and yet it can be used in the definition
 of a default.
 I merely promise that 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
getAColor
\end_layout

\end_inset

 will exist when the value of 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
z
\end_layout

\end_inset

 is needed and all is well.
 
\end_layout

\begin_layout Subsubsection
Extract or construct what else is needed from the user input
\end_layout

\begin_layout Standard
We can deduce information from the arguments that are passed in, we can
 eliminate the need for the user to pass in other arguments.
 We would never need to require both arguments x and xlab in the plot function,
 for example.
 xlab is optional because, if the user does not supply it, then deparse(substitu
te()) can find something to use as a label.
 The 2010 version of plotSlopes in rockchalk required the user to pass through
 many parameters that (I've since learned) can be deduced from the other
 arguments.
\end_layout

\begin_layout Standard
A key element in this is getting an understanding of the information that
 comes in with a standard R object.
 Quite a few--almost all--functions that generate interesting objects in
 R provide, as a part of their output, a copy of the call that ran the function.
 Inside our functions, we can inspect that call argument to see what the
 users ran, and from that we can often deduce what we need without requiring
 users to supply a lot of information.
 
\end_layout

\begin_layout Subsection
Return values versus attributes
\end_layout

\begin_layout Standard
If you review the R regression functions, like lm() or glm(), you notice
 that they build a list of things and pass it back to the user.
 That is the most obvious way to return information.
\end_layout

\begin_layout Standard
There is another way, however, which is a little interesting.
 One can mark an attribute on a return object.
 The R usage of attributes comes to mind at this point because, when we
 are scanning for inputs, we may need to inspect not only their values,
 but also their attributes.
 The most commonly inspected attribute, of course, is the class of the argument.
 So, if we are writing a package and we need to make sure that the objects
 being passed around among functions are correct for the purpose, then the
 object attributes are a very useful tool.
\end_layout

\begin_layout Standard
I don't want to write a lot about this so as to display my ignorance, but
 I would point user to 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
pctable()
\end_layout

\end_inset

 as an example of a rockchalk function that uses both a return list and
 attributes to get the job done.
 Maybe I'll fill in this example at a later time, once I figure out the
 most direct way to explain it.
\end_layout

\begin_layout Standard
The only problem I find with attributes is that the printing functions for
 R objects generally 
\begin_inset Quotes eld
\end_inset

splat out
\begin_inset Quotes erd
\end_inset

 the attributes when we don't want them.
 If one puts attributes on an object, then the default printer will show
 them.
 I've not learned all of the ins-and-outs of preventing that, but in 2015
 I became aware of it and learned that one way to deal with it can be found
 in the R source code for the method 
\begin_inset Flex Code
status collapsed

\begin_layout Plain Layout
print.table()
\end_layout

\end_inset

.
 
\end_layout

\begin_layout Section
Do This, Not That (Stub) 
\end_layout

\begin_layout Standard
R novices sometimes use Google to search for R advice and they find it,
 good or bad.
 They may find their way to the r-help email list, where advice is generally
 good, or to the StackOverflow pages for R, which may be better.
 A lot of advice is offered by people like me, who may have good intentions,
 but are simply not qualified to offer advice.
 
\end_layout

\begin_layout Standard
One of the few bits of advice that seems to grab widespread support is that
 
\begin_inset Quotes eld
\end_inset

for loops are bad.
\begin_inset Quotes erd
\end_inset

 One can write an lapply statement in one line, while a for loop can take
 3 lines.
 The code is shorter, but it won't necessarily run more quickly.
 I recall being jarred by this revelation in John Chambers's book, 
\emph on
Software for Data Analysis
\emph default
.
 The members of the apply family (apply, lapply, sapply, etc) can make for
 more readable code, but they aren't always faster.
 
\begin_inset Quotes eld
\end_inset

However, none of the apply mechanisms changes the number of times the supplied
 function is called, so serious improvements will be limited to iterating
 simple calculations many times.
 Otherwise, the n evaluations of the function can be expected to be the
 dominant fraction of the computation
\begin_inset Quotes erd
\end_inset


\begin_inset CommandInset citation
LatexCommand citep
after "213"
key "chambers_software_2008"

\end_inset

.
\end_layout

\begin_layout Standard
Todo: insert discussion of stackListItems-001.
\end_layout

\begin_layout Standard
Insert alternative methods of measuring executation time and measuring performan
ce
\end_layout

\begin_layout Standard
Balance time spent optimizing code versus time spent running program.
\end_layout

\begin_layout Section
Suggested Chores
\end_layout

\begin_layout Standard
I suggest the would be programmer should take on some basic challenges.
 
\end_layout

\begin_layout Enumerate
Try to create a generic function and several methods to handle classes of
 various types.
 If the term 
\begin_inset Quotes eld
\end_inset

generic function
\begin_inset Quotes erd
\end_inset

 and 
\begin_inset Quotes eld
\end_inset

method
\begin_inset Quotes erd
\end_inset

 cause disorientation, that means the reader is a user, not a programmer
 yet.
 
\end_layout

\begin_layout Enumerate
Consider developing a routine (or package) for statistical estimation or
 presentation.
 Find several R packages and compare the R code in them.
 Let me know if you agree with me about these points.
\end_layout

\begin_layout Itemize
Good code is compartmentalized.
 Separate pieces of work are handled by separated functions and results
 are returned in a well organized way.
\end_layout

\begin_layout Itemize
Functions should not be HUGE.
 
\end_layout

\begin_deeper
\begin_layout Standard
The components of a project should be small enough so that we can comprehend
 them.
 Generally speaking, if we are reading code and we come to a line that uses
 a variable that we cannot find on the screen, that's a problem.
 I used to correspond with a programmer at the University of Michigan who
 said he began to feel uncomfortable when a function filled up the entire
 terminal screen.
 
\end_layout

\end_deeper
\begin_layout Itemize
People who copy and paste sections over and over in order to handle slightly
 different cases are causing trouble for themselves and others.
 They should think harder on ways to separate that work into re-usable functions.
 
\end_layout

\begin_layout Standard
\begin_inset CommandInset bibtex
LatexCommand bibtex
bibfiles "rockchalk"
options "chicago"

\end_inset


\end_layout

\end_body
\end_document

%% LyX 2.3.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[11pt,letterpaper,english]{extarticle}
\usepackage{lmodern}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\setlength{\parskip}{\medskipamount}
\setlength{\parindent}{0pt}
\usepackage{babel}
\usepackage{float}
\usepackage[authoryear]{natbib}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=false,pdfborder={0 0 1},backref=false,colorlinks=false]
 {hyperref}

\makeatletter

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LyX specific LaTeX commands.
\special{papersize=\the\paperwidth,\the\paperheight}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
<<echo=F>>=
  if(exists(".orig.enc")) options(encoding = .orig.enc)
@
\providecommand*{\code}[1]{\texttt{#1}}

\@ifundefined{date}{}{\date{}}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
%\VignetteIndexEntry{outreg}

% Following comment is required. Do not delete it.
%\usepackage{Sweave}

\usepackage[includehead,includefoot]{geometry}
\geometry{
lmargin=1in,
rmargin=1in,
tmargin=0.75in,
bmargin=1.0in,
headheight=0pt,
headsep=0pt,
marginparwidth=0pt,
footskip=1.5\baselineskip,
}

\usepackage{booktabs}
\usepackage{dcolumn}

% paste in
%%\input{theme/guidePreambleHeader.tex}

% $ This nullifies emacs subscripting in following%
%%BEGIN: This begins the custom header
\usepackage{array}
\usepackage{xparse}
\usepackage{babel}
\usepackage{datetime}
\usepackage{xcolor}
\usepackage{colortbl}
\usepackage{graphicx}
\usepackage{multicol}

\definecolor{darkblue}{HTML}{1e2277}
\definecolor{kublue81186}{RGB}{0,81,186}
\newdateformat{monddyyyy}{\shortmonthname[\THEMONTH].~\THEDAY,~\THEYEAR}


%% BEGIN custom header
\ExplSyntaxOn
\NewDocumentCommand \guidesetup { m }
 {
  \keys_set:nn { guide } { #1 }
 }

\int_new:N \g_guide_authors_int
\prop_new:N \g_guide_authors_prop
\tl_new:N \l__guide_authors_tl

\keys_define:nn { guide }
 {
  author .code:n =
   {
    \int_gincr:N \g_guide_authors_int
    \keys_set:nn { guide / author } { #1 }
   },
  author/firstname .code:n = { \guide_add_author:nn { firstname } { #1 } },
  author/lastname .code:n = { \guide_add_author:nn { lastname } { #1 } },
  author/affiliation .code:n = { \guide_add_author:nn { affiliation } { #1 } },
  author/email .code:n = { \guide_add_author:nn { email } { #1 } },
  leftlogo .tl_set:N = \l_guide_leftlogo_tl,
  rightlogo .tl_set:N = \l_guide_rightlogo_tl,
  number .tl_set:N = \l_guide_number_tl,
  title .tl_set:N = \l_guide_title_tl,
  subtitle .tl_set:N = \l_guide_subtitle_tl,
  keywords .tl_set:N = \l_guide_keywords_tl,
  url .tl_set:N = \l_guide_url_tl,
  title .initial:n = {TITLE~NEEDED},
}
\NewDocumentCommand \guidehdr { +O{} }
 {
  \group_begin:
  \keys_set:nn { guide } { #1 }
  \guide_hdr:
  \group_end:
 }
\cs_new_protected:Nn \guide_hdr:
 {
   \centering
   \noindent\fcolorbox{kublue81186}{white}{
  \begin{tabular}
   {
    @{}c
    >{\centering}m{ \dim_eval:n { \textwidth - 2.5in - 6\tabcolsep - 2\arrayrulewidth } }
    c@{}
   }
   &&\tabularnewline[-\dim_eval:n { -\normalbaselineskip + 3ex }]
  \begin{tabular}{@{}>{\centering}p{1.25in}@{}}
  \includegraphics[width=.75in]{\l_guide_leftlogo_tl}
  \end{tabular}
  &
  \centering
    {\textsc{\large{}\l_guide_title_tl}}
    \tl_if_empty:NF \l_guide_subtitle_tl
    {
      {\\\textsc{\large{}\l_guide_subtitle_tl}}
    }
  \\
  \textcolor{kublue81186}{\rule[0.5ex]{1\linewidth}{0.7pt}}\\
  \guide_print_authors:
  &
  \begin{tabular}{@{}>{\centering}p{1.25in}@{}}
  \includegraphics[width=1.25in]{\l_guide_rightlogo_tl}
  \end{tabular}
  \tabularnewline
  \tl_if_empty:NTF \l_guide_number_tl
   { \leavevmode\vphantom{G}}
   {\small\bfseries { Guide~No: ~ \tl_use:N \l_guide_number_tl }} & 
   \tl_if_empty:NF \l_guide_keywords_tl
   {\centering
    \textbf{Keywords}: ~ \l_guide_keywords_tl}
   & 
   \small\bfseries\monddyyyy\today \tabularnewline
   \tl_if_empty:NF \l_guide_url_tl
   {
   \multicolumn{3}{c}{
    See ~ \url{\l_guide_url_tl} ~ for ~ updates.}
     }
  \tabularnewline
  \end{tabular}
  }
}

\cs_new_protected:Nn \guide_add_author:nn
 {
  \prop_gput:Nxn \g_guide_authors_prop
   { #1 \int_to_arabic:n { \g_guide_authors_int } }
   { #2 }
 }
\cs_generate_variant:Nn \prop_gput:Nnn { Nx }
\cs_new_protected:Nn \guide_print_authors:
 {
  \tl_clear:N \l__guide_authors_tl
  \int_step_inline:nnnn { 1 } { 1 } { \g_guide_authors_int }
   {
    \tl_put_right:Nn \l__guide_authors_tl
     {
      \textbf
       {
        \prop_item:Nn \g_guide_authors_prop { firstname ##1 }~
        \prop_item:Nn \g_guide_authors_prop { lastname ##1 },~
       }
       \prop_item:Nn \g_guide_authors_prop { affiliation##1 },~
       \href{mailto:\prop_item:Nn \g_guide_authors_prop { email##1 } }
       {\url{\prop_item:Nn \g_guide_authors_prop { email##1 }}}
       \\
     }
   }
   \tl_use:N \l__guide_authors_tl
 }
\ExplSyntaxOff
%%END This ends the custom header


%%Now the hyperref colors

\hypersetup{
 unicode=true,pdfusetitle,bookmarks=true,
 bookmarksnumbered=false, bookmarksopen=false,
 breaklinks=true, pdfborder={0 0 0},
 pdfborderstyle={}, backref=false,
 colorlinks=true, linkcolor=darkblue,
 urlcolor=darkblue, citecolor=darkblue}

%% Was previously in "preambleFloatCenter.tex"
%%BEGIN: graphics and such inside floats are centered by default
\renewenvironment{figure}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{figure}
 }{%
   \@float{figure}[#1]%
 }%
 \centering
}{%
 \end@float
}
\renewenvironment{table}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{table}
 }{%
   \@float{table}[#1]%
 }%

 \centering
}{%
 \end@float
}
%%END: graphics and such inside floats are centered by default

% paste
%%\input{theme/preambleFooter.tex}

\usepackage{fancyhdr}
\usepackage{lastpage}
\usepackage{xparse}

\ExplSyntaxOn
\NewDocumentCommand \footersetup { +m }
{
\keys_set:nn { footer } { #1 }
}
\keys_define:nn { footer }
{	
    leftside .code:n = {
\keys_set:nn { footer / leftside } { #1 }
},
rightside .code:n = {
\keys_set:nn { footer / rightside } { #1 }
},
    leftside / lone .tl_set:N = \l_footer_leftside_lone_tl,
    leftside / ltwo .tl_set:N = \l_footer_leftside_ltwo_tl,
    leftside / lthree .tl_set:N = \l_footer_leftside_lthree_tl,
    rightside / rone .tl_set:N = \l_footer_rightside_rone_tl,
    rightside / rtwo .tl_set:N = \l_footer_rightside_rtwo_tl,
    rightside / rthree .tl_set:N = \l_footer_rightside_rthree_tl,
    leftside .initial:n = {lone = , ltwo = ~, lthree = ~ },
    rightside .initial:n = {rone = ~, rtwo = ~,  rthree = ~},
    leftside .default:n = {lone = , ltwo = ~, lthree = ~ },
    rightside .default:n = {rone = ~, rtwo = ~,  rthree = ~},
}
\NewDocumentCommand \guideftr { +o }
{
  \IfValueT { #1 } 
  {
    \keys_set:nn { footer } { #1 }
  }
  \guide_ftr:
}
\cs_new_protected:Nn \guide_ftr:
{
  % Now fancy header/footer settings
  \lhead{}%
  \chead{}%
  \rhead{}%
  \renewcommand{\headrulewidth}{0pt}%
  \lfoot{}%
  \cfoot{{Page} ~ \thepage {} ~ { of } ~ {} \pageref{LastPage}}%
  \rfoot{}%

  \fancypagestyle{firststyle}%
  {%
    \lhead{}%
    \chead{}%
    \rhead{}%
    \lfoot{}%
    \cfoot{}%
    \rfoot{}%
    \renewcommand{\headrulewidth}{0pt}%
    \fancyfoot[L]{\l_footer_leftside_lone_tl {}\\%
      \l_footer_leftside_ltwo_tl {}\\%
      \l_footer_leftside_lthree_tl {}}%
    \fancyfoot[R]{\l_footer_rightside_rone_tl {}\\%
      \l_footer_rightside_rtwo_tl {}\\%
      \l_footer_rightside_rthree_tl {}}%
      \renewcommand{\footrulewidth}{.2pt}%
  }%
  \thispagestyle{firststyle}
}
\ExplSyntaxOff

\pagestyle{fancy}
%% End of fancy footer setup

%% Now the addressFooter file
\urlstyle{tt}
\footersetup{
  leftside={
    lone={CRMDA{,}~1425~Jayhawk~BLVD}, 
    ltwo={University~of~Kansas{,}~Waston~Library~Suite~470}, 
    lthree={Lawrence{,}~KS~66045}},
  rightside={
    rone=Web: \url{https://crmda.ku.edu},
    rtwo=Email: \href{mailto:pauljohn@ku.edu}{\url{pauljohn@ku.edu}},
    rthree=Phone: 785-864-3353}
}
\guideftr




% paste from
%%\input{theme/guidePreambleSweavel.tex}
%%% From beamer slide:
%% \usepackage{Sweave}
%% 
%% This controls display of code chunks
\usepackage{ae,fancyvrb,relsize,listings}

\providecommand{\Sweavesize}{\normalsize}
\providecommand{\Rsize}{}
\renewcommand{\Rsize}{\normalsize}
\providecommand{\Routsize}{\scriptsize}

\providecommand{\Rcolor}{\color[rgb]{0.1, 0.1, 0.1}}
\providecommand{\Routcolor}{\color[rgb]{0.2, 0.2, 0.2}}
\providecommand{\Rcommentcolor}{\color[rgb]{0.101, 0.43, 0.432}}

\providecommand{\Rbackground}{\color[gray]{0.91}}
\providecommand{\Routbackground}{\color[gray]{0.935}}
% Can specify \color[gray]{1} for white background or just \color{white}

\lstdefinestyle{Rinput}{
  language=R,
  escapechar=`,
  fancyvrb=false,%
  tabsize=2,%
  breaklines=true,
  breakatwhitespace=true,%
  captionpos=b,%
  frame=single,%
  framerule=0.2pt,%
  framesep=1pt,%
  showstringspaces=false,%
  basicstyle=\Rsize\Rcolor\ttfamily,%
  columns=fixed%,
  \lst@ifdisplaystyle\scriptsize\fi,%,
  commentstyle=\Rcommentcolor\ttfamily,%
  identifierstyle=,%
  keywords=\bfseries,%
  keywordstyle=\color[rgb]{0, 0.5, 0.5},
  escapeinside={(*}{*)},
  literate={~}{{$\sim$}}1{==}{{=\,=}}2{--}{{-\,-}}2,
  alsoother={$},
  alsoletter={.<-},%
  otherkeywords={!,!=,~,$$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},%
  backgroundcolor=\Rbackground,%
  numbers=left,%
  %numberblanklines=false,%
  stepnumber=5,
  firstnumber=1,
  numberstyle={\tiny}
}%

% Other options of interest:
% frame=single,framerule=0.1pt,framesep=1pt,rulecolor=\color{blue},
% numbers=left,numberstyle=\tiny,stepnumber=1,numbersep=7pt,
% keywordstyle={\bf\Rcolor}

\lstdefinestyle{Routput}{fancyvrb=false,
  literate={~}{{$\sim$}}1{R^2}{{$R^{2}$}}2{^}{{$^{\scriptstyle\wedge}$}}1{R-squared}{{$R^{2}$}}2,%
  basicstyle=\Routcolor\Routsize\ttfamily,%
  backgroundcolor=\Routbackground,
  language=R,
  escapechar=`,
  fancyvrb=false,%
  tabsize=2,%
  breaklines=true,
  breakatwhitespace=true,%
  captionpos=b,%
  frame=single,%
  framerule=0.2pt,%
  framesep=1pt,%
  showstringspaces=false,%
  columns=fixed%,
  \lst@ifdisplaystyle\scriptsize\fi,%
  identifierstyle=,%
  keywords=\bfseries,%
  keywordstyle=\color[rgb]{0, 0.5, 0.5},
  escapeinside={(*}{*)},
  literate={~}{{$\sim$}}1 {==}{{=\,=}}2,
  alsoother={$},
  alsoletter={.<-},%
  otherkeywords={!,!=,~,$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},
  numbers=left,
  %numberblanklines=false,%
  stepnumber=5,
  firstnumber=1,
  numberstyle={\tiny}
}

\newenvironment{Schunk}{}{}
\newenvironment{Sinput}{}{}
\let\Sinput\relax
\let\Scode\relax
\let\Soutput\relax
\lstnewenvironment{Sinput}{\lstset{style=Rinput}}{}
\lstnewenvironment{Scode}{\lstset{style=Rinput}}{}
\lstnewenvironment{Soutput}{\lstset{style=Routput}}{}
%%end paste in from guidePreambleSweavel.tex


\lstset{tabsize=2, breaklines=true, style=Rinput, breakatwhitespace=true}

\fvset{listparameters={\setlength{\topsep}{0em}}}

\usepackage{xcolor}
\definecolor{light-gray}{gray}{0.90}
\usepackage{realboxes}
\providecommand*{\code}[1]{\texttt{#1}}
\renewcommand{\code}[1]{%
\Colorbox{light-gray}{#1}%
}%


\usepackage{siunitx}
%following now in each table
\sisetup{
  input-symbols = ( )
}
%,
%  group-minimum-digits = 9,
%  table-number-alignment = center,
%  table-space-text-pre = (,
%  table-align-text-pre = false,
%  table-align-text-post = false,
%  table-space-text-post = {***},
%  parse-units = false
%}

\makeatother

\usepackage{listings}
\renewcommand{\lstlistingname}{\inputencoding{latin9}Listing}

\begin{document}
%% Fill in values of the arguments here, 
%% If blanks are needed, must insert value " ~ "
%% If comma needed inside value, wrap in {}.
%% Delete secondauthor and thirdauthor if not needed
\guidesetup{%
  author={
    lastname=Johson, 
    firstname=Paul, 
    affiliation=CRMDA,
    email=pauljohn@ku.edu},
  url={https://crmda.ku.edu/guides},
  keywords={R, rockchalk, outreg, LaTeX, regression tables},
  title={Regression Table Survey with outreg},
  leftlogo={theme/logoleft.pdf},
  rightlogo={theme/logo-vert.pdf},
  number=49,
}
\guidehdr

% tmpout directory must exist first
<<tmpout, echo=FALSE, include=FALSE, results=hide>>=
if(!dir.exists("tmpout")) dir.create("tmpout", showWarnings=FALSE)
@

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\SweaveOpts{prefix.string=tmpout/t, split=F, ae=FALSE, height=3, width=5}

<<Roptions, echo=FALSE, include=FALSE, results=hide>>=
opts.orig <- options()
options(width=100, prompt=" ", continue="  ")
options(useFancyQuotes = FALSE) 
set.seed(12345)
options(SweaveHooks=list(fig=function() par(ps=10)))
pdf.options(onefile=FALSE,family="Times",pointsize=10)
@

<<echo=FALSE,results=hide>>=
fn1 <- "theme/logoleft.pdf"
fn2 <- "theme/logo-vert.pdf"
if(!file.exists("theme")) dir.create("theme")

blankpdf <- function(file, height=3, width=3, pointsize=20, replace = FALSE){
    if(file.exists(file) && !replace) return(TRUE)
    pdf(file=file, width=width, height=height, paper="special", 
        onefile=FALSE, pointsize=pointsize)
    par(mar=c(1,1,1,1))
    plot(1:2, 1:2, type = "n", axes=FALSE, xlab="", ylab="")
    ##text(1.5, 1.5, "left\n logo", axes=FALSE) 
    ##box(which="plot")
    dev.off()
    if(file.exists(file)) TRUE else FALSE
}
blankpdf(fn1)
blankpdf(fn2)
@

%The listings class used here allows within-document style
%changes. R input boxes are governed by 
% "Rsize", "Rbackground" and "Rcolor", while R output boxes depend on
% "Routsize", "Routbackground", and "Routcolor". Colors
% can be specified in many ways, as shown here
%\def\Rsize{\huge\ttfamily}
%\def\Routsize{\huge}
%\def\Rbackground{\color[gray]{0.90}}
%\def\Routbackground{\color[gray]{0.40}}
%\def\Rcolor{\color[gray]{0.60}
%\def\Routcolor{\color[rgb]{0.9, 0.1, 0.1}]}
%\def\Rcommentcolor{\color{green}}
\begin{abstract}
\noindent This is a comprehensive demonstration of the \code{outreg()}
function in \code{rockchalk}. This vignette demonstrates the ability
to integrate model estimates, to assign labels for models and variables,
and to customize the selection of summary statistics. A major innovation
in recent versions is the introduction of the ability to generate
``decimal-centered'' columns. This has been the most widely requested
feature for a while.
\end{abstract}
\tableofcontents{}

The \code{rockchalk} package for R \citep{RCore} was created to
facilitate authors in stats classes who need to make presentable tables
and graphs for regression models. It does not aspire to be a comprehensive
table-maker, mainly because R package writers are not consistent in
their methodology. Procedures called ``regression'' within R's base
have a certain internal consistency, but user-contributed packages
do not. The \code{outreg} function works for regression models that
I use regularly, usually in the base of R, like \code{lm}, \code{glm},
and for \code{lme4} objects. Other regression packages may work.
I've tried to be somewhat general in the design.

The major new feature is ``decimal-centered'' columns. In the implementation
of centering, we tested approaches based on two LaTeX packages, \code{dcolumn}
and \code{siunitx}. \code{dcolumn} output it is mostly adequate,
but it leaves problems in column headers and non-numeric content alignment.
\code{siunitx} is a much more massive framework. There is much more
danger of outright failure because of the immensity of \code{siunitx}
itself. However, at the current time, all of the examples we have
tested indicate that the \code{siunitx}-based tables are superior.
there are hopeful signs it can deal with some very tough examples.
See Table \ref{tab:ex16s} I have a solution using \code{siunitx}
for a troublesome table that failed badly with \code{dcolumn} (Table
\ref{tab:ex16d}). The user can designate which method should be used
by assigning the parameter \code{centering = \textquotedbl dcolumn\textquotedbl}
or \code{centering = \textquotedbl siunitx\textquotedbl}. NOTE: it
is necessary for the user to insert either \code{\textbackslash usepackage\{dcolumn\}}
or \code{\textbackslash usepackage\{siunitx\}} in the preamble! The
aim is for this whole thing to ``just work'' without any more attention
than that. Please let me know if there is trouble.

\section{Briefly: outreg()}

First, run regression models. Second, give the fitted regression object(s)
to \code{outreg}. When \code{outreg} runs, it will have 2 effects.
First, it will write out LaTeX code to the screen (unless \code{print.results=FALSE}).
A user might ``copy/paste'' that code into a LaTeX document. (Or,
write it in a file and import it in the future. (Or, as in the case
of this Sweaved document, the output goes directly into the result
file.)

\section{Briefly: table vs tabular}

This is the most confusing thing for LaTeX beginners. \emph{Put bluntly,
a ``table'' is not a table}. 

Terminology
\begin{description}
\item [{tabular}] In LaTeX, a ``\code{tablular}'' object is a grid,
a display like a ``spreadsheet''. You'd call that a ``table'',
I believe.
\item [{table}] In LaTeX, a ``\code{table}'' is something else. It is
a document ``subsection'' that ``floats'' around in the document.
It is a container. A tabular is placed inside one of these table containers.
Table objects are numbered, can be used in cross references (if they
have labels)
\end{description}
If we are writing LaTeX code by hand, a tabular inside a table will
look like 

\inputencoding{latin9}\begin{lstlisting}
\begin{table}
    \caption{The title of the floating object is specified in caption}
    \label{tab:ex1}% cross referencing use \ref{tab:ex1}
\begin{tabular}
  ...code to create tablar defined here
\end{tabular}
\end{table}
\end{lstlisting}
\inputencoding{utf8}
The big question is ``should my outreg function write the tabular
only, or also the table container''?

I used to think the only reasonable answer was ``tabular only'',
but now I see reasons why we might want to do both. So it is an option,
as explained next.

\subsection*{rockchalk lets you choose, float or no float}

The rockchalk outreg function has a parameter, \code{float=FALSE}
or \code{float=TRUE}, to determine whether the output should include
the floating table part along with the tabular object. If outreg users
specify the argument title, then they are implicitly setting \code{float=TRUE}
and the value for title is used as the LaTeX caption. 

The default for \code{float} is \code{FALSE}, because I have usually
preferred to control the floating table from my document. However,
I recognize that many people who are better at LaTeX than I am disagree,
they want to write the table and tabular structures into one file. 

\subsection*{Why would a user prefer float=TRUE?}

If you are not using LyX, then it may be convenient to let float=TRUE
so the floating table can be handled from the outreg function. outreg
allows the author to specify the caption and the label. However, I
did not allow for any more fine-grained customizations, especially
table placement details.

\subsection*{Why I don't generally use float=TRUE}

LaTeX has options to control the placement of floating table structures
that the rockchalk outreg function does not adjust. I think that if
the LaTeX author wants to adjust those things, it is easier for the
author to control them in the document itself, rather than the outreg
function that writes a tabular object. 

In LyX, the user interface has a pull down menu to create floating
objects and the GUI includes a feature to set the title (sorry, the
'caption'), and it also includes a way to set a label. If I use this
method, then LyX is aware of this thing and the LyX cross-referencing
system is available. 

\subsection*{Another reason to use float=FALSE}

In an instructional document using Sweave, the code chunks will print
out where they ``are'' in the input file. 

In LyX, there are some cross-referencing tools. It is more desirable
to create the table container in LyX, so then cross-references work
correctly. If one creates a table (floating container), and then puts
the tabular-creating code chunk inside, then the code will print out
with the table. And cross references succeed using the LyX pull down
menus. That is demonstrated in Tables \ref{tab:ex11}, \ref{tab:ex11t},
\ref{tab:ex8}, \ref{tab:ex17}.

\section{Torture test of ``rockchalk'' outreg function}

In most of these examples, I've taken the one-step ``float=TRUE''
option, but I have some examples where I've manually created the LaTeX
float and placed an outreg tabular inside it.

<<>>=
set.seed(2134234)
dat <- data.frame(x1 = rnorm(100), x2 = rnorm(100))
dat$y1 <- 30 + 5 * rnorm(100) + 3 * dat$x1 + 4 * dat$x2
dat$y2 <- rnorm(100) + 5 * dat$x2
m1 <- lm(y1 ~ x1, data = dat)
m2 <- lm(y1 ~ x2, data = dat)
m3 <- lm(y1 ~ x1 + x2, data = dat)
gm1 <- glm(y1 ~ x1, family = Gamma, data = dat)
@

\subsection{tight and wide styles for regression tables}

In my terminology, a tight table has one ``narrow'' column for a
regression (parameter and standard error stacked on top of each other).
A table that is not in the tight format is wide, it has 2 columns
for each table (parameter and standard error side by side).

A tight table is the default, it seems to be what many political science,
sociology, and economics students will need. See Table \ref{tab:ex1}.
A wide format table is, in my opinion, more pleasant for the eyes
and seems to be more popular in psychology. See Table \ref{tab:ex1w}. 

<<ex1, echo=F, results=tex>>=
library(rockchalk)
vl <- c("(Intercept)" = "Intercept")
ex1 <- outreg(m1, title = "My One Tightly Printed Regression (uncentered)",
                    label = "tab:ex1",
                    float = TRUE, print.results = FALSE, varLabels=vl)
# cat that, don't print it
cat(ex1)
@

<<ex1w,results=tex>>=
library(rockchalk)
ex1w <- outreg(m1, title = "My Wide Format \"side-by-side\" columns (uncentered)", label = "tab:ex1w", tight = FALSE, float = TRUE, print.results = FALSE)
cat(ex1w)
@

\subsection*{Decimal-centered test case 1}

A decimal-centered version of the tight column format can be seen
in Table \ref{tab:ex2d} and the decimal-centered version of the wide
format is seen in Table \ref{tab:ex2wd}

<<ex1d, results=tex>>=
library(rockchalk)
ex2d <- outreg(m1, title = 'Tight column with centering = "dcolumn"', label = "tab:ex2d", centering = "dcolumn", float = TRUE, print.results=FALSE)
cat(ex2d)
@

<<ex1s, results=tex>>=
library(rockchalk)
ex2s <- outreg(m1, title = 'Tight column with centering = "siunitx"', label = "tab:ex2s", centering = "siunitx", float = TRUE)
@

<<ex2wd, results=tex>>=
library(rockchalk)
ex1wd <- outreg(m1, title = 'Wide (not tight) format with centering = "dcolumn"',  label = "tab:ex2wd", tight = FALSE, centering = "dcolumn",  float = TRUE, print.results = FALSE)
cat(ex1wd)
@

<<ex2ws, results=tex>>=
ex1ws <- outreg(m1, title = 'Wide (not tight) format with centering = "siunitx"',  label = "tab:ex2ws", tight = FALSE, centering = "siunitx", float = TRUE, print.results = FALSE)
cat(ex1ws)
@

\subsection*{alpha level adjustments}

In Table \ref{tab:ex2p}, I demonstrate that the alpha parameter can
be used to select different values for the critical region. In Tables
\ref{tab:ex2pd} and \ref{tab:ex2ps}, the centered-with-dcolumn and
centered-with-siunitx are presented. I see some trouble here because
the stars are not evenly spaced in the dcolumn-based Table {\ref{tab:ex2pd}}.
However, the siunitx based Table {\ref{tab:ex2ps}} may be adequate.
This is one of the reasons I now \emph{strongly} lean toward use of
\code{siunitx} if centering is desired. Several alpha adjustments
are scattered about in the examples in this essay, just to make sure
they work.

<<results=tex>>=
ex2p <- outreg(list("Fingers" = m1), tight = FALSE, title = "Ability to change p values (not centered)", label = "tab:ex2p",  float = TRUE, alpha = c(0.1, 0.05, 0.01)) 
@

<<results=tex>>=
ex2pd <- outreg(list("Fingers" = m1), tight = FALSE,  title = "Ability to change p values (dcolumn)", label = "tab:ex2pd", centering = "dcolumn", float = TRUE, alpha = c(0.1, 0.05, 0.01)) 
@

<<results=tex>>=
ex2ps <- outreg(list("Fingers" = m1), tight = FALSE,  title = "Ability to change p values (siunitx)", label = "tab:ex2ps", centering = "siunitx", float = TRUE, alpha = c(0.1, 0.05, 0.01)) 
@

\subsection*{Several models in same table}

One of the most valuable features of rockchalk is that one can align
several models side by side and compare them. The rows are matched
by the variable name.

About model names: If a list of regression fits is not named, then
the names will be bland, ``M1'', ``M2''. 

It is highly recommended that authors should name the regression models. 

Currently, I recommend that we provide the model names \textbf{in}
the list that provides the fitted regressions, as you see here. That
produces Table \ref{tab:ex3}.

<<results=tex>>=
ex3 <- outreg(list("Model A" = m1, "Model B has a longer heading" = m2), varLabels = list(x1 = "Billie"),   title = "My Two Linear Regressions (uncentered)", label = "tab:ex3", request = c(fstatistic = "F"),  print.results = FALSE)
cat(ex3)
@

In the original rockchalk, I had it differently. It is possible to
provide model names in a separate argument, \code{modelLabels}. That
is shown in the code below, and in output Table \ref{tab:ex3b}. The
modelLabels parameter takes precedence, it will replace the labels
in the first argument. However, I found it confusing to write tables
that way, so I made it work the other way too.

<<results=tex>>=
ex3b <- outreg(list("Model A" = m1, "Model B" = m2),   modelLabels = c("Overrides ModelA", "Overrides ModelB"), varLabels = list(x1 = "Billie"), title = "Note modelLabels Overrides model names (uncentered)",  label = "tab:ex3b"
)
@

The column-centered versions are in Table \ref{tab:ex3bd} and Table
\ref{tab:ex3bs}.

<<results=tex>>=
ex3bd <- outreg(list("Model A" = m1, "Model B" = m2), modelLabels = c("Overrides ModelA", "Overrides ModelB"), varLabels = list(x1 = "Billie"), title = "Note modelLabels Overrides model names (dcolumn)", label = "tab:ex3bd", centering = "dcolumn")
@

<<results=tex>>=
ex3bs <- outreg(list("Model A" = m1, "Model B" = m2), modelLabels = c("Overrides ModelA", "Overrides ModelB"),  varLabels = list(x1 = "Billie"), title = "Note modelLabels Overrides model names (siunitx)", label = "tab:ex3bs", centering = "siunitx")
@

\subsection*{Specifying additional summary information}

In the first version, I was thinking that everybody would be happy
enough if the table included N, the standard error of the regression
(which I dubbed RMSE in the old SAS style), and R-square. There were
requests for other summaries. 

At first, I was thinking I needed to add arguments for each request.
I created argument ``\code{showAIC=TRUE}'' to request Akaike's
information criterion. For example, see Table \ref{tab:ex5d} produced
by the following

<<ex5d, results=tex>>=
ex5d <- outreg(list("Whichever" = m1, "Whatever" = m2), title = "Still have showAIC argument (uncentered)", label = "tab:ex5d", showAIC = TRUE, float = TRUE)
@

The column-centered versions are in Table \ref{tab:ex5dd} and \ref{tab:ex5ds}.

<<ex5dd, results=tex>>=
ex5dd <- outreg(list("Whichever" = m1, "Whatever" = m2),  title = "Still have showAIC argument (dcolumn)", label = "tab:ex5dd", showAIC = TRUE, float = TRUE, centering = "dcolumn")
@

<<ex5ds, results=tex>>=
ex5ds <- outreg(list("Whichever" = m1, "Whatever" = m2), title = "Still have showAIC argument (siunitx)", label = "tab:ex5ds", showAIC = TRUE, float = TRUE, centering = "siunitx")
@

However, I anticipated that way might lead me down a bad path of writing
a parameter for every possible summary statistic.

My first idea was to create a recipe book of particular summary items
and make them available for requests by users. For example, I wrote
a customized reporter for F statistics and a parameter to ask for
that was called ``request''. For example, \code{request = c(fstatistic = \textquotedbl F\textquotedbl )}
asks for my special fstatistic and the label to be used for it in
the table would be ``F''. I ended up not making very many of those
fancy object, but in the semTable in the kutils package there is a
similar approach for the model Chi-Square and now it seems to me I
should come back and do more like that in outreg.

Because I expected that I would never have time to keep up with requests
for specialized summary items, I created a ``back door'' though
which users could use functions and include them in the summary. 

However, I shot myself in the foot by creating that general purpose
ability with a horrible, ungainly name for the function's argument,
\code{runFuns}. I admit that is a horrible name, but I had good intentions.
``runFuns'' is short for ``run functions''. It will run a function
named as the first argument and then label it with the right hand
side argument. So, if one has a lot of request like AIC, BIC, and
so forth, the R functions can be used without too much effort. 

<<ex6d,results=tex>>=
ex6d <- outreg(list("Whatever" = m1, "Whatever" =m2), title = "Another way to get AIC output", label="ex6d",  runFuns = c("AIC" = "Akaike IC"), centering = "dcolumn", print.results=FALSE)
cat(ex6d)
@

\subsection*{Insert more regressions in one table}

This code produces Table \ref{tab:ex7}, which is NOT decimal aligned

<<results=tex>>=
ex7 <- outreg(list("Amod" = m1, "Bmod" = m2, "Gmod" = m3), title = "My Three Linear Regressions", label="tab:ex7")
@

The column-aligned version of the same is found in Table \ref{tab:ex7d},
produced by the following code.

<<results=tex>>=
ex7d <- outreg(list("Amod" = m1, "Bmod" = m2, "Gmod" = m3), centering = "dcolumn", title = "My Three Linear Regressions (decimal aligned)", label="tab:ex7d")
@

I worried that users who are verbose might break the function. In
Table \ref{tab:ex11}, I show that some very long names, even ones
with periods, do not seem to cause horrible trouble. They WILL run
off the edge of the page if they get much longer. Author will need
to change to the tight columns if that is an issue, as seen in Table
\ref{tab:ex11t}.

\begin{table}[H]
\caption{Stress test very long titles (uncentered) \label{tab:ex11}}

<<results=tex>>=
ex11 <- outreg(list("I Love Long Titles" = m1, "Prefer Brevity" = m2, "Captain. Kirk. Named. This." = m3), tight = FALSE, float = FALSE, centering = "dcolumn")
@
\end{table}
\begin{table}[H]
\caption{Stress test very long titles (dcolumn) \label{tab:ex11t}}

<<results=tex>>=
ex11td <- outreg(list("I Love Long Titles" = m1, "Prefer Brevity" = m2, "Captain. Kirk. Named. This" = m3), float = FALSE,  centering = "dcolumn")
@
\end{table}
\begin{table}[H]
\caption{Stress test very long titles (dcolumn) \label{tab:ex11ts}}

<<results=tex>>=
ex11ts <- outreg(list("I Love Long Titles" = m1, "Prefer Brevity" = m2, "Captain. Kirk. Named. This" = m3), float = FALSE, centering = "siunitx")
@
\end{table}

\subsection*{Alternative standard errors}

The original rockchalk took the standard errors from the fitted model.
A student in Brazil wrote and asked me to make it possible for the
author to supply ``robust'' standard errors. That was a good idea.
The code to demonstrate how to create an alternative vector of standard
errors (in that case, Huber-White robust standard errors) will appear
with the output in Table \ref{tab:ex8}. The code is displayed there
because I created the outreg table with float=FALSE, and then I manually
created a table container into which I typed the R code chunk.

\begin{table}[H]
\caption{Robust Standard Errors (uncentered)\label{tab:ex8}}

<<results=tex>>=
if (require(car)){
   newSE <- sqrt(diag(car::hccm(m3)))
   ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, 
             "Model C w Robust SE" = m3),
             SElist= list("Model C w Robust SE" = newSE))
}
@
\end{table}
\begin{table}[H]
\caption{Robust Standard Errors (dcolumn)\label{tab:ex8d}}

<<results=tex>>=
if (require(car)){
   newSE <- sqrt(diag(car::hccm(m3)))
   ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, 
             "Model C w Robust SE" = m3),
             SElist= list("Model C w Robust SE" = newSE),
             centering = "dcolumn")
}
@
\end{table}
\begin{table}[H]
\caption{Robust Standard Errors (siunitx)\label{tab:ex8s}}

<<results=tex>>=
if (require(car)){
   newSE <- sqrt(diag(car::hccm(m3)))
   ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, 
             "Model C w Robust SE" = m3),
             SElist= list("Model C w Robust SE" = newSE),
             centering = "siunitx")
}
@
\end{table}

\subsection*{Aligning different kinds of fits}

The output from lm and glm fits may sometimes be usefully compared.
The parameter display from rockchalk will align same-named variables.
The difference in the available summary statistics is apparent because
the rows do not ``line up''. See Table \ref{tab:ex13} for the output
from the following.

<<results=tex>>=
ex13 <- outreg(list("OLS" = m1, "GLM" = gm1), float = TRUE,
               title = "OLS and Logit in same table (dcolumn)", 
               label="tab:ex13", alpha = c(0.05, 0.01), centering = "dcolumn")
@

As seen in Table \ref{tab:ex14}, we check if the supplemental parameter
requests hold up with dcolumn

<<results=tex>>=
ex14 <- outreg(list(OLS = m1, GLM = gm1), float = TRUE,
         title = "OLS and Logit with summary report features (dcolumn)",
         label = "tab:ex14",
         request = c(fstatistic = "F"), runFuns = c("BIC" = "BIC"),
         centering = "dcolumn")
@

What if the number of digits is dialed up and alpha is altered? See
Table \ref{tab:ex15}. 

<<results=tex>>=
ex15 <- outreg(list(OLS = m1, GLM = gm1), float = TRUE,
         title="OLS and GLM with more digits (digits)", 
         label="tab:ex15", 
         request = c(fstatistic = "F"), runFuns = c("BIC" = "BIC"),
         digits = 5, alpha = c(0.01), centering = "dcolumn")
@

In Table \ref{tab:ex16d}, output shows result when several runFuns
are requested. Again, I'm very sorry that parameter name is so awful.
I adjusted the alpha stars as well.

<<results=tex>>=
ex16d <- outreg(list("OLS 1" = m1, "OLS 2" = m2,  GLM = gm1), float = TRUE,
           title = "2 OLS and 1 Logit (dcolumn), additional runFuns", 
           label="tab:ex16d",
           request = c(fstatistic = "F"),
           runFuns = c("BIC" = "BIC", "logLik" = "ll"),
           digits = 5, alpha = c(0.1, 0.05, 0.01), centering = "dcolumn")
@

<<results=tex>>=
ex16s <- outreg(list("OLS 1" = m1, "OLS 2" = m2,  GLM = gm1), float = TRUE,
           title = "2 OLS and 1 Logit (siunitx), additional runFuns", 
           label="tab:ex16s",
           request = c(fstatistic = "F"),
           runFuns = c("BIC" = "BIC", "logLik" = "ll"),
           digits = 5, alpha = c(0.1, 0.05, 0.01), centering = "siunitx")
@

After a while, I noticed that the left hand side of runFuns need not
be in quotation marks. So my examples are not always consistent. For
example, in Table \ref{tab:ex17} I have some in quotes, some not.
And I also show that if authors want to create redundant rows, they
are allowed to do so (N, for example).

\begin{table}[H]
\caption{Additional test on summary stats (dcolumn)\label{tab:ex17}}

<<results=tex>>=
ex17 <- outreg(list("Model A" = gm1, "Model B label with Spaces" = m2),
         request = c(fstatistic = "F"),
         runFuns = c("BIC" = "Schwarz IC", "AIC" = "Akaike IC", "logLik" = "ll",
         "nobs" = "N Again?"), centering = "dcolumn")
@
\end{table}
\begin{table}[H]
\caption{Additional test on summary stats (siunitx)\label{tab:ex17s}}

<<results=tex>>=
ex17s <- outreg(list("Model A" = gm1, "Model B label with Spaces" = m2),
         request = c(fstatistic = "F"),
         runFuns = c("BIC" = "Schwarz IC", "AIC" = "Akaike IC", "logLik" = "ll",
         "nobs" = "N Again?"), centering = "siunitx")
@
\end{table}

\section{About examples here and your long-run work flow}

In this document, I'm using a lot of short cuts to make it easier
to produce examples with and without centering. I'm using the \code{float=TRUE}
option (or, equivalently, giving a title) often here to cut down on
the amount of work I'm doing. 

If you are learning how to make tables, I usually suggest doing it
in a different way. I would suggest instead that authors should create
an outreg output file, with a sequence like this

\inputencoding{latin9}\begin{lstlisting}
lm1 <- lm(... regression commands ...)
lm1.out <- outreg(lm1, print.results=FALSE, ...parameters like float=FALSE, tight = FALSE ...)
cat(lm1.out, file = "lm1.out.tex")
\end{lstlisting}
\inputencoding{utf8}
I'd usually have an output directory where the tex file would be placed,
but aside from this detail, that's my workflow. Then, when I want
to use that file in my document, I use the LaTeX code ``\textbackslash input\{lm1.out.tex\}''. 

One reason for using that 2-step workflow is that the \emph{automatically
produced outreg table may not be exactly perfect}. Perhaps a variable
label does not look right. Personally speaking, I wish the automatic
table were always perfect. Practically, I accept it is not. So I leave
open the opportunity that it might need to be revised by hand. 

\pagebreak{}

\bibliographystyle{apalike2}
\phantomsection\addcontentsline{toc}{section}{\refname}\bibliography{rockchalk}

\pagebreak{}

\section*{Replication Information\label{sec:Session-Info}}

<<session, echo=F>>=
sessionInfo()
if(!is.null(warnings())){
    print("Warnings:")
    warnings()}
@

<<opts20, include=F>>=
## Don't delete this. It puts the interactive session options
## back the way they were. If this is compiled within a session
## it is vital to do this.
options(opts.orig)
@
\end{document}

%% LyX 2.3.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[american,noae]{scrartcl}
\usepackage{lmodern}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{cmtt}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage{geometry}
\geometry{verbose,tmargin=1in,bmargin=1in,lmargin=1in,rmargin=1in}
\setlength{\parskip}{\smallskipamount}
\setlength{\parindent}{0pt}
\usepackage{color}
\usepackage{babel}
\usepackage{url}
\usepackage{enumitem}
\usepackage[authoryear]{natbib}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=true,pdfborder={0 0 0},pdfborderstyle={},backref=section,colorlinks=true]
 {hyperref}
\hypersetup{
 colorlinks=true, linkcolor=darkblue, urlcolor=darkblue, citecolor=darkblue}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
<<echo=F>>=
  if(exists(".orig.enc")) options(encoding = .orig.enc)
@
\newlength{\lyxlabelwidth}      % auxiliary length 
\newenvironment{lyxcode}
	{\par\begin{list}{}{
		\setlength{\rightmargin}{\leftmargin}
		\setlength{\listparindent}{0pt}% needed for AMS classes
		\raggedright
		\setlength{\itemsep}{0pt}
		\setlength{\parsep}{0pt}
		\normalfont\ttfamily}%
	 \item[]}
	{\end{list}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
%\VignetteIndexEntry{Rstyle}

\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage{babel}
\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%

 \ifthenelse{\equal{#1}{}}{%
   \@float{figure}
 }{%
   \@float{figure}[#1]%
 }%
 \centering
}{%
 \end@float
}
\renewenvironment{table}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{table}
 }{%
   \@float{table}[#1]%
 }%
 \centering
%  \setlength{\@tempdima}{\abovecaptionskip}%
%  \setlength{\abovecaptionskip}{\belowcaptionskip}%
% \setlength{\belowcaptionskip}{\@tempdima}%
}{%
 \end@float
}

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\normalsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.95}}
\def\Routbackground{\color{white}}
\def\Routcolor{\color{black}}


\usepackage{listings}% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, style=Rstyle}

\usepackage{xcolor}
\definecolor{darkblue}{HTML}{1e2277}

\makeatother

\usepackage{listings}
\renewcommand{\lstlistingname}{\inputencoding{latin9}Listing}

\begin{document}
\title{R Style. An Rchaeological Commentary. }
\author{Paul E. Johnson <pauljohn @ ku.edu>}
\maketitle

\section{Introduction: Ugly Code that Runs}

Because there is no comprehensive official R style manual, students
and package writers seem to think that there is no style whatsoever
to be followed. While it may be true that ``ugly code runs,'' it
is also 1) difficult to read and 2) frustrating to extend, and 3)
tiring to debug. Code is a language, a medium of communication, and
one should not feel free no ignore its customs. 

After students have finished a semester of statistics with R, they
may be ready to start preparing functions or packages. Those R users
are the ones I'm trying to address with this note. It is important
to realize that the readability of code makes a difference. It sometimes
difficult to know that there is a ``right way'' and a ``wrong way''
because there are so many examples to study on CRAN.

This note describes R style from an Rchaeological\footnote{Definitions:
\begin{description}
\item [{Rchaeology:}] The study of R programming by investigation of R
source code. It is the effort to discern the programming strategies,
idioms, and style of R programmers in order to better communicate
with them.
\item [{Rchaeologist:}] One who practices Rchaeology.
\end{description}
} perspective. By examining the work of the R Core Development Team
\citep{RCore} and other notable package writers, we are able to discern
an implicit style guide. However, this note is not ``official''
or endorsed from R Core.\footnote{Yet :)} With one exception at the
end of this note, none of the advice here is ``my'' advice. Instead,
it is my best description of the standards followed by the leading
R programmers. 

At one point, the only guide was the Google R style guide,\footnote{\url{https://google.github.io/styleguide/Rguide.xml}}
which was used as a policy for R-related ``Google Summer of Code''
projects. There are many excellent suggestions in Hadley Wickham's
Style Guide.\footnote{\url{http://adv-r.had.co.nz/Style.html}} In
what follows, I'll try to explain why there are some variations among
these projects and offer some advice about how we (the users) should
sort through their advice.

<<echo=F>>=
dir.create("plots", showWarnings=F)
@

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\SweaveOpts{prefix.string=plots/plot,ae=F,height=4,width=6}

<<Roptions, echo=F>>=
options(width=100, continue="+ ")
options(useFancyQuotes = FALSE) 
set.seed(12345)
pdf.options(onefile=F,family="Times",pointsize=12)
@

\section{Rchaeological Methodology}

I am a student of R as a programming language. I am also student of
the R community as an international success that created a working
open source computer program. One of the most interesting differences
between R and other open source projects I have observed is that R
attracts non-programmers. There is an abundance of statistical novices
and untrained computer programmers in the R user community. Many students
begin with R as a way of learning about computer programming. In contrast,
the developers of R are world-class software engineers. They have
formal training in computer programming and years of experience in
a variety of computer languages. The diversity creates a healthy tension
that is easy to see in the r-help email list or on Web forums for
R users. 

\subsection{``Use the Source, Luke,'' said Obi-Wan}

What should R code look like? Stop guessing. The implicit style guide
for R is the R source code itself. If users want to communicate with
R Core developers, they ought to communicate using the style that
developers use.

I'm often surprised to find that R users--even experienced ones--have
never looked at the R source code. Before going any further,
\begin{quote}
Open the source code for R. I mean, literally, download R-3.5.2.tar.gz
(or whatever is current when you read this). Unpack that, navigate
to the directory src/library/stats/R. Open the file ``lm.R''. 
\end{quote}
That's what R code should look like. 

Browse other R files in the source code. Notice the files are suffixed
by R, not r! 

Then go read a lot of R packages. Begin with the recommended packages
(in the R source code under src/library/Recommended). Then draw some
samples from CRAN. Choose packages that are prepared by members of
R Core, and then sample a few packages that are widely installed,
such as John Fox's car package \citep{fox_r_2011}. 

After that, pick a random sample of packages on CRAN. Don't be surprised
by ugly code in a randomly chosen R package. 

\subsection{Notice How R Describes its Own Style}

Type the name of a function at the R command prompt. That is the same
as using the function called \inputencoding{latin9}\lstinline!print.function()!\inputencoding{utf8}
to review the contents of a function from an R package. For example,
try ``lm''. The first few lines are\inputencoding{latin9}
\begin{lstlisting}
> lm
function (formula, data, subset, weights, na.action, method = "qr",
    model = TRUE, x = FALSE, y = FALSE, qr = TRUE, singular.ok = TRUE,
    contrasts = NULL, offset, ...)
{
    ret.x <- x
    ret.y <- y
    cl <- match.call()
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "subset", "weights", "na.action",
        "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    if (method == "model.frame")
        return(mf)
    else if (method != "qr")
        warning(gettextf("method = '%s' is not supported. Using 'qr'",
            method), domain = NA)
\end{lstlisting}
\inputencoding{utf8}
That's quite a bit like the code in the file lm.R, but it is not exactly
the same. Even if the code in lm.R were an ugly, horrible mess, its
output in the terminal would be indented and spaced just right. That
is an important Rchaeological finding!

Why can there be a difference between the code for a function in a
file (like ``lm.R'') and the output of the command (like ``lm'')?
Admittedly, this is difficult to understand. The on-screen output
is not (by default, anyway) the source that went into R, but rather
it is R's rendition of the internal structure of the function. I recently
had an epiphany while reading a section in the \emph{Writing R Extensions}
manual called ``Tidying R code''. That title is a bit misleading.
It is not about tidying R source code; rather, it is about beautifying
the rendition of internal structures for the terminal. ``R treats
function code loaded from packages and code entered by users differently.
By default code entered by users has the source code stored internally,
and when the function is listed, the original source is reproduced.
Loading code from a package (by default) discards the source code,
and the function listing is re-created from the parse tree of the
function.'' That is to say, if ugly code is syntatically valid, R
can parse it and structure it according to the internal dictates of
the R runtime system, and when we ask to see the function, we get
a nice looking result.

\subsection{Formulate SEA estimates.}

As already noted, there is no mandatory style for R code. The \emph{R
Internals} manual has a section ``R coding standards,'' but it is
quite brief. The main point that most readers take away concerns indentation:
subsections in code should be preceded by 4 blank spaces, not a tab
character. 

But there is a larger point in \emph{R Internals}, but novices don't
recognize the importance of it. R is a GNU project, and there are
GNU coding standards.\footnote{\url{http://www.gnu.org/prep/standards/standards.html}}
The R project's C code follows the standard closely. In the entire
body of the R source code, we find the GNU thumb print. The importance
of that fact is missed by untrained readers, who mistake the lack
of a comprehensive discussion of style for an encouragement to ``do
anything you want.'' 

In the following, I will try to point out the areas of greatest agreement
by assigning an SEA score to each point. SEA stands for ``Subjective
and completely unscientific personal Estimate of Agreement.'' These
are my Bayesian priors. If I could survey my favorite R programmers,
I'd find some variety, and I am trying to make it clear where the
disagreements might lie. But, then again, I may have been fooling
myself. It has recently been suggested to me that these recommendations
are not descriptions of the Rchaeological community I'm studying,
they are rather my personal litmus test for admirable R programmers. 

\section{Nearly Universally accepted standards.}

\subsection{(SEA 1.0) Indentation of code sections is required. }

This is explicitly spelled out in the R documentation. No tabs! Insert
4 blank spaces. Personally, I prefer 2 spaces, which has been the
default in Emacs. But I'm changing my code to use 4 spaces. If you
find my code with 2 spaces, please accept this apology and believe
that it is an oversight. 

\subsection{(SEA .95). Use ``<-'', not ``='', for assignments. }

One cannot find the equal sign used for assignments in any file in
the R source code. Nor can one find it in any of the Recommended packages
(so far as I can tell). 

Students who have learned R in introductory textbooks are sometimes
shocked to learn that they were taught wrong. I'm sympathetic to their
outrage. How can this be? 

The equal sign was used by mistake so frequently that the R system
was re-designed to tolerate that mistake. \emph{Most} usages of the
equal sign for assignments do not cause runtime errors. Not all possible
problems were eliminated, however. Thus the equal sign is not recommended,
it is tolerated. Nevertheless, A horrible profusion of textbooks and
packages ensued using the equal sign for assignment. 

\subsection{(SEA .98) Blank spaces around symbols are required. }

This is a general GNU coding standard. 
\begin{enumerate}
\item Insert spaces before and after 

\begin{enumerate}
\item mathematical symbols like: ``='', ''<-'', ``<'', ``{*}'',
''+''
\item R binary operations like: ``\%{*}\%'', ``\%o\%'', and ``\%in\%''.
\end{enumerate}
\item Put one space after commas. 
\item Insert one space before the opening squiggly braces ``\{''.
\item Put one space after the closing parenthesis ``)'' and the closing
squiggly brace ``\}''.
\end{enumerate}
This is purely a matter of convention and legibility, it does not
affect the ``rightness'' of code. 

Other observations about spaces,
\begin{enumerate}
\item Do not insert spaces between function names and their opening parentheses.
\item After reviewing the R source code, I was uncertain about whether one
ought to insert one space after ``if'' and ``for''. From an Rchaeological
perspective, this is a little bit perplexing. In the help page for
those terms (see help(``for'')), there is no space after ``if''
or ``for''. In the R-3.0.0 source code folder src/library/base/R,
I count 1741 instances of ``if(`` and 683 instances of ``if (``.
The former style seemed right to me, at least at first, because people
often say that R's ``if'' and ``for'' are functions. I asked for
clarification in the R-devel email list, and Peter Dalgaard explained
that the space should be used because those terms are

\begin{quote}
language constructs (and they \emph{are} keywords, not names, that's
why ?for won't work). The function calls are `if`(fee, \{foo\}, \{fie\})
and something rebarbative for `for`(....).

Besides, both constructs are harder to read without the spaces. (r-devel,
April 18, 2013)
\end{quote}
For me, that settles the question. For R code, as in C, ``if'' and
``for'' should be treated as keywords, and there would be a space
after them, as in ``\inputencoding{latin9}\lstinline!if (x < 7)!\inputencoding{utf8}''.
\item Do not insert ``extra spaces'' inside parentheses. 

Programmers who have written in the BASH scripting language may recall
that a space inside brackets is required. That training causes me
to think that R code is a little bit ``jammed together.'' This is
pleasant to my eye:

\inputencoding{latin9}\begin{lstlisting}
if ( (x == 1) & (y == 2) ) {
\end{lstlisting}
\inputencoding{utf8}
but, from an Rchaeological point of view, more the correct style is:

\inputencoding{latin9}\begin{lstlisting}
if((x == 1) & (y == 2)) {
\end{lstlisting}
\inputencoding{utf8}
The insertion of the interior parentheses for the smaller conditions
inside the if statement is consistent with the GNU standard for C.
\end{enumerate}

\subsubsection*{Is there an ``argument exception'' to the space rule for equal
signs?}

Package writers are not entirely consistent, and Rchaeologically speaking,
we cannot be sure if these variations are accidental. We sometimes
find no spaces, as in

\inputencoding{latin9}\begin{lstlisting}
plot(x, y, lwd=4, col=green, main="My Title")
\end{lstlisting}
\inputencoding{utf8}
It would surely be more correct like so:

\inputencoding{latin9}\begin{lstlisting}
plot(x, y, lwd = 4, col = green, main = "My Title")
\end{lstlisting}
\inputencoding{utf8}
Spaces may sometimes be omitted in an effort to keep code on one line.
Especially where publishers are concerned about the use of scarce
paper, the omission of spaces around equal signs is not uncommon.
Please note, however, that it is NEVER acceptable to omit the spaces
after commas!

\subsubsection*{What about indentation of long function declarations?}

One of the interesting space related questions is the indentation
of function declarations when there are many arguments. Consider the
R source code for the function lm():

\inputencoding{latin9}\begin{lstlisting}
lm <- function (formula, data, subset, weights, na.action,
                method = "qr", model = TRUE, x = FALSE, y = FALSE,
                qr = TRUE, singular.ok = TRUE, contrasts = NULL,
                offset, ...)
\end{lstlisting}
\inputencoding{utf8}
Note that lines 2-4 are indented under the letter ``f'' in formula.
If the function's name were longer, it would push all of that indented
code to the right, probably causing line wraps. The solution is to
put the function's name and the assignment symbol on separate line.
This is the format of R's function plot.lm(). 

\inputencoding{latin9}\begin{lstlisting}
plot.lm <-
function (x, which = c(1L:3L,5L), ## was which = 1L:4L,
          caption = list("Residuals vs Fitted", "Normal Q-Q",
          "Scale-Location", "Cook's distance",
          "Residuals vs Leverage",
          expression("Cook's dist vs Leverage  " * h[ii] / (1 - h[ii]))),
          panel = if(add.smooth) panel.smooth else points,
          sub.caption = NULL, main = "",
          ask = prod(par("mfcol")) < length(which) && dev.interactive(), ...,
          id.n = 3, labels.id = names(residuals(x)), cex.id = 0.75,
          qqline = TRUE, cook.levels = c(0.5, 1.0),
          add.smooth = getOption("add.smooth"),
          label.pos = c(4,2), cex.caption = 1)
{
\end{lstlisting}
\inputencoding{utf8}
The continuation is indented to be below the first argument. The benefit
of this ``declaration by itself'' approach is that the additional
lines are always re-formatted with consistent indentation and we are
not creating a huge empty white space due to indentation.

\subsubsection*{Try formatR::tidy.source()}

The advice so far mostly concerns ``white space''. We would like
a programmer's text editor to handle automatically as much of that
as possible. 

The R package ``formatR'' \citep{formatr} has a function called
tidy.source() which can often (but not always) clean up code. Below
I've pasted in part of an Emacs session. I wrote a badly formatted
function, myfn(), and copied it to the clipboard, and then tidy.source()
reads the clipboard. It works like magic. 

\inputencoding{latin9}\begin{lstlisting}
> myfn <- function(x){ if (x < 7) {i = 77; print(paste("x is less than 7 but i is", i))} else {print("x is excessive") }} 
> library(formatR)
> tidy.source()
function(x) {
    if (x < 7) {
        i = 77
        print(paste("x is less than 7 but i is", i))
    } else {
        print("x is excessive")
    }
}
\end{lstlisting}
\inputencoding{utf8}
The tidy.source() function can get rid of equals sign assignments
if we ask it to. (In my opinion, it should do that by default.)

\inputencoding{latin9}\begin{lstlisting}
> tidy.source(source = "clipboard", replace.assign = TRUE)
function(x) {
    if (x < 7) {
        i <- 77
        print(paste("x is less than 7 but i is", i))
    } else {
        print("x is excessive")
    }
} 
\end{lstlisting}
\inputencoding{utf8}
The tidy.source() function can receive input as files or whole directories.

There are two reasons why tidy.source() is not a panacea. First, by
design, tidy.source() will fail if there are programming errors in
the original source code. That leads to a Catch-22. I want to clean
up the code to find out why it does not run, but tidy.source() cannot
clean it up because it does not run. Second, quite often it happens
that tidy.source() chokes on unexpected user code. Especially problematic
is code that has comments inserted in unexpected places. For example,
I recently ran tidy.source() on the file emb.r in the package Amelia
\citep{Amelia}.

\inputencoding{latin9}\begin{lstlisting}
> library(formatR)
> tidy.source("emb.r")
Error in base::parse(text = text, srcfile = NULL) :
  152:88: unexpected SPECIAL
151: }
152: if (ncol(as.matrix(startvals)) == AMp+1 && nrow(as.matrix(startvals)) == AMp+1)        %InLiNe_IdEnTiFiEr%
                                                                                           ^
\end{lstlisting}
\inputencoding{utf8}
I would estimate that tidy.source() fails on about one-third of the
R code I randomly select from CRAN.

\subsection{(SEA .70) The ``\} else \{'' policy. }

Did you notice ``\inputencoding{latin9}\lstinline!} else {!\inputencoding{utf8}''
in the \inputencoding{latin9}\lstinline!tidy.source()!\inputencoding{utf8}
output for \inputencoding{latin9}\lstinline!myfn()!\inputencoding{utf8}?
That's the correct style. We should not have the left squiggly brace
``\inputencoding{latin9}\lstinline!}!\inputencoding{utf8}'' on
a separate line from the ``\inputencoding{latin9}\lstinline!else!\inputencoding{utf8},''
and the right squiggly brace ``\inputencoding{latin9}\lstinline!{!\inputencoding{utf8}''
should be on that same line. This is, well, obviously good (in my
opinion).

Why? Try this at the command line. 

\inputencoding{latin9}\begin{lstlisting}
> if (x < 10) print("hello") 
[1] "hello" 
> else print("goodbye") 
Error: unexpected 'else' in "else"
\end{lstlisting}
\inputencoding{utf8}
R does not realize that it is not yet finished with the if keyword's
work. The keyword else appears to begin a new thought, which is illegal.
The if's help page (run \inputencoding{latin9}\lstinline!help("if")!\inputencoding{utf8}
or \inputencoding{latin9}\lstinline!?"if"!\inputencoding{utf8}) is
referring to this problem when it says, 
\begin{quote}
In particular, you should not have a newline between ‘\}’ and ‘else’
to avoid a syntax error in entering a ‘if ... else’ construct at the
keyboard or via ‘source’. For that reason, one (somewhat extreme)
attitude of defensive programming is to always use braces, e.g., for
‘if’ clauses.
\end{quote}
I agree with the somewhat extreme attitude, but will compromise: If
one uses squiggly braces, always follow the ``\inputencoding{latin9}\lstinline!} else {!\inputencoding{utf8}''
policy. 

Some might follow a soft line on this, suggesting only that \textbf{users
should not} \textbf{begin a line with the word else}. That does not
go quite far enough for me. I'd add, \textbf{always use squiggles
after else.} This is simply a way of avoiding a very common coding
error. This code is OK:

\inputencoding{latin9}\begin{lstlisting}
if (x < 7) print("so far, so good") else
print("this is else")
\end{lstlisting}
\inputencoding{utf8}
But it invites a coding error like so:

\inputencoding{latin9}\begin{lstlisting}
if (x < 7) print("so far, so good") else
print("this is else")
print("and we want this also to be with else, but it is not")
\end{lstlisting}
\inputencoding{utf8}
To be perfectly clear, and to protect ourselves against editing errors
in the future, we could follow the ``somewhat extreme'' advice and
write this:

\inputencoding{latin9}\begin{lstlisting}
if (x < 7) {
	print("so far, so good") 
} else {
    print("this is else")
    print("and we want this also to be with else")
}
\end{lstlisting}
\inputencoding{utf8}

\subsubsection*{Counter-argument based on the R source code}

This would be a completely closed case if not for the fact that the
``\inputencoding{latin9}\lstinline!} else {!\inputencoding{utf8}''
policy is ignored in vast expanses of the R source code. In the R
source code, scan for the keyword else and in almost every file, one
finds:

\inputencoding{latin9}\begin{lstlisting}
}
  else
\end{lstlisting}
\inputencoding{utf8}
A naked else! This is frustrating for writers of style guides. It
ignores the advice in the ``if'' help page. We cannot run this code
line-by-line. 

On the other hand, the function that includes that apparently runs!
Why doesn't that code crash? When an if/else statement is enclosed
in a larger area that is demarcated by squiggly braces, then R will
understand the naked else when it finds it. Observe the fix at the
command line:

\inputencoding{latin9}\begin{lstlisting}
> x <- 1
> {
+ if (x < 10) print("hello")
+ else
+ print("My dangling else")
+ }
[1] "hello"
\end{lstlisting}
\inputencoding{utf8}
I don't think I'm going to have any luck persuading the R Core Development
Team that their naked elses need to be fixed. The best I can do is
to urge code writers to use ``\inputencoding{latin9}\lstinline!} else {!\inputencoding{utf8}''
and make them responsible for errors that result from ignoring that
rule. 

One will note another interesting anomaly while reviewing R source
code. Unlike programs written in C, where a consistent style for the
placement of squiggly braces will be followed, in R we observe files
that do not follow a particular rule. In src/library/src/logLik.R,
we find functions in both the K\&R (\citealp{kernighan_c_1988}) C
style

\inputencoding{latin9}\begin{lstlisting}
nobs.logLik <- function(object, ...) {
    res <- attr(object, "nobs")
    if (is.null(res)) stop("no \"nobs\" attribute is available")
    res
}
\end{lstlisting}
\inputencoding{utf8}
and we also find the vertically aligned squiggly braces approach:

\inputencoding{latin9}\begin{lstlisting}
print.logLik <- function(x, digits = getOption("digits"), ...)
{
    cat("'log Lik.' ", paste(format(c(x), digits=digits), collapse=", "),
        " (df=",format(attr(x,"df")),")\n",sep="")
    invisible(x)
}
\end{lstlisting}
\inputencoding{utf8}
I am at a loss to explain these stylistic variations, so I conclude
that R users can follow either style, while keeping in mind the ``\inputencoding{latin9}\lstinline!} else {!\inputencoding{utf8}''
policy, which strongly pushes us toward the K\&R style.

\section{How to name functions.}

Now we begin to consider some issues that are more subjective. Many
styles are legal, but some are more easily understood. R syntax has
changed over the years, and some things that were illegal are now
allowed. And some styles that were standard might now be discouraged. 

\subsection{(.98 SEA) Avoid using names that are already in use by R, especially
common ones.}

Don't write functions named ``\inputencoding{latin9}\lstinline!rep()!\inputencoding{utf8}'',
``\inputencoding{latin9}\lstinline!seq()!\inputencoding{utf8}'',
``\inputencoding{latin9}\lstinline!c()!\inputencoding{utf8}'',
and so forth. Notice that my new function \inputencoding{latin9}\lstinline!lm()!\inputencoding{utf8}
does not obliterate the one from the stats package, but it sure does
make it harder to use it.

\inputencoding{latin9}\begin{lstlisting}
> lm <- function(z) print("Hi, I'm z where lm was")
> x <- rnorm(100)
> y <- rnorm(100)
> lm (y ~ x)
[1] "Hi, I'm z where lm was"
> stats::lm(y ~ x)

Call:
stats::lm(formula = y ~ x)

Coefficients:
(Intercept)            x
    0.02688      0.01796
\end{lstlisting}
\inputencoding{utf8}
As long as we remember that \inputencoding{latin9}\lstinline!lm()!\inputencoding{utf8}
is in the namespace stats, we can find it. 

Similarly, packages can declare namespaces of their own. (Since R
version 2.14, all packages \emph{must} do so.) We are allowed to place
a new function like \inputencoding{latin9}\lstinline!seq()!\inputencoding{utf8}
or \inputencoding{latin9}\lstinline!lm()!\inputencoding{utf8} into
a package if we want to. Nevertheless, almost everybody will hate
to read code like that. 

The danger that user functions might interfere with core functionality
was at one time very serious. Now it is, for the most part, a historical
footnote. It is still possible to obliterate a function that is embedded
within a namespace, but doing so requires a bit of effort and mischief.\footnote{In case you wonder, here's how to cause the worst case scenario.
\begin{lyxcode}
nseq~<-~function(x)~print(\textquotedbl Hello,~good~to~see~you\textquotedbl )

assignInNamespace(\textquotedbl seq.default\textquotedbl ,~nseq,~\textquotedbl base\textquotedbl )
\end{lyxcode}
}

When we say that a namespace is imported, it means that all of the
functions in that namespace can be accessed by the function's name,
without the namespace name as a prefix. We might write \inputencoding{latin9}\lstinline!base::seq(1, 10, length.out = 40)!\inputencoding{utf8}
to be clear, but we need only write \inputencoding{latin9}\lstinline!seq(1, 10, length.out = 40)!\inputencoding{utf8}
because an R session imports the namespace base. I notice a trend
in R to suggest that one should not import whole namespaces unless
that is truly necessary, and even if a namespace is imported, we should
strive for clarity by using syntax that includes the namespace name.
In the source code for many R examples, one will find syntax like
\inputencoding{latin9}\lstinline!graphics::par()!\inputencoding{utf8}
where, until recently, that would have simply been \inputencoding{latin9}\lstinline!par()!\inputencoding{utf8}.

\subsection{(.65 SEA)Use periods to indicate classes, otherwise don't use periods
in function names. }

Instead, use camel case to name functions. This function name \inputencoding{latin9}\lstinline!mySuperThing()!\inputencoding{utf8}
is better than \inputencoding{latin9}\lstinline!my.super.thing()!\inputencoding{utf8}. 

The period in a function name has a special meaning in the S3 object-oriented
framework. A ``generic function'' (such as print() or summary())
is accompanied by methods that implement its work for particular kinds
of objects, such as \inputencoding{latin9}\lstinline!print.function()!\inputencoding{utf8}
or \inputencoding{latin9}\lstinline!print.lm()!\inputencoding{utf8}.
Before the period, we have a function's name, and after the period,
we have the class name of the object being managed. The function name
\inputencoding{latin9}\lstinline!my.super.thing()!\inputencoding{utf8}
suggests the user might have an object of class ``thing'' and that
\inputencoding{latin9}\lstinline!my.super(x)!\inputencoding{utf8}
would diagnose the class of x and send the work to \inputencoding{latin9}\lstinline!my.super.thing()!\inputencoding{utf8}.
A camel cased function name \inputencoding{latin9}\lstinline!mySuperThing()!\inputencoding{utf8}
will not convey the wrong meaning. 

If we were starting with a clean slate, I believe many R functions
would be re-named for the purposes of consistency. Since we do not
have a clean slate, we live with an accumulation of function names
from olde S and R. Changes in computer science--the growth of object-oriented
programming--cause new naming conventions. Consider some of the traditional
S function names that are still used in R, like \inputencoding{latin9}\lstinline!read.table!\inputencoding{utf8}
and \inputencoding{latin9}\lstinline!read.csv!\inputencoding{utf8}.
Those are not method implementations of a generic function read().
The period is simply part of a shorthand of the form ``action.qualifier''.
Otherwise, if one had an object of type table, then read(x) would
call read.table(x). But it does not:

\inputencoding{latin9}\begin{lstlisting}
> example(table)
> class(tab) 
[1] "xtabs" "table"
> read(tab) 
Error: could not find function "read" 
\end{lstlisting}
\inputencoding{utf8}
I believe that, if these functions were being created today, they
would be named \inputencoding{latin9}\lstinline!readTable()!\inputencoding{utf8}
and \inputencoding{latin9}\lstinline!readCSV()!\inputencoding{utf8}. 

In the R source code, there are some very confusing function names
and I have a hard time believing we would use them if we were re-designing
everything today. The file src/library/base/readhttp.R has a function
called \inputencoding{latin9}\lstinline!url.show()!\inputencoding{utf8},
which follows none of the styles that I recognize. There's no class
\inputencoding{latin9}\lstinline!show!\inputencoding{utf8} and \inputencoding{latin9}\lstinline!url()!\inputencoding{utf8}
is not a generic function. In the ``action.qualifier'' tradition,
it would be \inputencoding{latin9}\lstinline!show.url()!\inputencoding{utf8}.
And why not \inputencoding{latin9}\lstinline!showURL()!\inputencoding{utf8}?
I hasten to point out that the same file includes some camel cased
functions like \inputencoding{latin9}\lstinline!defaultUserAgent()!\inputencoding{utf8}.

I like camel cased function names. They are common in Objective-C
and Java. Some programmers vigorously disagree. Programmers trained
in C++ seem to hate camel case names, almost at a visceral level.
As a result, we find a division of opinion on function names. As a
spot check, consider two of my favorite packages, MASS \citep{venables_modern_2002}
and car \citep{fox_r_2011}. There are not many camel case function
names in the MASS, where we find brief names in lower case letters
(such as \inputencoding{latin9}\lstinline!boxcox()!\inputencoding{utf8}).
In contrast, car calls that \inputencoding{latin9}\lstinline!boxCox()!\inputencoding{utf8}.
When I started using R, Professor Fox used function names with periods,
but he has been systematically weeding them out and replacing them
with camel case names. If those two packages are counterbalancing
each other in my mind (for and against camel case functions), the
leading packages for mixed effects models, nlme \citep{pinheiro_nlme:_2012}
and lme4 \citep{lme4}, weigh in on the camel case side of the ledger. 

In conclusion, users should avoid gratuitous periods in function names
because, after S3, the period has special meaning in R. When a function
has been declared as a generic, then that function's name followed
by a period has an object-oriented meaning. A period is not merely
word separation. New functions introduced in R tend to use either
camel case names (\inputencoding{latin9}\lstinline!browseVignettes()!\inputencoding{utf8})
or underscores (\inputencoding{latin9}\lstinline!get_all_vars()!\inputencoding{utf8}).
Considering recent additions to R, I believe that the chance of finding
a decorative period in a new function name is almost zero. But we
are still living with an awful lot of older counter-examples.

\section{How to name variables (and objects).}

\subsection{(1.0 SEA) Follow the ``letters and numbers'' rule.}

R variable names must 
\begin{enumerate}
\item begin with an alphabetical character
\item include only letters, numbers and the symbols ``\_'' and ``.''. 
\end{enumerate}
They must not include ``{*}'',''?'',''!'',''\&'' or other
special symbols. They must not include spaces.

One peculiar side effect of this rule is that the ellipsis symbol,
three periods, ``...'', is actually a legal object name. That's
three periods, which is just as legal as aaa or bbb. Many R functions
allow the argument ``...'', most users don't realize it literally
is a word. When that is listed as a function argument, then any argument
that the user includes is gobbled up by ``...''. 

\subsection{(1.0 SEA) Never name a variable T or F. }

Almost everybody (99.999\%) will agree with this. These are too easily
mistaken for TRUE and FALSE values. Since R uses TRUE and FALSE as
vital elements of almost all commands and functions, and since users
are allowed to abbreviate those as T or F, a horrible confusion can
develop if variables are named T or F. 

Here's some good news. R will not allow users to name variables TRUE
or FALSE:

\inputencoding{latin9}\begin{lstlisting}
> TRUE <- 7
Error in TRUE <- 7 : invalid (do_set) left-hand side to assignment
\end{lstlisting}
\inputencoding{utf8}
But R will not prevent the usage of T and R for variable names.

\subsection{(.75 SEA) Avoid declaring variables that have the same names as widely
used functions.}

This is just a handy rule of thumb now, but it used to be a ``watch
out for that tree!'' warning. In 2001, I created a variable ``rep''
(for Republican party members) and nothing worked in my program. In
exasperation, I wrote to the r-help list, and learned that I had obliterated
R's own function \inputencoding{latin9}\lstinline!rep()!\inputencoding{utf8}
with my variable. \inputencoding{latin9}\lstinline!rep()!\inputencoding{utf8}
is used inside many R functions and thus obliterating it was a very
serious mistake. In 2002 or so, the R system was revised so that user-declared
variables cannot ``step on'' R system functions. Nevertheless, it
is disconcerting to me (probably others) when users create variables
with names like ``lm'', ``rep'', ``seq'', and so forth. 

\subsection{(0.50 SEA) Use long names for infrequently used variables. }

And use short names for variables that will be used very often.

If a variable is going to be used twice, we might as well be verbose
about it. ``xlog'' is better than ``xl'', if we are only writing
it a few times. If we are going to use a name 50 times in a 5 line
program, we should choose a short one. For abbreviations, include
a comment to remind the reader what the thing stands for.

\subsection{(0.10 SEA) Suggested naming scheme: keep related objects in an alphabetically
sorted scheme.}

This is my personal naming scheme. Nobody but me follows this policy
now, but I like it so much I'm tacking it onto the end of this essay.
I believe that R code is much more readable if objects that ``go
together'' begin with the common series of letters. As seen by ls(),
the related pieces should always be together. From now on, when I
work with a variable named ``x'', then all transformations will
begin with ``x''. I will use ``xlog'' rather than ``logx'' and
so forth. 

Example 1. Create a numeric variable, recode it as a factor, then
create the ``dummy'' variables that correspond. I include the output
in order to emphasize the clarity due to the alphabetical emphasis:

<<>>=
x <- runif(1000, min = 0, max = 100)
xf <- cut(x, breaks = c(-1, 20, 50, 80, 101), labels = c("cold", "luke", "warm", "hot"))
xfdummies <- contrasts(xf, contrasts = FALSE )[xf,]
colnames(xfdummies) <-  paste("xf", c("cold", "luke", "warm", "hot"), sep="")
rownames(xfdummies) <- names(x)
dat <- data.frame(x, xf, xfdummies)
head(dat)
@

I have not included the output of these code chunks, but the alphabetical
emphasis is demonstrated in them.

Example 2. Estimate a regression, calculate summary information.

<<echo=T, eval=F>>=
set.seed(12345)
x1 <- rnorm(200, m = 300, s = 140)
x2 <- rnorm(200, m = 80, s = 30)
y <- 3 + 0.2 * x1 + 0.4 * x2 + rnorm(200, s=400)
dat <- data.frame(x1, x2, y); rm(x1,x2,y)
m1 <- lm (y ~ x1 + x2, data = dat)
m1summary <- summary(m1)
m1se <- m1summary$sigma
m1rsq <- m1summary$r.squared
m1coef <- m1summary$coef
m1aic <- AIC(m1)
@

Example 3. Run a regression, collect mean-centered and residual centered
variants of it.

<<ps10, fig=T, eval=F, height=5, width=9>>=
library(rockchalk)
dat$y2 = with(dat, 3 + 0.02 * x1 + 0.05 * x2 + 2.65 * x1 *x2 + rnorm(200, s=4000))
par(mfcol=c(1,2))
m1 <- lm(y2 ~ x1 + x2, data = dat)
m1i <- lm(y2 ~ x1 * x2, data = dat)
m1ps <- plotSlopes(m1, plotx = "x1", modx = "x2")
m1ips <- plotSlopes(m1i, plotx = "x1", modx = "x2")
m1imc <- meanCenter(m1i)
m1irc <- residualCenter(m1i)	
@

\section{Conclusion}

R can be understood at several levels, varying in sophistication from
an elementary statistics course or to an advanced platform for the
development of computer programming concepts. In the future, I will
be more cautious to teach new R users about coding style. I intend
to prevent the accumulation of bad habits that result in code that
is difficult to read and hard to debug. 

Users who ask for help in the r-help email list \footnote{\url{http://www.r-project.org/mail.html}}
or on web forums \footnote{e.g., \url{http://stackoverflow.com/questions/tagged/r}}
are well advised to remember the importance of style. Most newcomers
believe that the experts will understand what they write, but that's
not true. Experts will find it much easier to spot errors in code
that has the correct indentation and uses a proper naming scheme for
variables and functions. In my experience, the most likely source
of trouble in R code is not actually the style, but rather poor compartmentalization
of separate calculations. The potential to compartmentalize, however,
is obscured by bad style. 

When users throw together 2000 lines of spaghetti code with no indentation
(I can point to examples on CRAN), there's almost no chance than anyone
except the author will be able to understand and extend that kind
of code. Ugly code writers will respond, ``my ugly code runs!''
That misses the point. Coding style is not about making things ``work,''
it is about making them work in a way that is understood by the widest
possible audience. And where possible, the code should be re-usable
and extended to other purposes. 

\bibliographystyle{chicago}
\bibliography{rockchalk}

\end{document}

%% LyX 2.3.1 created this file.  For more info, see http://www.lyx.org/.
%% Do not edit unless you really know what you are doing.
\documentclass[american,noae]{scrartcl}
\usepackage{lmodern}
\renewcommand{\sfdefault}{lmss}
\renewcommand{\ttdefault}{lmtt}
\usepackage[T1]{fontenc}
\usepackage[utf8]{inputenc}
\usepackage{geometry}
\geometry{verbose,tmargin=1in,bmargin=1in,lmargin=1in,rmargin=1in}
\setlength{\parskip}{\smallskipamount}
\setlength{\parindent}{0pt}
\usepackage{babel}
\usepackage{url}
\usepackage{enumitem}
\usepackage[authoryear]{natbib}
\usepackage[unicode=true,pdfusetitle,
 bookmarks=true,bookmarksnumbered=false,bookmarksopen=false,
 breaklinks=false,pdfborder={0 0 1},backref=section,colorlinks=false]
 {hyperref}

\makeatletter
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Textclass specific LaTeX commands.
<<echo=F>>=
  if(exists(".orig.enc")) options(encoding = .orig.enc)
@
\newlength{\lyxlabelwidth}      % auxiliary length 
\providecommand*{\code}[1]{\texttt{#1}}
\providecommand*{\strong}[1]{\textbf{#1}}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% User specified LaTeX commands.
%\VignetteIndexEntry{Rchaeology}

\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage{babel}
\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{figure}

 }{%

   \@float{figure}[#1]%

 }%

 \centering

}{%

 \end@float

}

\renewenvironment{table}[1][]{%

 \ifthenelse{\equal{#1}{}}{%

   \@float{table}

 }{%

   \@float{table}[#1]%

 }%

 \centering

%  \setlength{\@tempdima}{\abovecaptionskip}%

%  \setlength{\abovecaptionskip}{\belowcaptionskip}%

% \setlength{\belowcaptionskip}{\@tempdima}%

}{%

 \end@float

}


%\usepackage{listings}
% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true,style=Rstyle}

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\scriptsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.90}}



\usepackage{babel}

\usepackage[samesize]{cancel}



\usepackage{ifthen}



\usepackage{listings}% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, style=Rstyle}

\makeatother

\usepackage{listings}
\renewcommand{\lstlistingname}{\inputencoding{latin9}Listing}

\begin{document}
\title{Rchaeology: Idioms of R Programming}
\author{Paul E. Johnson <pauljohn @ ku.edu>}

\maketitle
This document was initiated on May 31, 2012. The newest copy will
always be available at \url{http://pj.freefaculty.org/R} and as a
vignette in the R package ``rockchalk''.
\begin{description}
\item [{Rchaeology:}] The study of R programming by investigation of R
source code. It is the effort to discern the programming strategies,
idioms, and style of R programmers in order to better communicate
with them.
\item [{Rchaeologist:}] One who practices Rchaeology.
\end{description}
These are Rcheological observations about the style and mannerisms
of R programmers in their native habitats. Almost all of the insights
here are gathered from the r-help and r-devel emails lists, the stackoverflow
website pages for R, and the R source code itself. These are lessons
from the ``school of hard knocks.''

How is this different from Rtips(\url{http://pj.freefaculty.org/R/Rtips.{pdf,html}})?
\begin{enumerate}
\item This is oriented toward programming R, rather than using R.
\item It is more synthetic, aimed more at finding ``what's right'' rather
than ``what works.''
\item It is written with Sweave (using Harrell's Sweavel style) so that
code examples work.
\end{enumerate}
Where did the ``R Style'' section go? It was removed to a separate
vignette, RStyle, in the rockchalk package.

\tableofcontents{}

<<echo=F>>=
dir.create("plots", showWarnings=F)
@

% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\SweaveOpts{prefix.string=plots/plot,ae=F,height=4,width=6}

<<Roptions, echo=F>>=
options(width=100, continue="  ")
options(useFancyQuotes = FALSE) 
set.seed(12345)
pdf.options(onefile=F,family="Times",pointsize=12)
@

\section{Introduction: R Idioms.}

This vignette is about the R idioms I have learned while working on
the rockchalk package. At the current time, I don't understand all
of the R idioms that are common in the advanced R programmers' conversation,
but I am getting some traction.

There is a language gap between an R user and an R programmer. Users
write ``scripts'' that use functions from R packages. Users don't
write (many) functions. Users don't make packages. And many users
are happy to keep it that way. For users who want to become programmers,
there is usually a harsh awakening. R for development is a different
language. Well, that's wrong. It is a different dialect. 

A transitional R user will have to learn a lot of terminology and
undergo a change of paradigm. There are resources available! Everybody
should subscribe to the email lists for the R project (\url{http://www.r-project.org/mail.html}),
especially r-help (for user questions) and r-devel. Rchaeologically
speaking, that is the native habitat of the R programmers. There is
also a burgeoning collection of blogs and Web forums, perhaps most
notably the R section on StackExchange (\url{http://stackoverflow.com/questions/tagged/r}).
Excellent books have been published. As an Rcheologist, I am drawn
to the books that are written by the R insiders. \emph{S Programming},
by William Venables and Brian Ripley \citeyearpar{venables_s_2000},
is a classic. Books by pioneering developers Robert Gentleman, \emph{R
Programming for Bioinformatics} (2009), and John Chambers, \emph{Software
for Data Analysis} \citeyearpar{chambers_software_2008}, are, well,
awesome. In 2012, I started teaching R programming using Norman Matloff's,
\emph{The Art of R Programming} (2012), which I think is great and
recommend strongly (even though Professor Matloff is not one of the
R Core Team members, so far as I know). 

There are now three types of object-oriented programming in R (S3,
S4, and reference classes) and the programmer is apparently free to
select among them without prejudice. The best brief explanation of
S3 that I've found is in Friedrich Leisch's brief note about R packaging,
``Creating R Packages: A Tutorial'' (2009). One should be mindful
of the fact that R is provided with several manuals, one of which
is the \emph{R Language Definition}. I find that one difficult to
understand, and I usually can't understand it until I search through
the R source code and packages for usage examples.

I usually approach R coding in three phases. I write code that ``works'',
however tedious and slow it might be. There will be a lot of ``copy
and paste'' constructions with tedious, manual editing of commands.
Then I read through the code and look for repetitious elements, and
I re-organize to make functions that abstract that work. If there
are any stanzas that look mostly the same, except that ``x1'' is
repalced by ``x2'', I know the work is not done. Finally, I look
through the code to find ``dumb'' constructions that can be cleaned
up and completed with fewer lines. A dumb construction is one that
I would be ashamed to show to an expert R programmer.

Beginning R programmers, the ones who have never studied C or Java
or Fortran, will often become ``stuck'' in stage one of that process.
Experienced programmers with more formal training will usually have
the ``little voice'' in the back of their minds saying ``there's
a better way,'' and, unfortunately, R novices don't hear that voice.
Lacking some of the discipline imposed by those other languages, the
usual ``rambling'' R script will seem ``good enough.'' The first
step for the transitional R programmer, the one who does not want
to be a novice any more, is to take a harsh look at the code that
has already been written.

The rockchalk package has many functions that receive fitted regression
models and re-arrange their components for re-fitting or plotting.
A good deal of the advice I have to offer is about the process of
managing R formula, but I have also accumulated some lessons in argument
handling. 

\section{do.call(), eval(), substitute(), formula().}

If the transitional programmer understands these four functions, she/he
will be well on the way to escaping the category of novice R programmer.
The other benefit is that the code written by the experts, including
the R source code itself, will become much more understandable.

\subsection{Rewriting Formulas. My Introductory Puzzle.}

On May 29, 2012, I was working on a regression problem in the rockchalk
package. I have a number of functions that receive elementary regressions
and then change them. I need to receive a fitted model, extract the
formula, change some variables, and then revise the formula to match
the new variables. And then run the model over again. 

The functions meanCenter() and residualCenter() receive a fitted regression
model, transform some variables, and then fit a new regression. Suppose
a regression has been fitted as ``\code{y \textasciitilde{} x1{*}x2}''.
The R result will estimate a predictive formula such as $\hat{y}_{i}=\hat{\beta}_{0}+\hat{\beta}_{1}x1_{i}+\hat{\beta}_{2}x2_{i}++\hat{\beta}_{3}x1_{i}\cdot x2_{i}$.
meanCenter() will replace the non-centered variables ``x1'' and
``x2'' with mean-centered variables ``x1c'' and ``x2c''. The
original formula y \textasciitilde{} x1{*}x2 must be replaced with
y \textasciitilde{} x1c{*}x2c. I found this to be a very complicated
problem with a very satisfying answer. 

My first effort used R's update function. That is the most obvious
approach. I learned very quickly that update() is not sufficient.
It is fairly easy to replace x1 with x1c in the formula, but not when
x1 is logged or otherwise transformed. Here is the runable example
code that demonstrates the problem. 

<<include=T, results=hide>>=
dat <- data.frame(x1 = rnorm(100, m = 50), x2 = rnorm(100, m = 50),
  x3 = rnorm(100, m = 50), x4 = rnorm(100, m=50), y = rnorm(100))
m2 <- lm(y ~ log(x1) + x2*x3, data = dat)
suffixX <- function(fmla, x, s){
    upform <- as.formula(paste(". ~ .", "-", x, "+", paste(x, s, sep = ""), sep=""))
    update.formula(fmla, upform)
}
newFmla <- formula(m2)
newFmla
suffixX(newFmla, "x2", "c")
suffixX(newFmla, "x1", "c")
@

Run that and check the last few lines of the output. See how the update
misses x1 inside log(x1) or in the interaction?

\inputencoding{latin9}\begin{lstlisting}
> newFmla <- formula(m2)
> newFmla
y ~ log(x1) + x2 * x3
> suffixX(newFmla, "x2", "c")
y ~ log(x1) + x3 + x2c + x2:x3
> suffixX(newFmla, "x1", "c")
y ~ log(x1) + x2 + x3 + x1c + x2:x3
\end{lstlisting}
\inputencoding{utf8}
I asked the members of r-help for assistance. Lately I've had very
good luck with r-help. Gabor Grothendieck wrote an answer to r-help
on May 29, 2012. He said simply, ``Try substitute,'' with this example.

\inputencoding{latin9}\begin{lstlisting}
> do.call("substitute", list(newFmla, setNames(list(as.name("x1c")), "x1")))
y ~ log(x1c) + x2 * x3
\end{lstlisting}
\inputencoding{utf8}
Problem solved, in a single line. 

That's very clever, I think. It packs together a half-dozen very deep
thoughts. It has most of the essential secrets of R's guts, laid out
in a single line. It has do.call(), substitute(), it interprets a
formula as a list, and it shows that every command in R is, when it
comes down to brass tacks, a list. 

I would like to take up these separate pieces in order.

\subsection{A Formula Object is a List.}

While struggling with this, I noticed this really interesting pattern.
The solution depends on it. The object ``newFmla'' is not just a
text string. It prints out as if it were text, but it is actually
an R list object. Its parts can be probed recursively, to eventually
reveal all of the individual pieces:

<<newFla10>>=
newFmla
newFmla[[1]]
newFmla[[2]]
newFmla[[3]]
newFmla[[3]][[2]]
newFmla[[3]][[2]][[2]]
@

How could I put that information to use? Read on.

\subsection{do.call() and eval()}

In my early work as an Rchaeologist, I had noticed eval() and do.call(),
but did not understand their significance. Coming to grips with these
ideas is a critical step in the R programmer's growth, because they
separate the ``script writer'' from the ``language programmer.''
Whenever difficult problems arise in r-help, the answer almost invariably
involves do.call() or eval(). 

\subsubsection{do.call()}

Let's concentrate on do.call first. The syntax is like this

\inputencoding{latin9}\begin{lstlisting}
do.call("someRFunction", aListOfArgumentsToGoInTheParentheses)
\end{lstlisting}
\inputencoding{utf8}
It is as if we were telling R to run this:

\inputencoding{latin9}\begin{lstlisting}
someRFunction(aListOfArgumentsToGoInTheParentheses)
\end{lstlisting}
\inputencoding{utf8}
We use do.call() because it is much more flexible than calling someRFunction()
directly.

Let's consider an example that runs a regression the ordinary way,
and then with do.call. In this example, the role of ``someRFunction''
will be played by lm() and the list of arguments will be the parameters
of the regression. The regression m1 will be constructed the ordinary
way, while m2 is constructed with do.call().

<<>>=
m1 <- lm(y ~ x1*x2, data = dat)
coef(m1)
regargs <- list(formula = y ~ x1*x2, data = quote(dat))
m2 <- do.call("lm", regargs)
coef(m2)
all.equal(m1, m2)
@

The object regargs is a list of arguments that R can understand when
they are supplied to the lm function. 

do.call() is a powerful, mysterious symbol. It holds flexibility;
we can calculate ``on the language'' to create commands and then
run them. I first needed do.call() when we had a simulation project
that ran very slowly. There's a writeup in the working examples distributed
with rockchalk called stackListItems-01.R. I was using rbind() over
and over to join the results of simulation runs. Basically, the code
was like this

\inputencoding{latin9}\begin{lstlisting}
for (i in 1:10000){
   dat <- someHugeSimulation(i)
   result <- rbind(result, dat)
}
\end{lstlisting}
\inputencoding{utf8}
That will call rbind() 10000 times. I had not realized that rbind()
is time-consuming. It accesses a new chunk of memory each time it
is run. On the other hand, we could collect those results in a list,
then we can call rbind() one time to smash together all of the results.

\inputencoding{latin9}\begin{lstlisting}
for (i in 1:10000){
   mylist[[i]] <- someHugeSimulation(i)
}
result <- do.call("rbind", mylist)
\end{lstlisting}
\inputencoding{utf8}
It is much faster to run rbind() only once. It would be OK if we typed
it all out like this:

\inputencoding{latin9}\begin{lstlisting}
result <- rbind(mylist[[1]], mylist[[2]], mylist[[3]], ..., mylist[[10000])
\end{lstlisting}
\inputencoding{utf8}
But who wants to do all of that typing? How tiresome! Thanks to Erik
Iverson in r-help, I understand that

\inputencoding{latin9}\begin{lstlisting}
result <- do.call("rbind", mylist)
\end{lstlisting}
\inputencoding{utf8}
is doing the EXACT same thing. ``mylist'' is a list of arguments.
do.call is \emph{constructing} a function call from the list of arguments.
It is \emph{as if} I had actually typed rbind with 10000 arguments.

The beauty in this is that we could design a program that can assemble
the list of arguments, and also choose the function to be run, on
the fly. We are not required to literally write the function in quotes,
as in ``rbind''. We could instead have a variable that is calculated
to select one function among many, and then use do.call on that. In
a very real sense, we could write a program that can write itself
as it runs. 

From all of this (and a peek at ?call), I arrive at an Rchaeological
eureka! A call object is a quoted command plus a list of arguments
for that command. 

\subsubsection{eval()}

Where does eval() fit into the picture? do.call() manufactures a call
and executes it immediately. It is possible to arrest the process
mid stream, to capture the call without evaluating it with eval().
The terminology in the R documentation on this is difficult, mainly
because there are several different ways to create a piece of not-yet-evaluated
code. By \strong{not-yet-evaluated code}, I mean an ``unevaluated
expression'' or a ``call object'', anything that can be handled
by the eval() function. A not-yet-evaluated object holds some syntax
that can we need later. I think of do.call() as a contraction of ``eval''
and ``call''. As far as I can tell, do.call(\textquotedbl lm\textquotedbl ,
list(arguments)) is the same as eval(call(``lm'', arguments)). 

There are many ways to ask R to create the not-yet-evaluated object,
that's one of the confusing things (in my opinion). Most obviously,
call() function can be used for that. But functions like quote() and
expression() can also create not-yet-evaluated code.

Everybody has experienced an error like this:

\inputencoding{latin9}\begin{lstlisting}
> x1 + x2
Error: object 'x1' not found
\end{lstlisting}
\inputencoding{utf8}
Whenever we use a variable, the R interpreter expects to find it.
We know those variables are in the data frame dat, but the interpreter
does not know that's what we mean. But we can ask R to trust us with
the quote() function, which, basically, means ``here's some code
we will evaluate later'':

\inputencoding{latin9}\begin{lstlisting}
> mycall <- quote(x1 + x2)
\end{lstlisting}
\inputencoding{utf8}
The object ``mycall'' is an R call object, something that might
be evaluated in the future. 

When we want to evaluate ``x1 + x2'', we ask R do to so:

\inputencoding{latin9}\begin{lstlisting}
eval(mycall, dat)
\end{lstlisting}
\inputencoding{utf8}The second argument supplies the place where x1 and x2 are to be found.
R users may never have noticed the eval() function, but most will
have used the with() function, which is simply a clever re-phrasing
of the eval function's interface. It does the same thing:\inputencoding{latin9}
\begin{lstlisting}
with(dat, x1 + x2)
\end{lstlisting}
\inputencoding{utf8}
and so does \inputencoding{latin9}
\begin{lstlisting}
with(dat, eval(mycall)).
\end{lstlisting}
\inputencoding{utf8}
Now, back to the regression example we were working on before. We
can create the call object using the call() function, which requires
the name of a function and a comma-separated list of quoted arguments.
I quote each of the bits of syntax in the arguments because I don't
want R to replace ``dat'' with the actual data frame, I only want
it to remember the name.

<<>>=
m3 <- lm(y ~ x1*x2, data = dat)
coef(m3)
mycall <- call("lm", quote(y ~ x1*x2), data = quote(dat))
m4 <- eval(mycall)
coef(m4)
@

The main reason for using eval is that we can ``piece together''
commands and then run them after we have assembled all the pieces.
The nickname for this seems to be ``computing on the language.''
There is a discussion of it in the R Language Definition. It is also
surveyed in Hadley Wickham's new book, \emph{Advanced R} \citeyearpar{Wickham2015}.

Did you ever notice that you get an error if you run ``x1 + x2'',
but if you run ``y $\sim$ x1 + x2'', then there is no error? It
is as if R is wrapping your code in protective tissue, something like
quote(). However, the protective wrapper in this case is actually
the function called formula(). The R interpreter notices the symbol
$\sim$ in the syntax, and so it assumes you meant to run

\inputencoding{latin9}\begin{lstlisting}
formula(y ~ x1 + x2)
\end{lstlisting}
\inputencoding{utf8}
A formula is another kind of not-yet-evaluated thing. It is something
that we can fiddle with before evaluating. 

Create a complicated formula. Note that the lm function receives that
formula object without trouble.

<<>>=
f1 <- y ~ x1 + x2 + x3 + log(x4)
class(f1)
m5 <- lm(f1, data = dat)
coef(m5)
@

The object f1 is a formula object. Its not just a text string. Observe
it has separate pieces, just like newFmla in the example problem that
started this section.

<<>>=
f1[[1]]
f1[[2]]
f1[[3]]
f1[[3]][[1]]
f1[[3]][[2]]
f1[[3]][[3]]
@\\
Note that f1 created in this way must be a syntactically valid R formula;
it cannot include any other regression options.

\inputencoding{latin9}\begin{lstlisting}
> f1 <- y ~ x1 + x2 + x3 + log(x4), data=dat 
Error: unexpected ',' in "f1 <- y ~ x1 + x2 + x3 + log(x4),"
\end{lstlisting}
\inputencoding{utf8}
If I declare f1exp as an expression, then R does not re-interpret
it as a formula (f1exp is an unevaluated expression, the R parser
has not translated it yet). To use that as a formula in the regression,
we have to evaluate it.

<<>>=
f1exp <- expression(y ~ x1 + x2 + x3 + log(x4))
class(f1exp)
m6 <- lm(eval(f1exp), data = dat)
@

When f1exp is evaluated, what do we have? Here's the answer.

<<>>=
f1expeval <- eval(f1exp)
class(f1expeval)
all.equal(f1expeval, f1)
m7 <- lm(f1expeval, data=dat)
all.equal(coef(m5), coef(m6), coef(m7))	
@

The point here is that the pieces of an ordinary use command can be
separated and put back together again before the work of doing calculations
begins. We can edit the formula. Suppose we replace a part:

<<>>=
f1[[3]][[2]] <- quote(x1 + log(x2))
m8 <- lm(f1, data = dat)
coef(m8)
@

Now we turn back to the main theme. How is eval() used in functions?
Some functions take a lot of arguments. We may want to do a lot of
fine tuning on the arguments, before they are assembled and evaluated.
Hence, it is very important that we can keep some bits of R that are
symbolic, rather than calculated, until we need them. 

If you are looking for examples of eval() in the R source code, there
is a nice one in the beginning of the lm() function. Suppose a user
submits a command like ``lm(y \textasciitilde{} x, data = dat, x
= TRUE, y = TRUE).'' Inside lm(), the import is re-organized. Here
are the first lines of the lm() function

\inputencoding{latin9}\begin{lstlisting}[breaklines=true,numbers=left,numberstyle={\footnotesize},tabsize=4]
lm <- function (formula, data, subset, weights, na.action, method = "qr",
    model = TRUE, x = FALSE, y = FALSE, qr = TRUE, singular.ok = TRUE,
    contrasts = NULL, offset, ...)
{
    ret.x <- x
    ret.y <- y
    cl <- match.call()
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "subset", "weights", "na.action",
        "offset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
\end{lstlisting}
\inputencoding{utf8}
Now suppose we turn on the debugger and run a regression in R with
a command like this.

\inputencoding{latin9}\begin{lstlisting}
debug(lm)
m1 <- lm(y ~ x1 * x2, data = dat, x = TRUE, y = TRUE)
\end{lstlisting}
\inputencoding{utf8}
After that, we are ``in'' the function, stepping through line-by-line.
In line 8, the match.call() function is used to grab a copy of the
command that I typed. We can see that mf is exactly the same as my
command, except R has named the arguments:

\inputencoding{latin9}\begin{lstlisting}
> mf
lm(formula = y ~ x1 * x2, data = dat, x = TRUE, y = TRUE)
\end{lstlisting}
\inputencoding{utf8}
That's not just a string of letters, however. It is a call object,
a list with individual pieces that can be revised. Recall that the
first element in a call object is the name of a function, and the
following elements are the arguments. Lines 10 and 11 check the names
of mf for the presence of certain arguments, and throw away the rest.
It only wants the arguments we would be needed to run the function
model.frame. Line 12 adds an argument to the list, drop.unused.levels.
Up to that point, then, we can look at the individual pieces of mf:

\inputencoding{latin9}\begin{lstlisting}
> names(mf) 
[1] ""      "formula"       "data" [4] "drop.unused.levels"
> mf[[1]]
lm
> mf[[2]]
y ~ x1 * x2
> mf[[3]]
dat
> mf[[4]] 
[1] TRUE
\end{lstlisting}
\inputencoding{utf8}
The object mf has separate pieces that can be revised and then evaluated.
Line 13 replaces the element 1 in mf with the symbol ``model.frame''.
That's the function that will be called. Line 14 is the coup de grâce,
when the revised call ``mf'' is sent to eval. In the end, it is
\emph{as if} lm had directly submitted the command

\inputencoding{latin9}\begin{lstlisting}
mf <- model.frame(y ~ x1 * x2, data = dat, drop.unused.levels = TRUE)
\end{lstlisting}
\inputencoding{utf8}
It would not do to simply write that into the lm function, however,
because some people use variables that have names different from y,
x1, and x2, and their data objects may not be called dat. lm allows
users to input whatever they want for a formula and data, and then
lm takes what it needs to build a model frame.

\subsection{substitute()}

Most R users I know have not used substitute, except possibly if they
try to use plotmath. In the context of plotmath, the problem is as
follows. Plotmath causes the R plot functions to convert expressions
into mathematical symbols in a way that is reminiscent of \LaTeX .
For example, a command like this:

\inputencoding{latin9}\begin{lstlisting}
text(4, 4, expression(gamma))
\end{lstlisting}
\inputencoding{utf8}will draw the gamma symbol at the position (4,4). We can use paste
to combine symbolic commands and text like so:

\inputencoding{latin9}\begin{lstlisting}
text(4, 4, expression(paste(gamma, " = 7")))
\end{lstlisting}
\inputencoding{utf8}The number 7 is a nice number, but what if we want to calculate something
and insert it into the expression? Your first guess might be to insert
a function that makes a calculation, such as the mean, but this fails:

\inputencoding{latin9}\begin{lstlisting}
text(4, 4, expression(paste(gamma, mean(x))))
\end{lstlisting}
\inputencoding{utf8}In order to smuggle the result of a calculation into an expression,
some fancy footwork is required. In the help page for plotmath, examples
using the functions bquote and substitute are offered. I have found
this to be quite frustrating and difficult to do, and if one examines
my collection of examples on \url{http://pj.freefaculty.org/R/WorkingExamples},
one will find quite a few plotmath exercises.

For the particular purpose of blending expressions with calculation
results, I find the bquote function to be more easily understandable.
However, when going beyond plotmath, I expect most R programmers will
need to use substitute() instead, so I will discuss that in this section.
The plotmath help page points to syntax like this:

<<fig=T, height=3, width=4>>=
plot(1:10, seq(1,5, length.out=10), type = "n", main="Illustrating Substitute with plotmath", xlab="x", ylab="y")
text(5, 4, substitute(gamma + x1mean, list(x1mean = mean(dat$x1))))
text(5, 2, expression(paste(gamma, " plus the mean of x1")))
@

Run ?substitute and one is brought to a famous piece of Rchaeological
pottery: 
\begin{quote}
‘substitute’ returns the parse tree for the (unevaluated) expression
‘expr’, substituting any variables bound in ‘env’. 
\end{quote}
Pardon me. parse tree? We've seen expressions already, that part is
not so off putting. But ``parse tree''? Really?

This is one of those points at which being an Rchaeologist has real
benefits. We have to dig deeper, to try to understand not only what
the R programmer says, but what she is actually trying to do. The
manual page gives us some insights into the R programmer, and it is
his or her view of his or her own actions, but it doesn't necessarily
speak to how we should understand substitute(). 

For me, the only workable approach is to build up a sequence of increasingly
complicated examples. I start by creating the list of replacements.
This replacement list can have a format like this: 

<<>>=
sublist <- list(x1 = "alphabet", x2 = "zoology")
@

Suppose I have some other object in which x1 and x2 need to be replaced.
Specifically, when an expression has ``x1'', I want ``alphabet'',
and ``x2'' should become ``zoology.'' The quotes in the R code
indicate that alphabet and zoology are character strings, not other
objects that already exist. Consider replacing x1 and x2 in x1 + x2
+ log(x1) + x3:

<<>>=
substitute(expression(x1 + x2 + log(x1) + x3), sublist)
@

Note that the substitution 1) leaves other variables alone (since
they are not named in sublist) and 2) it finds all valid uses of the
symbols x1 and x2 and replaces them.

This isn't quite what I wanted, however, because the strings have
been inserted into the middle of my expression. I don't want text.
I just want symbols. I've seen the conversion from character to symbol
done with as.symbol() and with as.name(), recently I realized that
they are synonymous. I usually use as.symbol() because that name has
more intuition for me, but in Gabor's answer to my question, as.name()
is used. Using as.name(), my example would become:

<<>>=
sublist <- list(x1 = as.name("alphabet"), x2 = as.name("zoology"))
substitute(expression(x1 + x2 + log(x1) + x3), sublist)
@

\subsection{setNames and names}

Almost every R user has noticed that the elements of R lists can have
names. In a data frame, the names of the list elements are thought
of as variable names, or column names. If dat is a data frame, the
names and colnames functions return the same thing, but that's not
true for other types of objects.

<<>>=
dat <- data.frame(x1=1:10, x2=10:1, x3=rep(1:5,2), x4=gl(2,5))
colnames(dat)
names(dat)
@

After dat is created, we can change the names inside it with a very
similar approach:

<<>>=
newnames <- c("whatever","sounds","good","tome")
colnames(dat) <- newnames
colnames(dat)
@

While used interactively, this is convenient, but it is a bit tedious
because we have to create dat first, and then set the names. The setNames()
function allows us to do this in one shot. I'll paste the data frame
creating commands and the name vector in for a first try:

<<>>=
dat2 <- setNames(data.frame(x1 = rnorm(10), x2 = rnorm(10),
    x3 = rnorm(10), x4 = gl(2,5)), c("good", "names", "tough", "find"))
head(dat2, 2)
@

In order to make this more generally useful, the first step is to
take the data-frame-creating code and set it into an expression that
is not immediately evaluated (that's datcommand in what follows).
When I want the data frame to be created, I use eval(), and then the
newnames vector is put to use. 

<<>>=
newnames <- c("iVar", "uVar", "heVar", "sheVar")
datcommand <- expression(data.frame(x1=1:10, x2=10:1, x3=rep(1:5,2), x4=gl(2,5)))
eval(datcommand)
dat3 <- setNames(eval(datcommand), newnames)
@

The whole point of this exercise is that we can write code that creates
the names, and creates the data frame, and then they all come together.

What if we have just one element in a list? In Gabor's answer to my
question, there is this idiom

\inputencoding{latin9}\begin{lstlisting}
setNames(list(as.name("x1c")), "x1")))
\end{lstlisting}
\inputencoding{utf8}
Consider this from the inside out. 
\begin{enumerate}
\item as.name(``x1c'') is an R symbol object,
\item list(as.name(``x1c'')) is a list with just one object, which is
that symbol object. 
\item Use setNames(). The object has no name! We would like to name it ``x1''. 
\end{enumerate}
It is as if we had run the command list(x1 = x1c). The big difference,
of course, is that this way is much more flexible because we can calculate
replacements. 

\subsection{The Big Finish}

In the meanCenter() function in rockchalk, some predictors are mean-centered
and their names are revised. A variable named ``age'' becomes ``agec''
or ``x1'' becomes ``x1c''. So the user's regression formula that
uses variables age or x1 must be revised. This is a function that
takes a formula ``fmla'' and replaces a symbol xname with newname.

\inputencoding{latin9}\begin{lstlisting}
formulaReplace <- function(fmla, xname, newname){ 
    do.call("substitute", list(fmla, setNames(list(as.name(newname)), xname))) 
}
\end{lstlisting}
\inputencoding{utf8}
This is put to use in meanCenter(). Suppose a vector of variable names
called nc (stands for ``needs centering'') has already been calculated.
The function std() creates a centered variable.

\inputencoding{latin9}\begin{lstlisting}
newFmla <- mc$formula
for (i in seq_along(nc)){
    icenter <- std(stddat[, nc[i]])
    newname <- paste(as.character(nc[i]), "c", sep = "")
    newFmla <- formulaReplace(newFmla,  as.character(nc[i]), newname)
    nc[i] <- newname
}
\end{lstlisting}
\inputencoding{utf8}
If one has a copy of rockchalk 1.6 or newer, the evidence of the success
of this approach should be evident in the output of the command example(meanCenter). 

\section{Make Re-Usable Tools (Rather than Cutting and Pasting)}

We seek concise solutions that are generalizable. It is almost never
right to cut and paste and then make minor edits in each copy to achieve
particular purposes. This advice goes against the grain of the graduate
students that I work with. They will almost always use cut and paste
solutions. 

Here is an example of the problem. A person needed ``dummy variables''.
The code had several pages like this:

\inputencoding{latin9}\begin{lstlisting}
  if(setcorr == 1){
        corr.10 <-1
        corr.20 <-0
        corr.30 <-0
        corr.40 <-0
        corr.50 <-0
        corr.60 <-0
        corr.70 <-0
        corr.80 <-0
    }
    if(setcorr == 2){
        corr.10 <-1
        corr.20 <-1
        corr.30 <-0
        corr.40 <-0
        corr.50 <-0
        corr.60 <-0
        corr.70 <-0
        corr.80 <-0
    }
    if(setcorr == 3){
        corr.10 <-1
        corr.20 <-1
        corr.30 <-1
        corr.40 <-0
        corr.50 <-0
        corr.60 <-0
        corr.70 <-0
        corr.80 <-0
    }
\end{lstlisting}
\inputencoding{utf8}
Well, that's understandable, but a little bit embarrassing. I asked
``why do you declare these separate variables, why not make a vector?''
and ``can't you see a more succinct way to declare those things?''
The answer is ``we do it that way in SAS'' or ``this runs''. One
might suspect that the author was eager to say, ``my project is based
on 10,000 lines of R code'' in order to impress a lay audience.

If the mission is to create a vector with a certain number of 1's
at the beginning, surely either of these functions would be better:

<<>>=
biVec1 <- function(n = 3, n1s = 1) {
    c(rep(1, n1s), rep(0, n - n1s))
}

biVec2 <- function(n = 3, n1s = 1){
    x <- numeric(length = n)
    x[1:n1s] <- 1
    x
}

(corr <- biVec1(n = 8, n1s = 3))
(corr <- biVec2(n = 8, n1s = 3))
@

The cut and paste way is not wrong, exactly. Its tedious. 

I think any reasonable user should want a vector, rather than the
separate variables. But suppose the user is determined, and really
wants the individual variables named corr.10, corr.20, and so forth.
We can re-design this so that those variables will be sitting out
in the workspace after the function is run. Read help(``assign'')
and try

<<>>=
biVec3 <- function(n = 3, n1s = 1) {
    X <- c(rep(1, n1s), rep(0, n - n1s))
    xnam <- paste("corr.", 1:8, "0", sep = "")
    for(i in 1:n) assign(xnam[i], X[i], envir = .GlobalEnv) 
}
ls()
biVec3(n = 8, n1s = 3)
ls()
@

I don't think most people will actually want that kind of result,
but if they do, there is a way to get it.

If we want to turn biVec1() or biVec2() into general purpose functions,
we need to start thinking about the problem of unexpected user input.
Notice what happens if we ask for more 1's than the result vector
is supposed to contain:

\inputencoding{latin9}\begin{lstlisting}
> biVec1(n = 3, n1s = 7)
Error in rep(0, n - n1s) (from #2) : invalid 'times' argument
> biVec2(n = 3, n1s = 7)
[1] 1 1 1 1 1 1 1
\end{lstlisting}
\inputencoding{utf8}
Perhaps I prefer biVec1() because it throws an error when there is
unreasonable input, while biVec2() returns a longer vector than expected
with no error. 

If the user mistakenly enters non-integers, neither function generates
an error, and they give us unexpected output.

<<>>=
biVec1(3.3, 1.8)
biVec2(3.3, 1.8)
@

To my surprise, it is not an easy thing to ask a number if it is a
whole number (see help(``is.integer'')). For that, we introduce
a new function, is.wholenumber(), and put it to use in our new and
improved function

<<>>=
is.wholenumber <- function(x, tol = .Machine$double.eps^0.5){
 abs(x - round(x)) < tol
}

biVec1 <- function(n = 3, n1s = 1) {
    if(!(is.wholenumber(n) & is.wholenumber(n1s)))
        stop("Both n and n1s must be whole numbers (integers)")
    if(n1s > n)
        stop("n must be greater than or equal to n1s")
    c(rep(1, n1s), rep(0, n - n1s))
}
@

In all of the test cases I've tried, that works, although the real
numbers 7.0 and 4.0 are able to masquerade as integers.

\inputencoding{latin9}\begin{lstlisting}
> biVec1(3, 7)
Error in biVec1(3, 7) (from #4) : n must be greater than or equal to n1s
> biVec1(7, 3)
[1] 1 1 1 0 0 0 0
> biVec1(3.3, 4.4)
Error in biVec1(3.3, 4.4) (from #2) : 
  Both n and n1s must be whole numbers (integers)
> biVec1(7.0, 4.0)
[1] 1 1 1 1 0 0 0
> biVec1(7.0, 4.0)
[1] 1 1 1 1 0 0 0
\end{lstlisting}
\inputencoding{utf8}

\section{Function Arguments.}

While developing functions for the rockchalk package, one of the most
important learning experiences I've had is in argument management. 

One of the reasons that it is difficult to design functions is that
there is no ``mandatory manual'' on this. The R framework allows
programmer creativity, which is a good thing. But it is also a bad
thing, since every programmer has his/her own opinion about what ought
to be done. To state the obvious, argument names should be clear,
unambiguous, convenient, short, and minimally necessary. 

Beyond that, what can we say? I urge new R programmers to study the
R source code. This is part of the Rchaeological method. Study the
most frequently used functions in the R distribution. I've learned
the most from the way arguments are handled in the functions \code{hist}(),
\code{lm()}, \code{plot()} and \code{termplot()}. These functions
have had the most eyes on them and, therefore, they are most likely
to use the best approach. I wrote several functions that eventually
went into rockchalk before I realized that important fact and I regret
it (on a weekly basis).

In this section, I offer this advice.
\begin{enumerate}
\item Protect your function's calculations from the user's workspace.
\item Check argument values.
\item Design the function so that it runs with a minimum number of arguments.
\end{enumerate}

\subsection{A few tidits about arguments}

R includes several functions that are very helpful in dealing with
arguments.
\begin{description}
\item [{missing()}] Inside a function, such as \code{plotme()} we often
need to check if the user supplied a value for an argument, such as
\code{col}. If the user did not supply a value, we might construct
one from the other input

\inputencoding{latin9}\begin{lstlisting}
if (missing(col)) col <- someTediousCalculation(y, z)
\end{lstlisting}
\inputencoding{utf8}
This is usually good enough, but there is danger that the user might
have called the function like so:

\inputencoding{latin9}\begin{lstlisting}
plotme(x, y, z, col = NULL)
\end{lstlisting}
\inputencoding{utf8}
This outsmarts missing(col), which will return FALSE. If our goal
is to make sure col is something, some non-NULL thing, a more careful
argument check would be

\inputencoding{latin9}\begin{lstlisting}
if (missing(col) | is.null(col)) col <- someTediousCalculation(y, z)
\end{lstlisting}
\inputencoding{utf8}
\item [{stop()}] Halts a function and returns a message to the user. Usage
example 

\inputencoding{latin9}\begin{lstlisting}
if (isTRUE(x)) stop(paste("I'm sorry, you failed to supply the right information.")))
\end{lstlisting}
\inputencoding{utf8}
\item [{stopifnot()}] It is so frequent to run ``\code{if(isTRUE(x)) stop()}''
that they created a wrapper, \code{stopifnot(x)}.
\item [{warning()}] If you don't want the function to fail, but you need
to warn the user about a condition that seems dubious, run

\inputencoding{latin9}\begin{lstlisting}
if (isTRUE(x)) warning(paste("Are you sure the value of x ought to be", x, "?"))
\end{lstlisting}
\inputencoding{utf8}
\item [{deparse(substitute())}] Inside a function, it is often important
to find out the symbolic value of the argument that the user supplied
to the function. I don't mean the \emph{value}, but rather the symbolic
name. To see what I mean, run this

<<eval=F>>=
dat <- data.frame(myx = c(1, 2, 3), myy = c(10, 5, 1))
debug(plot.default)
plot(dat$myx, dat$myy)
@

After you hit enter, the R debugger/browser will come open and the
prompt will allow you to inspect the situation. First, type ``n''
and hit return. That takes the first step into the calculations. After
that, run \code{match.call()}. The function is somewhat ``self aware''.
It knows how it was called. You should see this

\inputencoding{latin9}\begin{lstlisting}
Browse[2]> n 
debug: localAxiis <- function(..., col, bg, pch, cex, lty, lwd) Axis(...)
Browse[2]> match.call()
plot.default(x = dat$myx, y = dat$myy)
\end{lstlisting}
\inputencoding{utf8}
You asked for \code{plot()}, R sends the work to \code{plot.default()}.
All is well.

Still inside the debugger now, type ``x'' and then ``myx'':

\inputencoding{latin9}\begin{lstlisting}
Browse[2]> x
[1] 1 2 3
Browse[2]> myx
Error: object 'myx' not found
\end{lstlisting}
\inputencoding{utf8}
Inside the function, the vector value that we passed in, which was
referred to as dat\$myx, is now called x. The thing x is a data holder,
it is used in the following calculations. \code{x} does not remember
that its passed-in value was named \code{dat\$myx} in a previous
life. It is as if \code{x} is the new identity of the same old data
vector. However, we can ask \code{x} ``what name were you known
by in your previous life?'' with this interesting idiom.

\inputencoding{latin9}\begin{lstlisting}
Browse[2]> deparse(substitute(x))
[1] "dat$myx"
\end{lstlisting}
\inputencoding{utf8}
The substitute() function, as we have already seen, is a way to replace
a letter x with a value from somewhere. Maybe it would be more obvious
to run this in 2 steps.

\inputencoding{latin9}\begin{lstlisting}
Browse[2]> xsubs <- substitute(x)
Browse[2]> xsubs
dat$myx
Browse[2]> deparse(xsubs)
[1] "dat$myx"
\end{lstlisting}
\inputencoding{utf8}
\code{substitute()} grabs the object name from whence \code{x} was
calculated. We need to convert that back to a character string so
that we can know the symbolic name of the thing that eventually became
\code{x}. This is at the outer boundary of my R comprehension, so
I better stop here.

Incidentally, you can go exploring in the debugger. Run some standards
like \code{ls()} to see what's available. If you keep hitting return,
the function will run line by line. If you get tired of hitting return,
type ``c'' and the function will run to its conclusion. To let R
know that you don't want to debug that function any more, run:

\inputencoding{latin9}\begin{lstlisting}
> undebug(plot.default)
\end{lstlisting}
\inputencoding{utf8}
If you don't do that, then the function browser will pop open every
time you try to run plot.
\item [{list(...)}] The three periods, ``...'', are a valid word. (A
period is a valid character). R scans user syntax for function arguments
and it uses whatever it can understand (formal named arguments, positional
matching), and then the other arguments are thrown into a thing called
``...''.

In a simple case, we could just pass those additional arguments to
another function, as in

\inputencoding{latin9}\begin{lstlisting}
plotme <- function(x, y, ...){
    plot(x, y, ...)
}
\end{lstlisting}
\inputencoding{utf8}
Often, it is not sufficient to simply pass on the ``...'' variables,
and we instead need to scan them and sort them for various purposes.
Inside the function, we access those variables by turning them into
a list: 

\begin{Sinput}
dots <- list(...)
\end{Sinput}

We can still pass on the dots to other functions, but the usual problem
is that some of the arguments in dots may be intended for several
different functions. Suppose we are writing a plotter for categorical
variables. We might need to receive some arguments, then run \code{table()},
then run \code{layout()}, then run \code{mosaicplot()}. The user
may have put in arguments intended for those separate functions. There's
a chance of an error, or at least a warning, if irrelevant arguments
are passed to those arguments. 

The rockchalk functions \code{plotSlopes()} has examples that do
this work. They separate the dot arguments that are needed for the
\code{predict()} function and the \code{plot()} function. Because
I was learning about arguments when I wrote those functions, I did
not take the cleanest or most direct route to do this, but they do
stand as working examples that users can review.

A function may set a number of defaults that we want to use when the
users do not supply a lot of details. In a case like that, it is useful
to remember a very handy function in R called \code{modifyList()}.
The basic idea is that we can build a list of default arguments for
a function, and then we can scan the \code{dots} list and it will
replace the defaults in the first with changes in the second. There's
an example in rockchalk's \code{pctable()} function.

\begin{Sinput}
dots <- list(...)
dotnames <- names(dots)
tableargs <- list(rv, cv, dnn = c(rvlab, cvlab))
newargs <- modifyList(tableargs, dots, keep.null = TRUE)
t1 <- do.call("table", newargs)
\end{Sinput}The dots list, by definition, is named arguments, and it is often
handy to have a copy of their names for checking whether an argument
is included in the list. In the plotSlopes() function, here's what
I ended up with:

\begin{Sinput}
validForPredict <- c("se.fit", "dispersion", "terms", "na.action",
                     "level", "pred.var", "weights")
dotsForPredict <- dotnames[dotnames %in% validForPredict]
if (length(dotsForPredict) > 0) {
    parms <- modifyList(parms, dotargs[dotsForPredict])
    dotargs[[dotsForPredict]] <- NULL
}
np <- do.call("predictCI", parms)
\end{Sinput}This scans dotnames for elements in the list of arguments that are
valid for the predict method, and after they are put onto the parms
list, then those arguments are set to NULL in the dotargs collection.
Looking at that now, I see it would have been more clear to use \code{setdiff()}
on dotargs to clean that up.

The central challenge in that work is to find out the names of the
arguments that are intended for one function, then remove them from
\code{dots} so that they are not passed to a subsequent function
that needs them. How can one know what arguments are needed? This
requires some experience, but the first step is to read the interface
documentation of the function being considered. A shortcut is to run
the function \code{formals()}, which prints out the names of the
function being considered. This can sometimes be deceptive for implementations
of generic methods. Observe that the return from the generic is not
so informative as the return from the method:

<<>>=
formals(plot)
formals(plot.default)
@

If a method is not exported, if it does not show as ``visible''
in the output from \code{methods(plot)}, then a little more fancy
footwork is required to see all of the legal arguments.

<<>>=
formals(graphics:::plot.histogram)
@
\end{description}

\subsection{Protect your function's calculations from the user's workspace.}

In the early days of using R, around 2000, it was possible for a user
to obliterate base functions. I recall working on a model about Democratic
and Republican voters. I created a variable named \code{rep} and,
after that, the \code{lm()} and \code{plot()} functions would not
work. I wrote to r-help about this and learned that my \code{rep}
had obliterated the built-in function \code{rep()}, which is vital
in many calculations. Shortly after that, R was improved to protect
functions from obliteration in that way. Today I tried to cause mischief
by re-defining rep in a number of ways and it appears to me the problem
is completely solved.

There has also been a problem that users sometime use the variable
names \code{T} and \code{F}. Recall that R allows \code{T} as an
abbreviation for \code{TRUE} and \code{F} for \code{FALSE}. R will
allow the user to re-define T and F in ways that will break calculations.
However, R will not allow the user to re-define TRUE or FALSE. Observe

\inputencoding{latin9}\begin{lstlisting}
> TRUE <- 2342 
Error in TRUE <- 2342 : invalid (do_set) left-hand side to assignment
> FALSE <- function(x,y){x + y}
Error in FALSE <- function(x, y) { :
  invalid (do_set) left-hand side to assignment
\end{lstlisting}
\inputencoding{utf8}
In order to protect our function's calculations, we are urged to fully
write out the symbols TRUE and FALSE inside functions. This way, users
cannot cause damage. 

Another danger we need to be concerned about is that our functions
might have undefined variables in them. When R encounters these undefined
variables, the lexical scoping process looks outward for values. It
may find something we don't expect. Another danger is that a function
seems to work right when we practice it in our session, but another
user gets an error because something in our workspace is missing from
theirs. The lexical scope can be convenient, but it is also dangerous.

Here the example function relies on variables \code{y} and \code{z}
that are not passed in as arguments.

\inputencoding{latin9}\begin{lstlisting}
x <- c(1, 2, 3, 4)
y <- c(8, 9, 10, 11)
z <- c(1, 1, 2, 2)
plotme <- function(x){
    plot(x, y, col = z)
}
plotme(x)
\end{lstlisting}
\inputencoding{utf8}
That lets the function go and find \code{y} and \code{z}. This will
run, but it is pretty risky. It depends on R's willingness to reach
outside the boundaries of the function to find \code{y}, and \code{z}.
If I have inadvertently altered \code{y} in the workspace, this will
end badly.

If I'm writing a function for somebody else to use, as in a package,
I believe the function should to NOT rely on variables from the environment.
I want to prevent R from filling in the gaps for users. We'd rather
give the user an error ``variable not found'' than an erroneous
calculation that ``seems'' adequate. The best case scenario is that
\code{y} and \code{z} do not exist in the workspace, so the user
will receive an error message:

\begin{Sinput}
> plotme(x)
\end{Sinput}
\begin{Schunk}
   \begin{Soutput}
Error in xy.coords(x, y, xlabel, ylabel, log) : object 'y' not found
  \end{Soutput}
\end{Schunk}

The worst case scenario is that the user has some other variables
\code{y} and z\code{} sitting about in the workspace. Those ``found''
variables are put to use with a completely unintentional result. 

As a first step toward fixing this, I propose:
\begin{quote}
Every piece of input upon which we rely should be named in the list
of arguments. 
\end{quote}
If we define the function as follows,

\inputencoding{latin9}\begin{lstlisting}
plotme <- function(x, y, z){
    plot(x, y, col = z)
}
\end{lstlisting}
\inputencoding{utf8}
the user's use of \code{plotme(x)} will result in the error

\begin{Schunk}
   \begin{Soutput}
Error in xy.coords(x, y, xlabel, ylabel, log) :
    argument "y" is missing, with no default
   \end{Soutput}
\end{Schunk}Naming all of the arguments effectively protects the function against
the accidental importation of data. 

There's a function called \code{checkUsage()} in the \code{codetools}
package that can help check functions for reliance on global variables.
First, we'll make sure the workspace is clean by removing \code{x},
\code{y}, and \code{z}.

\begin{Schunk}
   \begin{Soutput}
> rm(x, y, z)
> library(codetools) 
> checkUsage(plotme) 
<anonymous>: no visible binding for global variable 'y' (:2) 
<anonymous>: no visible binding for global variable 'z' (:2)
   \end{Soutput}
\end{Schunk}

Note that, if the variables \code{x}, \code{y}, an \code{z} exist
in the user workspace, they are global and \code{checkUsage} will
not offer a warning about them. Hence, it is very important to run
\code{checkUsage()} after cleaning up the workspace. 

checkUsage() will also offer warnings about unused variables. This
can be helpful in cleaning up a function.

\subsection{Check argument values.}

We want users to provide input arguments for all information that
the calculations will rely upon. The next step is to check their inputs
to make sure they are reasonable. 

In some cases this is very easy. If we want the user to supply a data.frame,
we can stop when the user provides something else.

<<>>=
plotme <- function(x, y, data, ...){
    if(missing(data) | !is.data.frame(data)) 
        stop(paste0("plotme: the object you supplied as data: '", 
        deparse(substitute(data)) , "' is not a data.frame"))
    plot(as.formula(paste(y,  "~",  x)), data = data, ...)
}
myx <- rnorm(10)
myy <- rnorm(10)
dat <- data.frame(myx, myy)
@

This works properly

<<eval=F>>=
plotme("myx", "myy", data = dat)
@

However, if the user supplies something else where data ought to be,
an error is returned.

\begin{Sinput}
plotme("myx", "myy", rnorm(10))
\end{Sinput}
\begin{Soutput}
Error in plotme("myx", "myy", rnorm(10)) :
  plotme: the object you supplied as data: 'rnorm(10)' is not a data.frame
\end{Soutput}

The main point here is that if we definitely know what we expect the
user to supply, then we can add a filter that checks the input's type.
That's what is.character, is.vector, is.integer, and so forth are
intended for. (In an R session, type ``is'' and then hit the TAB
key a couple of times. You'll see all the completions.

The problem is not so easy to manage when we can't be sure what the
argument type must be. Sometimes, it appears the best we can do is
pass through what the users give us and hope R can make sense out
of it. I've wrestled with R's flexibility for the naming of colors.
Suppose our plotme() function displays x and y, and we allow the user
to specify colors for the points.

<<>>=
plotme <- function(x, y, z){
    plot(x, y, col = z)
}
myx <- rnorm(10)
myy <- rnorm(10)
@

R colors can be supplied in many formats, as integers, character strings,
or more elaborate HTML-style color designators. Any of these will
work.

<<eval=F>>=
mycol <- 1:10
plotme(myx, myy, z = mycol)
@

<<eval=F>>=
mycol <- rainbow(10)
plotme(myx, myy, z = mycol)
@

<<eval=F>>=
mycol <- gray.colors(10)
plotme(myx, myy, z = mycol)
@

R does not throw an error if the user runs this, but the result is
not what we wanted, probably.

<<eval=F>>=
mycol <- rnorm(10)
plotme(myx, myy, z = mycol)
@

Suppose the user's code ends up supplying a NULL value for \code{z},
as in \code{plotme(x, y, z = NULL)}. They would not do that on purpose,
but it might be an accident. If \code{z} is calculated in their code
and something has gone wrong. To protect against that, we find this
idiom is used in the R source code quite often:

<<eval=F>>=
plotme <- function(x, y, z){
    if (missing(z) | is.null(z)) z <- rep(2, length(x))   
    plot(x, y, col = z)
}
@

If \code{z} is either not supplied by the user's command, or if the
user's command explicitly sets it to NULL, then we'll fabricate \code{z}.

\subsection{Design the function so that it runs with a minimum number of arguments.}

Take a look at the most commonly used functions in R, such as lm()
or plot.default(). These functions allow a great many arguments, but
they will deliver what the users minimally need if the they run

<<eval=F>>=
m1 <- lm(y ~ x, data = dat)
plot(y ~ x, data = dat)
@

Put the arguments that users will use most often at the front of the
argument list, and then design the function so that it will work even
if all of the other arguments are not included. There are two elements
in making this work.

\subsubsection{Specify defaults}

A graph can use any color scheme, but we might put in something colorful
for users who like a little zest in life.

<<eval=F>>=
plotme <- function(x, y, z = rainbow(length(x))){     
    plot(x, y, col = z)
}
plotme(x = rnorm(100), y = rnorm(100))
@

We could as well have put the rainbow call inside the function if
we want to keep a cleaner function declaration. 

<<eval=F>>=
plotme <- function(x, y, z){     
  if(missing(z)) mycol <- rainbow(length(x))
  plot(x, y, col = mycol)
}
plotme(x = rnorm(100), y = rnorm(100))
@

It seems to me that the R source itself tends to have more elaborate
specifications of defaults within the declaration, rather than inside
the function.

In the R source code, I noticed a peculiar-looking example in which
an argument's default depends on a function that is within the function
itself. Observe that this runs fine:

<<eval=F>>=
plotme <- function(x, y, z = getAColor(length(x)))
{
    getAColor <- function(n){
             gray.colors(n)
    }
    plot(x, y, col = z, cex = 5)
}
plotme(rnorm(20), rnorm(20))
@

That was startling to me because the nested function, \code{getAColor},
does not exist outside the function, and yet it can be used in the
definition of a default. I merely promise that \code{getAColor} will
exist when the value of \code{z} is needed and all is well. 

\subsubsection{Extract or construct what else is needed from the user input}

We can deduce information from the arguments that are passed in, we
can eliminate the need for the user to pass in other arguments. We
would never need to require both arguments x and xlab in the plot
function, for example. xlab is optional because, if the user does
not supply it, then deparse(substitute()) can find something to use
as a label. The 2010 version of plotSlopes in rockchalk required the
user to pass through many parameters that (I've since learned) can
be deduced from the other arguments.

A key element in this is getting an understanding of the information
that comes in with a standard R object. Quite a few--almost all--functions
that generate interesting objects in R provide, as a part of their
output, a copy of the call that ran the function. Inside our functions,
we can inspect that call argument to see what the users ran, and from
that we can often deduce what we need without requiring users to supply
a lot of information. 

\subsection{Return values versus attributes}

If you review the R regression functions, like lm() or glm(), you
notice that they build a list of things and pass it back to the user.
That is the most obvious way to return information.

There is another way, however, which is a little interesting. One
can mark an attribute on a return object. The R usage of attributes
comes to mind at this point because, when we are scanning for inputs,
we may need to inspect not only their values, but also their attributes.
The most commonly inspected attribute, of course, is the class of
the argument. So, if we are writing a package and we need to make
sure that the objects being passed around among functions are correct
for the purpose, then the object attributes are a very useful tool.

I don't want to write a lot about this so as to display my ignorance,
but I would point user to \code{pctable()} as an example of a rockchalk
function that uses both a return list and attributes to get the job
done. Maybe I'll fill in this example at a later time, once I figure
out the most direct way to explain it.

The only problem I find with attributes is that the printing functions
for R objects generally ``splat out'' the attributes when we don't
want them. If one puts attributes on an object, then the default printer
will show them. I've not learned all of the ins-and-outs of preventing
that, but in 2015 I became aware of it and learned that one way to
deal with it can be found in the R source code for the method \code{print.table()}. 

\section{Do This, Not That (Stub) }

R novices sometimes use Google to search for R advice and they find
it, good or bad. They may find their way to the r-help email list,
where advice is generally good, or to the StackOverflow pages for
R, which may be better. A lot of advice is offered by people like
me, who may have good intentions, but are simply not qualified to
offer advice. 

One of the few bits of advice that seems to grab widespread support
is that ``for loops are bad.'' One can write an lapply statement
in one line, while a for loop can take 3 lines. The code is shorter,
but it won't necessarily run more quickly. I recall being jarred by
this revelation in John Chambers's book, \emph{Software for Data Analysis}.
The members of the apply family (apply, lapply, sapply, etc) can make
for more readable code, but they aren't always faster. ``However,
none of the apply mechanisms changes the number of times the supplied
function is called, so serious improvements will be limited to iterating
simple calculations many times. Otherwise, the n evaluations of the
function can be expected to be the dominant fraction of the computation''\citep[213]{chambers_software_2008}.

Todo: insert discussion of stackListItems-001.

Insert alternative methods of measuring executation time and measuring
performance

Balance time spent optimizing code versus time spent running program.

\section{Suggested Chores}

I suggest the would be programmer should take on some basic challenges. 
\begin{enumerate}
\item Try to create a generic function and several methods to handle classes
of various types. If the term ``generic function'' and ``method''
cause disorientation, that means the reader is a user, not a programmer
yet. 
\item Consider developing a routine (or package) for statistical estimation
or presentation. Find several R packages and compare the R code in
them. Let me know if you agree with me about these points.
\end{enumerate}
\begin{itemize}
\item Good code is compartmentalized. Separate pieces of work are handled
by separated functions and results are returned in a well organized
way.
\item Functions should not be HUGE. 

The components of a project should be small enough so that we can
comprehend them. Generally speaking, if we are reading code and we
come to a line that uses a variable that we cannot find on the screen,
that's a problem. I used to correspond with a programmer at the University
of Michigan who said he began to feel uncomfortable when a function
filled up the entire terminal screen. 
\item People who copy and paste sections over and over in order to handle
slightly different cases are causing trouble for themselves and others.
They should think harder on ways to separate that work into re-usable
functions. 
\end{itemize}
\bibliographystyle{chicago}
\bibliography{rockchalk}

\end{document}

#LyX 2.3 created this file. For more info see http://www.lyx.org/
\lyxformat 544
\begin_document
\begin_header
\save_transient_properties true
\origin unavailable
\textclass scrartcl
\begin_preamble
%\VignetteIndexEntry{Using rockchalk}

\usepackage{Sweavel}
\usepackage{graphicx}
\usepackage{color}

\usepackage[samesize]{cancel}



\usepackage{ifthen}

\makeatletter

\renewenvironment{figure}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{figure}
 }{%
   \@float{figure}[#1]%
 }%
 \centering
}{%
 \end@float
}

\renewenvironment{table}[1][]{%
 \ifthenelse{\equal{#1}{}}{%
   \@float{table}
 }{%
   \@float{table}[#1]%
 }%
 \centering
%  \setlength{\@tempdima}{\abovecaptionskip}%
%  \setlength{\abovecaptionskip}{\belowcaptionskip}%
% \setlength{\belowcaptionskip}{\@tempdima}%
}{%
 \end@float
}


%\usepackage{listings}
% Make ordinary listings look as if they come from Sweave
\lstset{tabsize=2, breaklines=true, %,style=Rstyle}
                        fancyvrb=false,escapechar=`,language=R,%
                        %%basicstyle={\Rcolor\Sweavesize},%
                        backgroundcolor=\Rbackground,%
                        showstringspaces=false,%
                        keywordstyle=\Rcolor,%
                        commentstyle={\Rcommentcolor\ttfamily\itshape},%
                        literate={<<-}{{$\twoheadleftarrow$}}2{~}{{$\sim$}}1{<=}{{$\leq$}}2{>=}{{$\geq$}}2{^}{{$^{\scriptstyle\wedge}$}}1{==}{{=\,=}}1,%
                        alsoother={$},%
                        alsoletter={.<-},%
                        otherkeywords={!,!=,~,$,*,\&,\%/\%,\%*\%,\%\%,<-,<<-,/},%
                        escapeinside={(*}{*)}}%


% In document Latex options:
\fvset{listparameters={\setlength{\topsep}{0em}}}
\def\Sweavesize{\scriptsize} 
\def\Rcolor{\color{black}} 
\def\Rbackground{\color[gray]{0.90}}

\usepackage{dcolumn}
\usepackage{siunitx}
\end_preamble
\use_default_options true
\begin_modules
sweave
logicalmkup
\end_modules
\maintain_unincluded_children false
\language american
\language_package default
\inputencoding utf8
\fontencoding global
\font_roman "lmodern" "default"
\font_sans "lmss" "default"
\font_typewriter "lmtt" "default"
\font_math "auto" "auto"
\font_default_family default
\use_non_tex_fonts false
\font_sc false
\font_osf false
\font_sf_scale 100 100
\font_tt_scale 100 100
\use_microtype false
\use_dash_ligatures false
\graphics default
\default_output_format default
\output_sync 0
\bibtex_command default
\index_command default
\paperfontsize default
\spacing single
\use_hyperref false
\papersize default
\use_geometry true
\use_package amsmath 1
\use_package amssymb 1
\use_package cancel 1
\use_package esint 1
\use_package mathdots 1
\use_package mathtools 1
\use_package mhchem 1
\use_package stackrel 1
\use_package stmaryrd 1
\use_package undertilde 1
\cite_engine natbib
\cite_engine_type authoryear
\biblio_style plainnat
\use_bibtopic false
\use_indices false
\paperorientation portrait
\suppress_date false
\justification true
\use_refstyle 1
\use_minted 0
\index Index
\shortcut idx
\color #008000
\end_index
\leftmargin 1in
\topmargin 1in
\rightmargin 1in
\bottommargin 1in
\secnumdepth 3
\tocdepth 3
\paragraph_separation indent
\paragraph_indentation default
\is_math_indent 0
\math_numbering_side default
\quotes_style english
\dynamic_quotes 0
\papercolumns 1
\papersides 1
\paperpagestyle default
\tracking_changes false
\output_changes false
\html_math_output 0
\html_css_as_file 0
\html_be_strict false
\end_header

\begin_body

\begin_layout Title
Using rockchalk for Regression Analysis
\begin_inset Foot
status open

\begin_layout Plain Layout
In case you want to be involved in development, the source code is available
 on GitHub.
 Please browse http://github.com/pauljohn32.
 The read-only git archive address is 
\begin_inset Flex URL
status open

\begin_layout Plain Layout

git://github.com/pauljohn32/rockchalk.git
\end_layout

\end_inset

.
 Once you learn how to 
\begin_inset Quotes eld
\end_inset

clone
\begin_inset Quotes erd
\end_inset

 the repository with a git client, then we can discuss ways for you to contribut
e.
\end_layout

\end_inset


\end_layout

\begin_layout Author
Paul E.
 Johnson
\end_layout

\begin_layout Abstract
The rockchalk package includes functions for estimating regressions, extracting
 diagnostic information from them, preparing presentable tables, and creating
 helpful two and three dimensional plots.
 It is primarily intended to facilitate teachers and students who are conducting
 ordinary least squares and elementary generalized models.
 Compared to other packages that help with regression analysis, this one
 offers more-than-the-usual amount of emphasis on the analysis of regression
 models that include interaction terms.
 It includes functions that can receive a fitted model and then return standardi
zed, mean-centered, and residual-centered regression results.
 The plotting functions address some limitations of R's 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
termplot
\end_layout

\end_inset

 function.
 Version 1.8 introduces a consistent framework for the creation of 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout
newdata
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 objects and the calculation of predicted values so that the marginal effects
 of separate predictors can be easily explored.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

% In document Latex options:
\end_layout

\begin_layout Plain Layout


\backslash
fvset{listparameters={
\backslash
setlength{
\backslash
topsep}{0em}}}
\end_layout

\begin_layout Plain Layout


\backslash
SweaveOpts{ae=F,nogin=T}
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<Roptions, echo=F>>=
\end_layout

\begin_layout Plain Layout

options(device = pdf)
\end_layout

\begin_layout Plain Layout

options(width=80, prompt=" ", continue="  ")
\end_layout

\begin_layout Plain Layout

options(useFancyQuotes = FALSE) 
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Section
Introduction
\end_layout

\begin_layout Standard
This is the Spring, 2019 update of the rockchalk package.
 I offer a course in regression analysis for social and behavioral scientists
 every year.
 As the course goes on, I keep track of the difficulties that the students
 experience with R and I craft functions that facilitate their work on particula
r assignments.
 The new features in this edition are the functions 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
descriptiveTables()
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
waldt()
\end_layout

\end_inset

 (for regression reports), tools for regression simulation, including 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
genCorrelatedData3()
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
cutFancy()
\end_layout

\end_inset

, some support for common recoding support for multi-level modeling.
 Finally, the regression table-writing function the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
outreg()
\end_layout

\end_inset

 can now generate decimal-aligned columns.
 A new vignette demonstrating the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
outreg()
\end_layout

\end_inset

 function is introduced with the package.
\end_layout

\begin_layout Standard
As in previous versions, a core of the effort is development of an internally
 coherent framework that allows students to estimate regressions (usually
 with lm and glm) and then create useful summary tables and diagnostic reports.
 It is easy to create 
\begin_inset Quotes eld
\end_inset

newdata
\begin_inset Quotes erd
\end_inset

 objects and interact with fitted regressions.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
newdata()
\end_layout

\end_inset

 function will be handy for just about any kind of regression.
 For the common kinds of regression in base R and lme4, there is also a
 convenience function called 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
predictOMatic()
\end_layout

\end_inset

 that allows one to specify a regression model and then receive predicted
 values for a range of input values.
 The default output will be similar to other regression software tools that
 provide 
\begin_inset Quotes eld
\end_inset

marginal effect
\begin_inset Quotes erd
\end_inset

 estimates for regression analysis.
 
\end_layout

\begin_layout Standard
I have exerted a great deal of effort to improve the consistency of predicted
 value plots for regression analysis.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotSlopes()
\end_layout

\end_inset

 function is a plotter for linear models with interaction terms.
 It draws a plot on the screen, but it also creates an output object that
 can be subjected to a follow-up hypothesis test (by the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
testSlopes()
\end_layout

\end_inset

 function).
 There is a plot method for testSlopes objects to help with analysis of
 interactions.
 
\end_layout

\begin_layout Standard
For people who want to plot nonlinear regressions, the function 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotCurves()
\end_layout

\end_inset

 is a replacement for 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotSlopes()
\end_layout

\end_inset

.
 It can handle any sort of nonlinearity and interaction on the right hand
 side.
 Where some regression formula can cause R's 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
termplot()
\end_layout

\end_inset

 function to fail, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotCurves()
\end_layout

\end_inset

 will succeed.
\end_layout

\begin_layout Standard
The core functionality of many functions has not been altered.
 Functions to provide standardized regression coefficients, mean-centered,
 or residual-centered regression coefficients have not changed much, although
 the plots of them are improved.
 
\end_layout

\begin_layout Section
Facilitating Collection of Summary Information
\end_layout

\begin_layout Subsection
summarize: A replacement for summary
\end_layout

\begin_layout Standard
When an R function provides output that is not suitable for a term paper,
 the student must cobble together some code and assemble a customized table.
 In my opinion, R's summary() function for data frames is not adequate.
 It lacks summaries for diversity of observations.
 In addition, the output is not formatted in a way that is conducive to
 the creation of plots or tables.
 As a result, I offer the function summarize(), which separates the numeric
 from factor variables and provides an alphabetized summary.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=F>>=
\end_layout

\begin_layout Plain Layout

library(rockchalk)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Consider the results of applying summarize() to Fox's Chile data set from
 the carData package:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T>>=
\end_layout

\begin_layout Plain Layout

library(carData)
\end_layout

\begin_layout Plain Layout

data(Chile)
\end_layout

\begin_layout Plain Layout

(summChile <- summarize(Chile))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The result object is a list that includes a data frame for the numeric variables
, with named rows and (optionally) alphabetized columns, as well as a separate
 report for each factor variable.
 Users who wish to summarize only the numeric variables can run summarizeNumeric
s() instead, while others who want to summarize only factors can run summarizeFa
ctors().
 The output from summarizeFactors() is a list of factor summaries.
 
\end_layout

\begin_layout Standard
A companion function is centralValues(), which will provide only one number
 for each variable in a data frame.
 For numerics, it returns the mean, while for factor variables, it returns
 the mode.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T>>=
\end_layout

\begin_layout Plain Layout

centralValues(Chile)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Easier predictions and newdata objects
\end_layout

\begin_layout Standard
Students struggle with R's predict() methods.
 The primary challenge is in the creation of a newdata object of interesting
 values of the predictors.
 If the newdata argument, here myNewDF in 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

predict(m1, newdata = myNewDF)
\end_layout

\end_inset

 is not exactly right, the command will fail.
 If the regression formula uses functions like 
\begin_inset Quotes eld
\end_inset

as.numeric()
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

as.factor()
\begin_inset Quotes erd
\end_inset

, its almost impossible for first-time R users to get this right.
 In version 1.8, I believe I have solved the problem entirely with the functions
 newdata() and predictOMatic().
 
\end_layout

\begin_layout Standard
Let's fit an example regression with the Chile data set from the carData
 package.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T>>=
\end_layout

\begin_layout Plain Layout

m1 <- lm(statusquo ~ age + income + population + region + sex, data = Chile)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
The default output of predictOMatic() will cycle through the predictors,
 one by one.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T>>=
\end_layout

\begin_layout Plain Layout

m1pred <- predictOMatic(m1)
\end_layout

\begin_layout Plain Layout

m1pred
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The newdata() and predictOMatic() functions handle the details and allow
 the user to adjust their requests to allow for very fine grained control.
 Here are the key arguments for these functions.
\end_layout

\begin_layout Enumerate
divider.
 The name of an algorithm to select among observed values for which predictions
 are to be calculated.
 These are the possibilities.
\begin_inset Separator latexpar
\end_inset


\end_layout

\begin_deeper
\begin_layout Description
\begin_inset Quotes eld
\end_inset

seq
\begin_inset Quotes erd
\end_inset

 an evenly spaced sequence of values from low to high across the variable.
\end_layout

\begin_layout Description
\begin_inset Quotes eld
\end_inset

quantile
\begin_inset Quotes erd
\end_inset

 quantile values that eminate from the center of the variable 
\begin_inset Quotes eld
\end_inset

outward
\begin_inset Quotes erd
\end_inset

.
 
\end_layout

\begin_layout Description
\begin_inset Quotes eld
\end_inset

std.dev.
\begin_inset Quotes erd
\end_inset

 the mean plus or minus the standard deviation, or 2 standard deviations,
 and so forth.
\end_layout

\begin_layout Description
\begin_inset Quotes eld
\end_inset

table
\begin_inset Quotes erd
\end_inset

 when a variable has only a small set of possible values, this selects the
 most frequently observed values.
\end_layout

\end_deeper
\begin_layout Enumerate
n.
 The number of values to be selected.
\end_layout

\begin_layout Enumerate
predVals.
 Where the divider argument sets the default algorithm to be used, the predVals
 argument can choose variables for focus and select divider algorithms separatel
y for the variables.
 It is also allowed to declare particular values.
\end_layout

\begin_layout Standard
The user can request that particular values of the predictors are used,
 or can declare one of several algorithms for selection of focal values.
 All of these details are managed by the argument called predVals, which
 is described in the help pages (with plenty of examples).
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T>>=
\end_layout

\begin_layout Plain Layout

mypred2 <- predictOMatic(m1, predVals = c("age", "region"), n = 3)
\end_layout

\begin_layout Plain Layout

mypred2
\end_layout

\begin_layout Plain Layout

mypred3 <- predictOMatic(m1, predVals = c(age = "std.dev.", region = "table"),
 n = 3)
\end_layout

\begin_layout Plain Layout

mypred3
\end_layout

\begin_layout Plain Layout

mypred4 <- predictOMatic(m1, predVals = list(age = c(18, 30, 45), region
 = c("SA", "C","N")), n = 3)
\end_layout

\begin_layout Plain Layout

mypred4
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I've invested quite a bit of effort to make sure this works dependably with
 complicated regression formulae.
 All formulae, such as y ~ x1 + log(x2 + alpha) + ploy(x3, d) will just
 work.
 (In case one is interested to know 
\emph on
why
\emph default
 this works, the secret recipe is a new function called model.data().
 Under the hood, this required some hard work, a frustrating chain of trial
 and error that is discussed in the vignette 
\emph on
Rchaeology
\emph default
, which is distributed with this package).
\end_layout

\begin_layout Standard
The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
predictOMatic()
\end_layout

\end_inset

 function will work when the regression model provides a predict function
 that can handle the newdata argument.
 If the regression package does not provide such a function, the user should
 step back and use the newdata function to create the examplar predictor
 data frames and then calculate predicted values manually.
 The newdata() function will set all variables to center values and then
 it will create a 
\begin_inset Quotes eld
\end_inset

mix and match
\begin_inset Quotes erd
\end_inset

 combination of the ones that the user asks for.
 This sequence shows ways to ask for various 
\begin_inset Quotes eld
\end_inset

mix and match
\begin_inset Quotes erd
\end_inset

 combinations of values from age and region.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T>>=
\end_layout

\begin_layout Plain Layout

mynewdf <- newdata(m1, predVals = c("age","region"), n = 3)
\end_layout

\begin_layout Plain Layout

mynewdf
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T>>=
\end_layout

\begin_layout Plain Layout

mynewdf2 <- newdata(m1, predVals = list(age = "std.dev.", region = c("SA",
 "C","N")))
\end_layout

\begin_layout Plain Layout

mynewdf2
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T>>=
\end_layout

\begin_layout Plain Layout

mynewdf3 <- newdata(m1, predVals = list(age = c(20, 30, 40), region = c("SA",
 "C","N")))
\end_layout

\begin_layout Plain Layout

mynewdf3
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\begin_inset Newline newline
\end_inset

Of course, functions from rockchalk or any other R package can be placed
 into the process for choosing focal values.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T>>=
\end_layout

\begin_layout Plain Layout

mynewdf <- newdata(m1, predVals = list(age = getFocal(Chile$age, n = 3),
 region = getFocal(Chile$region, n = 3)))
\end_layout

\begin_layout Plain Layout

mynewdf
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\begin_inset Newline newline
\end_inset

The function getFocal() is a generic function; it will receive variables
 of different types and 
\begin_inset Quotes eld
\end_inset

do the right thing.
\begin_inset Quotes erd
\end_inset

 By default, focal values of numeric variables are quantile values.
 For factor values, the most frequently observed values are selected.
 These are customizable, as explained in the documentation.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
newdata()
\end_layout

\end_inset

 output can be used in a predict() function call as demonstrated above.
 
\end_layout

\begin_layout Standard
It would be nice if every regression model's predicted values were accompanied
 by 95% confidence intervals.
 Models fit by 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
lm()
\end_layout

\end_inset

 can supply confidence intervals, but not 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
glm()
\end_layout

\end_inset

.
 At the current time, there are many competing methods that might be used
 to calculate those intervals; predict.glm() in R's stats package avoids
 the issue entirely by not calculating intervals.
 In rockchalk-1.8, I crafted some code to calculate confidence intervals
 for glm objects using the (admittedly crude) Wald-based approximation.
 In the scale of the linear predictor, we calculate a 95% confidence interval,
 and then use the inverse link function to transform that onto the space
 of the observed response.
 In this example, I replicate an example that is an R classic, from the
 help page of predict.glm.
 The reader will note that the output includes a warning about the construction
 of the confidence interval.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T>>=
\end_layout

\begin_layout Plain Layout

df <- data.frame(ldose = rep(0:5, 2), sex = factor(rep(c("M", "F"), c(6,
 6))), 
\end_layout

\begin_layout Plain Layout

	SF.numdead = c(1, 4, 9, 13, 18, 20, 0, 2, 6, 10, 12, 16))      
\end_layout

\begin_layout Plain Layout

df$SF.numalive <-  20 - df$SF.numdead
\end_layout

\begin_layout Plain Layout

budworm.lg <- glm(cbind(SF.numdead, SF.numalive) ~ sex*ldose, data = df,  family
 = binomial)
\end_layout

\begin_layout Plain Layout

predictOMatic(budworm.lg, predVals = c(ldose = "std.dev.", sex = "table"),
 interval = "confidence")  
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Descriptive Tables
\end_layout

\begin_layout Standard
In regression-based reports, it is common for authors to present a brief
 summary table to represent the data.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
descriptiveTable()
\end_layout

\end_inset

 function was created to make that process easier and more predictable.
 The aim is to provide the usual summary statistics for the numeric predictors
 and a table of proportions for the categorical predictors.
 See Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:descriptiveTable"
plural "false"
caps "false"
noprefix "false"

\end_inset

.
 The user can customize the requested statistical summaries for numeric
 predictors.
\end_layout

\begin_layout Standard
\begin_inset Float table
placement H
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
The descriptiveTable function
\begin_inset CommandInset label
LatexCommand label
name "tab:descriptiveTable"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

descriptiveTable(m1, digits=6)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
I my opinion, the output is, well, simple and understandable.
 More importantly, it will be sufficient for a term paper, once it is converted
 into a presentable final format.
 It can be converted into HTML or LaTeX with various R packages, such as
 xtable.
 Note that because this descriptive table is derived from the fitted regression,
 it summarizes 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
only the cases
\end_layout

\end_inset

 that were actually used in the regression analysis.
 If cases were lost by listwise deletion, the deleted cases are not taken
 into account here.
\end_layout

\begin_layout Section
Better Regression Tables: Some outreg Examples.
\end_layout

\begin_layout Standard
On May 8, 2006, Dave Armstrong, who was a political science PhD student
 at University of Maryland, posted a code snippet in r-help that demonstrated
 one way to use the 
\begin_inset Quotes eld
\end_inset

cat
\begin_inset Quotes erd
\end_inset

 function from R to write \SpecialChar LaTeX
 markup.
 That gave me the idea to write a \SpecialChar LaTeX
 output scheme that would help create
 some nice looking term and research papers.
 I wanted 
\begin_inset Quotes eld
\end_inset

just enough
\begin_inset Quotes erd
\end_inset

 information, but not too much.
\end_layout

\begin_layout Standard
Since 2006, many new R packages have been introduced for the creation of
 regression tables, but I still prefer to maintain 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
outreg
\end_layout

\end_inset

.
 I fight to keep this simple, but have added some features in response to
 user requests.
 For me, the biggest question is whether the format should be 
\begin_inset Quotes eld
\end_inset

wide
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

tight
\begin_inset Quotes erd
\end_inset

.
 A tight format stacks estimates and standard errors into a single column,
 while a wide format has them in separate columns.
 In response to user requests, stars for statistical significance were made
 customizable.
 It is now allowed to directly insert vectors of parameter estimates and
 standard errors.
 In this version, a new parameter 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
centering
\end_layout

\end_inset

 is introduced for decimal-centered columns.
 
\end_layout

\begin_layout Standard
In the following, I will demonstrate some tables for lm and glm fits on
 a simulated data set.
 This new simulation function, genCorrelatedData3(), is a convenient way
 to create multiple-predictor regression data sets of arbitrary size and
 complexity, allowing for interactions and nonlinearity.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<createdata1, echo=T>>=
\end_layout

\begin_layout Plain Layout

set.seed(1234)
\end_layout

\begin_layout Plain Layout

dat <- genCorrelatedData3(N = 100, means = c(x1 = 0, x2 = 10, x3 = 0), sds
 = c(1, 2, 1), rho = c(0, 0, 0), stde = 10, beta = c(0, -3, 4, 0), verbose
 = FALSE)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\begin_inset Newline newline
\end_inset

That creates a new matrix with variables 
\begin_inset Formula $x1,$
\end_inset

 
\begin_inset Formula $x2$
\end_inset

, 
\begin_inset Formula $x3$
\end_inset

, and y.
 We run some linear regressions and then create a categorical output variable
 for a logistic regression.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<createdata1, echo=T>>=
\end_layout

\begin_layout Plain Layout

m1 <- lm(y ~ x1 + x2, data = dat)
\end_layout

\begin_layout Plain Layout

m2 <- lm(y ~ x2, data = dat)
\end_layout

\begin_layout Plain Layout

m3 <- lm(y ~ x1 + x2 + x3, data = dat)
\end_layout

\begin_layout Plain Layout

## Create categorical variant
\end_layout

\begin_layout Plain Layout

myilogit <- function(x) exp(x)/(1 + exp(x))
\end_layout

\begin_layout Plain Layout

dat$y3 <- rbinom(100, size = 1, p = myilogit(scale(dat$y)))
\end_layout

\begin_layout Plain Layout

gm1 <- glm(y3 ~ x1 + x2, data = dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The outreg examples are offered in Tables 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Tab1"

\end_inset

 through 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Combined-OLSGLM"

\end_inset

.
 Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Tab1"

\end_inset

 is the 
\begin_inset Quotes eld
\end_inset

tight format output for three models, obtained from 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
outreg(list(m1, m2, m3), centering="siunitx")
\end_layout

\end_inset

.
 On the other hand, as illustrated in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Tab2"

\end_inset

, the author can request a wide table (tight = FALSE) and provide more elaborate
 model labels and adjust the significance stars.
 In Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:Combined-OLSGLM"

\end_inset

, observe that the linear and generalized linear model output peacefully
 co-exist, side-by-side.
 This output is, in my opinion, completely acceptable for inclusion in a
 professional presentation or conference paper.
 There are some warts in this output.
 
\end_layout

\begin_layout Standard
The default output from 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
outreg
\end_layout

\end_inset

 will have left-aligned columns.
 Because users requested decimal-centered columns, the argument centering
 has been introduced.
 If 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
centering="siunitx"
\end_layout

\end_inset

 or 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
centering=TRUE
\end_layout

\end_inset

, the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
outreg
\end_layout

\end_inset

 table will be adjusted to use features in the LaTeX package 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout
siunitx
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

.
 The LaTeX package dcolumn is probably more familiar to users, and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
"dcolumn"
\end_layout

\end_inset

 is also a legal value for centering, but the results are not quite as good.
 As a 
\series bold
caution
\series default
, I hasten to mention that if a user asks for decimal-centered columns,
 the user has the duty to insert into the document preamble either 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout

\backslash
usepackage{siunitx}
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset


\begin_inset Flex Code
status open

\begin_layout Plain Layout

\backslash
usepackage{dcolumn}
\end_layout

\end_inset


\begin_inset Quotes erd
\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Decimal-centered columns in the 
\begin_inset Quotes eld
\end_inset

tight
\begin_inset Quotes erd
\end_inset

 style
\begin_inset CommandInset label
LatexCommand label
name "tab:Tab1"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<outreg10, echo = F, results=tex>>=
\begin_inset Quotes erd
\end_inset


\end_layout

\begin_layout Plain Layout

or <- outreg(list(m1, m2, m3), centering = 
\begin_inset Quotes eld
\end_inset

siunitx
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
The wide format obtained with tight = FALSE
\begin_inset CommandInset label
LatexCommand label
name "tab:Tab2"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<outreg20, results=tex, echo=F>>=
\end_layout

\begin_layout Plain Layout

or1 <- outreg(list("The First Model with a Long Title" = m1, "Another Model"
 = m3), tight = FALSE, alpha = c(0.05, 0.01, 0.001), centering = 
\begin_inset Quotes eld
\end_inset

siunitx
\begin_inset Quotes erd
\end_inset

)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Combined OLS and GLM Estimates
\begin_inset CommandInset label
LatexCommand label
name "tab:Combined-OLSGLM"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<outreg70, results=tex, echo=F>>=
\end_layout

\begin_layout Plain Layout

or2 <- outreg(list(m1,gm1), modelLabels = c("OLS:y","GLM: Categorized y"))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
I understand that some authors need to include regression tables in documents
 that are not prepared with LaTeX.
 The parameter type = 
\begin_inset Quotes eld
\end_inset

HTML
\begin_inset Quotes erd
\end_inset

 will change the output format to HTML.
 An HTML file can be displayed in a web browser and it can be imported in
 traditional 
\begin_inset Quotes eld
\end_inset

word processor
\begin_inset Quotes erd
\end_inset

 programs.
\end_layout

\begin_layout Section
Plotting Regressions with Interactions 
\end_layout

\begin_layout Subsection
Interaction in Linear Regression.
 
\end_layout

\begin_layout Standard
One of the most fundamental skills in regression analysis is the interpretation
 of interactive predictors.
 It is much easier for students to understand the effect of an interaction
 if they can create a nice plot to show how the predicted value depends
 on various values of a predictor.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotSlopes()
\end_layout

\end_inset

 function was introduced in 2010 when I was teaching a large first-year
 graduate course (more than 50 students) and it became apparent that about
 20 percent of them would not be able to manage the R coding required to
 draw several lines on a single plot.
 Unfortunately, R's termplot() function will not draw regressions involving
 interactions.
\end_layout

\begin_layout Standard
The rockchalk package has two functions to help with this, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotSlopes()
\end_layout

\end_inset

 and 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotCurves()
\end_layout

\end_inset

.
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotCurves()
\end_layout

\end_inset

 is more general, it can handle any kind of formula that the user estimates.
 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotSlopes()
\end_layout

\end_inset

 is more limited, it is only for lm objects.
 In return for that limitation, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotSlopes()
\end_layout

\end_inset

 creates an output object which can be used to conduct post-hoc hypothesis
 tests.
 
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ps05, fig=T, echo=F, include=F, height=5, width=6>>=
\end_layout

\begin_layout Plain Layout

m1ps <- plotSlopes(m1, plotx = "x2", xlab = "x2 from model m1", interval
 = "confidence", opacity = 80, col = "red",
\end_layout

\begin_layout Plain Layout

ylim = c(-70, 15), legendArgs = list(x=
\begin_inset Quotes erd
\end_inset

topright
\begin_inset Quotes erd
\end_inset

))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
includegraphics[height=4in]{rockchalk-ps05}
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
plotSlopes: Linear Model with Confidence Interval
\begin_inset CommandInset label
LatexCommand label
name "fig:ps05"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
At its most elementary level, 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotSlopes()
\end_layout

\end_inset

 is a 
\begin_inset Quotes eld
\end_inset

one step
\begin_inset Quotes erd
\end_inset

 regression line plotter.
 If the regression model includes more than one predictor, then a single
 predictor is displayed on the horizontal axis and the other predictors
 are set on their central values.
 A plot for the model m1, that was illustrated above, is presented in Figure
 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ps05"

\end_inset

.
 In rockchalk-1.8, new arguments were added to allow the 
\begin_inset Quotes eld
\end_inset

see though
\begin_inset Quotes erd
\end_inset

 confidence region.
 The command to generate Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ps05"

\end_inset

 was
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T, eval=F>>=
\end_layout

\begin_layout Plain Layout

<<ps05>>
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
I've adjusted the color and opacity to illustrate the usage of those arguments.
 The y range is adjusted to make a little extra room for the legend.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
plotSlopes()
\end_layout

\end_inset

 function is very flexible.
 All of the label, color, and scale arguments of a plot function are also
 available.
 The plotSlopes function also works well if the moderator is a categorical
 variable.
 
\end_layout

\begin_layout Standard
It is important to note that the output object, m1ps, has the information
 necessary to re-create the plotted line in the form of a 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
newdata
\end_layout

\end_inset

 data frame.
 The first few lines in the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
newdata
\end_layout

\end_inset

 object are
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

m1ps$newdata[1:3, ]
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ps09, fig=F, echo=F, height=9, width=6>>=
\end_layout

\begin_layout Plain Layout

dat$y4 <- 1 + 0.1 * dat$x1 - 6.9 * dat$x2 + 0.5 * dat$x1*dat$x2 + 0.2 * dat$x3
 + rnorm(100, m = 0, sd = 10)
\end_layout

\begin_layout Plain Layout

m4 <- lm(y4 ~ x1*x2 + x3, data = dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ps10, fig=T, echo=F, height=9, width=6>>=
\end_layout

\begin_layout Plain Layout

par(mfcol=c(2,1))
\end_layout

\begin_layout Plain Layout

m4psa <- plotSlopes(m4, plotx = "x1", modx = "x2", xlab = "x1 is a fun plotx")
\end_layout

\begin_layout Plain Layout

m4psb <- plotSlopes(m4, plotx = "x2", modx = "x1", modxVals = "std.dev.",
 xlab = "x2 is plotx", ylim = c(-120, 15),
\end_layout

\begin_layout Plain Layout

legendArgs = list(x = 
\begin_inset Quotes eld
\end_inset

topright
\begin_inset Quotes erd
\end_inset

))
\end_layout

\begin_layout Plain Layout

par(mfcol=c(1,1))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
plotSlopes Illustrated
\begin_inset CommandInset label
LatexCommand label
name "fig:ps10"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
This is more interesting if we fit a regression with an interaction term,
 such as 
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

m4 <- lm(y4 ~ x1*x2 + x3, data = dat)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
We then ask 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
plotSlopes
\end_layout

\end_inset

 to draw the predicted values using one numeric variable as the horizontal
 axis and values of another variable (a moderator) are set at particular
 values.
 Either x1 or x2 can be viewed as the 
\begin_inset Quotes eld
\end_inset

moderator
\begin_inset Quotes erd
\end_inset

 predictor, the one on which the effect of the other depends.
 In rockchalk version 1.8, the selection of values of the moderator was generaliz
ed, so that the user can specify either a function that selects values,
 or a vector of values, or the name of an algorithm.
 The default algorithm will choose quantile values, but Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ps10"

\end_inset

 demonstrates also the 
\begin_inset Quotes eld
\end_inset

std.dev.
\begin_inset Quotes erd
\end_inset

 divider algorithm.
 The code to produce that figure was
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F, echo=T>>=
\end_layout

\begin_layout Plain Layout

<<ps10>>
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
When 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
modx
\end_layout

\end_inset

 is a numeric variable, then some particular values must be selected for
 calculation of predicted value lines.
 The 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
modxVals
\end_layout

\end_inset

 argument is used to either specify moderator values or an algorithm to
 select focal values.
 By default, three hypothetical values of 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
plotx
\end_layout

\end_inset

 are selected (the quantiles 25%, 50%, and 75%).
 
\end_layout

\begin_layout Standard
If 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
modx
\end_layout

\end_inset

 is a factor variable, then the most frequently observed scores will be
 selected for consideration.
 The default display will include the regression line as well as color-coded
 points for the subgroups represented by values of the moderator.
\end_layout

\begin_layout Standard
\noindent
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=F>>=
\end_layout

\begin_layout Plain Layout

fourCat <- gl(4,25, labels=c("East","West","South", "Midwest"))
\end_layout

\begin_layout Plain Layout

dat$x4 <- sample(fourCat, 100, replace = TRUE)
\end_layout

\begin_layout Plain Layout

dat$y5 <- 1 + 0.1 * dat$x1 + contrasts(dat$x4)[dat$x4, ] %*% c(-1,1,2) +
 rnorm(100,0, sd=10)
\end_layout

\begin_layout Plain Layout

m5 <- lm (y5 ~ x1*x4 + x3, data=dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ps20, fig=T, echo=F>>=
\end_layout

\begin_layout Plain Layout

m5psa <- plotSlopes(m5, plotx = "x1", modx = "x4", xlab = "x1 is a Continuous
 Predictor", xlim = magRange(dat$x1, c(1.2,1)))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
plotSlopes with a Categorical Moderator
\begin_inset CommandInset label
LatexCommand label
name "fig:ps20"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ps21, fig=T, echo=F>>=
\end_layout

\begin_layout Plain Layout

m5psb <- plotSlopes(m5, plotx = "x1", modx = "x4", modxVals = c("West","East"),
 xlab = "x1 is a Continuous Predictor", xlim=magRange(dat$x1, c(1.2,1)),
 interval = "conf")
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
plotSlopes: the interval argument
\begin_inset CommandInset label
LatexCommand label
name "fig:ps21"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset

Suppose we have a four-valued categorical variable, 
\begin_inset Quotes eld
\end_inset

West
\begin_inset Quotes erd
\end_inset

,
\begin_inset Quotes erd
\end_inset

Midwest
\begin_inset Quotes erd
\end_inset

, 
\begin_inset Quotes eld
\end_inset

South
\begin_inset Quotes erd
\end_inset

, and 
\begin_inset Quotes eld
\end_inset

East
\begin_inset Quotes erd
\end_inset

.
 If that variable is used in an interaction in the regression model, then
 the 
\begin_inset Flex Code
status open

\begin_layout Plain Layout
\noindent
plotSlopes
\end_layout

\end_inset

 output will include four lines, one for each region.
 For example, consider Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ps20"

\end_inset

, which is created by
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

<<ps20>>
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
The categorical variable is x4.
 
\end_layout

\begin_layout Standard
It is possible to superimpose confidence intervals for many subgroups, but
 sometimes these plots start to look a little bit 
\begin_inset Quotes eld
\end_inset

busy
\begin_inset Quotes erd
\end_inset

.
 The mixing of shades in overlapping intervals may help with that problem.
 A plot that focuses on just two subgroups is presented in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ps21"

\end_inset

, which is produced by 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=F>>=
\end_layout

\begin_layout Plain Layout

<<ps21>>
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset

 
\end_layout

\begin_layout Standard
In rockchalk version 1.8, I've exerted quite a bit of effort to make sure
 that colors are chosen consistently when users remove or insert groups
 in these plots.
 The same value of the moderator should always be plotted in the same way–the
 line, points, and interval colors should not change.
 Note, for example, in Figures 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ps20"

\end_inset

 and 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ps21"

\end_inset

, the line for East is black in both plots, while the line for West is red
 in both.
 
\end_layout

\begin_layout Subsection
testSlopes, a companion of plotSlopes
\end_layout

\begin_layout Standard
The students in psychology and political science are usually interested
 in conducting some diagnostic analysis of the interactive terms.
 
\begin_inset CommandInset citation
LatexCommand citet
key "Aiken1991"
literal "true"

\end_inset

 (and later 
\begin_inset CommandInset citation
LatexCommand citealp
key "Cohen2002"
literal "true"

\end_inset

Cohen, Cohen, West, and Aiken) propose using the t test to find out if the
 effect of the 
\begin_inset Quotes eld
\end_inset

plotx
\begin_inset Quotes erd
\end_inset

 variable is statistically significantly different from zero for each particular
 value of the moderator variable.
 The new version of rockchalk declares a method plot.testSlopes that handles
 the work of plotting the interaction.
\end_layout

\begin_layout Standard
The usual case would be the following.
 First, carry out the regression analysis.
 Then run plotSlopes, and run testSlopes, and then pass that result object
 to the plot method.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<ts10, fig=T, echo=F, height=6, width=6>>=
\end_layout

\begin_layout Plain Layout

m4psats <- testSlopes(m4psa)
\end_layout

\begin_layout Plain Layout

plot(m4psats)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
testSlopes for an Interactive Model
\begin_inset CommandInset label
LatexCommand label
name "fig:ts10"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

m4 <- lm(y ~ x1*x2 + x3, data = dat)
\end_layout

\begin_layout Plain Layout

m4ps <- plotSlopes(m4, plotx = "x1", modx ="x2", xlab = "x1 is a Continuous
 Predictor")
\end_layout

\begin_layout Plain Layout

m4psats <- testSlopes(m4ps)
\end_layout

\begin_layout Plain Layout

plot(m4psats)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
The output from testSlopes will differ, depending on whether modx is numeric
 or a factor.
 If it is a factor, then the slope of the lines and the associated t-test
 for each will be presented.
 My psychology students call these 
\begin_inset Quotes eld
\end_inset

simple-slopes
\begin_inset Quotes erd
\end_inset

, following the terminology of Aiken and West.
 The general idea is that we want to know if the 
\begin_inset Quotes eld
\end_inset

combined
\begin_inset Quotes erd
\end_inset

 effect of plotx is not zero.
 For a model stated with predictors 
\begin_inset Formula $plotx_{i}$
\end_inset

 and 
\begin_inset Formula $modx_{i}$
\end_inset

as
\begin_inset Formula 
\begin{equation}
y_{i}=b_{0}+b_{plotx}plotx_{i}+b_{modx}modx_{i}+b_{plotx:modx}plotx_{i}\cdot modx_{i}+\ldots+e_{i}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\noindent
the null hypothesis would be
\end_layout

\begin_layout Standard
\noindent
\begin_inset Formula 
\begin{equation}
H_{0}:0=\hat{b}_{simple\,slope}=\hat{b}_{plotx}+\hat{b}_{plotx:modx}modx
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\noindent
If modx is a factor, then we simply calculate the slope of each line and
 the test is straight-forward.
 If modx is a numeric variable, then we confront a problem that is a bit
 more interesting.
 We don't really want to say that the simple slope is different from 0 for
 particular values of 
\begin_inset Formula $modx$
\end_inset

, but instead we want to answer the question, 
\begin_inset Quotes eld
\end_inset

for which values of the moderator would the effect of the plotx variable
 be statistically significant?
\begin_inset Quotes erd
\end_inset

.
 This necessitates the calculation of the so-called Johnson-Neyman interval
 (
\begin_inset CommandInset citation
LatexCommand citeyear
key "JohnsonNeyman1936"
literal "true"

\end_inset

), a plot of which is presented in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ts10"

\end_inset

.
\end_layout

\begin_layout Standard
The method of calculation is outlined in 
\begin_inset CommandInset citation
LatexCommand citet
key "PreacherCurren2006"
literal "true"

\end_inset

.
 The values of 
\begin_inset Formula $modx$
\end_inset

 associated with a statistically significant effect of 
\begin_inset Formula $plotx$
\end_inset

 on the outcome is determined from the computation of a T statistic for
 
\begin_inset Formula $\hat{b}_{simple\,slope}$
\end_inset

.
 The J-N interval is the set of values of 
\begin_inset Formula $modx$
\end_inset

 for which the following (quadratic equation) holds:
\begin_inset Formula 
\begin{align}
\hat{t} & =\frac{\hat{b}_{simple\,slope}}{std.err(\hat{b}_{simple\,slope})}\nonumber \\
 & =\frac{\hat{b}_{simple\,slope}}{\sqrt{\widehat{Var(\hat{b}_{plotx})}+modx^{2}\widehat{Var(\hat{b}_{plotx\cdot modx})}+2modx\widehat{Cov(\hat{b}_{plotx},\hat{b}_{plotx\cdot modx})}}}\geq T_{\frac{\alpha}{2},df}
\end{align}

\end_inset


\end_layout

\begin_layout Standard
\noindent
Suppose there are two real roots, 
\begin_inset Formula $root1$
\end_inset

 and 
\begin_inset Formula $root2$
\end_inset

.
 The values of 
\begin_inset Formula $modx$
\end_inset

 for which the slope is statistically significant may be a compact interval,
 
\begin_inset Formula $[root1,root2]$
\end_inset

, as demonstrated in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ts10"

\end_inset

, or it may two open intervals, 
\begin_inset Formula $(-\infty,root1]$
\end_inset

 and 
\begin_inset Formula $[root2,\infty)$
\end_inset

.
 I had expected almost all applications to result in that latter case, but
 the somewhat surprising case illustrated in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ts10"

\end_inset

 is not too infrequently observed.
 
\end_layout

\begin_layout Subsection
plotCurves for nonlinear predictor formulae
\end_layout

\begin_layout Standard
In the most recent revision of this package, I believe I have made the plotCurve
s() function redundant.
 Before I delete the function entirely, I'm waiting for feedback.
 There was a problem in the past.
 Some students used plotSlopes() (which drew straight lines) when they intended
 to draw curves (and should have used plotcCurves).
\end_layout

\begin_layout Standard
plotCurves() was developed to generalize the plotting capability for regression
 formulas that include nonlinear transformations.
 It was for models that have polynomials or terms that are logged (or otherwise
 transformed).
 In that sense, plotCurves() is rather similar to R's own termplot() function,
 but plotCurves() has two major advantages.
 First, it allows interactions, and second, it handles some complicated
 formulae that termplot() is not able to manage.
 
\end_layout

\begin_layout Standard
Suppose a dependent variable y5 is created according to a nonlinear process.
\begin_inset Formula 
\begin{equation}
y5_{i}=-3x1_{i}+7*log(x2)+1.1x2_{i}+2.2x1_{i}\times x2_{i}+e_{i}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=F,include=F>>=
\end_layout

\begin_layout Plain Layout

dat$y5 <- with(dat, -3*x1 + 15*log(0.1 + x2 - min(x2)) + 1.1*x2 + 8.2 *x1 *
 x2 + 10*rnorm(100))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
The estimated model is
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

m5 <- lm(y5 ~ log(x2) + x1 * x2, data = dat)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
In the new version, the function calls for plotSlopes() and plotCurves()
 are the same:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<pcps20a, fig=T, eval=F, height=4, width=6>>=
\end_layout

\begin_layout Plain Layout

m5pc <- plotCurves(m5, plotx = "x2", modx = "x1", main = 
\begin_inset Quotes eld
\end_inset

plotCurves output
\begin_inset Quotes erd
\end_inset

, ylim = c(-210, 330),
\end_layout

\begin_layout Plain Layout

legendArgs = list(x = 
\begin_inset Quotes eld
\end_inset

topleft
\begin_inset Quotes erd
\end_inset

, title = 
\begin_inset Quotes eld
\end_inset

x1
\begin_inset Quotes erd
\end_inset

, cex = 0.8))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<psps20b, fig=T, eval=T, eval=F, echo = T, height = 4, width = 6>>=
\end_layout

\begin_layout Plain Layout

m5ps <- plotSlopes(m5, plotx = "x2", modx = "x1", main = 
\begin_inset Quotes eld
\end_inset

plotSlopes outout
\begin_inset Quotes erd
\end_inset

, ylim = c(-210, 330),
\end_layout

\begin_layout Plain Layout

legendArgs = list(x = 
\begin_inset Quotes eld
\end_inset

topleft
\begin_inset Quotes erd
\end_inset

, title = 
\begin_inset Quotes eld
\end_inset

x1
\begin_inset Quotes erd
\end_inset

, cex = 0.8))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
So far as I have been able to tell, the results are the same.
 See Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:pcps20"

\end_inset

.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout

\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=T, fig=T,echo=F,height = 4, width = 6>>=
\end_layout

\begin_layout Plain Layout

<<pcps20a>>
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<eval=T, fig=T, echo=F,height = 4, width = 6>>=
\end_layout

\begin_layout Plain Layout

<<psps20b>>
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Illustrations of m5 with plotSlopes and plotCurves
\begin_inset CommandInset label
LatexCommand label
name "fig:pcps20"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout

\end_layout

\end_inset


\end_layout

\begin_layout Subsection
plotPlane
\end_layout

\begin_layout Standard
The persp() function in R works well, but its interface is too complicated
 for most elementary and intermediate R users.
 To facilitate its use for regression users, the plotPlane() function is
 offered.
\end_layout

\begin_layout Standard
The plotPlane function offers a visualization of the mutual effect of two
 predictors, whether or not the regression model is linear.
 plotPlane() is designed to work like plotCurves(), to tolerate nonlinear
 components in the regression formula.
 plotPlane() allows the depiction of a 3 dimensional curving plane that
 
\begin_inset Quotes eld
\end_inset

sits
\begin_inset Quotes erd
\end_inset

 in the cloud of data points.
 The variables that are not explicitly pictured in the plotPlane() figure
 are set to central reference values.
 Recall model m4, which used the formula 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

y4 ~ x1*x2 + x3
\end_layout

\end_inset

.
 As illustrated in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:pp100"

\end_inset

, plotCurves() presents a reasonable view of the predicted values.
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<pp100, fig=T, echo=F>>=
\end_layout

\begin_layout Plain Layout

p100 <- plotPlane(m4, plotx1 = "x1", plotx2 = "x2", phi = 10, theta = -80,
 lcol = gray(.70))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
plotPlane for the Interactive Model m4
\begin_inset CommandInset label
LatexCommand label
name "fig:pp100"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Because plotPlane() is a simple convenience wrapper for R's persp() function,
 it responds to the same customizing arguments that perp would allow.
 The arguments phi and theta will rotate the figure, for example.
 The output in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:pp100"

\end_inset

 is produced by the following.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=T,eval=F>>=
\end_layout

\begin_layout Plain Layout

<<pp100>>
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
One of the major educational benefits of the 3-D figure is that students
 can easily see that a model with a simple interaction effect is 
\emph on
not a linear model any more.
 
\emph default
We will return to that point in the discussion of mean centering in regression
 analysis.
\end_layout

\begin_layout Standard
Recall that model m5 is the rather complicated nonlinear formula 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout

log(x2*x2) + x1 * x2
\end_layout

\end_inset

.
 The plotCurves() output for that was already presented in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:pcps20"

\end_inset

.
 The three dimensional view of the same is presented in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:pp110"

\end_inset

, but with an added twist.
 The twist is that the predicted value lines from the 2-D plot functions
 can be superimposed on the plane.
 The function addLines() does the work for translating 2-D plot object onto
 the regression plane.
 
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<pp111, fig = T, echo = F, height = 5, results = hide>>=
\end_layout

\begin_layout Plain Layout

ppm5 <- plotPlane(m5, plotx1 = "x2", plotx2 = "x1", phi = 0, npp = 15, lcol
 = gray(.80))
\end_layout

\begin_layout Plain Layout

addLines(from = m5pc, to = ppm5, col = m5pc$col)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Making plotSlopes and plotPlane Work Together
\begin_inset CommandInset label
LatexCommand label
name "fig:pp110"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Section
Standardized, Mean-Centered, and Residual-Centered Regressions 
\end_layout

\begin_layout Subsection
Standardized regression
\end_layout

\begin_layout Standard
Many of us learned to conduct regression analysis with SPSS, which reported
 both the ordinary (unstandardized) regression coefficients as well as a
 column of 
\begin_inset Quotes eld
\end_inset

beta weights
\begin_inset Quotes erd
\end_inset

, the output of a 
\begin_inset Quotes eld
\end_inset

standardized
\begin_inset Quotes erd
\end_inset

 regression analysis.
 Each variable, for example 
\begin_inset Formula $x1_{i}$
\end_inset

, was replaced by an estimated 
\begin_inset Formula $Z-score:$
\end_inset

 
\begin_inset Formula $(x1_{i}-\overline{x1})/std.dev.(x1_{i}$
\end_inset

).
 Some people think these coefficients are easier to interpret (but, for
 a strong cautionary argument against them, see 
\begin_inset CommandInset citation
LatexCommand citealp
key "King1986"
literal "true"

\end_inset

).
 R offers no such thing as standardized regression, probably because this
 practice is thought to be mistaken.
 The automatic standardization of all predictors, no matter whether they
 are categorical, interaction terms, or transformed values (such as logs)
 is dangerous.
 
\end_layout

\begin_layout Standard
To illustrate that, the rockchalk introduces a function called standardize().
 Each column of the design matrix is scaled to a new variable with mean
 0 and standard deviation 1.
 The result from standardize() will be an R lm object, which will respond
 to any follow-up analysis commands.
 For example:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

m4 <- lm (y4 ~ x1 * x2, data = dat)
\end_layout

\begin_layout Plain Layout

m4s <- standardize(m4)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I doubt that a reasonable person would actually want a standardized regression
 and have tried to warn users in the output.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

summary(m4s)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Comparing Ordinary and Standardized Regression
\begin_inset CommandInset label
LatexCommand label
name "tab:stdreg10"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<stdreg10, results=tex, echo=F>>=
\end_layout

\begin_layout Plain Layout

or10 <- outreg(list(m4, m4s), tight = F, modelLabels = c("Not Standardized","Sta
ndardized"))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Subsection
Mean-centered Interaction Models
\end_layout

\begin_layout Standard
Sometimes people will fit a model like this
\begin_inset Formula 
\begin{equation}
y_{i}=b_{o}+b_{1}x1_{i}+b_{2}x2_{i}+e_{i}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\noindent
and then wonder, 
\begin_inset Quotes eld
\end_inset

is there an interaction between 
\begin_inset Formula $x1_{i}$
\end_inset

 and 
\begin_inset Formula $x2_{i}$
\end_inset

?
\begin_inset Quotes erd
\end_inset

 The natural inclination is to run this model, 
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

m1 <- lm(y ~ x1*x2)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
or its equivalent
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

m2 <- lm(y ~ x1 + x2 + x1:x2)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
Researchers have been advised that they should not run the ordinary interaction
 model without 
\begin_inset Quotes eld
\end_inset

mean-centering
\begin_inset Quotes erd
\end_inset

 the predictors (Aiken and West, 1991).
 They are advised to replace 
\begin_inset Formula $x1_{i}$
\end_inset

 with 
\begin_inset Formula $(x1_{i}-\overline{x1})$
\end_inset

 and 
\begin_inset Formula $x2_{i}$
\end_inset

 with 
\begin_inset Formula $(x2_{i}-\overline{x2})$
\end_inset

, so that the fitted model will
\begin_inset Formula 
\begin{equation}
y_{i}=b_{o}+b_{1}(x1_{i}-\overline{x1})+b_{2}(x2_{i}-\overline{x2})+b_{3}(x1_{i}-\overline{x1})(x2_{i}-\overline{x2})+e_{i}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
This is a little tedious to do in R, so I provide a function meanCenter()
 that can handle the details.
 meanCenter() will receive a model, scan it for interaction terms, and then
 center the variables that are involved in interactions.
 We previously fit the model m4, and now we center it.
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

m4mc <- meanCenter(m4)
\end_layout

\begin_layout Plain Layout

summary(m4mc)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
By default, meanCenter() will only center the variables involved in an interacti
on, and it leaves the others unchanged.
 The user can request a different treatment of the variables.
 Version 1.8 introduces the argument 
\begin_inset Quotes eld
\end_inset

terms
\begin_inset Quotes erd
\end_inset

, which allows the user to list the names of the predictors that should
 be centered.
 If the user wants all of the numeric predictors to be mean-centered, the
 usage of the argument centerOnlyInteractors would be appropriate:
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

m4mc <- meanCenter(m4, centerOnlyInteractors = FALSE)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
By default, it does not standardize while centering (but the user can request
 standardization with the argument standardize = TRUE.
 The option centerDV causes the dependent variable to be centered as well.
\end_layout

\begin_layout Subsection
Residual-centered Models
\end_layout

\begin_layout Standard
Residual-centering 
\begin_inset CommandInset citation
LatexCommand citep
key "LittleBovaird2006"
literal "true"

\end_inset

 is another adjustment that has been recommended for models that include
 interactions or squared terms.
 Like mean-centering, it is often recommended as a way to obtain smaller
 standard errors or to make estimates more numerically stable.
 Like mean centering, it causes a superficial change in the estimated coefficien
ts, but the predicted values and the regression relationship is not actually
 changed.
 Nothing of substance is altered.
\end_layout

\begin_layout Standard
The residualCenter() function is used in the same manner as meanCenter().
 The user fits an interactive model and the result object is passed to residualC
enter() like so:
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<>>=
\end_layout

\begin_layout Plain Layout

m4rc <- residualCenter(m4)
\end_layout

\begin_layout Plain Layout

summary(m4rc)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
I would explain residual-centering as follows.
 Suppose we fit the linear model, with no interaction (note, I'm calling
 the coefficients 
\begin_inset Formula $c_{j}$
\end_inset

, not 
\begin_inset Formula $b_{j}$
\end_inset

 as usual):
\end_layout

\begin_layout Standard
\begin_inset Formula 
\begin{equation}
y=c_{0}+c_{1}x1+c_{2}x2+e{}_{i}.\label{eq:rc20}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\noindent
Let's proceed as if those parameter estimates, 
\begin_inset Formula $\hat{c}_{1}$
\end_inset

, 
\begin_inset Formula $\hat{c}_{2}$
\end_inset

, are the 
\begin_inset Quotes eld
\end_inset

right ones
\begin_inset Quotes erd
\end_inset

 for our analytical purpose.
 We'd like to fit an interactive model, but protect the linear parts from
 fluctuation.
 In R, when we run the model 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout
\noindent

lm(y ~ x1 * x2)
\end_layout

\end_inset

, we are allowing all of the coefficients of all variables to fluctuate
 
\begin_inset Formula 
\begin{equation}
y_{i}=b_{o}+b_{1}x1_{i}+b_{2}x2_{i}+b_{3}x1_{i}\times x2_{i}+e_{i}\label{eq:rcwant}
\end{equation}

\end_inset

Residual centering is one way to stabilize the estimation by assuring that
 
\begin_inset Formula $\hat{b}_{1}=\hat{c}_{1}$
\end_inset

 and 
\begin_inset Formula $\hat{b}_{2}=\hat{c}_{2}$
\end_inset

.
 Only the coefficient 
\begin_inset Formula $\hat{b}_{3}$
\end_inset

 floats freely.
\end_layout

\begin_layout Standard
One of the reasons that residual-centering is so appealing is that its stabilizi
ng benefit is obtained almost by accident.
 Here is the gist of the calculations.
 First, estimate a regression in which the left hand side is the interaction
 product term:
\begin_inset Formula 
\begin{equation}
(x1_{i}\times x2_{i})=d_{0}+d_{1}x1_{i}+d_{2}x2+u_{i}\label{eq:residCentered}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\noindent
The residuals from that regression are, by definition, orthogonal to both
 
\begin_inset Formula $x1$
\end_inset

 and 
\begin_inset Formula $x2$
\end_inset

.
 Call those fitted residuals 
\begin_inset Formula $\widehat{u_{i}}$
\end_inset

.
 The we run the interactive regression, replacing the column of the predictor
 
\begin_inset Formula $x1_{i}\times x2_{i}$
\end_inset

, with 
\begin_inset Formula $\widehat{u}_{i}$
\end_inset

.
 That is to say, the model we want, equation (
\begin_inset CommandInset ref
LatexCommand ref
reference "eq:rcwant"

\end_inset

), is estimated as: 
\begin_inset Formula 
\begin{equation}
y_{i}=b_{0}+b_{1}x1_{i}+b_{2}x2_{i}+b3\widehat{u_{i}}+e_{i},\label{eq:rc10-1}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\noindent
In essence, we have taken the interaction 
\begin_inset Formula $(x1_{i}\times x2_{i})$
\end_inset

, and purged it of its parts that are linearly related to 
\begin_inset Formula $x1_{i}$
\end_inset

 and 
\begin_inset Formula $x2_{i}$
\end_inset

.
 
\end_layout

\begin_layout Standard
rockchalk 1.6 included summary, print, and predict methods for the residual-cente
red regression objects.
 It is worth mentioning that the code can handle interactions of arbitrarily
 many predictors.
 If the formula has 
\begin_inset listings
inline true
status open

\begin_layout Plain Layout
\noindent

lm(y ~ x1 * x2 * x3 * x4)
\end_layout

\end_inset

, for example, this implies many separate interactions must be calculated.
 We need to calculate residual-centered residuals for 
\begin_inset Formula $x1\cdot x2$
\end_inset

, 
\begin_inset Formula $x1\cdot x3$
\end_inset

, 
\begin_inset Formula $x1\cdot x4$
\end_inset

, 
\begin_inset Formula $x2\cdot x3$
\end_inset

 and so forth, and then use them as predictors to get centered estimates
 of terms 
\begin_inset Formula $x1\cdot x2\cdot x3$
\end_inset

, and then their centered values are predictors four term interactions.
 Aspiring R programmers who want to learn about programming with R formula
 objects might benefit from the study of the function residualCenter.R in
 the rockchalk source code.
 
\end_layout

\begin_layout Section
A Brief Analysis of Mean-Centering
\end_layout

\begin_layout Standard
We can put the tools together by making a little detour into the question
 that seems to plague every regression analyst at one time or another: What
 does that interaction term 
\emph on
really mean
\emph default
? Along the way, we will try to dispel the idea that centering somehow makes
 estimates 
\begin_inset Quotes eld
\end_inset

better
\begin_inset Quotes erd
\end_inset

 or more numerically precise.
 The primary advocates of centering as a way to deal with numerical instability
 are 
\begin_inset CommandInset citation
LatexCommand citet
key "Aiken1991"
literal "true"

\end_inset

, who integrated that advice into the very widely used regression textbook,
 
\emph on
Applied Multiple Regression/Correlation for the Behavioral Sciences
\emph default
 
\begin_inset CommandInset citation
LatexCommand citep
key "Cohen2002"
literal "true"

\end_inset

.
 They claim that the inclusion of interactions causes 
\begin_inset Quotes eld
\end_inset

inessential multicollinearity
\begin_inset Quotes erd
\end_inset

 that is alleviated by centering.
 The advice is widely followed.
 One statistics book intended for biologists observed, for example, 
\begin_inset Quotes eld
\end_inset

We support the recommendation of Aiken & West (1991) and others that multiple
 regression with interaction terms should be fitted to data with centered
 predictor values
\begin_inset Quotes erd
\end_inset

 (
\begin_inset CommandInset citation
LatexCommand citealp
key "QuinnKeough2002"
literal "true"

\end_inset

, Chapter 6).
 
\end_layout

\begin_layout Standard
Technical rebuttals have been published already (
\begin_inset CommandInset citation
LatexCommand citealp
key "Kromrey1998"
literal "true"

\end_inset

), but the matter still seems not widely understood.
 The argument is not that mean-centering (or residual-centering) is wrong,
 but rather that it is unnecessary.
 It is irrelevant, and possibly, misleading.
\end_layout

\begin_layout Standard
At the core of the matter is the fact that our uncertainty about regression
 estimates depends on our point of view.
 Please review the confidence interval in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:ps05"

\end_inset

.
 The 
\begin_inset Formula $y$
\end_inset

 axis is not even 
\begin_inset Quotes eld
\end_inset

in the picture.
\begin_inset Quotes erd
\end_inset

 Would one's appreciation of the regression's predictive line be enhanced
 if the y axis were moved into the picture? 
\end_layout

\begin_layout Standard
We can move the 
\begin_inset Formula $y$
\end_inset

 axis by centering the predictor variable.
 Suppose we replace 
\begin_inset Formula $x_{i}$
\end_inset

 with 
\begin_inset Formula $x_{i}-5$
\end_inset

 and then re-estimate the model.
 That has the simple effect of moving the 
\begin_inset Formula $y$
\end_inset

 axis 5 units to the right.
 The slope is unchanged, and the reported intercept is changed in a completely
 superficial way.
 The predicted value line, the slope estimates, the residual standard error,
 and so forth, are not changed in any substantial way.
 This is simply a matter of user convenience.
 I believe that no reasonable person can say the regression is 
\begin_inset Quotes eld
\end_inset

better
\begin_inset Quotes erd
\end_inset

 after centering 
\begin_inset Formula $x_{i}$
\end_inset

.
\end_layout

\begin_layout Standard
However, there is a superficial difference that has deceived many authors.
 Notice that the confidence interval is hourglass shaped.
 
\emph on
At the new 
\begin_inset Formula $y$
\end_inset

 axis
\emph default
, our prediction is more precise.
 If we move the y axis further to the right, into the center of the data,
 say by mean-centering (
\begin_inset Formula $x_{i}-10$
\end_inset

), we move to the position that allows an even more precise prediction.
 The estimate of the intercept's standard error will be smaller for the
 obvious reason.
 We are not actually gaining certainty, we are simply reporting our most
 favorable 
\begin_inset Quotes eld
\end_inset

snapshot
\begin_inset Quotes erd
\end_inset

 of it.
 The predicted value, and the confidence interval for any observed value
 of 
\begin_inset Formula $x_{i}$
\end_inset

 is completely unchanged by centering.
\end_layout

\begin_layout Standard
It should not surprise the reader to learn that mean-centering interactive
 predictors enhances the standard errors in the same illusory way.
 Let's work though an example to see why this is so tempting.
 Suppose the true data generating mechanism is an interaction like so
\begin_inset Formula 
\begin{equation}
y_{i}=2+0.1x1_{i}+0.1x2_{i}+0.2\cdot(x1_{i}\times x2_{i})+e_{i},
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\noindent
where 
\begin_inset Formula $e_{i}\sim N(0,300^{2})$
\end_inset

 and 
\begin_inset Formula $\rho_{x1,x2}=0.4$
\end_inset

.
 
\end_layout

\begin_layout Standard
A regression analysis that ignores the interaction, 
\end_layout

\begin_layout Standard
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

lm(y ~ x1 + x2, data = dat2)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
is reported in the first column of Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:meancenter10-1"

\end_inset

.
 I've used outreg's new alpha argument to emphasize the 
\begin_inset Quotes eld
\end_inset

really good
\begin_inset Quotes erd
\end_inset

 estimates with more stars.
 Notice that everything is 
\begin_inset Quotes eld
\end_inset

statistically significant!
\begin_inset Quotes erd
\end_inset


\end_layout

\begin_layout Standard
Unable to leave well enough alone, the researcher wonders, 
\begin_inset Quotes eld
\end_inset

is there an interaction between 
\begin_inset Formula $x1$
\end_inset

 and 
\begin_inset Formula $x2$
\end_inset

?
\begin_inset Quotes erd
\end_inset

 Run 
\begin_inset listings
inline false
status open

\begin_layout Plain Layout

lm(y ~ x1 * x2, data = dat2)
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\noindent
The second column in Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:meancenter10-1"

\end_inset

 summarizes that regression.
 Be prepared for a shock when you scan the estimates.
 Almost everybody I know has said 
\begin_inset Quotes eld
\end_inset

what the heck?
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

Holy Cow!
\begin_inset Quotes erd
\end_inset

 or 
\begin_inset Quotes eld
\end_inset

Oh My God, my great result went to Hell, I'll never get tenure!
\begin_inset Quotes erd
\end_inset

 Neither of the key variables is 
\begin_inset Quotes eld
\end_inset

statistically significant
\begin_inset Quotes erd
\end_inset

 any more.
 
\end_layout

\begin_layout Standard
\begin_inset ERT
status open

\begin_layout Plain Layout

<<echo=F, include=F>>=
\end_layout

\begin_layout Plain Layout

dat2 <- genCorrelatedData3(N=400, rho=.4, stde=300, beta=c(2,0.1,0.1,0.2))
\end_layout

\begin_layout Plain Layout

m6linear <- lm (y ~ x1 + x2, data=dat2)
\end_layout

\begin_layout Plain Layout

m6int <- lm (y ~ x1 * x2, data=dat2)
\end_layout

\begin_layout Plain Layout

m6mc <- meanCenter(m6int)
\end_layout

\begin_layout Plain Layout

m6rc <- residualCenter(m6int)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Float table
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Comparing Regressions
\begin_inset CommandInset label
LatexCommand label
name "tab:meancenter10-1"

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<mcenter10, results=tex, echo=F>>=
\end_layout

\begin_layout Plain Layout

or11 <- outreg(list(m6linear, m6int, m6mc, m6rc), tight=F, modelLabels=c("Linear
", "Interaction","Mean-centered","Residual-centered"), alpha = c(0.05, 0.01))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Cohen, et al.
 claim that the apparent instability of the coefficients is a reflection
 of 
\begin_inset Quotes eld
\end_inset

inessential collinearity,
\begin_inset Quotes erd
\end_inset

 due to the fact that 
\begin_inset Formula $x1$
\end_inset

 and 
\begin_inset Formula $x2$
\end_inset

 are correlated with the new term, 
\begin_inset Formula $x1\times x2$
\end_inset

.
 They advised their readers to 
\begin_inset Quotes eld
\end_inset

mean-center
\begin_inset Quotes erd
\end_inset

 their predictors and run the regression again.
 
\end_layout

\begin_layout Standard
Remember the hourglass shape of the confidence interval.
 By mean-centering, we are re-positioning ourself for a much more favorable
 snapshot.
 The welcoming effect of the centered estimates is found in the third column
 of Table 
\begin_inset CommandInset ref
LatexCommand ref
reference "tab:meancenter10-1"

\end_inset

.
 The point estimates in that snapshot are 
\begin_inset Quotes eld
\end_inset

significant again.
\begin_inset Quotes erd
\end_inset

 It appears we have 
\begin_inset Quotes eld
\end_inset

solved
\begin_inset Quotes erd
\end_inset

 the problem of inessential collinearity.
 
\end_layout

\begin_layout Standard
The first hint of trouble is in the fact that the coefficients of the interactiv
e effects in columns 2 and 3 are identical.
 Those coefficients are the same because they are estimates of the cross
 partial derivative 
\begin_inset Formula $\partial^{2}y/\partial x1\partial x2$
\end_inset

.
 That particular value is the same, no matter where we position the 
\begin_inset Formula $y$
\end_inset

 axis, as it should be.
 Note as well that the root mean square and 
\begin_inset Formula $R^{2}$
\end_inset

 estimates are identical.
 Everything that we expect to remain the same is the same.
 Except for the fact that the slopes and their hypothesis tests seem better
 in the centered model, we would think there is nothing interesting here.
\end_layout

\begin_layout Standard
Here's a puzzle for you.
 Consider Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:pscentering"

\end_inset

, which shows the predicted values and confidence intervals from the centered
 and uncentered regressions.
 Is there any substantively important difference between these two regressions?
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<pscenter, fig=T, echo=F, include = F, height = 10, width = 5, results
 = hide>>=
\end_layout

\begin_layout Plain Layout

par(mfcol = c(2, 1))
\end_layout

\begin_layout Plain Layout

plotSlopes(m6int, plotx = "x1", modx = "x2", modxVals = "std.dev.",  n = 2,
 interval = "confidence", main= "Not Centered")
\end_layout

\begin_layout Plain Layout

plotSlopes(m6mc, plotx = "x1c", modx = "x2c", modxVals = "std.dev.", n = 2,
 interval = "confidence", main = "Mean Centered")
\end_layout

\begin_layout Plain Layout

par(mfcol = c(1, 1))
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
includegraphics[height=8in]{rockchalk-pscenter.pdf}
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
plotSlopes for the centered and non-centered regressions
\begin_inset CommandInset label
LatexCommand label
name "fig:pscentering"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
Perhaps the 2-D plots are not persuasive.
 Don't stop yet.
 In the 3-D plots will help quite a bit.
 We have not yet grasped the most fundamental changed caused by the insertion
 of the interaction term.
 When we insert 
\begin_inset Formula $x1_{i}\times x2_{i}$
\end_inset

, we change the fudamental nature of the regression surface.
 The surface of the fitted model is no longer a 
\begin_inset Quotes eld
\end_inset

flat
\begin_inset Quotes erd
\end_inset

 plane, but rather it is a curving surface.
 
\end_layout

\begin_layout Standard
I've assembled 3-D plots in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:mcenter30"

\end_inset

.
 We see that the ordinary interaction, mean-centered, and residual-centered
 models produce identical predicted values! The only difference in the figures
 is that the axes of the predictors have been re-scaled, so that the 
\begin_inset Formula $y$
\end_inset

 axis is (implicitly) re-positioned.
 Now that we understand the situation, we could play around with the data
 and move the axis back and forth until we arrive at a position that is
 most favorable to our interpretation.
 
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<mcenter50, fig=T,echo=FALSE, height=5, width=7>>=
\end_layout

\begin_layout Plain Layout

op <- par(no.readonly = TRUE)
\end_layout

\begin_layout Plain Layout

par(mfcol=c(2,2))
\end_layout

\begin_layout Plain Layout

par(mar=c(2,2,2,1))
\end_layout

\begin_layout Plain Layout

plotPlane(m6linear, plotx1="x1", plotx2="x2", plotPoints=FALSE, main="Linear",
 ticktype="detailed")
\end_layout

\begin_layout Plain Layout

plotPlane(m6int, plotx1="x1", plotx2="x2", plotPoints=FALSE, main="Interaction:
 Not Centered", ticktype="detailed")
\end_layout

\begin_layout Plain Layout

plotPlane(m6mc, plotx1="x1c", plotx2="x2c", plotPoints=FALSE, main="Mean-centere
d", ticktype="detailed")
\end_layout

\begin_layout Plain Layout

plotPlane(m6rc, plotx1="x1", plotx2="x2", plotPoints=FALSE, main="Residual-cente
red", ticktype="detailed")
\end_layout

\begin_layout Plain Layout

par(op)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Predicted Planes from Centered and Uncentered Fits Identical
\begin_inset CommandInset label
LatexCommand label
name "fig:mcenter30"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The regression coefficient estimates are snapshots, each summarizing the
 curvature at one particular point in a curving surface.
 It seems quite apparent in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:mcenter30"

\end_inset

 that the models are identical, and yet we receive different regression
 reports from different spots.
 The non-centered data offers us the slope estimate from the 
\begin_inset Quotes eld
\end_inset

front-left
\begin_inset Quotes erd
\end_inset

 part of the graph.
 Mean-centered estimates report on the slope in the middle of the graph.
 In the rockchalk examples folder, one can find a file called 
\begin_inset Quotes eld
\end_inset

centeredRegression.R
\begin_inset Quotes erd
\end_inset

 that walks through this argument step by step.
 
\end_layout

\begin_layout Standard
What about residual-centering? Because the transformation that it employs
 is more abstract, I initially thought it was actually a different model.
 And yet it is not.
 The residual-centered model is completely equivalent to the ordinary interactio
n model and the mean-centered model.
 For a given combination of the input values, the predicted values and confidenc
e intervals are the same.
 The predicted values of the ordinary interactive model, the mean-centered
 model, and the residual-centered models are illustrated in Figure 
\begin_inset CommandInset ref
LatexCommand ref
reference "fig:rcenter40"

\end_inset

.
 
\end_layout

\begin_layout Standard
\begin_inset Float figure
wide false
sideways false
status open

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout

<<rcenter40, fig=T,echo=FALSE, include=F, height=4, width=8>>=
\end_layout

\begin_layout Plain Layout

dat3 <- centerNumerics(dat2)
\end_layout

\begin_layout Plain Layout

##m6mcpred <- fitted(m6mc) ##
\end_layout

\begin_layout Plain Layout

m6mcpred <- predict(m6mc, newdata=dat3)
\end_layout

\begin_layout Plain Layout

##m6rcpred <- fitted(m6rc) ##
\end_layout

\begin_layout Plain Layout

m6rcpred <- predict(m6rc, newdata=dat3)
\end_layout

\begin_layout Plain Layout

##m6intpred <- fitted(m6int) ##
\end_layout

\begin_layout Plain Layout

m6intpred <- predict(m6int, newdata=dat3)
\end_layout

\begin_layout Plain Layout

op <- par(no.readonly = TRUE)
\end_layout

\begin_layout Plain Layout

par(mfcol=c(1,2))
\end_layout

\begin_layout Plain Layout

plot(m6intpred, m6rcpred, main="", xlab="Predictions of Uncentered Interaction",
 ylab="Residual-centered Predictions")
\end_layout

\begin_layout Plain Layout

predcor <- round(cor(m6intpred, m6rcpred),3)
\end_layout

\begin_layout Plain Layout

legend("topleft", legend=c(paste0("Correlation=", predcor)))
\end_layout

\begin_layout Plain Layout

plot(m6mcpred, m6rcpred, main="", xlab="Mean-centered Predictions", ylab
 = "Residual-centered Predictions")
\end_layout

\begin_layout Plain Layout

predcor <- round(cor(m6mcpred, m6rcpred),3)
\end_layout

\begin_layout Plain Layout

legend("topleft", legend=c(paste0("Correlation=", predcor)))
\end_layout

\begin_layout Plain Layout

par(op)
\end_layout

\begin_layout Plain Layout

@
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset ERT
status open

\begin_layout Plain Layout


\backslash
includegraphics[width = 6.5in]{rockchalk-rcenter40}
\end_layout

\end_inset


\end_layout

\begin_layout Plain Layout
\begin_inset Caption Standard

\begin_layout Plain Layout
Predicted Values of Mean and Residual-centered Models
\begin_inset CommandInset label
LatexCommand label
name "fig:rcenter40"

\end_inset


\end_layout

\end_inset


\end_layout

\end_inset


\end_layout

\begin_layout Standard
The take-away point from this is that there is no free lunch in regression
 analysis.
 If re-scaling a variable by adding or subtracting a constant seems to change
 a result, one should be cautious and suspect an error.
 
\end_layout

\begin_layout Standard
In order to drive the point home, I'd like to show that it is possible to
 translate between the estimates of any one of these fitted models and the
 estimates of the others.
 The ordinary model is
\begin_inset Formula 
\begin{equation}
y_{i}=b_{0}+b_{1}x1_{i}+b_{2}x2_{i}+b_{3}(x1_{i}\times x2_{i})+e1_{i}\label{eq:int}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
The mean-centered model is 
\begin_inset Formula 
\begin{equation}
y_{i}=c_{0}+c_{1}(x1_{i}-\overline{x1})+c_{2}(x2_{i}-\overline{x2})+c_{3}(x1_{i}-\overline{x1})\cdot(x2_{i}-\overline{x2})+e2_{i}\label{eq:mc1}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
In order to compare with equation 
\begin_inset CommandInset ref
LatexCommand ref
reference "eq:int"

\end_inset

, we would re-arrange like so
\end_layout

\begin_layout Standard
\begin_inset Formula 
\begin{equation}
y_{i}=c_{0}+c_{1}(x1_{i})-c_{1}\overline{x1}+c_{2}(x2_{i})-c_{2}\overline{x2}+c_{3}(x1_{i}x2_{i}+\overline{x1}\overline{x2}-\overline{x1}x2_{i}-\overline{x2}x1_{i})+e2_{i}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Formula 
\begin{equation}
y_{i}=c_{0}+c_{1}(x1_{i})-c_{1}\overline{x1}+c_{2}(x2_{i})-c_{2}\overline{x2}+c_{3}(x1_{i}x2_{i})+c_{3}\overline{x1}\overline{x2}-c_{3}\overline{x1}x2_{i}-c_{3}\overline{x2}x1_{i})+e2_{i}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\begin_inset Formula 
\begin{equation}
y_{i}=\{c_{0}-c_{1}\overline{x1}-c_{2}\overline{x2}+c_{3}\overline{x1}\overline{x2}\}+\{c_{1}-c_{3}\overline{x2}\}x1_{i}+\{c_{2}-c_{3}\overline{x1}\}x2_{i}+c_{3}(x1_{i}x2_{i})+e2_{i}\label{eq:mc3}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
One can then compare the parameter estimates from equations 
\begin_inset CommandInset ref
LatexCommand ref
reference "eq:int"

\end_inset

 and 
\begin_inset CommandInset ref
LatexCommand ref
reference "eq:mc3"

\end_inset

.
 Both 
\begin_inset CommandInset ref
LatexCommand ref
reference "eq:int"

\end_inset

 and 
\begin_inset CommandInset ref
LatexCommand ref
reference "eq:mc3"

\end_inset

 include a single parameter times 
\begin_inset Formula $(x1_{i}x2_{i}),$
\end_inset

 leading one to expect that the estimate 
\begin_inset Formula $\hat{b}_{3}$
\end_inset

 should be equal to the estimate of 
\begin_inset Formula $\hat{c}_{3}$
\end_inset

, and they are! Less obviously, one can use the fitted coefficients from
 either model to deduce the fitted coefficients from the other.
 The following equalities describe that relationship.
\begin_inset Formula 
\begin{eqnarray}
\hat{b}_{0} & = & \hat{c}_{0}-\hat{c}_{1}\overline{x1}-\hat{c}_{2}\overline{x2}+\hat{c}_{3}\overline{x1}\overline{x2}\\
\hat{b}_{1} & = & \hat{c}_{1}-\hat{c}_{3}\overline{x2}\\
\hat{b}_{2} & = & \hat{c}_{2}-\hat{c}_{3}\overline{x1}\\
\hat{b}_{3} & = & \hat{c}_{3}
\end{eqnarray}

\end_inset

The estimated fit of equation 
\begin_inset CommandInset ref
LatexCommand ref
reference "eq:mc1"

\end_inset

 would provide estimated coefficients 
\begin_inset Formula $\hat{c}_{j}$
\end_inset

, 
\begin_inset Formula $j=0,...,3$
\end_inset

, which would then be used to calculate the estimates from the non-centered
 model.
\end_layout

\begin_layout Standard
The estimation of the residual-centered model requires two steps.
 The residual from this regression model
\end_layout

\begin_layout Standard
\noindent
\begin_inset Formula 
\begin{equation}
\widehat{(x1_{i}\times x2_{i})}=\hat{d}_{0}+\hat{d}_{1}x1_{i}+\hat{d}_{2}x2_{i}.
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\noindent
is 
\begin_inset Formula 
\begin{equation}
\widehat{u}_{i}=(x1_{i}\times x2_{i})-\widehat{(x1_{i}\times x2_{i})},
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
\noindent
which is inserted into equation 
\begin_inset CommandInset ref
LatexCommand ref
reference "eq:int"

\end_inset

.
 
\end_layout

\begin_layout Standard
\begin_inset Formula 
\begin{equation}
y_{i}=h_{0}+h_{1}x1_{i}+h_{2}x2_{i}+h_{3}\{x1_{i}\times x2_{i}-\widehat{x1_{i}\times x2_{i}}\}+e3_{i}\label{eq:rc1}
\end{equation}

\end_inset


\end_layout

\begin_layout Standard
Replacing 
\begin_inset Formula $\widehat{x1_{i}\times x2_{i}}$
\end_inset

 with 
\begin_inset Formula $\hat{d}_{0}+\hat{d}_{1}x1_{i}+\hat{d}_{2}x2_{i}$
\end_inset

, 
\begin_inset CommandInset ref
LatexCommand ref
reference "eq:rc1"

\end_inset

 becomes
\end_layout

\begin_layout Standard
\begin_inset Formula 
\begin{eqnarray}
y_{i} & = & h_{0}+h_{1}x1_{i}+h_{2}x2_{i}+h_{3}\{x1_{i}\times x2_{i}-\hat{d}_{0}-\hat{d}_{1}x1_{i}-\hat{d}_{2}x2_{i}\}+e3_{i}\\
 & = & h_{0}+h_{1}x1_{i}+h_{2}x2_{i}+h_{3}\{x1_{i}\times x2_{i}\}-h_{3}\hat{d}_{0}-h_{3}\hat{d}_{1}x1_{i}-h_{3}\hat{d}_{2}x2_{i}\}+e3_{i}\\
 &  & \{h_{0}-h_{3}\hat{d}_{0}\}+\{h_{1}-h_{3}\hat{d}_{1}\}x1_{i}+\{h_{2}-h_{3}\hat{d}_{2}\}x2_{i}+h_{3}\{x1_{i}\times x2_{i}\}+e3_{i}
\end{eqnarray}

\end_inset


\end_layout

\begin_layout Standard
As in the previous comparison, we can translate coefficient estimates between
 the ordinary specification and the residual-centered model.
 The coefficient estimated for the product term, 
\begin_inset Formula $\hat{h}_{3}$
\end_inset

, should be equal to 
\begin_inset Formula $\hat{b}_{3}$
\end_inset

 and 
\begin_inset Formula $\hat{c}_{3}$
\end_inset

 (and it is!).
 If we fit the residual centered model, 
\begin_inset CommandInset ref
LatexCommand ref
reference "eq:rc1"

\end_inset

, we can re-generate the coefficients of the other models like so: 
\begin_inset Formula 
\begin{eqnarray}
\hat{b}_{0} & =\hat{c}_{0}-\hat{c}_{1}\overline{x1}-\hat{c}_{2}\overline{x2}+\hat{c}_{3}\overline{x1}\overline{x2}= & h_{0}-h_{3}\hat{d}_{0}\\
\hat{b}_{1} & =\hat{c}_{1}-\hat{c}_{3}\overline{x2}= & h_{1}-h_{3}\hat{d}_{1}\\
\hat{b}_{2} & =\hat{c}_{2}-\hat{c}_{3}\overline{x1}= & h_{2}-h_{3}\hat{d}_{2}
\end{eqnarray}

\end_inset


\end_layout

\begin_layout Standard
From the preceding, it should be clear enough that the three models are
 equivalent.
\end_layout

\begin_layout Section
Conclusion
\end_layout

\begin_layout Standard
The rockchalk package is offered as a system of support for teachers and
 students of regression analysis.
 It should help with the preparation of plots, summary tables, and other
 diagnostics.
 
\end_layout

\begin_layout Standard
A number of functions are currently offered in this package that have not
 been emphasized in this writeup.
 I would draw the reader to the help pages for these functions
\end_layout

\begin_layout Description
combineLevels a recoder for factor variables
\end_layout

\begin_layout Description
mcDiagnose a one stop shop for multicollinearity diagnostic information
\end_layout

\begin_layout Description
getDeltaRsquare calculate the change in the 
\begin_inset Formula $R^{2}$
\end_inset

 that results from the omission of each variable.
 This is the squared semi-partial correlation coefficient.
\end_layout

\begin_layout Description
getPartialCor calculates the partial correlation matrix of the predictors
 in a regression model
\end_layout

\begin_layout Description
lazyCor
\begin_inset space ~
\end_inset

and
\begin_inset space ~
\end_inset

lazyCov convenient ways to create correlation and covariance matrices that
 are needed in many simulation exercises
\end_layout

\begin_layout Description
mvrnorm a slightly improved version of the MASS package's multivariate normal
 generator.
\end_layout

\begin_layout Standard
\begin_inset CommandInset bibtex
LatexCommand bibtex
bibfiles "rockchalk"
options "chicago"

\end_inset


\end_layout

\end_body
\end_document

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{summarizeNumerics}
\alias{summarizeNumerics}
\title{Extracts numeric variables and presents an summary in
a workable format.}
\usage{
summarizeNumerics(dat, alphaSort = FALSE, probs = c(0, 0.5, 1),
  stats = c("mean", "sd", "skewness", "kurtosis", "nobs", "nmiss"),
  na.rm = TRUE, unbiased = TRUE, digits = 2)
}
\arguments{
\item{dat}{a data frame or a matrix}

\item{alphaSort}{If TRUE, the columns are re-organized in
alphabetical order. If FALSE, they are presented in the
original order.}

\item{probs}{Controls calculation of quantiles (see the R
\code{quantile} function's \code{probs} argument). If FALSE or
NULL, no quantile estimates are provided. Default is
\code{c("min" = 0, "med" = 0.5, "max" = 1.0)}, which will
appear in output as \code{c("min", "med", "max")}. Other
values between 0 and 1 are allowed. For example, \code{c(0.3, 0.7)}
will appear in output as \code{pctile_30\%} and
\code{pctile_70\%}.}

\item{stats}{A vector including any of these: \code{c("min",
"med", "max", "mean", "sd", "var", "skewness", "kurtosis",
"nobs", "nmiss")}. Default includes all except \code{var}.
"nobs" means number of observations with non-missing, finite
scores (not NA, NaN, -Inf, or Inf). "nmiss" is the number of
cases with values of NA. If FALSE or NULL, provide none of
these.}

\item{na.rm}{default TRUE. Should missing data be removed to
calculate summaries?}

\item{unbiased}{If TRUE (default), skewness and kurtosis are
calculated with biased corrected (N-1) divisor in the standard
deviation.}

\item{digits}{Number of digits reported after decimal
point. Default is 2}
}
\value{
a data.frame with one column per summary element (rows are
    the variables).
}
\description{
Finds the numeric variables, and ignores the others. (See
\code{summarizeFactors} for a function that handles non-numeric
variables). It will provide quantiles (specified \code{probs} as
well as other summary statistics, specified \code{stats}.  Results
are returned in a data frame. The main benefits from this compared
to R's default summary are 1) more summary information is returned
for each variable (dispersion), 2) the results are returned in a
form that is easy to use in further analysis, 3) the variables in
the output may be alphabetized.
}
\seealso{
summarize and summarizeFactors
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictOMatic.R
\name{focalVals}
\alias{focalVals}
\title{Create a focal value vector.}
\usage{
focalVals(x, divider = "quantile", n = 3)
}
\arguments{
\item{x}{The input variable may be numeric or a factor.}

\item{divider}{Either a quoted string name of an algorithm or a
function. Default = "quantile" for numeric variables, "table" for
factors. Other valid values: "seq" for an evenly spaced sequence
from minimum to maximum, "std.dev." for a sequence that has the
mean at the center and values on either side that are proportional
to the standard deviation.}

\item{n}{Desired number of focal values.}
}
\value{
A named vector of focal values selected from a variable. The
values of the names should be informative and useful for plotting or
other diagnostic work.
}
\description{
This selects some values of a variable and creates a new "focal vector"
from them. Can use one "divider" algorithm, to be selected by name.
}
\details{
This is a "wrapper" (or convenience) function that re-directs work
to other functions. The functions that do the work to select the
focal values for types ("table", "quantile", "std.dev.", "seq") are
(cutByTable(), cutByQuantile(), cutBySD(), and plotSeq())

The built-in R function \code{pretty()}
works as of rockchalk 1.7.2. Any function that accepts an argument
n will work, as long as it creates a vector of values.
}
\seealso{
\code{predictOMatic} \code{newdata}
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcDiagnose.R
\name{lmAuxiliary}
\alias{lmAuxiliary}
\title{Estimate leave-one-variable-out regressions}
\usage{
lmAuxiliary(model)
}
\arguments{
\item{model}{a fitted regression model}
}
\value{
a list of fitted regressions, one for each omitted variable.
}
\description{
This is a convenience for analysis of multicollinearity in regression.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testSlopes.R
\name{plot.testSlopes}
\alias{plot.testSlopes}
\title{Plot testSlopes objects}
\usage{
\method{plot}{testSlopes}(x, ..., shade = TRUE, col = rgb(1, 0, 0,
  0.1))
}
\arguments{
\item{x}{A testSlopes object.}

\item{...}{Additional arguments that are ignored currently.}

\item{shade}{Optional. Create colored polygon for significant regions.}

\item{col}{Optional. Color of the shaded area. Default transparent pink.}
}
\value{
\code{NULL}
}
\description{
plot.testSlopes is a method for the
generic function plot. It has been revised so that it creates a plot
illustrating the marginal effect, using the
Johnson-Neyman interval calculations to highlight the
"statistically significantly different from zero" slopes.
}
\author{
<pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meanCenter.R
\name{meanCenter}
\alias{meanCenter}
\alias{meanCenter.default}
\title{meanCenter}
\usage{
meanCenter(model, centerOnlyInteractors = TRUE, centerDV = FALSE,
  standardize = FALSE, terms = NULL)

\method{meanCenter}{default}(model, centerOnlyInteractors = TRUE,
  centerDV = FALSE, standardize = FALSE, terms = NULL)
}
\arguments{
\item{model}{a fitted regression model (presumably from lm)}

\item{centerOnlyInteractors}{Default TRUE. If FALSE, all numeric
predictors in the regression data frame are centered before
the regression is conducted.}

\item{centerDV}{Default FALSE. Should the dependent variable be
centered? Do not set this option to TRUE unless the dependent
variable is a numeric variable. Otherwise, it is an error.}

\item{standardize}{Default FALSE. Instead of simply mean-centering
the variables, should they also be "standardized" by first
mean-centering and then dividing by the estimated standard
deviation.}

\item{terms}{Optional. A vector of variable names to be
centered. Supplying this argument will stop meanCenter from
searching for interaction terms that might need to be
centered.}
}
\value{
A regression model of the same type as the input model,
with attributes representing the names of the centered variables.
}
\description{
meanCenter selectively centers or standarizes variables in a regression model.
}
\details{
Works with "lm" class objects, objects estimated by \code{glm()}. This
centers some or all of the the predictors and then re-fits the
original model with the new variables. This is a convenience to
researchers who are often urged to center their predictors.  This
is sometimes suggested as a way to ameliorate multi-collinearity
in models that include interaction terms (Aiken and West, 1991;
Cohen, et al 2002). Mean-centering may enhance interpretation of
the regression intercept, but it actually does not help with
multicollinearity.  (Echambadi and Hess, 2007). This function
facilitates comparison of mean-centered models with others by
calculating centered variables.  The defaults will cause a
regression's numeric interactive variables to be mean
centered. Variations on the arguments are discussed in details.

Suppose the user's formula that fits the original model is
\code{m1 <- lm(y ~ x1*x2 + x3 + x4, data = dat)}. The fitted model
will include estimates for predictors \code{x1}, \code{x2},
\code{x1:x2}, \code{x3} and \code{x4}. By default,
\code{meanCenter(m1)} scans the output to see if there are
interaction terms of the form \code{x1:x2}. If so, then x1 and x2
are replaced by centered versions (m1-mean(m1)) and
(m2-mean(m2)). The model is re-estimated with those new variables.
model (the main effect and the interaction). The resulting thing
is "just another regression model", which can be analyzed or
plotted like any R regression object.

The user can claim control over which variables are centered in
several ways. Most directly, by specifying a vector of variable
names, the user can claim direct control. For example, the
argument \code{terms=c("x1","x2","x3")} would cause 3 predictors
to be centered. If one wants all predictors to be centered, the
argument \code{centerOnlyInteractors} should be set to
FALSE. Please note, this WILL NOT center factor variables. But it
will find all numeric predictors and center them.

The dependent variable will not be centered, unless the user
explicitly requests it by setting centerDV = TRUE.

As an additional convenience to the user, the argument
\code{standardize = TRUE} can be used.  This will divide each
centered variable by its observed standard deviation. For people
who like standardized regression, I suggest this is a better
approach than the \code{standardize} function (which is brain-dead
in the style of SPSS). meanCenter with \code{standardize = TRUE}
will only try to standardize the numeric predictors.

To be completely clear, I believe mean-centering is not helpful
with the multicollinearity problem. It doesn't help, it doesn't
hurt.  Only a misunderstanding leads its proponents to claim
otherwise. This is emphasized in the vignette "rockchalk" that is
distributed with this package.
}
\examples{

library(rockchalk)
N <- 100
dat <- genCorrelatedData(N = N, means = c(100, 200), sds = c(20, 30),
                         rho = 0.4, stde = 10)
dat$x3 <- rnorm(100, m = 40, s = 4)

m1 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m1)
mcDiagnose(m1)

m1c <- meanCenter(m1)
summary(m1c)
mcDiagnose(m1c)

m2 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m2)
mcDiagnose(m2)

m2c <- meanCenter(m2, standardize = TRUE)
summary(m2c)
mcDiagnose(m2c)

m2c2 <- meanCenter(m2, centerOnlyInteractors = FALSE)
summary(m2c2)

m2c3 <- meanCenter(m2, centerOnlyInteractors = FALSE, centerDV = TRUE)
summary(m2c3)

dat <- genCorrelatedData(N = N, means = c(100, 200), sds = c(20, 30),
                         rho = 0.4, stde = 10)
dat$x3 <- rnorm(100, m = 40, s = 4)
dat$x3 <- gl(4, 25, labels = c("none", "some", "much", "total"))

m3 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m3)
## visualize, for fun
plotPlane(m3, "x1", "x2")

m3c1 <- meanCenter(m3)
summary(m3c1)

## Not exactly the same as a "standardized" regression because the
## interactive variables are centered in the model frame,
## and the term "x1:x2" is never centered again.
m3c2 <- meanCenter(m3, centerDV = TRUE,
                   centerOnlyInteractors = FALSE, standardize = TRUE)
summary(m3c2)

m3st <- standardize(m3)
summary(m3st)

## Make a bigger dataset to see effects better
N <- 500
dat <- genCorrelatedData(N = N, means = c(200,200), sds = c(60,30),
                         rho = 0.2, stde = 10)
dat$x3 <- rnorm(100, m = 40, s = 4)
dat$x3 <- gl(4, 25, labels = c("none", "some", "much", "total"))
dat$y2 <- with(dat,
               0.4 - 0.15 * x1 + 0.04 * x1^2 -
               drop(contrasts(dat$x3)[dat$x3, ] \%*\% c(-1.9, 0, 5.1))  +
               1000* rnorm(nrow(dat)))
dat$y2 <- drop(dat$y2)

m4literal <- lm(y2 ~ x1 + I(x1*x1) + x2 + x3, data = dat)
summary(m4literal)
plotCurves(m4literal, plotx="x1")
## Superficially, there is multicollinearity (omit the intercept)
cor(model.matrix(m4literal)[ -1 , -1 ])

m4literalmc <- meanCenter(m4literal, terms = "x1")
summary(m4literalmc)

m4literalmcs <- meanCenter(m4literal, terms = "x1", standardize = TRUE)
summary(m4literalmcs)

m4 <- lm(y2 ~ poly(x1, 2, raw = TRUE) + x2 + x3, data = dat)
summary(m4)
plotCurves(m4, plotx="x1")

m4mc1 <- meanCenter(m4, terms = "x1")
summary(m4mc1)

m4mc2 <- meanCenter(m4, terms = "x1", standardize = TRUE)
summary(m4mc2)

m4mc3 <- meanCenter(m4, terms = "x1", centerDV = TRUE, standardize = TRUE)
summary(m4mc3)
}
\references{
Aiken, L. S. and West, S.G. (1991). Multiple
    Regression: Testing and Interpreting Interactions. Newbury
    Park, Calif: Sage Publications.

Cohen, J., Cohen, P., West, S. G., and Aiken, L. S. (2002). Applied
Multiple Regression/Correlation Analysis for the Behavioral Sciences
(Third.). Routledge Academic.

Echambadi, R., and Hess, J. D. (2007). Mean-Centering Does Not Alleviate
Collinearity Problems in Moderated Multiple Regression Models.
Marketing Science, 26(3), 438-445.
}
\seealso{
\code{\link[rockchalk]{standardize}}
    \code{\link[rockchalk]{residualCenter}}
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{getFocal}
\alias{getFocal}
\alias{getFocal.default}
\alias{getFocal.factor}
\alias{getFocal.character}
\title{Select focal values from an observed variable.}
\usage{
getFocal(x, ...)

\method{getFocal}{default}(x, xvals = NULL, n = 3, pct = TRUE, ...)

\method{getFocal}{factor}(x, xvals = NULL, n = 3, pct = TRUE, ...)

\method{getFocal}{character}(x, xvals = NULL, n = 3, pct = TRUE, ...)
}
\arguments{
\item{x}{Required. A variable}

\item{...}{Other arguments that will be passed to the
user-specified xvals function.}

\item{xvals}{A function name (either "quantile", "std.dev.",
"table", or "seq") or a user-supplied function that can receive
x and return a selection of values.}

\item{n}{Number of values to be selected.}

\item{pct}{Default TRUE. Include percentage of observed cases
in variable name? (used in legends)}
}
\value{
A vector.

A named vector of values.
}
\description{
This is a generic function with 2 methods, getFocal.default
handles numeric variables, while getFocal.factor handles
factors. No other methods have been planned for preparation.

Many plotting functions need to select "focal" values from
a variable. There is a family of functions that are used
to do that.  User requests can be accepted in a number
of ways. Numeric and character variables will be treated
differently. Please see details.
}
\details{
This is used in functions like \code{plotSlopes} or
\code{plotCurves}.

If \code{xvals} is not provided, a default divider for numeric
variables will be the algorithm "quantile". The divider algorithms
provided with rockchalk are c("quantile", "std.dev.", "table",
"seq").  \code{xvals} can also be the name of a user-supplied
function, such as R's \code{pretty()}. If the user supplies a
vector of possible values, that selection will be checked to make
sure all elements are within a slightly expanded range of
\code{x}. If a value out of range is requested, a warning is
issued. Maybe that should be an outright error?

With factor variables, \code{xvals} is generally not used because
the only implemented divider algorithm is "table" (see
\code{cutByTable}), which selects the \code{n} most frequently
observed values. That is the default algorithm. It is legal to
specify xvals = "table", but there is no point in doing
that. However, xvals may take two other formats. It may be a
user-specified function that can select levels values from
\code{x} or it may be a vector of labels (or, names of
levels). The purpose of the latter is to check that the requested
levels are actually present in the supplied data vector
\code{x}. If the levels specified are not in the observed
variable, then this function stops with an error message.
}
\examples{
x <- rnorm(100)
getFocal(x)
getFocal(x, xvals = "quantile")
getFocal(x, xvals = "quantile", n = 5)
getFocal(x, xvals = "std.dev")
getFocal(x, xvals = "std.dev", n = 5)
getFocal(x, xvals = c(-1000, 0.2, 0,5))
x <- factor(c("A","B","A","B","C","D","D","D"))
getFocal(x)
getFocal(x, n = 2)

x <- c("A","B","A","B","C","D","D","D")
getFocal(x)
getFocal(x, n = 2)

}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictOMatic.R
\name{predictOMatic}
\alias{predictOMatic}
\title{Create predicted values after choosing values of predictors.  Can
demonstrate marginal effects of the predictor variables.}
\usage{
predictOMatic(model = NULL, predVals = "margins",
  divider = "quantile", n = 5, ...)
}
\arguments{
\item{model}{Required. A fitted regression model. A \code{predict}
method must exist for that model.}

\item{predVals}{Optional. How to choose predictor values?  Can be
as simple as a keyword "auto" or "margins". May also be very
fine-grained detail, including 1) a vector of variable names (for
which values will be automatically selected) 2) a named
vector of variable names and divider functions, or 3) a list
naming variables and values. See details and examples.}

\item{divider}{An algorithm name from c("quantile", "std.dev",
"seq", "table") or a user-provided function.  This sets the method
for selecting values of the predictor. Documentation for the
rockchalk methods can be found in the functions
\code{cutByQuantile}, \code{cutBySD}, \code{plotSeq}, and
\code{cutByTable},.}

\item{n}{Default = 5. The number of values for which
predictions are sought.}

\item{...}{Optional arguments to be passed to the predict
function. In particular, the arguments se.fit and interval are
extracted from ... and used to control the output.}
}
\value{
A data frame or a list of data frames.
}
\description{
It creates "newdata" frames which are passed
to predict. The key idea is that each predictor has certain focal
values on which we want to concentrate. We want a more-or-less
easy way to spawn complete newdata objects along with fitted values.
The \code{newdata} function creates those objects, its documentation
might be helpful in understanding some nuances.
}
\details{
If no predVals argument is supplied (same as
\code{predVals = "margins"}, predictOMatic creates a list of new
data frames, one for each predictor variable. It uses the default
divider algorithm (see the divider argument) and it estimates
predicted values for \code{n} different values of the predictor. A
model with formula \code{y ~ x1 + x2 + x3} will cause 3 separate
output data frames, one for each predictor. They will be named
objects in the list.

The default approach will have marginal tables, while the setting
\code{predVals = "auto"} will create a single large newdata frame
that holds the Cartesian product of the focal values of each predictor.

\code{predVals} may be a vector of variable names, or it may be a
list of names and particular values. Whether a vector or a list is supplied,
\code{predVals} must name only predictors that are fitted in the
model. \code{predictOMatic} will choose the mean or mode for
variables that are not explicitly listed, and selected values of
the named variables are "mixed and matched" to make a data set.
There are many formats in which it can be supplied.  Suppose a
regression formula is \code{y1 ~ sex + income + health +
height}. The simplest format for predVals will be a vector of
variable names, leaving the selection of detailed values to the
default algorithms. For example, \code{predVals = c("income",
"height")} will cause sex and health to be set at central values
and income and height will have target values selected according
to the divider algorithm (see the argument \code{divider}).

The user can spcecify divider algoriths to choose focal values,
\code{predvals = c(income = "quantile", height = "std.dev.")}. The
dividers provided by the rockchalk package are "quantile",
"std.dev.", "seq" and "table".  Those are discussed more
completely in the help for \code{focalVals}.  The appropriate
algorithms will select focal values of the predictors and they
will supply \code{n} values for each in a "mix and match" data
frame. After rockchalk 1.7.2, the divider argument can also be the
name of a function, such as R's pretty.

Finally, users who want very fine grained control over
predictOMatic can supply a named list of predictor
values. For example,
\code{predVals = list(height = c(5.5, 6.0, 6.5),
income = c(10, 20, 30, 40, 50), sex = levels(dat$sex))}. One can
also use algorithm names, \code{predVals = list(height =
c(5.5, 6.0, 6.5), income = "quantile")} and so forth. Examples are
offered below.

The variables named in the \code{predVals} argument should be the names
of the variables in the raw data frame, not the names that R
creates when it interprets a formula. We want "x", not the
transformation in the functions (not \code{log(x)}, or
\code{as.factor(x)} or \code{as.numeric(x)}). If a formula has a
predictor \code{poly(height, 3)}, then the predVals argument
should refer to height, not \code{poly(height, 3)}.  I've invested
quite a bit of effort to make sure this "just works" (many
alternative packages that calculate predicted values do not).

It it important to make sure that diagnostic plots and summaries
of predictions are calculated with the exact same data that was
used to fit the model. This is surprisingly difficult because
formulas can include things like log(income + d) and so forth. The
function \code{model.data} is the magic bullet for that part of
the problem.

Here is one example sequence that fits a model, discerns some
focal values, and then uses predictOMatic.

\code{
d <- 3
alpha <- 13
m1 <- lm(yout ~ xin + xout + poly(xother,2) + log(xercise + alpha), data = dat)
m1dat <- model.data(m1)
}

Now, when you are thinking about which values you might like to
specify in predVals, use m1dat to decide. Try

\code{summarize(m1dat)}

Then run something like

\code{predictOMatic( m1, predVals = list(xin = median(m1dat$xin), xout =
c(1,2,3), xother = quantile(m1dat$xother))}

Get the idea?
}
\examples{
library(rockchalk)

## Replicate some R classics.  The budworm.lg data from predict.glm
## will work properly after re-formatting the information as a data.frame:

## example from Venables and Ripley (2002, pp. 190-2.)
df <- data.frame(ldose = rep(0:5, 2),
                 sex = factor(rep(c("M", "F"), c(6, 6))),
                 SF.numdead = c(1, 4, 9, 13, 18, 20, 0, 2, 6, 10, 12, 16))
df$SF.numalive = 20 - df$SF.numdead

budworm.lg <- glm(cbind(SF.numdead, SF.numalive) ~ sex*ldose,
                  data = df,  family = binomial)

predictOMatic(budworm.lg)

predictOMatic(budworm.lg, n = 7)

predictOMatic(budworm.lg, predVals = c("ldose"), n = 7)

predictOMatic(budworm.lg, predVals = c(ldose = "std.dev.", sex = "table"))



## Now make up a data frame with several numeric and categorical predictors.

set.seed(12345)
N <- 100
x1 <- rpois(N, l = 6)
x2 <- rnorm(N, m = 50, s = 10)
x3 <- rnorm(N)
xcat1 <- gl(2,50, labels = c("M","F"))
xcat2 <- cut(rnorm(N), breaks = c(-Inf, 0, 0.4, 0.9, 1, Inf),
             labels = c("R", "M", "D", "P", "G"))
dat <- data.frame(x1, x2, x3, xcat1, xcat2)
rm(x1, x2, x3, xcat1, xcat2)
dat$xcat1n <- with(dat, contrasts(xcat1)[xcat1, , drop = FALSE])
dat$xcat2n <- with(dat, contrasts(xcat2)[xcat2, ])
STDE <- 15
dat$y <- with(dat,
              0.03 + 0.8*x1 + 0.1*x2 + 0.7*x3 + xcat1n \%*\% c(2) +
              xcat2n \%*\% c(0.1,-2,0.3, 0.1) + STDE*rnorm(N))
## Impose some random missings
dat$x1[sample(N, 5)] <- NA
dat$x2[sample(N, 5)] <- NA
dat$x3[sample(N, 5)] <- NA
dat$xcat2[sample(N, 5)] <- NA
dat$xcat1[sample(N, 5)] <- NA
dat$y[sample(N, 5)] <- NA
summarize(dat)


m0 <- lm(y ~ x1 + x2 + xcat1, data = dat)
summary(m0)
## The model.data() function in rockchalk creates as near as possible
## the input data frame.
m0.data <- model.data(m0)
summarize(m0.data)

## no predVals: analyzes each variable separately
(m0.p1 <- predictOMatic(m0))

## requests confidence intervals from the predict function
(m0.p2 <- predictOMatic(m0, interval = "confidence"))

## predVals as vector of variable names: gives "mix and match" predictions
(m0.p3 <- predictOMatic(m0, predVals = c("x1", "x2")))

## predVals as vector of variable names: gives "mix and match" predictions
(m0.p3s <- predictOMatic(m0, predVals = c("x1", "x2"), divider = "std.dev."))

## "seq" is an evenly spaced sequence across the predictor.
(m0.p3q <- predictOMatic(m0, predVals = c("x1", "x2"), divider = "seq"))

(m0.p3i <- predictOMatic(m0, predVals = c("x1", "x2"),
                         interval = "confidence", n = 3))

(m0.p3p <- predictOMatic(m0, predVals = c("x1", "x2"), divider = pretty))

## predVals as vector with named divider algorithms.
(m0.p3 <- predictOMatic(m0, predVals = c(x1 = "seq", x2 = "quantile")))
## predVals as named vector of divider algorithms

## same idea, decided to double-check
(m0.p3 <- predictOMatic(m0, predVals = c(x1 = "quantile", x2 = "std.dev.")))
getFocal(m0.data$x2, xvals =  "std.dev.", n = 5)


## Change from quantile to standard deviation divider
(m0.p5 <- predictOMatic(m0, divider = "std.dev.", n = 5))

## Still can specify particular values if desired
(m0.p6 <- predictOMatic(m0, predVals = list("x1" = c(6,7),
                            "xcat1" = levels(m0.data$xcat1))))

(m0.p7 <- predictOMatic(m0, predVals = c(x1 = "quantile", x2 = "std.dev.")))
getFocal(m0.data$x2, xvals =  "std.dev.", n = 5)

(m0.p8 <- predictOMatic(m0, predVals = list( x1 = quantile(m0.data$x1,
                        na.rm = TRUE, probs = c(0, 0.1, 0.5, 0.8,
                        1.0)), xcat1 = levels(m0.data$xcat1))))

(m0.p9 <- predictOMatic(m0, predVals = list(x1 = "seq", "xcat1" =
                                levels(m0.data$xcat1)), n = 8) )


(m0.p10 <- predictOMatic(m0, predVals = list(x1 = "quantile",
                                 "xcat1" = levels(m0.data$xcat1)), n =  5) )


(m0.p11 <- predictOMatic(m0, predVals = c(x1 = "std.dev."), n = 10))

## Previous same as

(m0.p11 <- predictOMatic(m0, predVals = c(x1 = "default"), divider =
                         "std.dev.", n = 10))

## Previous also same as

(m0.p11 <- predictOMatic(m0, predVals = c("x1"), divider = "std.dev.", n = 10))


(m0.p11 <- predictOMatic(m0,  predVals = list(x1 = c(0, 5, 8), x2 = "default"),
                         divider = "seq"))



m1 <- lm(y ~ log(10+x1) + sin(x2) + x3, data = dat)
m1.data <- model.data(m1)
summarize(m1.data)


(newdata(m1))
(newdata(m1, predVals = list(x1 = c(6, 8, 10))))
(newdata(m1, predVals = list(x1 = c(6, 8, 10), x3 = c(-1,0,1))))
(newdata(m1, predVals = list(x1 = c(6, 8, 10),
                 x2 = quantile(m1.data$x2, na.rm = TRUE), x3 = c(-1,0,1))))

(m1.p1 <- predictOMatic(m1, divider = "std.dev", n = 5))
(m1.p2 <- predictOMatic(m1, divider = "quantile", n = 5))

(m1.p3 <- predictOMatic(m1, predVals = list(x1 = c(6, 8, 10),
                            x2 = median(m1.data$x2, na.rm = TRUE))))

(m1.p4 <- predictOMatic(m1, predVals = list(x1 = c(6, 8, 10),
                                x2 = quantile(m1.data$x2, na.rm = TRUE))))

(m1.p5 <- predictOMatic(m1))
(m1.p6 <- predictOMatic(m1, divider = "std.dev."))
(m1.p7 <- predictOMatic(m1, divider = "std.dev.", n = 3))
(m1.p8 <- predictOMatic(m1, divider = "std.dev.", interval = "confidence"))


m2 <- lm(y ~ x1 + x2 + x3 + xcat1 + xcat2, data = dat)
##  has only columns and rows used in model fit
m2.data <- model.data(m2)
summarize(m2.data)

## Check all the margins
(predictOMatic(m2, interval = "conf"))

## Lets construct predictions the "old fashioned way" for comparison

m2.new1 <- newdata(m2, predVals = list(xcat1 = levels(m2.data$xcat1),
                           xcat2 = levels(m2.data$xcat2)), n = 5)
predict(m2, newdata = m2.new1)


(m2.p1 <- predictOMatic(m2,
                        predVals = list(xcat1 = levels(m2.data$xcat1),
                            xcat2 = levels(m2.data$xcat2)),
                        xcat2 = c("M","D")))
## See? same!

## Pick some particular values for focus
m2.new2 <- newdata(m2, predVals = list(x1 = c(1,2,3), xcat2 = c("M","D")))
## Ask for predictions
predict(m2, newdata = m2.new2)


## Compare: predictOMatic generates a newdata frame and predictions in one step

(m2.p2 <- predictOMatic(m2, predVals = list(x1 = c(1,2,3),
                                xcat2 = c("M","D"))))

(m2.p3 <- predictOMatic(m2, predVals = list(x2 = c(0.25, 1.0),
                                xcat2 = c("M","D"))))

(m2.p4 <- predictOMatic(m2, predVals = list(x2 = plotSeq(m2.data$x2, 10),
                                xcat2 = c("M","D"))))

(m2.p5 <- predictOMatic(m2, predVals = list(x2 = c(0.25, 1.0),
                                xcat2 = c("M","D")), interval = "conf"))

(m2.p6 <- predictOMatic(m2, predVals = list(x2 = c(49, 51),
                                xcat2 = levels(m2.data$xcat2),
                                x1 = plotSeq(dat$x1))))

plot(y ~ x1, data = m2.data)
by(m2.p6, list(m2.p6$xcat2), function(x) {
    lines(x$x1, x$fit, col = x$xcat2, lty = as.numeric(x$xcat2))
})

m2.newdata <- newdata(m2, predVals = list(x2 = c(48, 50, 52),
                              xcat2 = c("M","D")))
predict(m2, newdata = m2.newdata)

(m2.p7 <- predictOMatic(m2, predVals = list(x2 = c(48, 50, 52),
                                xcat2 = c("M","D"))))

(m2.p8 <- predictOMatic(m2,
             predVals = list(x2 = range(m2.data$x2, na.rm = TRUE),
             xcat2 = c("M","D"))))

(m2.p9 <- predictOMatic(m2, predVals = list(x2 = plotSeq(m2.data$x2),
             x1 = quantile(m2.data$x1, pr =c(0.33, 0.66), na.rm = TRUE),
             xcat2 = c("M","D"))))
plot(y ~ x2 , data = m2.data)

by(m2.p9, list(m2.p9$x1, m2.p9$xcat2), function(x) {lines(x$x2, x$fit)})



(predictOMatic(m2, predVals = list(x2 = c(50, 60), xcat2 = c("M","D")),
               interval = "conf"))

## create a dichotomous dependent variable
y2 <- ifelse(rnorm(N) > 0.3, 1, 0)
dat <- cbind(dat, y2)

m3 <- glm(y2 ~ x1 + x2 + x3 + xcat1, data = dat, family = binomial(logit))
summary(m3)
m3.data <- model.data(m3)
summarize(m3.data)

(m3.p1 <- predictOMatic(m3, divider = "std.dev."))

(m3.p2 <- predictOMatic(m3, predVals = list(x2 = c(40, 50, 60),
                             xcat1 = c("M","F")),
                        divider = "std.dev.", interval = "conf"))

## Want a full accounting for each value of x2?
(m3.p3 <- predictOMatic(m3,
                predVals = list(x2 = unique(m3.data$x2),
                    xcat1 = c("M","F")), interval = "conf"))


## Would like to write a more beautiful print method
## for output object, but don't want to obscure structure from user.
## for (i in names(m3.p1)){
##     dns <- cbind(m3.p1[[i]][i], m3.p1[[i]]$fit)
##     colnames(dns) <- c(i, "predicted")
##     print(dns)
## }


}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{formatSummarizedFactors}
\alias{formatSummarizedFactors}
\title{Prints out the contents of an object created by summarizeFactors
in the style of base::summary}
\usage{
formatSummarizedFactors(x, ...)
}
\arguments{
\item{x}{A summarizedFactors object produced by summarizeFactors}

\item{...}{optional arguments. Only value currently used is digits,
which defaults to 2.}
}
\value{
A table of formatted output
}
\description{
An object with class "summarizedFactors" is the input. Such an
object should be created with the function
rockchalk::summarizeFactors. Each element in that list is then
organized for printing in a tabular summary.  This should look
almost like R's own summary function, except for the additional
information that these factor summaries include.
}
\examples{
dat <- data.frame(xcat1 = gl(10, 3), xcat2 = gl(5, 6))
summarizeFactors(dat, maxLevels = 8)
formatSummarizedFactors(summarizeFactors(dat))
}
\seealso{
\code{\link{summarize}}, \code{\link{summarizeFactors}}, \code{\link{formatSummarizedNumerics}}
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cutByQuantile}
\alias{cutByQuantile}
\title{Calculates the "center" quantiles, always including the median, when n is odd.}
\usage{
cutByQuantile(x, n = 3)
}
\arguments{
\item{x}{A numeric vector.}

\item{n}{The number of quantile points. See details.}
}
\value{
A vector
}
\description{
If the numeric variable has fewer than 6 unique observed values,
this will send the data to \code{cutByTable}.  The default return
will find dividing points at three quantiles: c(0.25, 0.50, 0.75)
If n=4, the dividing points will be c(0.20, 0.40, 0.60, 0.80) If
n=5, c(0.0, 0.25, 0.50, 0.75, 1.0) Larger n that are odd will
include 0.5 and evenly spaced points out to proportions 0 and
1.0. Larger n that is even will return evenly spaced points
calculated by R's \code{pretty} function.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cutBySD}
\alias{cutBySD}
\title{Returns center values of x, the mean, mean-std.dev, mean+std.dev}
\usage{
cutBySD(x, n = 3)
}
\arguments{
\item{x}{A numeric variable}

\item{n}{Should be an odd number 1, 3, 5, or 7. If 2 < n < 5,
values that divide the data at c(m-sd, m, m+sd) are returned. If n
> 4, the returned values are c(m-2sd, m-sd, m, m+sd, m+2sd).}
}
\value{
A named vector
}
\description{
If the numeric variable has fewer than 6 unique observed values,
this will send the data to cutByTable.
}
\examples{
x <- rnorm(100, m = 100, s = 20)
cutBySD (x, n = 3)
cutBySD (x, n = 5)
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSlopes.R
\name{plotFancyCategories}
\alias{plotFancyCategories}
\title{Draw display for discrete predictor in plotSlopes}
\usage{
plotFancyCategories(newdf, olddf, plotx, modx = NULL, modxVals, xlab,
  xlim, ylab, ylim, col = c("black", "blue", "darkgreen", "red",
  "orange", "purple", "green3"), opacity = 120, main, space = c(0, 1),
  width = 0.2, llwd = 1, offset = 0, ..., gridArgs = list(lwd =
  0.3, lty = 5), legendArgs)
}
\arguments{
\item{newdf}{The new data object, possibly from predictOMatic}

\item{olddf}{The model data matrix}

\item{plotx}{Name of horizontal axis variable}

\item{modx}{Name of moderator}

\item{modxVals}{values for modx}

\item{xlab}{X axis label}

\item{xlim}{x axis limits. Don't bother setting this, the internal
numbering is too complicated.}

\item{ylab}{y axis label}

\item{ylim}{y axis limits}

\item{col}{color pallet for values of moderator variable}

\item{opacity}{Value in 0, 255 for darkness of interval shading}

\item{main}{main title}

\item{space}{same as space in barplot, vector c(0, 1) is
c(space_between, space_before_first)}

\item{width}{width of shaded bar area, default is 0.2. Maximum is 1.}

\item{llwd}{requested line width, will re-cycle.}

\item{offset}{Shifts display to right (not tested)}

\item{...}{Arguments sent to par}

\item{gridArgs}{A list of values to control printing of reference grid.
Set as "none" if no grid is desired.}

\item{legendArgs}{Arguments to the legend function. Set as "none"
if no legend is needed. Otherwise, provide a list}
}
\value{
None
}
\description{
There's plotFancy for numeric predictor. This is for discrete
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{formatSummarizedNumerics}
\alias{formatSummarizedNumerics}
\title{Reformat numeric summarize output as one column per variable,
similar to R summary}
\usage{
formatSummarizedNumerics(x, ...)
}
\arguments{
\item{x}{numeric summaries from summarize function}

\item{...}{Other arguments, such as digits}
}
\value{
An R \code{table} object
}
\description{
The summarizeNumeric function returns a data frame with the
variable names on the rows and summary statistics (mean, median,
std. deviation) in the columns.This transposes and abbreviates
the information to look more like R summary.
}
\examples{
set.seed(21234)
X <- matrix(rnorm(10000), ncol = 10, dimnames = list(NULL, paste0("xvar", 1:10)))
Xsum <- summarize(X)
Xsum$numerics
formatSummarizedNumerics(Xsum$numerics)
formatSummarizedNumerics(Xsum$numerics, digits = 5)
Xsum.fmt <- formatSummarizedNumerics(Xsum$numerics)
str(Xsum.fmt)
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutFancy.R
\name{cutFancy}
\alias{cutFancy}
\title{Create an ordinal variable by grouping numeric data input.}
\usage{
cutFancy(y, cutpoints = "quantile", probs, categories)
}
\arguments{
\item{y}{The input data from which the categorized variable will
be created.}

\item{cutpoints}{Optional paramter, a vector of thresholds at
which to cut the data. If it is not supplied, the default
value \code{cutpoints="quantile"} will take effect. Users can
supplement with \code{probs} and/or \code{categories} as shown
in examples.}

\item{probs}{This is an optional parameter, relevant only when the
R function \code{\link{quantile}} function is used to
calculate cutpoints. The length should be number of desired
categories PLUS ONE, as in \code{c(0, .3, .6, 1)}. That will
create categories that represent 1) less than .3, between .3
and .6, and above .6.  A common user error is to specify only
the internal divider values, such as \code{probs = c(.3,
.6)}. To anticipate and correct that error, this function will
insert the lower limit of 0 and the upper limit of 1 if they
are not already present in \code{probs}.}

\item{categories}{Can be a number to designate the number of
sub-groups created, or it can be a vector of names used. If
\code{cutpoints} and \code{probs} are not specified, the
parameter \code{categories} should be an integer to specify
how many data groups to create.It is required if
cutpoints="quantile" and probs is not specified. Can also be a
vector of names to be used for the categories that are
created. If category names are not provided, the names for the
ordinal variable will be the midpoint of the numeric range
from which they are constructed.}
}
\value{
an ordinal vector with attributes "cutpoints" and "props"
    (proportions)
}
\description{
This is a convenience function for usage of R's \code{cut}
function. Users can specify cutpoints or category labels or
desired proportions of groups in various ways. In that way, it has
a more flexible interface than \code{cut}. It also tries to notice
and correct some common user errors, such as omitting the outer
boundaries from the probs argument. The returned values are
labeled by their midpoints, rather than cut's usual boundaries.
}
\details{
The dividing points, thought of as "thresholds" or "cutpoints",
can be specified in several ways.  \code{cutFancy} will
automatically create equally-sized sets of observations for a
given number of categories if neither \code{probs} nor
\code{cutpoints} is specified. The bare minimum input needed is
\code{categories=5}, for example, to ask for 5 equally sized
groups. More user control can be had by specifying either
\code{cutpoints} or \code{probs}. If \code{cutpoints} is not
specified at all, or if \code{cutpoints="quantile"}, then
\code{probs} can be used to specify the proportions of the data
points that are to fall within each range. On the other hand, one
can specify \code{cutpoints = "quantile"} and then \code{probs} will
be used to specify the proportions of the data points that are to
fall within each range.

If \code{categories} is not specified, the category names will be
created. Names for ordinal categories will be the numerical
midpoints for the outcomes.  Perhaps this will deviate from your
expectation, which might be ordinal categories name "0", "1", "2",
and so forth.  The numerically labeled values we provide can be
used in various ways during the analysis process. Read "?factor"
to learn ways to convert the ordinal output to other
formats. Examples include various ways of converting the ordinal
output to numeric.

The \code{categories} parameter works together with
\code{cutpoints}.  \code{cutpoints} allows a character string
"quantile". If \code{cutpoints} is not specified, or if the user
specifies a character string \code{cutpoints="quantile"}, then the
\code{probs} would be used to determine the cutpoints.  However,
if \code{probs} is not specified, then the \code{categories}
argument can be used. If \code{cutpoints="quantile"}, then

\itemize{
\item if \code{categories} is one integer, then it is interpreted
as the number of "equally sized" categories to be created, or

\item \code{categories} can be a vector of names. The length
of the vector is used to determine the number of categories, and
the values are put to use as factor labels.
}
}
\examples{
set.seed(234234)
y <- rnorm(1000, m = 35, sd = 14)
yord <- cutFancy(y, cutpoints = c(30, 40, 50))
table(yord)
attr(yord, "props")
attr(yord, "cutpoints")
yord <- cutFancy(y, categories = 4L)
table(yord, exclude = NULL)
attr(yord, "props")
attr(yord, "cutpoints")
yord <- cutFancy(y, probs = c(0, .1, .3, .7, .9, 1.0),
                  categories = c("A", "B", "C", "D", "E"))
table(yord, exclude = NULL)
attr(yord, "props")
attr(yord, "cutpoints")
yord <- cutFancy(y, probs = c(0, .1, .3, .7, .9, 1.0))
table(yord, exclude = NULL)
attr(yord, "props")
attr(yord, "cutpoints")
yasinteger <- as.integer(yord)
table(yasinteger, yord)
yasnumeric <- as.numeric(levels(yord))[yord]
table(yasnumeric, yord)
barplot(attr(yord, "props"))
hist(yasnumeric)
X1a <-
   genCorrelatedData3("y ~ 1.1 + 2.1 * x1 + 3 * x2 + 3.5 * x3 + 1.1 * x1:x3",
                       N = 10000, means = c(x1 = 1, x2 = -1, x3 = 3),
                       sds = 1, rho = 0.4)
## Create cutpoints from quantiles
probs <- c(.3, .6)
X1a$yord <- cutFancy(X1a$y, probs = probs)
attributes(X1a$yord)
table(X1a$yord, exclude = NULL)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSlopes.R
\name{plotFancy}
\alias{plotFancy}
\title{Regression plots with predicted value lines, confidence intervals, color coded interactions}
\usage{
plotFancy(newdf, olddf, plotx, modx, modxVals, interval, plotPoints,
  legendArgs, col = NULL, llwd = 2, opacity, ...)
}
\arguments{
\item{newdf}{The new data frame with predictors and fit, lwr, upr
variables}

\item{olddf}{A data frame with variables(modxVar, plotxVar,
depVar)}

\item{plotx}{Character string for name of variable on horizontal
axis}

\item{modx}{Character string for name of moderator variable.}

\item{modxVals}{Values of moderator for which lines are desired}

\item{interval}{TRUE or FALSE: want confidence intervals?}

\item{plotPoints}{TRUE or FALSE: want to see observed values in
plot?}

\item{legendArgs}{Set as "none" for no legend. Otherwise, a list
of arguments for the legend function}

\item{col}{requested color scheme for lines and points. One per
value of modxVals.}

\item{llwd}{requested line width, will re-cycle.}

\item{opacity}{Value in 0, 255 for darkness of interval shading}

\item{...}{Other arguments passed to plot function.}
}
\value{
col, lty, and lwd information
}
\description{
This is the back-end for the functions plotSlopes and plotCurves. Don't use it directly.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcDiagnose.R
\name{mcDiagnose}
\alias{mcDiagnose}
\title{Multi-collinearity diagnostics}
\usage{
mcDiagnose(model)
}
\arguments{
\item{model}{a fitted regression model}
}
\value{
a list of the "auxiliary regressions" that were
fitted during the analysis
}
\description{
Conducts a series of checks for multicollinearity.
}
\examples{

library(rockchalk)
N <- 100
dat <- genCorrelatedData3(y~ 0 + 0.2*x1 + 0.2*x2, N=N, means=c(100,200),
                          sds=c(20,30), rho=0.4, stde=10)
dat$x3 <- rnorm(100, m=40, s=4)
m1 <- lm(y ~ x1 + x2 + x3, data=dat)
summary(m1)
m1d <- mcDiagnose(m1)

m2 <- lm(y ~ x1 * x2 + x3, data=dat)
summary(m2)
m2d <- mcDiagnose(m2)



m3 <- lm(y ~ log(10+x1) + x3 + poly(x2,2), data=dat)
summary(m3)
m3d <- mcDiagnose(m3)

N <- 100
x1 <- 50 + rnorm(N)
x2 <- log(rgamma(N, 2,1))
x3 <- rpois(N, lambda=17)
z1 <- gl(5, N/5)
dummies <- contrasts(z1)[ as.numeric(z1), ]
dimnames(dummies) <- NULL ## Avoids row name conflict in data.frame below
y3 <- x1  -.5 * x2 + 0.1 * x2^2 + dummies \%*\% c(0.1,-0.1,-0.2,0.2)+ 5 * rnorm(N)
dat <- data.frame(x1=x1, x2=x2, x3=x3,  z1=z1, y3 = y3)

m3 <- lm(y3 ~ x1 + poly(x2,2)  + log(x1) + z1, dat)
summary(m3)

mcDiagnose(m3)



}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vech2mat.R
\name{vech2mat}
\alias{vech2mat}
\title{Convert a half-vector (vech) into a matrix.}
\usage{
vech2mat(vech, diag = NULL, lowerOnly = FALSE)
}
\arguments{
\item{vech}{A vector}

\item{diag}{Optional. A single value or a vector for the
diagonal. A vech is a strictly lower triangluar vech, it
does not include diagonal values. diag can be either a single
value (to replace all elements along the diagonal) or a vector of
the correct length to replace the diagonal.}

\item{lowerOnly}{Default = FALSE.}
}
\description{
Fills a matrix from a vector that represents the lower triangle.
If user does not supply a value for diag, then the vech will fill
in the diagonal as well as the strictly lower triangle.  If diag
is provided (either a number or a vector), then vech is for the
strictly lower triangular part.  The default value for lowerOnly
is FALSE, which means that a symmetric matrix will be created. See
examples for a demonstration of how to fill in the lower triangle
and leave the diagonal and the upper triangle empty.
}
\examples{
x <- 1:6
vech2mat(x)
vech2mat(x, diag = 7)
vech2mat(x, diag = c(99, 98, 97, 96))
vech2mat(x, diag = 0, lowerOnly = TRUE)
}
\seealso{
Similar functions exist in many packages, see
\code{vec2sm} in corpcor, \code{xpnd} in MCMCpack
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeVec.R
\name{makeVec}
\alias{makeVec}
\title{makeVec for checking or creating vectors}
\usage{
makeVec(x = NULL, d = NULL)
}
\arguments{
\item{x}{A single value or a vector}

\item{d}{An integer, the desired size of the vector}
}
\value{
A vector of length d
}
\description{
This is a convenience for handling function arguments. If x is a
single value, it makes a vector of length d in which all values
are equal to x. If x is a vector, check that its length is d.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rockchalk-package.R
\docType{data}
\name{cheating}
\alias{cheating}
\title{Cheating and Looting in Japanese Electoral Politics}
\format{data.frame: 16623 obs. on 27 variables}
\source{
\url{http://faculty.arts.ubc.ca/bnyblade/publications.html}.
}
\usage{
data(cheating)
}
\description{
Extracted from the "cheating-replication.dta" data file
with permission by the authors, Benjamin Nyblade and Steven
Reed. The Stata data file provided by the authors included
many constructed variables that have been omitted.  Within
R, these can be easily re-contructed by users.
}
\details{
Special thanks to NyBlade and Reed for permission to repackage
this data. Also special thanks to them for creating an especially
transparent variable naming scheme.

The data set includes many columns for variables that can easily
be re-constructed from the columns that are provided here.  While
Stata users might need to manually create 'dummy variables' and
interactions, R users generally do not do that manually.

These variables from the original data set were omitted:

Dummy variables for the year variable: c("yrd1", "yrd2", ...,
"yrd17", "yrd18")

Dummy variables for the ku variable:
c("ku1", "ku2", ..., "ku141", "ku142")

Constructed product variables: c("actualratiosq", "viabsq",
"viab_candcamp_divm", "viab_candothercamp_divm",
"viabsq_candcamp_divm", "viabsq_candothercamp_divm",
"absviab_candcamp", "absviab_candothercamp",
"absviab_candcamp_divm", "absviab_candothercamp_divm",
"viabsq_candcamp", "viabsq_candothercamp", "viab_candcamp",
"viab_candothercamp", "candothercamp_divm", "candcamp_divm",
"candcampminusm", "candothercampminusm", "predratiosq", "absviab")

Mean centered variables: constr2 <- c("viab_candcampminusm",
"viab_candothercampminusm", "viabsq_candothercampminusm",
"viabsq_candcampminusm")

In the end, we are left with these variables:

[1] "ku"
[2] "prefecture"
[3] "dist"
[4] "year"
[5] "yr"
[6] "cdnr"
[7] "jiban"
[8] "cheating"
[9] "looting"
[10] "actualratio"
[11] "viab"
[12] "inc"
[13] "cons"
[14] "ur"
[15] "newcand"
[16] "jwins"
[17] "cons_cwins"
[18] "oth_cwins"
[19] "camp"
[20] "fleader"
[21] "incablast"
[22] "predratio"
[23] "m"
[24] "candcamp"
[25] "candothercamp"
[26] "kunocheat"
[27] "kunoloot"
}
\examples{
require(rockchalk)
data(cheating)

table1model2 <- glm(cheating ~ viab + I(viab^2) + inc + cons + ur
+ newcand + jwins + cons_cwins + oth_cwins, family = binomial(link
= "logit"), data = cheating)

predictOMatic(table1model2)

predictOMatic(table1model2, interval = "confidence")

## The publication used "rare events logistic", which I'm not bothering
## with here because I don't want to invoke additional imported packages.
## But the ordinary logit results are proof of concept.
}
\references{
Benjamin Nyblade and Steven Reed, "Who Cheats? Who
Loots? Political Competition and Corruption in Japan, 1947-1993."
American Journal of Political Science 52(4): 926-41. October 2008.
}
\author{
Paul E. Johnson \email{pauljohn@ku.edu}, on behalf of Benjamin Nyblade and Steven Reed
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rockchalk-package.R
\docType{package}
\name{rockchalk-package}
\alias{rockchalk-package}
\alias{rockchalk}
\title{rockchalk: regression functions}
\description{
Includes an ever-growing collection of functions that assist in
the presentation of regression models.  The initial function was
\code{\link{outreg}}, which produces LaTeX tables that summarize
one or many fitted regression models.  It also offers plotting
conveniences like \code{\link{plotPlane}} and
\code{\link{plotSlopes}}, which illustrate some of the variables
from a fitted regression model. For a detailed check on
multicollinearity, see \code{\link{mcDiagnose}}.  The user should
be aware of this fact: Not all of these functions lead to models
or types of analysis that we endorese.  Rather, they all lead to
analysis that is endorsed by some scholars, and we feel it is
important to facilitate the comparison of competing methods.  For
example, the function \code{\link{standardize}} will calculate
standardized regression coefficients for all predictors in a
regression model's design matrix in order to replicate results
from other statistical frameworks, no matter how unwise the use of
such coefficients might be. The function \code{\link{meanCenter}}
will allow the user to more selectively choose variables for
centering (and possibly standardization) before they are entered
into the design matrix.  Because of the importance of interaction
variables in regression analysis, the \code{\link{residualCenter}}
and \code{\link{meanCenter}} functions are offered.  While mean
centering does not actually help with multicollinearity of
interactive terms, many scholars have argued that it does.  The
meanCenter function can be compared with the "residual centering"
of interaction terms.
}
\references{
http://pj.freefaculty.org/R
}
\author{
Paul E. Johnson \email{pauljohn@ku.edu}
}
\keyword{hplot}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcDiagnose.R
\name{getDeltaRsquare}
\alias{getDeltaRsquare}
\title{Calculates the delta R-squares, also known as squared
semi-partial correlation coefficients.}
\usage{
getDeltaRsquare(model)
}
\arguments{
\item{model}{a fitted regression model}
}
\value{
a vector of estimates of the delta R-squares
}
\description{
The change in the R-square when a variable is removed from a
regression is called delta R-square. It is sometimes suggested as
a way to determine whether a variable has a substantial effect on
an outcome. This is also known as the squared semi-partial correlation
coefficient.
}
\examples{
dat1 <- genCorrelatedData(N=250, means=c(100,100),
sds=c(30,20), rho=0.0,  stde = 7, beta=c(1.1, 2.4, 4.1, 0))
m1 <- lm(y ~ x1 + x2, data=dat1)
getDeltaRsquare(m1)
## more problematic in presence of collinearity
dat2 <- genCorrelatedData(N=250, means=c(100,100),
sds=c(30,20), rho=0.6,  stde = 7, beta=c(1.1, 2.4, 4.1, 0))
m2 <- lm(y ~ x1 + x2, data=dat2)
getDeltaRsquare(m2)
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeSymmetric.R
\name{makeSymmetric}
\alias{makeSymmetric}
\title{Create Symmetric Matrices, possibly covariance or correlation
matrices, or check a matrix for symmetry and serviceability.}
\usage{
makeSymmetric(X, d = NULL, diag = NULL, corr = FALSE, cov = FALSE)
}
\arguments{
\item{X}{A single value, a vector (a vech), or a matrix}

\item{d}{Optional. An integer, the desired number of rows (or
columns). Don't specify this argument if X is already a
matrix.  Only required if X is an integer and diag is not
supplied. Otherwise, the function tries to deduce desired size
of output from X (as a vech) and diag.}

\item{diag}{Values for the diagonal. This is important because it
alters the way X is interpreted.  If diag is not provided,
then X is understood to include diagonal elements.}

\item{corr}{TRUE or FALSE: Should we construct a correlation
matrix}

\item{cov}{TRUE or FALSE: Should this be a covariance matrix?}
}
\value{
A d x d matrix
}
\description{
Check X and do the right thing. If X is a matrix, check that it is
a valid for the intended purpose (symmetric or correlation or
covariance).  If X a single value, use that to fill up a
matrix. If it is a vector, try to use it as a vech to fill the
lower triangle. If d is supplied as an integer, use that as desired size.
}
\examples{
makeSymmetric(X = 3, d = 4)
makeSymmetric(X = 3, d = 4, diag = c(99, 98, 97, 96))
makeSymmetric(c(1,2,3))
makeSymmetric(c(1,2,3), d = 5)
makeSymmetric(c(0.8,0.4, 0.2), cov = TRUE)
makeSymmetric(c(0.8,0.4, 0.2), cov = TRUE, diag = c(44, 55, 66))

}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/perspEmpty.R
\name{perspEmpty}
\alias{perspEmpty}
\title{perspEmpty}
\usage{
perspEmpty(x1, x2, y, x1lab = "x1", x2lab = "x2", ylab = "y", x1lim,
  x2lim, ...)
}
\arguments{
\item{x1}{data for the first horizontal axis, an R vector}

\item{x2}{data for the second horizontal axis, an R vector}

\item{y}{data for the vertical axis, an R vector}

\item{x1lab}{label for the x1 axis, (the one called "xlab" inside persp)}

\item{x2lab}{label for the x2 axis, (the one called "ylab" inside persp)}

\item{ylab}{label for the y (vertical) axis (the one called "zlab" inside persp)}

\item{x1lim}{Optional: limits for x1 axis (should be a vector with 2 elements)}

\item{x2lim}{Optional: limits for x2 axis (should be a vector with 2 elements)}

\item{...}{further arguments that are passed to persp. Please note
Please remember that y is the vertical axis, but for persp, that
is the one I call x2.  Thus dot-dot-dot arguments including xlab,
ylab, zlab, xlim, ylim, and zlim are going to be ignored.}
}
\value{
The perspective matrix that is returned by persp
}
\description{
Creates a persp plot without drawing anything in the interior.
Does equivalent \code{of plot( type="n")} for persp.
}
\details{
Regression demonstrations require a blank slate in which
points and planes can be drawn. This function creates that
blank persp canvas for those projects. It is not necessary
that x1, x2 and y be vectors of the same length, since this
function's only purpose is to plot an empty box with ranges
determined by the input variables. persp calls the 3 axes
x, y, and z, but here they are called x1, x2, and y.
}
\examples{
x1 <- 1:10
x2 <- 41:50
y <-  rnorm(10)
perspEmpty(x1, x2, y)
res <- perspEmpty(x1, x2, y, ticktype="detailed", nticks=10)
mypoints1 <- trans3d ( x1, x2, y, pmat = res )
points( mypoints1, pch = 16, col= "blue")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcDiagnose.R
\name{getAuxRsq}
\alias{getAuxRsq}
\title{retrieves estimates of the coefficient of determination from a list of regressions}
\usage{
getAuxRsq(auxRegs)
}
\arguments{
\item{auxRegs}{a list of fitted regression objects}
}
\value{
a numeric vector of the same length as auxRegs.
}
\description{
Asks each regression model in a list for a summary and then reports the R-squares.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genCorrelatedData.R
\name{genCorrelatedData2}
\alias{genCorrelatedData2}
\title{Generates a data frame for regression analysis.}
\usage{
genCorrelatedData2(N = 100, means = c(50, 50, 50), sds = c(10, 10,
  10), rho = c(0, 0, 0), stde = 100, beta = c(0, 0.15, 0.1, -0.1),
  intercept = FALSE, verbose = TRUE)
}
\arguments{
\item{N}{Number of cases desired}

\item{means}{P-vector of means for X. Implicitly sets the dimension of
the predictor matrix as N x P.}

\item{sds}{Values for standard deviations for columns of X. If
less than P values are supplied, they will be recycled.}

\item{rho}{Correlation coefficient for X. Several input formats
are allowed (see \code{lazyCor}). This can be a single number (common
correlation among all variables), a full matrix of correlations
among all variables, or a vector that is interpreted as the
strictly lower triangle (a vech).}

\item{stde}{standard deviation of the error term in the data
generating equation}

\item{beta}{beta vector of coefficients for intercept, slopes, and
interaction terma.  The first P+1 values are the
intercept and slope coefficients for the predictors. Additional
elements in beta are interpreted as coefficients for nonlinear and
interaction coefficients.  I have decided to treat these as a
column (vech) that fills into a lower triangular matrix. It
is easy to see what's going on if you run the examples. There
is also explanation in Details.}

\item{intercept}{Default FALSE. Should the predictors include an
intercept?}

\item{verbose}{TRUE or FALSE. Should information about the data
generation be reported to the terminal?}
}
\value{
A data matrix that has columns c(y, x1, x2, ..., xP)
}
\description{
Unlike \code{genCorrelatedData}, this new-and-improved
function can generate a data frame with as many predictors
as the user requests along with an arbitrarily complicated
regression formula.  The result will be a data frame with
columns named (y, x1, x2, ..., xp).
}
\details{
Arguments supplied must have enough information so that an
N x P matrix of predictors can be constructed.
The matrix X is drawn from a multivariate normal
distribution, the expected value vector (mu vector) is given by
the \code{means} and the var/covar matrix (Sigma) is
built from user supplied standard deviations \code{sds}
and the correlations between the columns of X, given by \code{rho}.
The user can also set the standard deviation
of the error term (\code{stde}) and the coefficients
of the regression equation (\code{beta}).

If called with no arguments, this creates a data frame with
X ~ MVN(mu = c(50,50,50), Sigma = diag(c(10,10,10))).
y = X %*% c(0.15, 0.1, -0.1) + error, where error
is N(m = 0, sd = 200). All of these details can be
changed by altering the arguments.

The y (output) variable is created according to the
equation\cr
\deqn{
y = b1 + b2 * x1 + ...+ bk * xk + b[k+1] * x1 * ...interactions.. + e}
\cr
For shorthand, I write b1 for beta[1], b2 for beta[2], and so forth.\cr

The first P+1 arguments in the argument beta are the coefficients
for the intercept and the columns of the X matrix.  Any additional
elements in beta are the coefficients for nonlinear and interaction terms.\cr

Those additional values in the beta vector are completely
optional. Without them, the true model is a linear
regression. However, one can incorporate the effect of squared terms
(conceptualize that as x1 * x1, for example) or interactions
(x1 * x2) easily.  This is easier to illustrate than describe.
Suppose there are 4 columns in X. Then a beta
vector like beta = c(0, 1, 2, 3, 4, 5, 6, 7, 8) would amount to
asking for\cr
\deqn{
y = 0 + 1 x1 + 2 x2 + 3 x3 + 4 x4 + 5 x1^2 + 6 x1 x2 + 7 x1 x3 + 8 x1 x4 + error
}
\cr
If beta supplies more coefficients, they are interpeted as additional
interactions.\cr

When there are a many predictors and the beta vector is long, this
can become confusing. I think of this as a vech for the lower
triangle of a coefficient matrix. In the example with 4
predictors, beta[1:5] are used for the intercepts and slopes. The
rest of the  beta elements lay in like so:\cr

X1   X2  X3  X4\cr
X1 b6   .    .\cr
X2 b7   b10  .\cr
X3 b8   b11  b13\cr
X4 b9   b12  b14 b15\cr

If the user only supplies b6 and b7, the rest are assumed  to  be 0.\cr

To make this clear, the formula used to calculate y is printed to
the console when genCorrelatedData2 is called.
}
\examples{
## 1000 observations of uncorrelated X with no interactions
set.seed(234234)
dat <- genCorrelatedData2(N = 10, rho = 0.0, beta = c(1, 2, 1, 1),
    means = c(0,0,0), sds = c(1,1,1), stde = 0)
summarize(dat)
## The perfect regression!
m1 <- lm(y ~ x1 + x2 + x3, data = dat)
summary(m1)

dat <- genCorrelatedData2(N = 1000, rho = 0,
    beta = c(1, 0.2, -3.3, 1.1), stde = 50)
m1 <- lm(y ~ x1 + x2 + x3, data = dat)
summary(m1)
predictOMatic(m1)
plotCurves(m1, plotx = "x2")

## interaction between x1 and x2
dat <- genCorrelatedData2(N = 1000, rho = 0.2,
    beta = c(1, 1.0, -1.1, 0.1, 0.0, 0.16), stde = 1)
summarize(dat)
## Fit wrong model? get "wrong" result
m2w <- lm(y ~ x1 + x2 + x3, data = dat)
summary(m2w)
## Include interaction
m2 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m2)

dat <- genCorrelatedData2(N = 1000, rho = 0.2,
    beta = c(1, 1.0, -1.1, 0.1, 0.0, 0.16), stde = 100)
summarize(dat)
m2.2 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m2.2)

dat <- genCorrelatedData2(N = 1000, means = c(100, 200, 300, 100),
    sds = 20,  rho = c(0.2, 0.3, 0.1, 0, 0, 0),
    beta = c(1, 1.0, -1.1, 0.1, 0.0, 0.16, 0, 0, 0.2, 0, 0, 1.1, 0, 0, 0.1),
    stde = 200)
summarize(dat)
m2.3w <- lm(y ~ x1 + x2 + x3, data = dat)
summary(m2)

m2.3 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m2.3)

predictOMatic(m2.3)
plotCurves(m2.3, plotx = "x1", modx = "x2", modxVals = "std.dev.", n = 5)

simReg <- lapply(1:100, function(x){
    dat <- genCorrelatedData2(N = 1000, rho = c(0.2),
        beta = c(1, 1.0, -1.1, 0.1, 0.0, 0.46), verbose = FALSE)
    mymod <- lm (y ~ x1 * x2 + x3, data = dat)
    summary(mymod)
})

x3est <- sapply(simReg, function(reg) {coef(reg)[4 ,1] })
summarize(x3est)
hist(x3est,  breaks = 40, prob = TRUE,
    xlab = "Estimated Coefficients for column x3")

r2est <- sapply(simReg, function(reg) {reg$r.square})
summarize(r2est)
hist(r2est,  breaks = 40, prob = TRUE, xlab = "Estimates of R-square")

## No interaction, collinearity
dat <- genCorrelatedData2(N = 1000, rho = c(0.1, 0.2, 0.7),
    beta = c(1, 1.0, -1.1, 0.1), stde = 1)
m3 <- lm(y ~ x1 + x2 + x3, data = dat)
summary(m3)

dat <- genCorrelatedData2(N=1000, rho=c(0.1, 0.2, 0.7),
    beta = c(1, 1.0, -1.1, 0.1), stde = 200)
m3 <- lm(y ~ x1 + x2 + x3, data = dat)
summary(m3)
mcDiagnose(m3)

dat <- genCorrelatedData2(N = 1000, rho = c(0.9, 0.5, 0.4),
    beta = c(1, 1.0, -1.1, 0.1), stde = 200)
m3b <- lm(y ~ x1 + x2 + x3, data = dat)
summary(m3b)
mcDiagnose(m3b)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/testSlopes.R
\name{testSlopes}
\alias{testSlopes}
\title{Hypothesis tests for Simple Slopes Objects}
\usage{
testSlopes(object)
}
\arguments{
\item{object}{Output from the plotSlopes function}
}
\value{
A list including 1) the hypothesis test table, 2) a copy of
the plotSlopes object, and, for numeric
modx variables, 3) the Johnson-Neyman (J-N) interval boundaries.
}
\description{
Conducts t-test of the hypothesis that the "simple slope" line for
one predictor is statistically significantly different from zero
for each value of a moderator variable. The user must first run
\code{plotSlopes()}, and then give the output object to
\code{plotSlopes()}. A plot method has been implemented for
testSlopes objects. It will create an interesting display, but
only when the moderator is a numeric variable.
}
\details{
This function scans the input object to detect the focal values of
the moderator variable (the variable declared as \code{modx} in
\code{plotSlopes}). Consider a regression with interactions

y <- b0 + b1*x1 + b2*x2 + b3*(x1*x2) + b4*x3 + ... + error

If \code{plotSlopes} has been run with the argument plotx="x1" and
the argument modx="x2", then there will be several plotted lines,
one for each of the chosen values of x2.  The slope of each of
these lines depends on x1's effect, b1, as well as the interactive
part, b3*x2.

This function performs a test of the null hypothesis of the slope
of each fitted line in a \code{plotSlopes} object is statistically
significant from zero. A simple t-test for each line is offered.
No correction for the conduct of multiple hypothesis tests (no
Bonferroni correction).

When \code{modx} is a numeric variable, it is possible to conduct
further analysis. We ask "for which values of \code{modx} would
the effect of \code{plotx} be statistically significant?"  This is
called a Johnson-Neyman (Johnson-Neyman, 1936) approach in
Preacher, Curran, and Bauer (2006). The interval is calculated
here.  A plot method is provided to illustrate the result.
}
\examples{
library(rockchalk)
library(carData)
m1 <- lm(statusquo ~ income * age + education + sex + age, data = Chile)
m1ps <- plotSlopes(m1, modx = "income", plotx = "age")
m1psts <- testSlopes(m1ps)
plot(m1psts)


dat2 <- genCorrelatedData(N = 400, rho = .1, means = c(50, -20),
                          stde = 300, beta = c(2, 0, 0.1, -0.4))
m2 <- lm(y ~ x1*x2, data = dat2)
m2ps <- plotSlopes(m2, plotx = "x1", modx = "x2")
m2psts <- testSlopes(m2ps)
plot(m2psts)
m2ps <- plotSlopes(m2, plotx = "x1", modx = "x2", modxVals = "std.dev", n = 5)
m2psts <- testSlopes(m2ps)
plot(m2psts)

## Try again with longer variable names

colnames(dat2) <- c("oxygen","hydrogen","species")
m2a <- lm(species ~ oxygen*hydrogen, data = dat2)
m2aps1 <- plotSlopes(m2a, plotx = "oxygen", modx = "hydrogen")
m2aps1ts <- testSlopes(m2aps1)
plot(m2aps1ts)
m2aps2 <- plotSlopes(m2a, plotx = "oxygen", modx = "hydrogen",
                     modxVals = "std.dev", n = 5)
m2bps2ts <- testSlopes(m2aps2)
plot(m2bps2ts)



dat3 <- genCorrelatedData(N = 400, rho = .1, stde = 300,
                          beta = c(2, 0, 0.3, 0.15),
                          means = c(50,0), sds = c(10, 40))
m3 <- lm(y ~ x1*x2, data = dat3)
m3ps <- plotSlopes(m3, plotx = "x1", modx = "x2")
m3sts <- testSlopes(m3ps)
plot(testSlopes(m3ps))
plot(testSlopes(m3ps), shade = FALSE)

## Finally, if model has no relevant interactions, testSlopes does nothing.
m9 <- lm(statusquo ~ age + income * education + sex + age, data = Chile)
m9ps <- plotSlopes(m9, modx = "education", plotx = "age", plotPoints = FALSE)
m9psts <- testSlopes(m9ps)
}
\references{
Preacher, Kristopher J, Curran, Patrick J.,and Bauer, Daniel J. (2006).
Computational Tools for Probing Interactions in Multiple Linear
Regression, Multilevel Modeling, and Latent Curve Analysis.
Journal of Educational and Behavioral Statistics. 31,4, 437-448.

Johnson, P.O. and Neyman, J. (1936). "Tests of certain linear
hypotheses and their applications to some educational problems.
Statistical Research Memoirs, 1, 57-93.
}
\seealso{
plotSlopes
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outreg.R
\name{outreg}
\alias{outreg}
\title{Creates a publication quality result table for
regression models. Works with models fitted with lm, glm, as well
as lme4.}
\usage{
outreg(modelList, type = "latex", modelLabels = NULL,
  varLabels = NULL, tight = TRUE, centering = c("none", "siunitx",
  "dcolumn"), showAIC = FALSE, float = FALSE, request, runFuns,
  digits = 3, alpha = c(0.05, 0.01, 0.001), SElist = NULL,
  PVlist = NULL, Blist = NULL, title, label, gofNames,
  print.results = TRUE, browse = identical(type, "html") &&
  interactive())
}
\arguments{
\item{modelList}{A regression model or an R list of regression
models. Default model names will be M1, M2, and so forth. User
specified names are allowed, such as \code{list("My Model" =
m1, "Her Model" = m2)}.  This is the currently recommended way
to supply model lables. This is less error prone than the use
of the modelLabels argument.}

\item{type}{Default = "latex". The alternatives are "html" and
"csv"}

\item{modelLabels}{This is allowed, but discouraged. A vector of
character string variables, one for each element in
modelList. Will override the names in modelList.}

\item{varLabels}{To beautify the parameter names printed.  Must be
a named vector in the format c(parmname = "displayName",
parmname = "displayName"). Include as many parameters as
desired, it is not necessary to supply new labels for all of
the parameters.}

\item{tight}{Table format. If TRUE, parameter estimates and
standard errors are printed in a single column.  If FALSE,
parameter estimates and standard errors are printed side by
side.}

\item{centering}{Default is "none", but may be "siunitx" or
"dcolumn". No centering has been the only way until this
version. User feedback requested.  Don't forget to insert
usepackage statment in document preamble for siunitx or
dcolumn. If user specifies \code{centering=TRUE}, the
\code{siunitx} method will be used. The \code{dcolumn}
approach assumes that the values reported in the column use
fewer than 3 integer places and 3 decimal places. Additional
room is allocated for the significance stars.}

\item{showAIC}{This is a legacy argument, before the
\code{request} argument was created.  If TRUE, the AIC
estimate is included with the diagnostic values. It has the
same effect as described by \code{request}.}

\item{float}{Default = FALSE. Include boilerplate for a LaTeX
table float, with the tabular markup inside it. Not relevant
if type = "html".}

\item{request}{Extra information to be retrieved from the
summary(model) and displayed. This must be a vector of named
arguments, such as c(adj.r.squared = "adj $R^2$", fstatistic =
"F"). The name must be a valid name of the output object, the
value should be the label the user wants printed in the
table. See details.}

\item{runFuns}{A list of functions}

\item{digits}{Default = 3. How many digits after decimal sign are
to be displayed.}

\item{alpha}{Default = c(0.05, 0.01, 0.001). I think stars are
dumb, but enough people have asked me for more stars that I'm
caving in.}

\item{SElist}{Optional. Replacement standard errors. Must be a
list of named vectors. \code{outreg} uses the R \code{summary}
to retrieve standard errors, but one might instead want to use
robust or bootstrapped standard errors.  This argument may
supply a new SE vector for each fitted regression model, but
it is also allowed to supply the SE replacement for just one
of the models. The format should be \code{list("A Model Label"
= c(0.1, 0.3, 0.4), "Another Model Label" = c(0.4, 0.2, 0.3)}.
On the left, one must use the same names that are used in the
modelList argument.}

\item{PVlist}{Optional. A list of replacement "p values". It must
be a list of named vectors, similar in format to SElist. The
which the elements are the "p values" that the user wants to
use for each model.}

\item{Blist}{Optional. This is only needed in the rare case where
a model's parameters cannot be discerned from its
summary. List must have names for models, and vectors slope
coefficient. See discussion of SElist and PVlist.}

\item{title}{A LaTeX caption for the table. Not relevant if type =
"html".}

\item{label}{A string to be used as a LaTeX label in the table to
be created. Not relevant if type = "html".}

\item{gofNames}{Optional pretty names. R regression summaries use
names like "sigma" or "r.squared" that we might want to revise
for presentation. I prefer to refer to "sigma" as "RMSE", but
perhaps you instead prefer something like \code{gofnames =
c("sigma" = "That Estimate I don't understand", "deviance" =
"Another Mystery")}. The words that you might replace are
"sigma", "r.squared", "deviance", "adj.r.squared",
"fstatistic".}

\item{print.results}{Default TRUE, marked-up table will be
displayed in session.  If FALSE, same result is returned as an
object.}

\item{browse}{Display the regression model in a browse? Defaults
to TRUE if type = "html"}
}
\value{
A character vector, one element per row of the regression
    table.
}
\description{
This provides "markup" that the user is will copy into a LaTeX
document. As of rockchalk 1.8.4, can also create HTML markup.
The rockchalk vignette demonstrates use of outreg in Sweave.
}
\details{
\code{outreg} returns a string vector. It is suggested that users
should save the outreg result and then use cat to save it. That is
myMod <- outreg(m1, ...)  cat(myMod, file = "myMod.html") or
cat(myMod, file = "myMod.tex".  In version 1.8.66, we write the
html file to a temporary location and display it in a web
browser. Many word processors will not accept a cut-and paste
transfer from the browser, they will, however, be able to open the
html file itself and automatically re-format it in the native
table format.

In version 1.8.111, an argument \code{print.results} was introduced.
This is TRUE by default, so the marked-up table is printed into
the session, and it is returned as well.  If the function should
run silently (as suggested in the last few versions), include
\code{print.results = TRUE}.

The table includes a minimally sufficient (in my opinion) model
summary.  It offers parameter estimates, standard errors, and
minimally sufficient goodness of fit.  My tastes tend toward
minimal tables, but users request more features, and
\code{outreg}'s interface hass been generalized to allow
specialized requests. See \code{request} and \code{runFuns}
arguments.

I don't want to write a separate table function for every
different kind of regression model that exists (how
exhausting). So I've tried to revise \code{outreg()} to work with
regression functions that follow the standard R framework. It is
known to work \code{lm} and \code{glm}, as well as \code{merMod}
class from \code{lme4}, but it will try to interact with other
kinds of regression models.  Those models should have methods
\code{summary()}, \code{coef()}, \code{vcov()} and \code{nobs()}.
Package writes should provide those, its not my job.

Do you want "robust standard errors"? P values calculated
according to some alternative logic?  Go ahead, calculate them in
your code, outreg will now accept them as arguments. As of Version
1.8.4, users can provide their own standard errors and/or p-values
for each model. Thus, if a model answers in the usual way to the
standard R request \code{coef(summary(model))}, outreg can work if
users supply standard errors.

About the customizations \code{request}.  The \code{request}
argument supplies a list of names of summary output elements that
are desired. The format is a pair, a value to be retrieved from
\code{summary(model)}, and a pretty name to be printed for
it. With the \code{lm()} regression, for example, one might want
the output of the F test and the adjusted R-square: Include
\code{request = c(adj.r.squared = "adj. $R^2$", "fstatistic" =
"F")}. The value on the left is the name of the desired
information in the summary object, while the value on the right is
\emph{any} valid LaTeX (or HTML) markup that the user desires to
display in the table. \code{request} terms that generate a single
numerical value will generally work fine, while requests that ask
for more structured information, such as the F test (including the
2 degrees of freedom values) may work (user feedback needed).

The \code{runFuns} argument is inspired by a user request: could
this include the BIC or other summaries that can be easily
calculated?  Any R function, such as \code{AIC} or \code{BIC},
should work, as long as it returns a single value.  This is a
two-part specification, a function name and a pretty label to be
used in printing. For example, \code{runFuns = c("AIC" = "Akaike
Criterion", "BIC" = "Schwartz Criterion", "logLik" = "LL")}.

About centering with dcolumn or siunitx. It appears now that
results are better with \code{siunitx} but \code{dcolumn} is more
familiar to users.  The user has the duty to make sure that the
document preamble includes the correct package,
\code{\\usepackage\{dcolumn\}} or \code{\\usepackage\{siunitx\}}.
In this version, I have eliminated the need for the user to
specify document-wide settings for \code{siunitx}. All of the
details are explicitly written in the header of each tabular.
It is done that way to more easily allow user customizations.
}
\note{
There are many R packages that can be used to create LaTeX
    regression tables. memisc, texreg, apsrtable, xtables, and rms
    are some. This "outreg" version was in use in our labs before
    we were aware that those packages were in development. It is
    not intended as a competitor, it is just a slightly different
    version of the same that is more suited to our needs.
}
\examples{
set.seed(2134234)
dat <- data.frame(x1 = rnorm(100), x2 = rnorm(100))
dat$y1 <- 30 + 5 * rnorm(100) + 3 * dat$x1 + 4 * dat$x2
dat$y2 <- rnorm(100) + 5 * dat$x2
m1 <- lm(y1 ~ x1, data = dat)
m2 <- lm(y1 ~ x2, data = dat)
m3 <- lm(y1 ~ x1 + x2, data = dat)
gm1 <- glm(y1 ~ x1, family = Gamma, data = dat)
outreg(m1, title = "My One Tightly Printed Regression", float = TRUE)
ex1 <- outreg(m1, title = "My One Tightly Printed Regression",
               float = TRUE, print.results = FALSE, centering = "siunitx")
## Show markup, Save to file with cat()
cat(ex1)
## cat(ex1, file = "ex1.tex")
 
ex2 <- outreg(list("Fingers" = m1), tight = FALSE, 
    title = "My Only Spread Out Regressions", float = TRUE,
    alpha = c(0.05, 0.01, 0.001)) 

ex3 <- outreg(list("Model A" = m1, "Model B label with Spaces" = m2),
    varLabels = list(x1 = "Billie"), 
    title = "My Two Linear Regressions", request = c(fstatistic = "F"),
    print.results = TRUE)
cat(ex3)

ex4 <- outreg(list("Model A" = m1, "Model B" = m2),
    modelLabels = c("Overrides ModelA", "Overrides ModelB"),
    varLabels = list(x1 = "Billie"),
    title = "Note modelLabels Overrides model names")
cat(ex4)
##'
ex5 <- outreg(list("Whichever" = m1, "Whatever" = m2),
    title = "Still have showAIC argument, as in previous versions",
    showAIC = TRUE, float = TRUE, centering = "siunitx")

ex5s <- outreg(list("Whichever" = m1, "Whatever" = m2),
    title = "Still have showAIC argument, as in previous versions",
    showAIC = TRUE, float = TRUE, centering = "siunitx")

\donttest{
## Launches HTML browse
ex5html <- outreg(list("Whichever" = m1, "Whatever" = m2),
    title = "Still have showAIC argument, as in previous versions",
    showAIC = TRUE, type = "html")
## Could instead, make a file:
## fn <- "some_name_you_choose.html"
## cat(ex5html, file = fn)
## browseURL(fn)
## Open that HTML file in LibreOffice or MS Word
}

ex6 <- outreg(list("Whatever" = m1, "Whatever" =m2),
    title = "Another way to get AIC output",
    runFuns = c("AIC" = "Akaike IC"))
cat(ex6)

ex7 <- outreg(list("Amod" = m1, "Bmod" = m2, "Gmod" = m3),
       title = "My Three Linear Regressions", float = FALSE)
cat(ex7)

## A new feature in 1.85 is ability to provide vectors of beta estimates
## standard errors, and p values if desired. 
## Suppose you have robust standard errors!
if (require(car)){
   newSE <- sqrt(diag(car::hccm(m3)))
   ex8 <- outreg(list("Model A" = m1, "Model B" = m2, "Model C" = m3, "Model C w Robust SE" = m3),
        SElist= list("Model C w Robust SE" = newSE))
   cat(ex8)
}

ex11 <- outreg(list("I Love Long Titles" = m1,
          "Prefer Brevity" = m2,
          "Short" = m3), tight = FALSE, float = FALSE)
cat(ex11)
##'
ex12 <- outreg(list("GLM" = gm1), float = TRUE)
cat(ex12)

ex13 <- outreg(list("OLS" = m1, "GLM" = gm1), float = TRUE,
        alpha = c(0.05, 0.01))
cat(ex13)
##'
ex14 <- outreg(list(OLS = m1, GLM = gm1), float = TRUE,
    request = c(fstatistic = "F"), runFuns = c("BIC" = "BIC"))
cat(ex14)
ex15 <- outreg(list(OLS = m1, GLM = gm1), float = TRUE,
    request = c(fstatistic = "F"), runFuns = c("BIC" = "BIC"),
    digits = 5, alpha = c(0.01))

ex16 <- outreg(list("OLS 1" = m1, "OLS 2" = m2,  GLM = gm1), float = TRUE,
    request = c(fstatistic = "F"),
    runFuns = c("BIC" = "BIC", logLik = "ll"),
    digits = 5, alpha = c(0.05, 0.01, 0.001))

ex17 <- outreg(list("Model A" = gm1, "Model B label with Spaces" = m2),
    request = c(fstatistic = "F"),
    runFuns = c("BIC" = "Schwarz IC", "AIC" = "Akaike IC",
    "nobs" = "N Again?"))

## Here's a fit example from lme4.
if (require(lme4) && require(car)){
  fm1 <- lmer(Reaction ~ Days + (Days | Subject), sleepstudy)
  ex18 <- outreg(fm1)
  cat(ex18)
  ## Fit same with lm for comparison
  lm1 <- lm(Reaction ~ Days, sleepstudy)
  ## Get robust standard errors
  lm1rse <- sqrt(diag(car::hccm(lm1)))

  if(interactive()){
  ex19 <- outreg(list("Random Effects" = fm1, 
       "OLS" = lm1, "OLS Robust SE" = lm1),
       SElist = list("OLS Robust SE" = lm1rse), type = "html")
  }
  ## From the glmer examples
  gm2 <- glmer(cbind(incidence, size - incidence) ~ period + (1 | herd),
                   data = cbpp, family = binomial)
  lm2 <- lm(incidence/size ~ period,  data = cbpp)
  lm2rse <- sqrt(diag(car::hccm(lm2)))
  ## Lets see what MASS::rlm objects do? Mostly OK
  rlm2 <- MASS::rlm(incidence/size ~ period, data = cbpp)
  \donttest{
  ex20 <- outreg(list("GLMER" = gm2, "lm" = lm2, "lm w/robust se" = lm2,
            "rlm" = rlm2), SElist = list("lm w/robust se" = lm2rse),
            type = "html")
  } 
}
}
\author{
Paul E. Johnson \email{<pauljohn@ku.edu>}
}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lazyCov.R
\name{lazyCov}
\alias{lazyCov}
\title{Create covariance matrix from correlation and standard deviation
information}
\usage{
lazyCov(Rho, Sd, d)
}
\arguments{
\item{Rho}{Required. May be a single value (correlation common
among all variables), a vector of the lower triangular values
(vech) of a correlation matrix, or a symmetric matrix of
correlation coefficients.}

\item{Sd}{Required. May be a single value (standard deviation
common among all variables) or a vector of standard deviations,
one for each variable.}

\item{d}{Optional. Number of rows or columns. lazyCov may be able
to deduce the required dimension of the final matrix from the
input. However, when the user supplies only a single value for
both Rho and Sd, d is necessary.}
}
\value{
covariance matrix.
}
\description{
This is a flexible function that allows lazy R programmers to
create covariance matrix. The user may be lazy because the
correlation and standard deviation infomation may be supplied in a
variety of formats.
}
\examples{
##correlation 0.8 for all pairs, standard deviation 1.0 of each
lazyCov(Rho = 0.8, Sd = 1.0, d = 3)
## supply a vech (lower triangular values in a column)
lazyCov(Rho = c(0.1, 0.2, 0.3), Sd = 1.0)
## supply vech with different standard deviations
lazyCov(Rho = c(0.1, 0.2, 0.3), Sd = c(1.0, 2.2, 3.3))
newRho <- lazyCor(c(0.5, 0.6, 0.7, -0.1, 0.1, 0.2))
lazyCov(Rho = newRho, Sd = 1.0)
lazyCov(Rho = newRho, Sd = c(3, 4, 5, 6))
}
\author{
<pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictOMatic.R
\name{predictCI}
\alias{predictCI}
\title{Calculate a predicted value matrix (fit, lwr, upr) for a
regression, either lm or glm, on either link or response scale.}
\usage{
predictCI(object, newdata = NULL, type = c("response", "link"),
  interval = c("none", "confidence", "prediction"), dispersion = NULL,
  scale = NULL, na.action = na.pass, level = 0.95, ...)
}
\arguments{
\item{object}{Regression object, class must include glm or lm.}

\item{newdata}{Data frame including focal values for predictors}

\item{type}{One of c("response", "link"), defaults to former.}

\item{interval}{One of c("none", "confidence",
"prediction"). "prediction" is defined only for lm objects, not
for glm.}

\item{dispersion}{Will be estimated if not provided. The variance
coefficient of the glm, same as scale squared. Dispersion is
allowed as an argument in predict.glm.}

\item{scale}{The square root of dispersion. In an lm, this is the
RMSE, called sigma in summary.lm.}

\item{na.action}{What to do with missing values}

\item{level}{Optional. Default = 0.95.  Specify whatever
confidence level one desires.}

\item{...}{Other arguments to be passed to predict}
}
\value{
c(fit, lwr, upr), and possibly more.
}
\description{
This adapts code from predict.glm and predict.lm. I eliminated
type = "terms" from consideration.
}
\details{
R's predict.glm does not have an interval argument. There are
about 50 methods to calculate CIs for predicted values of GLMs,
that's a major worry. This function takes the simplest route, calculating the
(fit, lwr, upr) in the linear predictor scale, and then if type=
"response", those 3 columns are put through linkinv().  This is
the same method that SAS manuals suggest they use, same as Ben
Bolker suggests in r-help (2010).  I'd rather use one of the
fancy tools like Edgeworth expansion, but that R code is not
available (but is promised).

Use predict.lm with se.fit = TRUE to calculate fit and se.fit.
Then calculate lwr and upr as fit +/- tval * se.fit. If model is
lm, the model df.residual will be used to get tval. If glm, this
is a normal approximation, so we thugishly assert tval = 1.98.

There's some confusing term translation. I wish R lm and glm would
be brought into line. For lm, residual.scale = sigma. For glm,
residual.scale = sqrt(dispersion)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/descriptiveTable.R
\name{descriptiveTable}
\alias{descriptiveTable}
\title{Summary stats table-maker for regression users}
\usage{
descriptiveTable(object, stats = c("mean", "sd", "min", "max"),
  digits = 4, probs = c(0, 0.5, 1), varLabels, ...)
}
\arguments{
\item{object}{A fitted regression or an R data.frame, or any
other object type that does not fail in code{model.frame(object)}.}

\item{stats}{Default is a vector c("mean", "sd", "min", "max"). Other
stats reported by rockchalk::summarize should work fine as well}

\item{digits}{2 decimal points is default}

\item{probs}{Probability cut points to be used in the calculation
of summaries of numeric variables.  Default is c(0, 0.5, 1), meaning
\code{min, median, max}.}

\item{varLabels}{A named vector of variables labels, as in outreg function.
Format is c("oldname"="newlabel").}

\item{...}{Other arguments passed to rockchalk::summarizeNumerics and
summarizeFactors.}
}
\value{
a character matrix
}
\description{
rockchalk::summarize does the numerical calculations
}
\details{
This is, roughly speaking, doing the right thing, but
not in a clever way. For the categorical variables, the
only summary is proportions.
}
\examples{
dat <- genCorrelatedData2(1000, means=c(10, 10, 10), sds = 3, 
                          stde = 3, beta = c(1, 1, -1, 0.5))
dat$xcat1 <- factor(sample(c("a", "b", "c", "d"), 1000, replace=TRUE))
dat$xcat2 <- factor(sample(c("M", "F"), 1000, replace=TRUE), levels = c("M", "F"),
labels = c("Male", "Female"))
dat$y <- dat$y + contrasts(dat$xcat1)[dat$xcat1, ] \%*\% c(0.1, 0.2, 0.3)
m4 <- lm(y ~ x1 + x2  + x3 + xcat1 + xcat2, dat)
m4.desc <- descriptiveTable(m4)
m4.desc
## Following may cause scientific notation, want to avoid.
dat <- genCorrelatedData2(1000, means=c(10, 100, 400), 
                 sds = c(3, 10, 20), stde = 3, beta = c(1, 1, -1, 0.5))
m5 <- lm(y ~ x1 + x2  + x3, dat)
m5.desc <- descriptiveTable(m5, digits = 4)
m5.desc

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recodeFactors.R
\name{combineLevels}
\alias{combineLevels}
\title{recode a factor by "combining" levels}
\usage{
combineLevels(fac, levs, newLabel = "combinedLevels")
}
\arguments{
\item{fac}{An R factor variable, either ordered or not.}

\item{levs}{The levels to be combined. Users may specify either a
numerical vector of level values, such as c(1,2,3), to combine the
first three elements of level(fac), or they may specify level
names.  This can be done as a character vector of *correctly
spelled* factor values, such as c("Yes","Maybe","Always") or it
may be provided as a subset of the output from levels, such as
levels(fac)[1:3].}

\item{newLabel}{A character string that represents the label of
the new level to be created when \code{levs} values are combined.}
}
\value{
A new factor variable, with unused levels removed.
}
\description{
This makes it easy to put levels together and create a new factor
variable. If a
factor variable is currently coded with levels
c("Male","Female","Man", "M"), and the user needs to combine the
redundant levels for males, this is the function to use!  This is a surprisingly difficult problem in R.
}
\details{
If the factor is an ordinal factor, then levels may be combined
only if they are adjacent. A factor with levels
c("Lo","Med","Hi","Extreme") allows us to combine responses "Lo"
and "Med", while it will NOT allow us to combine "Lo" with "Hi".

A non-ordered factor can be reorganized to combine any values, no
matter what positions they occupy in the levels vector.
}
\examples{
x <- c("M","A","B","C","A","B","A","M")
x <- factor(x)
levels(x)
x2a <- combineLevels(x, levs = c("M","A"), newLabel = "M_or_A")
addmargins(table(x2a, x, exclude=NULL))
x2b <- combineLevels(x, c(1,4), "M_or_A")
addmargins(table(x2b, x, exclude=NULL))
x3 <- combineLevels(x, levs = c("M","A","C"), newLabel = "MAC")
addmargins(table(x3, x, exclude=NULL))
## Now an ordinal factor
z <- c("M","A","B","C","A","B","A","M")
z <- ordered(z)
levels(z)
table(z, exclude=NULL)
z2a <-  combineLevels(z, levs = c(1,2), "Good")
addmargins(table(z2a, z, exclude = NULL))
z2b <- combineLevels(z, levs = c("A","B"), "AorB")
addmargins(table(z2b, z, exclude = NULL))

}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictOMatic.R
\name{newdata}
\alias{newdata}
\alias{newdata.default}
\title{Create a newdata frame for usage in predict methods}
\usage{
newdata(model, predVals, n, ...)

\method{newdata}{default}(model = NULL, predVals = NULL, n = 3,
  emf = NULL, divider = "quantile", ...)
}
\arguments{
\item{model}{Required. Fitted regression model}

\item{predVals}{Predictor Values that deserve investigation.
Previously, the argument was called "fl".  This can be 1) a
keyword, one of c("auto", "margins") 2) a vector
of variable names, which will use default methods for all named
variables and the central values for non-named variabled, 3) a
named vector with predictor variables and divider algorithms, or
4) a full list that supplies variables and possible values. Please
see details and examples.}

\item{n}{Optional. Default = 3. How many focal values are desired?
This value is used when various divider algorithms are put to use
if the user has specified keywords "default", "quantile", "std.dev."
"seq", and "table".}

\item{...}{Other arguments.}

\item{emf}{Optional. data frame used to fit model (not a model
frame, which may include transformed variables like
log(x1). Instead, use output from function \code{model.data}). It
is UNTRANSFORMED variables ("x" as opposed to poly(x,2).1 and
poly(x,2).2).}

\item{divider}{Default is "quantile". Determines the method of
selection. Should be one of c("quantile", "std.dev", "seq", "table").}
}
\value{
A data frame of x values that could be used as the
data = argument in the original regression model. The attribute
"varNamesRHS" is a vector of the predictor variable names.
}
\description{
This is a generic function. The default method covers almost all
regression models.
}
\details{
It scans the fitted model, discerns the names of the predictors,
and then generates a new data frame.  It can guess values of the
variables that might be substantively interesting, but that
depends on the user-supplied value of predVals.  If not supplied
with a predVals argument, newdata returns a data frame with one
row -- the central values (means and modes) of the variables in
the data frame that was used to fit the model. The user can supply
a keyword "auto" or "margins". The function will try to do the
"right thing."

The \code{predVals} can be a named list that supplies specific
values for particular predictors.  Any legal vector of values is
allowed. For example, \code{predVals = list(x1 = c(10, 20, 30), x2
= c(40, 50), xcat = levels(xcat)))}. That will create a newdata
object that has all of the "mix and match" combinations for those
values, while the other predictors are set at their central
values.

If the user declares a variable with the "default" keyword, then
the default divider algorithm is used to select focal values.  The
default divider algorithm is an optional argument of this
function. If the default is not desired, the user can specify a
divider algorithm by character string, either "quantile",
"std.dev.", "seq", or "table".  The user can mix and match
algorithms along with requests for specific focal values, as in
\code{predVals = list(x1 = "quantile", x2 = "std.dev.", x3 = c(10,
20, 30), xcat1 <- levels(xcat1))}
}
\examples{
library(rockchalk)

## Replicate some R classics.  The budworm.lg data from predict.glm
## will work properly after re-formatting the information as a data.frame:

## example from Venables and Ripley (2002, pp. 190-2.)
df <- data.frame(ldose = rep(0:5, 2),
                 sex = factor(rep(c("M", "F"), c(6, 6))),
                 SF.numdead = c(1, 4, 9, 13, 18, 20, 0, 2, 6, 10, 12, 16))
df$SF.numalive = 20 - df$SF.numdead

budworm.lg <- glm(cbind(SF.numdead, SF.numalive) ~ sex*ldose,
                  data = df,  family = binomial)

predictOMatic(budworm.lg)

predictOMatic(budworm.lg, n = 7)

predictOMatic(budworm.lg, predVals = c("ldose"), n = 7)

predictOMatic(budworm.lg, predVals = c(ldose = "std.dev.", sex = "table"))



## Now make up a data frame with several numeric and categorical predictors.

set.seed(12345)
N <- 100
x1 <- rpois(N, l = 6)
x2 <- rnorm(N, m = 50, s = 10)
x3 <- rnorm(N)
xcat1 <- gl(2,50, labels = c("M","F"))
xcat2 <- cut(rnorm(N), breaks = c(-Inf, 0, 0.4, 0.9, 1, Inf),
             labels = c("R", "M", "D", "P", "G"))
dat <- data.frame(x1, x2, x3, xcat1, xcat2)
rm(x1, x2, x3, xcat1, xcat2)
dat$xcat1n <- with(dat, contrasts(xcat1)[xcat1, , drop = FALSE])
dat$xcat2n <- with(dat, contrasts(xcat2)[xcat2, ])
STDE <- 15
dat$y <- with(dat,
              0.03 + 0.8*x1 + 0.1*x2 + 0.7*x3 + xcat1n \%*\% c(2) +
              xcat2n \%*\% c(0.1,-2,0.3, 0.1) + STDE*rnorm(N))
## Impose some random missings
dat$x1[sample(N, 5)] <- NA
dat$x2[sample(N, 5)] <- NA
dat$x3[sample(N, 5)] <- NA
dat$xcat2[sample(N, 5)] <- NA
dat$xcat1[sample(N, 5)] <- NA
dat$y[sample(N, 5)] <- NA
summarize(dat)


m0 <- lm(y ~ x1 + x2 + xcat1, data = dat)
summary(m0)
## The model.data() function in rockchalk creates as near as possible
## the input data frame.
m0.data <- model.data(m0)
summarize(m0.data)

## no predVals: analyzes each variable separately
(m0.p1 <- predictOMatic(m0))

## requests confidence intervals from the predict function
(m0.p2 <- predictOMatic(m0, interval = "confidence"))

## predVals as vector of variable names: gives "mix and match" predictions
(m0.p3 <- predictOMatic(m0, predVals = c("x1", "x2")))

## predVals as vector of variable names: gives "mix and match" predictions
(m0.p3s <- predictOMatic(m0, predVals = c("x1", "x2"), divider = "std.dev."))

## "seq" is an evenly spaced sequence across the predictor.
(m0.p3q <- predictOMatic(m0, predVals = c("x1", "x2"), divider = "seq"))

(m0.p3i <- predictOMatic(m0, predVals = c("x1", "x2"),
                         interval = "confidence", n = 3))

(m0.p3p <- predictOMatic(m0, predVals = c("x1", "x2"), divider = pretty))

## predVals as vector with named divider algorithms.
(m0.p3 <- predictOMatic(m0, predVals = c(x1 = "seq", x2 = "quantile")))
## predVals as named vector of divider algorithms

## same idea, decided to double-check
(m0.p3 <- predictOMatic(m0, predVals = c(x1 = "quantile", x2 = "std.dev.")))
getFocal(m0.data$x2, xvals =  "std.dev.", n = 5)


## Change from quantile to standard deviation divider
(m0.p5 <- predictOMatic(m0, divider = "std.dev.", n = 5))

## Still can specify particular values if desired
(m0.p6 <- predictOMatic(m0, predVals = list("x1" = c(6,7),
                            "xcat1" = levels(m0.data$xcat1))))

(m0.p7 <- predictOMatic(m0, predVals = c(x1 = "quantile", x2 = "std.dev.")))
getFocal(m0.data$x2, xvals =  "std.dev.", n = 5)

(m0.p8 <- predictOMatic(m0, predVals = list( x1 = quantile(m0.data$x1,
                        na.rm = TRUE, probs = c(0, 0.1, 0.5, 0.8,
                        1.0)), xcat1 = levels(m0.data$xcat1))))

(m0.p9 <- predictOMatic(m0, predVals = list(x1 = "seq", "xcat1" =
                                levels(m0.data$xcat1)), n = 8) )


(m0.p10 <- predictOMatic(m0, predVals = list(x1 = "quantile",
                                 "xcat1" = levels(m0.data$xcat1)), n =  5) )


(m0.p11 <- predictOMatic(m0, predVals = c(x1 = "std.dev."), n = 10))

## Previous same as

(m0.p11 <- predictOMatic(m0, predVals = c(x1 = "default"), divider =
                         "std.dev.", n = 10))

## Previous also same as

(m0.p11 <- predictOMatic(m0, predVals = c("x1"), divider = "std.dev.", n = 10))


(m0.p11 <- predictOMatic(m0,  predVals = list(x1 = c(0, 5, 8), x2 = "default"),
                         divider = "seq"))



m1 <- lm(y ~ log(10+x1) + sin(x2) + x3, data = dat)
m1.data <- model.data(m1)
summarize(m1.data)


(newdata(m1))
(newdata(m1, predVals = list(x1 = c(6, 8, 10))))
(newdata(m1, predVals = list(x1 = c(6, 8, 10), x3 = c(-1,0,1))))
(newdata(m1, predVals = list(x1 = c(6, 8, 10),
                 x2 = quantile(m1.data$x2, na.rm = TRUE), x3 = c(-1,0,1))))

(m1.p1 <- predictOMatic(m1, divider = "std.dev", n = 5))
(m1.p2 <- predictOMatic(m1, divider = "quantile", n = 5))

(m1.p3 <- predictOMatic(m1, predVals = list(x1 = c(6, 8, 10),
                            x2 = median(m1.data$x2, na.rm = TRUE))))

(m1.p4 <- predictOMatic(m1, predVals = list(x1 = c(6, 8, 10),
                                x2 = quantile(m1.data$x2, na.rm = TRUE))))

(m1.p5 <- predictOMatic(m1))
(m1.p6 <- predictOMatic(m1, divider = "std.dev."))
(m1.p7 <- predictOMatic(m1, divider = "std.dev.", n = 3))
(m1.p8 <- predictOMatic(m1, divider = "std.dev.", interval = "confidence"))


m2 <- lm(y ~ x1 + x2 + x3 + xcat1 + xcat2, data = dat)
##  has only columns and rows used in model fit
m2.data <- model.data(m2)
summarize(m2.data)

## Check all the margins
(predictOMatic(m2, interval = "conf"))

## Lets construct predictions the "old fashioned way" for comparison

m2.new1 <- newdata(m2, predVals = list(xcat1 = levels(m2.data$xcat1),
                           xcat2 = levels(m2.data$xcat2)), n = 5)
predict(m2, newdata = m2.new1)


(m2.p1 <- predictOMatic(m2,
                        predVals = list(xcat1 = levels(m2.data$xcat1),
                            xcat2 = levels(m2.data$xcat2)),
                        xcat2 = c("M","D")))
## See? same!

## Pick some particular values for focus
m2.new2 <- newdata(m2, predVals = list(x1 = c(1,2,3), xcat2 = c("M","D")))
## Ask for predictions
predict(m2, newdata = m2.new2)


## Compare: predictOMatic generates a newdata frame and predictions in one step

(m2.p2 <- predictOMatic(m2, predVals = list(x1 = c(1,2,3),
                                xcat2 = c("M","D"))))

(m2.p3 <- predictOMatic(m2, predVals = list(x2 = c(0.25, 1.0),
                                xcat2 = c("M","D"))))

(m2.p4 <- predictOMatic(m2, predVals = list(x2 = plotSeq(m2.data$x2, 10),
                                xcat2 = c("M","D"))))

(m2.p5 <- predictOMatic(m2, predVals = list(x2 = c(0.25, 1.0),
                                xcat2 = c("M","D")), interval = "conf"))

(m2.p6 <- predictOMatic(m2, predVals = list(x2 = c(49, 51),
                                xcat2 = levels(m2.data$xcat2),
                                x1 = plotSeq(dat$x1))))

plot(y ~ x1, data = m2.data)
by(m2.p6, list(m2.p6$xcat2), function(x) {
    lines(x$x1, x$fit, col = x$xcat2, lty = as.numeric(x$xcat2))
})

m2.newdata <- newdata(m2, predVals = list(x2 = c(48, 50, 52),
                              xcat2 = c("M","D")))
predict(m2, newdata = m2.newdata)

(m2.p7 <- predictOMatic(m2, predVals = list(x2 = c(48, 50, 52),
                                xcat2 = c("M","D"))))

(m2.p8 <- predictOMatic(m2,
             predVals = list(x2 = range(m2.data$x2, na.rm = TRUE),
             xcat2 = c("M","D"))))

(m2.p9 <- predictOMatic(m2, predVals = list(x2 = plotSeq(m2.data$x2),
             x1 = quantile(m2.data$x1, pr =c(0.33, 0.66), na.rm = TRUE),
             xcat2 = c("M","D"))))
plot(y ~ x2 , data = m2.data)

by(m2.p9, list(m2.p9$x1, m2.p9$xcat2), function(x) {lines(x$x2, x$fit)})



(predictOMatic(m2, predVals = list(x2 = c(50, 60), xcat2 = c("M","D")),
               interval = "conf"))

## create a dichotomous dependent variable
y2 <- ifelse(rnorm(N) > 0.3, 1, 0)
dat <- cbind(dat, y2)

m3 <- glm(y2 ~ x1 + x2 + x3 + xcat1, data = dat, family = binomial(logit))
summary(m3)
m3.data <- model.data(m3)
summarize(m3.data)

(m3.p1 <- predictOMatic(m3, divider = "std.dev."))

(m3.p2 <- predictOMatic(m3, predVals = list(x2 = c(40, 50, 60),
                             xcat1 = c("M","F")),
                        divider = "std.dev.", interval = "conf"))

## Want a full accounting for each value of x2?
(m3.p3 <- predictOMatic(m3,
                predVals = list(x2 = unique(m3.data$x2),
                    xcat1 = c("M","F")), interval = "conf"))


## Would like to write a more beautiful print method
## for output object, but don't want to obscure structure from user.
## for (i in names(m3.p1)){
##     dns <- cbind(m3.p1[[i]][i], m3.p1[[i]]$fit)
##     colnames(dns) <- c(i, "predicted")
##     print(dns)
## }


}
\seealso{
\code{predictOMatic}
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcGraph.R
\name{mcGraph1}
\alias{mcGraph1}
\alias{mcGraph2}
\alias{mcGraph3}
\title{Illustrate multicollinearity in regression, part 1.}
\usage{
mcGraph1(x1, x2, y, x1lab, x2lab, ylab, ...)

mcGraph2(x1, x2, y, rescaley = 1, drawArrows = TRUE, x1lab, x2lab,
  ylab, ...)

mcGraph3(x1, x2, y, interaction = FALSE, drawArrows = TRUE, x1lab,
  x2lab, ylab, ...)
}
\arguments{
\item{x1}{a predictor vector}

\item{x2}{a predictor vector}

\item{y}{the dependent variable}

\item{x1lab}{label for the x1 axis, (the one called "xlab" inside persp)}

\item{x2lab}{label for the x2 axis, (the one called "ylab" inside persp)}

\item{ylab}{label for the y (vertical) axis (the one called "zlab" inside persp)}

\item{...}{additional parameters passed to persp}

\item{rescaley}{a single scalar value or a vector of the same
length as y.}

\item{drawArrows}{TRUE or FALSE, do you want arrows from the plane to observed y?}

\item{interaction}{a TRUE or FALSE request for inclusion of the x1-x2 interaction in the regression calculation}
}
\value{
mcGraph1 and mcGraph2 return only the perspective matrix
from persp. It is returned so that users can add additional embellishments on the 3D plot (can be used with trans3d)

mcGraph3 returns a list of 2 objects. 1) the fitted
regression model2) the perspective matrix used with persp to draw
the image.
}
\description{
This is a set of functions that faciliates the examination
of multicollinearity. Suppose the "true" relationship is
 y[i] = 0.2 * x1[i] + 0.2 * x2[i] + e
where e is Normal(0, stde^2).

mcGraph1 draws the 3D regression space, but all of the points
are illustrated "in" the flat plane of x1-x2 variables.
}
\details{
These functions are specialized to a particular purpose. If you
just want to draw 3D regressions, look at plotPlane.
}
\examples{
set.seed(12345)
## Create data with x1 and x2 correlated at 0.10
dat <- genCorrelatedData(rho=.1, stde=7)

mcGraph1(dat$x1, dat$x2, dat$y, theta=20, phi=8, ticktype="detailed", nticks=10)

set.seed(12345)
## Create data with x1 and x2 correlated at 0.10
dat <- genCorrelatedData(rho=.1, stde=7)
## This will "grow" the "cloud" of points up from the
## x1-x2 axis
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.0, theta = 0)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.1, theta = 0)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.2, theta = 0)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.3, theta = 0)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.4, theta = 0)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.5, theta = 0)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.6, theta = 0)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.7, theta = 0)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.8, theta = 0)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 0.9, theta = 0)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 1, theta = 0)

##rotate this
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 1, theta = 20)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 1, theta = 40)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 1, theta = 60)
mcGraph2(dat$x1, dat$x2, dat$y, rescaley = 1, theta = 80)

## once they reach the top, make them glitter a while
for(i in 1:20){
  mcGraph2(dat$x1, dat$x2, dat$y, rescaley = runif(length(dat$x1), .9,1.1), theta = 0)
}

set.seed(12345)
## Create data with x1 and x2 correlated at 0.10
dat <- genCorrelatedData(rho=.1, stde=7)

mcGraph3(dat$x1, dat$x2, dat$y, theta = 0)

dat2 <- genCorrelatedData(rho = 0, stde = 7)

mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = 0, phi = 10)
mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = 30, phi = 10)
mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = -30, phi = 10)
mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = -30, phi = -10)
mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = -30, phi = -15)

## Run regressions with not-strongly correlated data
modset1 <- list()
for(i in 1:20){
  dat2 <- genCorrelatedData(rho = .1, stde = 7)
  summary(lm( y ~ x1 + x2 , data = dat2))
  modset1[[i]] <- mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = -30)
}


## Run regressions with strongly correlated data
modset2 <- list()
for(i in 1:20){
  dat2 <- genCorrelatedData(rho = .981, stde = 7)
  summary(lm( y ~ x1 + x2 , data = dat2))
  modset2[[i]] <- mcGraph3(dat2$x1, dat2$x2, dat2$y, theta = -30)
}

dat3 <- genCorrelatedData(rho = .981, stde = 100, beta=c(0.1, 0.2, 0.3, -0.1))
mcGraph3(dat3$x1, dat3$x2, dat3$y, theta=-10, interaction = TRUE)
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}
\keyword{hplot}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcDiagnose.R
\name{getPartialCor}
\alias{getPartialCor}
\title{Calculates partial correlation coefficients after retrieving data matrix froma fitted regression model}
\usage{
getPartialCor(model, dvonly = TRUE)
}
\arguments{
\item{model}{A fitted regression model, such as output from
lm(). Any object that has methods model.matrix and model.frame
will be sufficient.}

\item{dvonly}{Default = TRUE. Only show first column of the full
partial correlation matrix. That corresponds to the partial correlation of each predictor with y. I mean, r[yx].[others]}
}
\value{
A column or matrix of partial correlation coefficients
}
\description{
The input is a fitted regression model, from which the design
matrix is retrieved, along with the dependent variable. The
partial correlation is calculated using matrix algebra that
has not been closely inspected for numerical precision.  That is to
say, it is in the stats book style, rather than the numerically
optimized calculating format that functions like \code{lm()} have adopted.
}
\details{
I often criticize partial correlations because they change in a
very unstable way as terms are added or removed in regression
models. Nevertheless, I teach with books that endorse them, and in
order to have something to criticize, I need to have a function
like this. There are other packages that offer partial correlation
calculations, but they are either 1) not easy to install from CRAN
because of dependencies or 2) do not directly calculate the values
we want to see.

To students. 1) This gives the same result as the function
\code{cov2pcor} in \code{gRbase}, so far as I can tell. Why use
this?  Simply for convenenience.  We have found that installing
gRbase is a problem because it depends on packages in
Bioconductor.  2) By default, I show only one column of output,
the partial correlations involving the dependent variable as
something being explained. The other columns that would depict the
dependent variable as a predictor of the independent variables
have been omitted. You can let me know if you think that's wrong.

Please note I have not gone out of my way to make this calculation
"numerically stable."  It does not use any orthogonal matrix
calculations; it is using the same textbook theoretical stats
formula that is used by cov2pcor in gRbase and in every other
package or online source I could find. I prepared a little
WorkingExample file matrix-partial-correlations-1.R that
discusses this, in case you are interested (http://pj.freefaculty.org/R).
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{print.summarize}
\alias{print.summarize}
\title{print method for output from summarize}
\usage{
\method{print}{summarize}(x, digits, ...)
}
\arguments{
\item{x}{Object produced by summarize}

\item{digits}{Decimal values to display, defaults as 2.}

\item{...}{optional arguments for print function.}
}
\value{
x, unchanged
Prints objects created by summarize
}
\description{
Be aware that the unrounded numeric matrix
is available as an attribute of the returned object.
This method displays a rounded, character-formatted
display of the numeric varibles.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pctable.R
\name{pctable}
\alias{pctable}
\alias{pctable.default}
\alias{pctable.formula}
\alias{pctable.character}
\title{Creates a cross tabulation with counts and percentages}
\usage{
pctable(rv, ...)

\method{pctable}{default}(rv, cv, rvlab = NULL, cvlab = NULL,
  colpct = TRUE, rowpct = FALSE, rounded = FALSE, ...)

\method{pctable}{formula}(formula, data = NULL, rvlab = NULL,
  cvlab = NULL, colpct = TRUE, rowpct = FALSE, rounded = FALSE,
  ...)

\method{pctable}{character}(rv, cv, data = NULL, rvlab = NULL,
  cvlab = NULL, colpct = TRUE, rowpct = FALSE, rounded = FALSE,
  ...)
}
\arguments{
\item{rv}{A row variable name}

\item{...}{Other arguments. So far, the most likely additional
arguments are to be passed along to the table function, such as
"exclude", "useNA", or "dnn" (which will override the rvlab and
cvlab arguments provided by some methods). Some methods will
also pass along these arguments to model.frame, "subset" 
"xlev", "na.action", "drop.unused.levels".}

\item{cv}{Column variable}

\item{rvlab}{Optional: row variable label}

\item{cvlab}{Optional: col variable label}

\item{colpct}{Default TRUE: are column percentags desired in the
presentation of this result?}

\item{rowpct}{Default FALSE: are row percentages desired in the
presentation of this result}

\item{rounded}{Default FALSE, rounds to 10's for privacy purposes.}

\item{formula}{A two sided formula.}

\item{data}{A data frame.}
}
\value{
A list with tables (count, column percent, row percent) as
well as a copy of the call.
}
\description{
This function is pronounced "presentable"!  The original purpose
was to create a particular kind of cross tabulation that I ask for
in class: counts with column percentages. Requests from users have
caused a bit more generality to be built into the function. Now,
optionally, it will provide row percents. This is a generic function.
Most users will find the formula method most convenient. Use the
colpct and rowpct arguments to indicate if column or row percentages
are desired.

I suggest most users will use the formula method for this. Running
a command like this will, generally, do the right thing:

\code{tab <- pctable(y ~ x, data = dat)}

There is also a method that will work with characters representing
variable names.

\code{tab <- pctable("y", "x", data = dat)}
 
Running the function should write a table in the output console,
but it also creates an object (\code{tab}). That object
can be displayed in a number of ways.

A summary method is provided, so one could look at different
representations of the same table.

\code{summary(tab, rowpct = TRUE, colpct = FALSE)}

or

\code{summary(tab, rowpct = TRUE, colpct = TRUE)}

Tables that include only row or column percentages will be
compatible with the html and latex exporters in the
excellent \code{tables} package.

The formula method is the recommended method for users. Run
\code{pctable(myrow ~ mycol, data = dat)}. In an earlier version,
I gave different advice, so please adjust your usage.

The character method exists only for variety.  It accepts
character strings rather than a formula to define the columns that
should be plotted.  The method used most often for most users should
be the formula method.
}
\details{
Please bear in mind the following. The output object is a
list of tables of partial information, which are then assembled in
various ways by the print method (print.pctable). A lovely table
will appear on the screen, but the thing itself has more
information and a less beautiful structure.

A print method is supplied.
For any \code{pctable} object, it is possible to run follow-ups like

print(tab, rowpct = TRUE, colpct = FALSE)

The method \code{print.pctable(tab)} assembles the object into (my
opinion of) a presentable form. The print method has argumnets
\code{rowpct} and \code{colpct} that determine which percentages
are included in the presentation.

When using character arguments, the row variable rv rowvar must be
a quoted string if the user intends the method pctable.character
to be dispatched. The column variable cv may be a string or just a
variable name (which this method will coerce to a string).
}
\examples{
dat <- data.frame(x = gl(4, 25),
                  y = sample(c("A", "B", "C", "D", "E"), 100, replace= TRUE))
pctable(y ~ x, dat)
pctable(y ~ x, dat, exclude = NULL)
pctable(y ~ x, dat, rvlab = "My Outcome Var", cvlab = "My Columns")
pctable(y ~ x, dat, rowpct = TRUE, colpct = FALSE)
pctable(y ~ x, dat, rowpct = TRUE, colpct = TRUE)
pctable(y ~ x, dat, rowpct = TRUE, colpct = TRUE, exclude = NULL)
tab <- pctable(y ~ x, dat, rvlab = "Outcome", cvlab = "Predictor")
dat <- data.frame(x1 = gl(4, 25, labels = c("Good", "Bad", "Ugly", "Indiff")),
                x2 = gl(5, 20, labels = c("Denver", "Cincy", "Baltimore", "NY", "LA")), 
                y = sample(c("A", "B", "C", "D", "E"), 100, replace= TRUE))
tab <- pctable(y ~ x1, data = dat, rvlab = "my row label",
    subset = dat$x1 \%in\% c("Good", "Bad"),
    drop.unused.levels = TRUE)
tab <- pctable(y ~ x1, data = dat, rvlab = "my row label",
    subset = dat$x1 \%in\% c("Good", "Bad"))
pctable("y", "x1", dat)
pctable("y", x1, dat)
tab <- pctable(y ~ x2, data = dat, rvlab = "A Giant Variable")
summary(tab, rowpct = TRUE, colpct = FALSE)
tabsum <- summary(tab)
\donttest{
## if user has tables package, can push out to latex or html
if (require(tables) & require(Hmisc)){
    tabsumtab <- tables::as.tabular(tabsum)
    Hmisc::html(tabsumtab)
    fn <- tempfile(pattern = "file", tmpdir = tempdir(), 
            fileext = ".html")
    Hmisc::html(tabsumtab, file = fn)
    print(paste("The file saved was named", fn, "go get it."))
    if (interactive()) browseURL(fn)
    ## go get the fn file if you want to import it in document
    ## Now LaTeX output
    ## have to escape the percent signs
    tabsumtab <- apply(tabsumtab, 1:2, function(x) {gsub("\%", "\\\\\\\\\%", x) })
    fn2 <- tempfile(pattern = "file", tmpdir = tempdir(), 
                   fileext = ".tex")
    Hmisc::latex(tabsumtab, file = fn2)
    print(paste("The file saved was named", fn2, "go get it."))
}
}
}
\seealso{
\code{\link[tables]{tabular}} and the CrossTable function
in \code{gmodels} package.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/outreg.R
\name{outreg2HTML}
\alias{outreg2HTML}
\title{Convert LaTeX output from outreg to HTML markup}
\usage{
outreg2HTML(outreg, filename)
}
\arguments{
\item{outreg}{output from outreg}

\item{filename}{A file name into which the regression markup is to be saved. Should end in .html.}
}
\value{
A vector of strings
}
\description{
This function is deprecated. Instead, please use \code{outreg(type = "html")}
}
\details{
This will write the html on the screen, but if a filename argument is
supplied, it will write a file. One can
then open or insert the file into Libre Office or other popular
"word processor" programs.
}
\examples{
dat <- genCorrelatedData2(means = c(50,50,50,50,50,50),
    sds = c(10,10,10,10,10,10), rho = 0.2, beta = rnorm(7), stde = 50)
m1 <- lm(y ~ x1 + x2 + x3 + x4 + x5 + x6 + x1*x2, data = dat)
summary(m1)

m1out <- outreg(list("Great Regression" = m1), alpha = c(0.05, 0.01, 0.001),
         request = c("fstatistic" = "F"), runFuns = c(AIC = "AIC"),
         float = TRUE)
##html markup will appear on screen
outreg2HTML(m1out)
## outreg2HTML(m1out, filename = "funky.html")
## I'm not running that for you because you
## need to be in the intended working directory

m2 <- lm(y ~ x1 + x2, data = dat)

m2out <- outreg(list("Great Regression" = m1, "Small Regression" = m2),
               alpha = c(0.05, 0.01, 0.01),
                request = c("fstatistic" = "F"), runFuns = c(BIC = "BIC"))
outreg2HTML(m2out)
## Run this for yourself, it will create the output file funky2.html
## outreg2HTML(m2out, filename = "funky2.html")
## Please inspect the file "funky2.html

}
\author{
Paul E. Johnson \email{<pauljohn@ku.edu>}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSlopes.R
\name{plotSlopes}
\alias{plotSlopes}
\alias{plotSlopes.lm}
\title{Generic function for plotting regressions and interaction effects}
\usage{
plotSlopes(model, plotx, ...)

\method{plotSlopes}{lm}(model, plotx, modx = NULL, n = 3,
  modxVals = NULL, plotxRange = NULL, interval = c("none",
  "confidence", "prediction"), plotPoints = TRUE, legendPct = TRUE,
  legendArgs, llwd = 2, opacity = 100, ..., col = c("black", "blue",
  "darkgreen", "red", "orange", "purple", "green3"), type = c("response",
  "link"), gridArgs, width = 0.2)
}
\arguments{
\item{model}{Required. A fitted Regression}

\item{plotx}{Required. Name of one predictor from the fitted
model to be plotted on horizontal axis. May be numeric or factor.}

\item{...}{Additional arguments passed to methods. Often includes
arguments that are passed to plot. Any arguments that customize
plot output, such as lwd, cex, and so forth, may be
supplied. These arguments intended for the predict method will be
used: c("type", "se.fit", "interval", "level", "dispersion",
"terms", "na.action")}

\item{modx}{Optional. String for moderator variable name. May be
either numeric or factor. If omitted, a single predicted value
line will be drawn.}

\item{n}{Optional. Number of focal values of \code{modx}, used by
algorithms specified by modxVals; will be ignored if modxVals
supplies a vector of focal values.}

\item{modxVals}{Optional. Focal values of \code{modx} for which
lines are desired. May be a vector of values or the name of an
algorithm, "quantile", "std.dev.", or "table".}

\item{plotxRange}{Optional. If not specified, the observed range
of plotx will be used to determine the axis range.}

\item{interval}{Optional. Intervals provided by the
\code{predict.lm} may be supplied, either "confidence"
(confidence interval for the estimated conditional mean) or
"prediction" (interval for observed values of y given the rest
of the model).  The level can be specified as an argument
(which goes into ...  and then to the predict method)}

\item{plotPoints}{Optional. TRUE or FALSE: Should the plot include
the scatterplot points along with the lines.}

\item{legendPct}{Default = TRUE. Variable labels print with sample
percentages.}

\item{legendArgs}{Set as "none" if no legend is
desired. Otherwise, this can be a list of named arguments that
will override the settings I have for the legend.}

\item{llwd}{Optional, default = 2. Line widths for predicted
values. Can be single value or a vector, which will be
recycled as necessary.}

\item{opacity}{Optional, default = 100. A number between 1 and
255.  1 means "transparent" or invisible, 255 means very dark.
Determines the darkness of confidence interval regions}

\item{col}{Optional.I offer my preferred color vector as default.
Replace if you like. User may supply a vector of valid
color names, or  \code{rainbow(10)} or
\code{gray.colors(5)}. Color names will be recycled if there
are more focal values of \code{modx} than colors provided.}

\item{type}{Argument passed to the predict function. If model is glm,
can be either "response" or "link". For lm, no argument of this
type is needed, since both types have same value.}

\item{gridArgs}{Only used if plotx (horizontal axis) is a factor
variable. Designates reference lines between values. Set as
"none" if no grid lines are needed.  Default will be
\code{gridArgs = list(lwd = 0.3, lty = 5)}}

\item{width}{Only used if plotx (horizontal axis) is a factor. Designates
thickness of shading for bars that depict confidence intervals.}
}
\value{
Creates a plot and an output object that summarizes it.

The return object includes the "newdata" object that was
    used to create the plot, along with the "modxVals" vector, the
    values of the moderator for which lines were drawn, and the
    color vector. It also includes the call that generated the
    plot.
}
\description{
This is a function for plotting regression
objects. So far, there is an implementation for \code{lm()} objects.
I've been revising plotSlopes so that it should handle the work
performed by plotCurves.  As sure as that belief is verified,
the plotCurves work will be handled by plotSlopes. Different
plot types are created, depending on whether the x-axis predictor
\code{plotx} is numeric or categorical.
##'

This is a "simple slope" plotter for regression objects created by
\code{lm()} or similar functions that have capable predict methods
with newdata arguments. The term "simple slopes" was coined by
psychologists (Aiken and West, 1991; Cohen, et al 2002) for
analysis of interaction effects for particular values of a
moderating variable. The moderating variable may be continuous or
categorical, lines will be plotted for focal values of that
variable.
}
\details{
The original \code{plotSlopes} did not work well with nonlinear
predictors (log(x) and poly(x)). The separate function
\code{plotCurves()} was created for nonlinear predictive equations
and generalized linear models, but the separation of the two
functions was confusing for users. I've been working to make
plotSlopes handle everything and \code{plotCurves} will disappear
at some point.  \code{plotSlopes} can create an object which
is then tested with \code{testSlopes()} and that can be graphed
by a plot method.

The argument \code{plotx} is the name of the horizontal plotting
variable. An innovation was introduced in Version 1.8.33 so that
\code{plotx} can be either  numeric or categorical.

The argument \code{modx} is the moderator variable. It may be
either a numeric or a factor variable. As of version 1.7, the modx
argument may be omitted. A single predicted value line will be
drawn. That version also introduced the arguments interval and n.

There are many ways to specify focal values using the arguments
\code{modxVals} and \code{n}. This changed in rockchalk-1.7.0.  If
\code{modxVals} is omitted, a default algorithm for the variable
type will be used to select \code{n} values for
plotting. \code{modxVals} may be a vector of values (for a numeric
moderator) or levels (for a factor).  If modxVals is a vector of
values, then the argument \code{n} is ignored.  However, if
modxVals is one of the name of one of the algorithms, "table",
"quantile", or "std.dev.", then the argument \code{n} sets number
of focal values to be selected.  For numeric \code{modx}, n
defaults to 3, but for factors \code{modx} will be the number of
observed values of \code{modx}. If modxVals is omitted, the
defaults will be used ("table" for factors, "quantile" for numeric
variables).

For the predictors besides \code{modx} and \code{plotx} (the ones
that are not explicitly included in the plot), predicted values
are calculated with variables set to the mean and mode, for numeric
or factor variables (respectively). Those values can be reviewed
in the newdata object that is created as a part of the output from
this function
}
\examples{
## Manufacture some predictors
set.seed(12345)

dat <- genCorrelatedData2 (N = 100, means = rep(0,4), sds = 1, rho = 0.2,
                           beta = c(0.3, 0.5, -0.45, 0.5, -0.1, 0, 0.6),
                           stde = 2)

dat$xcat1 <- gl(2, 50, labels = c("M", "F"))
dat$xcat2 <- cut(rnorm(100), breaks = c(-Inf, 0, 0.4, 0.9, 1, Inf),
                 labels = c("R", "M", "D", "P", "G"))
## incorporate effect of categorical predictors
dat$y <- dat$y + 1.9 * dat$x1 * contrasts(dat$xcat1)[dat$xcat1] +
           contrasts(dat$xcat2)[dat$xcat2 , ] \%*\% c(0.1, -0.16, 0, 0.2)

m1 <- lm(y ~ x1 * x2 + x3 + x4 + xcat1* xcat2, data = dat)
summary(m1)

## New in rockchalk 1.7.x. No modx required:
plotSlopes(m1, plotx = "x1")
## Confidence interval, anybody?
plotSlopes(m1, plotx = "x1", interval = "conf")

## Prediction interval.
plotSlopes(m1, plotx = "x1", interval = "pred")

plotSlopes(m1, plotx = "x1", modx = "xcat2", modxVals = c("R", "M"))

plotSlopes(m1, plotx = "x1", modx = "xcat2",  interval = "pred")

plotSlopes(m1, plotx = "xcat1", modx = "xcat2",  interval = "conf", space = c(0,1))

plotSlopes(m1, plotx = "xcat1", modx = "xcat2", 
           modxVals = c("Print R" = "R" , "Show M" = "M"), gridArgs = "none")

## Now experiment with a moderator variable
## let default quantile algorithm do its job
plotSlopes(m1, plotx = "xcat2", interval = "none")
plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "none")
plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence",
           legendArgs = list(title = "xcat2"), ylim = c(-3, 3), lwd = 0.4)
plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence",
           legendArgs = list(title = "xcat2"), ylim = c(-3, 3), lwd = 0.4, width = 0.25)
m1.ps <- plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "prediction")
m1.ps <- plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "prediction", space=c(0,2))
plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "prediction", gridArgs = "none")

plotSlopes(m1, plotx = "xcat2", modx = "xcat1", interval = "confidence", ylim = c(-3, 3))
plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence", 
           col = c("black", "blue", "green", "red", "orange"), lty = c(1, 4, 6, 3))

plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence", 
           col = gray.colors(4, end = 0.5), lty = c(1, 4, 6, 3), legendArgs = list(horiz=TRUE))


plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence", 
           col = c("pink", "orange"))

plotSlopes(m1, plotx = "xcat1", interval = "confidence", 
           col = c("black", "blue", "green", "red", "orange"))

plotSlopes(m1, plotx = "xcat1", modx = "xcat2", interval = "confidence", 
           col = c("black", "blue", "green", "red", "orange"),
           gridlwd = 0.2)

## previous uses default equivalent to
## plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = "quantile")
## Want more focal values?
plotSlopes(m1, plotx = "x1", modx = "x2", n = 5)
## Pick focal values yourself?
plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = c(-2, 0, 0.5))
## Alternative algorithm?
plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = "std.dev.",
           main = "Uses \\"std.dev.\\" Divider for the Moderator",
           xlab = "My Predictor", ylab = "Write Anything You Want for ylab")

## Will catch output object from this one
m1ps <- plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = "std.dev.", n = 5,
                 main = "Setting n = 5 Selects More Focal Values for Plotting")

m1ts <- testSlopes(m1ps)

plot(m1ts)

### Examples with categorical Moderator variable

m3 <- lm (y ~ x1 + xcat1, data = dat)
summary(m3)
plotSlopes(m3, modx = "xcat1", plotx = "x1")
plotSlopes(m3, modx = "xcat1", plotx = "x1", interval = "predict")
plotSlopes(m3, modx = "x1", plotx = "xcat1", interval = "confidence",
           legendArgs = list(x = "bottomright", title = ""))

m4 <- lm (y ~ x1 * xcat1, data = dat)
summary(m4)
plotSlopes(m4, modx = "xcat1", plotx = "x1")
plotSlopes(m4, modx = "xcat1", plotx = "x1", interval = "conf")


m5 <- lm (y ~ x1 + x2 + x1 * xcat2, data = dat)
summary(m5)
plotSlopes(m5, modx = "xcat2", plotx = "x1")
m5ps <- plotSlopes(m5, modx = "xcat2", plotx = "x1", interval = "conf")

testSlopes(m5ps)



## Now examples with real data. How about Chilean voters?
library(carData)
m6 <- lm(statusquo ~ income * sex, data = Chile)
summary(m6)
plotSlopes(m6, modx = "sex", plotx = "income")
m6ps <- plotSlopes(m6, modx = "sex", plotx = "income", col = c("orange", "blue"))

testSlopes(m6ps)

m7 <- lm(statusquo ~ region * income, data= Chile)
summary(m7)
plotSlopes(m7, plotx = "income", modx = "region")

plotSlopes(m7, plotx = "income", modx = "region", plotPoints = FALSE)
plotSlopes(m7, plotx = "income", modx = "region", plotPoints = FALSE,
           interval = "conf")
plotSlopes(m7, plotx = "income", modx = "region", modxVals = c("SA","S", "C"),
           plotPoints = FALSE, interval = "conf")
## Same, choosing 3 most frequent values
plotSlopes(m7, plotx = "income", modx = "region", n = 3, plotPoints = FALSE,
           interval = "conf")


m8 <- lm(statusquo ~ region * income + sex + age, data= Chile)
summary(m8)
plotSlopes(m8, modx = "region", plotx = "income")


m9 <- lm(statusquo ~ income * age + education + sex + age, data = Chile)
summary(m9)
plotSlopes(m9, modx = "income", plotx = "age")

m9ps <- plotSlopes(m9, modx = "income", plotx = "age")
m9psts <- testSlopes(m9ps)
plot(m9psts) ## only works if moderator is numeric

## Demonstrate re-labeling
plotSlopes(m9, modx = "income", plotx = "age", n = 5,
           modxVals = c("Very poor" = 7500,  "Rich" = 125000), 
           main = "Chile Data", legendArgs = list(title = "Designated Incomes"))

plotSlopes(m9, modx = "income", plotx = "age", n = 5, modxVals = c("table"), 
           main = "Moderator: mean plus/minus 2 SD")

## Convert education to numeric, for fun
Chile$educationn <- as.numeric(Chile$education)
m10 <- lm(statusquo ~ income * educationn + sex + age, data = Chile)
summary(m10)
plotSlopes(m10, plotx = "educationn",  modx = "income")

## Now, the occupational prestige data. Please note careful attention
## to consistency of colors selected
data(Prestige)
m11 <- lm(prestige ~ education * type, data = Prestige)

plotSlopes(m11, plotx = "education", modx = "type", interval = "conf")
dev.new()
plotSlopes(m11, plotx = "education", modx = "type",
           modxVals = c("prof"), interval = "conf")
dev.new()
plotSlopes(m11, plotx = "education", modx = "type",
           modxVals = c("bc"), interval = "conf")
dev.new()
plotSlopes(m11, plotx = "education", modx = "type",
           modxVals = c("bc", "wc"), interval = "conf")
}
\references{
Aiken, L. S. and West, S.G. (1991). Multiple
Regression: Testing and Interpreting Interactions. Newbury
Park, Calif: Sage Publications.

Cohen, J., Cohen, P., West, S. G., and Aiken, L. S. (2002).
Applied Multiple Regression/Correlation Analysis for the Behavioral
Sciences (Third.). Routledge Academic.
}
\seealso{
\code{\link[rockchalk]{testSlopes}} \code{\link[rockchalk]{plotCurves}}
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{summary.factor}
\alias{summary.factor}
\title{Tabulates observed values and calculates entropy}
\usage{
\method{summary}{factor}(y)
}
\arguments{
\item{y}{a factor (non-numeric variable)}
}
\value{
A list, including the summary table and vector of summary
    stats if requested.
}
\description{
This adapts code from R base summary.factor. It adds
the calculation of entropy as a measure of diversity.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plyr.replace.R
\name{rbindFill}
\alias{rbindFill}
\title{Stack together data frames}
\usage{
rbindFill(...)
}
\arguments{
\item{...}{Data frames}
}
\value{
A stacked data frame
}
\description{
In the end of the code for plyr::rbind.fill, the author explains
that is uses an experimental function to build the data.frame.  I
would rather not put any weight on an experimental function, so I
sat out to create a new rbindFill. This function uses no
experimental functions. It does not rely on any functions from
packages that are not in base of R, except, of course, for functions
in this package.
}
\details{
Along the way, I noticed a feature that seems to be a flaw in both
rbind and rbind.fill.  In the examples, there is a demonstration
of the fact that base R rbind and plyr::rbind.fill both have
undesirable properties when data sets containing factors and
ordered variables are involved. This function introduces a
"data consistency check" that prevents corruption of variables
when data frames are combined.  This "safe" version will notice
differences in classes of variables among data.frames and stop
with an error message to alert the user to the problem.
}
\examples{
set.seed(123123)
N <- 10000
dat <- genCorrelatedData2(N, means = c(10, 20, 5, 5, 6, 7, 9), sds = 3,
           stde = 3, rho = .2,  beta = c(1, 1, -1, 0.5))
dat1 <- dat
dat1$xcat1 <- factor(sample(c("a", "b", "c", "d"), N, replace=TRUE))
dat1$xcat2 <- factor(sample(c("M", "F"), N, replace=TRUE),
                    levels = c("M", "F"), labels = c("Male", "Female"))
dat1$y <- dat$y +
          as.vector(contrasts(dat1$xcat1)[dat1$xcat1, ] \%*\% c(0.1, 0.2, 0.3))
dat1$xchar1 <- rep(letters[1:26], length.out = N)
dat2 <- dat
dat1$x3 <- NULL
dat2$x2 <- NULL
dat2$xcat2 <- factor(sample(c("M", "F"), N, replace=TRUE),
                     levels = c("M", "F"), labels = c("Male", "Female"))
dat2$xcat3 <- factor(sample(c("K1", "K2", "K3", "K4"), N, replace=TRUE))
dat2$xchar1 <- "1"
dat3 <- dat
dat3$x1 <- NULL
dat3$xcat3 <-  factor(sample(c("L1", "L2", "L3", "L4"), N, replace=TRUE)) 
dat.stack <- rbindFill(dat1, dat2, dat3)
str(dat.stack)

## Possible BUG alert about base::rbind and plyr::rbind.fill
## Demonstrate the problem of a same-named variable that is factor in one and
## an ordered variable in the other
dat5 <- data.frame(ds = "5", x1 = rnorm(N),
                   xcat1 = gl(20, 5, labels = LETTERS[20:1]))
dat6 <- data.frame(ds = "6", x1 = rnorm(N),
                   xcat1 = gl(20, 5, labels = LETTERS[1:20], ordered = TRUE))
## rbind reduces xcat1 to factor, whether we bind dat5 or dat6 first.
stack1 <- base::rbind(dat5, dat6)
str(stack1)
## note xcat1 levels are ordered T, S, R, Q
stack2 <- base::rbind(dat6, dat5)
str(stack2)
## xcat1 levels are A, B, C, D
## stack3 <- plyr::rbind.fill(dat5, dat6)
## str(stack3)
## xcat1 is a factor with levels T, S, R, Q ...
## stack4 <- plyr::rbind.fill(dat6, dat5)
## str(stack4)
## oops, xcat1 is ordinal with levels A < B < C < D
## stack5 <- rbindFill(dat5, dat6)
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictOMatic.R
\name{model.data.default}
\alias{model.data.default}
\title{Create a data frame suitable for estimating a model}
\usage{
\method{model.data}{default}(model, na.action = na.omit, ...)
}
\arguments{
\item{model}{A fitted model}

\item{na.action}{Defaults to na.omit, so model as it would appear
in user workspace is re-created, except that rows with missing
values are deleted.  Changing this argument to na.pass will
provide the data as it was in the workspace.}

\item{...}{Place holder for other arguments, not used at present}
}
\value{
A data frame
}
\description{
This is the default method. Works for lm and glm fits.
}
\examples{
library(rockchalk)






## first, check if model.data works when there is no data argument
## This used to fail, now OK

x1 <- rnorm(100, m = 100, s = 10)
x2 <- rnorm(100, m = 50, s =20)
y <- rnorm(100, m = 40, s = 3)

m0 <- lm(y ~ log(10+x1) + x2)
m0.data <- model.data(m0)
head(m0.data)


m1 <- lm(log(43 + y) ~ log(10+x1) + x2)
m1.data <- model.data(m1)
head(m1.data)

d <- 3

m2 <- lm(log(d + y) ~ log(10+x1) + x2)
m2.data <- model.data(m2)
head(m2.data)

m3 <- lm(log(y + d) ~ log(10+x1) + x2)
m3.data <- model.data(m3)
head(m3.data)



## check numeric and categorical predictors

x1 <- rpois(100, l=6)
x2 <- rnorm(100, m=50, s=10)
x3 <- rnorm(100)
xcat1 <- gl(2,50, labels=c("M","F"))
xcat2 <- cut(rnorm(100), breaks=c(-Inf, 0, 0.4, 0.9, 1, Inf),
             labels=c("R", "M", "D", "P", "G"))
dat <- data.frame(x1, x2, x3, xcat1, xcat2)
rm(x1, x2, x3, xcat1, xcat2)
dat$xcat1n <- with(dat, contrasts(xcat1)[xcat1, ,drop=FALSE])
dat$xcat2n <- with(dat, contrasts(xcat2)[xcat2, ])

STDE <- 20
dat$y <- with(dat,
              0.03 + 0.8*x1 + 0.1*x2 + 0.7*x3 +
              xcat1n \%*\% c(2) + xcat2n \%*\% c(0.1,-2,0.3, 0.1) +
              STDE*rnorm(100))



m1 <- lm(y ~ poly(x1, 2), data=dat)
m1.data <- model.data(m1)
head(m1.data)
attr(m1.data, "varNamesRHS")

## Check to make sure d is not mistaken for a data column
d <- 2
m2 <- lm(y ~ poly(x1, d), data=dat)
m2.data <- model.data(m2)
head(m2.data)
attr(m2.data, "varNamesRHS")


## Check to see how the 10 in log is handled
m3 <- lm(y ~ log(10 + x1) + poly(x1, d) + sin(x2), data=dat)
m3.data <- model.data(m3)
head(m3.data)
attr(m3.data, "varNamesRHS")


m4 <- lm(log(50+y) ~ log(d+10+x1) + poly(x1, 2), data=dat)
m4.data <- model.data(m4)
head(m4.data)
attr(m4.data, "varNamesRHS")


m5 <- lm(y ~ x1*x1, data=dat)
m5.data <- model.data(m5)
head(m5.data)
attr(m5.data, "varNamesRHS")


m6 <- lm(y ~ x1 + I(x1^2), data=dat)
m6.data <- model.data(m6)
head(m6.data)
attr(m6.data, "varNamesRHS")


## Put in some missings.
## poly doesn't work if there are missings, but
## can test with log
dat$x1[sample(100, 5)] <- NA
dat$y[sample(100, 5)] <- NA
dat$x2[sample(100, 5)] <- NA
dat$x3[sample(100,10)] <- NA

m1 <- lm(y ~ log(10 + x1), data=dat)
m1.data <- model.data(m1)
head(m1.data)
summarize(m1.data)
attr(m1.data, "varNamesRHS")



m2 <- lm(y ~ log(x1 + 10), data=dat)
m2.data <- model.data(m2)
head(m2.data)
summarize(m1.data)
attr(m1.data, "varNamesRHS")

d <- 2
m3 <- lm(log(50+y) ~ log(d+10+x1) + x2 + sin(x3), data=dat)
m3.data <- model.data(m3)
head(m3.data)
summarize(m3.data)
attr(m3.data, "varNamesRHS")


m4 <- lm(y ~ I(x1) + I(x1^2) + log(x2), data=dat)
m4.data <- model.data(m4)
summarize(m4.data)
attr(m4.data, "varNamesRHS")


m5 <- lm(y ~ x1 + I(x1^2) + cos(x2), data=dat)
m5.data <- model.data(m5)
head(m5.data)
summarize(m5.data)
attr(m5.data, "varNamesRHS")



## Now try with some variables in the dataframe, some not

x10 <- rnorm(100)
x11 <- rnorm(100)



m6 <- lm(y ~ x1 + I(x1^2) + cos(x2) + log(10 + x10) + sin(x11) +
         x10*x11, data = dat)
m6.data <- model.data(m6)
head(m6.data)
dim(m6.data)
summarize(m5.data)
attr(m6.data, "varNamesRHS")


}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{padW0}
\alias{padW0}
\title{Pad with 0's.}
\usage{
padW0(x)
}
\arguments{
\item{x}{a numeric  variable.}
}
\value{
A character string vector padded with 0's
}
\description{
Sometimes we receive this c(1, 22, 131) and we need character
variables of the same size, such as c("001", "022", "131").  This
happens if a user has mistakenly converted a zip code (US regional
identifier) like "00231" to a number. This function converts the
number back to a 0 padded string.
}
\details{
This works differently if the number provided is an integer, or a
character string.  Integers are left padded with the character
"0".  A character string will be left-padded with blanks.
}
\examples{
x <- c(1 , 11, 22, 121, 14141, 31)
(xpad <- padW0(x))
x <- rpois(7, lambda = 11)
(xpad <- padW0(x))
x <- c("Alabama", "Iowa", "Washington")
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictOMatic.R
\name{model.data}
\alias{model.data}
\title{Create a "raw" (UNTRANSFORMED) data frame equivalent to the input
data that would be required to fit the given model.}
\usage{
model.data(model, ...)
}
\arguments{
\item{model}{A fitted regression model in which the data argument
is specified. This function will fail if the model was not fit
with the data option.}

\item{...}{Arguments passed to implementing methods.}
}
\value{
A data frame
}
\description{
This is a generic method. Unlike model.frame and
model.matrix, this does not return transformed variables. It deals
with regression formulae that have functions like poly(x, d) in
them. It differentiates x from d in those expressions. And it also
manages log(x + 10). The default method works for standarad R
regression models like lm and glm.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/waldt.R
\name{waldt}
\alias{waldt}
\title{T-test for the difference in 2 regression parameters}
\usage{
waldt(parm1, parm2, model, model.cov = NULL,
  digits = getOption("digits"))
}
\arguments{
\item{parm1}{A parameter name, in quotes!}

\item{parm2}{Another parameter name, in quotes!}

\item{model}{A fitted regression model}

\item{model.cov}{Optional, another covariance matrix to use while
calculating the test. Primarily used for robust (or otherwise
adjusted) standard errors}

\item{digits}{How many digits to print? This affects only the on-screen
printout. The return object is numeric, full precision.}
}
\value{
A vector with the difference, std. err., t-stat, and p
    value. Prints a formatted output statement.
}
\description{
This is the one the students call the "fancy t test". It is just
the simplest, most easy to use version of the t test to decide if
2 coefficients are equal. It is not as general as other functions
in other packages. This is simpler to use for beginners.  The
\code{car} package's function \code{linearHypothesis} is more
general, but its documentation is much more difficult to understand.
It gives statistically identical results, albeit phrased as an F
test.
}
\details{
I did this because we have trouble understanding terminology in
documentation for more abstract functions in other R
packages.

It has an additional feature, it can import robust standard errors
to conduct the test.
}
\examples{
mdat <- data.frame(x1 = rnorm(100), x2 = rnorm(100))
stde <- 2
mdat$y <- 0.2 * mdat$x1 + 0.24 * mdat$x2 + stde * rnorm(100)
m1 <- lm(y ~ x1 + x2, data = mdat)
waldt("x1", "x2", m1)
waldt("x1", "x2", m1, digits = 2)
## Returned object is not "rounded characters". It is still numbers
stillnumeric <-  waldt("x1", "x2", m1, digits = 2)
stillnumeric
## Equivalent to car package linearHypothesis:
if(require(car)){
    linearHypothesis(m1, "x1 = x2")
}
## recall t = sqrt(F) for a 1 degree of freedom test.
## If we could understand instructions for car, we probably
## would not need this function, actually.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{summarizeFactors}
\alias{summarizeFactors}
\title{Extracts non-numeric variables, calculates summary information,
including entropy as a diversity indicator.}
\usage{
summarizeFactors(dat = NULL, maxLevels = 5, alphaSort = TRUE,
  stats = c("entropy", "normedEntropy", "nobs", "nmiss"), digits = 2)
}
\arguments{
\item{dat}{A data frame}

\item{maxLevels}{The maximum number of levels that will be
reported.}

\item{alphaSort}{If TRUE (default), the columns are re-organized
in alphabetical order. If FALSE, they are presented in the
original order.}

\item{stats}{Default is \code{c("nobs", "nmiss", "entropy",
"normedEntropy")}.}

\item{digits}{Default 2.}
}
\value{
A list of factor summaries
}
\description{
This function finds the non- numeric variables and ignores the
others. (See \code{summarizeNumerics} for a function that
handles numeric variables.)  It then treats all non-numeric
variables as if they were factors, and summarizes each. The main
benefits from this compared to R's default summary are 1) more
summary information is returned for each variable (entropy
estimates ofdispersion), 2) the columns in the output are
alphabetized. To prevent alphabetization, use alphaSort = FALSE.
}
\details{
Entropy is one possible measure of diversity. If all outcomes are
equally likely, the entropy is maximized, while if all outcomes
fall into one possible category, entropy is at its lowest
values. The lowest possible value for entropy is 0, while the
maximum value is dependent on the number of categories. Entropy is
also called Shannon's information index in some fields of study
(Balch, 2000 ; Shannon, 1949 ).

Concerning the use of entropy as a diversity index, the user might
consult Balch(). For each possible outcome category, let p
represent the observed proportion of cases. The diversity
contribution of each category is -p * log2(p). Note that if p is
either 0 or 1, the diversity contribution is 0.  The sum of those
diversity contributions across possible outcomes is the entropy
estimate. The entropy value is a lower bound of 0, but there is no
upper bound that is independent of the number of possible
categories. If m is the number of categories, the maximum possible
value of entropy is -log2(1/m).

Because the maximum value of entropy depends on the number of
possible categories, some scholars wish to re-scale so as to bring
the values into a common numeric scale. The normed entropy is
calculated as the observed entropy divided by the maximum possible
entropy.  Normed entropy takes on values between 0 and 1, so in a
sense, its values are more easily comparable. However, the
comparison is something of an illusion, since variables with the
same number of categories will always be comparable by their
entropy, whether it is normed or not.

Warning: Variables of class POSIXt will be ignored. This will be
fixed in the future. The function works perfectly well with
numeric, factor, or character variables.  Other more elaborate
structures are likely to be trouble.
}
\examples{
set.seed(21234)
x <- runif(1000)
xn <- ifelse(x < 0.2, 0, ifelse(x < 0.6, 1, 2))
xf <- factor(xn, levels=c(0,1,2), labels("A","B","C"))
dat <- data.frame(xf, xn, x)
summarizeFactors(dat)
##see help for summarize for more examples
}
\references{
Balch, T. (2000). Hierarchic Social Entropy: An Information
Theoretic Measure of Robot Group Diversity. Auton. Robots, 8(3),
209-238.

Shannon, Claude. E. (1949). The Mathematical Theory of
Communication. Urbana: University of Illinois Press.
}
\seealso{
\code{\link{summarizeNumerics}}
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residualCenter.R
\name{residualCenter}
\alias{residualCenter}
\alias{residualCenter.default}
\alias{predict.rcreg}
\title{Calculates a "residual-centered" interaction regression.}
\usage{
residualCenter(model)

\method{residualCenter}{default}(model)

\method{predict}{rcreg}(object, ...)
}
\arguments{
\item{model}{A fitted lm object}

\item{object}{Fitted residual-centered regression from residualCenter}

\item{...}{Other named arguments. May include newdata, a dataframe of 
predictors. That should include values for individual predictor, need 
not include interactions that are constructed by residualCenter. 
These parameters that will be passed to the predict method of the model.}
}
\value{
a regression model of the type as the input model, with
the exception that the residualCentered predictor is used in place
of the original interaction. The return model includes new
variable centeringRegressions: a list including each of the
intermediate regressions that was calculated in order to create
the residual centered interaction terms. These latter objects may
be necessary for diagnostics and to calculate predicted values for
hypothetical values of the inputs. If there are no interactive
terms, then NULL is returned.
}
\description{
Given a fitted \code{lm}, this function scans for coefficients
estimated from "interaction terms" by checking for colon
symbols. The function then calculates the "residual centered"
estimate of the interaction term and replaces the interaction term
with that residual centered estimate. It works for any order of
interaction, unlike other implementations of the same
approach. The function \code{lmres} in the now-archived package
pequod was a similar function.

Calculates predicted values of
residual centered interaction regressions estimated in
any type of regression framework (lm, glm, etc).
}
\examples{
set.seed(123)
x1 <- rnorm(100)
x2 <- rnorm(100)
x3 <- rnorm(100)
x4 <- rnorm(100)
y <- rnorm(100)
dat <- data.frame(y, x1,x2,x3,x4)
rm(x1,x2,x3,x4,y)
m1 <- lm(y~ x1*x2 + x4, data = dat)
 
m1RC <- residualCenter(m1)

m1RCs <- summary(m1RC)
## The stage 1 centering regressions can be viewed as well
## lapply(m1RC$rcRegressions, summary)

## Verify residualCenter() output against the manual calculation
dat$x1rcx2 <- as.numeric(resid(lm(I(x1*x2) ~ x1 + x2, data = dat)))
m1m <- lm(y ~ x1 + x2 + x4 + x1rcx2, data=dat)
summary(m1m)
cbind("residualCenter" = coef(m1RC), "manual" = coef(m1m))


m2 <- lm(y~ x1*x2*x3 + x4, data=dat)
m2RC <- residualCenter(m2)
m2RCs <- summary(m2RC)

## Verify that result manually
dat$x2rcx3 <- as.numeric(resid(lm(I(x2*x3) ~ x2 + x3, data = dat)))
dat$x1rcx3 <- as.numeric(resid(lm(I(x1*x3) ~ x1 + x3, data = dat)))
dat$x1rcx2rcx3 <- as.numeric( resid(lm(I(x1*x2*x3) ~ x1 + x2 + x3 + x1rcx2 +
                                       x1rcx3 + x2rcx3 , data=dat)))
(m2m <- lm(y ~ x1 + x2 + x3+ x4 + x1rcx2 + x1rcx3 + x2rcx3 + x1rcx2rcx3,
           data = dat))

cbind("residualCenter" = coef(m2RC), "manual" = coef(m2m))


### As good as pequod's lmres
### not run because pequod generates R warnings
###
### if (require(pequod)){
###  pequodm1 <- lmres(y ~ x1*x2*x3 + x4, data=dat) 
###  pequodm1s <- summary(pequodm1)
###  coef(pequodm1s)
### }

### Works with any number of interactions. See:

m3 <- lm(y~ x1*x2*x3*x4, data=dat)
m3RC <- residualCenter(m3)
summary(m3RC)
##'
## Verify that one manually (Gosh, this is horrible to write out)
dat$x1rcx4 <- as.numeric(resid(lm(I(x1*x4) ~ x1 + x4, data=dat)))
dat$x2rcx4 <- as.numeric(resid(lm(I(x2*x4) ~ x2 + x4, data=dat)))
dat$x3rcx4 <- as.numeric(resid(lm(I(x3*x4) ~ x3 + x4, data=dat)))
dat$x1rcx2rcx4 <- as.numeric(resid(lm(I(x1*x2*x4) ~ x1 + x2 + x4 +
                                      x1rcx2 + x1rcx4 + x2rcx4, data=dat)))
dat$x1rcx3rcx4 <- as.numeric(resid(lm(I(x1*x3*x4) ~ x1 + x3 + x4 +
                                      x1rcx3 + x1rcx4 + x3rcx4, data=dat)))
dat$x2rcx3rcx4 <- as.numeric(resid(lm(I(x2*x3*x4) ~ x2 + x3 + x4 +
                                      x2rcx3 + x2rcx4 + x3rcx4, data=dat)))
dat$x1rcx2rcx3rcx4 <-
    as.numeric(resid(lm(I(x1*x2*x3*x4) ~ x1 + x2 + x3 + x4 +
                        x1rcx2 + x1rcx3 + x2rcx3 + x1rcx4  + x2rcx4 +
                        x3rcx4  + x1rcx2rcx3 + x1rcx2rcx4 + x1rcx3rcx4 +
                        x2rcx3rcx4, data=dat)))
(m3m <- lm(y ~ x1 + x2 + x3 + x4 + x1rcx2 + x1rcx3 + x2rcx3 + x1rcx4 +
           x2rcx4 + x3rcx4 + x1rcx2rcx3 + x1rcx2rcx4 + x1rcx3rcx4 +
           x2rcx3rcx4 + x1rcx2rcx3rcx4, data=dat))

cbind("residualCenter"=coef(m3RC), "manual"=coef(m3m))

### If you want to fit a sequence of models, as in pequod, can do.

tm <-terms(m2)
tmvec <- attr(terms(m2), "term.labels")
f1 <- tmvec[grep(":", tmvec, invert = TRUE)]
f2 <- tmvec[grep(":.*:", tmvec, invert = TRUE)]
f3 <- tmvec[grep(":.*:.*:", tmvec, invert = TRUE)]

## > f1
## [1] "x1" "x2" "x3" "x4"
## > f2
## [1] "x1"    "x2"    "x3"    "x4"    "x1:x2" "x1:x3" "x2:x3"
## > f3
## [1] "x1"       "x2"       "x3"       "x4"       "x1:x2"    "x1:x3"    "x2:x3"   
## [8] "x1:x2:x3"

f1 <- lm(as.formula(paste("y","~", paste(f1, collapse=" + "))), data=dat)
f1RC <- residualCenter(f1)
summary(f1RC)

f2 <- lm(as.formula(paste("y","~", paste(f2, collapse=" + "))), data=dat)
f2RC <- residualCenter(f2)
summary(f2RC)

f3 <- lm(as.formula(paste("y","~", paste(f3, collapse=" + "))), data=dat)
f3RC <- residualCenter(f3)
summary(f3RC)

library(rockchalk)
dat <- genCorrelatedData(1000, stde=5)

m1 <- lm(y ~ x1 * x2, data=dat)

m1mc <- meanCenter(m1)
summary(m1mc)

m1rc <- residualCenter(m1)
summary(m1rc)


newdf <- apply(dat, 2, summary)
newdf <- as.data.frame(newdf)

predict(m1rc, newdata=newdf)
}
\references{
Little, T. D., Bovaird, J. A.,
& Widaman, K. F. (2006). On the Merits of Orthogonalizing
Powered and Product Terms: Implications for Modeling
Interactions Among Latent Variables.
Structural Equation Modeling, 13(4), 497-519.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCurves.R
\name{plotCurves}
\alias{plotCurves}
\title{Assists creation of predicted value curves for regression models.}
\usage{
plotCurves(model, plotx, nx = 40, modx, plotxRange = NULL, n,
  modxVals = NULL, interval = c("none", "confidence", "prediction"),
  plotPoints = TRUE, plotLegend = TRUE, legendTitle = NULL,
  legendPct = TRUE, col = c("black", "blue", "darkgreen", "red",
  "orange", "purple", "green3"), llwd = 2, opacity = 100,
  envir = environment(formula(model)), ...)
}
\arguments{
\item{model}{Required. Fitted regression object. Must have a
predict method}

\item{plotx}{Required. String with name of predictor for the x axis}

\item{nx}{Number of values of plotx at which to calculate the predicted
value.  Default = 40.}

\item{modx}{Optional. String for moderator variable name. May be
either numeric or factor.}

\item{plotxRange}{Optional. If not specified, the observed
range of plotx will be used to determine the axis range.}

\item{n}{Optional.  Number of focal values of \code{modx}, used by
algorithms specified by modxVals; will be ignored if modxVals
supplies a vector of focal values.}

\item{modxVals}{Optional. A vector of focal values for which
predicted values are to be plotted. May also be a character string
to select an algorithm ("quantile","std.dev." or "table"), or a
user-supplied function to select focal values (a new method
similar to \code{getFocal}). If modx is a factor, currently, the
only available algorithm is "table" (see \code{getFocal.factor}.}

\item{interval}{Optional. Intervals provided by the
\code{predict.lm} may be supplied, either "conf" (95% confidence
interval for the estimated conditional mean) or "pred" (95%
interval for observed values of y given the rest of the model).}

\item{plotPoints}{Optional. TRUE or FALSE: Should the plot include
the scatterplot points along with the lines.}

\item{plotLegend}{Optional. TRUE or FALSE: Should the default
legend be included?}

\item{legendTitle}{Optional. You'll get an automatically generated
title, such as "Moderator: modx", but if you don't like that,
specify your own string here.}

\item{legendPct}{Default = TRUE. Variable labels print with sample percentages.}

\item{col}{I offer my preferred color vector as default.
Replace if you like. User may supply a vector of valid
color names, or  \code{rainbow(10)} or
\code{gray.colors(5)}. Color names will be recycled if there
are more focal values of \code{modx} than colors provided.}

\item{llwd}{Optional. Line widths for predicted values. Can be
single value or a vector, which will be recycled as necessary.}

\item{opacity}{Optional, default = 100. A number between 1 and
255. 1 means "transparent" or invisible, 255 means very dark.  the
darkness of confidence interval regions}

\item{envir}{environment to search for variables.}

\item{...}{further arguments that are passed to plot or
predict. The arguments that are monitored to be sent to predict
are c("type", "se.fit", "dispersion", "interval", "level",
"terms", "na.action").}
}
\value{
A plot is created as a side effect, a list is returned
including 1) the call, 2) a newdata object that includes
information on the curves that were plotted, 3) a vector modxVals,
the values for which curves were drawn.
}
\description{
Creates a predicted value plot that includes a separate predicted
value line for each value of a focal variable. The x axis variable
is specified by the \code{plotx} argument. As of rockchalk 1.7.x,
the moderator argument, modx, is optional. Think of this a new
version of R's \code{termplot}, but it allows for
interactions. And it handles some nonlinear transformations more
gracefully than termplot.
}
\details{
This is similar to \code{plotSlopes}, but it accepts regressions
in which there are transformed variables, such as "log(x1)".
It creates a plot of the predicted dependent
variable against one of the numeric predictors, \code{plotx}. It
draws a predicted value line for each value of \code{modx}, a
moderator variable. The moderator may be a numeric or categorical
moderator variable.

The user may designate which particular values of the moderator
are used for calculating the predicted value lines.  That is,
\code{modxVals = c( 12,22,37)} would draw lines for values 12, 22,
and 37 of the moderator. User may instead supply a character
string to choose one of the built in algorithms. The default
algorithm is "quantile", which will select \code{n} values that
are evenly spaced along the \code{modx} axis. The algorithm
"std.dev" will select the mean of \code{modx} (m) and then it will
select values that step away from the mean in standard deviation
sd units. For example, if \code{n = 3}, the focal
values will \code{m, m - sd, am + sd}.
}
\examples{

library(rockchalk)

## Replicate some R classics.  The budworm.lg data from predict.glm
## will work properly after re-formatting the information as a data.frame:

## example from Venables and Ripley (2002, pp. 190-2.)
df <- data.frame(ldose = rep(0:5, 2),
                 sex = factor(rep(c("M", "F"), c(6, 6))),
                 SF.numdead = c(1, 4, 9, 13, 18, 20, 0, 2, 6, 10, 12, 16))
df$SF.numalive = 20 - df$SF.numdead

budworm.lg <- glm(cbind(SF.numdead, SF.numalive) ~ sex*ldose, data = df,
                  family = binomial)


plotCurves(budworm.lg, plotx = "ldose", modx = "sex", interval = "confidence",
           ylim = c(0, 1))

## See infert
model2 <- glm(case ~ age + parity + education + spontaneous + induced,
              data = infert, family = binomial())

## Curvature so slight we can barely see it
model2pc1 <- plotCurves(model2, plotx = "age", modx = "education",
                        interval = "confidence", ylim = c(0, 1))


model2pc2 <- plotCurves(model2, plotx = "age", modx = "education",
                        modxVals = levels(infert$education)[1],
                        interval = "confidence", ylim = c(0, 1))


model2pc2 <- plotCurves(model2, plotx = "age", modx = "education",
                        modxVals = levels(infert$education)[c(2,3)],
                        interval = "confidence", ylim = c(0, 1))

model2pc2 <- plotCurves(model2, plotx = "age", modx = "education",
                        modxVals = levels(infert$education)[c(2,3)],
                        ylim = c(0, 1), type = "response")





## Manufacture some data
set.seed(12345)
N <- 500
dat <- genCorrelatedData2(N = 500, means = c(5, 0, 0, 0), sds = rep(1, 4),
                          rho = 0.2, beta = rep(1, 5), stde = 20)

dat$xcat1 <- gl(2, N/2, labels = c("Monster", "Human"))
dat$xcat2 <- cut(rnorm(N), breaks = c(-Inf, 0, 0.4, 0.9, 1, Inf),
             labels = c("R", "M", "D", "P", "G"))

###The design matrix for categorical variables, xcat numeric
dat$xcat1n <- with(dat, contrasts(xcat1)[xcat1, ])
dat$xcat2n <- with(dat, contrasts(xcat2)[xcat2, ])


stde <- 2
dat$y <- with(dat, 0.03 + 11.5 * log(x1) * contrasts(dat$xcat1)[dat$xcat1] +
              0.1 * x2 + 0.04 * x2^2 + stde*rnorm(N))

stde <- 1
dat$y2 <- with(dat, 0.03 + 0.1 * x1 + 0.1 * x2 + 0.25 * x1 * x2 + 0.4 * x3 -
               0.1 * x4 + stde * rnorm(N))
stde <- 8
dat$y3 <- with(dat, 3 + 0.5 * x1 + 1.2 * (as.numeric(xcat1)-1) +
-0.8 * (as.numeric(xcat1)-1) * x1 +  stde * rnorm(N))

stde <- 8
dat$y4 <- with(dat, 3 + 0.5 * x1 +
               contrasts(dat$xcat2)[dat$xcat2, ] \%*\% c(0.1, -0.2, 0.3, 0.05)  +
               stde * rnorm(N))


## Curvature with interaction
m1 <- lm(y ~ log(x1)*xcat1 + x2 + I(x2^2), data=dat)
summary(m1)

## First, with no moderator
plotCurves(m1, plotx = "x1")

plotCurves(m1, plotx = "x1", modx = "xcat1")

## ## Verify that plot by comparing against a manually contructed alternative
## par(mfrow=c(1,2))
## plotCurves(m1, plotx = "x1", modx = "xcat1")
## newdat <- with(dat, expand.grid(x1 = plotSeq(x1, 30), xcat1 = levels(xcat1)))
## newdat$x2 <-  with(dat, mean(x2, na.rm = TRUE))
## newdat$m1p <- predict(m1, newdata = newdat)
## plot( y ~ x1, data = dat, type = "n", ylim = magRange(dat$y, c(1, 1.2)))
## points( y ~ x1, data = dat, col = dat$xcat1, cex = 0.4, lwd = 0.5)
## by(newdat, newdat$xcat1, function(dd) {lines(dd$x1, dd$m1p)})
## legend("topleft", legend=levels(dat$xcat1), col = as.numeric(dat$xcat1), lty = 1)
## par(mfrow = c(1,1))
## ##Close enough!


plotCurves(m1, plotx = "x2", modx = "x1")

plotCurves(m1, plotx = "x2", modx = "xcat1")

plotCurves(m1, plotx = "x2", modx = "xcat1", interval = "conf")


m2 <- lm(y ~ log(x1)*xcat1 + xcat1*(x2 + I(x2^2)), data = dat)
summary(m2)
plotCurves(m2, plotx = "x2", modx = "xcat1")

plotCurves(m2, plotx  ="x2", modx = "x1")


m3a <- lm(y ~ poly(x2, 2) + xcat1, data = dat)

plotCurves(m3a, plotx = "x2")
plotCurves(m3a, plotx = "x2", modx = "xcat1")
#OK

m4 <- lm(log(y+10) ~ poly(x2, 2)*xcat1 + x1, data = dat)
summary(m4)
plotCurves(m4, plotx = "x2")

plotCurves(m4, plotx  ="x2", modx = "xcat1")

plotCurves(m4, plotx = "x2", modx = "x1")

plotCurves(m4, plotx = "x2", modx = "xcat1")

plotCurves(m4, plotx = "x2", modx = "xcat1", modxVals = c("Monster"))


##ordinary interaction
m5 <- lm(y2 ~ x1*x2 + x3 +x4, data = dat)
summary(m5)
plotCurves(m5, plotx = "x1", modx = "x2")
plotCurves(m5, plotx = "x1", modx = "x2", modxVals = c( -2, -1, 0, 1, 2))
plotCurves(m5, plotx = "x1", modx = "x2", modxVals = c(-2))
plotCurves(m5, plotx = "x1", modx = "x2", modxVals = "std.dev.")
plotCurves(m5, plotx = "x1", modx = "x2", modxVals = "quantile")
plotCurves(m5, plotx = "x3", modx = "x2")


if(require(carData)){
    mc1 <- lm(statusquo ~ income * sex, data = Chile)
    summary(mc1)
    plotCurves(mc1, plotx = "income")
    plotCurves(mc1, modx = "sex", plotx = "income")
    plotCurves(mc1, modx = "sex", plotx = "income", modxVals = "M")
    
    mc2 <- lm(statusquo ~ region * income, data =  Chile)
    summary(mc2)
    plotCurves(mc2, modx = "region", plotx = "income")
    plotCurves(mc2, modx = "region", plotx = "income",
               modxVals = levels(Chile$region)[c(1,4)])
    plotCurves(mc2, modx = "region", plotx = "income", modxVals = c("S","M","SA"))
    plotCurves(mc2, modx = "region", plotx = "income", modxVals = c("S","M","SA"),
               interval = "conf")
    
    plotCurves(mc2, modx = "region", plotx = "income", plotPoints = FALSE)
 
    mc3 <- lm(statusquo ~ region * income + sex + age, data =  Chile)
    summary(mc3)
    plotCurves(mc3, modx = "region", plotx = "income")
 
    mc4 <- lm(statusquo ~ income * (age + I(age^2)) + education + sex + age, data = Chile)
    summary(mc4)
    plotCurves(mc4, plotx = "age")
    plotCurves(mc4, plotx = "age", interval = "conf")
     
    plotCurves(mc4, plotx = "age", modx = "income")
    plotCurves(mc4, plotx = "age", modx = "income", plotPoints = FALSE)
    
    plotCurves(mc4,  plotx = "income", modx = "age")
    plotCurves(mc4,  plotx = "income", modx = "age", n = 8)
    
    plotCurves(mc4,  plotx = "income", modx = "age", modxVals = "std.dev.")
    plotCurves(mc4, modx = "income", plotx = "age", plotPoints = FALSE)
}
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lazyCor.R
\name{lazyCor}
\alias{lazyCor}
\title{Create correlation matrices.}
\usage{
lazyCor(X, d)
}
\arguments{
\item{X}{Required. May be one value, a vech, or a matrix}

\item{d}{Optional. The number of rows in the correlation matrix to
be created. lazyCor will deduce the desired size from X if
possible. If X is a single value, d is a required argument.}
}
\value{
A correlation matrix.
}
\description{
Use can supply either a single value (the common correlation among
all variables), a column of the lower triangular values for a
correlation matrix, or a candidate matrix. The function will check
X and do the right thing. If X is a matrix, check that it
is a valid correlation matrix. If its a single value, use that
to fill up a matrix. If itis a vector, try to use it as a vech
to fill the lower triangle..
}
\examples{
lazyCor(0.5, 8)
lazyCor(c(0.1, 0.2, 0.3))
lazyCor(c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6))
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/centralValues.R
\name{centralValues}
\alias{centralValues}
\title{Central Tendency estimates for variables}
\usage{
centralValues(x)
}
\arguments{
\item{x}{a data frame}
}
\value{
a data frame with the same variables and one row, the summary indicators.
}
\description{
This is needed for the creation of summaries and predicted values
of regression models. It takes a data frame and returns a new data
frame with one row in which the mean or mode of the columns is
reported.
}
\examples{
myDat <- data.frame(x=rnorm(100), y=rpois(100,l=4), z = cut(rnorm(100), c(-10,-1,0,10)))
centralValues(myDat)
}
\author{
Paul E. Johnson \email{pauljohn@ku.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meanCenter.R
\name{standardize}
\alias{standardize}
\alias{standardize.lm}
\title{Estimate standardized regression coefficients for all variables}
\usage{
standardize(model)

\method{standardize}{lm}(model)
}
\arguments{
\item{model}{a fitted lm object}
}
\value{
an lm fitted with the standardized variables

a standardized regression object
}
\description{
This is brain-dead standardization of all variables in the design
matrix.  It mimics the silly output of SPSS, which standardizes
all regressors, even if they represent categorical variables.
}
\examples{

library(rockchalk)
N <- 100
dat <- genCorrelatedData(N = N, means = c(100,200), sds = c(20,30), rho = 0.4, stde = 10)
dat$x3 <- rnorm(100, m = 40, s = 4)

m1 <- lm(y ~ x1 + x2 + x3, data = dat)
summary(m1)

m1s <- standardize(m1)
summary(m1s)



m2 <- lm(y ~ x1 * x2 + x3, data = dat)
summary(m2)

m2s <- standardize(m2)
summary(m2s)

m2c <- meanCenter(m2)
summary(m2c)

}
\seealso{
\code{\link[rockchalk]{meanCenter}} which will center or
re-scale only numberic variables
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{checkIntFormat}
\alias{checkIntFormat}
\title{A way of checking if a string is a valid file name.}
\usage{
checkIntFormat(s)
}
\arguments{
\item{s}{An integer}
}
\value{
logical: TRUE or FALSE
}
\description{
A copy from R's grDevices:::checkIntFormat because it is not
exported there
}
\author{
R Core Development Team
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/groupMeanCenter.R
\name{gmc}
\alias{gmc}
\title{Group Mean Center: Generate group summaries and individual deviations within groups}
\usage{
gmc(dframe, x, by, FUN = mean, suffix = c("_mn", "_dev"),
  fulldataframe = TRUE)
}
\arguments{
\item{dframe}{a data frame.}

\item{x}{Variable names or a vector of variable names. Do NOT
supply a variable like dat$x1, do supply a quoted variable
name "x1" or a vector c("x1", "x2")}

\item{by}{A grouping variable name or a vector of grouping
names. Do NOT supply a variable like dat$xfactor, do supply a
name "xfactor", or a vector c("xfac1", "xfac2").}

\item{FUN}{Defaults to the mean, have not tested alternatives}

\item{suffix}{The suffixes to be added to column 1 and column 2}

\item{fulldataframe}{Default TRUE. original data frame is returned
with new columna added (which I would call "Stata style"). If
FALSE, this will return only newly created columns, the
variables with suffix[1] and suffix[2] appended to names.
TRUE is easier (maybe safer), but also wastes memory.}
}
\value{
Depending on \code{fulldataframe}, either a new data frame
    with center and deviation columns, or or original data frame
    with "x_mn" and "x_dev" variables appended (Stata style).
}
\description{
Multilevel modelers often need to include predictors like the
within-group mean and the deviations of individuals around the
mean. This function makes it easy (almost foolproof) to calculate
those variables.
}
\details{
This was originally just for "group mean-centered" data, but now
is more general, can accept functions like median to calculate
center and then deviations about that center value within the
group.

Similar to Stata egen, except more versatile and fun! Will create
2 new columns for each variable, with suffixes for the summary and
deviations (default suffixes are "_mn" and "_dev". Rows will match
the rows of the original data frame, so it will be easy to merge
or cbind them back together.
}
\examples{
## Make a data frame out of the state data collection (see ?state)
data(state)
statenew <- as.data.frame(state.x77)
statenew$region <- state.region
statenew$state <- rownames(statenew)
head(statenew.gmc1 <- gmc(statenew, c("Income", "Population"), by = "region"))
head(statenew.gmc2 <- gmc(statenew, c("Income", "Population"), by = "region",
   fulldataframe = FALSE))
## Note dangerous step: assumes row alignment is correct.
## return has rownames from original set to identify danger
head(statenew2 <- cbind(statenew, statenew.gmc2))
if(!all.equal(rownames(statenew), rownames(statenew.gmc2))){
   warning("Data row-alignment probable error")
}
## The following box plots should be identical
boxplot(Income ~ region, statenew.gmc1)
boxplot((Income_mn + Income_dev) ~ region, statenew.gmc1)
## Multiple by variables
fakedat <- data.frame(i = 1:200, j = gl(4, 50), k = gl(20, 10),
                      y1 = rnorm(200), y2 = rnorm(200))
head(gmc(fakedat, "y1", by = "k"), 20)
head(gmc(fakedat, "y1", by = c("j", "k"), fulldataframe = FALSE), 40)
head(gmc(fakedat, c("y1", "y2"), by = c("j", "k"), fulldataframe = FALSE))
## Check missing value management
fakedat[2, "k"] <- NA
fakedat[4, "j"] <- NA##' head(gmc(fakedat, "y1", by = "k"), 20)
head(gmc(fakedat, "y1", by = c("j", "k"), fulldataframe = FALSE), 40)
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mcDiagnose.R
\name{getVIF}
\alias{getVIF}
\title{Converts the R-square to the variance inflation factor}
\usage{
getVIF(rsq)
}
\arguments{
\item{rsq}{a vector of real values, presumably fitted R-squares}
}
\value{
a vector of vif estimates
}
\description{
calculates vif = 1/(1-R-square)
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mvrnorm.R
\name{mvrnorm}
\alias{mvrnorm}
\title{Minor revision of mvrnorm (from \code{MASS}) to facilitate replication}
\usage{
mvrnorm(n = 1, mu, Sigma, tol = 1e-06, empirical = FALSE)
}
\arguments{
\item{n}{the number of samples ("rows" of data) required.}

\item{mu}{a vector giving the means of the variables.}

\item{Sigma}{positive-definite symmetric matrix specifying the
covariance matrix of the variables.}

\item{tol}{tolerance (relative to largest variance) for numerical lack
of positive-definiteness in \code{Sigma}}

\item{empirical}{logical. If true, mu and Sigma specify the empirical
not population mean and covariance matrix.}
}
\value{
If \code{n = 1} a vector of the same length as \code{mu}, otherwise an
 \code{n} by \code{length(mu)} matrix with one sample in each row.
}
\description{
This is the \code{\link[MASS]{mvrnorm}} function from the MASS
package (Venables and Ripley, 2002), with one small modification
to facilitate replication of random samples. The aim is to make
sure that, after the seed is reset, the first rows of generated
data are identical no matter what value is chosen for n.  The one
can draw 100 observations, reset the seed, and then draw 110
observations, and the first 100 will match exactly. This is done
to prevent unexpected and peculiar patterns that are observed
when n is altered with MASS package's mvrnorm.
}
\details{
To assure replication, only a very small change is made. The code
in \code{MASS::mvrnorm} draws a random sample and fills a matrix
by column, and that matrix is then decomposed.  The change
implemented here fills that matrix by row and the problem is
eliminated.

Some peculiarities are noticed when the covariance matrix changes
from a diagonal matrix to a more general symmetric matrix
(non-zero elements off-diagonal).  When the covariance is strictly
diagonal, then just one column of the simulated multivariate
normal data will be replicated, but the others are not. This has
very troublesome implications for simulations that draw samples of
various sizes and then base calculations on the separate simulated
columns (i.e., some columns are identical, others are completely
uncorrelated).
}
\examples{
library(MASS)
library(rockchalk)

set.seed(12345)
X0 <- MASS::mvrnorm(n=10, mu = c(0,0,0), Sigma = diag(3))
## create a smaller data set, starting at same position
set.seed(12345)
X1 <- MASS::mvrnorm(n=5, mu = c(0,0,0), Sigma = diag(3))
## Create a larger data set
set.seed(12345)
X2 <- MASS::mvrnorm(n=15, mu = c(0,0,0), Sigma = diag(3))
## The first 5 rows in X0, X1, and X2 are not the same
X0
X1
X2
set.seed(12345)
Y0 <- mvrnorm(n=10, mu = c(0,0,0), Sigma = diag(3))
set.seed(12345)
Y1 <- mvrnorm(n=5, mu = c(0,0,0), Sigma = diag(3))
set.seed(12345)
Y2 <- mvrnorm(n=15, mu = c(0,0,0), Sigma = diag(3))
# note results are the same in the first 5 rows:
Y0
Y1
Y2
identical(Y0[1:5, ], Y1[1:5, ])
identical(Y1[1:5, ], Y2[1:5, ])

myR <- lazyCor(X = 0.3, d = 5)
mySD <- c(0.5, 0.5, 0.5, 1.5, 1.5)
myCov <- lazyCov(Rho = myR, Sd = mySD)

set.seed(12345)
X0 <- MASS::mvrnorm(n=10, mu = rep(0, 5), Sigma = myCov)
## create a smaller data set, starting at same position
set.seed(12345)
X1 <- MASS::mvrnorm(n=5, mu = rep(0, 5), Sigma = myCov)
X0
X1
##' set.seed(12345)
Y0 <- rockchalk::mvrnorm(n=10, mu = rep(0, 5), Sigma = myCov)
## create a smaller data set, starting at same position
set.seed(12345)
Y1 <- rockchalk::mvrnorm(n=5, mu = rep(0, 5), Sigma = myCov)
Y0
Y1

}
\references{
Venables, W. N. & Ripley, B. D. (2002) Modern Applied Statistics with
S. Fourth Edition. Springer, New York. ISBN 0-387-95457-0
}
\seealso{
For an alternative multivariate normal generator
function, one which has had this fix applied to it,
consider using the new versions of \code{\link[mvtnorm]{rmvnorm}} in the
package \code{mvtnorm}.
}
\author{
Ripley, B.D. with revision by Paul E. Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genCorrelatedData.R
\name{genCorrelatedData3}
\alias{genCorrelatedData3}
\title{Generate correlated data for simulations (third edition)}
\usage{
genCorrelatedData3(formula, N = 100, means = c(x1 = 50, x2 = 50, x3 =
  50), sds = 10, rho = 0, stde = 1, beta = c(0, 0.15, 0.1, -0.1),
  intercept = FALSE, col.names, verbose = FALSE, ...,
  distrib = rnorm)
}
\arguments{
\item{formula}{a text variable, e.g., \code{"y ~ 1 + 2*x1"}. Use
":" to create squared and interaction terms,
\code{"y ~ 1 + 2*x1 + 1.1*x1:x1 + 0.2*x1:x2".} Multi-way
interactions are allowed, eg
\code{"y ~ 1 + 2*x1 + .4*x2 + .1*x3 + 1.1*x1:x1 + 0.2*x1:x2:x3".}
Note author can specify any order of interation.}

\item{N}{sample size}

\item{means}{averages of predictors, can include names c(x1 = 10,
x2 = 20) that will be used in the data.frame result.}

\item{sds}{standard deviations, 1 (common value for all variables)
or as many elements as in \code{means}.}

\item{rho}{correlations, can be 1 or a vech for a correlation
matrix}

\item{stde}{The scale of the error term. If \code{distrib=rnorm},
stde is the standard deviation of the error term. If the user
changes the distribution, this is a scale parameter that may
not be equal to the standard deviation. For example,
\code{distrib=rlogist} has a scale parameter such that a value
of stde implies the error's standard deviation will be
\eqn{stde * pi / sqrt(3)}.}

\item{beta}{slope coefficients, use either this or \code{formula},
not both. It is easier (less error prone) to use named
coefficients, but (for backwards compatability with
\code{genCorrelatedData2}) names are not required. If named,
use "Intercept" for the intercept coefficient, and use
variable names that match the \code{xmeans} vector.  Un-named
coefficients follow same rules as in
\code{\link{genCorrelatedData2}}. The first (1 + p) values are
for the intercept and p main effects.  With 3 predictors and
no squares or interactions, specify four betas corresponding
to \code{c(Intercept, x1, x2, x3)}. The squared and
interaction terms may follow.  The largest possible model
would correspond to \code{c(Intercept, x1, x2, x3, x1:x1,
x1:x2, x1:x3, x2:x2, x2:x3, x3:x3)}.  Squares and interactions
fill in a "lower triangle". The unnamed beta vector can be
terminated with the last non-zero coefficient, the function
will insert 0's for the coefficients at the end of the vector.}

\item{intercept}{TRUE or FALSE. Should the output data set include
a column of 1's. If beta is an unnamed vector, should the
first element be treated as an intercept?}

\item{col.names}{Can override names in means vector}

\item{verbose}{TRUE for diagnostics}

\item{...}{extra arguments, ignored for now. We use that to ignore
unrecognized parameters.}

\item{distrib}{An R random data generating function.  Default is
\code{rnorm}. Also \code{rlogis} or any other two-parameter
location/scale distribution will work. Special configuration
allows \code{rt}. See details.}
}
\value{
a data frame
}
\description{
This is a revision of \code{genCorrelatedData2}.  The output is a
data frame that has columns for the predictors along with an error
term, the linear predictor, and the observed value of the outcome
variable.  The new features are in the user interface. It has a
better way to specify beta coefficients. It is also more flexible
in the specification of the names of the predictor columns.
}
\details{
The enhanced methods for authors to specify a data-generating
process are as follows. Either way will work and the choice
between the methods is driven by the author's convenience.\cr
\itemize{
\item 1. Use the formula argument as a quoted string:
\code{"1 + 2.2 * x1 + 2.3 * x2 + 3.3 * x3 + 1.9 * x1:x2"}.
The "*" represents multiplication of coefficient times variable,
and the colon ":" has same meaning but it is used for products of variables.
\item 2. Use the beta argument with parameter names, \code{beta =
c("Intercept" = 1, x1 = 2.2, x2 = 2.3, x3 = 3.3, "x1:x2" = 1.9)}
where the names are the same as the names of the variables in the
formula. Names of the variables in the formula or the beta vector
should be used also in either the means parameter or the col.names
parameter.
}

The error distribution can be specified. Default is normal, with
draws provided by R's \code{rnorm}. All error models assume
\eqn{E[e] = 0} and the scale coefficient is the parameter
\code{stde}. Thus, the default setup's error will be drawn from
\code{rnorm(N, 0, stde)}. Any two parameter "location" and "scale"
distribution should work as well, as long as the first coefficient
is location (because we set that as 0 in all cases) and the second
argument is scale. For example, \code{distrib=rlogis}, will lead
to errors drawn from \code{rlogis(N, 0, stde)}. Caution: in rlogis,
the scale parameter is not the same as standard deviation.

The only one parameter distribution currently supported is the T
distribution.  If user specifies \code{distrib=rt}, then the
\code{stde} is passed through to the parameter \code{df}. Note
that if increasing the stde parameter will cause the standard
deviation of \code{rt} to get smaller. \code{df=1} implies sd =
794.6; \code{df=2} implies sd = 3.27; \code{df=3} implies 1.7773.

Methods to specify error distributions in a more flexible way need
to be considered.
}
\examples{
set.seed(123123)
## note: x4 is an unused variable in formula
X1a <-
    genCorrelatedData3("y ~ 1.1 + 2.1 * x1 + 3 * x2 + 3.5 * x3 + 1.1 * x1:x3",
                       N = 1000, means = c(x1 = 1, x2 = -1, x3 = 3, x4 = 1),
                       sds = 1, rho = 0.4, stde = 5)
lm1a <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X1a)
## note that normal errors have std.error. close to 5
summary(lm1a)
attr(X1a, "beta") 
attr(X1a, "formula")
## Demonstrate name beta vector method to provide named arguments
set.seed(123123)
X2 <- genCorrelatedData3(N = 1000, means = c(x1 = 1, x2 = -1, x3 = 3, x4 = 1),
          sds = 1, rho = 0.4, 
          beta = c("Intercept" = 1.1, x1 = 2.1, x2 = 3,
                    x3 = 3.5, "x1:x3" = 1.1),
                    intercept = TRUE, stde = 5)
attr(X2, c("beta"))
attr(X2, c("formula"))
head(X2)
lm2 <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X2)
summary(lm2)

## Equivalent with unnamed beta vector. Must carefully count empty
## spots, fill in 0's when coefficient is not present. This
## method was in genCorrelated2. Order of coefficents is
## c(intercept, x1, ..., xp, x1:x1, x1:x2, x1:xp, x2:x2, x2:x3, ..., )
## filling in a lower triangle.
set.seed(123123)
X3 <- genCorrelatedData3(N = 1000, means = c(x1 = 1, x2 = -1, x3 = 3, x4 = 1),
          sds = 1, rho = 0.4, 
          beta = c(1.1, 2.1, 3, 3.5, 0, 0, 0, 1.1),
                    intercept = TRUE, stde = 5)
attr(X3, c("beta"))
attr(X3, c("formula"))
head(X3)
lm3 <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X3)
summary(lm3)

## Same with more interesting variable names in the means vector
X3 <- genCorrelatedData3(N = 1000,
          means = c(friend = 1, enemy = -1, ally = 3, neutral = 1),
          sds = 1, rho = 0.4, 
          beta = c(1.1, 2.1, 3, 3.5, 0, 0, 0, 1.1),
                    intercept = TRUE, stde = 5)
head(X3)
attr(X3, c("beta"))


X3 <- genCorrelatedData3(N = 1000, means = c(x1 = 50, x2 = 50, x3 = 50),
          sds = 10, rho = 0.4,
          beta = c("Intercept" = .1, x1 = .01, x2 = .2, x3 = .5,
                   "x1:x3" = .1))
lm3 <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X3)


## Names via col.names argument: must match formula
X2 <- genCorrelatedData3("y ~ 1.1 + 2.1 * educ + 3 * hlth + 3 * ses + 1.1 * educ:ses",
         N = 100, means = c(50, 50, 50, 20),
         sds = 10, rho = 0.4, col.names = c("educ", "hlth", "ses", "wght"))
str(X2) 

X3 <- genCorrelatedData3("y ~ 1.1 + 2.1 * educ + 3 * hlth + 3 * ses + 1.1 * educ:ses",
         N = 100, means = c(50, 50, 50, 20),
         sds = 10, rho = 0.4, col.names = c("educ", "hlth", "ses", "wght"),
         intercept = TRUE)
str(X3)

## note the logistic errors have residual std.error approximately 5 * pi/sqrt(3)
X1b <-
    genCorrelatedData3("y ~ 1.1 + 2.1 * x1 + 3 * x2 + 3.5 * x3 + 1.1 * x1:x3",
                       N = 1000, means = c(x1 = 1, x2 = -1, x3 = 3),
                       sds = 1, rho = 0.4, stde = 5, distrib = rlogis)
lm1b <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X1b)
summary(lm1b)

## t distribution is very sensitive for fractional df between 1 and 2 (recall
## stde parameter is passed through to df in rt.
X1c <-
    genCorrelatedData3("y ~ 1.1 + 2.1 * x1 + 3 * x2 + 3.5 * x3 + 1.1 * x1:x3",
                       N = 1000, means = c(x1 = 1, x2 = -1, x3 = 3),
                       sds = 1, rho = 0.4, stde = 1.2, distrib = rt)
lm1c <- lm(y ~ x1 + x2 + x3 + x1:x3, data = X1c)
summary(lm1c)

}
\author{
Paul Johnson <pauljohn@ku.edu> and Gabor Grothendieck <ggrothendieck@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genCorrelatedData.R
\name{genCorrelatedData}
\alias{genCorrelatedData}
\title{Generates a data frame for regression analysis}
\usage{
genCorrelatedData(N = 100, means = c(50, 50), sds = c(10, 10),
  rho = 0, stde = 1, beta = c(0, 0.2, 0.2, 0))
}
\arguments{
\item{N}{Number of cases desired}

\item{means}{2-vector of means for x1 and x2}

\item{sds}{2-vector of standard deviations for x1 and x2}

\item{rho}{Correlation coefficient for x1 and x2}

\item{stde}{standard deviation of the error term in the data
generating equation}

\item{beta}{beta vector of at most 4 coefficients for intercept,
slopes, and interaction}
}
\description{
The output is a data frame (x1, x2, y) with user-specified
correlation between x1 and x2. The y (output) variable is created
according to the equation\cr
\deqn{
y = beta1 + beta2 * x1 + beta3 * x2 + beta4 * x1 * x2 + e.
}
The arguments determine the scales of the X matrix, the random
error, and the slope coefficients.
}
\details{
The vector (x1,x2) is drawn from a multivariate normal
distribution in which the expected value (argument \code{means}).
The covariance matrix of X is
built from the standard deviations (\code{sds})
and the specified correlation between x1 and x2 (\code{rho}).
It is also necessary to specify the standard deviation
of the error term (\code{stde}) and the coefficients
of the regression equation (\code{beta}).
}
\examples{
## 1000 observations of uncorrelated x1 and x2 with no
## interaction between x1 and x2
dat <- genCorrelatedData(N=1000, rho=0, beta=c(1, 1.0, -1.1, 0.0))
  mcGraph1(dat$x1, dat$x2, dat$y, theta=20, phi=8,
  ticktype="detailed", nticks=10)
m1 <- lm(y ~ x1 + x2, data = dat)
plotPlane(m1, plotx1 = "x1", plotx2 = "x2")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genCorrelatedData.R
\name{genX}
\alias{genX}
\title{Generate correlated data (predictors) for one unit}
\usage{
genX(N, means, sds, rho, Sigma = NULL, intercept = TRUE,
  col.names = NULL, unit = NULL, idx = FALSE)
}
\arguments{
\item{N}{Number of cases desired}

\item{means}{A vector of means for p variables. It is optional to
name them.  This implicitly sets the dimension of the
predictor matrix as N x p. If no names are supplied, the
automatic variable names will be "x1", "x2", and so forth. If
means is named, such as c("myx1" = 7, "myx2" = 13, "myx3" =
44), those names will be come column names in the output
matrix.}

\item{sds}{Standard deviations for the variables.  If less than p
values are supplied, they will be recycled.}

\item{rho}{Correlation coefficient for p variables. Several input
formats are allowed (see \code{lazyCor}). This can be a single
number (common correlation among all variables), a full matrix
of correlations among all variables, or a vector that is
interpreted as the strictly lower triangle (a vech).}

\item{Sigma}{P x P variance/covariance matrix.}

\item{intercept}{Default = TRUE, do you want a first column filled
with 1?}

\item{col.names}{Names supplied here will override column names
supplied with the means parameter. If no names are supplied
with means, or here, we will name variables x1, x2, x3,
... xp, with Intercept at front of list if intercept =
TRUE.}

\item{unit}{A character string for the name of the unit being
simulated. Might be referred to as a "group" or "district" or
"level 2" membership indicator.}

\item{idx}{If set TRUE, a column "idx" is added, numbering the
rows from 1:N. If the argument unit is not NULL, then idx is
set to TRUE, but that behavior can be overridded by
setting idx = FALSE.}
}
\value{
A data frame with rownames to specify unit and
    individual values, including an attribute "unit" with the
    unit's name.
}
\description{
This is used to generate data for one unit. It is recently
re-designed to serve as a building block in a multi-level data
simulation exercise.  The new arguments "unit" and "idx" can be
set as NULL to remove the multi-level unit and row naming
features.  This function uses the rockchalk::mvrnorm function, but
introduces a convenience layer by allowing users to supply
standard deviations and the correlation matrix rather than the
variance.
}
\details{
Today I've decided to make the return object a data frame.  This
allows the possibility of including a character variable "unit"
within the result.  For multi-level models, that will help.  If
unit is not NULL, its value will be added as a column in the data
frame. If unit is not null, the rownames will be constructed by
pasting "unit" name and idx. If unit is not null, then idx will
be included as another column, unless the user explicitly sets
idx = FALSE.
}
\examples{
X1 <- genX(10, means = c(7, 8), sds = 3, rho = .4)
X2 <- genX(10, means = c(7, 8), sds = 3, rho = .4, unit = "Kansas")
head(X2)
X3 <- genX(10, means = c(7, 8), sds = 3, rho = .4, idx = FALSE, unit = "Iowa")
head(X3)
X4 <- genX(10, means = c("A" = 7, "B" = 8), sds = c(3), rho = .4)
head(X4)
X5 <- genX(10, means = c(7, 3, 7, 5), sds = c(3, 6),
            rho = .5, col.names = c("Fred", "Sally", "Henry", "Barbi"))
head(X5)
Sigma <- lazyCov(Rho = c(.2, .3, .4, .5, .2, .1), Sd = c(2, 3, 1, 4))
X6 <- genX(10, means = c(5, 2, -19, 33), Sigma = Sigma, unit = "Winslow_AZ")
head(X6)

}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pctable.R
\name{print.summary.pctable}
\alias{print.summary.pctable}
\title{print method for summary.pctable objects}
\usage{
\method{print}{summary.pctable}(x, ...)
}
\arguments{
\item{x}{a summary.pctable object}

\item{...}{Other arguments to print method}
}
\value{
Nothing is returned
}
\description{
prints pctab objects. Needed only to deal properly with quotes
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotPlane.R
\name{addLines}
\alias{addLines}
\title{Superimpose regression lines on a plotted plane}
\usage{
addLines(to = NULL, from = NULL, col, lwd = 2, lty = 1)
}
\arguments{
\item{to}{a 3d plot object produced by plotPlane}

\item{from}{output from a plotSlopes or plotCurves function (class="rockchalk")}

\item{col}{color of plotted lines (default: "red")}

\item{lwd}{line width of added lines (default: 2)}

\item{lty}{line type of added lines (default: 1)}
}
\value{
NULL, nothing, nicht, nada.
}
\description{
The examples will demonstrate the intended usage.
}
\details{
From an educational stand point, the objective is to assist with the
student's conceptualization of the two and three dimensional regression
relationships.
}
\examples{
##library(rockchalk)


set.seed(12345)

dfadd <- genCorrelatedData2(100, means = c(0,0,0,0), sds = 1, rho = 0,
       beta = c(0.03, 0.01, 0.1, 0.4, -0.1), stde = 2)

dfadd$xcat1 <- gl(2,50, labels=c("M","F"))

dfadd$xcat2 <- cut(rnorm(100), breaks=c(-Inf, 0, 0.4, 0.9, 1, Inf),
                   labels=c("R", "M", "D", "P", "G"))

dfadd$y2 <- 0.03 + 0.1*dfadd$x1 + 0.1*dfadd$x2 +
    0.25*dfadd$x1*dfadd$x2 + 0.4*dfadd$x3 - 0.1*dfadd$x4 +
    0.2 * as.numeric(dfadd$xcat1) +
    contrasts(dfadd$xcat2)[as.numeric(dfadd$xcat2), ] \%*\% c(-0.1, 0.1, 0.2, 0) +
    1 * rnorm(100)

summarize(dfadd)

## linear ordinary regression
m1 <- lm(y ~ x1 + x2 + x3 + x4, data = dfadd)
summary(m1)

mcDiagnose(m1)

## These will be parallel lines

plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = "std.dev.",
           n = 5, main = "A plotSlopes result with \\"std.dev.\\" values of modx")


m1ps <- plotSlopes(m1, plotx = "x1", modx = "x2", modxVals = c(-2,0,2))

m1pp <- plotPlane(m1, plotx1 = "x1", plotx2 = "x2",
                  ticktype = "detailed", npp = 10)

addLines(from = m1ps, to = m1pp, lty = 1, lwd = 2)

m1pp <- plotPlane(m1, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed",
                  npp = 10)
addLines(from = m1ps, to = m1pp, lty = 2, lwd = 5, col = "green")

## Other approach would wrap same into the linesFrom argument in plotPlane

plotPlane(m1, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed",
          npp = 10, linesFrom = m1ps)

## Need to think more on whether dotted lines from ps object should
## be converted to solid lines in plotPlane.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{summarize}
\alias{summarize}
\title{Sorts numeric from discrete variables and returns separate
summaries for those types of variables.}
\usage{
summarize(dat, alphaSort = FALSE, stats = c("mean", "sd", "skewness",
  "kurtosis", "entropy", "normedEntropy", "nobs", "nmiss"), probs = c(0,
  0.5, 1), digits = 3, ...)
}
\arguments{
\item{dat}{A data frame}

\item{alphaSort}{If TRUE, the columns are re-organized in
alphabetical order. If FALSE, they are presented in the
original order.}

\item{stats}{A vector of desired summary statistics. Set
\code{stats = NULL} to omit all stat summaries. Legal elements
are \code{c("min", "med", "max", "mean", "sd", "var",
"skewness", "kurtosis", "entropy", "normedEntropy", "nobs",
"nmiss")}. The statistics \code{c("entropy", "normedEntropy")}
are available only for factor variables, while mean, variance,
and so forth will be calculated only for numeric variables.
\code{"nobs"} is the number of observations with non-missing,
finite scores (not NA, NaN, -Inf, or Inf). \code{"nmiss"} is
the number of cases with values of NA. The default setting for
\code{probs} will cause \code{c("min", "med", "max")} to be
included, they need not be requested explicitly. To disable
them, revise \code{probs}.}

\item{probs}{For numeric variables, is used with the
\code{quantile} function. The default is
\code{probs = c(0, .50, 1.0)}, which are labeled in output as
\code{c("min", "med", and "max")}. Set \code{probs = NULL}
to prevent these in the output.}

\item{digits}{Decimal values to display, defaults as 2.}

\item{...}{Optional arguments that are passed to
\code{summarizeNumerics} and \code{summarizeFactors}. For
numeric variables, one can specify \code{na.rm} and
\code{unbiased}.  For discrete variables, the key argument is
\code{maxLevels}, which determines the number of levels that
will be reported in tables for discrete variables.}
}
\value{
Return is a list with two objects 1) output from
    summarizeNumerics: a data frame with variable names on rows
    and summary stats on columns, 2) output from summarizeFactors:
    a list with summary information about each discrete
    variable. The display on-screen is governed by a method
    \code{print.summarize}.
}
\description{
The work is done by the functions \code{summarizeNumerics} and
\code{summarizeFactors}.  Please see the help pages for those
functions for complete details.
}
\details{
The major purpose here is to generate summary data structure
that is more useful in subsequent data analysis.  The numeric
portion of the summaries are a data frame that can be used in
plots or other diagnostics.

The term "factors" was used, but "discrete variables" would have
been more accurate.  The factor summaries will collect all
logical, factor, ordered, and character variables.

Other variable types, such as Dates, will be ignored, with a
warning.
}
\examples{
library(rockchalk)


set.seed(23452345)
N <- 100
x1 <- gl(12, 2, labels = LETTERS[1:12])
x2 <- gl(8, 3, labels = LETTERS[12:24])
x1 <- sample(x = x1, size=N, replace = TRUE)
x2 <- sample(x = x2, size=N, replace = TRUE)
z1 <- rnorm(N)
a1 <- rnorm(N, mean = 1.2, sd = 11.7)
a2 <- rpois(N, lambda = 10 + abs(a1))
a3 <- rgamma(N, 0.5, 4)
b1 <- rnorm(N, mean = 211.3, sd = 0.4)
dat <- data.frame(z1, a1, x2, a2, x1, a3, b1)
summary(dat)

summarize(dat)

summarize(dat, digits = 4)

summarize(dat, stats = c("min", "max", "mean", "sd"),
          probs = c(0.25, 0.75))

summarize(dat, probs = c(0, 0.20, 0.80),
          stats = c("nobs", "mean", "med", "entropy"))

summarize(dat, probs = c(0, 0.20, 0.50),
          stats = c("nobs", "nmiss", "mean", "entropy"), maxLevels=10)

dat.sum <- summarize(dat, probs = c(0, 0.20, 0.50),
                     stats = c("nobs", "nmiss", "mean", "entropy"), maxLevels=10)
dat.sum
## Inspect unformatted structure of objects within return
dat.sum[["numerics"]]
dat.sum[["factors"]]

## Only quantile values, no summary stats for numeric variables
## Discrete variables get entropy
summarize(dat, 
          probs = c(0, 0.25, 0.50, 0.75, 1.0),
          stats = "entropy", digits = 2)

## Quantiles and the mean for numeric variables.
## No diversity stats for discrete variables (entropy omitted)
summarize(dat, 
          probs = c(0, 0.25, 0.50, 0.75, 1.0),
          stats = "mean")

summarize(dat, 
          probs = NULL,
          stats = "mean")

## Note: output is not beautified by a print method
dat.sn <- summarizeNumerics(dat)
dat.sn
formatSummarizedNumerics(dat.sn)
formatSummarizedNumerics(dat.sn, digits = 5)

dat.summ <- summarize(dat)


dat.sf <- summarizeFactors(dat, maxLevels = 20)
dat.sf
formatSummarizedFactors(dat.sf)

## See actual values of factor summaries, without
## beautified printing
summarizeFactors(dat, maxLevels = 5)
formatSummarizedFactors(summarizeFactors(dat, maxLevels = 5))

summarize(dat, alphaSort = TRUE) 

summarize(dat, digits = 6, alphaSort = FALSE)


summarize(dat, maxLevels = 2)

datsumm <- summarize(dat, stats = c("mean", "sd", "var", "entropy", "nobs"))

## Unbeautified numeric data frame, variables on the rows
datsumm[["numerics"]]
## Beautified versions 1. shows saved version:
attr(datsumm, "numeric.formatted")
## 2. Run formatSummarizedNumerics to re-specify digits:
formatSummarizedNumerics(datsumm[["numerics"]], digits = 10)

datsumm[["factors"]]
formatSummarizedFactors(datsumm[["factors"]])
formatSummarizedFactors(datsumm[["factors"]], digits = 6, maxLevels = 10)
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rockchalk-package.R
\docType{data}
\name{religioncrime}
\alias{religioncrime}
\title{Religious beliefs and crime rates}
\format{data.frame: 51 obs. of 3 variables}
\source{
Anonymous researcher who claims the data is real.
}
\usage{
data(religioncrime)
}
\description{
The data national-level summary indicators of public opinion about
the existence of heaven and hell as well as the national rate of
violent crime.
}
\examples{
require(rockchalk)
data(religioncrime)
mod1 <- lm(crime ~ heaven, data=religioncrime)
mod2 <- lm(crime ~ hell, data=religioncrime)
mod3 <- lm(crime ~ heaven + hell, data=religioncrime)
with(religioncrime,
mcGraph1(heaven, hell, crime)
)
with(religioncrime,
mcGraph2(heaven, hell, crime)
)
mod1 <- with(religioncrime,
mcGraph3(heaven, hell, crime)
)
summary(mod1[[1]])
##TODO: Draw more with perspective matrix mod1[[2]]
}
\author{
Paul E. Johnson \email{pauljohn@ku.edu} and Anonymous
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{kurtosis}
\alias{kurtosis}
\title{Calculate excess kurtosis}
\usage{
kurtosis(x, na.rm = TRUE, excess = TRUE, unbiased = TRUE)
}
\arguments{
\item{x}{A numeric variable (vector)}

\item{na.rm}{default TRUE. If na.rm = FALSE and there are missing
values, the mean and variance are undefined and this function
returns NA.}

\item{excess}{default TRUE. If true, function returns excess
kurtosis (kurtosis -3). If false, the return is simply
kurtosis as defined above.}

\item{unbiased}{default TRUE. Should the denominator of the
variance estimate be divided by N-1, rather than N?}
}
\value{
A scalar value or NA
}
\description{
Kurtosis is a summary of a distribution's shape, using the Normal
distribution as a comparison. A distribution with high kurtosis is
said to be leptokurtic. It has wider, "fatter" tails and a
"sharper", more "peaked" center than a Normal distribution. In a
standard Normal distribution, the kurtosis is 3.  The term
"excess kurtosis" refers to the difference \eqn{kurtosis - 3}.
Many researchers use the term kurtosis to refer to
"excess kurtosis" and this function follows suit.  The user may
set excess = FALSE, in which case the uncentered kurtosis is
returned.
}
\details{
If kurtosis is smaller than 3 (or excess kurtosis is negative),
the tails are "thinner" than the normal distribution (there is
lower chance of extreme deviations around the mean). If kurtosis
is greater than 3 (excess kurtosis positive), then the tails are
fatter (observations can be spread more widely than in the Normal
distribution).

The kurtosis may be calculated with the small-sample
bias-corrected estimate of the variance. Set unbiased = FALSE if
this is not desired.  It appears somewhat controversial whether
this is necessary. According to the
US NIST,
\url{http://www.itl.nist.gov/div898/handbook/eda/section3/eda35b.htm},
kurtosis is defined as
                  
\deqn{kurtosis =  ( mean((x - mean(x))^4) )/ var(x)^2}
            
where \eqn{var(x)} is calculated with the denominator \eqn{N},
rather than \eqn{N-1}.

A distribution is said to be leptokurtic if it is tightly bunched
in the center (spiked) and there are long tails. The long tails
reflect the probability of extreme values.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drawnorm.R
\name{drawnorm}
\alias{drawnorm}
\title{draw a normal distribution with beautiful illustrations}
\usage{
drawnorm(mu = 0, sigma = 1, xlab = "A Normally Distributed Variable",
  ylab = "Probability Density", main, ps = par("ps"), ...)
}
\arguments{
\item{mu}{The mu parameter}

\item{sigma}{The sigma parameter}

\item{xlab}{Label for x axis}

\item{ylab}{Label for Y axis}

\item{main}{Title for plot. OK to ignore this, we'll make a nice one for you}

\item{ps}{pointsize of text}

\item{...}{arguments passed to par}
}
\description{
This was developed for the R Working Example collection
in my website, pj.freefaculty.org/R/WorkingExamples
}
\examples{
drawnorm(mu = 10, sigma = 20)
drawnorm(mu= 0, sigma = 1)
drawnorm(mu = 102, sigma = 313)
drawnorm(mu = 0, sigma = 1, main = "A Standard Normal Distribution, N(0,1)",
         xlab = "X", ylab = "Density", ps = 7)
drawnorm(mu = 0, sigma = 1, ylab = "Density", ps = 14) 
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/removeNULL.R
\name{removeNULL}
\alias{removeNULL}
\title{Remove NULL values variables from a list}
\usage{
removeNULL(aList)
}
\arguments{
\item{aList}{A list}
}
\value{
Same list with NULL's removed
}
\description{
Unlike vectors, lists can hold objects with
value NULL. This gets rid of them.
}
\details{
This version is NOT recursive

plyr::rbind.fill uses an experimental function
that I choose to avoid. This is the "safe" version.
}
\examples{
## Note it is non-recursive, NULL remains in e
x <- list(a = rnorm(5), b = NULL, c = rnorm(5), d = NULL,
     e = list(f = rnorm(2), g = NULL))
x
removeNULL(x)
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pctable.R
\name{print.pctable}
\alias{print.pctable}
\title{Display pctable objects}
\usage{
\method{print}{pctable}(x, colpct = TRUE, rowpct = FALSE, ...)
}
\arguments{
\item{x}{A pctable object}

\item{colpct}{Default TRUE: include column percentages?}

\item{rowpct}{Default FALSE: include row percentages?}

\item{...}{Other arguments passed through to print method}
}
\value{
A table object for the final printed table.
}
\description{
This is not very fancy. Note that the saved pctable object
has the information inside it that is required to write both
column and row percentages. The arguments colpct and rowpct
are used to ask for the two types.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotPlane.R
\name{plotPlane}
\alias{plotPlane}
\alias{plotPlane.default}
\title{Draw a 3-D regression plot for two predictors from any linear or nonlinear lm or glm object}
\usage{
plotPlane(model = NULL, plotx1 = NULL, plotx2 = NULL,
  drawArrows = FALSE, plotPoints = TRUE, npp = 20, x1lab, x2lab,
  ylab, x1lim, x2lim, x1floor = 5, x2floor = 5, pch = 1,
  pcol = "blue", plwd = 0.5, pcex = 1, llwd = 0.3, lcol = 1,
  llty = 1, acol = "red", alty = 4, alwd = 0.3, alength = 0.1,
  linesFrom, lflwd = 3, envir = environment(formula(model)), ...)

\method{plotPlane}{default}(model = NULL, plotx1 = NULL,
  plotx2 = NULL, drawArrows = FALSE, plotPoints = TRUE, npp = 20,
  x1lab, x2lab, ylab, x1lim, x2lim, x1floor = 5, x2floor = 5,
  pch = 1, pcol = "blue", plwd = 0.5, pcex = 1, llwd = 0.3,
  lcol = 1, llty = 1, acol = "red", alty = 4, alwd = 0.3,
  alength = 0.1, linesFrom, lflwd = 3,
  envir = environment(formula(model)), ...)
}
\arguments{
\item{model}{an lm or glm fitted model object}

\item{plotx1}{name of one variable to be used on the x1 axis}

\item{plotx2}{name of one variable to be used on the x2 axis}

\item{drawArrows}{draw red arrows from prediction plane toward observed values TRUE or FALSE}

\item{plotPoints}{Should the plot include scatter of observed scores?}

\item{npp}{number of points at which to calculate prediction}

\item{x1lab}{optional label}

\item{x2lab}{optional label}

\item{ylab}{optional label}

\item{x1lim}{optional lower and upper bounds for x1, as vector like c(0,1)}

\item{x2lim}{optional lower and upper bounds for x2, as vector like c(0,1)}

\item{x1floor}{Default=5. Number of "floor" lines to be drawn for variable x1}

\item{x2floor}{Default=5. Number of "floor" lines to be drawn for variable x2}

\item{pch}{plot character, passed on to the "points" function}

\item{pcol}{color for points, col passed to "points" function}

\item{plwd}{line width, lwd passed to "points" function}

\item{pcex}{character expansion, cex passed to "points" function}

\item{llwd}{line width, lwd passed to the "lines" function}

\item{lcol}{line color, col passed to the "lines" function}

\item{llty}{line type, lty passed to the "lines" function}

\item{acol}{color for arrows, col passed to "arrows" function}

\item{alty}{arrow line type, lty passed to the "arrows" function}

\item{alwd}{arrow line width, lwd passed to the "arrows" function}

\item{alength}{arrow head length, length passed to "arrows" function}

\item{linesFrom}{object with information about "highlight" lines to be added to the 3d plane (output from plotCurves or plotSlopes)}

\item{lflwd}{line widths for linesFrom highlight lines}

\item{envir}{environment from whence to grab data}

\item{...}{additional parameters that will go to persp}
}
\value{
The main point is the plot that is drawn, but for record
keeping the return object is a list including 1) res: the
transformation matrix that was created by persp 2) the call that
was issued, 3) x1seq, the "plot sequence" for the x1 dimension, 4)
x2seq, the "plot sequence" for the x2 dimension, 5) zplane, the
values of the plane corresponding to locations x1seq and x2seq.
}
\description{
This allows user to fit a regression model with many variables and
then plot 2 of its predictors and the output plane for those
predictors with other variables set at mean or mode (numeric or
factor).  This is a front-end (wrapper) for R's persp function.
Persp does all of the hard work, this function reorganizes the
information for the user in a more readily understood way.  It
intended as a convenience for students (or others) who do not
want to fight their way through the details needed to use persp to
plot a regression plane.  The fitted model can have any number of
input variables, this will display only two of them. And, at least
for the moment, I insist these predictors must be numeric
variables. They can be transformed in any of the usual ways, such
as poly, log, and so forth.
}
\details{
Besides a fitted model object, plotPlane requires two additional
arguments, plotx1 and plotx2. These are the names of the plotting
variables. Please note, that if the term in the regression is
something like poly(fish,2) or log(fish), then the argument to
plotx1 should be the quoted name of the variable "fish".
plotPlane will handle the work of re-organizing the information so
that R's predict functions can generate the desired information.
This might be thought of as a 3D version of "termplot", with a
significant exception. The calculation of predicted values depends
on predictors besides plotx1 and plotx2 in a different ways. The
sample averages are used for numeric variables, but for factors
the modal value is used.

This function creates an empty 3D drawing and then fills in the
pieces. It uses the R functions \code{lines}, \code{points}, and
\code{arrows}. To allow customization, several parameters are
introduced for the users to choose colors and such. These options
are prefixed by "l" for the lines that draw the plane, "p" for the
points, and "a" for the arrows. Of course, if plotPoints=FALSE or
drawArrows=FALSE, then these options are irrelevant.
}
\examples{
library(rockchalk)


set.seed(12345)
x1 <- rnorm(100)
x2 <- rnorm(100)
x3 <- rnorm(100)
x4 <- rnorm(100)
y <- rnorm(100)
y2 <- 0.03 + 0.1*x1 + 0.1*x2 + 0.25*x1*x2 + 0.4*x3 -0.1*x4 + 1*rnorm(100)
dat <- data.frame(x1,x2,x3,x4,y, y2)
rm(x1, x2, x3, x4, y, y2)

## linear ordinary regression
m1 <- lm(y ~ x1 + x2 +x3 + x4, data = dat)

plotPlane(m1, plotx1 = "x3", plotx2 = "x4")

plotPlane(m1, plotx1 = "x3", plotx2 = "x4", drawArrows = TRUE)

plotPlane(m1, plotx1 = "x1", plotx2 = "x4", drawArrows = TRUE)


plotPlane(m1, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE, npp = 10)
plotPlane(m1, plotx1 = "x3", plotx2 = "x2", drawArrows = TRUE, npp = 40)

plotPlane(m1, plotx1 = "x3", plotx2 = "x2", drawArrows = FALSE,
          npp = 5, ticktype = "detailed")


## regression with interaction
m2 <- lm(y ~ x1  * x2 +x3 + x4, data = dat)

plotPlane(m2, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE)


plotPlane(m2, plotx1 = "x1", plotx2 = "x4", drawArrows = TRUE)
plotPlane(m2, plotx1 = "x1", plotx2 = "x3", drawArrows = TRUE)

plotPlane(m2, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE,
          phi = 10, theta = 30)



## regression with quadratic;
## Required some fancy footwork in plotPlane, so be happy
dat$y3 <- 0 + 1 * dat$x1 + 2 * dat$x1^2 + 1 * dat$x2 +
    0.4*dat$x3 + 8 * rnorm(100)
m3 <- lm(y3 ~ poly(x1,2) + x2 +x3 + x4, data = dat)
summary(m3)

plotPlane(m3, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE,
          x1lab = "my great predictor", x2lab = "a so-so predictor",
          ylab = "Most awesomest DV ever")

plotPlane(m3, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE,
          x1lab = "my great predictor", x2lab = "a so-so predictor",
          ylab = "Most awesomest DV ever", phi = -20)

plotPlane(m3, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE,
          phi = 10, theta = 30)

plotPlane(m3, plotx1 = "x1", plotx2 = "x4", drawArrows = TRUE,
          ticktype = "detailed")
plotPlane(m3, plotx1 = "x1", plotx2 = "x3", drawArrows = TRUE)

plotPlane(m3, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE,
          phi = 10, theta = 30)

m4 <- lm(y ~ sin(x1) + x2*x3 +x3 + x4, data = dat)
summary(m4)


plotPlane(m4, plotx1 = "x1", plotx2 = "x2", drawArrows = TRUE)
plotPlane(m4, plotx1 = "x1", plotx2 = "x3", drawArrows = TRUE)



eta3 <- 1.1 + .9*dat$x1 - .6*dat$x2 + .5*dat$x3
dat$y4 <- rbinom(100, size = 1, prob = exp( eta3)/(1+exp(eta3)))
gm1 <- glm(y4 ~ x1 + x2 + x3, data = dat, family = binomial(logit))
summary(gm1)
plotPlane(gm1, plotx1 = "x1", plotx2 = "x2")
plotPlane(gm1, plotx1 = "x1", plotx2 = "x2", phi = -10)

plotPlane(gm1, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed")
plotPlane(gm1, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed",
          npp = 30, theta = 30)
plotPlane(gm1, plotx1 = "x1", plotx2 = "x3", ticktype = "detailed",
          npp = 70, theta = 60)

plotPlane(gm1, plotx1 = "x1", plotx2 = "x2", ticktype = c("detailed"),
          npp = 50, theta = 40)

dat$x2 <- 5 * dat$x2
dat$x4 <- 10 * dat$x4
eta4 <- 0.1 + .15*dat$x1 - 0.1*dat$x2 + .25*dat$x3 + 0.1*dat$x4
dat$y4 <- rbinom(100, size = 1, prob = exp( eta4)/(1+exp(eta4)))
gm2 <- glm(y4 ~ x1 + x2 + x3 + x4, data = dat, family = binomial(logit))
summary(gm2)
plotPlane(gm2, plotx1 = "x1", plotx2 = "x2")
plotPlane(gm2, plotx1 = "x2", plotx2 = "x1")
plotPlane(gm2, plotx1 = "x1", plotx2 = "x2", phi = -10)
plotPlane(gm2, plotx1 = "x1", plotx2 = "x2", phi = 5, theta = 70, npp = 40)

plotPlane(gm2, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed")
plotPlane(gm2, plotx1 = "x1", plotx2 = "x2", ticktype = "detailed",
          npp = 30, theta = -30)
plotPlane(gm2, plotx1 = "x1", plotx2 = "x3", ticktype = "detailed",
          npp = 70, theta = 60)

plotPlane(gm2, plotx1 = "x4", plotx2 = "x3", ticktype = "detailed",
          npp = 50, theta = 10)

plotPlane(gm2, plotx1 = "x1", plotx2 = "x2", ticktype = c("detailed"))
}
\seealso{
\code{\link[graphics]{persp}}, \code{\link[scatterplot3d]{scatterplot3d}}, \code{\link[HH]{regr2.plot}}
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meanCenter.R
\name{centerNumerics}
\alias{centerNumerics}
\title{Find numeric columns, center them, re-name them, and join them with the original data.}
\usage{
centerNumerics(data, center, standardize = FALSE)
}
\arguments{
\item{data}{Required. data frame or matrix.}

\item{center}{Optional. If nc is NOT supplied, then all numeric columns
in data will be centered (possiblly scaled).  Can be specified in 2 formats. 1) Vector of column names that are to be centered, 2) Vector named elements giving values of means to be used in centering.  Values must be named, as in c("x1" = 17, "x2" = 44).
(possibly scaled).}

\item{standardize}{Default FALSE. If TRUE, the variables are
first mean-centered, and then divided by their standard deviations
(scaled). User can supply a named vector of scale values by which
to divide each variable (otherwise sd is used). Vector must have same
names and length as center argument. Variables can be entered in any order (will be resorted inside function).}
}
\value{
A data frame with 1) All original columns 2) additional
columns with centered/scaled data, variables renamed "c" or "cs"
to indicate the data is centered or centered and
scaled. Attributes "centers" and "scales" are created for "record
keeping" on centering and scaling values.
}
\description{
The meanCentered regression function requires centered-inputs when
calculations are predicted. For comparison with ordinary
regression, it is convenient to have both centered and the
original data side-by-side.  This function handles that.  If the
input data has columns c("x1","x2","x3"), then the centered result
will have columns c("x1","x2","x3","x1c","x2c","x3c"), where "c"
indicates "mean-centered". If standardize=TRUE, then the result
will have columns c("x1","x2","x3","x1cs","x2cs","x3cs"), where "cs"
indicate "centered and scaled".
}
\examples{
set.seed(12345)
dat <- data.frame(x1=rnorm(100, m = 50), x2 = rnorm(100, m = 50),
    x3 = rnorm(100, m = 50), y = rnorm(100),
    x4 = gl(2, 50, labels = c("Male","Female")))
datc1 <- centerNumerics(dat)
head(datc1)
summarize(datc1)
datc2 <- centerNumerics(dat, center=c("x1", "x2"))
head(datc2)
summarize(datc2)
attributes(datc2)
datc3 <- centerNumerics(dat, center = c("x1" = 30, "x2" = 40))
head(datc3)
summarize(datc3)
attributes(datc3)
datc4 <- centerNumerics(dat, center=c("x1", "x2"), standardize = TRUE)
head(datc3)
summarize(datc4)
attributes(datc4)
datc5 <- centerNumerics(dat, center=c("x1"=30, "x2"=40),
standardize = c("x2" = 5, "x1" = 7))
head(datc5)
summarize(datc5)
attributes(datc5)
}
\author{
<pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/magRange.R
\name{magRange}
\alias{magRange}
\title{magRange
Magnify the range of a variable.}
\usage{
magRange(x, mult = 1.25)
}
\arguments{
\item{x}{an R vector variable}

\item{mult}{a multiplier by which to magnify the range of the
variable.  A value of 1 leaves the range unchanged. May be a
scalar, in which case both ends of the range are magnified by the
same amount.  May also be a two valued vector, such as c(minMag,
maxMag), in which case the magnification applied to the minimum is
minMag and the magnification of the maximum is maxMag.  After
version 1.5.5, mult may be smaller than 1, thus shrinking the
range. Setting mult to values closer to 0 causes the range to
shrink to the center point from both sides.}
}
\description{
By default, R's range function returns the minimum and maximum
values of a variable. This returns a magnified range. It is used
for some plotting functions in the rockchalk package
}
\examples{
x1 <- c(0, 0.5, 1.0)
range(x1)
magRange(x1, 1.1)
magRange(x1, c(1.1, 1.4))
magRange(x1, 0.5)
magRange(x1, c(0.1, 0.1))
x1 <- rnorm(100)
range(x1)
magRange(x1)
magRange(x1, 1.5)
magRange(x1, c(1,1.5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cutByTable}
\alias{cutByTable}
\title{Select most frequently occurring values from numeric or categorical variables.}
\usage{
cutByTable(x, n = 5, pct = TRUE)
}
\arguments{
\item{x}{A numeric or character variable}

\item{n}{The maximum number of values that may be returned.}

\item{pct}{Default = TRUE. Include percentage of responses within each category}
}
\value{
A named vector.
}
\description{
The "n" most frequently occurring values are returned, sorted by
frequency of occurrence (in descending order). The names attribute
includes information about the percentage of cases that have the
indicated values.
}
\details{
This is used by plotSlopes, plotCurves, and other "newdata" making
functions.
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarize.R
\name{skewness}
\alias{skewness}
\title{Calculate skewness}
\usage{
skewness(x, na.rm = TRUE, unbiased = TRUE)
}
\arguments{
\item{x}{A numeric variable (vector)}

\item{na.rm}{default TRUE. Should missing data be removed?}

\item{unbiased}{default TRUE. Should the denominator of the
variance estimate be divided by N-1?}
}
\value{
A scalar value or NA
}
\description{
Skewness is a summary of the symmetry of a distribution's
probability density function. In a Normal distribution, the
skewness is 0, indicating symmetry about the expected value.
}
\details{
If na.rm = FALSE and there are missing values, the mean and
variance are undefined and this function returns NA.

The skewness may be calculated with the small-sample bias-corrected
estimate of the standard deviation.  It appears somewhat controversial
whether this is necessary, hence the argument unbiased is provided.
Set unbiased = FALSE if it is desired to have the one recommended
by NIST, for example. According to the US NIST,
\url{http://www.itl.nist.gov/div898/handbook/eda/section3/eda35b.htm},
skewness is defined as the mean of cubed deviations divided by the
cube of the standard deviation.
      
               mean((x - mean(x))^3)
skewness =    ___________________
                 sd(x)^3

where sd(x) is calculated with the denominator N, rather than
N-1. This is the Fisher-Pearson coefficient of skewness, they claim.
The unbiased variant uses the standard deviation divisor (N-1) to
bias-correct the standard deviation.
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSlopes.R
\name{se.bars}
\alias{se.bars}
\title{Draw standard error bar for discrete variables}
\usage{
se.bars(x, y, lwr, upr, width = 0.2, col, opacity = 120, lwd = 1,
  lty = 1)
}
\arguments{
\item{x}{The x center point}

\item{y}{The fitted "predicted" value}

\item{lwr}{The lower confidence interval bound}

\item{upr}{The upper confidence interval bound}

\item{width}{Thickness of shaded column}

\item{col}{Color for a bar}

\item{opacity}{Value in c(0, 254). 120 is default, that's partial see through.}

\item{lwd}{line width, usually 1}

\item{lty}{line type, usually 1}
}
\description{
Used with plotSlopes if plotx is discrete. This is not currently
exported.
}
\author{
Paul Johnson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mvrnorm.R
\name{checkPosDef}
\alias{checkPosDef}
\title{Check a matrix for positive definitness}
\usage{
checkPosDef(X, tol = 1e-06)
}
\arguments{
\item{X}{A matrix}

\item{tol}{Tolerance (closeness to 0 required to declare failure)}
}
\value{
TRUE or FALSE
}
\description{
Uses eigen to check positive definiteness. Follows example used
in \code{MASS} package by W. N. Venables and Brian D. Ripley
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pctable.R
\name{summary.pctable}
\alias{summary.pctable}
\title{Extract presentation from a pctable object}
\usage{
\method{summary}{pctable}(object, ..., colpct = TRUE, rowpct = FALSE)
}
\arguments{
\item{object}{A pctable object}

\item{...}{Other arguments, currently unused}

\item{colpct}{Default TRUE: should column percents be included}

\item{rowpct}{Default FALSE: should row percents be included}
}
\value{
An object of class summary.pctable
}
\description{
Creates a column and/or row percent display of a pctable
result
}
\author{
Paul Johnson <pauljohn@ku.edu>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{plotSeq}
\alias{plotSeq}
\title{Create sequences for plotting}
\usage{
plotSeq(x, length.out = length(x))
}
\arguments{
\item{x}{an R vector variable}

\item{length.out}{the number of elements in the desired plotting sequence.}
}
\description{
\code{plotSeq} is a convenience for the creation of sequence
across the range of a variable.
By default, the length of the plotting
sequence will be equal to the length of the original sequence.
In that case, the only effect is to create an evenly-spaced
set of values. If \code{length.out} is specified, the user
determines the number of elements in plotSeq.
}
\details{
The primary intended usage is for the creation of
plotting sequences of numeric variables.  It takes
a variable's range and the fills in evenly spaced steps.
If x is a factor variable, the levels will be returned.
Uses of this functionality are planned in the future.
}
\examples{
#Create a quadratic regression

stde <- 14
x <- rnorm(100, m = 50, s = 10)
y <- 0.2 - 02*x + 0.2*x^2 + stde*rnorm(100)
mod1 <- lm (y ~ poly(x, 2))

plot(x, y, main="The Quadratic Regression")
seqx <- plotSeq(x, length.out = 10)
seqy <- predict(mod1, newdata = data.frame(x = seqx))
lines(seqx, seqy, col = "red")

# Notice the bad result when a plotting sequence is
# not used.
plot(x, y, main = "Bad Plot Result")
seqy <- predict(mod1)
lines(x, seqy, col = "green")

}
\seealso{
\code{pretty}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vech2Corr.R
\name{vech2Corr}
\alias{vech2Corr}
\title{Convert the vech (column of strictly lower trianglar values from a matrix) into a correlation matrix.}
\usage{
vech2Corr(vech)
}
\arguments{
\item{vech}{A vector of values for the strictly lower triangle of
a matrix. All values must be in the [0,1] interval (because
they are correlations) and the matrix formed must be positive
definite.}
}
\value{
A symmetric correlation matrix, with 1's on the diagonal.
}
\description{
vech2Corr is a convenience function for creating correlation matrices
from a vector of the lower triangular values. It checks the arguments
to make sure they are consistent with the requirements of a
correlation matrix. All values must be in [-1, 1], and the number
of values specified must be correct for a lower triangle.
}
\details{
Use this in combination with the \code{lazyCov} function to
convert a vector of standard deviations and the correlation matrix
into a covariance matrix.
}
\examples{
v <- c(0.1, 0.4, -0.5)
vech2Corr(v)
v <- c(0.1, 0.4, -0.4, 0.4, 0.5, 0.1)
vech2Corr(v)
}
\seealso{
Similar functions exist in many packages, see
    \code{vec2sm} in corpcor, \code{xpnd} in MCMCpack
}
\author{
Paul E. Johnson <pauljohn@ku.edu>
}
